<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:fo="http://www.w3.org/1999/XSL/Format" xmlns:config="http://purl.org/dc/elements/1.1/"
	xmlns:ns10="http://jazz.net/xmlns/alm/qm/v0.1/executionworkitem/v0.1"
	xmlns:ns2="http://jazz.net/xmlns/alm/qm/v0.1/" xmlns:ns3="http://jazz.net/xmlns/alm/v0.1/"
	xmlns:ns4="http://schema.ibm.com/vega/2008/"
	xmlns:ns5="http://jazz.net/xmlns/alm/qm/v0.1/testscript/v0.1/"
	xmlns:ns6="http://jazz.net/xmlns/alm/qm/v0.1/executionresult/v0.1"
	xmlns:ns7="http://jazz.net/xmlns/alm/qm/v0.1/tsl/v0.1/"
	xmlns:ns8="http://jazz.net/xmlns/alm/qm/qmadapter/v0.1"
	xmlns:ns9="http://jazz.net/xmlns/alm/qm/qmadapter/task/v0.1"
	exclude-result-prefixes="ns9 ns10 ns8 ns7 ns6 ns5 ns4 ns3 ns2 config xsl">
	<xsl:variable name="lang">
		<xsl:value-of select="locale"/>
	</xsl:variable>
	<xsl:template match="transformArgs">
		<xsl:choose>
			<xsl:when test="(componentType='STAND_ALONE') or (componentType='STAND_ALONE_SUMMARY')">
				<fo:page-sequence xmlns:fo="http://www.w3.org/1999/XSL/Format"
					initial-page-number="1" force-page-count="no-force"
					master-reference="PageMaster">
					<fo:static-content flow-name="xsl-region-before">
						<fo:block font-size="8pt" text-align="right">
							<fo:page-number/> / <fo:page-number-citation ref-id="{reqNumber}"/>
						</fo:block>
					</fo:static-content>
					<fo:static-content flow-name="xsl-region-after">
						<fo:block-container height="0.1in" padding-after="0.07in">
							<fo:block text-align="center">
								<fo:leader leader-pattern='rule' rule-thickness='1pt' leader-length='100%'/>
							</fo:block>
						</fo:block-container>
						<fo:block text-align="center">
							<fo:retrieve-marker retrieve-class-name="artifactUri"/>
						</fo:block>
					</fo:static-content>
					<fo:flow flow-name="xsl-region-body">
						<fo:marker marker-class-name="artifactUri">
							<fo:block font-size="6pt">
								<fo:basic-link external-destination="{originUrl}" color="blue"
									text-decoration="underline">
									<xsl:value-of select="originUrl"/>
								</fo:basic-link>
							</fo:block>
						</fo:marker>
						<xsl:apply-templates select="descendant::ns2:configuration"/>
						<fo:block id="{reqNumber}"/>
					</fo:flow>
				</fo:page-sequence>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="descendant::ns2:configuration"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<xsl:template match="artifactXML/ns2:configuration">
		<!--  grab all necassary data at beginning-->
		<xsl:variable name="Server">
			<xsl:call-template name="getAsset">
				<xsl:with-param name="assetName" select="'Server'"/>
				<xsl:with-param name="configString" select="ns2:configuration"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="Browser">
			<xsl:call-template name="getAsset">
				<xsl:with-param name="assetName" select="'Browser'"/>
				<xsl:with-param name="configString" select="ns2:configuration"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="CPU">
			<xsl:call-template name="getAsset">
				<xsl:with-param name="assetName" select="'CPU'"/>
				<xsl:with-param name="configString" select="ns2:configuration"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="Database">
			<xsl:call-template name="getAsset">
				<xsl:with-param name="assetName" select="'Database'"/>
				<xsl:with-param name="configString" select="ns2:configuration"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ManagementAgent">
			<xsl:call-template name="getAsset">
				<xsl:with-param name="assetName" select="'ManagementAgent'"/>
				<xsl:with-param name="configString" select="ns2:configuration"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="OperatingSystem">
			<xsl:call-template name="getAsset">
				<xsl:with-param name="assetName" select="'OperatingSystem'"/>
				<xsl:with-param name="configString" select="ns2:configuration"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="TestAdapter">
			<xsl:call-template name="getAsset">
				<xsl:with-param name="assetName" select="'TestAdapter'"/>
				<xsl:with-param name="configString" select="ns2:configuration"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:choose>
			<xsl:when
				test="(../../componentType='SUMMARY') or (../../componentType='STAND_ALONE_SUMMARY') or (../../componentType='COMPOSITE_SUMMARY')">
				<xsl:choose>
					<xsl:when
						test="../../componentType='STAND_ALONE_SUMMARY'  or ../../componentType='COMPOSITE_SUMMARY'">
						<xsl:if test="../../componentType='STAND_ALONE_SUMMARY'">
							<fo:block font-size="12pt" space-before="6mm" space-after="2mm"
									><xsl:value-of
									select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelSummary"
								/>: <xsl:value-of
									select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelConfiguration"
								/></fo:block>
						</xsl:if>
						<fo:table border="0.5pt solid black" text-align="left">
							<fo:table-header>
								<fo:table-row keep-together="always">
									<fo:table-cell keep-together.within-column="always">
										<fo:block-container overflow="hidden">
											<fo:block font-weight="bold" font-size="10pt">
												<xsl:value-of
												select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelName"
												/>
											</fo:block>
										</fo:block-container>
									</fo:table-cell>
									<fo:table-cell keep-together.within-column="always">
										<fo:block-container overflow="hidden">
											<fo:block font-weight="bold" font-size="10pt"
												>Server</fo:block>
										</fo:block-container>
									</fo:table-cell>
									<fo:table-cell keep-together.within-column="always">
										<fo:block-container overflow="hidden">
											<fo:block font-weight="bold" font-size="10pt">
												<xsl:value-of
												select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/Browsers"
												/>
											</fo:block>
										</fo:block-container>
									</fo:table-cell>
									<fo:table-cell keep-together.within-column="always">
										<fo:block-container overflow="hidden">
											<fo:block font-weight="bold" font-size="10pt">
												<xsl:value-of
												select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/CPU"
												/>
											</fo:block>
										</fo:block-container>
									</fo:table-cell>
									<fo:table-cell keep-together.within-column="always">
										<fo:block-container overflow="hidden">
											<fo:block font-weight="bold" font-size="10pt">
												<xsl:value-of
												select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/DataBase"
												/>
											</fo:block>
										</fo:block-container>
									</fo:table-cell>
									<fo:table-cell keep-together.within-column="always">
										<fo:block-container overflow="hidden">
											<fo:block font-weight="bold" font-size="10pt">
												<xsl:value-of
												select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/ManagementAgent"
												/>
											</fo:block>
										</fo:block-container>
									</fo:table-cell>
									<fo:table-cell keep-together.within-column="always">
										<fo:block-container overflow="hidden">
											<fo:block font-weight="bold" font-size="10pt">
												<xsl:value-of
												select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/operatingsystem"
												/>
											</fo:block>
										</fo:block-container>
									</fo:table-cell>
									<fo:table-cell keep-together.within-column="always">
										<fo:block-container overflow="hidden">
											<fo:block font-weight="bold" font-size="10pt">
												<xsl:value-of
												select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/TestAdapter"
												/>
											</fo:block>
										</fo:block-container>
									</fo:table-cell>
								</fo:table-row>
							</fo:table-header>
							<fo:table-body>
								<fo:table-row keep-together="always">
									<xsl:variable name="name" select="ns2:name"/>
									<fo:table-cell keep-together.within-column="always"
										padding="3pt" border="0.5pt solid black">
										<fo:block-container overflow="hidden">
											<fo:block font-size="6pt">
												<xsl:choose>
												<xsl:when test="string-length($name) &gt; 30">
												<xsl:value-of select="substring($name,1,30)"/>
												<xsl:text>...</xsl:text>
												</xsl:when>
												<xsl:otherwise>
												<xsl:value-of select="$name"/>
												</xsl:otherwise>
												</xsl:choose>
											</fo:block>
										</fo:block-container>
									</fo:table-cell>
									<fo:table-cell keep-together.within-column="always"
										padding="3pt" border="0.5pt solid black">
										<fo:block-container overflow="hidden">
											<fo:block font-size="6pt">
												<xsl:value-of select="$Server"/>
											</fo:block>
										</fo:block-container>
									</fo:table-cell>
									<fo:table-cell keep-together.within-column="always"
										padding="3pt" border="0.5pt solid black">
										<fo:block-container overflow="hidden">
											<fo:block font-size="6pt">
												<xsl:value-of select="$Browser"/>
											</fo:block>
										</fo:block-container>
									</fo:table-cell>
									<fo:table-cell keep-together.within-column="always"
										padding="3pt" border="0.5pt solid black">
										<fo:block-container overflow="hidden">
											<fo:block font-size="6pt">
												<xsl:value-of select="$CPU"/>
											</fo:block>
										</fo:block-container>
									</fo:table-cell>
									<fo:table-cell keep-together.within-column="always"
										padding="3pt" border="0.5pt solid black">
										<fo:block-container overflow="hidden">
											<fo:block font-size="6pt">
												<xsl:value-of select="$Database"/>
											</fo:block>
										</fo:block-container>
									</fo:table-cell>
									<fo:table-cell keep-together.within-column="always"
										padding="3pt" border="0.5pt solid black">
										<fo:block-container overflow="hidden">
											<fo:block font-size="6pt">
												<xsl:value-of select="$ManagementAgent"/>
											</fo:block>
										</fo:block-container>
									</fo:table-cell>
									<fo:table-cell keep-together.within-column="always"
										padding="3pt" border="0.5pt solid black">
										<fo:block-container overflow="hidden">
											<fo:block font-size="6pt">
												<xsl:value-of select="$OperatingSystem"/>
											</fo:block>
										</fo:block-container>
									</fo:table-cell>
									<fo:table-cell keep-together.within-column="always"
										padding="3pt" border="0.5pt solid black">
										<fo:block-container overflow="hidden">
											<fo:block font-size="6pt">
												<xsl:value-of select="$TestAdapter"/>
											</fo:block>
										</fo:block-container>
									</fo:table-cell>
								</fo:table-row>
							</fo:table-body>
						</fo:table>
					</xsl:when>
					<xsl:otherwise>
						<fo:table-row keep-together="always">
							<xsl:variable name="name" select="ns2:name"/>
							<fo:table-cell keep-together.within-column="always" padding="3pt"
								border="0.5pt solid black">
								<fo:block-container overflow="hidden">
									<fo:block font-size="6pt">
										<xsl:choose>
											<xsl:when test="string-length($name) &gt; 30">
												<xsl:value-of select="substring($name,1,25)"/>
												<xsl:text>...</xsl:text>
											</xsl:when>
											<xsl:otherwise>
												<xsl:value-of select="$name"/>
											</xsl:otherwise>
										</xsl:choose>
									</fo:block>
								</fo:block-container>
							</fo:table-cell>
							<fo:table-cell keep-together.within-column="always" padding="3pt"
								border="0.5pt solid black">
								<fo:block-container overflow="hidden">
									<fo:block font-size="6pt">
										<xsl:value-of select="$Server"/>
									</fo:block>
								</fo:block-container>
							</fo:table-cell>
							<fo:table-cell keep-together.within-column="always" padding="3pt"
								border="0.5pt solid black">
								<fo:block-container overflow="hidden">
									<fo:block font-size="6pt">
										<xsl:value-of select="$Browser"/>
									</fo:block>
								</fo:block-container>
							</fo:table-cell>
							<fo:table-cell keep-together.within-column="always" padding="3pt"
								border="0.5pt solid black">
								<fo:block-container overflow="hidden">
									<fo:block font-size="6pt">
										<xsl:value-of select="$CPU"/>
									</fo:block>
								</fo:block-container>
							</fo:table-cell>
							<fo:table-cell keep-together.within-column="always" padding="3pt"
								border="0.5pt solid black">
								<fo:block-container overflow="hidden">
									<fo:block font-size="6pt">
										<xsl:value-of select="$Database"/>
									</fo:block>
								</fo:block-container>
							</fo:table-cell>
							<fo:table-cell keep-together.within-column="always" padding="3pt"
								border="0.5pt solid black">
								<fo:block-container overflow="hidden">
									<fo:block font-size="6pt">
										<xsl:value-of select="$ManagementAgent"/>
									</fo:block>
								</fo:block-container>
							</fo:table-cell>
							<fo:table-cell keep-together.within-column="always" padding="3pt"
								border="0.5pt solid black">
								<fo:block-container overflow="hidden">
									<fo:block font-size="6pt">
										<xsl:value-of select="$OperatingSystem"/>
									</fo:block>
								</fo:block-container>
							</fo:table-cell>
							<fo:table-cell keep-together.within-column="always" padding="3pt"
								border="0.5pt solid black">
								<fo:block-container overflow="hidden">
									<fo:block font-size="6pt">
										<xsl:value-of select="$TestAdapter"/>
									</fo:block>
								</fo:block-container>
							</fo:table-cell>
						</fo:table-row>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:when test="(../../componentType='COMPOSITE') and (../../recursive)">
				<fo:wrapper>
					<fo:table-row keep-together="always">
						<xsl:variable name="name" select="ns2:name"/>
						<fo:table-cell keep-together.within-column="always" padding="3pt"
							border="0.5pt solid black">
							<fo:block-container overflow="hidden">
								<fo:block font-size="6pt">
									<xsl:choose>
										<xsl:when test="string-length($name) &gt; 30">
											<xsl:value-of select="substring($name,1,25)"/>
											<xsl:text>...</xsl:text>
										</xsl:when>
										<xsl:otherwise>
											<xsl:value-of select="$name"/>
										</xsl:otherwise>
									</xsl:choose>
								</fo:block>
							</fo:block-container>
						</fo:table-cell>
						<fo:table-cell keep-together.within-column="always" padding="3pt"
							border="0.5pt solid black">
							<fo:block-container overflow="hidden">
								<fo:block font-size="6pt">
									<xsl:value-of select="$Server"/>
								</fo:block>
							</fo:block-container>
						</fo:table-cell>
						<fo:table-cell keep-together.within-column="always" padding="3pt"
							border="0.5pt solid black">
							<fo:block-container overflow="hidden">
								<fo:block font-size="6pt">
									<xsl:value-of select="$Browser"/>
								</fo:block>
							</fo:block-container>
						</fo:table-cell>
						<fo:table-cell keep-together.within-column="always" padding="3pt"
							border="0.5pt solid black">
							<fo:block-container overflow="hidden">
								<fo:block font-size="6pt">
									<xsl:value-of select="$CPU"/>
								</fo:block>
							</fo:block-container>
						</fo:table-cell>
						<fo:table-cell keep-together.within-column="always" padding="3pt"
							border="0.5pt solid black">
							<fo:block-container overflow="hidden">
								<fo:block font-size="6pt">
									<xsl:value-of select="$Database"/>
								</fo:block>
							</fo:block-container>
						</fo:table-cell>
						<fo:table-cell keep-together.within-column="always" padding="3pt"
							border="0.5pt solid black">
							<fo:block-container overflow="hidden">
								<fo:block font-size="6pt">
									<xsl:value-of select="$ManagementAgent"/>
								</fo:block>
							</fo:block-container>
						</fo:table-cell>
						<fo:table-cell keep-together.within-column="always" padding="3pt"
							border="0.5pt solid black">
							<fo:block-container overflow="hidden">
								<fo:block font-size="6pt">
									<xsl:value-of select="$OperatingSystem"/>
								</fo:block>
							</fo:block-container>
						</fo:table-cell>
						<fo:table-cell keep-together.within-column="always" padding="3pt"
							border="0.5pt solid black">
							<fo:block-container overflow="hidden">
								<fo:block font-size="6pt">
									<xsl:value-of select="$TestAdapter"/>
								</fo:block>
							</fo:block-container>
						</fo:table-cell>
					</fo:table-row>
					<fo:block xmlns:fo="http://www.w3.org/1999/XSL/Format">
						<!--		<xsl:attribute name="id"><xsl:value-of-->
						<!--								select="req:identifier" /></xsl:attribute>-->
						<fo:block font-size="14pt" space-before="2mm" space-after="2mm"
							border-after-style="solid" border-after-width="thin"><xsl:value-of
								select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelConfiguration"
							/>: <xsl:value-of select="ns2:name"/></fo:block>
						<fo:block><xsl:value-of
								select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labeldescription"
							/>: <xsl:value-of select="ns2:description"/></fo:block>
						<fo:block><xsl:value-of
								select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelSummary"
							/>: <xsl:value-of select="ns2:summary"/></fo:block>
						<fo:block space-after="4mm"><xsl:value-of
								select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelModifiedDate"
							/>: <xsl:value-of select="ns3:updated"/>
						</fo:block>
						<xsl:if test="string-length($Server) &gt; 0">
							<fo:block>Server: <xsl:value-of select="$Server"/></fo:block>
						</xsl:if>
						<xsl:if test="string-length($Browser) &gt; 0">
							<fo:block><xsl:value-of
									select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/Browsers"
								/>: <xsl:value-of select="$Browser"/></fo:block>
						</xsl:if>
						<xsl:if test="string-length($CPU) &gt; 0">
							<fo:block><xsl:value-of
									select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/CPU"
								/>: <xsl:value-of select="$CPU"/></fo:block>
						</xsl:if>
						<xsl:if test="string-length($Database) &gt; 0">
							<fo:block><xsl:value-of
									select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/DataBase"
								/>: <xsl:value-of select="$Database"/></fo:block>
						</xsl:if>
						<xsl:if test="string-length($ManagementAgent) &gt; 0">
							<fo:block><xsl:value-of
									select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/ManagementAgent"
								/>: <xsl:value-of select="$ManagementAgent"/></fo:block>
						</xsl:if>
						<xsl:if test="string-length($OperatingSystem) &gt; 0">
							<fo:block><xsl:value-of
									select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/operatingsystem"
								/>: <xsl:value-of select="$OperatingSystem"/></fo:block>
						</xsl:if>
						<xsl:if test="string-length($TestAdapter) &gt; 0">
							<fo:block><xsl:value-of
									select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/TestAdapter"
								/>: <xsl:value-of select="$TestAdapter"/></fo:block>
						</xsl:if>
					</fo:block>
				</fo:wrapper>
			</xsl:when>
			<xsl:otherwise>
				<fo:block xmlns:fo="http://www.w3.org/1999/XSL/Format">
					<xsl:if test="(../../componentType='COMPOSITE')">
						<xsl:attribute name="break-after">page</xsl:attribute>
					</xsl:if>
					<fo:block font-size="14pt" space-before="2mm" space-after="2mm"
						border-after-style="solid" border-after-width="thin"><xsl:value-of
							select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelConfiguration"
						/>: <xsl:value-of select="ns2:name"/></fo:block>
					<fo:block><xsl:value-of
							select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labeldescription"
						/>: <xsl:value-of select="ns2:description"/></fo:block>
					<fo:block><xsl:value-of
							select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelSummary"
						/>: <xsl:value-of select="ns2:summary"/></fo:block>
					<fo:block space-after="4mm"><xsl:value-of
							select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelModifiedDate"
						/>: <xsl:value-of select="ns3:updated"/>
					</fo:block>
					<xsl:if test="string-length($Server) &gt; 0">
						<fo:block>Server: <xsl:value-of select="$Server"/></fo:block>
					</xsl:if>
					<xsl:if test="string-length($Browser) &gt; 0">
						<fo:block><xsl:value-of
								select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/Browsers"
							/>: <xsl:value-of select="$Browser"/></fo:block>
					</xsl:if>
					<xsl:if test="string-length($CPU) &gt; 0">
						<fo:block><xsl:value-of
								select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/CPU"
							/>: <xsl:value-of select="$CPU"/></fo:block>
					</xsl:if>
					<xsl:if test="string-length($Database) &gt; 0">
						<fo:block><xsl:value-of
								select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/DataBase"
							/>: <xsl:value-of select="$Database"/></fo:block>
					</xsl:if>
					<xsl:if test="string-length($ManagementAgent) &gt; 0">
						<fo:block><xsl:value-of
								select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/ManagementAgent"
							/>: <xsl:value-of select="$ManagementAgent"/></fo:block>
					</xsl:if>
					<xsl:if test="string-length($OperatingSystem) &gt; 0">
						<fo:block><xsl:value-of
								select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/operatingsystem"
							/>: <xsl:value-of select="$OperatingSystem"/></fo:block>
					</xsl:if>
					<xsl:if test="string-length($TestAdapter) &gt; 0">
						<fo:block><xsl:value-of
								select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/TestAdapter"
							/>: <xsl:value-of select="$TestAdapter"/></fo:block>
					</xsl:if>
				</fo:block>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<xsl:template name="getAsset">
		<xsl:param name="assetName"/>
		<xsl:param name="configString"/>
		<xsl:variable name="searchValue">
			<xsl:choose>
				<xsl:when
					test="$assetName='Browser' or $assetName='Database' or $assetName='ManagementAgent' or $assetName='TestAdapter' or $assetName='Server'">
					<xsl:value-of select="'SoftwareInstall'"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="$assetName"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="first" select="substring-before($configString,&quot;{&quot;)"/>
		<xsl:variable name="rest" select="substring-after($configString,&quot;{&quot;)"/>
		<xsl:if test="contains($first, $searchValue)">
			<xsl:variable name="start"
				select="substring-after($first, '&quot;name&quot;:&quot;')"/>
			<xsl:variable name="result" select="substring-before($start, '&quot;')"/>
			<xsl:choose>
				<xsl:when test="$assetName='Database'">
					<xsl:if
						test="contains($result, 'DB2') or contains($result, 'Oracle') or contains($result, 'SQL') or contains($result, 'Sybase') or contains($result, 'Informix')">
						<xsl:value-of select="$result"/>
					</xsl:if>
				</xsl:when>
				<xsl:when test="$assetName='Browser'">
					<xsl:if
						test="contains($result, 'Internet') or contains($result, 'Firefox') or contains($result, 'Safari')">
						<xsl:value-of select="$result"/>
					</xsl:if>
				</xsl:when>
				<xsl:when test="$assetName='Server'">
					<xsl:if
						test="contains($result, 'Tomcat') or contains($result, 'WAS') or contains($result, 'BEA') or contains($result, 'Weblogic')">
						<xsl:value-of select="$result"/>
					</xsl:if>
				</xsl:when>
				<xsl:when test="$assetName='ManagementAgent'">
					<xsl:if test="contains($result, 'Agent')">
						<xsl:value-of select="$result"/>
					</xsl:if>
				</xsl:when>
				<xsl:when test="$assetName='TestAdapter'">
					<xsl:if test="contains($result, 'Rational') or contains($result, 'Sample:')">
						<xsl:value-of select="$result"/>
					</xsl:if>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="$result"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:if>
		<xsl:if test="$rest">
			<xsl:call-template name="getAsset">
				<xsl:with-param name="assetName" select="$assetName"/>
				<xsl:with-param name="configString" select="$rest"/>
			</xsl:call-template>
		</xsl:if>
	</xsl:template>
</xsl:stylesheet>
