<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:fo="http://www.w3.org/1999/XSL/Format" xmlns:ewi="http://purl.org/dc/elements/1.1/"
    xmlns:ns10="http://jazz.net/xmlns/alm/qm/v0.1/executionworkitem/v0.1"
    xmlns:ns2="http://jazz.net/xmlns/alm/qm/v0.1/" xmlns:ns3="http://jazz.net/xmlns/alm/v0.1/"
    xmlns:ns4="http://schema.ibm.com/vega/2008/"
    xmlns:ns5="http://jazz.net/xmlns/alm/qm/v0.1/testscript/v0.1/"
    xmlns:ns6="http://jazz.net/xmlns/alm/qm/qmadapter/task/v0.1"
    xmlns:ns7="http://jazz.net/xmlns/alm/qm/qmadapter/v0.1"
    xmlns:ns8="http://jazz.net/xmlns/alm/qm/v0.1/executionresult/v0.1"
    xmlns:ns9="http://jazz.net/xmlns/alm/qm/v0.1/tsl/v0.1/" xmlns:atom="http://www.w3.org/2005/Atom"
    exclude-result-prefixes="ns10 ns9 ns8 ns7 ns6 ns5 ns4 ns3 ns2 atom ewi xsl">
    <xsl:include href="wrapping-rules.xsl"/>
    
    <xsl:variable name="lang">
        <xsl:value-of select="locale"/>
    </xsl:variable>
    
    <xsl:template match="transformArgs">
        <xsl:choose>
            <xsl:when test="(componentType='STAND_ALONE') or (componentType='STAND_ALONE_SUMMARY')">
                <fo:page-sequence xmlns:fo="http://www.w3.org/1999/XSL/Format"
                    initial-page-number="1" force-page-count="no-force"
                    master-reference="PageMaster">
                    <fo:static-content flow-name="xsl-region-before">
                        <fo:block font-size="8pt" text-align="right">
                            <fo:page-number/> / <fo:page-number-citation ref-id="{reqNumber}"/>
                        </fo:block>
                    </fo:static-content>
                    <fo:static-content flow-name="xsl-region-after">
                    	<fo:block-container height="0.1in" padding-after="0.07in">
							<fo:block text-align="center">
								<fo:leader leader-pattern='rule' rule-thickness='1pt' leader-length='100%'/>
							</fo:block>
						</fo:block-container>
                        <fo:block text-align="center">
                            <fo:retrieve-marker retrieve-class-name="artifactUri"/>
                        </fo:block>
                    </fo:static-content>
                    <fo:flow flow-name="xsl-region-body">
                        <fo:marker marker-class-name="artifactUri">
                            <fo:block font-size="6pt">
                                <fo:basic-link external-destination="{originUrl}" color="blue"
                                    text-decoration="underline">
                                    <xsl:value-of select="originUrl"/>
                                </fo:basic-link>
                            </fo:block>
                        </fo:marker>
                        <xsl:apply-templates select="descendant::ns2:executionworkitem"/>
                        <fo:block id="{reqNumber}"/>
                    </fo:flow>
                </fo:page-sequence>
            </xsl:when>
            <xsl:otherwise>
                <xsl:apply-templates select="descendant::ns2:executionworkitem"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="ns2:executionworkitem">
        <xsl:choose>
            <xsl:when
                test="(../../componentType='SUMMARY') or (../../componentType='STAND_ALONE_SUMMARY') or (../../componentType='COMPOSITE_SUMMARY')">
                <xsl:choose>
                    <xsl:when
                        test="../../componentType='STAND_ALONE_SUMMARY' or (../../componentType='COMPOSITE_SUMMARY')">
                        <xsl:if test="../../componentType='STAND_ALONE_SUMMARY'">
                            <fo:block font-size="14pt" space-before="6mm" space-after="2mm">
                                <xsl:value-of
                                    select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelSummary"
                                /> : <xsl:value-of
                                    select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelTestExecutionRecord"
                                />
                            </fo:block>
                        </xsl:if>
                        <fo:table border="0.5pt solid black" text-align="left">
                            <fo:table-header>
                                <fo:table-row keep-together="always" background-color="rgb(249,245,219)" font-size="8pt" font-weight="bold">
                                    <!-- NAME HEADER -->
                                    <fo:table-cell keep-together.within-column="always" width="30mm" padding="2pt" border="0.5pt solid black">
                                        <fo:block-container overflow="hidden">
                                            <fo:block>
                                                <xsl:value-of select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelName"/>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <!-- TEST ENVIRONMENT HEADER -->
                                    <fo:table-cell keep-together.within-column="always" padding="3pt" border="0.5pt solid black">
                                        <fo:block-container overflow="hidden">
                                            <fo:block>
                                                <xsl:value-of select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelTestEnvironment"/>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <!-- TEST CASE HEADER -->
                                    <fo:table-cell keep-together.within-column="always"
                                        padding="3pt" border="0.5pt solid black">
                                        <fo:block-container overflow="hidden">
                                            <fo:block>
                                                <xsl:value-of select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelTestCase"/>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <!-- TEST PLAN HEADER -->
                                    <fo:table-cell keep-together.within-column="always" padding="3pt" border="0.5pt solid black">
                                        <fo:block-container overflow="hidden">
                                            <fo:block>
                                                <xsl:value-of select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelTestPlan"/>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <!-- TEST SCRIPT HEADER -->
                                    <fo:table-cell keep-together.within-column="always" 
                                        padding="3pt" border="0.5pt solid black">
                                        <fo:block-container overflow="hidden">
                                            <fo:block>
                                                <xsl:value-of select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelTestScript"/>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <!-- TEST MILESTONE HEADER -->
                                    <fo:table-cell keep-together.within-column="always"
                                        padding="3pt" border="0.5pt solid black">
                                        <fo:block-container overflow="hidden">
                                            <fo:block>
                                                <xsl:value-of select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelTestMilestone"/>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <!-- OWNER HEADER -->
                                    <fo:table-cell keep-together.within-column="always" padding="3pt" border="0.5pt solid black">
                                        <fo:block-container overflow="hidden">
                                            <fo:block>
                                                <xsl:value-of select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelOwner"/>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <!-- LAST RESULT HEADER -->
                                    <fo:table-cell keep-together.within-column="always" padding="3pt" border="0.5pt solid black">
                                        <fo:block-container overflow="hidden">
                                            <fo:block>
                                                <xsl:value-of select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelLastResult"/>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <!-- LAST MODIFIED DATE HEADER -->
                                    <fo:table-cell keep-together.within-column="always" padding="3pt" border="0.5pt solid black">
                                        <fo:block-container overflow="hidden">
                                            <fo:block>
                                                <xsl:value-of select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelModifiedDate"/>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <!-- WEIGHT HEADER -->
                                    <fo:table-cell keep-together.within-column="always" width="12mm" padding="3pt" border="0.5pt solid black">
                                        <fo:block-container overflow="hidden">
                                            <fo:block>
                                                <xsl:value-of select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelWeight"/>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                </fo:table-row>
                            </fo:table-header>
                            <fo:table-body>
                                <fo:table-row keep-together="always" font-size="8pt">
                                    <!-- NAME -->
                                    <fo:table-cell keep-together.within-column="always" width="30mm" padding="3pt" border="0.5pt solid black">
                                        <fo:block-container overflow="hidden">
                                            <fo:block>
                                                <xsl:call-template name="wrap-on-all-characters">
                                        	    	<xsl:with-param name="text-content" select="ewi:title"/>
                                                </xsl:call-template>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <!-- TEST ENVIRONMENT -->
                                    <fo:table-cell keep-together.within-column="always" padding="3pt" border="0.5pt solid black">
                                        <fo:block-container overflow="hidden">
                                            <xsl:choose>
                                                <xsl:when test="../../atom:titles/atom:configurations/atom:title">
                                                  <fo:block> 
                                                      <xsl:call-template name="wrap-on-all-characters">
                                        	    	      <xsl:with-param name="text-content" select="../../atom:titles/atom:configurations/atom:title"/>
                                                      </xsl:call-template>
                                                  </fo:block>
                                                </xsl:when>
                                                <xsl:otherwise>
                                                  <fo:block>
                                                    <xsl:call-template name="wrap-on-all-characters">
                                        	    	  <xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelUnassigned"/>
                                                    </xsl:call-template>
                                                  </fo:block>
                                                </xsl:otherwise>
                                            </xsl:choose>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <!-- TEST CASE -->
                                    <fo:table-cell keep-together.within-column="always" padding="3pt" border="0.5pt solid black">
                                        <fo:block-container overflow="hidden">
                                            <fo:block> 
                                                <xsl:call-template name="wrap-on-all-characters">
                                        	    	<xsl:with-param name="text-content" select="../../atom:titles/atom:testCaseTitle"/>
                                                </xsl:call-template>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <!-- TEST PLAN -->
                                    <fo:table-cell keep-together.within-column="always" padding="3pt" border="0.5pt solid black">
                                        <fo:block-container overflow="hidden">
                                            <xsl:choose>
                                                <xsl:when test="../../atom:titles/atom:testPlanTitle">
                                                  <fo:block> 
                                                  	<xsl:call-template name="wrap-on-all-characters">
                                        	          <xsl:with-param name="text-content" select="../../atom:titles/atom:testPlanTitle"/>
                                                    </xsl:call-template>
                                                  </fo:block>
                                                </xsl:when>
                                                <xsl:otherwise>
                                                  <fo:block>
                                                    <xsl:call-template name="wrap-on-all-characters">
                                        	          <xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelUnassigned"/>
                                                    </xsl:call-template>
                                                  </fo:block>
                                                </xsl:otherwise>
                                            </xsl:choose>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <!-- TEST SCRIPT -->
                                    <fo:table-cell keep-together.within-column="always" padding="3pt" border="0.5pt solid black">
                                        <fo:block-container overflow="hidden">
                                            <xsl:choose>
                                                <xsl:when test="../../atom:titles/atom:testScriptTitle">
                                                  <fo:block>
                                                    <xsl:call-template name="wrap-on-all-characters">
                                        	          <xsl:with-param name="text-content" select="../../atom:titles/atom:testScriptTitle"/>
                                                    </xsl:call-template> 
                                                  </fo:block>
                                                </xsl:when>
                                                <xsl:otherwise>
                                                  <fo:block>
                                                    <xsl:call-template name="wrap-on-all-characters">
                                        	          <xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelNone"/>
                                                    </xsl:call-template>
                                                  </fo:block>
                                                </xsl:otherwise>
                                            </xsl:choose>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <!-- MILESTONE -->
                                    <fo:table-cell keep-together.within-column="always" padding="3pt" border="0.5pt solid black">
                                        <fo:block-container overflow="hidden">
                                            <xsl:choose>
                                                <xsl:when test="../../atom:titles/atom:testPhaseTitle">
                                                  <fo:block>
                                                    <xsl:call-template name="wrap-on-all-characters">
                                        	          <xsl:with-param name="text-content" select="../../atom:titles/atom:testPhaseTitle"/>
                                                    </xsl:call-template>
                                                  </fo:block>
                                                </xsl:when>
                                                <xsl:otherwise>
                                                  <fo:block>
                                                    <xsl:call-template name="wrap-on-all-characters">
                                        	          <xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelUnassigned"/>
                                                    </xsl:call-template>
                                                  </fo:block>
                                                </xsl:otherwise>
                                            </xsl:choose>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <!-- OWNER -->
                                    <fo:table-cell keep-together.within-column="always" padding="3pt" border="0.5pt solid black">
                                        <fo:block-container overflow="hidden">
                                            <fo:block>
                                                <xsl:call-template name="wrap-on-all-characters">
                                                  <xsl:with-param name="text-content" select="ns3:owner"/>
                                                </xsl:call-template>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <!-- LAST RESULT -->
                                    <fo:table-cell keep-together.within-column="always" padding="3pt" border="0.5pt solid black">
                                        <fo:block-container overflow="hidden">
                                            <fo:block>
                                                <xsl:choose>
	                                                <xsl:when
	                                                test="../../atom:titles/atom:currentResultState='com.ibm.rqm.execution.common.state.passed'">
	                                                    <xsl:call-template name="wrap-on-all-characters">
	                                      	              <xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelPassed"/>
	                                                    </xsl:call-template>
	                                                </xsl:when>
	                                                <xsl:when
	                                                test="../../atom:titles/atom:currentResultState='com.ibm.rqm.execution.common.state.failed'">
	                                                    <xsl:call-template name="wrap-on-all-characters">
	                                      	              <xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelFailed"/>
	                                                    </xsl:call-template>
	                                                </xsl:when>
	                                                <xsl:when
	                                                test="../../atom:titles/atom:currentResultState='com.ibm.rqm.execution.common.state.blocked'">
	                                                    <xsl:call-template name="wrap-on-all-characters">
	                                      	              <xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelBlocked"/>
	                                                    </xsl:call-template>
	                                                </xsl:when>
	                                                <xsl:when
	                                                test="../../atom:titles/atom:currentResultState='com.ibm.rqm.execution.common.state.complete'">
	                                                    <xsl:call-template name="wrap-on-all-characters">
	                                      	              <xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelComplete"/>
	                                                    </xsl:call-template>
	                                                </xsl:when>
	                                                <xsl:when
	                                                test="../../atom:titles/atom:currentResultState='com.ibm.rqm.execution.common.state.inconclusive'">
	                                                    <xsl:call-template name="wrap-on-all-characters">
	                                      	              <xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelInconclusive"/>
	                                                    </xsl:call-template>
	                                                </xsl:when>
	                                                <xsl:when
	                                                test="../../atom:titles/atom:currentResultState='com.ibm.rqm.execution.common.state.error'">
	                                                    <xsl:call-template name="wrap-on-all-characters">
	                                      	              <xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelError"/>
	                                                    </xsl:call-template>
	                                                </xsl:when>
                                                    <xsl:otherwise> -- </xsl:otherwise>
                                                </xsl:choose>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <!-- LAST UPDATED DATE -->
                                    <fo:table-cell keep-together.within-column="always" padding="3pt" border="0.5pt solid black">
                                        <fo:block-container overflow="hidden">
                                            <fo:block>
                                                <xsl:value-of select="ns3:updated"/>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <!-- WEIGHT -->
                                    <fo:table-cell keep-together.within-column="always" width="11mm" padding="3pt" border="0.5pt solid black">
                                        <fo:block-container overflow="hidden">
                                            <fo:block>
                                                <xsl:call-template name="wrap-on-all-characters">
                                        	        <xsl:with-param name="text-content" select="ns10:weight"/>
                                                </xsl:call-template>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                </fo:table-row>
                            </fo:table-body>
                        </fo:table>
                    </xsl:when>
                    <xsl:otherwise>
                        <fo:table-row keep-together="always" font-size="8pt">
                            <!-- NAME -->
                            <fo:table-cell keep-together.within-column="always" width="30mm" padding="3pt" border="0.5pt solid black">
                                <fo:block-container overflow="hidden">
                                    <fo:block >
                                        <xsl:call-template name="wrap-on-all-characters">
                                            <xsl:with-param name="text-content" select="ewi:title"/>
                                        </xsl:call-template>
                                    </fo:block>
                                </fo:block-container>
                            </fo:table-cell>
                            <fo:table-cell keep-together.within-column="always" padding="3pt"
                                border="0.5pt solid black">
                                <fo:block-container overflow="hidden">
                                    <xsl:choose>
                                        <xsl:when
                                            test="../../atom:titles/atom:configurations/atom:title">
                                            <fo:block >
                                            	<xsl:call-template name="wrap-on-all-characters">
                                                	<xsl:with-param name="text-content" select="../../atom:titles/atom:configurations/atom:title"/>
                                                </xsl:call-template>
                                            </fo:block>
                                        </xsl:when>
                                        <xsl:otherwise>
                                            <fo:block >
                                                <xsl:call-template name="wrap-on-all-characters">
                                                    <xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelUnassigned" />
                                                </xsl:call-template>
                                            </fo:block>
                                        </xsl:otherwise>
                                    </xsl:choose>
                                </fo:block-container>
                            </fo:table-cell>
                            <fo:table-cell keep-together.within-column="always" padding="3pt"
                                border="0.5pt solid black">
                                <fo:block-container overflow="hidden">
                                    <fo:block >
                                    	<xsl:call-template name="wrap-on-all-characters">
                                        	<xsl:with-param name="text-content" select="../../atom:titles/atom:testCaseTitle"/>
                                        </xsl:call-template>
                                    </fo:block>
                                </fo:block-container>
                            </fo:table-cell>
                            <fo:table-cell keep-together.within-column="always" padding="3pt"
                                border="0.5pt solid black">
                                <fo:block-container overflow="hidden">
                                    <xsl:choose>
                                        <xsl:when test="../../atom:titles/atom:testPlanTitle">
                                            <fo:block >
                                                <xsl:call-template name="wrap-on-all-characters">
                                        	        <xsl:with-param name="text-content" select="../../atom:titles/atom:testPlanTitle"/>
                                                </xsl:call-template>
                                            </fo:block>
                                        </xsl:when>
                                        <xsl:otherwise>
                                            <fo:block >
                                            	<xsl:call-template name="wrap-on-all-characters">
                                                	<xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelUnassigned" />
                                                </xsl:call-template>
                                            </fo:block>
                                        </xsl:otherwise>
                                    </xsl:choose>
                                </fo:block-container>
                            </fo:table-cell>
                            <fo:table-cell keep-together.within-column="always" padding="3pt"
                                border="0.5pt solid black">
                                <fo:block-container overflow="hidden">
                                    <xsl:choose>
                                        <xsl:when test="../../atom:titles/atom:testScriptTitle">
                                            <fo:block >
                                            	<xsl:call-template name="wrap-on-all-characters">
                                                	<xsl:with-param name="text-content" select="../../atom:titles/atom:testScriptTitle" />
                                                </xsl:call-template>
                                            </fo:block>
                                        </xsl:when>
                                        <xsl:otherwise>
                                            <fo:block >
                                            	<xsl:call-template name="wrap-on-all-characters">
                                                	<xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelNone" />
                                                </xsl:call-template>
                                            </fo:block>
                                        </xsl:otherwise>
                                    </xsl:choose>
                                </fo:block-container>
                            </fo:table-cell>
                            <fo:table-cell keep-together.within-column="always"
                                padding="3pt" border="0.5pt solid black">
                                <fo:block-container overflow="hidden">
                                    <xsl:choose>
                                        <xsl:when
                                          test="../../atom:titles/atom:testPhaseTitle">
                                          <fo:block >
                                            <xsl:call-template name="wrap-on-all-characters">
                                              <xsl:with-param name="text-content" select="../../atom:titles/atom:testPhaseTitle" />
                                            </xsl:call-template>
                                          </fo:block>
                                        </xsl:when>
                                        <xsl:otherwise>
                                          <fo:block >
                                            <xsl:call-template name="wrap-on-all-characters">
                                              <xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelUnassigned" />
                                            </xsl:call-template>
                                          </fo:block>
                                        </xsl:otherwise>
                                    </xsl:choose>
                                </fo:block-container>
                            </fo:table-cell>
                            <fo:table-cell keep-together.within-column="always" padding="3pt"
                                border="0.5pt solid black">
                                <fo:block-container overflow="hidden">
                                    <fo:block >
                                        <xsl:call-template name="wrap-on-all-characters">
                                            <xsl:with-param name="text-content" select="ns3:owner"/>
                                        </xsl:call-template>
                                    </fo:block>
                                </fo:block-container>
                            </fo:table-cell>
                            <fo:table-cell keep-together.within-column="always" padding="3pt"
                                border="0.5pt solid black">
                                <fo:block-container overflow="hidden">
                                    <fo:block >
                                        <xsl:choose>
                                            <xsl:when
                                            test="../../atom:titles/atom:currentResultState='com.ibm.rqm.execution.common.state.passed'">
                                                <xsl:call-template name="wrap-on-all-characters">
                                  	              <xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelPassed"/>
                                                </xsl:call-template>
                                            </xsl:when>
                                            <xsl:when
                                            test="../../atom:titles/atom:currentResultState='com.ibm.rqm.execution.common.state.failed'">
                                                <xsl:call-template name="wrap-on-all-characters">
                                  	              <xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelFailed"/>
                                                </xsl:call-template>
                                            </xsl:when>
                                            <xsl:when
                                            test="../../atom:titles/atom:currentResultState='com.ibm.rqm.execution.common.state.blocked'">
                                                <xsl:call-template name="wrap-on-all-characters">
                                  	              <xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelBlocked"/>
                                                </xsl:call-template>
                                            </xsl:when>
                                            <xsl:when
                                            test="../../atom:titles/atom:currentResultState='com.ibm.rqm.execution.common.state.complete'">
                                                <xsl:call-template name="wrap-on-all-characters">
                                  	              <xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelComplete"/>
                                                </xsl:call-template>
                                            </xsl:when>
                                            <xsl:when
                                            test="../../atom:titles/atom:currentResultState='com.ibm.rqm.execution.common.state.inconclusive'">
                                                <xsl:call-template name="wrap-on-all-characters">
                                  	              <xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelInconclusive"/>
                                                </xsl:call-template>
                                            </xsl:when>
                                            <xsl:when
                                            test="../../atom:titles/atom:currentResultState='com.ibm.rqm.execution.common.state.error'">
                                                <xsl:call-template name="wrap-on-all-characters">
                                  	              <xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelError"/>
                                                </xsl:call-template>
                                            </xsl:when>
                                            <xsl:otherwise> -- </xsl:otherwise>
                                        </xsl:choose>
                                    </fo:block>
                                </fo:block-container>
                            </fo:table-cell>
                            <fo:table-cell keep-together.within-column="always" padding="3pt"
                                border="0.5pt solid black">
                                <fo:block-container overflow="hidden">
                                    <fo:block >
                                        <xsl:value-of select="ns3:updated"/>
                                    </fo:block>
                                </fo:block-container>
                            </fo:table-cell>
                            <!-- WEIGHT -->
                            <fo:table-cell keep-together.within-column="always" width="11mm" padding="3pt" border="0.5pt solid black">
                                <fo:block-container overflow="hidden">
                                    <fo:block>
                                        <xsl:call-template name="wrap-on-all-characters">
                                        	<xsl:with-param name="text-content" select="ns10:weight"/>
                                        </xsl:call-template>
                                    </fo:block>
                                </fo:block-container>
                            </fo:table-cell>
                        </fo:table-row>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="(../../componentType='COMPOSITE') and (../../recursive)">
                <fo:wrapper>
                    <fo:table-row keep-together="always" font-size="8pt">
                        <!-- NAME -->
                        <fo:table-cell keep-together.within-column="always" width="30mm" padding="3pt" border="0.5pt solid black">
                            <fo:block-container overflow="hidden">
                                <fo:block>
                                    <xsl:call-template name="wrap-on-all-characters">
                                    	<xsl:with-param name="text-content" select="ewi:title"/>
                                    </xsl:call-template>
                                </fo:block>
                            </fo:block-container>
                        </fo:table-cell>
                        <fo:table-cell keep-together.within-column="always" padding="3pt"
                            border="0.5pt solid black">
                            <fo:block-container overflow="hidden">
                                <xsl:choose>
                                    <xsl:when
                                        test="../../atom:titles/atom:configurations/atom:title">
                                        <fo:block >
                                        	<xsl:call-template name="wrap-on-all-characters">
                                            	<xsl:with-param name="text-content" select="../../atom:titles/atom:configurations/atom:title"/>
                                            </xsl:call-template>
                                        </fo:block>
                                    </xsl:when>
                                    <xsl:otherwise>
                                        <fo:block >
                                        	<xsl:call-template name="wrap-on-all-characters">
                                            	<xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelUnassigned" />
                                            </xsl:call-template>
                                        </fo:block>
                                    </xsl:otherwise>
                                </xsl:choose>
                            </fo:block-container>
                        </fo:table-cell>
                        <fo:table-cell keep-together.within-column="always" padding="3pt"
                            border="0.5pt solid black">
                            <fo:block-container overflow="hidden">
                                <fo:block >
                                	<xsl:call-template name="wrap-on-all-characters">
                                    	<xsl:with-param name="text-content" select="../../atom:titles/atom:testCaseTitle"/>
                                    </xsl:call-template>
                                </fo:block>
                            </fo:block-container>
                        </fo:table-cell>
                        <fo:table-cell keep-together.within-column="always" padding="3pt"
                            border="0.5pt solid black">
                            <fo:block-container overflow="hidden">
                                <xsl:choose>
                                    <xsl:when test="../../atom:titles/atom:testPlanTitle">
                                        <fo:block >
                                        	<xsl:call-template name="wrap-on-all-characters">
                                            	<xsl:with-param name="text-content" select="../../atom:titles/atom:testPlanTitle"/>
                                        	</xsl:call-template>
                                        </fo:block>
                                    </xsl:when>
                                    <xsl:otherwise>
                                        <fo:block >
                                        	<xsl:call-template name="wrap-on-all-characters">
                                            	<xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelUnassigned" />
                                            </xsl:call-template>
                                        </fo:block>
                                    </xsl:otherwise>
                                </xsl:choose>
                            </fo:block-container>
                        </fo:table-cell>
                        <fo:table-cell keep-together.within-column="always" padding="3pt"
                            border="0.5pt solid black">
                            <fo:block-container overflow="hidden">
                                <xsl:choose>
                                    <xsl:when test="../../atom:titles/atom:testScriptTitle">
                                        <fo:block >
                                        	<xsl:call-template name="wrap-on-all-characters">
                                            	<xsl:with-param name="text-content" select="../../atom:titles/atom:testScriptTitle"/>
                                            </xsl:call-template>
                                        </fo:block>
                                    </xsl:when>
                                    <xsl:otherwise>
                                        <fo:block >
                                        	<xsl:call-template name="wrap-on-all-characters">
                                            	<xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelNone" />
                                            </xsl:call-template>
                                        </fo:block>
                                    </xsl:otherwise>
                                </xsl:choose>
                            </fo:block-container>
                        </fo:table-cell>
                        <fo:table-cell keep-together.within-column="always"
                            padding="3pt" border="0.5pt solid black">
                            <fo:block-container overflow="hidden">
                                <xsl:choose>
                                    <xsl:when
                                      test="../../atom:titles/atom:testPhaseTitle">
                                      <fo:block >
                                      	  <xsl:call-template name="wrap-on-all-characters">
                                              <xsl:with-param name="text-content" select="../../atom:titles/atom:testPhaseTitle"/>
                                          </xsl:call-template>
                                      </fo:block>
                                    </xsl:when>
                                    <xsl:otherwise>
                                      <fo:block >
                                          <xsl:call-template name="wrap-on-all-characters">
                                              <xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelUnassigned" />
                                          </xsl:call-template>
                                      </fo:block>
                                    </xsl:otherwise>
                                </xsl:choose>
                            </fo:block-container>
                        </fo:table-cell>
                        <fo:table-cell keep-together.within-column="always" padding="3pt"
                            border="0.5pt solid black">
                            <fo:block-container overflow="hidden">
                                <fo:block >
                                    <xsl:call-template name="wrap-on-all-characters">
                                        <xsl:with-param name="text-content" select="ns3:owner"/>
                                    </xsl:call-template>
                                </fo:block>
                            </fo:block-container>
                        </fo:table-cell>
                        <fo:table-cell keep-together.within-column="always" padding="3pt"
                            border="0.5pt solid black">
                            <fo:block-container overflow="hidden">
                                <fo:block >
                                    <xsl:choose>
                                        <xsl:when
                                        test="../../atom:titles/atom:currentResultState='com.ibm.rqm.execution.common.state.passed'">
                                            <xsl:call-template name="wrap-on-all-characters">
                              	              <xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelPassed"/>
                                            </xsl:call-template>
                                        </xsl:when>
                                        <xsl:when
                                        test="../../atom:titles/atom:currentResultState='com.ibm.rqm.execution.common.state.failed'">
                                            <xsl:call-template name="wrap-on-all-characters">
                              	              <xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelFailed"/>
                                            </xsl:call-template>
                                        </xsl:when>
                                        <xsl:when
                                        test="../../atom:titles/atom:currentResultState='com.ibm.rqm.execution.common.state.blocked'">
                                            <xsl:call-template name="wrap-on-all-characters">
                              	              <xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelBlocked"/>
                                            </xsl:call-template>
                                        </xsl:when>
                                        <xsl:when
                                        test="../../atom:titles/atom:currentResultState='com.ibm.rqm.execution.common.state.complete'">
                                            <xsl:call-template name="wrap-on-all-characters">
                              	              <xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelComplete"/>
                                            </xsl:call-template>
                                        </xsl:when>
                                        <xsl:when
                                        test="../../atom:titles/atom:currentResultState='com.ibm.rqm.execution.common.state.inconclusive'">
                                            <xsl:call-template name="wrap-on-all-characters">
                              	              <xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelInconclusive"/>
                                            </xsl:call-template>
                                        </xsl:when>
                                        <xsl:when
                                        test="../../atom:titles/atom:currentResultState='com.ibm.rqm.execution.common.state.error'">
                                            <xsl:call-template name="wrap-on-all-characters">
                              	              <xsl:with-param name="text-content" select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelError"/>
                                            </xsl:call-template>
                                        </xsl:when>
                                        <xsl:otherwise> -- </xsl:otherwise>
                                    </xsl:choose>
                                </fo:block>
                            </fo:block-container>
                        </fo:table-cell>
                        <fo:table-cell keep-together.within-column="always" padding="3pt"
                            border="0.5pt solid black">
                            <fo:block-container overflow="hidden">
                                <fo:block >
                                    <xsl:value-of select="ns3:updated"/>
                                </fo:block>
                            </fo:block-container>
                        </fo:table-cell>
                        <!-- WEIGHT -->
                        <fo:table-cell keep-together.within-column="always" width="11mm" padding="3pt"
                            border="0.5pt solid black">
                            <fo:block-container overflow="hidden">
                                <fo:block >
                                	<xsl:call-template name="wrap-on-all-characters">
                                        <xsl:with-param name="text-content" select="ns10:weight"/>
                                    </xsl:call-template>
                                </fo:block>
                            </fo:block-container>
                        </fo:table-cell>
                    </fo:table-row>
                    <fo:block xmlns:fo="http://www.w3.org/1999/XSL/Format">
                        <xsl:attribute name="id">
                            <xsl:value-of select="ns3:identifier"/>
                        </xsl:attribute>
                        <fo:block font-size="14pt" space-before="2mm" space-after="2mm"
                            border-after-style="solid" border-after-width="thin">
                            <xsl:value-of
                                select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelTestExecutionRecord"
                            /> : <xsl:value-of select="ewi:title"/>
                        </fo:block>
                        <fo:block>
                            <xsl:value-of
                                select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelModifiedDate"
                            /> : <xsl:value-of select="ns3:updated"/>
                        </fo:block>
                        <fo:block>
                            <xsl:value-of
                                select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/reqLabel_created_by"
                            /> : <xsl:value-of select="ewi:creator"/>
                        </fo:block>
                        <fo:block>
                            <xsl:value-of
                                select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelOwnedBy"
                            /> : <xsl:value-of select="ns3:owner"/>
                        </fo:block>
                        <fo:block space-before="2mm">
                            <xsl:value-of
                                select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelTestCase"
                            /> : <xsl:value-of select="../../atom:titles/atom:testCaseTitle"/>
                        </fo:block>
                        <fo:block space-before="2mm">
                            <xsl:value-of
                                select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelTestPlan"
                            /> : <xsl:choose>
                                <xsl:when test="../../atom:titles/atom:testPlanTitle">
                                    <xsl:value-of select="../../atom:titles/atom:testPlanTitle"/>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:value-of
                                        select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelUnassigned"
                                    />
                                </xsl:otherwise>
                            </xsl:choose>
                        </fo:block>
                        <fo:block space-before="2mm">
                            <xsl:value-of
                                select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelTestScript"
                            /> : <xsl:choose>
                                <xsl:when test="../../atom:titles/atom:testScriptTitle">
                                    <xsl:value-of select="../../atom:titles/atom:testScriptTitle"/>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:value-of
                                        select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelNone"
                                    />
                                </xsl:otherwise>
                            </xsl:choose>
                        </fo:block>
                        <fo:block space-before="2mm">
                            <xsl:value-of
                                select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelTestEnvironment"
                            /> : <xsl:choose>
                                <xsl:when test="../../atom:titles/atom:configurations/atom:title">
                                    <xsl:value-of
                                        select="../../atom:titles/atom:configurations/atom:title"/>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:value-of
                                        select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelUnassigned"
                                    />
                                </xsl:otherwise>
                            </xsl:choose>
                        </fo:block>
                        <fo:block space-before="2mm">
                            <xsl:value-of
                                select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelTestMilestone"
                            /> : <xsl:choose>
                                <xsl:when test="../../atom:titles/atom:testPhaseTitle">
                                    <xsl:value-of select="../../atom:titles/atom:testPhaseTitle"/>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:value-of
                                        select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelUnassigned"
                                    />
                                </xsl:otherwise>
                            </xsl:choose>
                        </fo:block>
                        <fo:block space-before="2mm">
                            <xsl:value-of
                                select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelLastResult"
                            />: <xsl:choose>
                                <xsl:when
                                    test="../../atom:titles/atom:currentResultState='com.ibm.rqm.execution.common.state.passed'">
                                    <xsl:value-of
                                        select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelPassed"
                                    />
                                </xsl:when>
                                <xsl:when
                                    test="../../atom:titles/atom:currentResultState='com.ibm.rqm.execution.common.state.failed'">
                                    <xsl:value-of
                                        select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelFailed"
                                    />
                                </xsl:when>
                                <xsl:when
                                    test="../../atom:titles/atom:currentResultState='com.ibm.rqm.execution.common.state.blocked'">
                                    <xsl:value-of
                                        select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelBlocked"
                                    />
                                </xsl:when>
                                <xsl:when
                                    test="../../atom:titles/atom:currentResultState='com.ibm.rqm.execution.common.state.complete'">
                                    <xsl:value-of
                                        select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelComplete"
                                    />
                                </xsl:when>
                                <xsl:when
                                    test="../../atom:titles/atom:currentResultState='com.ibm.rqm.execution.common.state.inconclusive'">
                                    <xsl:value-of
                                        select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelInconclusive"
                                    />
                                </xsl:when>
                                <xsl:when
                                    test="../../atom:titles/atom:currentResultState='com.ibm.rqm.execution.common.state.error'">
                                    <xsl:value-of
                                        select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelError"
                                    />
                                </xsl:when>
                                <xsl:otherwise> -- </xsl:otherwise>
                            </xsl:choose>
                        </fo:block>
                    </fo:block>
                </fo:wrapper>
            </xsl:when>
            <xsl:otherwise>
                <fo:block xmlns:fo="http://www.w3.org/1999/XSL/Format">
                    <xsl:if test="(../../componentType='COMPOSITE' )">
                    	<xsl:if test="//pageBreaks = 'true'">
                        	<xsl:attribute name="break-after">page</xsl:attribute>
                        </xsl:if>
                        <xsl:attribute name="id">
                            <xsl:value-of select="ns3:identifier"/>
                        </xsl:attribute>
                    </xsl:if>
                    <fo:block font-size="14pt" space-before="2mm" space-after="2mm"
                        border-after-style="solid" border-after-width="thin">
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelTestExecutionRecord"
                        /> : <xsl:value-of select="ewi:title"/>
                    </fo:block>
                    <fo:block>
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelModifiedDate"
                        /> : <xsl:value-of select="ns3:updated"/>
                    </fo:block>
                    <fo:block>
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/reqLabel_created_by"
                        /> : <xsl:value-of select="ewi:creator"/>
                    </fo:block>
                    <fo:block>
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelOwnedBy"
                        /> : <xsl:value-of select="ns3:owner"/>
                    </fo:block>
                    <fo:block space-before="2mm">
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelTestCase"
                        /> : <xsl:value-of select="../../atom:titles/atom:testCaseTitle"/>
                    </fo:block>
                    <fo:block space-before="2mm">
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelTestPlan"
                        /> : <xsl:choose>
                            <xsl:when test="../../atom:titles/atom:testPlanTitle">
                                <xsl:value-of select="../../atom:titles/atom:testPlanTitle"/>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:value-of
                                    select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelUnassigned"
                                />
                            </xsl:otherwise>
                        </xsl:choose>
                    </fo:block>
                    <fo:block space-before="2mm">
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelTestScript"
                        /> : <xsl:choose>
                            <xsl:when test="../../atom:titles/atom:testScriptTitle">
                                <xsl:value-of select="../../atom:titles/atom:testScriptTitle"/>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:value-of
                                    select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelNone"
                                />
                            </xsl:otherwise>
                        </xsl:choose>
                    </fo:block>
                    <fo:block space-before="2mm">
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelTestEnvironment"
                        /> : <xsl:choose>
                            <xsl:when test="../../atom:titles/atom:configurations/atom:title">
                                <xsl:value-of
                                    select="../../atom:titles/atom:configurations/atom:title"/>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:value-of
                                    select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelUnassigned"
                                />
                            </xsl:otherwise>
                        </xsl:choose>
                    </fo:block>
                    <fo:block space-before="2mm">
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelTestMilestone"
                        /> : <xsl:choose>
                            <xsl:when test="../../atom:titles/atom:testPhaseTitle">
                                <xsl:value-of select="../../atom:titles/atom:testPhaseTitle"/>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:value-of
                                    select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelUnassigned"
                                />
                            </xsl:otherwise>
                        </xsl:choose>
                    </fo:block>
                    <fo:block space-before="2mm">
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelLastResult"
                        />: <xsl:choose>
                            <xsl:when
                                test="../../atom:titles/atom:currentResultState='com.ibm.rqm.execution.common.state.passed'">
                                <xsl:value-of
                                    select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelPassed"
                                />
                            </xsl:when>
                            <xsl:when
                                test="../../atom:titles/atom:currentResultState='com.ibm.rqm.execution.common.state.failed'">
                                <xsl:value-of
                                    select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelFailed"
                                />
                            </xsl:when>
                            <xsl:when
                                test="../../atom:titles/atom:currentResultState='com.ibm.rqm.execution.common.state.blocked'">
                                <xsl:value-of
                                    select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelBlocked"
                                />
                            </xsl:when>
                            <xsl:when
                                test="../../atom:titles/atom:currentResultState='com.ibm.rqm.execution.common.state.complete'">
                                <xsl:value-of
                                    select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelComplete"
                                />
                            </xsl:when>
                            <xsl:when
                                test="../../atom:titles/atom:currentResultState='com.ibm.rqm.execution.common.state.inconclusive'">
                                <xsl:value-of
                                    select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelInconclusive"
                                />
                            </xsl:when>
                            <xsl:when
                                test="../../atom:titles/atom:currentResultState='com.ibm.rqm.execution.common.state.error'">
                                <xsl:value-of
                                    select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelError"
                                />
                            </xsl:when>
                            <xsl:otherwise> -- </xsl:otherwise>
                        </xsl:choose>
                    </fo:block>
                </fo:block>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
</xsl:stylesheet>
