<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:fo="http://www.w3.org/1999/XSL/Format" xmlns:req="http://purl.org/dc/elements/1.1/"
    xmlns:ns2="http://jazz.net/xmlns/alm/qm/v0.1/" xmlns:ns3="http://jazz.net/xmlns/alm/v0.1/"
    xmlns:ns4="http://schema.ibm.com/vega/2008/"
    xmlns:ns5="http://jazz.net/xmlns/alm/qm/v0.1/testscript/v0.1/"
    xmlns:ns6="http://jazz.net/xmlns/alm/qm/v0.1/executionresult/v0.1"
    xmlns:ns7="http://jazz.net/xmlns/alm/qm/v0.1/executionworkitem/v0.1"
    xmlns:ns8="http://jazz.net/xmlns/alm/qm/v0.1/tsl/v0.1/"
    exclude-result-prefixes="ns8 ns7 ns6 ns5 ns4 ns3 ns2 xsl">
    
    <xsl:include href="xhtml-to-xslfo.xsl"/>
    <xsl:include href="wrapping-rules.xsl"/>
    
    <xsl:variable name="lang">
        <xsl:value-of select="locale"/>
    </xsl:variable>
    
    <xsl:template match="transformArgs">
        <xsl:choose>
            <xsl:when test="(componentType='STAND_ALONE') or (componentType='STAND_ALONE_SUMMARY')">
                <fo:page-sequence xmlns:fo="http://www.w3.org/1999/XSL/Format"
                    initial-page-number="1" force-page-count="no-force"
                    master-reference="PageMaster">
                    <fo:static-content flow-name="xsl-region-before">
                        <fo:block font-size="8pt" text-align="right">
                            <fo:page-number/> / <fo:page-number-citation ref-id="{reqNumber}"/>
                        </fo:block>
                    </fo:static-content>
                    <fo:static-content flow-name="xsl-region-after">
                    	<fo:block-container height="0.1in" padding-after="0.07in">
							<fo:block text-align="center">
								<fo:leader leader-pattern='rule' rule-thickness='1pt' leader-length='100%'/>
							</fo:block>
						</fo:block-container>
                        <fo:block text-align="center">
                            <fo:retrieve-marker retrieve-class-name="artifactUri"/>
                        </fo:block>
                    </fo:static-content>
                    <fo:flow flow-name="xsl-region-body">
                        <fo:marker marker-class-name="artifactUri">
                            <fo:block font-size="6pt">
                                <fo:basic-link external-destination="{originUrl}" color="blue"
                                    text-decoration="underline">
                                    <xsl:value-of select="originUrl"/>
                                </fo:basic-link>
                            </fo:block>
                        </fo:marker>
                        <xsl:apply-templates select="descendant::ns2:requirement"/>
                        <fo:block id="{reqNumber}"/>
                    </fo:flow>
                </fo:page-sequence>
            </xsl:when>
            <xsl:otherwise>
                <xsl:apply-templates select="descendant::ns2:requirement"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="ns2:requirement">
        <xsl:choose>
            <xsl:when
                test="(../../componentType='SUMMARY') or (../../componentType='STAND_ALONE_SUMMARY') or (../../componentType='COMPOSITE_SUMMARY')">
                <xsl:choose>
                    <xsl:when
                        test="../../componentType='STAND_ALONE_SUMMARY' or (../../componentType='COMPOSITE_SUMMARY')">
                        <xsl:if test="../../componentType='STAND_ALONE_SUMMARY'">
                            <fo:block font-size="14pt" space-before="6mm" space-after="2mm"
                                    ><xsl:value-of
                                    select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelSummary"
                                />: <xsl:value-of
                                    select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelRequirements"
                                /></fo:block>
                        </xsl:if>
                        <fo:table border="0.5pt solid black" font-size="7pt">
                            <fo:table-header keep-with-next="always" font-size="7pt" font-weight="bold" text-align="center" background-color="rgb(249,245,219)">
                                <!-- id -->
                                <fo:table-cell padding="3pt" border="0.5pt solid black" width="25">
                                    <fo:block-container overflow="hidden">
                                        <fo:block font-weight="bold">
                                            <xsl:value-of select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelId"/>
                                        </fo:block>
                                    </fo:block-container>
                                </fo:table-cell>
                                <!-- status -->
                                <fo:table-cell padding="3pt" border="0.5pt solid black" width="50">
                                    <fo:block-container overflow="hidden">
                                        <fo:block font-weight="bold">
                                            <xsl:apply-templates select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/reqLabel_status"/>
                                        </fo:block>
                                    </fo:block-container>
                                </fo:table-cell>
                                <!-- name -->
                                <fo:table-cell padding="3pt" border="0.5pt solid black" width="75">
                                    <fo:block-container overflow="hidden">
                                        <fo:block font-weight="bold">
                                            <xsl:apply-templates select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelName"/>
                                        </fo:block>
                                    </fo:block-container>
                                </fo:table-cell>
                                <!-- description -->
                                <fo:table-cell padding="3pt" border="0.5pt solid black">
                                    <fo:block-container overflow="hidden">
                                        <fo:block font-weight="bold">
                                            <xsl:apply-templates select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labeldescription"/>
                                        </fo:block>
                                    </fo:block-container>
                                </fo:table-cell>
                                <!-- owner -->
                                <fo:table-cell padding="3pt" border="0.5pt solid black" width="55">
                                    <fo:block-container overflow="hidden">
                                        <fo:block font-weight="bold">
                                            <xsl:apply-templates select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelOwner"/>
                                        </fo:block>
                                    </fo:block-container>
                                </fo:table-cell>
                            </fo:table-header>
                            <fo:table-body>
                                <fo:table-row keep-together="always">
                                    <!-- id -->
                                    <fo:table-cell padding="3pt" border="0.5pt solid black">
                                        <fo:block-container overflow="hidden">
                                            <fo:block>
                                                <xsl:value-of select="//ns2:generatedId"/>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <!-- status -->
                                    <fo:table-cell padding="3pt" border="0.5pt solid black">
                                        <fo:block-container overflow="hidden">
                                            <fo:block>
                                        		<xsl:choose>
                                            		<xsl:when test="ns3:suspectStatus='updated'"> Updated </xsl:when>
                                            		<xsl:when test="ns3:suspectStatus='removed'"> Removed </xsl:when>
                                            		<xsl:when test="ns3:suspectStatus='normal'"> Normal </xsl:when>
                                            		<xsl:when test="ns3:suspectStatus='new'"> Normal </xsl:when>
                                            		<xsl:when test="not(ns3:suspectStatus)"> Normal </xsl:when>
                                        		</xsl:choose>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <!-- name -->
                                    <fo:table-cell padding="3pt" border="0.5pt solid black">
                                        <fo:block-container overflow="hidden">
                                            <fo:block >
                                                <xsl:apply-templates select="req:title"/>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <!-- description -->
                                    <fo:table-cell padding="3pt" border="0.5pt solid black">
                                        <fo:block-container overflow="hidden">
                                            <fo:block >
                                                <xsl:apply-templates select="req:description"/>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <!-- owner -->
                                    <fo:table-cell padding="3pt" border="0.5pt solid black">
                                        <fo:block-container overflow="hidden">
                                            <fo:block >
                                                <xsl:apply-templates select="ns3:owner"/>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                </fo:table-row>
                            </fo:table-body>
                        </fo:table>
                    </xsl:when>
                    <xsl:otherwise>
                        <fo:table-row keep-together="always">
                            <!-- id -->
                            <fo:table-cell padding="3pt" border="0.5pt solid black">
                                <fo:block-container overflow="hidden">
                                    <fo:block>
                                    	<xsl:value-of select="//ns2:generatedId"/>
                                    </fo:block>
                                </fo:block-container>
                            </fo:table-cell>
                            <!-- status -->
                            <fo:table-cell padding="3pt" border="0.5pt solid black">
                                <fo:block-container overflow="hidden">
                                    <fo:block>
                                        <xsl:choose>
                                            <xsl:when test="ns3:suspectStatus='updated'"> Updated </xsl:when>
                                            <xsl:when test="ns3:suspectStatus='removed'"> Removed </xsl:when>
                                            <xsl:when test="ns3:suspectStatus='normal'"> Normal </xsl:when>
                                            <xsl:when test="ns3:suspectStatus='new'"> Normal </xsl:when>
                                            <xsl:when test="not(ns3:suspectStatus)"> Normal </xsl:when>
                                        </xsl:choose>
                                    </fo:block>
                                </fo:block-container>
                            </fo:table-cell>
                            <!-- title -->
                            <fo:table-cell padding="3pt" border="0.5pt solid black">
                                <fo:block-container overflow="hidden">
                                    <fo:block>
                                        <xsl:apply-templates select="req:title"/>
                                    </fo:block>
                                </fo:block-container>
                            </fo:table-cell>
                            <!-- description -->
                            <fo:table-cell padding="3pt" border="0.5pt solid black">
                                <fo:block-container overflow="hidden">
                                    <fo:block>
                                        <xsl:apply-templates select="req:description"/>
                                    </fo:block>
                                </fo:block-container>
                            </fo:table-cell>
                            <!-- owner -->
                            <fo:table-cell padding="3pt" border="0.5pt solid black">
                                <fo:block-container overflow="hidden">
                                    <fo:block>
                                        <xsl:apply-templates select="ns3:owner"/>
                                    </fo:block>
                                </fo:block-container>
                            </fo:table-cell>
                        </fo:table-row>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="(../../componentType='COMPOSITE') and (../../recursive)">
                <fo:wrapper>
                    <fo:table-row keep-together="always">
                        <!-- id -->
                        <fo:table-cell padding="3pt" border="0.5pt solid black" width="25">
                            <fo:block-container overflow="hidden">
                                <fo:block>
                                    <fo:basic-link internal-destination="{req:identifier}" color="blue" text-decoration="underline">
                                    	<xsl:value-of select="//ns2:generatedId"/>
                                    </fo:basic-link>
                                </fo:block>
                            </fo:block-container>
                        </fo:table-cell>
                        <!-- status -->
                        <fo:table-cell padding="3pt" border="0.5pt solid black">
                            <fo:block-container overflow="hidden">
                                <fo:block>
                                    <xsl:choose>
                                        <xsl:when test="ns3:suspectStatus='updated'"> Updated </xsl:when>
                                        <xsl:when test="ns3:suspectStatus='removed'"> Removed </xsl:when>
                                        <xsl:when test="ns3:suspectStatus='normal'"> Normal </xsl:when>
                                        <xsl:when test="ns3:suspectStatus='new'"> Normal </xsl:when>
                                        <xsl:when test="not(ns3:suspectStatus)"> Normal </xsl:when>
                                    </xsl:choose>
                                </fo:block>
                            </fo:block-container>
                        </fo:table-cell>
                        <!-- name -->
                        <fo:table-cell padding="3pt" border="0.5pt solid black">
                            <fo:block-container overflow="hidden">
                                <fo:block>
                                    <xsl:apply-templates select="req:title"/>
                                </fo:block>
                            </fo:block-container>
                        </fo:table-cell>
                        <!-- description -->
                        <fo:table-cell padding="3pt" border="0.5pt solid black">
                            <fo:block-container overflow="hidden">
                                <fo:block>
                                    <xsl:apply-templates select="req:description"/>
                                </fo:block>
                            </fo:block-container>
                        </fo:table-cell>
                        <!-- owner -->
                        <fo:table-cell padding="3pt" border="0.5pt solid black">
                            <fo:block-container overflow="hidden">
                                <fo:block>
                                    <xsl:apply-templates select="ns3:owner"/>
                                </fo:block>
                            </fo:block-container>
                        </fo:table-cell>
                    </fo:table-row>
                <fo:block xmlns:fo="http://www.w3.org/1999/XSL/Format">
                    <xsl:if test="(../../componentType='COMPOSITE')">
                        <xsl:attribute name="id">
                        	<xsl:value-of select="//ns2:generatedId"/>
                        </xsl:attribute>
                    </xsl:if>
                    <fo:block font-size="14pt" space-after="4mm" border-after-style="solid"
                        border-after-width="thin">
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelRequirement"/>
                        <xsl:value-of select="//ns2:generatedId"/>
                    </fo:block>
                    <fo:block font-size="10pt">
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/reqLabel_status"
                        /> : <xsl:choose>
                            <xsl:when test="ns3:state='authoring.s3'"> Resolved By <xsl:value-of
                                    select="ns2:resolver"/>
                            </xsl:when>
                            <xsl:when test="ns3:state='authoring.s1'"> New </xsl:when>
                            <xsl:when test="ns3:state='authoring.s2'"> In Progress </xsl:when>
                            <xsl:when test="ns3:state='authoring.s6'"> Reopened </xsl:when>
                        </xsl:choose>
                    </fo:block>
                    <fo:block font-size="10pt">
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/reqLabel_created_on"
                        /> : <xsl:value-of select="ns2:creationtime"/>
                    </fo:block>
                    <fo:block font-size="10pt">
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/reqLabel_created_by"
                        /> : <xsl:value-of select="req:creator"/>
                    </fo:block>
                    <fo:block font-size="10pt">
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelOwnedBy"
                        /> : <xsl:value-of select="ns3:owner"/>
                    </fo:block>
                    <fo:block font-size="10pt">
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelModifiedDate"
                        /> : <xsl:value-of select="ns3:updated"/>
                    </fo:block>
                    <fo:block font-size="10pt" space-before="4mm">
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelSeverity"
                        /> : <xsl:value-of select="ns2:severity"/>
                    </fo:block>
                    <fo:block font-size="10pt">
                    	<xsl:value-of
                   			select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/reqLabel_priority"
                        /> : <xsl:for-each select="ns2:customAttributes/ns2:customAttribute">
                        		 <xsl:if test="ns2:name='Requirement Priority'">
                        		 	<xsl:value-of select="ns2:value" />
                        		 </xsl:if>
                        	 </xsl:for-each>
                    </fo:block>
                    <fo:block font-size="10pt">
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelReviewStatus"
                        /> : <xsl:value-of select="ns2:reviewStatus"/>
                    </fo:block>
                    <fo:block font-size="10pt">
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelTags"
                        /> : <xsl:for-each select="ns2:tags/tag">
                        	 	<xsl:value-of select="text" />
                        	 	<xsl:if test="position()!=last()">
        							<xsl:text>, </xsl:text>
      							</xsl:if>                        	 	
                        	 </xsl:for-each>
                    </fo:block>
                    <fo:block font-size="10pt" space-before="4mm" space-after="4mm">
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labeldescription"
                        /> : <xsl:value-of select="req:description"/>
                    </fo:block>
                    <xsl:apply-templates select="ns2:comments"/>
                </fo:block>
                </fo:wrapper>
            </xsl:when>
            <xsl:otherwise>
                <fo:block xmlns:fo="http://www.w3.org/1999/XSL/Format">
                    <xsl:if test="(../../componentType='COMPOSITE')">
                        <xsl:attribute name="id">
                            <xsl:value-of select="req:identifier"/>
                        </xsl:attribute>
                    </xsl:if>
                    <fo:block font-size="14pt" space-after="4mm" border-after-style="solid"
                        border-after-width="thin">
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelRequirement"/>
                        <xsl:value-of select="//ns2:generatedId"/> :
                        <xsl:value-of select="req:title"/>
                    </fo:block>
                    <fo:block font-size="10pt">
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/reqLabel_status"
                        /> : <xsl:choose>
                            <xsl:when test="ns3:state='authoring.s3'"> Resolved By <xsl:value-of
                                    select="ns2:resolver"/>
                            </xsl:when>
                            <xsl:when test="ns3:state='authoring.s1'"> New </xsl:when>
                            <xsl:when test="ns3:state='authoring.s2'"> In Progress </xsl:when>
                            <xsl:when test="ns3:state='authoring.s6'"> Reopened </xsl:when>
                        </xsl:choose>
                    </fo:block>
                    <fo:block font-size="10pt">
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/reqLabel_created_on"
                        /> : <xsl:value-of select="ns2:creationtime"/>
                    </fo:block>
                    <fo:block font-size="10pt">
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/reqLabel_created_by"
                        /> : <xsl:value-of select="req:creator"/>
                    </fo:block>
                    <fo:block font-size="10pt">
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelOwnedBy"
                        /> : <xsl:value-of select="ns3:owner"/>
                    </fo:block>
                    <fo:block font-size="10pt">
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelModifiedDate"
                        /> : <xsl:value-of select="ns3:updated"/>
                    </fo:block>
                    <fo:block font-size="10pt" space-before="4mm">
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelSeverity"
                        /> : <xsl:value-of select="ns2:severity"/>
                    </fo:block>
                    <fo:block font-size="10pt">
                    	<xsl:value-of
                   			select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/reqLabel_priority"
                        /> : <xsl:for-each select="ns2:customAttributes/ns2:customAttribute">
                        		 <xsl:if test="ns2:name='Requirement Priority'">
                        		 	<xsl:value-of select="ns2:value" />
                        		 </xsl:if>
                        	 </xsl:for-each>
                    </fo:block>
                    <fo:block font-size="10pt">
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelReviewStatus"
                        /> : <xsl:value-of select="ns2:reviewStatus"/>
                    </fo:block>
                    <fo:block font-size="10pt">
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelTags"
                        /> : <xsl:for-each select="ns2:tags/tag">
                        	 	<xsl:value-of select="text" />
                        	 	<xsl:if test="position()!=last()">
        							<xsl:text>, </xsl:text>
      							</xsl:if>                        	 	
                        	 </xsl:for-each>
                    </fo:block>
                    <fo:block font-size="10pt" space-before="4mm" space-after="4mm">
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labeldescription"
                        /> : <xsl:value-of select="req:description"/>
                    </fo:block>
                    <xsl:apply-templates select="ns2:comments"/>
                </fo:block>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="ns2:comments">
        <xsl:if test="comment">
            <fo:block xmlns:fo="http://www.w3.org/1999/XSL/Format">
                <fo:table>
                    <fo:table-column column-width="10mm"/>
                    <fo:table-column/>
                    <fo:table-column/>
                    <fo:table-body>
                        <fo:table-row keep-together="always" background-color="#E5EFF5">
                            <fo:table-cell keep-together.within-column="always" padding="3pt"
                                number-columns-spanned="3" border="0.5pt solid black">
                                <fo:block-container overflow="hidden">
                                    <fo:block font-size="10pt" space-after="4mm">
                                        <xsl:value-of
                                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/reqLabel_discussion"
                                        />: ( <xsl:value-of select="count(comment)"/>
                                        <xsl:value-of
                                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/reqLabel_comments"
                                        /> ) </fo:block>
                                </fo:block-container>
                            </fo:table-cell>
                        </fo:table-row>
                        <xsl:for-each select="comment">
                            <fo:table-row keep-with-next="always" keep-together="always"
                                background-color="#F3F3F3">
                                <fo:table-cell keep-together.within-column="always" padding="3pt"
                                    border-left="0.5pt solid black">
                                    <fo:block-container overflow="hidden">
                                        <fo:block font-size="10pt">
                                            <xsl:number/> . </fo:block>
                                    </fo:block-container>
                                </fo:table-cell>
                                <fo:table-cell keep-together.within-column="always" padding="3pt">
                                    <fo:block-container overflow="hidden">
                                        <fo:block font-size="10pt" text-align="left">
                                            <xsl:value-of select="creator"/>
                                        </fo:block>
                                    </fo:block-container>
                                </fo:table-cell>
                                <fo:table-cell keep-together.within-column="always" padding="3pt"
                                    border-right="0.5pt solid black">
                                    <fo:block-container overflow="hidden">
                                        <fo:block>
                                            <xsl:value-of select="creationDate"/>
                                        </fo:block>
                                    </fo:block-container>
                                </fo:table-cell>
                            </fo:table-row>
                            <fo:table-row keep-together="always">
                                <fo:table-cell keep-together.within-column="always" padding="3pt"
                                    border="0.5pt solid black" number-columns-spanned="3">
                                    <fo:block-container overflow="hidden">
                                        <fo:block font-size="10pt" text-align="left">
                                            <xsl:apply-templates select="content"/>
                                        </fo:block>
                                    </fo:block-container>
                                </fo:table-cell>
                            </fo:table-row>
                        </xsl:for-each>
                    </fo:table-body>
                </fo:table>
            </fo:block>
        </xsl:if>
    </xsl:template>
    
    <xsl:template match="content">
        <xsl:apply-templates select="*|text()"/>
    </xsl:template>
</xsl:stylesheet>
