<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:fo="http://www.w3.org/1999/XSL/Format"
    xmlns:ns8="http://jazz.net/xmlns/alm/qm/v0.1/tsl/v0.1/"
    xmlns:ns7="http://jazz.net/xmlns/alm/qm/v0.1/executionworkitem/v0.1"
    xmlns:ns6="http://jazz.net/xmlns/alm/qm/v0.1/executionresult/v0.1"
    xmlns:ns5="http://jazz.net/xmlns/alm/qm/v0.1/testscript/v0.1/"
    xmlns:ns3="http://purl.org/dc/elements/1.1/" xmlns:ns4="http://jazz.net/xmlns/alm/v0.1/"
    xmlns:ns2="http://schema.ibm.com/vega/2008/" xmlns:wi="http://jazz.net/xmlns/alm/qm/v0.1/"
    exclude-result-prefixes="ns8 ns7 ns6 ns5 ns4 ns3 ns2 wi xsl">
    <xsl:variable name="lang">
        <xsl:value-of select="locale"/>
    </xsl:variable>
    <xsl:template match="transformArgs">
        <xsl:choose>
            <xsl:when
                test="(componentType='STAND_ALONE') or (../../componentType='STAND_ALONE_SUMMARY')">
                <fo:page-sequence xmlns:fo="http://www.w3.org/1999/XSL/Format"
                    initial-page-number="1" force-page-count="no-force"
                    master-reference="PageMaster">
                    <fo:static-content flow-name="xsl-region-before">
                        <fo:block font-size="8pt" text-align="right">
                            <fo:page-number/> / <fo:page-number-citation ref-id="{reqNumber}"/>
                        </fo:block>
                    </fo:static-content>
                    <fo:static-content flow-name="xsl-region-after">
                        <fo:block text-align="center">
                            <fo:retrieve-marker retrieve-class-name="artifactUri"/>
                        </fo:block>
                    </fo:static-content>
                    <fo:flow flow-name="xsl-region-body">
                        <fo:marker marker-class-name="artifactUri">
                            <fo:block font-size="6pt">
                                <fo:basic-link external-destination="{originUrl}" color="blue"
                                    text-decoration="underline">
                                    <xsl:value-of select="originUrl"/>
                                </fo:basic-link>
                            </fo:block>
                        </fo:marker>
                        <xsl:apply-templates select="descendant::wi:workitem"/>
                        <fo:block id="{reqNumber}"/>
                    </fo:flow>
                </fo:page-sequence>
            </xsl:when>
            <xsl:otherwise>
                <xsl:apply-templates select="descendant::wi:workitem"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <xsl:template match="wi:workitem">
        <xsl:choose>
            <xsl:when
                test="(../../componentType='SUMMARY') or (../../componentType='STAND_ALONE_SUMMARY') or (../../componentType='COMPOSITE_SUMMARY')">
                <xsl:choose>
                    <xsl:when
                        test="../../componentType='STAND_ALONE_SUMMARY' or (../../componentType='COMPOSITE_SUMMARY')">
                        <xsl:if test="../../componentType='STAND_ALONE_SUMMARY'">
                            <fo:block font-size="14pt" space-before="6mm" space-after="2mm"
                                    ><xsl:value-of
                                    select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelSummary"
                                />: <xsl:value-of
                                    select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelWorkItems"
                                /></fo:block>
                        </xsl:if>
                        <fo:table border="0.5pt solid black" text-align="left">
                            <fo:table-header>
                                <fo:table-row keep-together="always"
                                    background-color="rgb(249,245,219)">
                                    <fo:table-cell keep-together.within-column="always" width="25mm"
                                        padding="3pt" border="0.5pt solid black">
                                        <fo:block-container overflow="hidden">
                                            <fo:block font-weight="bold">
                                                <xsl:value-of
                                                  select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelType"
                                                />
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <fo:table-cell keep-together.within-column="always" width="7mm"
                                        padding="3pt" border="0.5pt solid black">
                                        <fo:block-container overflow="hidden">
                                            <fo:block font-weight="bold">
                                                <xsl:value-of
                                                  select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelId"
                                                />
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <fo:table-cell keep-together.within-column="always"
                                        padding="3pt" border="0.5pt solid black">
                                        <fo:block-container overflow="hidden">
                                            <fo:block font-weight="bold">
                                                <xsl:value-of
                                                  select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelSummary"
                                                />
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <fo:table-cell keep-together.within-column="always" width="20mm"
                                        padding="3pt" border="0.5pt solid black">
                                        <fo:block-container overflow="hidden">
                                            <fo:block font-weight="bold">
                                                <xsl:value-of
                                                  select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelState"
                                                />
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                </fo:table-row>
                            </fo:table-header>
                            <fo:table-body>
                                <fo:table-row keep-together="always">
                                    <fo:table-cell keep-together.within-column="always"
                                        padding="3pt" border="0.5pt solid black">
                                        <fo:block-container overflow="hidden">
                                            <fo:block font-size="10pt">
                                                <xsl:choose>
                                                  <xsl:when test="wi:type='defect'"> Defect </xsl:when>
                                                  <xsl:when test="wi:type='task'"> Task </xsl:when>
                                                  <xsl:when test="wi:type='authoring'"> Task-Quality </xsl:when>
                                                  <xsl:when test="wi:type='review'"> Task-Review </xsl:when>
                                                  <xsl:when test="wi:type='requirement'">
                                                  Requirement </xsl:when>
                                                </xsl:choose>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <fo:table-cell keep-together.within-column="always"
                                        padding="3pt" border="0.5pt solid black">
                                        <fo:block-container overflow="hidden">
                                            <fo:block font-size="10pt">
                                                <xsl:variable name="ident" select="ns3:identifier"/>
                                                <xsl:value-of
                                                  select="substring-after($ident,'workitem:')"/>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <fo:table-cell keep-together.within-column="always"
                                        padding="3pt" border="0.5pt solid black">
                                        <fo:block-container overflow="hidden">
                                            <fo:block font-size="10pt">
                                                <xsl:value-of select="wi:summary"/>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <fo:table-cell keep-together.within-column="always"
                                        padding="3pt" border="0.5pt solid black">
                                        <fo:block-container overflow="hidden">
                                            <fo:block font-size="10pt">
                                                <xsl:value-of select="*[local-name()='state']"/>
                                            </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                </fo:table-row>
                            </fo:table-body>
                        </fo:table>
                    </xsl:when>
                    <xsl:otherwise>
                        <fo:table-row keep-together="always">
                            <fo:table-cell keep-together.within-column="always" padding="3pt"
                                border="0.5pt solid black">
                                <fo:block-container overflow="hidden">
                                    <fo:block font-size="10pt">
                                        <xsl:choose>
                                            <xsl:when test="wi:type='defect'"> Defect </xsl:when>
                                            <xsl:when test="wi:type='task'"> Task </xsl:when>
                                            <xsl:when test="wi:type='authoring'"> Task-Quality </xsl:when>
                                            <xsl:when test="wi:type='review'"> Task-Review </xsl:when>
                                            <xsl:when test="wi:type='requirement'"> Requirement
                                            </xsl:when>
                                        </xsl:choose>
                                    </fo:block>
                                </fo:block-container>
                            </fo:table-cell>
                            <fo:table-cell keep-together.within-column="always" padding="3pt"
                                border="0.5pt solid black">
                                <fo:block-container overflow="hidden">
                                    <fo:block font-size="10pt">
                                        <xsl:variable name="ident" select="ns3:identifier"/>
                                        <xsl:value-of select="substring-after($ident,'workitem:')"/>
                                    </fo:block>
                                </fo:block-container>
                            </fo:table-cell>
                            <fo:table-cell keep-together.within-column="always" padding="3pt"
                                border="0.5pt solid black">
                                <fo:block-container overflow="hidden">
                                    <fo:block font-size="10pt">
                                        <xsl:value-of select="wi:summary"/>
                                    </fo:block>
                                </fo:block-container>
                            </fo:table-cell>
                            <fo:table-cell keep-together.within-column="always" padding="3pt"
                                border="0.5pt solid black">
                                <fo:block-container overflow="hidden">
                                    <fo:block font-size="10pt">
                                        <xsl:value-of select="*[local-name()='state']"/>
                                    </fo:block>
                                </fo:block-container>
                            </fo:table-cell>
                        </fo:table-row>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="(../../componentType='COMPOSITE') and (../../recursive)">
                <fo:wrapper>
                    <fo:table-row keep-together="always">
                        <fo:table-cell keep-together.within-column="always" padding="3pt"
                            border="0.5pt solid black">
                            <fo:block-container overflow="hidden">
                                <fo:block font-size="10pt">
                                    <xsl:choose>
                                        <xsl:when test="wi:type='defect'"> Defect </xsl:when>
                                        <xsl:when test="wi:type='task'"> Task </xsl:when>
                                        <xsl:when test="wi:type='authoring'"> Task-Quality </xsl:when>
                                        <xsl:when test="wi:type='review'"> Task-Review </xsl:when>
                                        <xsl:when test="wi:type='requirement'"> Requirement
                                        </xsl:when>
                                    </xsl:choose>
                                </fo:block>
                            </fo:block-container>
                        </fo:table-cell>
                        <fo:table-cell keep-together.within-column="always" padding="3pt"
                            border="0.5pt solid black">
                            <fo:block-container overflow="hidden">
                                <fo:block font-size="10pt">
                                    <fo:basic-link internal-destination="{ns3:identifier}"
                                        color="blue" text-decoration="underline">
                                        <xsl:variable name="ident" select="ns3:identifier"/>
                                        <xsl:value-of select="substring-after($ident,'workitem:')"/>
                                    </fo:basic-link>
                                </fo:block>
                            </fo:block-container>
                        </fo:table-cell>
                        <fo:table-cell keep-together.within-column="always" padding="3pt"
                            border="0.5pt solid black">
                            <fo:block-container overflow="hidden">
                                <fo:block font-size="10pt">
                                    <xsl:value-of select="wi:summary"/>
                                </fo:block>
                            </fo:block-container>
                        </fo:table-cell>
                        <fo:table-cell keep-together.within-column="always" padding="3pt"
                            border="0.5pt solid black">
                            <fo:block-container overflow="hidden">
                                <fo:block font-size="10pt">
                                    <xsl:value-of select="*[local-name()='state']"/>
                                </fo:block>
                            </fo:block-container>
                        </fo:table-cell>
                    </fo:table-row>
                    <fo:block xmlns:fo="http://www.w3.org/1999/XSL/Format">
                        <xsl:attribute name="id">
                            <xsl:value-of select="ns3:identifier"/>
                        </xsl:attribute>
                        <fo:block font-size="14pt" space-before="2mm" space-after="2mm"
                            border-after-style="solid" border-after-width="thin">
                            <xsl:value-of
                                select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelWorkitem"
                            />: <xsl:value-of select="ns3:title"/>
                        </fo:block>
                        <fo:block>
                            <xsl:value-of
                                select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelType"
                            />: <xsl:choose>
                                <xsl:when test="wi:type='defect'"> Defect </xsl:when>
                                <xsl:when test="wi:type='task'"> Task </xsl:when>
                                <xsl:when test="wi:type='authoring'"> Task-Quality </xsl:when>
                                <xsl:when test="wi:type='review'"> Task-Review </xsl:when>
                                <xsl:when test="wi:type='requirement'"> Requirement </xsl:when>
                            </xsl:choose>
                        </fo:block>
                        <fo:block space-after="2mm">
                            <xsl:value-of
                                select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/reqLabel_status"
                            /> : <xsl:value-of select="ns4:state"/>
                        </fo:block>
                        <fo:block>
                            <xsl:value-of
                                select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/reqLabel_created_by"
                            /> : <xsl:value-of select="ns3:creator"/>
                        </fo:block>
                        <fo:block>
                            <xsl:value-of
                                select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelOwnedBy"
                            /> : <xsl:value-of select="ns4:owner"/>
                        </fo:block>
                        <fo:block space-after="2mm">
                            <xsl:value-of
                                select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelModifiedDate"
                            /> : <xsl:value-of select="ns4:updated"/>
                        </fo:block>
                        <fo:block>
                            <xsl:value-of
                                select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labeldescription"
                            />: <xsl:value-of select="ns3:description"/>
                        </fo:block>
                    </fo:block>
                </fo:wrapper>
            </xsl:when>
            <xsl:otherwise>
                <fo:block xmlns:fo="http://www.w3.org/1999/XSL/Format">
                    <xsl:if test="(../../componentType='COMPOSITE' )">
                        <xsl:attribute name="id">
                            <xsl:value-of select="ns3:identifier"/>
                        </xsl:attribute>
                        <xsl:attribute name="break-after">page</xsl:attribute>
                    </xsl:if>
                    <fo:block font-size="14pt" space-before="2mm" space-after="2mm"
                        border-after-style="solid" border-after-width="thin">
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelWorkitem"
                        />: <xsl:value-of select="ns3:title"/>
                    </fo:block>
                    <fo:block>
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelType"
                        />: <xsl:choose>
                            <xsl:when test="wi:type='defect'"> Defect </xsl:when>
                            <xsl:when test="wi:type='task'"> Task </xsl:when>
                            <xsl:when test="wi:type='authoring'"> Task-Quality </xsl:when>
                            <xsl:when test="wi:type='review'"> Task-Review </xsl:when>
                            <xsl:when test="wi:type='requirement'"> Requirement </xsl:when>
                        </xsl:choose>
                    </fo:block>
                    <fo:block space-after="2mm">
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/reqLabel_status"
                        /> : <xsl:value-of select="ns4:state"/>
                    </fo:block>
                    <fo:block>
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/reqLabel_created_by"
                        /> : <xsl:value-of select="ns3:creator"/>
                    </fo:block>
                    <fo:block>
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelOwnedBy"
                        /> : <xsl:value-of select="ns4:owner"/>
                    </fo:block>
                    <fo:block space-after="2mm">
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labelModifiedDate"
                        /> : <xsl:value-of select="ns4:updated"/>
                    </fo:block>
                    <fo:block>
                        <xsl:value-of
                            select="document('fopmessages.xml')/fopmessages//*[name()=$lang]/labeldescription"
                        />: <xsl:value-of select="ns3:description"/>
                    </fo:block>
                </fo:block>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
</xsl:stylesheet>
