<xsl:stylesheet version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format"
	xmlns:atom="http://www.w3.org/2005/Atom" xmlns:fox="http://xml.apache.org/fop/extensions"
	xmlns:xhtml="http://www.w3.org/1999/xhtml"
	exclude-result-prefixes="xhtml xsl">
	
	<xsl:variable name="lang">
		<xsl:value-of select="locale" />
	</xsl:variable>
	
	<xsl:template match="transformArgs">
		<xsl:choose>
			<xsl:when
				test="(componentType='STAND_ALONE') or (componentType='STAND_ALONE_SUMMARY')">
				<fo:page-sequence xmlns:fo="http://www.w3.org/1999/XSL/Format"
					initial-page-number="1" force-page-count="no-force"
					master-reference="PageMaster">
					<fo:static-content flow-name="xsl-region-before">
						<fo:block font-size="8pt" text-align="right">
							<fo:page-number />
							/
							<fo:page-number-citation ref-id="{reqNumber}" />
						</fo:block>
					</fo:static-content>
					<fo:static-content flow-name="xsl-region-after">
						<fo:block text-align="center">
							<fo:retrieve-marker retrieve-class-name="artifactUri" />
						</fo:block>
					</fo:static-content>
					<fo:flow flow-name="xsl-region-body">
						<fo:marker marker-class-name="artifactUri">
							<fo:block font-size="6pt">
								<fo:basic-link external-destination="{originUrl}"
									color="blue" text-decoration="underline">
									<xsl:value-of select="originUrl" />
								</fo:basic-link>
							</fo:block>
						</fo:marker>
						<xsl:apply-templates select="descendant::atom:feed" />
						<fo:block id="{reqNumber}"></fo:block>
					</fo:flow>
				</fo:page-sequence>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="descendant::atom:feed" />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	
	<xsl:template match="atom:feed">
		<fo:block font-size="14pt" space-before="2mm"
			space-after="2mm" border-after-style="solid" border-after-width="thin">
			<xsl:value-of select="atom:title"/>
		</fo:block>
		<xsl:apply-templates select="atom:entry"/>
	</xsl:template>
	
	<xsl:template match="atom:entry">
		<fo:block space-after="4mm">
			<fo:block font-weight="bold">
				<xsl:value-of select="atom:title"/>
			</fo:block>
			<fo:block space-after="2mm">
				<xsl:value-of select="atom:updated"/>
			</fo:block>
			<xsl:apply-templates select="atom:summary"/>
		</fo:block>
	</xsl:template>
	
	<xsl:template match="atom:summary[@type='xhtml']">
		<xsl:apply-templates />
	</xsl:template>
	
	<xsl:template match="atom:summary[@type='html']">
		<fo:block text-align="center" font-weight="bold">
			The HTML was not well formed the PDF cannot render this section.
		</fo:block>
	</xsl:template>
	
	<xsl:template match="xhtml:div">
		<xsl:apply-templates />
	</xsl:template>
	
	<xsl:template match="xhtml:p">
		<fo:block><fo:leader></fo:leader></fo:block>
		<xsl:apply-templates />
	</xsl:template>
	
	<xsl:template match="xhtml:table">
		<fo:table>
			<xsl:attribute name="border">
				<xsl:value-of select="concat(@border,'pt solid black')"/>
			</xsl:attribute>
			<xsl:attribute name="padding">
				<xsl:value-of select="@cellpadding"/>
			</xsl:attribute>
			<fo:table-header>
				<fo:table-row>
					<xsl:apply-templates select="xhtml:tr/xhtml:th"></xsl:apply-templates>
				</fo:table-row>
			</fo:table-header>
			<fo:table-body>
				<xsl:apply-templates select="xhtml:tr"></xsl:apply-templates>
			</fo:table-body>
		</fo:table>
	</xsl:template>
	
	<xsl:template match="xhtml:th">
		<fo:table-cell font-weight="bold" border="from-nearest-specified-value()" padding="from-nearest-specified-value()">
			<xsl:if test="@align">
				<xsl:attribute name="text-align">
					<xsl:value-of select="@align" />
				</xsl:attribute>
			</xsl:if>
			<fo:block><xsl:value-of select="."/></fo:block></fo:table-cell>
	</xsl:template>
	
	<xsl:template match="xhtml:tr">
		<xsl:if test="xhtml:td">
			<fo:table-row><xsl:apply-templates select="xhtml:td" /></fo:table-row>
		</xsl:if>
	</xsl:template>
	
	<xsl:template match="xhtml:td">
		<fo:table-cell border="from-nearest-specified-value()"  padding="from-nearest-specified-value()">
			<xsl:if test="@colspan">
				<xsl:attribute name="number-columns-spanned">
					<xsl:value-of select="@colspan" />
				</xsl:attribute>
			</xsl:if>
			<xsl:if test="@align">
				<xsl:attribute name="text-align">
					<xsl:value-of select="@align" />
				</xsl:attribute>
			</xsl:if>
			
			<xsl:variable name="label">
				<xsl:value-of select="." />
			</xsl:variable>
			
			<fo:block-container overflow="hidden">
				<fo:block>
					<xsl:choose>
						<xsl:when
							test="document('fopmessages.xml')/fopmessages//*[name()=$lang]//*[name()=$label]">
							<xsl:value-of
								select="document('fopmessages.xml')/fopmessages//*[name()=$lang]//*[name()=$label]" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:apply-templates />
						</xsl:otherwise>
					</xsl:choose>
				</fo:block>	
			</fo:block-container>
		</fo:table-cell>
	</xsl:template>
	
	<xsl:template match="xhtml:br">
	 <fo:block></fo:block>
	</xsl:template>
	
	<xsl:template match="xhtml:style">
	</xsl:template>
	
</xsl:stylesheet>