/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rqm.defects.web.client.ChangeRequestManager");

dojo.require("com.ibm.rqm.defects.web.client.AbstractOSLCManager");
dojo.require("com.ibm.rqm.defects.web.client.OSLCCmProvider");

(function(){
    var LINK_TYPE_AFFECT_ER = "calm:affectsExecutionResult";
    var LINK_TYPE_BLOCK_ER = "calm:blocksTestExecutionRecord";
    var LINK_TYPE_TESTED_BY = "calm:testedByTestCase";
    
    var PROVIDER_LINK_TYPE_CQREST = "cqrest";
    var PROJECT_LINK_TYPE_TESTS = "tests";
    dojo.declare("com.ibm.rqm.defects.web.client.ChangeRequestManager", com.ibm.rqm.defects.web.client.AbstractOSLCManager, {
        /* implementation */
        getAPIConstructor: function(){
            return com.ibm.rqm.defects.web.client.OSLCCmProvider
        },
        
        getProjectLinkType: function(){
            return PROJECT_LINK_TYPE_TESTS;
        },
        
        __sentinel: null // terminates this class definition
    });
    
    var MANAGER_INSTANCE = null;
    
    /**
     * <p>
     * Example usage:
     * </p>
     * <pre>
     * dojo.require("com.ibm.rqm.defects.web.client.ChangeRequestManager");
     *
     * var serviceProviderManager = com.ibm.rqm.defects.web.client.ChangeRequestManager.getInstance();
     *
     * // discover dialog uri
     * serviceProviderManager.getCurrentServiceProvider(dojo.hitch(this, function(currentServiceProvider) {
     * 		if(currentServiceProvider) {
     * 			var creationDialogUri = currentServiceProvider.providerUris.defaultCreationDialogURI;
     * 			// create an item in the creation dialog
     * 			var selectionDialogUri = currentServiceProvider.providerUris.selectionDialogURI;
     * 			// select an item from the selection dialog
     * 		} else {
     * 			// Neither cq rest nor RTC project area is configured.
     * 		}
     * }));
     * // invoke CM APIs
     * serviceProviderManager.getCurrentServiceProvider(dojo.hitch(this, function(currentServiceProvider) {
     * 		if(currentServiceProvider) {
     * 			//post a new link to the CM service provider
     * 			currentServiceProvider.postLink(linkDescription, responseCallback);
     * 		} else {
     * 			// Neither cq rest nor RTC project area is configured.
     * 		}
     * }));
     * // invoke CM filters for query
     * serviceProviderManager.getCurrentServiceProvider(dojo.hitch(this, function(currentServiceProvider) {
     * 		if(currentServiceProvider) {
     * 			// blocked change request filter uri
     * 			var blockedFilterUri = currentServiceProvider.providerUris.blockedFilterUri
     * 		} else {
     * 			// Neither cq rest nor RTC project area is configured.
     * 		}
     * }));
     *
     * </pre>
     */
    com.ibm.rqm.defects.web.client.ChangeRequestManager.getInstance = function(){
        if (MANAGER_INSTANCE == null) {
            var instance = new com.ibm.rqm.defects.web.client.ChangeRequestManager();
            MANAGER_INSTANCE = instance;
        }
        
        return MANAGER_INSTANCE;
    };
    
    com.ibm.rqm.defects.web.client.ChangeRequestManager.LINK_TYPE_AFFECT_ER = LINK_TYPE_AFFECT_ER;
    com.ibm.rqm.defects.web.client.ChangeRequestManager.LINK_TYPE_BLOCK_ER = LINK_TYPE_BLOCK_ER;
    com.ibm.rqm.defects.web.client.ChangeRequestManager.LINK_TYPE_TESTED_BY = LINK_TYPE_TESTED_BY;
    com.ibm.rqm.defects.web.client.ChangeRequestManager.PROVIDER_LINK_TYPE_CQREST = PROVIDER_LINK_TYPE_CQREST;
    com.ibm.rqm.defects.web.client.ChangeRequestManager.INIT_INITIALIZED = false;
})();
