/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.appscan.web.client.internal.Constants");

(function() {

dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.requireLocalization("com.ibm.rqm.appscan.web.ui.internal", "messages");

// define a message bundle id for AppScan Tester Edition
var MessageBundleASE = "appscan.web";

dojo.declare("com.ibm.rqm.appscan.web.client.internal._MessageLoader", null,
{
    constructor: function(){
		// initializes the message bundle
		MessageUtil.registerRequireBundle(MessageBundleASE, "com.ibm.rqm.appscan.web.ui.internal", "messages");
		console.debug("Registered AppScan Tester Edition bundle: \"" + MessageUtil.getMessage(MessageBundleASE, "bundle_description") + "\"" );
    }
});

com.ibm.rqm.appscan.web.client.internal.MessageLoader = new com.ibm.rqm.appscan.web.client.internal._MessageLoader();

com.ibm.rqm.appscan.web.client.internal.Constants = {

	DEFAULT_FOLDER_ID: 1,
	DEFAULT_SEVERITY_THRESHOLD: 300,
	MAX_NEW_SCRIPTS: 100,
	
	// Issue severities
	SEVERITY_HIGH: 400,
	SEVERITY_MEDIUM: 300,
	SEVERITY_LOW: 200,
	SEVERITY_INFORMATION: 100,
	SEVERITY_HIGH_TEXT: "high",
	SEVERITY_MEDIUM_TEXT: "medium",
	SEVERITY_LOW_TEXT: "low",
	SEVERITY_INFORMATION_TEXT: "information",
	
	ITERATION_LASTCOMPLETED: -3,
	
	TEST_SCRIPT_PROPERTY_COUNT: 4, // Count of properties below:
	
	// ASE Test Script Properties
	JOBID: "jobId",
	REPORTPACKID: "reportPackId",
	TEMPLATENAME: "templateName",
	SEVERITYTHRESHOLD: "severityThreshold",
	
	// Execution Result Properties
	MESSAGE: "message",
	STARTDATE: "StartDate",
	ITERATION: "iteration",
	
	TOVIEWLETBUYLINK: "http://eb90.elearn.ihost.com/wps/portal/appscan",
	
	/**
     * Validate the message bundle in case anything goes wrong during load time,
     * register the bundle here explicitly.
     */
    validateMessageBundle: function() {
        if (!MessageUtil.getBundle(MessageBundleASE)) {
            MessageUtil.registerBundle(MessageBundleASE, "com.ibm.rqm.appscan.web.ui.internal", "messages");
            console.debug("Explicitly registered AppScan Tester Edition bundle: \"" + MessageUtil.getMessage(MessageBundleASE, "bundle_description") + "\"");
        }
    },
    
    /**
     * Return the message from the message bundle with the specified keyword.
     * Optional token arguments are for replacing place holders in the message
     * definition.  The place holders have the syntax "%number" with the number
     * starting from 1.  For example, if the message "myStr" is defined as
     * "This is %1 of %2 message".  getMessage("myStr", "one", "my many") will return
     * "This is one of my many message".
     */
    getMessage: function(keyword, tokens) {
        var args = [MessageBundleASE];
        for( var i = 0; i < arguments.length; i++){ 
            args.push(arguments[i]);
        }
        return MessageUtil.getMessage.apply(MessageUtil, args);
    }

};
})();