/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.reports.cognos.web.client.internal.Constants");

(function() {
    
dojo.require("com.ibm.asq.common.web.client.internal.CommonPreferenceClient");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.requireLocalization("com.ibm.rqm.reports.cognos.web.ui.internal", "messages");

// already defined message bundle id for RTLM
var MessageBundleRTLM = "RTLM";

// define a message bundle id for RQM
var MessageBundleCognos = "RQMCognos";

var CommonPreferenceClient = com.ibm.asq.common.web.client.internal.CommonPreferenceClient;

dojo.declare("com.ibm.rqm.reports.cognos.web.client.internal._MessageLoader", null,
{
    constructor: function(){
        // initializes the rqm message bundle
        MessageUtil.registerRequireBundle(MessageBundleCognos, "com.ibm.rqm.reports.cognos.web.ui.internal", "messages");
        console.debug("Registered QM bundle: \"" + MessageUtil.getMessage(MessageBundleCognos, "bundle_description") + "\"" );
    }
});

// create an instance to ensure the message bundle is loaded
com.ibm.rqm.reports.cognos.web.client.internal.MessageLoader = new com.ibm.rqm.reports.cognos.web.client.internal._MessageLoader();

com.ibm.rqm.reports.cognos.web.client.internal.Constants = {

	NEW_PAGE_TITLE: MessageUtil.getMessage(MessageBundleCognos, "labelNewPageTitle"),
    COGNOS_BROWSE_TITLE: MessageUtil.getMessage(MessageBundleCognos, "labelReportBrowse"),
		
    /**
     * Validate the message bundle in case anything goes wrong during load time,
     * register the bundle here explicitly.
     */
    validateMessageBundle: function() {
        if (!MessageUtil.getBundle(MessageBundleCognos)) {
            dojo.requireLocalization("com.ibm.rqm.reports.cognos.web.ui.internal", "messages");
            MessageUtil.registerBundle(MessageBundleCognos, "com.ibm.rqm.reports.cognos.web.ui.internal", "messages");
            console.debug("Explicitly registered QM bundle: \"" + MessageUtil.getMessage(MessageBundleCognos, "bundle_description") + "\"");
        }
    },
    
    /**
     * Return the message from the message bundle with the specified keyword.
     * Optional token arguments are for replacing place holders in the message
     * definition.  The place holders have the syntax "%number" with the number
     * starting from 1.  For example, if the message "myStr" is defined as
     * "This is %1 of %2 message".  getMessage("myStr", "one", "my many") will return
     * "This is one of my many message".
     */
    getMessage: function(keyword, tokens) {
        var args = [MessageBundleCognos];
        for( var i = 0; i < arguments.length; i++){ 
            args.push(arguments[i]);
        }
        return MessageUtil.getMessage.apply(MessageUtil, args);
    },
    
    /**
     * Returns the message stored in the common message bundle.
     * @param {Object} keyword
     * @param {Object} tokens
     */
    getCommonMessage: function(keyword, tokens) {
        var args = [MessageBundle.COMMON];
        for( var i = 0; i < arguments.length; i++){ 
            args.push(arguments[i]);
        }
        return MessageUtil.getMessage.apply(MessageUtil, args);
    },
    
    /**
     * Returns the message stored in the LM message bundle.
     * @param {Object} keyword
     * @param {Object} tokens
     */
    getLmMessage: function(keyword, tokens) {
        var args = [MessageBundleRTLM];
        for( var i = 0; i < arguments.length; i++){ 
            args.push(arguments[i]);
        }
        return MessageUtil.getMessage.apply(MessageUtil, args);
    },
    
    getExtensionDisplayMessage: function(name) {
        if (name && dojo.isString(name)) {
            return (name.search("RQM-KEY-") == 0) ? this.getMessage(name) : name;
        } else {
            return name;
        }
    }
};
})();
