/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rqm.defects.web.client.internal.ResourceLinkService");

dojo.require("com.ibm.rqm.defects.web.util.XMLUtil");
dojo.require("dojox.data.dom");

(function(){
    var XMLUtil = com.ibm.rqm.defects.web.util.XMLUtil;
    // XML resource types
    var HEADER_ACCEPT_REQ = "application/x-oslc-rdm-requirement+xml";
    var HEADER_ACCEPT_REQ_COL = "application/x-oslc-rdm-requirement-collection+xml";
    var HEADER_ACCEPT_XML = "application/xml";
    var RESOURCE_TYPE_REQ = "rm-req";
    var RESOURCE_TYPE_REQ_COL = "rm-req-col";
    
    // XML element tags
    var TAG_VALIDATES_REQ = "validatesRequirement";
    var TAG_VALIDATES_REQ_SET = "validatesRequirementSet";
    var TAG_LABEL = "label";
    var TAG_RESOURCE = "resource";
    var TAG_VALIDATED_BY = "validatedBy";
    var TAG_LINK_COL = "LinkCollection";
    var TAG_RDF = "RDF";
    var TAG_REQ = "Requirement";
    var TAG_REQ_COL = "RequirementCollection";
    var TAG_LI = "li";
    var TAG_TITLE = "title";
	var TAG_PREX_RDF = "rdf";
	var TAG_PREX_OSLC_QM = "oslc_qm";
	var TAG_PREX_RDFS = "rdfs";
	var TAG_PREX_CALM = "calm";
    
    // Link collection id
    var ID_VALIDATES_COL = "links/validatesRequirement";
    var ID_VALIDATES_REQ_SET_COL = "links/validatesRequirementSet";
    var ID_VALIDATED_BY_COL = "validatedByCollection";
    
    // Summary:
    // 		A client that provides utils to handle resource request, 
    //		parse resource representation and so on
    //
    
    com.ibm.rqm.defects.web.client.internal.ResourceLinkService = {
        /***
         * Retrieve a qm resource's 'validates' link collection.
         * @param resourceUri
         * @return - callback([{summary, uri}])
         */
        getValidatesLinks: function(resourceType, resourceUri, callback){
            _getValidatesLinks(resourceType, resourceUri, callback);
        },
        
        /***
         * Update a qm resource's 'validates' link collection
         * @param {Object} resourceUri
         * @param {Object} addedLinks [{summary, uri}]
         * @param {Object} removedLinks [{summary, uri}]
         */
        updateValidatesLinks: function(resourceType, resourceUri, addedLinks, removedLinks, callback){
            // 1. get the resource doc 
            _getValidatesDoc(resourceType, resourceUri, dojo.hitch(this, function(doc){
                //2. modify links in the doc according to user's request
                _modifyValidatesLinks(resourceType, doc, addedLinks, removedLinks);
                //3. update the resource doc
                _updateValidatesDoc(resourceType, resourceUri, doc, callback);
            }));
            
        },
        
        /***
         * Retrieve a requirement collection's linked requirements
         * @param {Object} collectionUri
         * @param {Object} callback
         * @return array of uri string
         */
        getLinksFromReqCollection: function(collectionUri, callback, errorCallback){
            _getLinksFromReqCollection(collectionUri, callback, errorCallback);
        },
        
        /***
         * Remove an existing link from rm resource's 'validatedBy' link collection
         * @param {Object} resourceUri
         * @param {Object} linkDesc [{linkComment, rqmUri}]
         */
        removeValidatedByLink: function(resourceType, resourceUri, linkDesc){
            var removedLinks = [];
            removedLinks.push({
                'uri': linkDesc.rqmUri,
                'summary': linkDesc.linkComment
            });
            _getValidatedByDoc(resourceType, resourceUri, function(doc){
                _modifyValidatedByLinks(doc, [], removedLinks);
                _updateValidatedByDoc(resourceType, resourceUri, doc);
            })
        },
        
        /***
         * add a new link in rm resource's 'validatedBy' link collection
         * @param {Object} resourceUri
         * @param {Object} linkDesc [{linkComment, rqmUri}]
         */
        addValidatedByLink: function(resourceType, resourceUri, linkDesc){
            var addedLinks = [];
            addedLinks.push({
                'uri': linkDesc.rqmUri,
                'summary': linkDesc.linkComment
            });
            _getValidatedByDoc(resourceType, resourceUri, function(doc){
                _modifyValidatedByLinks(doc, addedLinks, []);
                _updateValidatedByDoc(resourceType, resourceUri, doc);
            })
        },
        
        __sentinel: null // terminates this class definition
    };
    
    function _getValidatesLinks(resourceType, resourceUri, callback){
        var validatesTagName = _getValidatesTagName(resourceType);
        _getValidatesDoc(resourceType, resourceUri, function(doc){
			var links = [];
			if (doc) {
				var rootNode = XMLUtil.getNode(doc.childNodes, validatesTagName, XMLUtil.NAMESPACE_CALM);
				var linkNodes = XMLUtil.getNodes(rootNode.childNodes, validatesTagName, XMLUtil.NAMESPACE_CALM);
				
				dojo.forEach(linkNodes, function(linkNode){
					var summaryAttrNode = XMLUtil.getAttributeNode(linkNode, TAG_LABEL, XMLUtil.NAMESPACE_OSLC_QM);
					var summary = XMLUtil.getAttributeNodeValue(summaryAttrNode);
					var uriAttrNode = XMLUtil.getAttributeNode(linkNode, TAG_RESOURCE, XMLUtil.NAMESPACE_RDF);
					var uri = XMLUtil.getAttributeNodeValue(uriAttrNode);
					links.push({
						'summary': summary,
						'uri': uri
					});
				});
			}
            callback(links);
        });
    }
    
    function _getValidatesDoc(resourceType, resourceUri, callback){
        var linkType = _getValidatesLinkType(resourceType);
        var deferred = jazz.client.xhrGet({
            url: resourceUri + '/' + linkType,
            headers: {
                'Accept': HEADER_ACCEPT_XML
            },
            handleAs: 'xml',
            load: function(doc){
                callback(doc);
            }, 
			error: function(result) {
				callback(null);
			}
        })
    }
    
    function _updateValidatesDoc(resourceType, resourceUri, doc, callback){
        var linkType = _getValidatesLinkType(resourceType);
        var deferred = jazz.client.xhrPut({
            url: resourceUri + '/' + linkType,
            contentType: HEADER_ACCEPT_XML,
            postData: dojox.data.dom.innerXML(doc),
            load: function(result){
				if (callback && dojo.isFunction(callback)) {
					callback(result);
				}
            }
        })
    }
    
    function _modifyValidatesLinks(resourceType, doc, addedLinks, removedLinks){
        var validatesTagName = _getValidatesTagName(resourceType);
        var rootNode = XMLUtil.getNode(doc.childNodes, validatesTagName, XMLUtil.NAMESPACE_CALM);
        dojo.forEach(addedLinks, function(link){
            var linkNode = null;
            if (dojo.isIE) { // handle IE xml attribute manipulation 
                linkNode = doc.createElement(TAG_PREX_CALM + ":" + validatesTagName);
//                linkNode.namespaceURI = XMLUtil.NAMESPACE_CALM;
                linkNode.setAttribute(TAG_PREX_RDF + ":" + TAG_RESOURCE, link.uri);
                linkNode.setAttribute(TAG_PREX_OSLC_QM + ":" + TAG_LABEL, link.summary);
            }
            else {
                linkNode = document.createElementNS(XMLUtil.NAMESPACE_CALM, validatesTagName);
                linkNode.setAttributeNS(XMLUtil.NAMESPACE_RDF, TAG_RESOURCE, link.uri);
                linkNode.setAttributeNS(XMLUtil.NAMESPACE_OSLC_QM, TAG_LABEL, link.summary);
            }
            rootNode.appendChild(linkNode);
        });
        dojo.forEach(removedLinks, function(link){
            var linkNodes = XMLUtil.getNodes(rootNode.childNodes, validatesTagName, XMLUtil.NAMESPACE_CALM);
            dojo.forEach(linkNodes, function(linkNode){
                var uriAttrNode = XMLUtil.getAttributeNode(linkNode, TAG_RESOURCE, XMLUtil.NAMESPACE_RDF);
                var uri = XMLUtil.getAttributeNodeValue(uriAttrNode);
                if (link.uri == uri) {
                    rootNode.removeChild(linkNode);
                }
            });
        });
    }
    
    function _getLinksFromReqCollection(collectionUri, callback, errorCallback){
        _getReqCollectionDoc(collectionUri, function(doc){
            var rdfNode = XMLUtil.getNode(doc.childNodes, TAG_RDF, XMLUtil.NAMESPACE_RDF);
            var rootNode = XMLUtil.getNode(rdfNode.childNodes, TAG_REQ_COL, XMLUtil.NAMESPACE_OSLC_RDM);
            var linkNodes = XMLUtil.getNodes(rootNode.childNodes, TAG_LI, XMLUtil.NAMESPACE_RDF);
            
            var uris = [];
            dojo.forEach(linkNodes, function(linkNode){
                var uriAttrNode = XMLUtil.getAttributeNode(linkNode, TAG_RESOURCE, XMLUtil.NAMESPACE_RDF);
                var uri = XMLUtil.getAttributeNodeValue(uriAttrNode);
                uris.push(uri);
            });
            if (uris.length > 0) {
            	_getLinksDetail(uris, callback);
            } else {
            	callback([]);
            }
        }, errorCallback);
    }
    
    function _getLinksDetail(uris, callback){
        var links = [];
        var index = 0;
        dojo.forEach(uris, function(uri){
            _getReqResource(uri, function(req){
            	if (req) {
                	links.push(req);
                }
                if (++index == uris.length) 
                    callback(links);
                
            });
        });
    }
    
    function _getReqResource(uri, callback){
        var acceptHeader = HEADER_ACCEPT_REQ;
        var deferred = jazz.client.xhrGet({
            url: uri,
            headers: {
                'Accept': acceptHeader
            },
            handleAs: 'xml',
            load: function(doc){
                var req = {};
                req.uri = uri;
                var rdfNode = XMLUtil.getNode(doc.childNodes, TAG_RDF, XMLUtil.NAMESPACE_RDF);
                var rootNode = XMLUtil.getNode(rdfNode.childNodes, TAG_REQ, XMLUtil.NAMESPACE_OSLC_RDM);
                var summaryNode = XMLUtil.getNode(rootNode.childNodes, TAG_TITLE, XMLUtil.NAMESPACE_DCTERMS);
                if (summaryNode) {
                    req.summary = dojox.data.dom.textContent(summaryNode);
                }
                else {
                    req.summary = "_blank";
                }
                callback(req);
            },
            error: function(result) {
            	callback(null);// For 22129, if resource doesn't exist any more, the process should proceed
            }
        });
    }
    
    function _getReqCollectionDoc(collectionUri, callback, errorCallback){
        var acceptHeader = HEADER_ACCEPT_REQ_COL;
        var deferred = jazz.client.xhrGet({
            url: collectionUri,
            headers: {
                'Accept': acceptHeader
            },
            handleAs: 'xml',
            load: function(doc){
                callback(doc);
            },
            error: function(error){
                errorCallback(error)
            }
            
        });
    }
    
    function _getValidatedByDoc(resourceType, resourceUri, callback){
        var acceptHeader = null;
        if (resourceType == RESOURCE_TYPE_REQ) {
            acceptHeader = HEADER_ACCEPT_REQ;
        }
        else {
            acceptHeader = HEADER_ACCEPT_REQ_COL;
        }
        var deferred = jazz.client.xhrGet({
            url: resourceUri + '/' + ID_VALIDATED_BY_COL,
            headers: {
                'Accept': acceptHeader
            },
            handleAs: 'xml',
            load: function(doc){
                callback(doc);
            }
        })
    }
    
    function _updateValidatedByDoc(resourceType, resourceUri, doc){
        var contentType = null;
        if (resourceType == RESOURCE_TYPE_REQ) {
            contentType = HEADER_ACCEPT_REQ;
        }
        else {
            contentType = HEADER_ACCEPT_REQ_COL;
        }
        var deferred = jazz.client.xhrPut({
            url: resourceUri + '/' + ID_VALIDATED_BY_COL,
            contentType: contentType,
            postData: dojox.data.dom.innerXML(doc)
        })
    }
    
    function _modifyValidatedByLinks(doc, addedLinks, removedLinks){
        var rdfNode = XMLUtil.getNode(doc.childNodes, TAG_RDF, XMLUtil.NAMESPACE_RDF);
        var rootNode = XMLUtil.getNode(rdfNode.childNodes, TAG_LINK_COL, XMLUtil.NAMESPACE_OSLC_RDM);
        dojo.forEach(addedLinks, function(link){
            var linkNode = null;
            if (dojo.isIE) { // handle IE xml attribute manipulation
                linkNode = doc.createElement(TAG_PREX_CALM + ":" + TAG_VALIDATED_BY);
//                linkNode.namespaceURI = XMLUtil.NAMESPACE_CALM;
                linkNode.setAttribute(TAG_PREX_RDF + ":" + TAG_RESOURCE, link.uri);
                linkNode.setAttribute(TAG_PREX_RDFS + ":" + TAG_LABEL, link.summary);
            }
            else {
                linkNode = document.createElementNS(XMLUtil.NAMESPACE_CALM, TAG_VALIDATED_BY);
                linkNode.setAttributeNS(XMLUtil.NAMESPACE_RDF, TAG_RESOURCE, link.uri);
                linkNode.setAttributeNS(XMLUtil.NAMESPACE_RDFS, TAG_LABEL, link.summary);
            }
            rootNode.appendChild(linkNode);
        });
        dojo.forEach(removedLinks, function(link){
            var linkNodes = XMLUtil.getNodes(rootNode.childNodes, TAG_VALIDATED_BY, XMLUtil.NAMESPACE_CALM);
            dojo.forEach(linkNodes, function(linkNode){
                var uriAttrNode = XMLUtil.getAttributeNode(linkNode, TAG_RESOURCE, XMLUtil.NAMESPACE_RDF);
                var uri = XMLUtil.getAttributeNodeValue(uriAttrNode);
                if (link.uri == uri) {
                    rootNode.removeChild(linkNode);
                }
            });
        });
    }
    
    function _getValidatesLinkType(resourceType){
        var linkType = null;
        if (resourceType == RESOURCE_TYPE_REQ) {
            linkType = ID_VALIDATES_COL;
        }
        else {
            linkType = ID_VALIDATES_REQ_SET_COL;
        }
        return linkType;
    }
    
    function _getValidatesTagName(resourceType){
        var tagName = null;
        if (resourceType == RESOURCE_TYPE_REQ) {
            tagName = TAG_VALIDATES_REQ;
        }
        else {
            tagName = TAG_VALIDATES_REQ_SET;
        }
        return tagName;
    }
    
    com.ibm.rqm.defects.web.client.internal.ResourceLinkService.RESOURCE_TYPE_REQ = RESOURCE_TYPE_REQ;
    com.ibm.rqm.defects.web.client.internal.ResourceLinkService.RESOURCE_TYPE_REQ_COL = RESOURCE_TYPE_REQ_COL;
})();
