/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
(function(){
dojo.provide("com.ibm.asq.common.web.client.internal._catalog.Configuration");
dojo.require("com.ibm.asq.common.web.client.internal._catalog._ConfigurationBase");

dojo.declare("com.ibm.asq.common.web.client.internal._catalog.Configuration", com.ibm.asq.common.web.client.internal._catalog._ConfigurationBase, 
{
    // Get the label for one of the assets
    getLabel: function(index){
        return this.rootAttribute.getLabel(index);
    },
    // Get the label for one of the assets
    getLabelByTag: function(tag){
        return this.rootAttribute.getLabelByTag(tag);
    },
    // Set the label for one of the assets
    setLabelByTag: function(tag, label){
        this.rootAttribute.setLabelByTag(tag, label);
    },
    // Set the label for one of the assets
    setLabel: function(index, label){
        this.rootAttribute.setLabel(index, label);
    },
    // Get the identifier for one of the assets
    getIdentifier: function(index){
        return this.rootAttribute.getIdentifier(index);
    },
    // Get the identifier for one of the assets
    getIdentifierByTag: function(tag){
        return this.rootAttribute.getIdentifierByTag(tag);
    },
    // Set the identifer for one of the assets
    setIdentifierByTag: function(tag, theIdentifier){
        this.rootAttribute.setIdentifierByTag(tag, theIdentifier);
    },
    // Set the identifier for one of the assets
    setIdentifier: function(index, theIdentifier){
        this.rootAttribute.setIdentifier(index, theIdentifier);
    },
    // Get a duplicate of the Configuration.
    clone: function() {
        var jsonStr = dojo.toJson(this.instanceObj,false);
        return new com.ibm.asq.common.web.client.internal._catalog.Configuration(jsonStr);
    },
    getSubConfiguration: function(index){
        var clonedConfig = this.clone();
        var newConfig = new com.ibm.asq.common.web.client.internal._catalog.Configuration(null, this.typeName);
        var part = clonedConfig.getRootPart(index);
        var rootAtt = newConfig.rootAttribute;
        rootAtt.addArrayValue(part);
        return newConfig;
    },
    insertConfig: function(config){
        // ensure that the types are the same
        if(this.typeName != config.typeName){
            throw new Error("Cannot insert " + config.typeName + " parts into a " + this.typeName + " Configuration.");
        }
        var tempConfig = config.clone();
        var parts = tempConfig.getRootAttribute().getArrayValues();
        var rootAtt = this.getRootAttribute();
        for(var i=0; i<parts.length; i++){
            rootAtt.addArrayValue(parts[i]);
        }
    },
	// * Depricated *
    setValue: function(displayType, value){
        // Forward to use the new objectized type.
        this.setValueByDisplayObject(displayType, value);
    },
	
	// * Depricated *
    setValueByIndex: function(partIndex, displayType, value){
		if (partIndex >= this.getRootParts().length)
			return;
        var root = this.getRootPart(partIndex);
        if (displayType == "OperatingSystem") {
            var osAtt = root.getAttribute("operatingsystem");
			if (osAtt) {
				if (osAtt.getArrayValues().length != 0) {
					osAtt.setArrayValue(0, value);
				}
				else {
					osAtt.addArrayValue(value);
				}
			}
        }
        else if (displayType == "CPU") {
            root.setAttributeValue("cpu", value);
        }
        else {
            var osAtt = root.getAttribute("operatingsystem");
            if (osAtt) {
				// auto create the OS part to hold the SW attribute.
				if (osAtt.getArrayValues().length == 0) {
					osAtt.addArrayValue(osAtt.getRootPart().getName());
				}
				var osPart = osAtt.getArrayValue(0);
				var swAtt = osPart.getAttribute("SoftwareInstall");
				if (swAtt) {
					// get the existing parts
					var parts = swAtt.getArrayValues();
					// check if a value for this subtype has already been set
					var length=parts.length;
					for (var i = 0; i < length; i++) {
						var part = parts[i];
						if (part.isAncestor(displayType)) {
							swAtt.setArrayValue(i, value);
							return;
						}
					}
					swAtt.addArrayValue(value);
				}
			}
        }
    },
	
	// * Depricated *
    getValue: function(displayType){
        // Forward to use the new objectized type.
        return this.getValueByDisplayObject(displayType);
	},

	// * Depricated *
    getValueByIndex: function(partIndex, displayType){
		if (partIndex >= this.getRootParts().length)
			return;
        var root = this.getRootPart(partIndex);
        if (displayType == "OperatingSystem") {
            var osAtt = root.getAttribute("operatingsystem");
			if (osAtt) {
				var osPart = osAtt.getArrayValue(0);
				if (osPart != null) {
					return osPart.getName();
				}
			}
            return null;
        }
        else if (displayType == "CPU") {
            var cpuPart =  root.getAttributeValue("cpu");
            if(cpuPart != null){
                return cpuPart.getName();
            }
            return null;
        }
        else {
            var osAtt = root.getAttribute("operatingsystem");
            if (osAtt && osAtt.getArrayValues().length > 0) {
                var osPart = osAtt.getArrayValue(0);
                var swAtt = osPart.getAttribute("SoftwareInstall");
                // get the existing parts
                var parts = swAtt.getArrayValues();
                // check for a value for this subtype
                var length=parts.length;
                for (var i = 0; i < length; i++) {
                    var part = parts[i];
                    
                    // NOTE: if we ever going to disallow the 
                    // software root types, we should remove
                    // the condition of showing the root types
                    //
                    if (part.isAncestor(displayType) || part.getName() == displayType) {
                        return part.getName();
                    }
                }
            }
            return null;
        }
    },

    setValueByDisplayObject: function(displayObject, value){
 		this.setValueByDisplayObjectByIndex(0, displayObject, value);
    },
	
    setValueByDisplayObjectByIndex: function(partIndex, displayObject, value){
        var pathArray = this._getDisplayObjectAttributePath(displayObject);
		var part = this._getPartByDisplayObject(pathArray);
		var attname = pathArray[pathArray.length - 1];
		var attribute = part.getAttribute(attname);
        
        // TODO: special case operation system value. It can only have
        // one value.
        if(attname == "operatingsystem") {
            if (attribute.getArrayValues().length != 0) {
                attribute.setArrayValue(0, value);
            }
            else {
                attribute.addArrayValue(value);
            }
            return;
        }
		if(attribute.isArray()) {
			var values = attribute.getArrayValues();
            // check if a value for this subtype has already been set
            var length=values.length;
            for (var i = 0; i < length; i++) {
                var part = values[i];
                if (part.isAncestor(displayObject.partName)) {
                    attribute.setArrayValue(i, value);
                    return;
                }
            }
            attribute.addArrayValue(value);
		} else {
			attribute.setValue(value);
		}
    },
	
    getValueByDisplayObject: function(displayObject){
		return this.getValueByDisplayObjectByIndex(0, displayObject);
	},

    getValueByDisplayObjectByIndex: function(partIndex, displayObject){
    	var pathArray = this._getDisplayObjectAttributePath(displayObject);
		var part = this._getPartByDisplayObject(pathArray);
		var attname = pathArray[pathArray.length - 1];
		var attribute = part.getAttribute(attname);
		var value = null;
		if(attribute.isArray()) {
            var values = attribute.getArrayValues();
            // check if a value for this subtype has already been set
            var length=values.length;
            for (var i = 0; i < length; i++) {
                var part = values[i];
                if (part.isAncestor(displayObject.partName)) {
                    value = part;
                }
            }
		} else {
			value = attribute.getValue();
		}
        // the value is a part object, return its name.
		return value ? value.getName() : null;
    },
    
    _getDisplayObjectAttributePath: function(displayObject) {
        // NOTE:
        // We need to reference the catalog to access type information.
        // BUT catalog also references this class, Configuration. Avoid
        // doing an explicit require on the Catalog here, which would
        // create a circular dependency. Just access the catalog object 
        // via its name for now. Need to better organize the catalog
        // code structure.
        var catClass = dojo.getObject("com.ibm.asq.common.web.client.internal.Catalog");
        var cat = new catClass();
        
        // The whitch tag records the chosen attribute that the user want
        // to create in the configuration. Get the path info out from it.
        // Most likely this is the 'implicit' path, when there's only one
        // accessible path to the type in a configuration, that path is automatically
        // recorded in the tag without user selection.
        var template = cat.getTemplate(displayObject.template);
        var part = template.getPart(displayObject.partName);
        var tag = part.getWhitchTag();
        return tag.path.components.component;
    },
    
    _getPartByDisplayObject: function(path) {
    	// We will only have a single root
    	var part = this.getRootPart(0);
    	var length=path.length - 1;
    	for(var i = 1; i < length; i++) {
	   		var attname = path[i];
    		var attribute = part.getAttribute(attname);
    		if(attribute.isArray()) {
    			part = part.getAttributeArrayValue(attname, 0, true);
    		} else {
	    		part = part.getAttributeValue(attname, true);
	    	}
	   	}
    	return part;
    }
    
});
})();
