/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
(function(){
dojo.provide("com.ibm.asq.common.web.client.internal._catalog.Part");
dojo.require("com.ibm.asq.common.web.client.internal._catalog._NodeBase");

var _tagCounter = 0;

dojo.declare("com.ibm.asq.common.web.client.internal._catalog.Part", com.ibm.asq.common.web.client.internal._catalog._NodeBase, 
{
    constructor: function(document, parent, instanceRoot, templateObj, path){
        this.isResolved = false;
        if (instanceRoot != null) {
            this.instanceObj = instanceRoot.AssetType;
            if(instanceRoot.uuid != null) {
                this.isResolved = true;
            }
        }
        if (path == null) {
            this.path = templateObj.type;
        }
        else {
            this.path = path + "." + templateObj.type;
        }
        this.attributeMap = {};
        if(this.instanceObj != null){
            this._initialize();
        }
        this.tag = _tagCounter++;
        
        this.enumTypeLookup = {"NumericType": "int", "StringType": "string"};
    },
    // Get the name of the Part
    getName: function(){
        return this.templateObj.name;
    },
    // Get the name of the Part
    getType: function(){
        return this.templateObj.type;
    },
    // Get the tag for the Part
    getTag: function(){
        return this.tag;
    },
    // Get the children of this Part.  For example, RedHat 
    // Linux might be a child of Linux.
    getChildParts: function(){
        if(!this.subParts == null) {
            return this.subParts;
        }
        var partArrayObj = this._util._getArrayObject(this.templateObj, "Subtypes", "AssetType");
        var partArray = new Array();
        for(var i=0; i<partArrayObj.length; i++) {
            var part = new Part(this.document, this, null, partArrayObj[i], null);
            partArray.push(part);
        }
        return partArray;
    },
    // Get a list of the names of all parts in the subtree of this
    // part.  Returns an array of strings.
    getAllSubPartNames: function() {
        return this._getAllSubPartNames().sort();
    },
    _getAllSubPartNames: function() {
        var partNames = new Array();
        var childParts = this.getChildParts();
        for(var i=0; i < childParts.length; i++) {
            partNames.push(childParts[i].templateObj.name);
            partNames = partNames.concat(childParts[i]._getAllSubPartNames());
        }
        return partNames;
    },
    // Get the parent of this Part.  This will return the template
    // if the Part is a root Part.
    getParent: function(){
        return this.parent;
    },
    // Get the names of the attributes for the Part.
    getAttributeNames: function(){
        return this._getAttributeNames(true).sort();
    },
    // Walk the parent Parts to get all the attribute names.
    _getAttributeNames: function(fetchInherited){
        var attNames = new Array();
        if(fetchInherited && this.getParent() instanceof Part) {
            attNames = this.getParent()._getAttributeNames(true);
        }
        var attributeArrayObject = this._util._getArrayObject(this.templateObj, "Attributes", "Attribute");
        for(var i=0; i<attributeArrayObject.length; i++) {
            attNames.push(attributeArrayObject[i].name);
        }
        if(attNames == null){
        	attNames = [];
        }
        return attNames;
    },
    // Get the names of the attributes on this Part.
    // Does not return attributes on the ancestor parts.
    getNonInheritedAttributeNames: function(){
        return this._getAttributeNames(false).sort();
    },
    // Mark the attribute as ignored in a search config
    setAttributeIgnored: function(attributeName, ignore){
        this.getAttribute(attributeName).setIgnored(ignore);
    },
    // Get if the attribute is ignored in a search config
    isAttributeIgnored: function(attributeName){
        return this.getAttribute(attributeName).isIgnored();
    },
    // Get an attribute of the Part by its name.
    getAttribute: function(attributeName){
        // Parts in the template must create new attributes
        if (this.instanceObj == null) {
            return (this._initializeAttribute(attributeName));
        }
        else {
            return (this.attributeMap[attributeName]);
        }
    },
    // Determine if an attribute is an array.
    isAttributeArray: function(attributeName){
        return this.getAttribute(attributeName).isArray();
    },
    // Get an attribute value of the Part by its name.
	// If createPartIfMissing is true and the attribute value is null,
	// and the attribute represents a Part, a generic instance of the 
	// Part will be created and then returned.  This method is
    // only valid for non-arrays.
    getAttributeValue: function(attributeName, createPartIfMissing){
        if(this.isTemplatePart == true){
            throw new Error("Template Parts do not have attribute values!");
        }
        var value = null;
		var attr = this.getAttribute(attributeName);
		if (attr){
			value =  attr.getValue();
			if (createPartIfMissing && value == null){
				if (attr.getType() != 'ConfigurationPartType'){
					throw new Error("createPartIfMissing only applies to configuration parts: "+attributeName)
				}
				var typeName = attr.getPartTypeName();
				this.setAttributeValue(attributeName, typeName);
				attr = this.getAttribute(attributeName);
				value = attr.getValue();
			}
		}
		return value;
    },
    // Get an attribute value of the Part by its name.
	// If createPartIfMissing is true and the attribute value is null,
	// and the attribute represents a Part, a generic instance of the 
	// Part will be created and then returned.  This method is
    // only valid for non-arrays.
    getAttributeArrayValues: function(attributeName, createPartIfMissing){
        if(this.isTemplatePart == true){
            throw new Error("Template Parts do not have attribute values!");
        }
        var values = null;
		var attr = this.getAttribute(attributeName);
		if (attr){
			values =  attr.getArrayValues();
			if (createPartIfMissing && (values == null || values.length == 0)){
				if (attr.getType() != 'ConfigurationPartType'){
					throw new Error("createPartIfMissing only applies to configuration parts: "+attributeName)
				}
				var typeName = attr.getPartTypeName();
				this.addAttributeArrayValue(attributeName, typeName);
				attr = this.getAttribute(attributeName);
				values = attr.getArrayValues();
			}
		}
		return values;
    },
    // Get an attribute value of the Part by its name and part tag.
    // This method is only valid for part type attributes.
    // If createPartIfMissing is true and the attribute value is null,
    // and the attribute represents a Part, a generic instance of the 
    // Part will be created and then returned.  This method is
    // only valid for arrays.
    getAttributeArrayValueByTag: function(attributeName, tag, createPartIfMissing){
        var attr = this.getAttribute(attributeName);
        var index = attr.getIndexByTag(tag);
        return this.getAttributeArrayValue(attributeName, index, createPartIfMissing);
    },
    // Get an attribute value of the Part by its name.
	// If createPartIfMissing is true and the attribute value is null,
	// and the attribute represents a Part, a generic instance of the 
	// Part will be created and then returned.  This method is
    // only valid for arrays.
    getAttributeArrayValue: function(attributeName, index, createPartIfMissing){
        if(this.isTemplatePart == true){
            throw new Error("Template Parts do not have attribute values!");
        }
        var value = null;
		var attr = this.getAttribute(attributeName);
		if (attr){
			var values =  attr.getArrayValues();
			if (createPartIfMissing && (values == null || values.length == 0)){
				if (attr.getType() != 'ConfigurationPartType'){
					throw new Error("createPartIfMissing only applies to configuration parts: "+attributeName)
				}
				var typeName = attr.getPartTypeName();
				this.addAttributeArrayValue(attributeName, typeName);
				attr = this.getAttribute(attributeName);
				value = attr.getArrayValue(0);
			} else {
                value = values[index];
            }
		}
		return value;
    },
    // Set an attribute value of the Part.
    setAttributeValue: function(attributeName, value){
        if(this.isTemplatePart == true){
            throw new Error("Template Parts do not have attribute values!");
        }
        var att = this.getAttribute(attributeName);
		if (att){
			att.setValue(value);
		}else{
			throw Error( "invalid attribute");	
		}
    },
    // Add an attribute value of the Part.
    addAttributeArrayValue: function(attributeName, value){
        if(this.isTemplatePart == true){
            throw new Error("Template Parts do not have attribute values!");
        }
        var att = this.getAttribute(attributeName);
		if (att){
			att.addArrayValue(value);
		}else{
			throw Error( "invalid attribute");	
		}
    },
    // Set an attribute value of the Part.
    // This method is only valid for part type attributes.
    setAttributeArrayValueByTag: function(attributeName, tag, value){
        var attr = this.getAttribute(attributeName);
        var index = attr.getIndexByTag(tag);
        return this.setAttributeArrayValue(attributeName, index, value);
    },
    // Set an attribute value of the Part.
    setAttributeArrayValue: function(attributeName, index, value){
        if(this.isTemplatePart == true){
            throw new Error("Template Parts do not have attribute values!");
        }
        var att = this.getAttribute(attributeName);
		if (att){
			att.setArrayValue(index, value);
		}else{
			throw Error( "invalid attribute");	
		}
    },
    // Remove an attribute value of the Part by the part tag.  This will
    // re-index the array of values.
    // This method is only valid for part type attributes.
    removeAttributeArrayValueByTag: function(attributeName, tag){
        var attr = this.getAttribute(attributeName);
        var index = attr.getIndexByTag(tag);
        return this.removeAttributeArrayValue(attributeName, index);
    },
    // Remove an attribute value of the Part.  This will
    // re-index the array of values.
    removeAttributeArrayValue: function(attributeName, index){
        if(this.isTemplatePart == true){
            throw new Error("Template Parts do not have attribute values!");
        }
        var att = this.getAttribute(attributeName);
		if (att){
			att.removeArrayValue(index);
		}else{
			throw Error( "invalid attribute");	
		}
    },
    // Get the template that contains the Part
    getTemplate: function(){
        var currParent = this.getParent();
        while(currParent instanceof Part) {
            currParent = currParent.getParent();
        }
        if(!(currParent instanceof com.ibm.asq.common.web.client.internal._catalog.Template)) {
            console.debug("*************!!!!!!!!!!!!!!!!>>>>>>>>>>>>> improperly constructed Part.  No template at root!");
            throw new Error("improperly constructed Part.  No template at root!");
        }
        return currParent;
    }, 
    // Get the root Part from the template that contains this 
    // Part. 
    getRoot: function(){
        var current = this;
        while(current.parent instanceof Part) {
            current = current.parent;
        }
        return current;
    },
    // Check if the part is a generic (ie Root) part
    isGeneric: function(){
        if(this.parent instanceof Part) {
            return false;
        } else {
            return true;
        }
    }, 
    // Determine if this is a resolved config part 
    _isResolved: function(){
		var uuid = this.instanceRoot.uuid;
        return(uuid != null);
    },
    // Get the enum values for this type starting with
    // this part and working up the tree.
    _getEnumValues: function(name, fetchInherited, attributeType){
        return this._getEnumValuesInner(name, fetchInherited, attributeType).sort();
    },
    _getEnumValuesInner: function(name, fetchInherited, attributeType){
        var values = this._getAttributeValues(name, attributeType);
        if(!fetchInherited || values != null){
        	if(values == null){
        		values = [];
        	}
            return values;
        } else{
            var par = this.getParent();
            if(par instanceof Part){
                return par._getEnumValues(name, fetchInherited, attributeType);
            } else{
                return new Array();
            }
        }
    },
    // Determine if a given attribute is an enum
    _isEnum: function(name, attributeType){
        var hasValues = this._hasAttributeValues(name, attributeType);
        if(hasValues){
            return true;
        } else{
            var par = this.getParent();
            if(par instanceof Part){
                return par._isEnum(name, attributeType);
            } else{
                return false;
            }
        }
    },
    // Get a set of attribute values
    _getAttributeValues: function(name, attributeType){
        var ranges = this.templateObj.AttributeValueRanges;
        if(ranges == null || ranges == ""){
            return null;
        }
        var rangeArray = ranges.AttributeValueRange;
		
		if (!dojo.isArray(rangeArray)) {
			rangeArray = [rangeArray];
		}
		
        var found = -1;
        for(var i=0; i<rangeArray.length && found == -1; i++){
            if(rangeArray[i].name == name){
                found = i;
            }
        }
        if(found == -1){
            return null;
        }
        return this._util._getArrayObject(rangeArray[found], "value", this.enumTypeLookup[attributeType]);
    },
    // Determine if there are attribute values
    _hasAttributeValues: function(name, attributeType){
        if(this._getAttributeValues(name, attributeType) == null){
            return false;
        }
        else return true;
    },
    // Determine if the values used by an attribute are
    // from the named part
    _usingAttributeValuesFrom: function(partName, attributeName, attributeType){
        if(this.templateObj.name == partName){
            return true;
        }
        if(this._hasAttributeValues(attributeName, attributeType)){
            return false;
        }
        return this.parent._usingAttributeValuesFrom(partName, attributeName, attributeType);
    },
    // Create a Part that is a "clone" of this part and
    // can be added to a Configuration.
    _createInstancePart: function(document, instanceRoot, path) {
        var partRoot = null;
        if (instanceRoot != null) {
            partRoot = instanceRoot;
        }
        else {
            partRoot = {
                AssetType: {
                    type: this.templateObj.type,
                    name: this.templateObj.name,
                    Attributes: {
                        Attribute: new Array()
                    }
                }
            }
        }
        var part = new Part(
                document,
                this.parent,
                partRoot,
                this.templateObj,
                path);
        return part;
    },
    // Create Parts and Attributes for all objects in the part's attribute tree.
    _initialize: function(){
        var attNames = this._getAttributeNames(true);
        for(var i=0;i<attNames.length;i++){
            var att = this._initializeAttribute(attNames[i]);
            this.attributeMap[attNames[i]] = att;
        }
    },
    _initializeAttribute: function(attributeName){
        // First find the attribute in the template.  
        var part = this;
        var templateAttributeObject = null;
        while(templateAttributeObject == null &&
              part instanceof Part) {
                  
            // See if the attribute is in this part
            var templateAttributeArrayObject = part._util._getArrayObject(part.templateObj, "Attributes", "Attribute");
            var length=templateAttributeArrayObject.length;
            for(var i=0; i<length; i++) {
                if(templateAttributeArrayObject[i].name == attributeName) {
                    templateAttributeObject = templateAttributeArrayObject[i];
                    break;
                }
            }
            part = part.getParent();
        }
        if(templateAttributeObject == null) {
            console.debug("*************!!!!!!!!!!!!!!!!>>>>>>>>>>>>> Invalid attribute name - " + attributeName + "!");
            throw Error( "Attribute not found");    
        }
        
        // Now see if there is an instance object for this attribute
        var attributeObject = null;
        if (this.instanceObj != null) {
            var attributeArrayObject = this._util._getArrayObject(this.instanceObj, "Attributes", "Attribute");
            var length=attributeArrayObject.length;
            for(var i=0; i< length; i++) {
                if(attributeArrayObject[i].name == attributeName) {
                    attributeObject = attributeArrayObject[i];
                    break;
                }
            }
        }
        var att = new com.ibm.asq.common.web.client.internal._catalog.Attribute(this.document, this, attributeObject, templateAttributeObject, this.path);
        return att;    
    },
    _getCommonRoot: function(part) {
        // Create a hash of the names in this parts parent hierarchy
        var partHash = {};
        var name = this.templateObj.name;
        partHash[name] = this;
        var node = this.parent;
        while(node instanceof Part){
            name = node.templateObj.name;
            partHash[name] = node;
            node = node.parent;
        }
        // Find the node in the passed in part that is in the hash
        node = part;
        while(node instanceof Part){
            name = node.templateObj.name;
            var commonPart = partHash[name];
            if(commonPart != null){
                return partHash[name];
            }
            node = node.parent;
        }
        return null;
    },
	// _setValuesFromPart is used when we want to change the type of
	// a part but maintain the values.  It will copy all of the values
	// and the uuids.  For part type values, it will reuse the parts, not
	// make copies of them.
    _setValuesFromPart: function(oldPart){
        // Transfer the uuid and stateId first as the serialization
		// needs them to be the first attributes.
        var oldUuid = oldPart.instanceRoot.uuid;
        if(oldUuid!= null){
	        // Need to force this ordering
            var instanceRoot = this.instanceRoot;
	        var at = instanceRoot.AssetType;
	        this.instanceRoot = {};
            instanceRoot = this.instanceRoot;
	        instanceRoot.uuid = oldUuid;
            instanceRoot.stateId = oldPart.instanceRoot.stateId;
	        instanceRoot.AssetType = at;
            this.instanceObj = instanceRoot.AssetType;
        }
        // Get the common root
        var commonRoot = oldPart._getCommonRoot(this);
        var commonRootName = commonRoot.getName();
        // Set the value for each of the attributes in the common root
        var attNames = commonRoot._getAttributeNames(true);
        var length=attNames.length;
        for(var i=0;i<length;i++){
            var name = attNames[i];
            var oldAtt = oldPart.getAttribute(name);
            var newAtt = this.getAttribute(name);
            var oldAttType = oldAtt.getType();
            var newAttType = newAtt.getType();
            var commonRootAtt = commonRoot.getAttribute(name);
            // If this is an enum, get the available values
            var enumVals = null;
            if(newAtt.isEnum()){
                // We will get the enum values from the common root
                // only if neither the new attribute nor the old
                // attribute has overridden those values.
                if (this._usingAttributeValuesFrom(commonRootName, name, newAttType) &&
                    oldPart._usingAttributeValuesFrom(commonRootName, name, oldAttType)) {
                    enumVals = {};
                    var valArr = commonRootAtt.getEnumValues();
                    var valLength=valArr.length;
                    for (var j = 0; j < valLength; j++) {
                    	var x = valArr[j]
                        enumVals[x] = x;
                    }
                }
            }
            if(oldAtt.isArray()){
                var values = oldAtt.getArrayValues();
                var valLength=values.length;
                for(var j=0;j<valLength;j++){
                    if (enumVals == null) {
                        newAtt._addArrayValue(values[j], false);
                    }
                    else {
                        // Check if the value is in the enum
                        if(enumVals[values[j]] != null){
                            newAtt._addArrayValue(values[j], false);
                        }
                    }
                }
            }else{
                if (enumVals == null) {
	                // Don't need to set nulls
	                var value = oldAtt.getValue();
	                if (value != null) {
	                    newAtt._setValue(oldAtt.getValue(), false);
	                }
                }
                else {
	                // Check if the value is in the enum
	                var value = oldAtt.getValue();
	                if(value != null) {
	                    if (enumVals[value] != null) {
	                        newAtt._setValue(value, false);
	                    }
	                }
	            }
            }
        }
    },
    
    // _copyValuesFromPart is used when we want to make a duplicate of the
	// part.  It will copy all of the values but not the uuids.  For part
	// type values, it will create new part objects, it will not reuse
    // the existing objects.  The type and name of the old part must be
	// the same as this or the method will fail.
    _copyValuesFromPart: function(oldPart){
        // Set the value for each of the attributes.
        var attNames = this.getAttributeNames();
        for(var i=0;i<attNames.length;i++){
            var name = attNames[i];
            var oldAtt = oldPart.getAttribute(name);
            var isPart = (oldAtt.getType() == "ConfigurationPartType");
            if(oldAtt.isArray()){
                var values = oldAtt.getArrayValues();
				if (values.length > 0) {
					var newAtt = this.getAttribute(name);
					var length=values.length;
					for (var j = 0; j < length; j++) {
						var value=values[j];
    					newAtt._addArrayValue(value, false);
						if(isPart){
							var part = newAtt.getArrayValue(j);
							part._copyValuesFromPart(value);
						}
					}
				}
            }else{
                // Don't need to set nulls
                var value = oldAtt.getValue();
                if (value != null) {
                    var newAtt = this.getAttribute(name);
                    newAtt._setValue(oldAtt.getValue(), false);
                    if(isPart){
                        var part = newAtt.getValue();
                        part._copyValuesFromPart(value);
                    }
                }
            }
        }
    },
    
    isAncestor: function(parentName){
        var parent = this.getParent();
        if(!(parent instanceof Part)){
            return false;
        }
        if(parent.getName() == parentName){
            return true;
        }
        return parent.isAncestor(parentName);
    },
	
	// Creates a new part that is a child of the this Part.
	// The Part will be created with no attributes and will
	// default to being visible.
	createChildPart: function(name){
		// Create the object
		var object = {};
		object["toplevel"] = "false";
		object["type"] = name;
		object["name"] = name;
		object["Attributes"] = {};
		object["Attributes"]["Attribute"] = [];
		object["AttributeValueRanges"] = "";
		object["AttributeValueRanges"]["AttributeValueRange"] = [];
		object["Subtypes"] = "";
		object["Subtypes"]["AssetType"] = [];
		
		// Add the part object to the templates object as a sub part
		var subTypeArray = this._util._getArrayObject(this.templateObj, "Subtypes", "AssetType");
		subTypeArray.push(object);
		
		// Create the Part
		var part = new Part(this.document, this, null, object, null);
        part.isTemplatePart = true;
		part.templateObj.editable = "true";

		// Add the object to the template
		var template = this.getTemplate();
		template.partMap[name] = part;

		return part;
	},
		
	// Delete's an editable part from the sandbox.  Only
	// editable parts may be deleted.  Parts which have been
	// saved to the server may only be hidden.
	deletePart: function() {
		var name = this.getName();
		var isRoot  = this.isGeneric();
		var template = this.getTemplate();
		var templateName = template.getName();
		var parent = this.getParent();
		var cat = com.ibm.asq.common.web.client.internal.Catalog.theCatalog;
		
		// If this is the root part, we need to delete
		// the template from the sandbox.
		if(isRoot){
			// Need to find the template id so it can be deleted
			var idToDelete = null;
			for(var templateId in cat.sandboxTemplateMap) {
				var templ = cat.sandboxTemplateMap[templateId];
				var name = templ.getName();
				if(name == templateName){
					idToDelete = templateId;
				}
			}
		}

		// Delete the json object
		delete this.templateObj;
		
		// Remove the object from the template map
		delete template[name];
		
		// Remove the part from the parent's
        if(parent instanceof Part){
        	var partArrayObj = parent.templateObj.Subtypes.AssetType;
        	var length=partArrayObj.length;
        	for(var i=0; i<length; i++) {
        		if(partArrayObj[i].name == name) {
        			parent.templateObj.Subtypes.AssetType = this._util._removeFromArray(partArrayObj, i);
        		}
        	}
        }
		
		// If this is the root part, we need to delete
		// the template from the sandbox.
		if(isRoot){
			if(idToDelete != null) {
				delete cat.sandboxTemplateMap[idToDelete];
			}
		}
	},

	// Creates a new attribute on the part who type must be
	// one of {'StringType', 'NumericType', 'PointerType',
    // 'ConfigurationPartType','TimeStampType'}.  The attribute 
	// must be specified and an isArray flag.  If the type is
	// 'ConfigurationPartType', the template name of the part
	// must be specified. 
	createAttribute: function(type, name, isArray, templateName, templateId, enumValues){
		// Create the object
		var object = {};
		object["type"] = type;
		object["name"] = name;
		object["hidden"] = "false";
		if(isArray){
			object["cardinality"] = "ZERO_OR_MORE";
		} else {
			object["cardinality"] = "ZERO_OR_ONE";
		}
		if (type == 'ConfigurationPartType') {
			object["AssetType"] = {};
			object["AssetType"]["id"] = templateId;
			object["AssetType"]["type"] = templateName;
			object["AssetType"]["name"] = templateName;
		}

		// Add the array object to the templates object
		var attArray = this._util._getArrayObject(this.templateObj, "Attributes", "Attribute");
		attArray.push(object);
		
		// Create the Attribute
		var attribute = new com.ibm.asq.common.web.client.internal._catalog.Attribute(this.document, this, null, object, this.path);
		attribute.templateObj.editable = "true";

		// Add the object to the Part
		this.attributeMap[name] = attribute;

		// Set the enumeration values
		if(enumValues != null || enumValues.length != 0) {
			this._setEnumValues(name, null, enumValues, attribute.getType());
		}
		return attribute;
		
	},
		
	// Deletes a new attribute on the part.  This method will
	// not work on an attribute that has been saved to the server.
	deleteAttribute: function(name, isEnum){
		// Delete any enum values sets
		if(isEnum){
			this._deleteEnumValues(name);
		}
		
		// Delete the json object
		// Find the correct object in the array
		var attributeArray = this.templateObj.Attributes.Attribute;
		var found = -1;
		for(i=0; i<attributeArray.length; i++){
			if(attributeArray[i].name == name){
				found = i;
			}
		}
		if(found != -1){
			this.templateObj.Attributes.Attribute = this._util._removeFromArray(attributeArray, found);
		}
	},
	
	// Edits the part changing it's name.  This method will
	// only work on "editable" Parts (ie one that have never
	// been saved on the server).
	edit: function(name){
		if(!this.isEditable()) {
			throw new Error("Part is not editable");
		}
		
		var oldname = this.templateObj.name;
		this.templateObj.name = name;
		this.templateObj.type = name;
		var template = this.getTemplate();
		template.partMap[name] = this;
		delete template.partMap[oldname];
	},
		
	// Returns whether the part is editable (ie has never been
	// saved on the server).
	isEditable: function(){
		var editable = this.templateObj.editable;
		if(editable == "true"){
			return true;
		}
		return false;
	},
		
	// Marks a part as hidden.  Hidden parts will not be returned 
	// from the server in templates or lab assets.
	hide: function(){
		this.templateObj.hidden = "true";
	},

	// Marks a part as no longer hidden.  
	show: function(){
		this.templateObj.hidden = "false";
	},
		
	// Returns true if the part is hidden.
	isHidden: function(){
		if(this.templateObj.hidden == null || this.templateObj.hidden == "false") {
			return false;
		}
		return true;
	},

    _setEnumValues: function(name, id, enumValues, attributeType){
    	if(enumValues != null && !dojo.isArray(enumValues)) {
    		throw new Error("enumValues must be an array or null");
    	}
        if(this.templateObj.AttributeValueRanges == null || this.templateObj.AttributeValueRanges == ""){
            this.templateObj.AttributeValueRanges = {};
            this.templateObj.AttributeValueRanges["AttributeValueRange"] = [] ;
        }
        if(this.templateObj.AttributeValueRanges.AttributeValueRange == null || this.templateObj.AttributeValueRanges.AttributeValueRange == ""){
            this.templateObj.AttributeValueRanges["AttributeValueRange"] = [] ;
        }
        var rangeArray = this.templateObj.AttributeValueRanges["AttributeValueRange"];
        if (!dojo.isArray(rangeArray)) {
            rangeArray = [rangeArray];
        }
        
        var found = -1;
        for(var i=0; i<rangeArray.length && found == -1; i++){
            if(rangeArray[i].name == name){
                found = i;
            }
        }
		var temp = {};
		temp.name = name;
		if(id != null) {
			temp.id = id;
		}
		temp.value = {};
	    temp.value[this.enumTypeLookup[attributeType]] = enumValues;
		if(found != -1){
			if(enumValues == null || enumValues.length == 0) {
				this._deleteEnumValues(name);
			} else {
				this.templateObj.AttributeValueRanges["AttributeValueRange"][found] = temp;
			}
		} else {
			if(enumValues != null && enumValues.length != 0) {
				this._util._getArrayObject(this.templateObj, "AttributeValueRanges", "AttributeValueRange").push(temp);	        	
	        }
	    }
    },

    _deleteEnumValues: function(name){
        if(this.templateObj.AttributeValueRanges == null || this.templateObj.AttributeValueRanges == ""){
            this.templateObj.AttributeValueRanges = {};
            this.templateObj.AttributeValueRanges["AttributeValueRange"] = [] ;
        }
        if(this.templateObj.AttributeValueRanges.AttributeValueRange == null || this.templateObj.AttributeValueRanges.AttributeValueRange == ""){
            this.templateObj.AttributeValueRanges["AttributeValueRange"] = [] ;
        }
        var rangeArray = this.templateObj.AttributeValueRanges["AttributeValueRange"];
        if (!dojo.isArray(rangeArray)) {
            rangeArray = [rangeArray];
        }
        
        var found = -1;
        for(var i=0; i<rangeArray.length && found == -1; i++){
            if(rangeArray[i].name == name){
                found = i;
            }
        }
        if(found != -1){
            this.templateObj.AttributeValueRanges["AttributeValueRange"] = 
            	this._util._removeFromArray(this.templateObj.AttributeValueRanges["AttributeValueRange"], found);
        }
    },

    // Returns the attribute path from an asset type of which the part can be 
    // created. Returns null if there's none.
    getAccessibleAttributePaths: function() {
        var root = this.getRoot();
        
        // normalize to null if undefined or is empty.
        if(!root.templateObj.Paths || !root.templateObj.Paths.Path)
            return null;

        return this._util._getArrayObject(root.templateObj, "Paths", "Path");
    },
    
	// Sets a name value pair on the part in the template.  This
	// property will be set on all instances of the part.
	setProperty: function(name, value) {
		this.templateObj[name] = value;
	},
	
	// Gets the palue of a property thathas been set.
	getProperty: function(name) {
		return this.templateObj[name];
	},
		
	// Adds the tag that will be shown
	// in the Whitch configuration builder.
	setWhitchTag: function(name, accessPath){
        var whitchTag = {
            'name': name,
            'path': accessPath,
            'rootId': this.getRoot().templateObj.id
        };
		this.templateObj.WhitchTags = {};
        // TODO: inner 'WhitchTag' needs to be an array.
        // it's not coming back as an array. Keep it the
        // same for now. Also, need to replace, or append
        // a whtich tag based on the path.
        this.templateObj.WhitchTags.WhitchTag = whitchTag;
	},

	// Removes the tag that will be shown in
	// the Whitch configuration builder.  This
	// will cause this part to not be shown in
	// that UI.
	removeWhitchTag: function(){
        if(this.templateObj.WhitchTags)
            delete this.templateObj.WhitchTags;
		//this.templateObj.WhitchTags = null;
	},
		
    // Returns the tag, if any, for
    // the Whitch configuration builder.
    getWhitchTag: function(){
        if(!this.templateObj.WhitchTags) {
            return null;
        }
        return this.templateObj.WhitchTags.WhitchTag;
    },
	
	// Returns true if the part has a tag for
	// the Whitch configuration builder.
	hasWhitchTag: function(){
		if(!this.templateObj.WhitchTags) {
			return false;
		}
		return true;
	}
	
}   
)
var Part=com.ibm.asq.common.web.client.internal._catalog.Part;
}
)();
