
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
(function(){
dojo.provide("com.ibm.asq.common.web.client.internal._catalog.Template");
dojo.require("com.ibm.asq.common.web.client.internal._catalog._DocumentBase");
dojo.declare("com.ibm.asq.common.web.client.internal._catalog.Template", com.ibm.asq.common.web.client.internal._catalog._DocumentBase, 
{
    constructor: function(obj){
        this.referencedTemplateMap = {};
        this.partMap = {};
        this.rootPart = null;
        if (typeof obj == 'string'){
            obj = dojo.fromJson(obj);
        }
        this.obj = obj; // obj is the javascript object (JSON) used by the catalog system
        if (obj){
            var inner = obj.AssetTemplate;
            if (!inner){
                if(obj.AssetType != null) {
                    inner = obj;
                }
                if (!inner) {
                    throw new Error("cannot find root config part!");
				}
            }
            this.rootObj = inner.AssetType;
            if (!this.rootObj.Attributes){
                this.rootObj.Attributes = {"Attribute": []};
            }
        } 
    },
    // Get the name of the template.
    getName: function(){
        return this.rootPart.getName();
    },
    // Get the id of the template.
    getId: function(){
        return this.rootObj.id;
    },
    // Get whether the template is an Asset Template.  An Asset 
    // Template is one that is a type that can exist 
    // independently for other types.  Types in an Asset 
    // template are things that could be requested.)
    isAssetTemplate: function(){
       var toplevel = this.rootObj.toplevel;
	   return (toplevel == "true");
   },
    // Get the names of the parts in the template.    Note that 
    // this list will never contain the name of the root Part. 
    getPartNames: function(){
        var names = new Array();
        var rootName = this.rootPart.getName();
        for(var part in this.partMap) {
            var name = this.partMap[part].getName();
            if(name != rootName) {
                names.push(name);
            }
        }
        return names.sort();
    },
    // Get a Part by name.
    getPart: function(partName){
        return(this.partMap[partName]);
    },
    // Get the template's root Part.
    getRoot: function(){
        return this.rootPart;
    },
    // Get the names of all the attributes in all of the Parts
    // in the template
    getAllAttributeNames: function(){
        console.debug("getAllAttributeNames");
        var attNameArray = [];
        var partMap = this.partMap;
        for(var partName in partMap){
            var part = partMap[partName];
            var attributeArrayObject = part._util._getArrayObject(part.templateObj, "Attributes", "Attribute");
            for(var i=0; i<attributeArrayObject.length; i++) {
                var name = attributeArrayObject[i].name;
                attNameArray.push(name);
            }
        }
        return attNameArray.sort();  
    },
    
    // return the object as JSON.  Optionally pretty-print it.
    toJson: function(prettyPrint){
        return dojo.toJson(this.obj,prettyPrint);
    }

}   
)
})();
