/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient");

dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.planning.web.client.internal.WorkItemUtils");

(function(){ //module container
	var getCurrentPA            = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
    var ServiceRequest          = com.ibm.team.repository.web.transport.ServiceRequest;
    var TeamServerClient        = com.ibm.team.repository.web.transport.TeamServerClient;
	var ServiceResponseHandler	= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    var WorkItemUtils           = com.ibm.rqm.planning.web.client.internal.WorkItemUtils;
    
    com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient = {
    
        TEST_CASE_SERVICE_URI: "com.ibm.rqm.planning.common.service.rest.ITestCaseRestService",
        EWI_REST_SERVICE_URI:"com.ibm.rqm.execution.common.service.rest.IExecutionWorkItemRestService",
		
        _invokeService: function(responseHandler, method, appArgs){
            appArgs = appArgs ? appArgs : {};
			if (!appArgs.processArea) {
				appArgs.processArea = getCurrentPA().itemId;
			}			
            var serviceRequest = new ServiceRequest(this.TEST_CASE_SERVICE_URI, method, appArgs);
            TeamServerClient.invokeService(serviceRequest, responseHandler);
			return serviceRequest; //return the service request - enable clients to abort request
        },
        
        getTestCases: function(serviceResponseHandler, args){
            return this._invokeService(serviceResponseHandler, "getTestCasesDTO", args);
        },
		
        getTestPlanTestCases: function(serviceResponseHandler, args){
			//expected args: versionedTestPlanItemId
            return this._invokeService(serviceResponseHandler, "getTestPlanTestCasesDTO", args);
        },		

        getTestPlanTestCasesMinimal: function(serviceResponseHandler, args){
            args = args ? args : {};
			args.minimal = true;
			return this._invokeService(serviceResponseHandler, "getTestPlanTestCasesDTO", args);
        },
		        
        getTestCasesSummary: function(serviceResponseHandler, args){
            return this._invokeService(serviceResponseHandler, "getTestCasesSummaryDTO", args);
        },
		
        getTestScriptTestCases: function(serviceResponseHandler, versionableItemId) {
			var args = {
				versionedTestScriptItemId: versionableItemId
			}
            return this._invokeService(serviceResponseHandler, "getTestScriptTestCases", args);
        },
		
        searchTestCases: function(serviceResponseHandler, args)
        {
            /*
                    VALID PARAMETERS
                    --------------------------------
                    String processArea;         //itemId    
                    String name;                    
                    String[] originatorUUIDs;   //itemId               
                    String[] stateIds;          //eg. 'com.ibm.rqm.state.new'          
                    String[] categories;        //itemId               
                    boolean myItemsOnly = false;                    
                    boolean includeSnapshots = false;
                    boolean includeArchived = false;
                    String policy;              //'enforce', 'truncate', 'ignore'  
             */         
            if ( !args ) args = { };
            return this._invokeService(serviceResponseHandler, "getSearchTestCases", args);
        },  		
		
        getTestCasesMinimal: function(serviceResponseHandler, args){			
            args = args ? args : {};
            args.minimal = true;
            return this._invokeService(serviceResponseHandler, "getTestCasesSummaryDTO", args);
        },		
		
        getTestCase: function(serviceResponseHandler, testId){
            var actualHandler = this.getWorkItemMappingHandler(serviceResponseHandler);
            return this._invokeService(actualHandler, "getTestCasesDTO", {
                id: testId,
                resolveWorkItems: true,
				includeArchived: true
            });
        },
        
        getTestCaseRevision: function(serviceResponseHandler, testId, aRevision){
            var actualHandler = this.getWorkItemMappingHandler(serviceResponseHandler);
            return this._invokeService(actualHandler, "getTestCasesDTO", {
                uid: testId,
                resolveWorkItems: true,
                revision: aRevision
            });
        },
        
        removeTestCase: function(serviceResponseHandler, removeArgs){
            return this._invokeService(serviceResponseHandler, "postDeleteTestCase", removeArgs);
        },
        
        archiveTestCase: function(serviceResponseHandler, args)
        {
            return this._invokeService(serviceResponseHandler, "postArchiveTestCase", args);
        },
		
        postLockTestCase: function(serviceResponseHandler, args)
        {
            var actualHandler = this.getWorkItemMappingHandler(serviceResponseHandler);
            return this._invokeService(actualHandler, "postLockTestCase", args);
        },      		
       
		changeSuspectTestCase: function(serviceResponseHandler, args) {
			var actualArgs= {};
			if (args) {
				var jsonArgs= dojo.toJson(args);
				actualArgs.jsonString= jsonArgs;
				if(args.itemsJsonString) {
					actualArgs.itemsJsonString= args.itemsJsonString;
				}
			}
			return this._invokeService(serviceResponseHandler, "postChangeSuspectTestCase", actualArgs);	
		},
		
		postBulkChangeTestCase: function(serviceResponseHandler, args)
		{
			var actualHandler = this.getWorkItemMappingHandler(serviceResponseHandler);			
			return this._invokeService(actualHandler, "postBulkChangeTestCaseSummaryDTO", args);
		},
		
        saveTestCase: function(serviceResponseHandler, aTestObject){
            var actualHandler = this.getWorkItemMappingHandler(serviceResponseHandler);
            return this._invokeService(actualHandler, "postTestCaseDTO", aTestObject);
        },
        
        getSnapshots: function(serviceResponseHandler, args){
            if (!args) 
                args = {};
            return this._invokeService(serviceResponseHandler, "getSnapshots", args);
        },
        
        getSnapshotsMultipleItems: function(serviceResponseHandler, args){
            if (!args) 
                args = {};
            return this._invokeService(serviceResponseHandler, "getSnapshotsMultipleItems", args);
        },
		
        getWorkItemMappingHandler: function(responseHandler) {
			var start = new Date();
            var internalHandler = {
                success: function(testCaseDTOs){
                    //fixup each workitem for each test case
                    testCaseDTOs = WorkItemUtils.fixupArtifactsWorkItems(testCaseDTOs);
                    responseHandler.doCallback(testCaseDTOs);
                },
                failure: function(error){
                    responseHandler.doErrorCallback(error);
                }
            };
            var handler = new ServiceResponseHandler(internalHandler, "success", "failure");
            return handler;
        },
		postArchiveEWIsForTC: function(serviceResponseHandler, params)
		{
			params = params ? params : {};
			if (!params.projectAreaItemId) {
				params.projectAreaItemId = getCurrentPA().itemId;
			}			
            var serviceRequest = new ServiceRequest(this.EWI_REST_SERVICE_URI, "postArchiveEWIsForTC", params);
            TeamServerClient.invokeService(serviceRequest, serviceResponseHandler);
			return serviceRequest;
		}
    };
    
})();
