/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
/*
*
*	This class will be the base class for entities
*	It should not be instantiated, and is in effect abstract
*	This class is extended by the specific types of entities, 
*	Which specify the serviceInterface by calling setServiceInterface()
*
*/
(function() {
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.asq.common.web.client.internal.api");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");

dojo.provide("com.ibm.asq.common.web.client.internal.container");

//
//extends the api so that it has access to global vars like the server url
//

dojo.declare("com.ibm.asq.common.web.client.internal.container", com.ibm.asq.common.web.client.internal.api, 
		
		
		{
		constructor: function(){
			//Superclass seems to be autoconstructed.
			//console.debug("container constructed");
		},
		/*
		*	This function can be used to create an entity.
		*/
		
		setServiceInterface: function(serviceInterface){
			this.serviceInterface = serviceInterface;
		},
		
		_invokeService : function (responseHandler, method, args) {
			var myServiceRequest=
				new com.ibm.team.repository.web.transport.ServiceRequest(
					this.serviceInterface,
					method, 
					args);
	 						
			com.ibm.team.repository.web.transport.TeamServerClient.invokeService(
				myServiceRequest, responseHandler );
		
		},

		/*
		* This function returns the details about the entities that met the specified attribute criteria.
		*/
		getByAttributes: function(responseHandler, attributes) {
			return this._invokeService(responseHandler, "getByAttributes", attributes); 
		
		},
		
        /**
         * This function returns the list of attribute names of the object.
         */
        getAttributeNames: function(responseHandler) {
            return this._invokeService(responseHandler, "getAttributeNames"); 
        },
        
        /**
         * This function returns the possible values for an attribute.
         */
        getAttributeValues: function(responseHandler, attribute) {
            return this._invokeService(responseHandler, "getAttributeValues", attribute); 
        },
		
		_removeNulls: function( args )
		{
			for( var name in args )
			{
				if( args[ name ] == null )
				{
					delete args[ name ];
				}
			}
			
			return( args );
		}
         
		/*
		* This function returns the details about a given entity by its name.
		*/
/*		
		findByName: function(userName){
			console.debug("Attempting to find "+this.type+" called "+userName);
			var deferred = dojo.rawXhrPost( {
		    url: this.entityPath,
		    handleAs: "json",
		    timeout: this.httpRequestTimeout, // Time in milliseconds
		    postData:userName,
		    headers:{action:'findByName'},
		    
		    //The handle is called in either the error or success case
		    handle: function(response, ioArgs) {
		        //return for other callbacks
		        return response;
		    }
		    	
	  		})
	  		
	  		deferred.addCallback(function(response){
			        console.debug("Got a successful server response");
			       
			        //You MUST return the response object from this function, otherwise the error callbacks
			        //will be called.
			        return response;
			});
	  		
	  		//Add another callback for the error case
			deferred.addErrback(function(response){
				        if(response.dojoType == "cancel") {
		                // The request was canceled by some other JavaScript code
		                	console.debug("Request canceled.");
		           		} else if(response.dojoType == "timeout") {
		                // The request took over 5 seconds to complete
		                	console.debug("Request timed out.");
		            	} else {
		                // Some other error
		                console.error(response);
		               }
				       
				        //Return response to propagate the same response to other callback handlers.
				        return response;
				})
			
			//give the deferred object back to the frontend so they can add additional callbacks for UI
			return deferred;
		},
*/		

/*		
		create: function(entityInformationObject){
			console.debug("Attempting creation of "+this.type);
			var deferred = dojo.rawXhrPost( {
		    url: this.entityPath,
		    handleAs: "json",
		    timeout: this.httpRequestTimeout, // Time in milliseconds
		    postData: dojo.toJson(entityInformationObject),
		    headers:{action:'new'},
			
		    //The handle is called in either the error or success case
		    handle: function(response, ioArgs) {
		        //return for other callbacks
		        return response;
		    }
		    	
	  		})
	  		
	  		
	  		deferred.addCallback(function(response){
			        console.debug("Got a successful server response");
			       
			        //You MUST return the response object from this function, otherwise the error callbacks
			        //will be called.
			        return response;
			});
	  		
	  		//Add another callback for the error case
			deferred.addErrback(function(response){
				        if(response.dojoType == "cancel") {
		                // The request was canceled by some other JavaScript code
		                	console.debug("Request canceled.");
		           		} else if(response.dojoType == "timeout") {
		                // The request took over 5 seconds to complete
		                	console.debug("Request timed out.");
		            	} else {
		                // Some other error
		                console.error(response);
		               }
				       
				        //Return response to propagate the same response to other callback handlers.
				        return response;
				})
			
			//give the deferred object back to the frontend so they can add additional callbacks for UI
			return deferred;
		},
*/		
		
		
		
		/*
		*	This function can be used to create an entity.
		*/
/*		
		update: function(entityInformationObject){
			console.debug("Attempting update of "+this.type);

			var deferred = dojo.rawXhrPut( {
			//pull user id from json for url
		    url: this.entityPath+entityInformationObject.uuid,
		    handleAs: "json",
		    timeout: this.httpRequestTimeout, // Time in milliseconds
		  	putData: dojo.toJson(entityInformationObject),
			
		    //The handle is called in either the error or success case
		    handle: function(response, ioArgs) {
		        //return for other callbacks
		        return response;
		    }
		    	
	  		})
	  		
	  		deferred.addCallback(function(response){
			        console.debug("Got a successful server response");
			       
			        //You MUST return the response object from this function, otherwise the error callbacks
			        //will be called.
			        return response;
			});
	  		
	  		//Add another callback for the error case
			deferred.addErrback(function(response){
				        if(response.dojoType == "cancel") {
		                // The request was canceled by some other JavaScript code
		                	console.debug("Request canceled.");
		           		} else if(response.dojoType == "timeout") {
		                // The request took over 5 seconds to complete
		                	console.debug("Request timed out.");
		            	} else {
		                // Some other error
		                console.error(response);
		               }
				       
				        //Return response to propagate the same response to other callback handlers.
				        return response;
				})
			
			//give the deferred object back to the frontend so they can add additional callbacks for UI
			return deferred;
		},
*/		
		
		/*
		*	This function can be used to create an entity.
		*/
/*		
		remove: function(entityId){
			console.debug("Attempting deletion of "+this.type);
			var deferred = dojo.xhrDelete( {
		    url: this.entityPath+entityId,
		    handleAs: "json",
		    timeout: this.httpRequestTimeout, // Time in milliseconds
			
		    //The handle is called in either the error or success case
		    handle: function(response, ioArgs) {
		        //return for other callbacks
		        return response;
		    }
		    	
	  		})
	  		
	  		deferred.addCallback(function(response){
			        console.debug("Got a successful server response");
			       
			        //You MUST return the response object from this function, otherwise the error callbacks
			        //will be called.
			        return response;
			});
	  		
	  		//Add another callback for the error case
			deferred.addErrback(function(response){
				        if(response.dojoType == "cancel") {
		                // The request was canceled by some other JavaScript code
		                	console.debug("Request canceled.");
		           		} else if(response.dojoType == "timeout") {
		                // The request took over 5 seconds to complete
		                	console.debug("Request timed out.");
		            	} else {
		                // Some other error
		                console.error(response);
		               }
				       
				        //Return response to propagate the same response to other callback handlers.
				        return response;
				})
			
			//give the deferred object back to the frontend so they can add additional callbacks for UI
			return deferred;
		},
*/		
/*
		getAttributes: function(){
			console.debug("Attempting to get "+this.type+" attribute names");
			var deferred = dojo.rawXhrPost( {
		    url: this.entityPath,
		    handleAs: "json",
		    timeout: this.httpRequestTimeout, // Time in milliseconds
		    headers:{action:'getAttributes'},
		    
		    //The handle is called in either the error or success case
		    handle: function(response, ioArgs) {
		        //return for other callbacks
		        return response;
		    }
		    	
	  		});
	  		
	  		deferred.addCallback(function(response){
			        console.debug("Got a successful server response");
			       
			        //You MUST return the response object from this function, otherwise the error callbacks
			        //will be called.
			        return response;
			});
	  		
	  		//Add another callback for the error case
			deferred.addErrback(function(response){
				        if(response.dojoType == "cancel") {
		                // The request was canceled by some other JavaScript code
		                	console.debug("Request canceled.");
		           		} else if(response.dojoType == "timeout") {
		                // The request took over 5 seconds to complete
		                	console.debug("Request timed out.");
		            	} else {
		                // Some other error
		                console.error(response);
		               }
				       
				        //Return response to propagate the same response to other callback handlers.
				        return response;
				})
			
			//give the deferred object back to the frontend so they can add additional callbacks for UI
			return deferred;
		
		}
*/
	
		
});
})();
