/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

(function(){
dojo.provide("com.ibm.rational.test.lm.web.client.internal.reservations");
	
dojo.require("com.ibm.asq.common.web.client.internal.container");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var Catalog = com.ibm.asq.common.web.client.internal.Catalog;
// Summary:
//    Internal object to normalize the reservation response with catalog information.
//    Convert catalog configuration string to objects for client consumption.
//
dojo.declare("reservations.callbackObject", null,
{
    constructor: function(handler)
    {
        this.responseHandler = handler;
    },
    successFunc: function(response) {
//       if(response) {
//           var cat = new Catalog();
//           // create the configuration object from the JSON string in the reservation response.
//           for( var i = 0; i < response.length; i++ )
//           {
//				var config = cat.createAssetFromJson(response[i].configuration);
//				response[i].configuration = config;
//           }
//        }
        if(this.responseHandler)
            this.responseHandler.doCallback(response);
    },
    errorFunc: function(errorObj) {
        if(this.responseHandler)
            this.responseHandler.doErrorCallback(errorObj);
    }
});

/**
 * Provides service methods dealing with reservations.
 */
dojo.declare("com.ibm.rational.test.lm.web.client.internal.reservations", com.ibm.asq.common.web.client.internal.container,       
    {
        constructor: function()
        {
            this.setServiceInterface('com.ibm.rational.test.lm.common.IReservationRestService');
        },

        _getProjectAreaArgs: function()
		{
		    return {"projectAreaItemId" : com.ibm.team.process.web.projectArea.itemId };	
		},
		     
        // Get the reservations associated with the logged in user.
        // Returns an array of ReservationDTO JSON objects.
        getReservationsFromCurrentUser: function(responseHandler)
        {
            var args = this._getProjectAreaArgs();
            var callbackObj = new reservations.callbackObject(responseHandler);
            var myHandler = new ServiceResponseHandler(callbackObj, "successFunc", "errorFunc");
            return this._invokeService(myHandler, "getReservationsFromCurrentUser", args);
        },     
        
        // Get the reservations associated with a user, where the user is specified
        // through a string representing the UUID of the user.
        // Returns an array of ReservationDTO JSON objects. 
        getReservationsFromUser: function(responseHandler, userId)
        {
            var args = this._getProjectAreaArgs();
            args["uuid"] = userId;
            var callbackObj = new reservations.callbackObject(responseHandler);
            var myHandler = new ServiceResponseHandler(callbackObj, "successFunc", "errorFunc");
            return this._invokeService(myHandler, "getReservationsFromUser", args);
        },
        
		// Get the pending reservations for a set of lab resources within a
		// time window starting "now" and extending to "window" number of 
		// minutes.  Returns an array of PerResourceReservationsDTO JSON 
		// objects.  Each object in the top-level array contains:
		//
		//     resourceId: lab resource item ID
		//     canAutomate: boolean flag indicating whether the current user 
		//                  can configure automations against the current
		//                  resource
		//     canDiscover: boolean flag inidcating whether the current user can
		//                  perform discovery against the current resource
		//     reservations: an array of ReservationSummaryDTO JSON objects that
		//                   apply to the resource identified by resourceId with
		//                   the format:
		//
		//        id: reservation item ID
		//        reserveFrom: reservation starting timestamp
		//        reserveTo: reservation ending timestamp
		//        reservedFor: RTLM Contributor DTO:
		//
		//            name: reserving user's name
		//            id: reserving user's item ID
		getReservationsInWindow: function(responseHandler, resourceIds, window)
		{
			var args = {"resourceIds" : resourceIds, "window" : window};
			return this._invokeService(responseHandler, "postReservationsInWindow", args);
		},
		  
         // Get this list of views available for the current user.
         // Returns a "named code" object which contains a name and an id.
         getViewIdsForCurrentUser: function(responseHandler) {
            return this._invokeService(responseHandler, "getViewIds", this._getProjectAreaArgs());
         },
         
         // Gets the reservations for a specified view id.
         // The argument should be and integer view id.
         // The return is an array of reservations.
         getViewData: function(responseHandler, viewId) {
		 	var args = this._getProjectAreaArgs();
			
			args["viewId"] = viewId;
			
            var callbackObj = new reservations.callbackObject(responseHandler);
			var myHandler = new ServiceResponseHandler(callbackObj, "successFunc", "errorFunc");
         
            return this._invokeService(myHandler, "getViewData", args);         
         },

        // Cancel a reservation where the reservation is specified through a string
        // representing the UUID of the reservation record.
        // Returns a boolean value indicating success of the cancellation
        cancelReservation: function(responseHandler, reservationId)
        {
            var args = {};
            args["uuid"] = reservationId;
            return this._invokeService(responseHandler, "postCancelReservation", args);
        },
        
        // End a reservation (set end time to "now) where the reservation is specified 
        // through a string representing the UUID of the reservation record.
        // Returns a boolean value indicating success of the reservation end operation
        endReservation: function(responseHandler, reservationId)
        {
            var args = {};
            args["uuid"] = reservationId;
            return this._invokeService(responseHandler, "postEndReservation", args);
        },
        
        // Create a self-service reservation.  Requires the asset ID and 
        // reservation start/end timestamps.  Returns a DTO representing the newly-
        // created reservation.
        createReservation: function(responseHandler, assetUuid, from, to)
        {
            return this._invokeService(responseHandler, 
                                       "postReserve",
                                        {
                                            "assetUuid" : assetUuid, 
                                            "from" : from,
                                            "to" : to
                                        }); 
        },
		
		// Fill any gaps in the reservation.  Requires the asset ID and 
        // reservation start/end timestamps.  Returns an array of itemIDs of the
        // resulting reservations (which may be empty).  The final parameter is
		// an indicator if the service should fail if there is a reservation in the gap
		// by another user. 
        fillReservationGaps: function(responseHandler, assetUuid, from, to, failIfReserved)
        {
            return this._invokeService(responseHandler, 
                                       "postReserveGaps",
                                        {
                                            "assetUuid" : assetUuid, 
                                            "from" : from,
                                            "to" : to,
											"failIfReserved" : failIfReserved
                                        }); 
        },
		
		// Returns boolean "true" if any of the lab resources
		// contained in the array of asset UUIDs, assetUUIDs,
		// have a  current or futu
		checkReservations: function(responseHandler, assetUUIDs)
		{
			return this._invokeService(responseHandler,
			                            "getAreMachinesReserved",
										{
	                                       "uuid" : assetUUIDs
                                        });
  		}
    }   
)
})();
