/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rqm.defects.web.helper.RemoteItemPicker");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.rqm.defects.web.helper.RemoteChooserContainer");
dojo.require("jazz.ui.Dialog");

(function(){

    dojo.declare("com.ibm.rqm.defects.web.helper.RemoteItemPicker", [dijit._Widget, dijit._Templated], {
        /*
         * Parameters for RemoteItemPicker:
         *   parentElement (req): the parent element to place the dialog as a child
         *   dialogTitle (req): dialog title
         *   secondaryTitle (req): secondary dialog title
         *   pickerURL (req): the picker url
         *   remoteProjectAreaName (opt): the name of the remote project area
         *   action (opt): an element passed on with the result
         *   width (opt): the width of the dialog
         *   height (opt): the height of the dialog
         *   allowsMultiple (opt): if set, multiple elements can be selected
         *   exclusionList (opt): If set, specifies elements that should be excluded
         *   onOk: (req) function(result, action) called when items have been selected. Result type is [ { linkComment, url } ]
         */
        constructor: function(args){
            if (!args.dialogTitle) 
                throw new Error("Missing 'dialogTitle'");
            if (!args.secondaryTitle) 
                throw new Error("Missing 'secondaryTitle'");
            if (!args.pickerURL) 
                throw new Error("Missing 'pickerURL'");
            if (!dojo.isFunction(args.onOk)) 
                throw new Error("Missing 'onOk'");
        },
        
        postCreate: function(){
            this._initContentWidget();
        },
		
        _onClose: function(){
            this.floatingPane.close();
        },
		
        _onOk: function(result, frameArguments){
            this.onOk(result, this.action);
            this._onClose();
        },
		
        _initContentWidget: function(){
            var frameContainerArgs = {
                remoteLocation: this.pickerURL,
                onData: dojo.hitch(this, this._onOk)
            };
            
            var dialogContent = new com.ibm.rqm.defects.web.helper.RemoteChooserContainer(frameContainerArgs);
            
            var floatingPaneParams = {
                primaryTitle: this.dialogTitle,
                secondaryTitle: this.secondaryTitle,
				width: this.width || "597px",
				height: this.height || "400px",
                onClose: this.onClose,
                contentNode: dialogContent.domNode
            };
            this.floatingPane = new jazz.ui.Dialog(floatingPaneParams);
        }
    });
})();
