/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rqm.reports.cognos.web.ui.internal.NewTemplateDialog");

dojo.require("com.ibm.rqm.reports.cognos.web.client.internal.CognosReportsClient");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.reports.cognos.web.ui.internal.CognosReportsBrowseDialog");
dojo.require("com.ibm.rqm.reports.cognos.web.client.internal.Constants");

//Needed for loading saved server information
dojo.require("com.ibm.asq.common.web.client.internal.option");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Tree");
dojo.require("dojo.data.ItemFileReadStore");
dojo.require("net.jazz.ajax.ui.internal.Dialog");

var CognosReportsClient = com.ibm.rqm.reports.cognos.web.client.internal.CognosReportsClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var TransportServiceResponseWrapper=com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var NewTemplateDialog = com.ibm.rqm.reports.cognos.web.ui.internal.NewTemplateDialog;
var CognosReportsBrowseDialog = com.ibm.rqm.reports.cognos.web.ui.internal.CognosReportsBrowseDialog;
var PlanningConstants = com.ibm.rqm.planning.web.client.internal.Constants;
var Constants = com.ibm.rqm.reports.cognos.web.client.internal.Constants;
var getCurrentProjectArea  = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
(function() { // module container



var AlertDialog 	= com.ibm.asq.common.web.ui.internal.widgets.AlertDialog;

/**
 * Displays a dialog to upload a cognos report to RTC server
 */
dojo.declare(
	"com.ibm.rqm.reports.cognos.web.ui.internal.NewTemplateDialog",
	[dijit._Widget, dijit._Templated],	{
	
	templatePath: dojo.moduleUrl("com.ibm.rqm.reports.cognos.web", "ui/internal/templates/NewTemplateDialog.html"),
	labelOK: Constants.getMessage("labelOK"),
	labelCancel: Constants.getMessage("labelCancel"),
	labelFolder: Constants.getMessage("labelFolder"),
	labelBrowse: Constants.getMessage("labelBrowse"),
	labelProperties: Constants.getMessage("labelProperties"),
	labelName: Constants.getMessage("labelName"),
	labelDescription: Constants.getMessage("labelDescription"),
	labelShared: Constants.getMessage("labelShared"),
	labelReportDetails: Constants.getMessage("labelReportDetails"),
	labelCognosReport: Constants.getMessage("labelCognosReport"),
	
	constructor: function() {
		//Declare necessary bits and try to read saved server info
		this.reportServer = [];	
		this.reportServerAddress = null;
		this.reportServerUser = null;
		this.reportServerPass = null;
		this._optionsClient = new com.ibm.asq.common.web.client.internal.option();
		this._preference = null;
		
		//set up folder list
        var foldersHandler = new ServiceResponseHandler(this, "_setFolders", "_FolderError");
        ReportsClient.getFolderDescriptors(foldersHandler, {flags: ReportsClient.SHARED});
				
		var args = {
			projectAreaItemId: getCurrentProjectArea().itemId,
			key: PlanningConstants.REPORT_SERVER_PREFERENCE_KEY
		}
		var handler = new TransportServiceResponseWrapper(this, "_setPreference", "_Error");
		this._optionsClient.getProjectPreferences(handler, args.projectAreaItemId, args.key);

	},
	
	uninitialize: function() {
		if (this.cognosReportsBrowseDialog){
			this.cognosReportsBrowseDialog.destroy();
		}
	},
	
	_setFolders: function(folders) {
		console.debug(this.declaredClass + ":_setSharedFolders --> entry.");
		for (var i = 0; i < (folders ? folders.length : 0); i++) {
			var optn = document.createElement("OPTION");
			optn.text=folders[i].name;
			optn.value=folders[i].folderUUID;
			this._folderInput.options.add(optn);
		}
		var optn = document.createElement("OPTION");
		optn.text="System";
		optn.value="";
		this._folderInput.options.add(optn);
	},
	
	_setPreference: function(preference) {
    	if (preference != null) {
        	this._preference = preference;
            this.reportServer = dojo.fromJson(preference.value);
            this.reportServerAddress = this.reportServer[0].address;
            if (this.reportServerAddress.charAt(this.reportServerAddress.length-1,1) == "/")
            	this.reportServerAddress = this.reportServerAddress.substring(0,this.reportServerAddress.length-1);
            this.reportServerUser = this.reportServer[0].user;
            this.reportServerPass = this.reportServer[0].pass;
        } else {
        	this.reportServer = PlanningConstants.REPORT_SERVER_PREFERENCE_DEFAULTS.concat([]);   
        	this.reportServerAddress = this.reportServer[0].address;
        	if (this.reportServerAddress.charAt(this.reportServerAddress.length-1,1) == "/")
            	this.reportServerAddress = this.reportServerAddress.substring(0,this.reportServerAddress.length-1);
            this.reportServerUser = this.reportServer[0].user;
            this.reportServerPass = this.reportServer[0].pass;
        }
	},
	
	postCreate: function() {
		this.dialog = new net.jazz.ajax.ui.internal.Dialog({
			contentNode: this.contentNode,
			primaryTitle: Constants.getMessage("labelImportReport"),
			width: "390px",
			isClosable: true
		});
		this.connect(this._browseButton, "onclick", this._onBrowse);
		this.connect(this._okButton, "onclick", this._onOk);
		this.connect(this._cancelButton, "onclick", this._onCancel);
	},
	
	_onBrowse: function() {
	  var args = {
	  	viewer: this,
	    ip:this.reportServerAddress,
		username:this.reportServerUser,
		password:this.reportServerPass,
		"namespace":this._namespaceInput.value,
		elementsToBeDisplayed: "ReportTemplate"
	  }	;
	  var currentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();

	  this.cognosReportsBrowseDialog = new CognosReportsBrowseDialog(args);
	  this.connect(this.cognosReportsBrowseDialog._okButton, "onclick", this._onOkClick);	  
	},
	
	_onOkClick: function() {
		this.reportNode = this.cognosReportsBrowseDialog.node;
		this.reportPath = this.cognosReportsBrowseDialog.reportPath;
		this._IDInput.value = this.cognosReportsBrowseDialog.reportId;
		var fileName = this.reportNode.label;
		this._nameInput.value = fileName;
		this._reportInput.value = fileName;
		this._okButton.disabled = false;
		this.cognosReportsBrowseDialog.close();
	},
	
	_onCancel: function() {
		this.dialog.close();
	},
	
	_handleError: function(){
		(new AlertDialog({'message': Constants.getMessage("errorCantCommunicate") })).open();
	},
	
	_Error: function(){
		(new AlertDialog({'message': Constants.getMessage("errorCantCommunicate") })).open();
	},
	
	_FolderError: function(){
		(new AlertDialog({'message': Constants.getMessage("errorCantCommunicate") })).open();
	},
	
	_onOk: function(){
		this.dialog.close();
	}	
});

})();