/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.defects.web.ui.internal.RTCWorkItemSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.process.web.client.projectarea");

dojo.require("com.ibm.rqm.defects.web.helper.RemoteItemPicker");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");
dojo.require("com.ibm.rqm.defects.web.ui.internal.LinksTableView");
dojo.require("com.ibm.rqm.defects.web.client.internal.CmIntegrationClient");
dojo.require("com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil");
dojo.require("com.ibm.rqm.defects.web.client.ChangeRequestManager");
dojo.require("com.ibm.rqm.defects.web.ui.view.common.ProgressDialog");
dojo.require("com.ibm.rqm.defects.web.util.ItemPickerUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

(function(){ // module container
    var AbstractArtifactEditorSection = com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection;
    var RemoteItemPicker = com.ibm.rqm.defects.web.helper.RemoteItemPicker;
    var LinksTableView = com.ibm.rqm.defects.web.ui.internal.LinksTableView;
    var CmIntegrationClient = com.ibm.rqm.defects.web.client.internal.CmIntegrationClient;
    var ChangeRequestManager = com.ibm.rqm.defects.web.client.ChangeRequestManager;
    var DefectsUIUtil = com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil;
    var ProgressDialog = com.ibm.rqm.defects.web.ui.view.common.ProgressDialog;
    var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
    var WorkItemPickerUtil = com.ibm.rqm.defects.web.util.ItemPickerUtil;
    
    dojo.declare("com.ibm.rqm.defects.web.ui.internal.RTCWorkItemSection", [dijit._Widget, dijit._Templated, AbstractArtifactEditorSection], {
        templateString: '<div class="com-ibm-rqm-rtcworkitem-section">' +
        '   <div dojoAttachPoint="_tableContainer"></div>' +
        '</div>',
        
        constructor: function(args){
            this._items = [];
            this._newAddedItems = [];
            this._removedItems = [];
        },
        
        postCreate: function(){
            this.inherited('postCreate', arguments);
        },
        
        //@Overriden from AbstractArtifactEditorSection
        initialize: function(isEditable){
            this._onInitialized();
        },
        
        _onInitialized: function(){
            if (this.getArtifact()) {
                var paramsGetLinks = {};
                paramsGetLinks.testArtifactItemId = this.getArtifact().versionableItemId;
                CmIntegrationClient.getPlanItemLinks(paramsGetLinks, dojo.hitch(this, "_initalizeTableViewer"));
            }
            else {
                this._initalizeTableViewer(null, []);
            }
        },
        
        _initalizeTableViewer: function(status, items){
            console.debug("RTC work item section:: Init - " + items);
            if (!this._linksTableViewer) {
                dojox.data.dom.removeChildren(this._tableContainer)
                this._linksTableViewer = new LinksTableView({
                    editable: this.isEditable(),
                    printable: this.isPrintView(),
                    enableRowSelection: true,
                    actions: this.getTableActions()
                });
                this._tableContainer.appendChild(this._linksTableViewer.domNode);
            }
            if (items) {
                this._items = items;
            }
            this._linksTableViewer.refresh(this._items);
        },
        
        //@Overridden from AbstractArtifactEditorSection
        saveFinished: function(savedItem){
            this.inherited('saveFinished', arguments);
            if (this._newAddedItems.length == 0 && this._removedItems.length == 0) {
                console.debug("No links to be saved")
                return;
            }
            var currentProjectArea = getCurrentProjectArea(); 
            var projectAreaUUID = currentProjectArea.itemId;
            var self = this;
            ProgressDialog.doRunWithProgress(DefectsUIUtil.getMessage("msgLinkgWithTeamConcert"), dojo.hitch(self, function(monitor){
                // posting remote links
                for (var index in self._newAddedItems) {
                    monitor.beginTask(DefectsUIUtil.getMessage("msgPostingWorkItemLink", self._newAddedItems[index].id));
                    var testArtifact = self.getArtifact();
                    var testArtifactType = self.getArtifactType() == "Test Plan" ? "testplan" : "testcase";
                    var testLinkUrl = location.protocol + "//" + location.host + net.jazz.ajax._contextRoot + "/oslc_qm/contexts/" + projectAreaUUID + "/resources"
                    if (testArtifactType == "testplan") {
                    	testLinkUrl += "/com.ibm.rqm.planning.TestPlan/";
                    }
                    else {
                    	testLinkUrl += "/com.ibm.rqm.planning.TestCase/";
                    }
                    testLinkUrl += testArtifact.itemId;
                    var testLinkComment = self.getArtifactType() + " " + testArtifact.id + " - " + testArtifact.name;
                    var testLinkType = com.ibm.rqm.defects.web.client.ChangeRequestManager.LINK_TYPE_TESTED_BY;
                    var linkDescription = {};
                    linkDescription.id = self._newAddedItems[index].id;
                    linkDescription.url = self._newAddedItems[index].url;
                    linkDescription.linkComment = testLinkComment;
                    linkDescription.rqmUrl = testLinkUrl;
                    linkDescription.linkType = testLinkType;
                    
                    ChangeRequestManager.getInstance().getCurrentServiceProvider(dojo.hitch(this, function(currentProvider){
                        currentProvider.postLink(linkDescription, dojo.hitch(this, function(status, response){
                            if (status.isOK()) {
                                console.debug("posting remote links succeed");
                            }
                            else {
                                console.debug("posting remote links error");
                                monitor.setCancelled();
                            }
                        }));
                    }));
                }
                console.debug("Plan Item section - Posting local link");
                // posting local links
                if (self._newAddedItems.length > 0) {
                    var postParams = {};
                    postParams.testArtifactItemId = self.getArtifact().versionableItemId;
                    var workItems = [];
                    for (var index in self._newAddedItems) {
                        workItems.push(dojo.toJson({
                            uri: self._newAddedItems[index].url,
                            summary: self._newAddedItems[index].summary
                        }));
                    }
                    postParams.workItemJsonStrings = workItems;
                    monitor.beginTask(DefectsUIUtil.getMessage("msgPostingLocalLinks"));
                    CmIntegrationClient.postNewPlanItemLinks(postParams, dojo.hitch(self, function(status, result){
                        if (status.isOK()) {
                            self._newAddedItems = [];
                        }
                        else {
                            console.debug("posting local links error");
                            monitor.setCancelled();
                        }
                    }));
                }
                
                
                if (self._removedItems.length > 0) {
                    for (var index in self._removedItems) {
                        // deleting remote links
                        monitor.beginTask(DefectsUIUtil.getMessage("msgDeletingWorkItemLink"), self._removedItems[index].id);
                        var testArtifact = self.getArtifact();
                        var testArtifactType = self.getArtifactType() == "Test Plan" ? "testplan" : "testcase";
                        var testLinkUrl = location.protocol + "//" + location.host + net.jazz.ajax._contextRoot + "/service/com.ibm.rqm.integration.service.IIntegrationService/resources/" + projectAreaUUID + "/" + testArtifactType + "/urn:com.ibm.rqm:" + testArtifactType + ":" + testArtifact.id;
                        var testLinkType = com.ibm.rqm.defects.web.client.ChangeRequestManager.LINK_TYPE_TESTED_BY;
                        var linkDescription = {};
                        linkDescription.url = self._removedItems[index].url || self._removedItems[index].uri;
                        linkDescription.rqmUrl = testLinkUrl;
                        linkDescription.linkType = testLinkType;
                        
                        ChangeRequestManager.getInstance().getCurrentServiceProvider(dojo.hitch(this, function(currentProvider){
                            currentProvider.deleteLink(linkDescription, dojo.hitch(this, function(status, response){
                                if (status.isOK()) {
                                    console.debug("deleting remote links succeed");
                                }
                                else {
                                    console.debug("deleting remote links error");
                                    monitor.setCancelled();
                                }
                            }));
                        }));
                    }
                    
                    // deleting local links
                    var postDeleteParams = {};
                    postDeleteParams.testArtifactItemId = self.getArtifact().versionableItemId;
                    var uris = [];
                    for (var index in self._removedItems) {
                        uris.push(self._removedItems[index].url || self._removedItems[index].uri);
                    }
                    postDeleteParams.workItemUris = uris;
                    monitor.beginTask(DefectsUIUtil.getMessage("msgDeletingLocalLinks"));
                    CmIntegrationClient.postDeletePlanItemLinks(postDeleteParams, dojo.hitch(self, function(status, result){
                        if (status.isOK()) {
                            self._removedItems = [];
                        }
                        else {
                            console.debug("deleting local links error");
                            monitor.setCancelled();
                        }
                    }));
                }
                monitor.done();
            }));
        },
        
        //actions to be displayed in the table header
        getTableActions: function(){
            var actions = [];
            //actions.push({
            //    label: DefectsUIUtil.getMessage("labelCreateNewWorkitem"),
            //    actionHandler: dojo.hitch(this, "_handleCreateWorkItemClicked"),
            //    iconClass: "add-new-requirement-image",
            //    disabledOnReadOnly: true
            //});
            actions.push({
                label: DefectsUIUtil.getMessage("labelAddNewWorkitem"),
                actionHandler: dojo.hitch(this, "_handleAddWorkItemClicked"),
                iconClass: "add-icon-image",
                disabledOnReadOnly: true
            });
            actions.push({
                label: DefectsUIUtil.getMessage("labelRemoveWorkitem"),
                actionHandler: dojo.hitch(this, "_handleRemoveWorkitemClicked"),
                iconClass: "remove-reqirement-image",
                disabledOnReadOnly: true
            });
            return actions;
        },
        
        _handleCreateWorkItemClicked: function(){
            var self = this;
            var serviceProviderManager = com.ibm.rqm.defects.web.client.ChangeRequestManager.getInstance();
            
            // discover dialog uri
            serviceProviderManager.getCurrentServiceProvider(dojo.hitch(this, function(currentServiceProvider){
                if (currentServiceProvider) {
                    var creationDialogUri = currentServiceProvider.providerUris.defaultCreationDialogURI;
                    // select an item from the selection dialog
                    var params = {
                        dialogTitle: DefectsUIUtil.getMessage("labelPlanItemSelectionPicker"),
                        secondaryTitle: DefectsUIUtil.getMessage("labelPlanItemCreationPickerTitle"),
                        pickerURL: creationDialogUri,
                        height: '400px',
                        width: '650px',
                        parentElement: this.domNode,
                        onOk: function(selectedItems){
                            var workItems = currentServiceProvider.linkType == ChangeRequestManager.PROVIDER_LINK_TYPE_CQREST ? WorkItemPickerUtil.parseFromCQ(selectedItems) : WorkItemPickerUtil.parseFromRTC(selectedItems);
                            for (var index = 0; index < workItems.length; index++) {
                                self._newAddedItems.push(workItems[index]);
                                self._items.push(workItems[index]);
                            }
                            self._linksTableViewer.refresh(self._items);
                            self.setDirty();
                        },
                        onClose: function(){
                        }
                    };
                    new RemoteItemPicker(params);
                }
                else {
                    // Neither cq rest nor RTC project area is configured.
                }
            }));
        },
        
        _handleAddWorkItemClicked: function(){
            var self = this;
            var serviceProviderManager = com.ibm.rqm.defects.web.client.ChangeRequestManager.getInstance();
            
            // discover dialog uri
            serviceProviderManager.getCurrentServiceProvider(dojo.hitch(this, function(currentServiceProvider){
                if (currentServiceProvider) {
					var planItemSelectionDialog = currentServiceProvider.services.getSelectionDialog("planItem", true);
                    var selectionDialogUri = planItemSelectionDialog.uri;
                    // select an item from the selection dialog
                    var params = {
                        dialogTitle: DefectsUIUtil.getMessage("labelPlanItemSelectionPicker"),
                        secondaryTitle: DefectsUIUtil.getMessage("labelPlanItemSelectionPickerTitle"),
                        pickerURL: selectionDialogUri,
                        height: planItemSelectionDialog.height,
                        width: planItemSelectionDialog.width,
                        parentElement: this.domNode,
                        onOk: function(selectedItems){
                            var workItems = currentServiceProvider.linkType == ChangeRequestManager.PROVIDER_LINK_TYPE_CQREST ? WorkItemPickerUtil.parseFromCQ(selectedItems) : WorkItemPickerUtil.parseFromRTC(selectedItems);
                            if (workItems && workItems.length > 0) {
                                dojo.forEach(workItems, function(workItem){
                                    if (self._containsUri(self._newAddedItems, workItem.uri || workItem.url) || self._containsUri(self._items, workItem.uri || workItem.url)) {
                                        // do nothing
                                    }
                                    else 
                                        if (self._containsUri(self._removeItems, workItem.uri || workItem.url)) {
                                            self._removeItemFromArray(self._removeItems, workItem.uri || workItem.url);
                                            self._items.push(workItem);
                                        }
                                        else {
                                            self._newAddedItems.push(workItem);
                                            self._items.push(workItem);
                                        }
                                });
                                if (self._newAddedItems && self._newAddedItems.length > 0) {
                                    self._linksTableViewer.refresh(self._items);
                                    self.setDirty();
                                }
                            }
                        },
                        onClose: function(){
                        }
                    };
                    new RemoteItemPicker(params);
                }
                else {
                    // Neither cq rest nor RTC project area is configured.
                    var alertDialog = new com.ibm.asq.common.web.ui.internal.widgets.AlertDialog();
                    alertDialog.setMessage(DefectsUIUtil.getMessage("msgNoneCMProvidersConfigured"));
                    alertDialog.open();
                }
            }));
        },
        
        _handleRemoveWorkitemClicked: function(){
            var checkedRows = this._linksTableViewer.getCheckedRows();
            var self = this;
            dojo.forEach(checkedRows, function(item){
                self._removeItem(item);
            })
            if (this._removedItems && this._removedItems.length > 0) {
                this.setDirty();
            }
            this._linksTableViewer.refresh(this._items);
        },
        
        _removeItem: function(item){
            if (this._containsUri(this._items, item.uri || item.url)) {
                this._items = this._removeItemFromArray(this._items, item.uri || item.url);
            }
            var newAdded = false;
            if (this._containsUri(this._newAddedItems, item.uri || item.url)) {
                this._newAddedItems = this._removeItemFromArray(this._newAddedItems, item.uri || item.url);
                newAdded = true;
            }
            if (!newAdded) {
                this._removedItems.push(item);
            }
        },
        
        _containsUri: function(items, uri){
            var result = false;
            dojo.forEach(items, function(item){
                if (item.uri == uri || item.url == uri) {
                    result = true;
                }
            });
            return result;
        },
        
        _removeItemFromArray: function(array, itemUri){
            var updatedArray = [];
            for (var i = 0; i < array.length; i++) {
                if (array[i].uri != itemUri && array[i].url != itemUri) {
                    updatedArray.push(array[i]);
                }
            }
            return updatedArray;
        }
    });
    
})();
