/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.defects.web.ui.internal.RmCollectionLinksSection");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");
dojo.require("com.ibm.rqm.defects.web.ui.internal.RmLinksSection");
dojo.require("com.ibm.rqm.defects.web.client.internal.ResourceLinkService");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.planning.PlanningPageClient");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.rqm.defects.web.ui.view.common.ProgressDialog");
dojo.require("com.ibm.rqm.planning.web.client.internal.template.TemplateClient");
dojo.require("com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil");

(function(){ // module container
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var ResourceLinkService = com.ibm.rqm.defects.web.client.internal.ResourceLinkService;
    var PermissionsClient = com.ibm.asq.common.web.client.internal.PermissionsClient;
    var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    var TestCaseClient = com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient;
    var PlanningClient = com.ibm.rqm.planning.web.client.internal.planning.PlanningPageClient;
    var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
    var ProgressDialog = com.ibm.rqm.defects.web.ui.view.common.ProgressDialog;
    var TemplateClient = com.ibm.rqm.planning.web.client.internal.template.TemplateClient;
    var DefectsUIUtil = com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil;
    
    dojo.declare("com.ibm.rqm.defects.web.ui.internal.RmCollectionLinksSection", com.ibm.rqm.defects.web.ui.internal.RmLinksSection, {
        getResourceType: function(){
            return ResourceLinkService.RESOURCE_TYPE_REQ_COL;
        },
        
        getResourcePrefix: function(){
            return '/resources/com.ibm.rqm.planning.TestPlan/';
        },
        
        getResourceLabel: function(){
            return Constants.getMessage("labelTestPlan");
        },
        
        getResourceSelectDialog: function(provider){
            return provider.services.collectionSelectDialog;
        },
        
        //actions to be displayed in the table header
        getTableActions: function(){
            var actions = this.inherited(arguments);
            //add "create test cases" button
            var tcPerm = PermissionsClient.getStoredPermission(PermissionsClient.OP_TEST_CASE);
            if (!tcPerm || tcPerm.create) {
                actions.push({
                    label: Constants.getMessage("labelCreateTestCases"),
                    actionHandler: dojo.hitch(this, "_handleCreateTestCasesClicked"),
                    iconClass: "add-new-testcase-image",
                    disabledOnReadOnly: true
                });
            }
            return actions;
        },
        
        _handleCreateTestCasesClicked: function(){
            var checkedRows = this._linksTableViewer.getCheckedRows();
            if (!this.getArtifact() || checkedRows.length == 0) {
                return;
            }
            var requirements = [];
            var self = this;
            dojo.forEach(checkedRows, dojo.hitch(this, function(collection){
                ProgressDialog.doRunWithProgress("", dojo.hitch(this, function(monitor){
                    // Pop up the monitor before sending any xhr requests. This is to prevent user from double click the actions.
                    monitor.beginTask(DefectsUIUtil.getMessage("msgCreatingTestCase"));
                    ResourceLinkService.getLinksFromReqCollection(collection.uri, dojo.hitch(this, function(requirements){
                        if (requirements.length > 0) {
                            var handler = {
                                success: function(template){
                                    self._generateTestCases(template, requirements, monitor);
                                },
                                error: function(){
                                }
                            };
                            var responseHandler = new ServiceResponseHandler(handler, "success", "error");
                            TemplateClient.getDefaultTemplate(responseHandler, "com.ibm.rqm.planning.editor.testcase");
                        } else {
                        	// no requirements in the collection
                        	monitor.done();
                        }
                    }), function(error){
                        monitor.done();
                        alert(error.responseText || error.message || error);
                    });
                }));
            }));
        },
        
        _generateTestCases: function(template, requirements, monitor){
            var testcases = [];
            var index = 0;
            dojo.forEach(requirements, dojo.hitch(this, function(requirement){
                // 1. create test case per requirement
                var aTestCaseObj = {};
                aTestCaseObj.newTestCase = true;
                aTestCaseObj.name = requirement.summary;
                aTestCaseObj.templateItemId = template.itemId;
				aTestCaseObj.state = Constants.STATETRANSITION_STATE_DRAFT;
                var self = this;
                var handler = {
                    success: function(testcase){
                        index++;
                        testcases.push(testcase);
                        // 2.1 create test case's requirement link;
                        monitor.beginTask(DefectsUIUtil.getMessage("msgLinkingTestCase", requirement.summary));
                        ResourceLinkService.updateValidatesLinks(ResourceLinkService.RESOURCE_TYPE_REQ, self._getTestCaseResourceUri(testcase), [requirement], []);
                        
                        // 2.2 create requirement's back link
                        var linkDescription = self._getTestCaseLinkDesc(testcase);
                        ResourceLinkService.addValidatedByLink(ResourceLinkService.RESOURCE_TYPE_REQ, requirement.uri, linkDescription);
                        
                        if (index == requirements.length) {
                            // 3. add test cases into test plan
                            monitor.beginTask(DefectsUIUtil.getMessage("msgAddingTestCase"));
                            var addedUUIDs = [];
                            dojo.forEach(testcases, function(testcase){
                                addedUUIDs.push(testcase.versionableItemId);
                            });
                            var parms = {};
                            parms.itemId = self.getArtifact().itemId;
                            parms.stateId = self.getArtifact().stateId;
                            parms.testCasesAdded = addedUUIDs;
                            var handler = {
                                success: function(){
                                    monitor.done();
                                    self.refreshArtifact();// refresh editor w/ latest state as the state has changed post linking
                                },
                                error: function(){
                                    monitor.done();
                                }
                            };
                            var responseHandler = new ServiceResponseHandler(handler, "success", "error");
                            PlanningClient.postTestPlan(responseHandler, parms);
                        }
                    },
                    error: function(e){
                    }
                }
                var responseHandler = new ServiceResponseHandler(handler, "success", "error");
                TestCaseClient.saveTestCase(responseHandler, aTestCaseObj);
            }));
        },
        
        _getTestCaseLinkDesc: function(artifact){
            var testLinkUrl = this._getTestCaseResourceUri(artifact);
            var testLinkComment = Constants.getMessage("labelTestCase") + " " + artifact.id + " - " + artifact.name;
            var linkDescription = {};
            linkDescription.linkComment = testLinkComment;
            linkDescription.rqmUri = testLinkUrl;
            return linkDescription;
        },
        
        _getTestCaseResourceUri: function(testcase){
            var projectAreaId = getCurrentProjectArea().itemId;
            var resourceUri = location.protocol + '//' + location.host + net.jazz.ajax._contextRoot + "/oslc_qm/contexts/" + projectAreaId + '/resources/com.ibm.rqm.planning.TestCase/' + testcase.itemId;
            return resourceUri;
        }
    });
    
})();
