/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //    
dojo.provide("com.ibm.asq.common.web.ui.internal.catalog.CatalogModel");

dojo.require("com.ibm.asq.common.web.client.internal.Catalog");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");

(function() {

var ServiceResponseHandler  = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;

dojo.declare("com.ibm.asq.common.web.ui.internal.catalog.CatalogModel",
             null,
{   
    partIsAlwaysContainer: false,
    showHidden : false,
    showWhitchTag: false,
	filterNonWhitchTagParts: false,
    
    postscript: function(params){
        dojo.mixin(this, params);
        
        this._catalog = new com.ibm.asq.common.web.client.internal.Catalog();
        this.editErrors = {};
        this.labelChangedListener = [];
    },
    
    addLabelChangedListener: function(listener)
    {
        this.labelChangedListener.push(listener);    
    },
    
    hasEditErrors: function()
    {
        var errCnt = 0;
        for(var i in this.editErrors) {
            errCnt++;
        }
        return errCnt > 0;
    },
    
    _checkStoreErrors: function(item)
    {
        var id = item.id;

        // check for error in the name, if there's an error
        // store the error in question. Clear the error if
        // it has been corrected.  
        if(this.hasError(item)) {
            this.editErrors[id] = true;
        }
        else if(this.editErrors[id]) {
            delete this.editErrors[id];
        }
    },
    
    // Summary:
    //   API method to save the changes made to the catalog model. This method is asychronous, the call
    //   must pass in a callback method to be invoked when the operation is completed on the server. The 
    //   callback will be called with this parameter: {hasError: <true|false>, errorMessage: <string message>}
    updateCatalog: function(cb) 
    {
        var handlerObj = {
            _self: this,
            success: function(){
                    cb({hasError: false, errorMessage: ""});
            },
            failure: function(e){
                var message = (e.message) ? e.message : MessageUtil.getMessage(MessageBundle.COMMON, "msgErrSaveCatalog");
				var callbackData = {
					hasError: true,
                    errorMessage: message
				};
				if (e.responseText) {
					callbackData.responseText = e.responseText;
				} 
                cb(callbackData);
            }
        };
        var responseHandler = new ServiceResponseHandler(handlerObj, "success", "failure");
        this._catalog.updateCatalog(responseHandler);
    },
    
	initCatalogSandbox: function(respondHandler)
	{
        this._catalog.initializeSandboxTemplates(respondHandler);		
	},
	
    // dijit.tree.Model  API
    getRoot: function(/*function(items)*/ onComplete){
        if(!this.root) {
            this.root = this.newItem({
                part: null,
                name: MessageUtil.getMessage(MessageBundle.COMMON, "labelCatalogRootTypes"),
                id: "root",
                type:"root"}, null);
        }
        onComplete(this.root);
    },
	
	getCatalog: function()
	{
		return this._catalog;
	},
    
    _filterRootPart: function(part) {
        if(!this.filterNonWhitchTagParts)
            return false;
        
        // filter out non witch taggable parts. The whitch taggable part types
        // must have subparts, not an asset type, and have accessible attribute
        // that the type can be created in a configuration.
        var noSubParts = part.getChildParts().length < 1; 
        var isAssetType = part.getTemplate().isAssetTemplate();      
        var noAttrPaths = part.getAccessibleAttributePaths() == null;
        
        return noSubParts || isAssetType || noAttrPaths; 
    
    },
    
    // dijit.tree.Model  API
    mayHaveChildren: function(/*dojo.data.Item*/ item){
        var hasChildren = item.mayHaveChildren || (item.children && item.children.length > 0);
        return Boolean(hasChildren);
    },
    
    _getCatalogTemplates: function()
    {
        var ids = this.getCatalog().getTemplateIdsFromSandbox();
        var templates = [];
        
        for(var i = 0; i < ids.length; i++) {
            templates.push(this.getCatalog().getTemplateFromSandboxById(ids[i]));
        }
        templates.sort(function(t1, t2){
            var v1, v2;
            v1 = MessageUtil.getMessage(MessageBundle.LM_CATALOG, t1.getName()).toLowerCase();
            v2 = MessageUtil.getMessage(MessageBundle.LM_CATALOG, t2.getName()).toLowerCase();
            return (v1 > v2) ? 1 : -1;
        });
        return templates;
    },
    
    // dijit.tree.Model  API
    getChildren: function(/*dojo.data.Item*/ parentItem, /*function(items)*/ onComplete){
        // parent has no children
        if (!this.mayHaveChildren(parentItem)) {
            onComplete([]);
            return;
        }
        // create a 'veneer' object over the catalog parts for the UI.
        // This simplifies sorting the items, etc. Always reconstruct 
        // the items from the catalog.
        parentItem.children = [];
        if (parentItem.type == "root") {
            var templates = this._getCatalogTemplates();
            dojo.forEach(templates, dojo.hitch(this, function(template){
                var rootPart = template.getRoot();
                
                if(rootPart.isHidden() && !this.showHidden)
                    return;
                
                if(this._filterRootPart(rootPart))
                    return;
                        
                this.newItem({
                    part: rootPart,
                    name: rootPart.getName(),
                    type: "part"
                }, parentItem);
            }));
        }
        else if (parentItem.type == "part") {
            var subParts = parentItem.part.getChildParts();
            
            dojo.forEach(subParts, dojo.hitch(this, function(subPart){
                // don't create the item if the part is hidden and
                // we're not showing hidden.
                if(subPart.isHidden() && !this.showHidden)
                    return;
                
				// retrieve the new pos from the part. It is stored when it's
				// created. Honor the position it was created.
                if (subPart.isEditable()) {
                    var cnt = subPart.getProperty("newPos") || (cnt + 1);
                    subPart.newCount = cnt;
                }
                this.newItem({
                    part: subPart,
                    name: subPart.getName(),
                    type: "part"
                }, parentItem);
            }));
            
            parentItem.children.sort(dojo.hitch(this, function(item1, item2){
                if(item1.part.newCount && item2.part.newCount) {
                    return item1.part.newCount > item2.part.newCount ? 1 : -1;
                }
                else if(item1.part.newCount) {
                    return -1;
                }
                else if(item2.part.newCount) {
                    return 1;
                }
                
                // sort by name only. 
                var val1 = this.getLabel(item1).toLowerCase();
                var val2 = this.getLabel(item2).toLowerCase();
                
                return (val1 > val2) ? 1 : -1;
            }));
        }    
        onComplete(parentItem.children);
    },
    
    // dijit.tree.Model  API
    getIdentity: function(/* item */ item){
        return item.id;
    },
    
    // dijit.tree.Model  API
    getLabel: function(/*dojo.data.Item*/ item, noWhitchTag){
		var label = MessageUtil.getMessage(MessageBundle.LM_CATALOG, item.name);
		
        noWhitchTag = (typeof noWhitchTag != "undefined" && noWhitchTag == true);
		if(!noWhitchTag && this.showWhitchTag && item.type == "part") {
			var tag = item.part.getWhitchTag();
			if(tag) {
				label = label + " (" + tag.name + ")";
			}
		}
        return label;
    },
    
    // dijit.tree.Model  API
    newItem: function(/* Object? */ args, /*Item?*/ parent){
        var mayHaveChildren = true; 
        var iid = args.id || this._generateItemId(parent, args.part);
        
        if(args.type == "part") {
            mayHaveChildren = this.partIsAlwaysContainer || this._hasChildParts(args.part);
        }
        var item = {
            part: args.part,
            name: args.name,
            type: args.type,
            id: iid,
            mayHaveChildren: mayHaveChildren,
            parent: parent,
            destroyed: false,
            children: null
        };
        if (parent != null) {
            if(!parent.children)
                parent.children = [];
            parent.children.push(item);
        }
        return item;
    },
    
    checkDescendantsHasAttribute: function(item, attribute)
    {
        return this._checkPartDescendantsHasAttribute(item.part, attribute);
    },
    
    _checkPartDescendantsHasAttribute: function(part, attribute)
    {
        var hasValue = false;
        var children = part.getChildParts();
        
        for (var i = 0; i < children.length; i++) {
            if (attribute == "hidden") {
                hasValue = children[i].isHidden();
            }
            else if (attribute == "whitchTagged") {
                hasValue = children[i].hasWhitchTag();
            }
            if(!hasValue)
                hasValue = this._checkPartDescendantsHasAttribute(children[i], attribute);
                
            if(hasValue)
                return hasValue;
        }
        return hasValue;
    },
    
    // Extended  API
    isLabelEditable: function(/*Item*/item) {
        if(item.type == "part")
            return item.part.isEditable();
        return false;    
    },

    // Extended API for catalog
    labelChanged: function(/*Item*/item, newName) {
        if(this.isLabelEditable(item)) {
            item.name = newName;
            item.part.edit(newName);
            this.onChange(item);
            
            this._checkStoreErrors(item);   
            
            for(var i = 0; i < this.labelChangedListener.length; i++) {
                var listener = this.labelChangedListener[i];
                if(listener.onLabelChanged && dojo.isFunction(listener.onLabelChanged)) {
                    listener.onLabelChanged(item);
                }
            }         
        }
    },
    
    // Extended API for catalog
    setWhitchTagOnItem: function(item, name, accessPath)
    {
        item.part.setWhitchTag(name, accessPath);
    },

    // Extended API for catalog
    removeWhitchTagOnItem: function(item)
    {
        item.part.removeWhitchTag();
    },
    
    // Extended API for catalog
	getWhitchTagOnItem: function(item)
	{
	    return item.part.getWhitchTag();
	},
	
    getPartAccessiblePaths: function(item)
    {
        // the path is stored on the root part. Get it from the root.
        var rootPart = item.part.getRoot();
        var paths = rootPart.getAccessibleAttributePaths();
        var accPaths = [];
        
        if(paths) {
            dojo.forEach(paths, function(path) {
                // only create machine config right now...
                if(path.components.component[0] != "VirtualImage") {
                    accPaths.push(path);
                }
            })
        }
        return accPaths;
    },
    
    // Extended API for catalog
    removeItem: function(parentItem, item)
    {
        // temp new item, we can delete it.
        if(item.part.isEditable()) {
            var i;
            for(i = 0; i < parentItem.children.length; i++) {
                if(parentItem.children[i].id == item.id)
                    break;
            }
            if(i < parentItem.children.length) {
                parentItem.children.splice(i, 1);
            }
            item.part.deletePart();
            
            // also clear any error associated w/ it.
            if(this.editErrors[item.id]) {
                delete this.editErrors[item.id];
            }
        }
        else {
            item.part.hide();
        }
    },
    
    // Extended API for catalog
    restoreItem: function(parentItem, item)
    {
        if (item.part.isHidden()) {
            item.part.show();
        }
    },
    
    // Extended API for catalog
    addNewItem: function(parentItem, inChild) {
        var parent = parentItem;
        if(!parent.children) {
            parent.children = [];
        }
        var cnt = 1;
        dojo.forEach(parent.children, function(item) {
            if(item.part.isEditable()) {
                cnt++;
            }
        });
        var isRootType = (parent.type == "root");
        var typeName = this.getLabel(parent);
        var uniqueName = (isRootType) ? MessageUtil.getMessage(MessageBundle.COMMON, "labelNewRootTypePrompt", cnt) : MessageUtil.getMessage(MessageBundle.COMMON, "labelNewTypePrompt", typeName, cnt);
        var newPartType;
        
        // Make sure that the proposed name doesn't clash with an existing name.
        // This is a highly unlikely scenario, but notheless a possible case. This 
        // happens when the user accepts the prompt name as the name of the new type
        // without editing it and saved it.
        var clashCnt = 0;
        dojo.forEach(parent.children, function(item) {
            if(item.part.getName() == uniqueName) {
                clashCnt++;
            }
        });
        if(clashCnt > 0) {
            cnt += clashCnt;
            uniqueName = (isRootType) ? MessageUtil.getMessage(MessageBundle.COMMON, "labelNewRootTypePrompt", cnt) : MessageUtil.getMessage(MessageBundle.COMMON, "labelNewTypePrompt", typeName, cnt);
        }
        
        if(isRootType) {
            var newTemplate = this._catalog.createTemplate(uniqueName);
            newPartType = newTemplate.getRoot();
        }
        else if (parent.type == "part") {
            var part = parent.part;
            newPartType = part.createChildPart(uniqueName);
        }
        // store the new count of the part in the part
        // object. We want to keep the new items to stay
        // in the same spot when we close the item and show it
        // again, etc.
		newPartType.setProperty("newPos", cnt);
        
        var newItem = {
            part: newPartType,
            name: uniqueName,
            type: "part",
            id: this._generateItemId(parentItem, newPartType),
            mayHaveChildren: true,
            parent: parentItem,
            destroyed: false,
            children: null
        };
        parent.children.splice((cnt - 1), 0, newItem);
        
        // return the position of the new item in the
        // parent.
        return cnt - 1;
    },
    
    // Extended API for catalog
    hasError: function(item) {
        if(this.isLabelEditable(item)) {
            var parent = item.parent;
            if(!parent.children || parent.children.length < 1)
                return false;

            for(var i = 0; i < parent.children.length; i++) {
                var childItem = parent.children[i];
                if(childItem.id != item.id) {
                    if(!childItem.part.isHidden() && childItem.name == item.name) {
                        return true;
                    }
                }
            }                
        }
        return false;
    },
        
    _generateItemId: function(parentItem, childPart)
    {
        if(!childPart.isEditable()) {
            return parentItem.id + "." + childPart.getName();
        }
        var cnt = childPart.getProperty("newPos");
        return parentItem.id + "." + "newtype" + cnt;
    },
    
    _hasChildParts: function(part){
        if (part) {
            var subparts = part.getChildParts();
            return subparts && subparts.length > 0;
        }
        return false;
    },
    
    // dijit.tree.Model  API
    pasteItem: function(/*Item*/ childItem, /*Item*/ oldParentItem, /*Item*/ newParentItem, /*Boolean*/ bCopy){
        // not implemented
    },
    
    // dijit.tree.Model  API
    onChange: function(/*dojo.data.Item*/ item){
        //not implemented
    },
    
    // dijit.tree.Model  API
    onChildrenChange: function(/*dojo.data.Item*/ parent, /*dojo.data.Item[]*/ newChildrenList){
        // not implemented
    }
});

})();