/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.buildintegration.web.ui.internal.config.BuildPageProjectArea");

dojo.require("dijit._Templated");
dojo.require("dijit.form.TextBox");
dojo.require("dijit.form.ValidationTextBox");

dojo.require("dojox.data.dom");

dojo.require("com.ibm.team.repository.web.transport.ExternalTeamServerDescription");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("com.ibm.rqm.buildintegration.web.ui.internal.wizard.WizardPage");
dojo.require("com.ibm.team.repository.web.foundation.Status");
dojo.require("com.ibm.rqm.buildintegration.web.client.internal.RemoteTeamConcertClient");

dojo.require("com.ibm.rqm.buildintegration.web.ui.internal.BuildUiUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

(function(){

var RemoteTeamConcertClient				= com.ibm.rqm.buildintegration.web.client.internal.RemoteTeamConcertClient;
var ServiceResponseHandler				= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var Status								= com.ibm.team.repository.web.foundation.Status;
var ExternalTeamServerDescription		= com.ibm.team.repository.web.transport.ExternalTeamServerDescription;
var BuildUiUtil 					= com.ibm.rqm.buildintegration.web.ui.internal.BuildUiUtil;

dojo.declare("com.ibm.rqm.buildintegration.web.ui.internal.config.BuildPageProjectArea", 
			 [com.ibm.rqm.buildintegration.web.ui.internal.wizard.WizardPage, dijit._Templated], {

	_context: null,
	_rootNode: null,
	_projectAreaLabel: null,
	_projectAreaSelection: null,
	
	_cachedProjectAreas: null,
	_cachedProjectAreasServer: null,

	constructor: function() {
		this.setPageComplete(false);
	},

	getContentNode: function() {
		return this._rootNode;
	},
	_onCancel:function(){
		if(this._timer)
			clearTimeout(this._timer);
		dojo.disconnect(this._wizardDialogOnCancel);
	},
	
	aboutToShow: function(runWhenReady) {
		if(this._timer)
			clearTimeout(this._timer);
		if(!this._wizardDialogOnCancel)
			this._wizardDialogOnCancel = dojo.connect(this._wizardDialog,"_onCancel",this,"_onCancel");
		if (this._cachedProjectAreasServer !== this._context.url) {
			dojox.data.dom.removeChildren(this._projectAreaSelection);
			this._cachedProjectAreasServer= null;

			this.runWithProgress(dojo.hitch(this, function(monitor) {
				monitor.beginTask(BuildUiUtil.getMessage("labelFetchingProjectAreas"));

				var teamConcertServer= new ExternalTeamServerDescription(this._context.name, this._context.description, this._context.url);
				var self = this;
				var serviceCallback= dojo.hitch(this, function(status, result) {
					try {
						if(self._timer)
							clearTimeout(self._timer);
						var serviceResult= status;
						try {
							if (status.isOK()) {
								this._cachedProjectAreasServer= this._context.url;
								this._displayProjectAreas(result.collection);
								if(this._wizardDialog._currentPage == null)
									this._wizardDialog._currentPage = this;
								this._update();
							} else {
								serviceResult= new Status(Status.ERROR, BuildUiUtil.getMessage("labelFailedConnectServer"));
							}
						} finally {
							runWhenReady(Status.OK_STATUS);
						}
					} finally {
						monitor.done();
					}
				});
		
				RemoteTeamConcertClient.getProjectAreas(teamConcertServer, serviceCallback, { requestLabel: BuildUiUtil.getMessage("labelConfigureBuildProvider") } );
				this._timer = setTimeout(function(){
					var alertDialog = new com.ibm.asq.common.web.ui.internal.widgets.AlertDialog({
                		'message': BuildUiUtil.getMessage("labelFailedConnectServer")
					});
            		alertDialog.open();
					monitor.done();
				},60000)
			}));
		} else {
			runWhenReady(Status.OK_STATUS);
		}
	},
	
	aboutToHide: function() {
		this._context.projectArea= this._projectAreaSelection.value;
	},

	templatePath: dojo.moduleUrl("com.ibm.rqm.buildintegration.web", "ui/internal/config/templates/BuildPageProjectArea.html"),
	widgetsInTemplate: true,

	postCreate: function() {
		this.connect(this._projectAreaSelection, "onchange", "_update");
	},

	_update: function() {
		this.setPageComplete(dojo.isString(this._projectAreaSelection.value) && this._projectAreaSelection.value.length > 0);
	},
	
	_displayProjectAreas: function(projectAreaDescs) {
		dojo.forEach(projectAreaDescs, dojo.hitch(this, function(projectAreaDesc) {
			var projectAreaOption= document.createElement("option");
			projectAreaOption.appendChild(document.createTextNode(projectAreaDesc.projectArea.name.value));
			projectAreaOption.setAttribute("value", projectAreaDesc.projectArea.name.value);
			
			if (projectAreaDesc.projectArea.name.value == this._context.projectArea) {
				projectAreaOption.setAttribute("selected", "selected");
			}
			this._projectAreaSelection.appendChild(projectAreaOption);

		}));
	},

	__sentinel: null // terminates this class definition
});

com.ibm.rqm.buildintegration.web.ui.internal.config.BuildPageProjectArea.create= function(context) {
	var args= com.ibm.rqm.buildintegration.web.ui.internal.wizard.WizardPage.constructorArgs(BuildUiUtil.getMessage("labelProjectArea"), BuildUiUtil.getMessage("labelSelectProjectArea"));
	dojo.mixin(args, {
		_context: context
	});
	return new com.ibm.rqm.buildintegration.web.ui.internal.config.BuildPageProjectArea(args, null);
};

})();
