/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.model.SaveContext");
dojo.require("com.ibm.rqm.planning.web.client.internal.DocumentRestClient");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function() {
	
var DocumentRestClient		= com.ibm.rqm.planning.web.client.internal.DocumentRestClient;
var ServiceResponseHandler	= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var Constants     			= com.ibm.rqm.planning.web.client.internal.Constants;

dojo.declare("com.ibm.rqm.planning.web.ui.internal.model.SaveContext", null, {	
	
    constructor: function() {
        this.init();
    },
    
	_isInitialized: false,

	// initialize the save context
	init: function()
	{
		this._participants = new dojox.collections.ArrayList();
		// entries in the referencesNew array should have the form
		// [ sourceParticipant  referenceName  targetParticipant  ]
		this._referencesNew = new dojox.collections.ArrayList();
		// entries in the referencesRemoved array should have the form
		// [ sourceParticipant  referenceName  targetId  ]
		this._referencesRemoved = new dojox.collections.ArrayList();
		
		// entries in the linksAdded/linksRemoved array should have the form
		// [sourceParticipant linkType itemId]
		this._linksAdded = new dojox.collections.ArrayList();
		this._linksRemoved = new dojox.collections.ArrayList();
		
		this._transactionId = 1;
		this._isInitialized = true;
		this._onSaveFinished = null;
	},

    // destroy the save context.
    uninitialize: function()
    {
        this._participants.clear();
        this._referencesNew.clear();
        this._referencesRemoved.clear();
        this._linksAdded.clear();
        this._linksRemoved.clear();
        
        delete this._participants;
        delete this._referencesNew;
        delete this._referencesRemoved;
        delete this._linksAdded;
        delete this._linksRemoved;
    },
	
	// add a new save participant
	addParticipant: function(participant)
	{
		if (!this._isInitialized) return;
		if (!participant.isASaveParticipant)
		{
			throw new Error("SaveContext#addParticipant: object is not a SaveParticipant");
		}
		if (!this._participants.contains(participant))
		{
            participant.setParticipantId(this._participants.count); //give participant a unique sequential id
			this._participants.add(participant);
		}
	},

	// find the participant with the provided id (itemId or participantId are valid params)
	findParticipant: function(/*itemId || participantId*/ id)
	{
		if (id == null) return null;
		var iter = this._participants.getIterator();
		while (!iter.atEnd())
		{
			var participant = iter.get();
			if (participant.getItemId() == id || participant.getParticipantId() == id)
			{
				return participant;
			}
		}
		return null;
	},

	// remove a save participant
	removeParticipant: function(participant)
	{
		if (!this._isInitialized) return;
		this._participants.remove(participant);
	},
	
	// create a reference between two save participants
	createReference: function(sourceParticipant, referenceName, targetParticipant)
	{
		if (!this._isInitialized) return;
		if (!sourceParticipant.isASaveParticipant || !targetParticipant.isASaveParticipant)
		{
			throw new Error("SaveContext#createReference: object is not a SaveParticipant");
		}
		
		 // check for duplicate entry
		var iter = this._referencesNew.getIterator();
		while (!iter.atEnd()) {
			var ref = iter.get();
			if (ref[0]==sourceParticipant && ref[1]==referenceName && ref[2]==targetParticipant) {
				return;
			}
		}
		
		this._referencesNew.add([sourceParticipant,referenceName, targetParticipant]);
	},
	
	// remove an existing reference
	removeReference: function(sourceParticipant, referenceName, /*itemId*/ targetId)
	{
		if (!this._isInitialized) return;
		this._referencesRemoved.add([sourceParticipant, referenceName, targetId]);
	},
	
	// create a link from a save participant to an item
	createLink: function(sourceParticipant, linkType, itemId)
	{
		if (!this._isInitialized) return;
		if (!sourceParticipant.isASaveParticipant)
		{
			throw new Error("SaveContext#createLink: object is not a SaveParticipant");
		}
		
		 // check for duplicate entry
		var iter = this._linksAdded.getIterator();
		while (!iter.atEnd()) {
			var ref = iter.get();
			if (ref[0]==sourceParticipant && ref[1]==linkType && ref[2]==itemId) {
				return;
			}
		}
		
		this._linksAdded.add([sourceParticipant,linkType, itemId]);
	},
	
	// remove an existing link
	removeLink: function(sourceParticipant, linkType, /*itemId*/ targetId)
	{
		if (!this._isInitialized) return;
		this._linksRemoved.add([sourceParticipant, linkType, targetId]);
	},

	// unremove a reference, i.e. clear the reference removal from
	// the list of pending changes
	unremoveReference: function(sourceParticipant, referenceName, /*itemId*/ targetId)
	{
		if (!this._isInitialized) return null;
		var iter = this._referencesRemoved.getIterator();
		var refToClear = null;
		while (!iter.atEnd())
		{
			var ref = iter.get();
			var source = ref[0];
			var refName = ref[1];
			var target = ref[2];
			if (sourceParticipant==source && refName==referenceName && targetId==target)
			{
				// don't remove while iterator is still active
				refToClear = ref;
				break;
			}
		}
		if (refToClear!=null)
		{
			this._referencesRemoved.remove(refToClear);
		}
	},
		
	// uncreate a reference, i.e. clear the reference creation from
	// the list of pending changes
	uncreateReference: function(sourceParticipant, referenceName, /*itemId || participantId*/ targetId)
	{
		if (!this._isInitialized) return null;
		var iter = this._referencesNew.getIterator();
		var refToClear = null;
		while (!iter.atEnd())
		{
			var ref = iter.get();
			var source = ref[0];
			var refName = ref[1];
			var target = ref[2];
			if (sourceParticipant==source && refName==referenceName && (targetId==target.getItemId() || targetId==target.getParticipantId()))
			{
				// don't remove while iterator is still active
				refToClear = ref;
				break;
			}
		}
		if (refToClear!=null)
		{
			this._referencesNew.remove(refToClear);
		}
	},
	
	// unremove a link
	unremoveLink: function(sourceParticipant, linkType, /*itemId*/ targetId)
	{
		if (!this._isInitialized) return null;
		var iter = this._linksRemoved.getIterator();
		var linkToClear = null;
		while (!iter.atEnd())
		{
			var link = iter.get();
			var source = link[0];
			var type = link[1];
			var target = link[2];
			if (sourceParticipant==source && linkType==type && targetId==target)
			{
				// don't remove while iterator is still active
				linkToClear = link;
				break;
			}
		}
		if (linkToClear!=null)
		{
			this._linksRemoved.remove(linkToClear);
		}
	},
		
	// uncreate a link
	uncreateLink: function(sourceParticipant, linkType, /*itemId || participantId*/ targetId)
	{
		if (!this._isInitialized) return null;
		var iter = this._linksAdded.getIterator();
		var linkToClear = null;
		while (!iter.atEnd())
		{
			var link = iter.get();
			var source = link[0];
			var type = link[1];
			var target = link[2];
			if (sourceParticipant==source && linkType==type && targetId==target)
			{
				// don't remove while iterator is still active
				linkToClear = link;
				break;
			}
		}
		if (linkToClear!=null)
		{
			this._linksAdded.remove(linkToClear);
		}
	},
	
	setOnSaveFinished: function(/* Function */ onSaveFinished) {
		if (onSaveFinished != null && dojo.isFunction(onSaveFinished)) {
            this._onSaveFinished = onSaveFinished;		  	
		}
	},
	
	// two phase commit.  first phase allows all participants to 
	// validate.   second phase gets the attributes from all
	// participants and sends to the DocumentRestService
	commit: function()
	{
		if (!this._isInitialized)
		{
			throw new Error("SaveContext#commit: SaveContext is not initialized");
		}
		
		// commit phase 1, let all participants validate
		var iter = this._participants.getIterator();
		while (!iter.atEnd())
		{
			var participant = iter.get();
			if (participant.validate() != true)
			{
				return Constants.STATUS_FAILURE;
			}
		}
		
		// commit phase 2, gather the changes into a transaction
		// object and send to the server
		var transaction = new Object();
		transaction["ItemsCreate"] = new Array();
		transaction["ItemsUpdate"] = new Array();
		
		iter.reset();
		while (!iter.atEnd())
		{
			var participant = iter.get();
			if (!participant._isProcessing())
			{
				var item = this._processItem(participant);
				var itemStack = item["ItemId"] ? transaction["ItemsUpdate"] : transaction["ItemsCreate"];
				var details = item["ItemDetails"];
				// only save the item if it is dirty or has
				// a new or removed reference or link
				if (participant.isDirty() 
				  || details["Links"]!=null
				  || details["ReferencesNew"].length>0
				  || details["ReferencesNewWithCreate"].length>0
				  || details["ReferencesRemoved"].length>0 )
				{
					itemStack.push(item);
				}
			}
		}
		
		//ensure that there are some elements for the transaction, otherwise save is not needed
		if (!transaction || (transaction["ItemsCreate"].length == 0 && transaction["ItemsUpdate"].length == 0)) {
			//reset all participants' _processing attr
			iter.reset();
			while (!iter.atEnd()) {
				var participant = iter.get();
				participant._setProcessing(false);
			}		
			 
			//notify caller that no update is necessary
			return Constants.STATUS_NOT_NECESSARY;
		}
		
		// unset all the "processing" flags
		iter.reset()
		while (!iter.atEnd())
		{
			var participant = iter.get();
			participant._setProcessing(false);
		}
		
		// send to the server
		var responseHandler = new ServiceResponseHandler(this, "_documentSaved", "_error");
		DocumentRestClient.postDocument(responseHandler, transaction);
		
		return Constants.STATUS_SAVING;
	},
	
	// create the JSON for a new or modified item
	_processItem: function(participant)
	{
		var item = new Object();
		item["ItemDetails"] = new Object(); 
		item["ItemDetails"]["Links"] = participant.getLinks();

		var attributes = new Object();
		if (participant.isDirty())
		{
			 attributes = participant.getAttributes();
		}
		item["ItemDetails"]["Attributes"] = attributes;
		
		// type
		var itemType = participant.getItemType() || Constants.MODEL_TYPE_UNKNOWN;
		var tmp = itemType.split(".");
		var itemTypeName = tmp.pop();
		var packageURI = tmp.join(".");
		item["ItemType"] = {PackageURI: packageURI, ItemTypeName: itemTypeName};
		
		// create a transaction id
		var itemId = participant.getItemId();		
		
		var transactionId = participant._getTransactionId();
		if (transactionId == null) {
			transactionId = this._transactionId++;
			participant._setTransactionId(transactionId);
		}
		item["TransactionId"] = "" + transactionId;
		
		if (itemId != null) {
			item["ItemId"] = itemId;
		}
		
		//append the sateId of the item, if available
		var stateId = participant.getStateId();
		if (stateId) {
			item["StateId"] = stateId;
		}
		
		// new references
		var referencesNew = new Array();
		item["ItemDetails"]["ReferencesNew"] = referencesNew;
		var referencesNewWithCreate = new Array();
		item["ItemDetails"]["ReferencesNewWithCreate"] = referencesNewWithCreate;
		
		var references = this._getReferencesNewFor(participant);
		for (var i=0; i<references.length; i++)
		{
			var reference = references[i];
			var sourceParticipant = reference[0];
			var referenceName     = reference[1];
			var targetParticipant = reference[2];
			
			var newReference = new Object();
			newReference["ReferenceName"] = referenceName;
			
			// item needs to be created
			if (targetParticipant.getItemId() == null)
			{
				newReference["ItemsCreate"] =  [ this._processItem(targetParticipant) ] ;
				referencesNewWithCreate.push(newReference);
			}
			// item already exists
			else
			{
				newReference["ItemId"] = targetParticipant.getItemId();
				referencesNew.push(newReference);
			} 
		}
		
		// removed references
		var referencesRemoved = new Array();
		item["ItemDetails"]["ReferencesRemoved"] = referencesRemoved;
		references = this._getReferencesRemovedFor(participant);
		for (var i=0; i<references.length; i++)
		{
			var reference = references[i];
			var newReference = new Object();
			newReference["ReferenceName"] = reference[1];
			newReference["ItemId"] = reference[2];
			referencesRemoved.push(newReference);
		}
		
		// added links
		var linksNew = new Array();
		item["ItemDetails"]["LinksNew"] = linksNew;
		
		var links = this._getLinksNewFor(participant);
		for (var i=0; i<links.length; i++)
		{
			var link = links[i];
			var sourceParticipant = link[0];
			var linkType     = link[1];
			var itemId = link[2];
			
			var newLink = new Object();
			newLink["LinkType"] = linkType;
			
			// TODO: handle link to new item
			newLink["ItemId"] = itemId;
			linksNew.push(newLink);
		}
		
		// removed links
		var linksRemoved = new Array();
		item["ItemDetails"]["LinksRemoved"] = linksRemoved;
		links = this._getLinksRemovedFor(participant);
		for (var i=0; i<links.length; i++)
		{
			var link = links[i];
			var newLink = new Object();
			newLink["LinkType"] = link[1];
			newLink["ItemId"] = link[2];
			linksRemoved.push(newLink);
		}
		
		participant._setProcessing(true);
		return item;
	},
	
	_getReferencesNewFor: function(participant)
	{
		var references = new Array();
		var iter = this._referencesNew.getIterator();
		while (!iter.atEnd())
		{
			var candidate = iter.get();
			var sourceParticipant = candidate[0];
			var targetParticipant = candidate[2];
			if (sourceParticipant==participant && targetParticipant!=null)
			{
				references.push(candidate);
			}
		}
		return references;
	},
	
	_getReferencesRemovedFor: function(participant)
	{
		var references = new Array();
		var iter = this._referencesRemoved.getIterator();
		while (!iter.atEnd())
		{
			var candidate = iter.get();
			var sourceParticipant = candidate[0];
			if (sourceParticipant==participant)
			{
				references.push(candidate);
			}
		}
		return references;
	},
	
	_getLinksNewFor: function(participant)
	{
		var links = new Array();
		var iter = this._linksAdded.getIterator();
		while (!iter.atEnd())
		{
			var candidate = iter.get();
			var sourceParticipant = candidate[0];
			if (sourceParticipant==participant)
			{
				links.push(candidate);
			}
		}
		return links;
	},
	
	_getLinksRemovedFor: function(participant)
	{
		var links = new Array();
		var iter = this._linksRemoved.getIterator();
		while (!iter.atEnd())
		{
			var candidate = iter.get();
			var sourceParticipant = candidate[0];
			if (sourceParticipant==participant)
			{
				links.push(candidate);
			}
		}
		return links;
	},
	
	// called back from DocumentRestClient
	_documentSaved: function(status)
	{
		//var code = status.code;
		var exception = status.exception;
		//var message = status.message;
		// var multiStatus = status.multiStatus;
		var ok = status.ok
		var children = status.children;
		
		if (ok != true) {
			this._error(status);

		} else {
			if (children) {
				// give each child its saved document
				for (var i = 0; i < children.length; i++) {
					var child = children[i];
					var id = child.transactionId;
					if (id == -1) {
						id = child.itemId;
					}
					var participant = this._findParticipant(id);
					if (participant == null) {
						throw new Error("SaveContext#_documentSaved: document save service returned an unknown itemId");
					}
					participant.saveFinished(child.resultDTO);
					participant._setTransactionId(null);
				}
			}
			// clear out all the added and removed references
			// since they have now been saved
			this._referencesNew.clear();
			this._referencesRemoved.clear();
			this._linksAdded.clear();
			this._linksRemoved.clear();
		}	
		
		//if a post-save callback has been registered, invoke it
        if (this._onSaveFinished != null && dojo.isFunction(this._onSaveFinished)) {
            this._onSaveFinished(status);            
        }		
	},
	
	_findParticipant: function(id)
	{
		var iter = this._participants.getIterator();
		while (!iter.atEnd())
		{
			var participant = iter.get();
			if (id == participant.getItemId() || id == participant._getTransactionId() || id == participant.getVersionableItemId())
			{
				return participant;
			}
		}
		return null;
	},
		
	_findChild: function(children, id)
	{
		if (id==null||children==null) return null;
		for (var i=0; i<children.length; i++)
		{
			var child = children[i];
			if (child.itemId==id || child.transactionId==id)
			{
				return child;
			}
		}
		return null;
	},

	_notifyFailure: function(status)
	{
		var iter = this._participants.getIterator();
		if (status.multiStatus == true)
		{
			while (!iter.atEnd())
			{
				var participant = iter.get();
				var id = participant.getItemId();
				if (id == null) {
					id = participant._getTransactionId();
				}
				var childStatus = this._findChild(status.children, id);
				participant.saveFailed(childStatus);
			}
		}
		else
		{
			while (!iter.atEnd())
			{
				var participant = iter.get();
				participant.saveFailed(status);
			}
		}
	},
	
	_error: function(e)
	{
		if (e.multiStatus!=true)
		{
			//alert("Error: [" + e.message + "] @ SaveContext#_error");
		}
		this._notifyFailure(e);
	}
	
});

})();
