/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.execution.web.ui.internal.page.execution.ExecutionPageView");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

// Require the controller widget
dojo.require("com.ibm.rqm.execution.web.ui.internal.page.execution.ExecutionPageController");

// Require the repository widget needed to show the two column layout
dojo.require("net.jazz.ajax.ui.SideNavigationLayout");
dojo.require("net.jazz.ajax.ui.PlatformUI");

// Utilities
dojo.require("com.ibm.rqm.execution.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.execution.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
//Require local widgets
dojo.require("com.ibm.rqm.execution.web.ui.internal.view.execution.ExecutionLogBody");
dojo.require("com.ibm.rqm.execution.web.ui.internal.view.execution.CreateExecutionWorkItem");
dojo.require("com.ibm.rqm.execution.web.ui.internal.view.execution.ExecutionPageBody");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.ListNavigator");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.planning.SearchNavWidget");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.NavSectionGroup");
dojo.require("com.ibm.rqm.execution.web.ui.internal.view.execution.ResultHistory");
dojo.require("com.ibm.rqm.execution.web.ui.internal.view.execution.EWIList");
// Require local clients

dojo.require("com.ibm.rqm.execution.web.ui.internal.view.table.ExecutionLogTableView");
dojo.require("com.ibm.rqm.execution.web.ui.internal.view.table.ExecutionResultTableView");

(function() {

	var PlatformUI						= net.jazz.ajax.ui.PlatformUI;
	var ActionRegistry					= null; // initialize later
	var Utility							= com.ibm.rqm.execution.web.ui.internal.util.Utility;
	var ImageCache						= com.ibm.rqm.execution.web.ui.internal.view.common.ImageCache;
    var Constants                       = com.ibm.rqm.planning.web.client.internal.Constants;
	var ExecutionResultTableView     = com.ibm.rqm.execution.web.ui.internal.view.table.ExecutionResultTableView;
	var ExecutionLogTableView        = com.ibm.rqm.execution.web.ui.internal.view.table.ExecutionLogTableView;
	
	dojo.declare("com.ibm.rqm.execution.web.ui.internal.page.execution.ExecutionPageView",[dijit._Widget, dijit._Templated],    
	{      
		controller: null,      
		templatePath: dojo.moduleUrl("com.ibm.rqm.execution.web", "ui/internal/page/templates/execution/ExecutionPageView.html"),
		// initialization      
		_currentQuery: null,
			
		postCreate: function(){	
			this.controller = new com.ibm.rqm.execution.web.ui.internal.page.execution.ExecutionPageController(this);
			this._mainView = new net.jazz.ajax.ui.SideNavigationLayout();
			this._pageContainer.appendChild(this._mainView.domNode);

			this._initializeNavigation( { parentController: this.controller } );			
			
			// lazily initialize the ActionRegistry alias
			ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
						
			// register actions			
			var actions = [ /* "newEwi",*/ "viewEwi", "viewEwis", "startExecution", "viewResult", "viewResults", "viewExecutedItems", "viewEWIResults"];
			var len = actions.length;
			for (var i = 0; i < len; i++) {
		        ActionRegistry.registerAction("com.ibm.rqm.execution.execution." + actions[i], this.controller, actions[i]);
			}
			var self = this;
		},
		
		_initializeNavigation: function(args){
			this._searchNavWidget = new com.ibm.rqm.planning.web.ui.internal.view.planning.SearchNavWidget(args);
			this._mainView.addNavigationSection(this._searchNavWidget.domNode, "Search");
			args.bold = false;	
			this._createEWIViewWidget = new com.ibm.rqm.planning.web.ui.internal.view.common.ListNavigator(args);	
			this._createEWIViewWidget.addEntry({ name: Constants.getMessage("labelAllEWI"), href: Utility.getEwisURI(Constants.getMessage("labelAllEWI"), Utility.EXECUTION_PERSPECTIVE)});
			this._createEWIViewWidget.addEntry({ name: Constants.getMessage("labelExeItem"), href: Utility.executedItemsURI(Constants.getMessage("labelExeItem"))});
			this._createEWIViewWidget.addEntry({ name: Constants.getMessage("labelExeLog"), href: Utility.showResultsURI(Constants.getMessage("labelExeLog"))});
			this._createEWIViewWidget.addEntry({ name: Constants.getMessage("labelExeTest"), href: Utility.getEwisURI(Constants.getMessage("labelExeTest"), Utility.EXECUTION_PERSPECTIVE)});			
			
			var EWIsections= new Array();
			EWIsections.push(this._createEWIViewWidget);
			this._createEWIWidget = new com.ibm.rqm.planning.web.ui.internal.view.common.NavSectionGroup({label: Constants.getMessage("labelEWIView"), sectionWidgets: EWIsections, open: true, flagStatus: 3});
			this._mainView.addNavigationSection(this._createEWIWidget.domNode, "");
		},
		
		newEWI: function(viewName){
			var _createEWI =new com.ibm.rqm.execution.web.ui.internal.view.execution.CreateExecutionWorkItem({ parentController: this.controller });
			this._mainView.setMainContent(_createEWI.domNode);
		},
		
		showAllEWIs: function(ewis, args){
			var param = {
				parentController: this.controller,
				ewis: ewis
			}
			if(args.owner){
				param.owner = args.owner;
			}
			var _ewiContainer = new com.ibm.rqm.execution.web.ui.internal.view.execution.EWIList(param);
			this._mainView.setMainContent(_ewiContainer.domNode);
		},
				
		startExecution: function(kwArgs){
			var ewi = kwArgs.ewi;
			var runScript = kwArgs.runScript;
			var _resultId = null;
			if(kwArgs._resultId)
				_resultId =kwArgs._resultId; 
			var _manualExecPage =new com.ibm.rqm.execution.web.ui.internal.view.execution.ExecutionPageBody({ parentController: this.controller, ewi: ewi, runScript: runScript, _resultId: _resultId });
			this._mainView.setMainContent(_manualExecPage.domNode);
		},
		
		showAllResults: function(results, queryName) {
			this._currentQuery = queryName;
			this._logTableViewer = new ExecutionLogTableView({results: results, queryName: this._currentQuery});
			this._mainView.setMainContent(this._logTableViewer.domNode); 			
		},
		
		showResult: function(kwArgs){
			var args ={
				parentController: this.controller,
				ewiId: kwArgs.ewiId,
				resultItemId: kwArgs.resultItemId
			}
			if(kwArgs.editable)
				args.editable = kwArgs.editable;
			var _executionLog = new com.ibm.rqm.execution.web.ui.internal.view.execution.ExecutionLogBody(args);
			this._mainView.setMainContent(_executionLog.domNode);	
		},
		
		showExecutedEWIs: function(executedEWIs, queryName) {
	        //build current execution status objects for table to display
			var results = [];
			var len = executedEWIs.length;
			for (var i = 0; i < len; i++) {
				var ewi = executedEWIs[i];
	            var result = ewi.currentResult;
	            if (result != null) {
	               result.ewi = {
	                  id: ewi.id,
	                  itemId: ewi.itemId,
	                  name: ewi.name,
	                  owner: ewi.owner,
	                  testCase: {id: ewi.testCase.id,name: ewi.testCase.name}
	               }
	               results.push(result);
	            }
	            
	         }
	         this._currentQuery = queryName;
	         this._resultTableViewer = new ExecutionResultTableView({results: results, queryName: this._currentQuery});
	         this._mainView.setMainContent(this._resultTableViewer.domNode);                     
		},

		showEWIResults: function(args){
			var _resultHistory = new com.ibm.rqm.execution.web.ui.internal.view.execution.ResultHistory({ parentController: this.controller, ewiItemId: args.ewiItemId });
			this._mainView.setMainContent(_resultHistory.domNode);
		},

		_error: function(e){
			alert("An error occurred: [" + e.message + "] @ ExecutionPageView#_error");
		}		
		
	});
})();
