/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 
dojo.provide("com.ibm.asq.common.web.ui.internal.util.CSHelp");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");

dojo.declare("com.ibm.asq.common.web.ui.internal.util._CSHelpEngine", [dijit._Widget, dijit._Templated],
{
    //
    // Summary:
    //   The CS help is in a self-contained UA WAR file that is hosted in the same server
    //   with the application.  A _CSHelpEngine is associated with a UA web app that has a
    //   specific URL.  There can be more than one URL for the CS help within the UA WAR file.
    //   The different URL for the CS help is specified with a provider id so that the CS
    //   help contents for different module can be self contained within the UA WAR file.
    //   The _CSHelpEngine is consumed by the CSHelp widget with a specific context id and
    //   optional provider id for the CS help URL. The _CSHelpEngine retrieves the content of
    //   a CS help based on the conext id and renders it in a floating CS help panel (DIV).
    //   The CS help panel that will be closed by mouse click, key press, or losing focus.
    //
    //   The bottom of the CS help panel always contains a link to the main info help center.
    //
    //   In the usual case there is no need to have more than one _CSHelpEngine i.e. having
    //   more than one UA web app.  A singleton "CSHelpEngine" is created for all the CSHelp
    //   widgets to consume with a default URL for CS help.  The only call to "CSHelpEngine"
    //   is to initialize the URL of the UA web app for CS help help and main info center.
    //
    //   In the case that more than one URL for CS help is needed, simply register a CS help
    //   provider with an id and URL.  CSHelp widget with the optional parameter of provider
    //   id will trigger the _CSHelpEngine to use the corresponding URL to fetch the content
    //   instead of the default CS help URL.
    //

    // Initialize the CSHelpEngine.
    //
    // cshelpURL: default URL for the CS help, for example "/help/rtopic/com.ibm.rational.test.lm.cshelp/topics/"
    // infoCenterLabel: label for the main info center
    // infoCenterHref: URL for the main info center
    //    
    init: function(cshelpURL, infoCenterLabel, infoCenterHref) {
        this._cshelpProvider["_DEFAULT_"] = cshelpURL;
        this.csHelpInfoCtrLabel.innerHTML = infoCenterLabel;
        this.csHelpInfoCtr.href = infoCenterHref;
    },
    
    registerCSHelpProvider: function(id, url) {
        this._cshelpProvider[id] = url;
    },
    
    templateString: 
    '<div dojoAttachPoint="domNode,cshelpPanel" class="com-ibm-asq-common-web-ui-csHelp-panel" align="center"'+
    '     style="z-index:30000 !important; position:absolute; display:none;" tabindex="-1">' +
    '  <div align="right" tabindex="-1">' +
    '    <div class="csHelp-close-image" waiRole="button" tabindex="30000"' +
    '         dojoAttachEvent="onclick:hideHelp,onkeyup:onKeyUpOnClose,onfocus:onFocus,onblur:onBlur">' +
    '      <span class="csHelp-close-text" dojoAttachPoint="csHelpCloseText" tabindex="-1">X</span>' +
    '    </div>' +
    '  </div>' +
    '  <div class="csHelp-content" dojoAttachPoint="cshelpContent" waiRole="region" align="left" tabindex="30001"' +
    '          dojoAttachEvent="onkeypress:onKeyPressInContent,onfocus:onFocus,onblur:onBlur">' +
    '  </div>' +
    '  <div align="left" tabindex="-1">' +
    '    <a dojoAttachPoint="csHelpInfoCtr" class="csHelp-info-link" tabindex="30100" target="_blank"' +
    '          dojoAttachEvent="onkeypress:onKeyPressInContent,onfocus:onFocus,onblur:onBlur,onclick:onLaunchExt">' +
    '      <span dojoAttachPoint="csHelpInfoCtrLabel" tabindex="-1"></span>' +
    '    </a>' +
    '  </div>' +
    '</div>',
    
    _cshelpProvider: {}, // URL's of provider for CS help
    
    postCreate: function() {
        com.ibm.asq.common.web.ui.internal.util._CSHelpEngine.superclass.postCreate.apply(this, arguments);

        // add the hidden CS help panel DIV to the document body
        dojo.body().appendChild(this.domNode);
        
        // the domNode must have an id for bgIframe
        if (!this.domNode.id) {
            this.domNode.id = "com-ibm-asq-common-web-CS-help-panel";
        }
        
        // create the backing iframe to work around the z-index support on IE6
        this.bgIframe = new dijit.BackgroundIframe(this.domNode);
        
        this._cshelpProvider = {};
    },

    // Return the URL for the CS help provider with the specified id.  If the id is not
    // specified or the id is not in the provider list, the default URL will be returned.
    //
    getCSHelpURL: function(id) {
        var url = null;
        if (id) {
            url = this._cshelpProvider[id];
        }
        if (!url) {
            url = this._cshelpProvider["_DEFAULT_"];
        }
        return url;
    },

    // Key event handler for the "X" close icon - close if it is the space bar,
    // the Enter key, or the Escape key.
    //
    onKeyUpOnClose: function(event)
    {
        var charCode = (event.which) ? event.which : event.keyCode
        if ((charCode == 32) || (charCode == 13) || (charCode == 27))
            this.hideHelp();
    },
    
    // Key event handler for the content div - close if it is the Escape key.
    //
    onKeyPressInContent: function(event)
    {
        var charCode = (event.which) ? event.which : event.keyCode
        if (charCode == 27)
            this.hideHelp();
    },
    
    // The button of the CSHelp widget that brings up the help panel.
    callerButton: null,
    
    // Restoring the focus to the help button should be skipped if it is launching an external
    // window from links within the help content.  The window focus event handler will take
    // care of it.  Using this boolean to determine if we should restore focus on closing
    // the help panel.
    _launchExt: false,

    windowOnFocus: function(event)
    {
        if (this.windowFocusHandle) {
            this.disconnect(this.windowFocusHandle);
            delete this.windowFocusHandle;
            this.windowFocusHandle = null;
        }

        this._launchExt = false;
        if (this.callerButton && (this.cshelpPanel.style.display == "none")) {
            // don't bother focus on the caller button if the button is
            // no longer displayed on the screen
            var parent = this.callerButton.parentNode;
            while (parent) {
                if (parent.style && ((parent.style.display == "none") || (parent.style.visibility == "hidden"))) {
                    this.callerButton = null;
                    return;
                }
                parent = parent.parentNode;
            }
            // stop the event to prevent focus of the button implicitly triggers the window focus
            dojo.stopEvent(event);
            // clear the caller button to ensure no recursive call from the window getting focus
            // that is triggered from setting the caller button in focus
            var temp = this.callerButton;
            this.callerButton = null;
            temp.focus();
        }
    },
    
    // Losing focus of a DOM object in the CS help panel can be from switching focus to
    // another object outside of the CS help panel or from switching focus to another
    // DOM object within the same CS help panel.  In order to support closing the CS help
    // panel from clicking outside of the panel, we need to determine if it is switching
    // focus within the same CS help panel.  Using the _focusCount and a delay to try to
    // hide the panel we can check if other DOM object within the same CS help panel has
    // gain focus and ignore the blur event.
    //
    onBlur: function(event)
    {
        this._focusCount--;

        if (this._focusCount == 0) {
            setTimeout(dojo.hitch(this, "tryHideHelp"), 200);
        }
    },
    
    onLaunchExt: function()
    {
        this._launchExt = true;
        return true;
    },
    
    // Add a count of focused DOM object.
    onFocus: function(event)
    {
        this._focusCount++;
    },
    
    // Keep track of focused DOM object count to determine if it should close the CS help panel.
    _focusCount: 0,
    
    // Flag indicating starting to show the help panel so the delayed closing from the blur
    // event can be ignored.  It is needed when the user click on another CSHelp button on
    // the same page while the CS help panel is already up.
    _startShowing: false,
    
    // Try to hide the context sensitive help panel is no DOM object within the same
    // CS help panel is in focus.
    //
    tryHideHelp: function() 
    {
        if (this._focusCount == 0) {
            this.hideHelp();
        }
    },

    // Hide the context sensitive help panel.
    //
    hideHelp: function()
    {
        if (!this._startShowing) {
            // there may be blur event from clicking another CS help button so
            // do nothing if it is starting to show the panel

            // clear previous links connected handles
            if (this.linkHandles) {
                var previous_length = this.linkHandles.length;
                for (var i = 0; i < previous_length; i++) {
                    var handles = this.linkHandles[i];
                    this.disconnect(handles.focusHandle);
                    this.disconnect(handles.blurHandle);
                    this.disconnect(handles.keypressHandle);
                    this.disconnect(handles.clickHandle);
                }
                delete this.linkHandles;
                this.linkHandles = null;
            }
            
            if (this.cshelpPanel) 
            {
                // clear the help panel, just DOM no widget
                this.cshelpContent.innerHTML = "";
                this.cshelpPanel.style.display = 'none';
            }
            if (this.callerButton) 
            {
                // restore the focus on the button which bring up the help
                // only if the current window is in focus
                if (!this._launchExt) {
                    this.callerButton.focus();
                    this.callerButton = null;

                    if (this.windowFocusHandle) {
                        this.disconnect(this.windowFocusHandle);
                        delete this.windowFocusHandle;
                        this.windowFocusHandle = null;
                    }
                }
            }
            this._focusCount = 0;
        }
    },
    
    _formatLocale: function(locale) {
        var langAndCountry = locale.split('-');
        var lang = langAndCountry[0];
        if (langAndCountry.length > 1) {
            var country = langAndCountry[1];
            return lang + "-" + country.toUpperCase();
        } else {
            return locale;
        }
    },
    
    // Show the context sensitive help content in a predefined help panel DIV.
    //
    showHelpContent: function(helpButton, contextId, providerId) 
    {
        var csHelpURL = this.getCSHelpURL(providerId);
        if (!csHelpURL) {
            throw new Error(MessageUtil.getMessage(MessageBundle.COMMON, "csHelp_engineNotInit"));
        }
        
        // indicate starting to show the help panel
        this._startShowing = true;
        this._focusCount = 0;
        
        // save the button which bring up the help so
        // we can restore to focus when we are done
        this.callerButton = helpButton;
            
        if ((this.cshelpPanel != null) && (this.cshelpContent != null)) 
        {
            // clear the help panel
            this.cshelpContent.innerHTML = "";
            this.cshelpContent.style.height = "";

            // get the content from the server
            dojo.xhrGet({
                url : csHelpURL + contextId + "?lang=" + this._formatLocale(dojo.locale),
                    handleAs: "text",
                    timeout: 5000, // timeout in milliseconds
                    parentCSHelp: this,
        
                    // Set the help content with the response and display it.
                    load: function(response, ioArgs) {
                        this.parentCSHelp.showHelpPopup(this.parentCSHelp.rel2absImgHack(providerId, response));
                        this.parentCSHelp._startShowing = false;
                        return response;
                    },
        
                    // The ERROR function will be called in an error case.
                    error: function(response, ioArgs) {
                        var topic = ioArgs.url.split("/").pop();
                        this.parentCSHelp.showHelpPopup(MessageUtil.getMessage(MessageBundle.COMMON, "csHelp_failedGetContent", topic));
                        console.error("HTTP status code: ", ioArgs.xhr.status);
                        this.parentCSHelp._startShowing = false;
                        return response;
                    }
                });
        } else {
            this._startShowing = false;
        }
    },

    // convert relative path img tag absolute path
    rel2absImgHack: function(providerId, cshelpContent) {
        var csHelpURL = this.getCSHelpURL(providerId);
        if (cshelpContent.search(/src="..\/sout.gif"/) != -1) {
            cshelpContent = cshelpContent.replace(/src="..\/sout.gif"/g, 'src="' + csHelpURL + '../sout.gif"');
        }             
        return cshelpContent;
    },

    // Position and size the help panel to display the help content.
    //
    showHelpPopup: function(cshelpContent) 
    {
        // Get the position of the Help button
        var buttonXY = dojo.coords(this.callerButton, true);
        var helpbutton_x = buttonXY.x;
        var helpbutton_y = buttonXY.y;
    
        // Set the position of the help panel
        this.cshelpPanel.style.left = (helpbutton_x + 20) + 'px';
        this.cshelpPanel.style.top = (helpbutton_y + 20) + 'px';
        
        // Get the width and height of the visible document
        var docWidth = document.body.clientWidth;
        var docHeight = document.body.clientHeight;
        
        // width is default to 1/3 of the document width
        this.cshelpPanel.style.width = Math.round(docWidth / 3) + 'px';
    
        // Panel height is driven by the content and the max is 2/3 of the document height
        var maxHeight = Math.round(docHeight * 2 / 3);
        
        // hide the help panel before setting to display in order
        // to gather size and position of the panel
        this.cshelpPanel.style.visibility = 'hidden';
        this.cshelpPanel.style.display = 'block';
    
        this.cshelpContent.innerHTML += cshelpContent;
    
        // get the panel height after setting the content
        var panelHeight = this.cshelpPanel.clientHeight;
        var panelWidth = this.cshelpPanel.clientWidth;
        
        // resize if it is more than max...  pad 100 pixels for
        // buffer of adding scroll bar 
        if (panelHeight > (maxHeight + 100)) {
            // set the content height so the scroll bar is within the content
            // and not the outer panel
            this.cshelpContent.style.height = maxHeight + 'px';
            // update the panel height
            panelHeight = this.cshelpPanel.clientHeight;
        }
            
        // determine whether the help panel is out of the scope of the document
        if ((helpbutton_x + panelWidth + 20) > docWidth) 
        {
            this.cshelpPanel.style.left = (docWidth - panelWidth - 20) + 'px';
        }
        if ((helpbutton_y + panelHeight + 20) > docHeight) 
        {
            // make sure if the must be clipping, clipping is at the bottom to show the close button
            this.cshelpPanel.style.top = Math.max(0, (docHeight - panelHeight - 20)) + 'px';
        }

        // loop through all the links in the help content to set tab index and event handlers
        var allLinks = this.cshelpContent.getElementsByTagName("a");
        
        // the close icon has index 30000, the content DIV has index 30001, and
        // the static link to the help info center has index 30100
        var index = 30002;
        var links_length = allLinks ? allLinks.length : 0;
        
        this.linkHandles = new Array(links_length);
        for (var i = 0; i < links_length; i++) {
            allLinks[i].tabIndex = index++;
            var aLink = allLinks[i];
            var handles = {};
            handles['focusHandle'] = this.connect(aLink, "onfocus", "onFocus");
            handles['blurHandle'] = this.connect(aLink, "onblur", "onBlur");
            handles['keypressHandle'] = this.connect(aLink, "onkeypress", "onKeyPressInContent");
            handles['clickHandle'] = this.connect(aLink, "onclick", "onLaunchExt");
            
            this.linkHandles[i] = handles;
            // target must be set on the links
            if (!aLink.target)
                aLink.target = "_blank";
        }

        // now we can show the panel
        this.cshelpPanel.style.visibility = 'visible';
        this.cshelpContent.focus();

        // We set the focus back to the CS help button on closing the CS help
        // panel.  There can be links in the help content which will launch an
        // external window and we don't want to set any focus in our app window
        // to bring it to the top.  We will just wait until our app window gain
        // focus again to set the focus to the button. 
        this.windowFocusHandle = this.connect(window, "onfocus", "windowOnFocus");
    }
});

// Create a singleton CSHelpEngine for all CSHelp widgets to consume.
CSHelpEngine = new com.ibm.asq.common.web.ui.internal.util._CSHelpEngine();

dojo.declare("com.ibm.asq.common.web.ui.internal.util.CSHelp", dijit.form.Button,
{
    //
    // Summary:
    //   The CSHelp widget is simply a subclass of Dojo button with a "?" icon and the
    //   click handler to bring up the CS help panel for the specified context id.
    //
     
    contextID: null,
    iconClass: "csHelp-icon",
    csHelpEngine: null,
    csHelpProviderId: null,
    "class": "com-ibm-asq-common-web-ui-csHelp",
    
    postCreate: function() {
        // set text to "?" for display only in high contrast
        this.containerNode.innerHTML = "?";
        if (!this.csHelpEngine) {
            this.csHelpEngine = CSHelpEngine;
        }
    },
    
	_onMouse : function(/*Event*/ event){
		com.ibm.asq.common.web.ui.internal.util.CSHelp.superclass._onMouse.apply(this, arguments);
		if(!this.disabled && event.type=="mouseover"){
			this.focus();
		}
	},
	
    onClick: function(evt) {
        if (!this.contextID) {
            throw new Error("No context id specified.");
        }
        this.csHelpEngine.showHelpContent(this.focusNode, this.contextID, this.csHelpProviderId);
    },
	
	setButtonLabel: function(label) {
        this.containerNode.innerHTML = label;
        
        // always set the title on the title node to have a readable label.
        this.titleNode.title = dojo.trim(this.containerNode.innerText || this.containerNode.textContent || '');
	}
});
