/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
var CategoryUtil = dojo.provide("com.ibm.rqm.planning.web.ui.internal.util.CategoryUtil");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentFilterProvider"); 
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
(function(){

var TableContentFilterProvider = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentFilterProvider;

    com.ibm.rqm.planning.web.ui.internal.util.CategoryUtil = {
		
		// Summary:
        //    A convenient util to compare two category filters. 
        //     
        isCategoryFiltersSame: function(filtersA, filtersB){
			if(filtersA.length != filtersB.length) {
				return false;
			}
			var old = this.flattenCategoryFilters(filtersA);
            var news = this.flattenCategoryFilters(filtersB);
			if(old.length != news.length) {
				return false;
			}
			if (old.join() != news.join()) {
				return false;
			}
			return true;
        },
    
        // Summary:
        //    A convenient util to compare two category filters. It picks categories' itemId in given category filter.
        //    The result is a sorted itemIds array.
        //     
        flattenCategoryFilters: function(filters){
            var len = filters ? filters.length : 0;
            var newArray = [];
            for (var i = 0; i < len; i++) {
                var optionLen = filters[i].params.options.length;
                for (var j = 0; j < optionLen; j++) {
                    newArray.push(filters[i].params.options[j].value);
                }
            }
            newArray = newArray.sort();
            return newArray;
        },
        
        // Summary:
        //    A method to construct category filters. 
        //     
        getCategoryTypesFilters: function(categoryTypes){
            var filters = [];
            // initialize filters for all the categoryType
            if (dojo.isArray(categoryTypes) && (categoryTypes.length > 0)) {
                for (var i = 0; i < categoryTypes.length; i++) {
                    // proceed only if there is actual unarchived categories
                    if (!categoryTypes[i].archived && dojo.isArray(categoryTypes[i].categories) && (categoryTypes[i].categories.length > 0)) {
                        var categories = [];
                        for (var j = 0; j < categoryTypes[i].categories.length; j++) {
                            if (!categoryTypes[i].categories[j].archived) {
                                categories.push({
                                    name: categoryTypes[i].categories[j].name,
                                    value: categoryTypes[i].categories[j].itemId,
                                    checked: false
                                });
                            }
                        }
                        
                        var title = categoryTypes[i].name;
                        var categoryFilter = {
                            name: "categories" + i, // each category type is a filter
                            title: title + com.ibm.rqm.planning.web.client.internal.Constants.getMessage("labelColen"),
                            promptTitle: title,
                            type: TableContentFilterProvider.FIELD_TOGGLELIST,
                            defaultValue: null,
                            params: {
                                options: categories
                            }
                        };
                        filters.push(categoryFilter);
                    }
                }
            }
            return filters;
        }
    };
})();
