/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.util.EditorRegistry");

dojo.require("jazz.core.loader");

dojo.require("com.ibm.team.repository.web.client.init.InitializationData");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function(){
    var InitializationData    = com.ibm.team.repository.web.client.init.InitializationData;
    var Constants             = com.ibm.rqm.planning.web.client.internal.Constants;
	
	dojo.declare("com.ibm.rqm.planning.web.ui.internal.util.EditorRegistry", null, {
		
	    constructor: function() {	
            this._allModules = [];
			this._loadedModules = [];	
			this._failedModules = [];
			this._initialize();			
	    },
		
        ////////////////////////////////////////////////////////////////////////////////////////////////////
        //                                             API                                                //
        ////////////////////////////////////////////////////////////////////////////////////////////////////        
        getEditor: function(editorWidget, /*optional*/directBindOnly) {
            return this._editorMap[editorWidget.declaredClass];
        },
		
        getEditorById: function(editorId) {
			for (var widgetClass in this._editorMap) {
				var editor = this._editorMap[widgetClass];
				if (editor.id == editorId) return editor;
			}
            return null;
        },		
        
        getSection: function(editorWidget, sectionId) {
            var editor = this._editorMap[editorWidget.declaredClass];
            if (editor && dojo.isArray(editor.sections)) {
                for (var i = 0; i < editor.sections.length; i++) {
                    if (editor.sections[i].id == sectionId) {
                        return editor.sections[i];
                    }
                }
            }
            return null;
        },

        ////////////////////////////////////////////////////////////////////////////////////////////////////
        //                                           INTERNALS                                            //
        ////////////////////////////////////////////////////////////////////////////////////////////////////        		
	    _initialize: function() {
            var planningData = InitializationData.get(Constants.REGISTRY_INIT_DATA_KEY);
            if (!planningData) {
                throw new Error("InitializationData missing required property: " + Constants.REGISTRY_INIT_DATA_KEY);
            }
            var registry = planningData ? planningData.editorRegistry : null;
            if (!registry) {
                throw new Error("InitializationData missing required property: editorRegistry");
            }
			
            this._registry = registry;
	        this._editorMap = {};
	        for (var i = 0; i < registry.length; i++) {
	            var editor = this._configureEditor(registry[i]);
	            this._editorMap[editor.jsClass] = editor;
	        }		
		},		
	    
		/*
		 * dynamically loads all contributed section widgets
		 */
		_loadModules: function() {
			//gather all modules references from the editor configuration
            this._allModules = [];
			for (var editorId in this._editorMap) {
				var editor = this._editorMap[editorId];
				var sections = editor && editor.sections ? editor.sections : [];				
				for (var i = 0; i < sections.length; i++) {
					if (sections[i].jsClass && !this._contains(this._allModules, sections[i].jsClass)) {
						this._allModules.push(sections[i].jsClass);
					}
				}
			}
			
			//load each module individually
			this._loadStarted = new Date().getTime();
            //for (var i = 0; i < this._allModules.length; i++) {
				//var module = this._allModules[i];
				var module = this._allModules;
	            var callback = dojo.hitch(this, "_moduleLoaded", module);
	            var errorCallback = dojo.hitch(this, "_moduleFailed", module);
				jazz.core.loader.batch_load_async(module, callback);
            //}	
		},
		
		_moduleLoaded: function(module) {
			console.info("Module Loading Succeeded...");
			//this._loadedModules.push(module);
			//this._loadCompleted();
		},
		
        _moduleFailed: function(err, module) {
			console.info("Module Loading Failed");
            //this._failedModules.push(module);
			//this._loadCompleted();
        },		
		
		_loadCompleted: function() {
			var all = this._allModules.length;
			var loaded = this._loadedModules.length; 
			var failed = this._failedModules.length;
			if (all == (loaded + failed)) {
	            var elapsed = new Date().getTime() - this._loadStarted;
				var message = this.declaredClass.substr(this.declaredClass.lastIndexOf(".") + 1) + "._loadCompleted: ";
				message += "Contributed module load Completed (in " + elapsed + "ms): ";
				message += "Total/Loaded/Failed: [" + all + "/" + loaded + "/" + failed + "]";
	            console.info(message);
			}
		},
	    
	    _configureEditor: function(editorDTO) {
	        //for each section, add id of the group to which it belongs
	        var sections = editorDTO.sections;
	        for (var i = 0; i < (sections ? sections.length : 0); i++) {
	            var group = this._getGroup(sections[i], editorDTO.sectionGroups);
	            if (group != null) {
	                sections[i].groupId = group.id;
	            }
	        }
	        editorDTO.sections = sections;
	        return editorDTO;
	    }, 
	    
	    _getGroup: function(section, groups) {
	        for (var i = 0; i < (groups ? groups.length : 0); i++) {
	            var sections = groups[i].sections;
	            for (var j = 0; j < (sections ? sections.length : 0); j++) {
	                if (sections[j] == section.id) {
	                    return groups[i];
	                }
	            }
	        }    
	        return null;
	    },
		
		_contains: function(/* String[] */values, /* String */ value) {
			if (dojo.isArray(values)) {
				for (var i = 0; i < values.length; i++) {
					if (values[i] == value) {
						return true;
					}
				}
			}
			return false;
		}	
	});

})();
