/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");

dojo.require("dojox.data.dom");
dojo.require("dijit.Tooltip");

dojo.require("com.ibm.asq.common.web.client.internal.option");

dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.rqm.planning.web.client.internal.UserClient");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.team.repository.web.client.init.InitializationData");

dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.rqm.planning.web.client.internal.RegistryClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.PreferenceClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.TooltipBuilder");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.util.SelectUtility");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");

(function() {
var ServiceResponseHandler 		= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var UserClient 					= com.ibm.rqm.planning.web.client.internal.UserClient;
var Utility						= com.ibm.rqm.planning.web.ui.internal.util.Utility;
var Constants    				= com.ibm.rqm.planning.web.client.internal.Constants;
var ImageCache    				= com.ibm.rqm.planning.web.ui.internal.view.common.ImageCache;
var TooltipBuilder				= com.ibm.rqm.planning.web.ui.internal.util.TooltipBuilder;
var RegistryClient				= com.ibm.rqm.planning.web.client.internal.RegistryClient;
var PreferenceClient            = com.ibm.rqm.planning.web.client.internal.PreferenceClient;

var isGuest                      = com.ibm.team.repository.web.client.session.isGuest;
var getAuthenticatedContributor  = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
var InitializationData           = com.ibm.team.repository.web.client.init.InitializationData;
var getCurrentProjectArea        = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
var SelectUtility			     = dojo.getObject( "com.ibm.asq.common.web.ui.internal.util.SelectUtility" );

var EXCLUDED_CONTRIBUTORS        = ["ADMIN", "GUEST"];

var ICON_PREFIX					= "/jazz/secure/service/com.ibm.team.workitem.common.internal.model.IImageContentService";
var ICON_STATE_NEW				= "/plugin/com.ibm.team.workitem.common/icons/obj16/state_open.gif";
var ICON_STATE_IN_PROGRESS		= "/plugin/com.ibm.team.workitem.common/icons/obj16/state_inprogress.gif";
var ICON_STATE_RESOLVED			= "/plugin/com.ibm.team.workitem.common/icons/obj16/state_resolved.gif";
var ICON_STATE_REOPENED			= "/plugin/com.ibm.team.workitem.common/icons/obj16/state_reopened.gif";

com.ibm.rqm.planning.web.ui.internal.util.EditorUtil = {
	
	getCurrentContributor: function() {
		return getAuthenticatedContributor();
	},

   getCurrentProjectArea: function() {
      return getCurrentProjectArea();
   },
	   
   isGuest: function() {
      return isGuest();
   },
	
	isAdmin: function() {
	   var permissionInitializer = InitializationData.get("com.ibm.team.repository.service.canGrantPermissionInitializer");
	   return (permissionInitializer != null && permissionInitializer.canGrantPermission);
	},	
	
	isProjectAreaAdmin: function() {
		return dojo.getObject(Constants.IS_PROJECT_AREA_ADMIN) == true;
	},
	
	initContributors: function(select, selectedItemId, fillUserId) {
		//get the list of contributors
		var self = this;
		var serviceObj = {
			success: function(contributors) {
				var filtered = self._filterContributors(contributors);
				self._doInitContributors(filtered, select, selectedItemId, fillUserId);
			},
			failure: function(e) {
				self._error(e);
			}
		};
		var responseHandler = new ServiceResponseHandler(serviceObj, "success", "failure");			
		UserClient.getContributors(responseHandler);			
	},
	
	_doInitContributors: function(contributors, select, selectedItemId, fillUserId) {			
		if (select) {
			dojox.data.dom.removeChildren(select);	
		}
        if (contributors.length > 0) {
            for (var i = 0; i < contributors.length; i++) {
                //Skip archived users that are NOT the selected user
                if (contributors[i].archived) {
                    if (!selectedItemId || selectedItemId != contributors[i].itemId) 
                        continue;
                }
				if(com.ibm.asq.common.web.ui.internal.util.Utility.isUnassignedUserId(contributors[i].itemId))
					contributors[i].name = Constants.getMessage("labelUnassigned");
                var option = new Option(contributors[i].name, fillUserId ? contributors[i].userId : contributors[i].itemId);
                option.disabled = contributors[i].archived; //disable archived users
                if (selectedItemId != null && selectedItemId == contributors[i].itemId) {
                    option.selected = true;
                }else if (selectedItemId == null && com.ibm.asq.common.web.ui.internal.util.Utility.isUnassignedUserId(contributors[i].itemId)) {
					option.selected = true;
                }
                SelectUtility.add(select, option);
            }
        } else {
			var UNASSIGNED_ITEMID = com.ibm.asq.common.web.ui.internal.util.Utility.getUnassignedUserId()
            var unassignedOption = new Option(Constants.getMessage("labelUnassigned"), UNASSIGNED_ITEMID);
            unassignedOption.selected = true;
            SelectUtility.add(select, unassignedOption);
        }
	},
	
    getContributors: function(callbackContext, successCallbackName, errorCallbackName) {
        //get the list of contributors
        //TODO implement some caching mechanism here
		var self = this;
        var serviceObj = {
            success: function(contributors) {
                var filtered = self._filterContributors(contributors);
                callbackContext[successCallbackName](filtered);
            },
            failure: function(e) {
                callbackContext.errorCallbackName(e);
            }
        };		
		var responseHandler = new ServiceResponseHandler(serviceObj, "success", "failure");         
        UserClient.getContributors(responseHandler);          
    },
	
	_filterContributors: function(contributors) {
		//for weeding out default system users
		var excluded = function(entry) {
			for (var i = 0; i < EXCLUDED_CONTRIBUTORS.length; i++) {
				if (EXCLUDED_CONTRIBUTORS[i] == entry.name) return true; 
			}
			return false;
		}
		//return only those users not in the 'excluded' list
		var filtered = [];
        for ( var i = 0; i < contributors.length; i++ ) {
            if (!excluded(contributors[i])) {
				filtered.push(contributors[i]);
			}
        }
		filtered = filtered.sort(function(c1, c2) {
            var cSort1 = c1.name == Constants.getMessage("labelUnassigned") ? "0" : c1.name;
            var cSort2 = c2.name == Constants.getMessage("labelUnassigned") ? "0" : c2.name;
            return cSort1 < cSort2 ? -1 : 1;
        });
		return filtered;
	},
		
	/*
	 * BIG NOTE.
	 * There is a HACK here in loading the States, Actions, and Transitions.
	 * Because of the lack of time (and expertise), I did not implement the loading of all this data as a single service call and DTO.
	 * Instead, the loading of this data uses the SimpleExtensionTypeDTO.
	 * Unfortunately, that means that there are repeated back end service calls:
	 * 1  Get the States
	 * 2  Get the Actions
	 * 3  Get the Transitions  (This requires multiple back end calls.  One for each State.
	 * 
	 * In order to get that all to happen so that it appears to be one call there is a cascade of service calls
	 * A callback can be specified in the call to initArtifactStates.  When ALL the loading is done, that callback gets invoked.
	 * 
	 * THIS SHOULD REALLY BE REPLACED BY A SINGLE CALL THAT CAN RETRIEVE AND PROCESS ALL THIS DATA AT ONCE.
	 * IT WILL BE MUCH LESS ERROR PRONE, FASTER, AND EASIER TO FOLLOW.
	 * 
	 * The good news is that the data is only loaded once and stored in class variables.  Subsequent calls to initArtifactStates will
	 * simply result in an invocation of the callback method.
	 *
	 */
	initArtifactStates: function(callback) {
		
		//See if they are already loaded
		var util = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
		if (util.artifactStates) {
			if (callback)
				callback();
			return;
		}
		else //Load the states from the planning plugin.xml
		{
			var self = this;
			var serviceObj = {
				success: function(states){
					self._doinitArtifactStates(states, callback);
				},
				failure: function(e){
					self._error(e);
				}
			};
			var responseHandler = new ServiceResponseHandler(serviceObj, "success", "failure");
			RegistryClient.getAllArtifactStateTypes(responseHandler);
		}			
	},
	
	//Load the artifact states from the plugin.xml and store them in the class variable "artifactStates
	_doinitArtifactStates: function(states, callback) {
		var util = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
		util.artifactStates = []; 
		for ( var i = 0; i < states.length; i++ ) {
			var state = {};
			state.name = states[i].name;
			state.id = states[i].id;
			util.artifactStates.push(state);
		}
		//Now load the Actions (The cascade effect mentioned in the comment above initArtifactStates.
		this.initArtifactActions(callback);
	},
	
	//Load the Artifact states.
	initArtifactActions: function(callback)
	{
		var util = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
		if (util.artifactActions) 
			return;
		else {
			var self = this;
			var serviceObj = {
				success: function(actions){
					self._doinitArtifactActions(actions, callback);
				},
				failure: function(e){
					self._error(e);
				}
			};
			var responseHandler = new ServiceResponseHandler(serviceObj, "success", "failure");
			RegistryClient.getAllArtifactActionTypes(responseHandler);
		}
	},
	
	//Load the artifact actions from the plugin.xml and store them in the class variable "artifactActions
	_doinitArtifactActions: function(actions, callback) {
		var util = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
		util.artifactActions = []; 
		for ( var i = 0; i < actions.length; i++ ) {
			var action = {};
			action.name = actions[i].name;
			action.id = actions[i].id;
			util.artifactActions.push(action);
		}
		//Cascade number 2.  Load the Artifact transitions.  
		//Pass the callback method along. It will be invoked when all the data is loaded
		this.initArtifactTransitions(callback);
	},
	
	initArtifactTransitions: function(callback)
	{
		//Note, initArtifactStates
		var util = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
		if (util.artifactTransitions) 
			return;
		else {
				util.artifactTransitions = [];
				//Remember the number of artifacts states.  This will be counted down so we know we've loaded them all
				this.stateCounter = util.artifactStates.length;
				for (var i = 0; i < util.artifactStates.length; i++) {
					var state = util.artifactStates[i];
					var self = this;
					var serviceObj = {
						success: function(transitions){
							self._doinitArtifactTransitions(transitions, callback);
						},
						failure: function(e){
							self._error(e);
						}
					};
					var responseHandler = new ServiceResponseHandler(serviceObj, "success", "failure");
					RegistryClient.getAllArtifactTransitions(responseHandler, state.id);
				}
		}							
	},

	//Load the artifact transitions from the plugin.xml and store them in the class variable "artifactTransitions"
	_doinitArtifactTransitions: function(transitions, callback) {
		var util = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
		for ( var i = 0; i < transitions.length; i++ ) {
			var transition = {};
			transition.initialState = transitions[i].id;
			transition.actionID = transitions[i].name;
			transition.endState = transitions[i].description;
			util.artifactTransitions.push(transition);
		}
		//when the counter (initialzed in initArtifactTransitions) gets to zero we know
		//we have processed the last of the initial states so we can invoke the callback.
		this.stateCounter--;
		if (callback && this.stateCounter == 0) 
		{
			this.loadStateTransitionConstraintSettings(callback);
		}
	},

	//Load the available actions into the provided select control.
	//The available actions is based on the current state passed into the method.
    loadArtifactActions: function(actionSelect, currentState, optionClass) {	
		var util = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
		
		var stateID = currentState;

		if ( actionSelect ) {
			dojox.data.dom.removeChildren(actionSelect);	
		}
		//Add the select action prompt
		var option = new Option(Constants.getMessage("labelSelectAction"), "NULL");  //The "NULL" value is checked for in AbstractArtifactEditor.js
		option.style.color = "gray";
		SelectUtility.add( actionSelect, option );
		
		//Loop over all the transitions and look for the ones that have an
		//initial state matching the currentState parameter supplied to this method
		for ( var i = 0; i < util.artifactTransitions.length; i++ ) 
		{
			var transition = util.artifactTransitions[i];
			if (transition.initialState == stateID) 
			{
				var actionID = transition.actionID;
				var actionName = this.getActionNameFromID(actionID);
				var option = new Option(actionName, actionID);
				dojo.addClass(option, optionClass ? optionClass : "");
				SelectUtility.add( actionSelect, option );
			}
		}
	},
	
	getStateNameFromID: function(id)
	{
		return Constants.getMessage(id);
	},	
	
	getActionNameFromID: function(id)
	{
		return Constants.getMessage(id);
	},
	
	//Determine the next state based on the current state and action
	getNextState: function(currentState, action)
	{
		var util = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
		
		//Loop over all the transitions looking for a matching initial state and action
		for ( var i = 0; i < util.artifactTransitions.length; i++ ) 
		{
			var transition = util.artifactTransitions[i];
			if (transition.initialState == currentState && transition.actionID == action) 
			{
				return transition.endState;
			}
		}
		return currentState;  //Should never happen but if it does, just return the current state
		//NOTE that this DOES occur in the CHROME browser.  This is because programmatically loading the select control and setting the selected item in CHROME is firing the onChange event
		//This causes us to end up in this method with an invalid currentState/Action pair.
	},
	
	//Load the constraints on the state transitions and such from the preferences
	//These constraints are things like "don't allow editing of an artifact in the Approved state"
	loadStateTransitionConstraintSettings : function(callback, forceLoad)
	{
		if (com.ibm.rqm.planning.web.ui.internal.util.EditorUtil.stateTransitionConstraintPrefs && !forceLoad) 
		{
			if (callback)
				callback();
		}
		else 
		{
			var self = this;
			var handlerObj = {
				success: function(response)
				{
					self._doGetConstraints(response, callback);
				},
				failure: function(response)
				{
					self._doGetConstraints(null, callback)
				}
			}
			var option = new com.ibm.asq.common.web.client.internal.option();
			var handler = new ServiceResponseHandler(handlerObj, "success", "failure");
			option.getProjectPreferences(handler, getCurrentProjectArea().itemId, Constants.ARTIFACT_STATE_TRANSITION_CONSTRAINTS_CATEGORY);
		}

	},
	
	_doGetConstraints : function(response, callback)
	{
		var constraintArray = [];
		if (response)
		{
			var prefs = dojo.fromJson(response.value);
			if (prefs && prefs.constrains && prefs.constrains.length > 0) 
				for (var i = 0; i < prefs.constrains.length; i++) 
				{
					var item = prefs.constrains[i];
					var constraint = {
						id: item.id,
						state: item.state
					};
					constraintArray.push(constraint);
				}
		}
		else
		{
			//Use our default settings
			constraintArray = [	{id:Constants.STATETRANSITION_PREFERENCE_NOEDITAPPROVED,
									state:true},
								{id:Constants.STATETRANSITION_PREFERENCE_ALLWIAPPROVED,
									state:true},
								{id:Constants.STATETRANSITION_PREFERENCE_ALLAPPROVED,
									state:true},
								{id:Constants.STATETRANSITION_PREFERENCE_ALLCHILDCASESAPPROVED,
									state:false},
								{id:Constants.STATETRANSITION_PREFERENCE_ALLCHILDSCRIPTSAPPROVED,
									state:false},
								{id:Constants.STATETRANSITION_PREFERENCE_ONLYEXECUTEAPPROVED,
									state:false}
							  ];
		}
		com.ibm.rqm.planning.web.ui.internal.util.EditorUtil.stateTransitionConstraintPrefs = constraintArray;
		if (callback)
			callback();
	},
	
    getArtifactOptions : function(callback, artifactType) {
            var self = this;
            var handlerObj = {
                success: function(optionsObj) {
	                optionsObj = optionsObj || {
	                    //defaults
	                    lockingEnabled: Constants.ARTIFACT_OPTIONS_PREFERENCE_LOCKING_DEFAULT,
	                    searchResultLimit: Constants.ARTIFACT_OPTIONS_PREFERENCE_SEARCH_LIMIT_DEFAULT
	                };					
					callback(optionsObj);
                },
                failure: function(error) {
                    callback(error);
                }
            }
            var handler = new ServiceResponseHandler(handlerObj, "success", "failure");
			var prefix = Constants.ARTIFACT_OPTIONS_PREFERENCE_KEY_PREFIX;
			if (artifactType) {
				var modelType = Utility.getModelTypeByItemType(artifactType);
				prefix += modelType.substr(modelType.lastIndexOf("."));
			}
            PreferenceClient.getPreferenceEntries(handler, getCurrentProjectArea().itemId, prefix, true);
    },
	
	//Return previously fetched transition constraint prefs
	getStateTransitionConstraintSettings: function()
	{
		return com.ibm.rqm.planning.web.ui.internal.util.EditorUtil.stateTransitionConstraintPrefs;
	},
	
	//Return true if the supplied constraint should be applied, false otherwise
	applyTransactionConstraint: function(constraint)
	{
		var transitionPrefs = this.getStateTransitionConstraintSettings();
		if (!transitionPrefs || transitionPrefs.length == 0)
			return false;  //Probably an error condition.  Don't constrain
			
		for (var i = 0; i < transitionPrefs.length; i++)
		{
			if (transitionPrefs[i].id == constraint)
				return transitionPrefs[i].state;
		}
		return false;
	},
	
    initScriptTypes: function(selectNode, selectedType) {
		SelectUtility.add( selectNode, new Option("Loading...", ""));
        var self = this;
        var handlerObj = {
            success: function(types) {
                self._doInitScriptTypes(types, selectNode, selectedType);
            },
            failure: function(e) {
                alert("EditorUtil._initScriptTypes - Exception: " + e.message ? e.message : e);
            }
        };        
        var handler= new ServiceResponseHandler(handlerObj, "success", "failure");
        RegistryClient.getAllExecutionScriptTypes(handler);         
    },
    
    _doInitScriptTypes: function(types, selectNode, selectedType) {     
        if ( types != null && types.length > 0 ) {              
            dojox.data.dom.removeChildren(selectNode);
            for (var i = 0; i < types.length; i++) {
                if (types[i].name != "Unknown" && types[i].name != "NoScript") { 
					var optDisplayName = (types[i].name == "Manual") ? Constants.getMessage("labelManual") : Constants.getExtensionDisplayMessage(types[i].name);
                    var opt = new Option(optDisplayName, types[i].name);
                    if ( types[i].name == selectedType ) {
                        opt.selected = true;
                    }
 					SelectUtility.add( selectNode, opt );
               }
            }
        }           
    },        
	
	//TODO this should not be hardcoded and instead be properly integrated with WI workflow
	initWorkItemStates: function(select, currentWorkItemState) {			
		if ( select ) {
			dojox.data.dom.removeChildren(select);	
		}
		
		if(currentWorkItemState == null) {
			currentWorkItemState = "New";
			select.disabled = true;
		}
		
		var currentState = new Option(currentWorkItemState, currentWorkItemState);
		SelectUtility.add( select, currentState);
		currentState.selected = true;
		
		var action1 = new Option("Start Working", "Start Working");
		var action2 = new Option("Resolve", "Resolve");
		var action3 = new Option("Stop Working", "Stop Working");
		var action4 = new Option("Reopen", "Reopen");
		
		if(currentWorkItemState == "New") {
			SelectUtility.add( select, action1);
			SelectUtility.add( select, action2);
		} else if(currentWorkItemState == "In Progress") {
			SelectUtility.add( select, action2);
			SelectUtility.add( select, action3);
		} else if(currentWorkItemState == "Resolved") {
			SelectUtility.add( select, action4);
			SelectUtility.add( select, action2);
		} else if(currentWorkItemState == "Reopened") {
			SelectUtility.add( select, action1);
			SelectUtility.add( select, action2);
		}
	},
	
	//TODO this should not be hardcoded and instead be properly integrated with WI workflow
	//the numbers returned below match the stateIds as specified in process specification for
	//authoring work items
	getWIStateForAction: function(selection) {	
		var workFlowInfo = {
			name: "New",
			stateId: "2",
			icon: ICON_STATE_NEW
		}; 		
		
		if(selection == "Start Working" || selection == "In Progress") {
			workFlowInfo.name = "In Progress";
			workFlowInfo.stateId = "2";
			workFlowInfo.icon = ICON_STATE_IN_PROGRESS;
		} else if(selection == "Resolve" || selection == "Resolved" ) {
			workFlowInfo.name = "Resolved";
			workFlowInfo.stateId = "3";
			workFlowInfo.icon = ICON_STATE_RESOLVED;
		} else if(selection == "Stop Working" || selection == "New") {
			workFlowInfo.name = "New";
			workFlowInfo.stateId = "1";
			workFlowInfo.icon = ICON_STATE_NEW;
		} else if(selection == "Reopen" || selection == "Reopened") {
			workFlowInfo.name = "Reopened";
			workFlowInfo.stateId = "6";
			workFlowInfo.icon = ICON_STATE_REOPENED;
		}
		
		workFlowInfo.icon = ICON_PREFIX + workFlowInfo.icon;
	
		return workFlowInfo;
	},
	
	getStateImageForSelection: function(selectedAction) {			
		if(selectedAction == "Start Working") {
			return "2"; 
		} else if(selectedAction == "Resolve") {
			return "3";
		} else if(selectedAction == "Stop Working") {
			return "1";
		} else if(selectedAction == "Reopen") {
			return "6";
		}
		return "1";
	},
	
	setWorkItemNumberAsURL: function(workItemNumber, workItem)
	{
		var url = Utility.getViewWorkItemURI(workItem.id);
		dojox.data.dom.removeChildren(workItemNumber);
		var anchor = document.createElement("A");
		anchor.className = "";
		anchor.href = url;
		var tn = document.createTextNode("(WorkItem " + workItem.id + ")");
		anchor.appendChild(tn);
		workItemNumber.appendChild(anchor);
		TooltipBuilder.addWorkItemTip(workItem, anchor);
		return anchor;
	},
		
	setPageTitle: function(domNode, artifactType, artifact) {
		dojox.data.dom.removeChildren(domNode);
		
		//add the image
		var img = document.createElement("img");
		img.title = artifactType;
		img.alt = artifactType;
		img.src= Utility.getArtifactIconURL(artifactType);
		img.className = "image";
		domNode.appendChild(img);
		
		//add the text
		var text = document.createElement("span");
		text.className = "title-text";
		if ( artifact != null ) {
			text.innerHTML = new String(artifact.name);
			if (artifact.snapshotInfo!=null) {
				var snapshotInfo = document.createElement("span");
				snapshotInfo.className = "revision-info"
				snapshotInfo.innerHTML += "  (Snapshot " + artifact.snapshotInfo.name + ")";
				text.appendChild(snapshotInfo);
			}							
		} else {
			var title = new String(Constants.NEW_PAGE_TITLE);
			title = title.replace(/_type/, artifactType);
			title = title.replace(/_id/, this.getTimeString());
			text.innerHTML = title;
		} 						
		domNode.appendChild( text );
	},	
	
	setLastSaved: function(domNode, status, error) {
		dojox.data.dom.removeChildren(domNode);
		var text = "";
		var style = null;
		if ( status == Constants.STATUS_SUCCESS ) {
			text = "Saved successfully at: " + this.getTimeString();		
			//style = "status-success";		
		} else if ( status == Constants.STATUS_FAILURE ) {
			// see WorkItem 4241.  License error messages must be shown 
			// in a consistent way with RTLM (js alert dialog)
            if(error && error.message && error.message.toLowerCase().indexOf('license') > -1) {
				var licenseMsg = MessageUtil.getMessage(MessageBundle.COMMON, "serverLicenseErrMsg");
			    this.showServerErrorMessage(licenseMsg);			
			}
			text = "Saved failed at: " + this.getTimeString();	
			//style = "status-failure";	
		} else if ( status == Constants.STATUS_SAVING ) {
			text = "Saving...";
		} else if ( status == Constants.STATUS_NEW ) {
			text = "New / Not Yet Saved";
			//style = "status-failure";
		} else if ( status == Constants.STATUS_DIRTY ) {
			text = "Contains Unsaved Changes";
			//style = "status-failure";
		}
		
		//status message
		var statusElement = document.createElement("span");			
		statusElement.innerHTML = text;
		dojo.addClass(statusElement, "status");
		if (style != null) {
			dojo.addClass(statusElement, style);
		}		
		domNode.appendChild(statusElement);		
		
		//error icon & message, if present
		if (error && error.message) {
			var icon = document.createElement("img");
			icon.src = ImageCache.HELP;
			dojo.addClass(icon, "status");
			domNode.appendChild(icon);
		
			var html = "<b>Error: </b><br/>" + error.message;
			if (error.caughtAt) {
				html += "<br/><b>Caught At:</b> " + error.caughtAt;
			}
			//TooltipBuilder.addTooltip(html, icon);
			var x = new dijit.Tooltip({label: html, showDelay: 100, connectId:[domNode]});
		}
	},

	getTimeString: function() {
		var now = new Date();
		return dojo.string.pad(now.getHours(), 2, '0') + ":" 
			 + dojo.string.pad(now.getMinutes(), 2, '0') + ":" 
			 + dojo.string.pad(now.getSeconds(), 2, '0');		
	},
	
	// copied from RtlmUiUtil.js to avoid adding a .js dependency
	// 
	// Summary:
    //    Displays server error message in a popup dialog. Normalizes the server message to 
    //    replace encoded character strings (%2D), etc.
    //
    showServerErrorMessage: function(message)
    {
        var c = new RegExp("\%[a-zA-Z0-9]{2}");
        var ss = message.split(c);
        var str = "";
        for(var i = 0; i < ss.length; i++)
            str = str + ss[i];
           
        alert(str);
    },
	
	// Summary: 
	//    If exploror's locale is iw/he/ar/fa/ur/yi, the UI's direction is RTL, and return true. 
	isDirectionRtl: function() {
		var lang = dojo.locale.split('-')[0];
		var RTL = {ar:true,fa:true,he:true,ur:true,yi:true,iw:true};
		return RTL[lang]; // Boolean
	}
};

})();
