/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.ui.internal.util.FunctionalTooltip");

dojo.require("dijit.Tooltip");

(function() {

dojo.declare("com.ibm.asq.common.web.ui.internal.util.FunctionalTooltip", dijit.Tooltip, {
	
	hideDelay: 190,
	
	showDelay: 600,
	
	constructor: function(args) {
		this._closeCallback = args.closeCallback;
		this._populateCallback = args.callback;
		this._populateCallbackArg = args.callbackArg;
		this._repopulateDelay = args.repopulateDelay;
		this._preDisplayCheck = args.preDisplayCheck;
		if (this._populateCallbackArg)
			this._populateCallbackArg.tooltip = this;
	},
	
	open: function() {
		if (this._preDisplayCheck && !this._preDisplayCheck())
			return;
		this._eventHandles = new Array();
		dojo.forEach(["onMouseOver", "onHover", "onMouseOut", "onUnHover"], function(event){
				this._eventHandles.push(this.connect(dijit.MasterTooltip.domNode, event.toLowerCase(), "_"+event+"Tooltip"));
			}, this);
		if (this._populateCallback) {
			var populate = true;
			if (this._repopulateDelay) {
				var date = new Date();
				var time = date.getTime();
				if (this._timeStamp && (this._repopulateDelay == -1 || this._timeStamp + this._repopulateDelay > time))
					populate = false;
				else
					this._timeStamp = time;
			}
			if (populate)
				this._populateCallback(this._populateCallbackArg);
		}	
		this.inherited(arguments);
		this.validateConnectNode();
	},
	
	validateConnectNode: function() {
		var theNode = this._connectNode;
		while(theNode) {
			if (theNode == document.body) {
				this._validateNodeTimeout = setTimeout(dojo.hitch(this,this.validateConnectNode),500);
				return;
			}
			theNode = theNode.parentNode;
		}
		this.close();
			
	},
	
	close: function() {
		if (this._validateNodeTimeout) {
			clearTimeout(this._validateNodeTimeout);
			delete this._validateNodeTimeout;
		}
		this.inherited(arguments);
		if (this._eventHandles) {
			this.disconnect(this._eventHandles);
			this._eventHandles = null;
		}
		if (this._closeCallback) {
			this._closeCallback();
		}
	},
	
	refresh: function() {
		if (this._hover)
			dijit.MasterTooltip.show(this.label || this.domNode.innerHTML, this._connectNode);
	},
	
	_onMouseOverTooltip: function(e) {
		this._onHoverTooltip(e);
	},
	
	_onHoverTooltip: function(e) {
		this.isTipHovered = true;
	},
	
	_onMouseOutTooltip: function(e) {
		if(dojo.isDescendant(e.relatedTarget, dijit.MasterTooltip.domNode)){
			// false event; just moved from target to target child; ignore.
			return;
		}
		this._onUnHoverTooltip(e);
	},
	
	_onUnHoverTooltip: function(e) {
		this.isTipHovered = false;
		if (e.relatedTarget == this._connectNode || dojo.isDescendant(e.relatedTarget, this._connectNode)) {
			// moved from tooltip back to connectNode; ignore.
			return;
		}
		this._onUnHover(e);
	},
	
	_onClickTooltip: function(e) {
		this.close();
	},
	
	_onHover: function(e) {
		if (this._unHoverTimer) {
			clearTimeout(this._unHoverTimer);
			delete this._unHoverTimer;
		}
		this.inherited(arguments);
	},
	
	_onUnHover: function(e) {
		var args = arguments;
		this._unHoverTimer = setTimeout(dojo.hitch(this,function(){
			if (!this.isTipHovered)
				this.inherited(args);
			delete this._unHoverTimer;
		}), this.hideDelay);
	}
});

})();

// P.C. 03/20/2008
// FunctionalTooltip is a clone of Jazz com.ibm.team.dashboard.viewlets.web.ui.internal.utils.PersistentTooltip.
// The override of the _MasterTooltip is to work around a bug on positioning the call out anchor to the tool tip
// prior to Dojo 1.0.  Believe that the bug is fixed and the PersistentTooltip is no longer in the Jazz viewlets
// web plugin.  This should be deleted after verifying that there is no regression with this in cemment.
//
//dojo.addOnLoad(function(){
//	// override the dijit._MasterTooltip.show() function
//	dijit._MasterTooltip.prototype.show = function(/*String*/ innerHTML, /*DomNode*/ aroundNode){
//		// summary:
//		//	Display tooltip w/specified contents to right specified node
//		//	(To left if there's no space on the right, or if LTR==right)
//
//		if(this.fadeOut.status() == "playing"){
//			// previous tooltip is being hidden; wait until the hide completes then show new one
//			this._onDeck=arguments;
//			return;
//		}
//		this.containerNode.innerHTML=innerHTML;
//		
//		// Firefox bug. when innerHTML changes to be shorter than previous
//		// one, the node size will not be updated until it moves.  
//		this.domNode.style.top = (this.domNode.offsetTop + 1) + "px";
//		
//		// position the element and change CSS according to position
//		var align = this.getAlign(aroundNode);
//		if (!align)
//			return;
//		var pos = dijit.placeOnScreenAroundElement(this.domNode, aroundNode, align);
//		this.domNode.className="dijitTooltip dijitTooltip" + (pos.corner=='TL' || pos.corner=='BL' ? "Right" : "Left");
//		if (pos.corner=='TL' || pos.corner=='TR') {
//			dojo.addClass(dojo.query(".dijitTooltipConnector",this.domNode)[0], "dijitTooltipConnectorTop");
//			dojo.addClass(this.containerNode, "dijitTooltipContainerTop");
//		} else {
//			dojo.removeClass(dojo.query(".dijitTooltipConnector",this.domNode)[0], "dijitTooltipConnectorTop");
//			dojo.removeClass(this.containerNode, "dijitTooltipContainerTop");
//		}
//		
//		// show it
//		dojo.style(this.domNode, "opacity", 0);
//		this.fadeIn.play();
//		this.isShowingNow = true;
//	};
//	
//	// determine where the node is in the viewport and generate the align param for dijit.placeOnScreenAroundElement accordingly
//	//   if aroundNode is in the right half of the viewport, try to place the tooltip on the left and vice versa
//	//   if aroundNode is in the top half of the viewport, try to place the tooltip on the bottom and vice versa
//	dijit._MasterTooltip.prototype.getAlign = function(/*DomNode*/ aroundNode){
//		var aroundNodePos = dojo.coords(aroundNode, true);
//		// if all the coordinates are 0, the node no longer exists and the show should be cancelled
//		if (aroundNodePos.h == 0 && aroundNodePos.l == 0 && aroundNodePos.t == 0 && aroundNodePos.w == 0 && aroundNodePos.x == 0 && aroundNodePos.y == 0)
//			return false;
//			
//		var scrollTop = self.pageYOffset;
//		if (!scrollTop)
//			scrollTop = document.body.scrollTop;
//		var scrollLeft = self.pageXOffset;
//		if (!scrollLeft)
//			scrollLeft = document.body.scrollLeft;
//		
//		var invertRightLeft = document.body.offsetWidth - ((aroundNodePos.x - (aroundNodePos.w / 2)) - scrollLeft) < document.body.offsetWidth / 2;
//		invertRightLeft = invertRightLeft ? !this.isLeftToRight() : this.isLeftToRight();
//		if (document.body.offsetHeight - ((aroundNodePos.y - (aroundNodePos.h / 2)) - scrollTop) < document.body.offsetHeight / 2)
//			return invertRightLeft ? {'BR': 'BL', 'BL': 'BR'} : {'BL': 'BR', 'BR': 'BL'};
//		return invertRightLeft ? {'TR': 'TL', 'TL': 'TR'} : {'TL': 'TR', 'TR': 'TL'};
//	};
//});
//
