/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
var Utility = dojo.provide("com.ibm.rqm.execution.web.ui.internal.util.Utility");

dojo.require("dojo.date");
dojo.require("dojo.date.locale");

dojo.require("com.ibm.asq.common.web.client.internal.CommonPreferenceClient");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.execution.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.rqm.execution.web.client.internal.execution.ExecutionResultStates");
dojo.require("com.ibm.rqm.execution.web.client.internal.execution.ExecutionElementResultStates");
dojo.require("com.ibm.rqm.planning.web.client.internal.PreferenceClient");
dojo.require("com.ibm.team.process.web.client.projectarea");

(function() 
{
	var ImageCache 					= com.ibm.rqm.execution.web.ui.internal.view.common.ImageCache;
	var Constants  					= com.ibm.rqm.planning.web.client.internal.Constants;
	var ExecutionResultStates 		= com.ibm.rqm.execution.web.client.internal.execution.ExecutionResultStates;
	var ExecutionElementResultStates = com.ibm.rqm.execution.web.client.internal.execution.ExecutionElementResultStates;
	var ASQUtility					= com.ibm.asq.common.web.ui.internal.util.Utility;
    var ACTION_PREFIX   			= "#action=com.ibm.rqm.planning.home.actionDispatcher";
	var PreferenceClient				= com.ibm.rqm.planning.web.client.internal.PreferenceClient;
	var getCurrentProjectArea			= com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
	
	var LABEL_NOT_RUN=Constants.getMessage("labelNotRun");
			
	com.ibm.rqm.execution.web.ui.internal.util.Utility = 
	{
			_stepsHaveVerdict: true,
			executionPreferences: {},
			
			PLANNING_PERSPECTIVE: 1,
			CONSTRUCTION_PERSPECTIVE: 2,
			EXECUTION_PERSPECTIVE: 3,
			
		_planningPrefix: ACTION_PREFIX + "&subAction=",
		_constructionPrefix: ACTION_PREFIX + "&subAction=",
		_executionPrefix: ACTION_PREFIX + "&subAction=",
		
		ATTRIBUTENAMES: ["SoftwareInstall", "cpu", "operatingsystem"],
		
		/******************************************************************************************
		 * Generic URI functions
		 * 
		 *****************************************************************************************/
		_getPlanningActionUri: function(action, kwArgs)
		{
			var str = this._planningPrefix + action;
			return(this._getActionUri(str, kwArgs));
		},
		
		_getConstructionActionUri: function(action, kwArgs)
		{
			var str = this._constructionPrefix + action;
			return(this._getActionUri(str, kwArgs));
		},
		
		_getExecutionActionUri: function(action, kwArgs)
		{
			var str = this._executionPrefix + action;
			return(this._getActionUri(str, kwArgs));
		},
		
		_getActionUri: function(str, kwArgs) 
		{
		    for (var key in kwArgs) 
			{
	        	str += "&" + key + "=" + kwArgs[key];
	        }
			return str;
		},

		_buildUri: function(method, args, perspective)
		{
			var result;
			
			switch(perspective)
			{
				case this.PLANNING_PERSPECTIVE:
					result = this._getPlanningActionUri(method, args);
					break;
				case this.CONSTRUCTION_PERSPECTIVE:
					result = this._getConstructionActionUri(method, args);
					break;
				case this.EXECUTION_PERSPECTIVE:
					result = this._getExecutionActionUri(method, args);
					break;
			}
			
			return(result);
		},
		
		/******************************************************************************************
		 * Action URI methods
		 * 
		 * @param {Object} id
		 *****************************************************************************************/
		getGenericSearchUri: function(str)	{
			return(this._getPlanningActionUri("genericSearch", {searchStr: str}));
		},

		getNewQueryUri: function()	{
			return(this._getPlanningActionUri("newQuery"));
		},

		getViewWorkItemURI: function(id){
			return this._getPlanningActionUri("viewWorkItem", {id: id});
		},
		
		getNewTestGroupURI: function(name)	{
			return this._getPlanningActionUri("newTestGroup", {name: name});
		},
	
		getTestGroupURI: function(groupId){
			var args = {};
			if (groupId != null){
				args = {id: groupId};
			}
			return this._getPlanningActionUri("viewTestGroup", args);
		},		
			
		getTestGroupsURI: function(queryName){
			var args = {};
			if (queryName != null)	{
				args = {name: queryName};
			}
			return this._getPlanningActionUri("viewTestGroups", args);
		},
		
		getNewTestCaseURI: function(name) 
		{
			return this._getPlanningActionUri("newTestCase", {name: name});
		},
			
		getTestCaseURI: function(testId){
			var args = {};
			if (testId != null)	{
				args = {id: testId};
			}
			return this._getConstructionActionUri("viewTestCase", args);
		},
				
		getTestCasesURI: function(queryName){
			var args = {};
			if (queryName != null)	{
				args = {name: queryName};
			}
			return this._getPlanningActionUri("viewTestCases", args);
		},
		
		getNewTestScriptURI: function(name, perspective){
			if (perspective == null){
				perspective = this.PLANNING_PERSPECTIVE;
			}

			return(this._buildUri("newTestScript", {name: name}, perspective));
		},
			
		getTestScriptURI: function(scriptId, perspective, /*optional*/ revision, /*optional*/ snapshotName){
			var args = {};
            if (scriptId != null){
                if (revision != null) {
                    args.uid = scriptId;
                    args.revision = revision;
                    if (snapshotName!=null) {
                        args.snapshotName = escape(snapshotName);
                    }
                } else {
                    args.id = scriptId;
                }
            }
			if (perspective == null){
				perspective = this.PLANNING_PERSPECTIVE;
			}
			return(this._buildUri("viewTestScript", args, perspective));
		},
				
		getTestScriptsURI: function(queryName, perspective)	{
			var args = {};

			if (queryName != null) {
				args = {name: queryName};
			}

			if (perspective == null){
				perspective = this.PLANNING_PERSPECTIVE;
			}
			return(this._buildUri("viewTestScripts", args, perspective));
		},
		
		getNewTestPlanURI: function(name){
			return this._getPlanningActionUri("newTestPlan", {name: name});
		},
			
		getTestPlanURI: function(kwargs){
			var args = {};
			if ( kwargs.itemId != null ){
				args = {itemId: kwargs.itemId};
			}
			else if (kwargs.id != null)	{
				args = {id: kwargs.id};
			}
			else if (kwargs.name != null){
				args = {name: kwargs.name};
			}
			return this._getPlanningActionUri("viewTestPlan", args);
		},
				
		getTestPlansURI: function(queryName){
			var args = {};
			if (queryName != null) 	{
				args = {name: queryName};
			}
			return this._getPlanningActionUri("viewTestPlans", args);
		},
		
		getNewEwiURI: function(name, testCaseId) {
			var testId = testCaseId != null ? testCaseId : null; 
			return this._getPlanningActionUri("newEwi", {name: name, testCaseId: testId});
		},
			
		getEwiURI: function(id, perspective) {
			var args = {};
			if (id != null) {
				args = {id: id};
			}

			if (perspective == null){
				perspective = this.CONSTRUCTION_PERSPECTIVE;
			}
			return(this._buildUri("viewEwi", args, perspective));
		},
				
		getEwisURI: function(queryName, perspective){
			var args = {};
			if (queryName != null) {
				args.name = queryName;
			}
			if (perspective == null){
				perspective = this.PLANNING_PERSPECTIVE;
			}
			return(this._buildUri("viewEwis", args, perspective));
		},
		
		getSizingRecordURI: function(sizingRecordId) {
			var args = {};
			if (sizingRecordId != null)	{
				args = {uid: sizingRecordId};
			}
			return this._getPlanningActionUri("viewTestSizing", args);
		},
		
		startExecution: function(ewi, scriptId, resultItemId, requestId, ergId)	{
			var args = {};
			
			if (ewi != null){
				args = {
					ewi: ewi,
					runScript: scriptId
				};
				if(resultItemId != null)
					args._resultId = resultItemId;
			}
			if(requestId)
				args.requestId = requestId;
			if(ergId)
				args.ergId = ergId;	
			return this._getExecutionActionUri("startExecution", args);
		},
		
		runSuite: function(kwArgs){
			var args = {};
			if (kwArgs.groupId != null)
				args.groupId = kwArgs.groupId
			if (kwArgs.itemId != null)
				args.itemId = kwArgs.itemId
			return this._getExecutionActionUri("runSuite", args);
		},
	
		executedItemsURI: function (queryName){
			var args = {};			
			if (queryName != null) {
				args.name = queryName;
			}
			return this._getExecutionActionUri("viewExecutedItems", args);
		},
	
		getEWIResultsURI: function(ewiItemId){
			var args = {};			
			if (ewiItemId != null) {
				args.ewiItemId = ewiItemId;
			}			
			return this._getExecutionActionUri("viewEWIResults", args);
		},
		
		showResultsURI: function(queryName){
			var args = {};			
			if (queryName != null) {
				args.name = queryName;
			}			
			return this._getExecutionActionUri("viewResults", args);
		},
		
		getResultURI: function(args){
			return this._getExecutionActionUri("viewResult", args );
		},
		
		showSuiteResult: function(kwArgs){
			var args = {};			
			if (kwArgs.id != null)
				args.id = kwArgs.id;
			if(kwArgs.itemId != null)
				args.itemId = kwArgs.itemId;			
			return this._getExecutionActionUri("showSuiteResult", args);
		},
		getTestSuiteURI: function(suiteId, perspective){
			var args = {};
			args.id = suiteId;
			return this._buildUri("viewTestSuite", args, perspective ? perspective : this.CONSTRUCTION_PERSPECTIVE);
		},

		/******************************************************************************************
		 * Generate Unique URL
		 * 
		 * @param {Object} url
		 *****************************************************************************************/
		reuniquifyUrl: function(url){
			return(url.replace(/&tq=([0-9]+)/, "") + "&tq=" + this.getUniqueId());
		},
		
		getUniqueId: function(){
			return Date.parse(new Date().toString());
		},

		/******************************************************************************************
		 * Set the window location to an action with keywords
	 	 *****************************************************************************************/
		setLocationToActionWithKeywords: function(action, kwArgs){
			//TODO: fix this for all perspectives
			this._setLocationHash(this.getPlanningActionURI(action, kwArgs));
		},
	
		/******************************************************************************************
		 * Set the window location hash.
		 * This is pushed to a method so all changes to the URI can be traced if needed 
		 *****************************************************************************************/
		_setLocationHash: function(locationHash){
			window.location.hash = locationHash;
		},

		/******************************************************************************************
		 * QSort functions
		 * 
		 * @param {Object} list
		 * @param {Object} a
		 * @param {Object} b
		 *****************************************************************************************/
		swap: function(list, a, b){
			var tmp = list[a];
			list[a] = list[b];
			list[b] = tmp;
		},
		
		partition: function(array, begin, end, pivot){
			var piv = array[pivot];
			this.swap(array, pivot, end-1);
			
			var store=begin;
	
			for(var ix = begin; ix < end-1; ++ix) {
				if(array[ix] <= piv){
					this.swap(array, store, ix);
					++store;
				}
			}
			
			this.swap(array, end-1, store);
		
			return store;
		},
		
		qsort: function(list, begin, end){
			if(end-1>begin){
				var pivot = begin+Math.floor(Math.random()*(end-begin));
				pivot = this.partition(list, begin, end, pivot);
				this.qsort(list, begin, pivot);
				this.qsort(list, pivot+1, end);
			}		
		},
		
		partition_obj: function(array, index, begin, end, pivot){
			var piv = array[pivot].elements[index].data;
			if (typeof piv == "string"){
				piv = piv.toLowerCase();
			}
	
			this.swap(array, pivot, end-1);
			
			var store=begin;
	
			for(var ix = begin; ix < end-1; ++ix){
				data = array[ix].elements[index].data;
				if (typeof data == "string"){
					data = data.toLowerCase();
				}
				
				if(data <= piv)	{
					this.swap(array, store, ix);
					++store;
				}
			}
			
			this.swap(array, end-1, store);
		
			return store;
		},
		
		qsort_obj: function(list, index, begin, end){
			if(end-1>begin)	{
				var pivot = begin+Math.floor(Math.random()*(end-begin));
	
				pivot = this.partition_obj(list, index, begin, end, pivot);
	
				this.qsort_obj(list, index, begin, pivot);
				this.qsort_obj(list, index, pivot+1, end);
			}		
		},
		
		/******************************************************************************************
		 * Get Icon/Image functions
		 *  
		 * @param {Object} path
		 *****************************************************************************************/
		getArtifactIconURL: function(artifactType){
			switch(artifactType) {
				case Constants.TYPE_PLAN:
					return ImageCache.TEST_PLAN;
				case Constants.TYPE_GROUP:
					return ImageCache.TEST_GROUP;
				case Constants.TYPE_CASE:
					return ImageCache.TEST_CASE;
				case Constants.TYPE_SCRIPT:
					return ImageCache.TEST_SCRIPT;
				case Constants.TYPE_PLAN_SIZING:
					return ImageCache.TEST_PLAN;
				case Constants.TYPE_GROUP_SIZING:
					return ImageCache.TEST_GROUP;
				case Constants.TYPE_CASE_SIZING:
					return ImageCache.TEST_CASE;
				case Constants.TYPE_SCRIPT_SIZING:
					return ImageCache.TEST_SCRIPT;
				case Constants.TYPE_EWILIST:
					return ImageCache.EWI;	
				default: 
					return "";
					
			};		
		},
		
		/******************************************************************************************
		 * Returns the Constant representing the artifactType
		 *  
		 * @param {Object} artifact - the QM artifact DTO object instance 
		 *****************************************************************************************/		
		getArtifactType: function(artifact) {
			var fullType = artifact._eQualifiedClassName;
			var type = fullType.substr(fullType.lastIndexOf(':') + 1);
			
			switch(type) {
				case Constants.CLASSNAME_PLAN:
					return Constants.TYPE_PLAN;
				case Constants.CLASSNAME_GROUP:
					return Constants.TYPE_GROUP;
				case Constants.CLASSNAME_CASE:
					return Constants.TYPE_CASE;
				case Constants.CLASSNAME_SCRIPT:
					return Constants.TYPE_SCRIPT;
				case Constants.CLASSNAME_EWI:
					return Constants.TYPE_EWI;	
				case Constants.CLASSNAME_PLAN_SIZING:
					return Constants.TYPE_PLAN_SIZING;
				case Constants.CLASSNAME_GROUP_SIZING:
					return Constants.TYPE_GROUP_SIZING;
				case Constants.CLASSNAME_CASE_SIZING:
					return Constants.TYPE_CASE_SIZING;
				case Constants.CLASSNAME_SCRIPT_SIZING:
					return Constants.TYPE_SCRIPT_SIZING;
				default:
					return null;											
			}			
		},		
		
		/******************************************************************************************
		 * Returns the Constant representing the item's model type
		 *  
		 * @param {Object} item - the QM artifact DTO object instance 
		 *****************************************************************************************/		
		getModelType: function(item) {
			var fullType = item._eQualifiedClassName;
			var type = fullType.substr(fullType.lastIndexOf(':') + 1);
			
			switch (type) {
				case Constants.CLASSNAME_PLAN:
					return Constants.MODEL_TYPE_PLAN;
				case Constants.CLASSNAME_GROUP:
					return Constants.MODEL_TYPE_GROUP;
				case Constants.CLASSNAME_CASE:
					return Constants.MODEL_TYPE_CASE;
				case Constants.CLASSNAME_SCRIPT:
					return Constants.MODEL_TYPE_SCRIPT;
				default:
					return null;
			}											
		},
		
		
		/******************************************************************************************
		 * Get time as a string
		 *****************************************************************************************/
		//@Deprecated
		//Use com.ibm.asq.common.web.ui.internal.util.Utility.getTimeString instead
		getTimeString: function(/*accepts long*/time) {
			return ASQUtility.getTimeString(time);	
		},
		
		/******************************************************************************************
		 * Get Date object from ISO string.  Need to wrap call to dojo.date.stamp.fromISOString.
		 *****************************************************************************************/	
		 //@Deprecated
		 //Use com.ibm.asq.common.web.ui.internal.util.Utility.dateFromISOString instead
		dateFromISOString: function(timestamp) {
			return ASQUtility.dateFromISOString(timestamp);
		},		
	
		/******************************************************************************************
		 * Function: formatDate
		 * @param {Object} modified
		 * @param {Object} isIso
		 *****************************************************************************************/
		//@Deprecated
		 //Use com.ibm.asq.common.web.ui.internal.util.Utility.formatDate instead
		formatDate: function(modified, isIso){
           return ASQUtility.formatDate(modified, isIso);
		},
	
		getImageForState: function(/* id or object */ result){
			var img = ImageCache.INCON_ICON;
			if (dojo.isString(result)) {
				return ExecutionResultStates.ICONS[result];
			} else if (result && result.id) {
				return ExecutionResultStates.ICONS[result.id];
			}
			return img;
		},
		
        getImageForResult: function(/* id or object */ result){
            var img = ImageCache.INCON_ICON;
            if (dojo.isString(result)) {
                return ExecutionElementResultStates.ICONS[result];
            } else if (result && result.id) {
                return ExecutionElementResultStates.ICONS[result.id];
            }
            return img;
        },		
	
	/******************************************************************************************
       * Function: formatDateAsLong
       * @param {Object} modifiedAsLong
       * @param {Object} isIso
       *****************************************************************************************/
	  //@Deprecated
	   //Use com.ibm.asq.common.web.ui.internal.util.Utility.formatDateFromLong instead
      formatDateFromLong: function(modifiedAsLong, isIso){
		return ASQUtility.formatDateFromLong(modifiedAsLong, isIso);
      },	
     
      
	  getStatusText: function(/* ExecutionResultDTO */ result) {
         if (result && result.state) {
			return result.state.name;
         }
         return LABEL_NOT_RUN;
      },		
		
		isPaused: function(/* ExecutionResultDTO */ result) {
         if (result && result.state.name == "In Progress"){ 
            return true;         
         }
         return false;			
		},
		
		getResultsTooltip: function(args){
			if(args){
				var str = [];
				if(args.result.state && args.result.state.id == ExecutionResultStates.IN_PROGRESS){
					str.push("<br/>" + Constants.getMessage("labelInProgress"));
				}
				else
					str.push(Constants.getMessage("labelLastResult")+": <b>"+args.result.state.name+"</b><br/>");
				str.push(Constants.getMessage("labelLastExecTime",this._convertLongToDate(parseInt(args.result.startTime))));
				return str.join("");
			}else{
                var str = Constants.getMessage("labelNoResultsFound");
				return str;
			}	
		},
		
		getTestInformationToolTip: function(args){
			var str = [];
			if(args){		 		
				str.push("<table><tr><td> </td><td> </td></tr>");
				str.push("<tr><td align=left>"+Constants.getMessage("labelType")+": "+"</td><td>"+args.type+"</td><tr>");
				str.push("<tr><td> </td><td> </td></tr>");
				str.push("<tr><td align=left>"+Constants.getMessage("labelTestPlan")+": "+"</td><td>"+args.testPlan+"</td><tr>");				
				str.push("<tr><td> </td><td> </td></tr>");
				str.push("<tr><td align=left>"+Constants.getMessage("labelIteration")+": "+"</td><td>"+args.testPhase+"</td><tr>"); 		// CML: should be "test milestone"				
				str.push("</table>");
			}else{
				str.push("<table><tr><td> </td></tr>");
				str.push("<tr><td align=right>"+Constants.getMessage("labelNoInfoFound")+"</td><tr>");
				str.push("</table>");
			}	
			return str.join("");
		},

		getAdapterInfoToolTip: function(adapter, availablity){
			var str = [];
			if(adapter){				
				var status = availablity ? availablity : (adapter.busy ? Constants.getMessage("labelUnavailable") : Constants.getMessage("labelAvailable"));
				str.push("<table><tr><td> </td><td> </td></tr>");
				str.push("<tr><td align=left>"+Constants.getMessage("labelMachine")+": "+"</td><td>"+adapter.hostName+"</td><tr>");
				str.push("<tr><td> </td><td> </td></tr>");
				str.push("<tr><td align=left>"+Constants.getMessage("labelStatus")+": "+"</td><td>"+ status +"</td><tr>");				
				str.push("<tr><td> </td><td> </td></tr>");
				str.push("<tr><td align=left>"+Constants.getMessage("labelDescription")+": "+"</td><td>"+adapter.description+"</td><tr>");				
				str.push("</table>");
			}else{						
				str.push("<table><tr><td> </td></tr>");
				str.push("<tr><td align=right>"+Constants.getMessage("labelNoInfoFound")+"</td><tr>");
				str.push("</table>");
			}	
			return str.join("");
		},

		getRequestStateToolTip: function(state){
			var str = [];
			if(state && state.id){
				var status = this.getStateInfo(state.id, true);
				str.push("<table><tr><td> </td><td> </td></tr>");
				str.push("<tr><td align=left>"+Constants.getMessage("labelstatus")+": "+"</td><td>"+ state.name +"</td><tr>");				
				str.push("<tr><td> </td><td> </td></tr>");
				str.push("<tr><td align=left>"+Constants.getMessage("labelDescription")+": "+"</td><td>"+status+"</td><tr>");				
				str.push("</table>");
			}else{						
				str.push("<table><tr><td> </td></tr>");
				str.push("<tr><td align=right>"+Constants.getMessage("labelNoInfoFound")+"</td><tr>");
				str.push("</table>");
			}
			return str.join("");
		},
		
		getStateInfo: function(stateId, getMsg, getIcon){
			var msg = Constants.getMessage("labelNoInfoFound");
			var icon = ImageCache.NOIMAGE;
			switch (stateId){
				case "com.ibm.rqm.executionframework.common.requeststate.nottaken":
					msg = Constants.getMessage("msgReqQueued");
					icon = ImageCache.REQ_QUEUED;
					break;
				case "com.ibm.rqm.executionframework.common.requeststate.taken":
					msg = Constants.getMessage("msgReqRunning");
					icon = ImageCache.REQ_RUNNING;
					break;
				case "com.ibm.rqm.executionframework.common.requeststate.cancelled":
					msg = Constants.getMessage("msgReqCancelled");
					icon = ImageCache.REQ_CANCELLED;
					break;
				case "com.ibm.rqm.executionframework.common.requeststate.complete":
					msg = Constants.getMessage("msgReqCompleted");
					icon = ImageCache.REQ_COMPLETED;
					break;
			}
			if(getMsg){
				return msg
			}else if(getIcon){
				return icon;
			}
			return null;
		},
		
		_convertLongToDate: function(/*accepts long*/longDate,/*boolean*/returnAsDate){
			var date = new Date();
			//subtract time zone offset; January is month #0; year base is perhaps 1900
			date.setTime(longDate);
			if(!returnAsDate){
				return dojo.date.locale.format(date, {formatLength: "medium"});
			}else{
				var dateFormat = {selector: "date", formatLength: "medium"};
				return dojo.date.locale.format(date, dateFormat);
			}
		},
		
		_getYearOffset: function (){
			dateNow = new Date();
			if (dateNow.getYear() < 1900)
				return 1900;
			else
				return 0;
		},
		
		_convertDateToLong: function (/*accepts Date*/date) {
			output = date.getTime();
			return output;
		},

		// returns the local-specific time zone offset in milliseconds, ready to be
		// added to the long values
		_getTimeZoneOffset: function()
		{
			dateNow = new Date();
			tzo = dateNow.getTimezoneOffset();	//minutes!
			return (tzo * 60 * 1000);
		},

		getTimeDifferenceInHrMinSec: function(duration){
			if(duration < 0)
				duration = 0;
			if(duration == 0)
				return Constants.getMessage("labelNumberSec", duration); 
			var time = duration / 1000;
			var hours = Math.floor(parseFloat((time / 3600), 10));
			var minutes = Math.floor(parseFloat((time % 3600)/60), 10);
			var seconds = Math.ceil(parseFloat((time % 60)), 10);
			var testDuration = "";
			if (hours != 0) {
				testDuration += Constants.getMessage("labelNumberHr", hours);
			}
			if (minutes != 0) {
				testDuration += Constants.getMessage("labelNumberMin", minutes);
			}

			if (seconds != 0) {
				testDuration += Constants.getMessage("labelNumberSec", seconds);
			}
			return testDuration;
		},
        /**
         * Don't know why we need a slightly modified copy of utility.js here from the planning.web plugin.
         * This is a work around copy as well from utility.js in the planning.web plugin.
         */
        getMessage: function(keyword, tokens) {
            var MessageBundleQM = "RQM";
            var args = [MessageBundleQM];
			var length = arguments.length;
            for( var i = 0; i < length; i++){ 
                args.push(arguments[i]);
            }
            return MessageUtil.getMessage.apply(MessageUtil, args);
        },
		
		_restoreTags: function(value) {
			var rawText= value;
			rawText= rawText.replace(/\&gt;/gi,">");
			rawText= rawText.replace(/\&lt;/gi,"<");
			rawText= rawText.replace(/\&amp;/gi,"&");
			rawText= rawText.replace(/\&tilde;/gi,"~");
			rawText= rawText.replace(/\&quot;/gi,"\"");
			rawText= rawText.replace(/\&circ;/gi,"^");
			rawText= rawText.replace(/\&nbsp;/gi," ");
			rawText= rawText.replace(/<br\/>/gi,"\n");
			rawText= rawText.replace(/\&#09;/g, "\t");
			return rawText;
		},
		
		getStepsHaveVerdict: function(){
			return this._stepsHaveVerdict;
		},

		setStepsHaveVerdict: function(stepsHaveVerdict){
			this._stepsHaveVerdict = stepsHaveVerdict;
		},
		
		getExecutionPreferences: function(){
			var util = com.ibm.rqm.execution.web.ui.internal.util.Utility;
			return util.executionPreferences;
		},
		
		initExecutionPreference: function(callback){
			console.debug("Utility.initExecutionPreference()");
			var util = com.ibm.rqm.execution.web.ui.internal.util.Utility;
			var handler = {
				success: function(executionPref){
					console.debug("Loaded ExecutionPreferences..");
					util.executionPreferences = executionPref && executionPref.value ? dojo.fromJson(executionPref.value) : {};
					if(callback)callback();
				},
				failure: function(){
					util.executionPreferences = {}
					if(callback)callback();
					console.error("Unable to load execution pref");
				}
			}
			var option = new com.ibm.asq.common.web.client.internal.option();
			var handler = new ServiceResponseHandler(handler, "success", "failure");
			option.getProjectPreferences(handler, getCurrentProjectArea().itemId, Constants.EXECUTION_SYSTEM_CONSTRAINTS);
		}
	};
}
)();
