/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
var Utility = dojo.provide("com.ibm.rational.test.lm.sequencer.web.ui.internal.util.Utility");
/*
 * author: vasrivas
 */
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("jazz.core.loader");
dojo.require("com.ibm.rational.test.lm.sequencer.web.ui.internal.Constants");
(function() 
{
	var ASQUtility					= com.ibm.asq.common.web.ui.internal.util.Utility;
    var ACTION_PREFIX   			= "#action=com.ibm.rqm.planning.home.actionDispatcher";
	var Constants					= com.ibm.rational.test.lm.sequencer.web.ui.internal.Constants;
	
	com.ibm.rational.test.lm.sequencer.web.ui.internal.util.Utility = 
	{
			PLANNING_PERSPECTIVE: 1,
			CONSTRUCTION_PERSPECTIVE: 2,
			EXECUTION_PERSPECTIVE: 3,

		_planningPrefix: ACTION_PREFIX + "&subAction=",
		_constructionPrefix: ACTION_PREFIX + "&subAction=",
		_executionPrefix: ACTION_PREFIX + "&subAction=",
		
		
		/******************************************************************************************
		 * Generic URI functions
		 * 
		 *****************************************************************************************/
		_getPlanningActionUri: function(action, kwArgs)
		{
			var str = this._planningPrefix + action;
			return(this._getActionUri(str, kwArgs));
		},
		
		_getConstructionActionUri: function(action, kwArgs)
		{
			var str = this._constructionPrefix + action;
			return(this._getActionUri(str, kwArgs));
		},
		
		_getExecutionActionUri: function(action, kwArgs)
		{
			var str = this._executionPrefix + action;
			return(this._getActionUri(str, kwArgs));
		},
		
		_getActionUri: function(str, kwArgs) 
		{
		    for (var key in kwArgs) 
			{
	        	str += "&" + key + "=" + kwArgs[key];
	        }
			return str;
		},

		_buildUri: function(method, args, perspective)
		{
			var result;
			
			switch(perspective)
			{
				case this.PLANNING_PERSPECTIVE:
					result = this._getPlanningActionUri(method, args);
					break;
				case this.CONSTRUCTION_PERSPECTIVE:
					result = this._getConstructionActionUri(method, args);
					break;
				case this.EXECUTION_PERSPECTIVE:
					result = this._getExecutionActionUri(method, args);
					break;
			}
			
			return(result);
		},
		
		loadModules: function(modsNeeded, callback){
			var modsNeededLength = modsNeeded.length;
			var modulesToLoad = [];
			for(var i=0;i<modsNeededLength;i++){
				if(!dojo.getObject(modsNeeded[i])){
					console.debug("adding Module for load: "+modsNeeded[i]);
					modulesToLoad.push(modsNeeded[i]);
				}
			}
			if(modulesToLoad.length >0){
				console.debug("batch loading ");
				if(!callback)
					callback = dojo.hitch(this, "_modulesLoaded")
				jazz.core.loader.batch_load_async(modulesToLoad, callback);
			}else if(dojo.isFunction(callback))
					callback();
		},
		
		_modulesLoaded: function(){
			console.debug("Modules loaded in Utility");
		},
		
		/******************************************************************************************
		 * Action URI methods
		 * 
		 * @param {Object} id
		 *****************************************************************************************/
		getSequenceURI: function(id){
			var args = {};
			if (id != null)	{
				args = {sequenceId: id};
			}
			return this._getExecutionActionUri("viewSequence", args);
		},
		
		getSequenceResultsURI: function(id){
			var args = {};
			if (id != null)	{
				args = {sequenceId: id};
			}
			return this._getExecutionActionUri("viewSequenceResults", args);
		},
		
		getAllSequenceResultsURI: function(){
			var args = {};
			return this._getExecutionActionUri("viewAllSequenceResults", args);
		},
		
		getSequenceResultURI: function(resultItemId){
			var args = {itemId: resultItemId};
			return this._getExecutionActionUri("viewSequenceResult", args);
		},

		showSuiteResult: function(itemId){
			var args = {};
			if(itemId != null)
				args.itemId = itemId;
			return this._getExecutionActionUri("showSuiteResult", args);
		},
		
		getResultURI: function(itemId){
			var args = {};
			if(itemId != null)
				args.resultItemId = itemId;
			return this._getExecutionActionUri("viewResult", args);
		},
		getResultString:function(resultState){
			switch(resultState){
				case "IN_PROGRESS":
					return Constants.getMessage("labelInProgress");
				case "CANCELLED":
					return Constants.getMessage("labelCancelledSequence");
				case "NOTSTARTED":
					return Constants.getMessage("labelNotStarted");	
				case "PAUSED":
					return Constants.getMessage("labelCancelledSequence");
				case "COMPLETE":
					return Constants.getMessage("labelComplete");	
				case "PASS":
					return Constants.getRQMMessage("labelPass");
				case "INCONCLUSIVE":
					return Constants.getRQMMessage("labelInconclusive");	
				case "FAIL":
					return Constants.getRQMMessage("labelFail");	
				case "ERROR":
					return Constants.getRQMMessage("labelError");
				case "BLOCKED":
					return Constants.getRQMMessage("labelBlocked");	
				case "UNKNOWN":
					return Constants.getRQMMessage("labelUnknown");	
				default:
					return resultState;
			}			 
		}
};
}
)();