/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.reports.web.ui.internal.view.ReportsTableView");
 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

dojo.require("com.ibm.rqm.reports.web.ui.internal.util.ReportUtil");
dojo.require("com.ibm.rqm.reports.cognos.web.ui.internal.UploadCognosReport");

(function() {

var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
var ReportIcon = dojo.moduleUrl("com.ibm.rqm.reports.web", "ui/internal/images/report-obj-16.gif");
var ReportUtil = com.ibm.rqm.reports.web.ui.internal.util.ReportUtil;
var Constants = com.ibm.rqm.planning.web.client.internal.Constants;

dojo.declare("ReportsTableView.TableListener", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener,
{
    constructor: function(parentViewer)
    {
        this.parentViewer = parentViewer;
    },
    
    // Summary:
    //   If the table's rows are checkable, when a row is checked, or unchecked, this method is
    //   called.
    //
    rowCheckStateChanged: function(tableViewer, rowObject, state)
    {
		//enable/disable actions based on state
        var actionButtons = tableViewer.getActionButtons();
        for (var i = 0; i < actionButtons.length; i++) {
            var actionObj = actionButtons[i].actionObject;
            if (actionObj && actionObj.requiresSelection == true) {
                actionButtons[i].setDisabled(!state);
            }
        }
        if(this.parentViewer && this.parentViewer.onRowSelected && dojo.isFunction(this.parentViewer.onRowSelected)) {
            this.parentViewer.onRowSelected();
        }
    },
    
    // Summary:
    //    If the client have registered table actions, this message will be fired when an action (button)
    //    is invoked by the user.
    //
    tableContentActionInvoked: function(tableViewer, actionId, callbackData, callback)
    {                
        if (callback != null && dojo.isFunction(callback)) {
            if (callbackData != null) {
                callback(callbackData);
            } else {
                callback();
            }
        } 
    }    
});

dojo.declare("ReportsTableView.CellModifier", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier,
{
    // Summary:
    //    Method to be called when a cell renderer changes the cell value. This notifies
    //    the cell's value change event. 
    //    
    //    
    cellValueChanged: function(oldValue, newValue, rowObject, columnIndex)
    {
        alert("cell value changed on object: " + rowObject.col1 + ", new Value: " + newValue);
        
        // store the new values in our internal data model
        if(columnIndex == 6) {
            if(dojo.isString(newValue)) {
                rowObject.selection = newValue;
            }
        }
    },
    
    cellActionInvoked: function(rowObject, columnIndex)
    {
		//handle delete action
		if (columnIndex == 6) {
		   alert("Not yet supported.");
		}
    }
});

dojo.declare("ReportsTableView.ListRowRenderer",
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer,
{    
    // Summary:
    //   Override the base list row renderer's create list content method
    //   to supply more custom contents.
    //
    _createListContents: function(viewer, rowObject, listWidget)
    {
        this.inherited("_createListContents", arguments);   
        
        var moreContents = document.createElement("div");
        var toggle = document.createElement("button");
        toggle.innerHTML = Constants.getMessage("labelAddedCustomControl");
        moreContents.appendChild(toggle);
        listWidget.contentNode.appendChild(moreContents);
    }
});

dojo.declare("ReportsTableView.TableProvider", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider,
{
	
	constructor: function(args) {
		dojo.mixin(this, args);
		this.COLUMNS = {
            TYPE: this.toColumn("", 2, false, TableConstants.ALIGN_CENTER, TableConstants.RENDER_TYPE_IMAGE),
			THEME: this.toColumn(Constants.getMessage("labelTheme"), 20, true, TableConstants.ALIGN_LEFT, TableConstants.RENDER_TYPE_TEXT),
            NAME: this.toColumn(Constants.getMessage("labelName"), 30, false, TableConstants.ALIGN_LEFT, TableConstants.RENDER_TYPE_LINK),
			DESCRIPTION: this.toColumn(Constants.getMessage("labelDescription"), 45, false, TableConstants.ALIGN_LEFT, TableConstants.RENDER_TYPE_TEXT),
			SHARED: this.toColumn(Constants.getMessage("labelShared"), 3, true, TableConstants.ALIGN_CENTER, TableConstants.RENDER_TYPE_TEXT)
        }	   
        this.COLUMNS.ALL = [this.COLUMNS.TYPE, this.COLUMNS.THEME, this.COLUMNS.NAME, this.COLUMNS.DESCRIPTION, this.COLUMNS.SHARED];  	   
	},
	
    toColumn: function(name, width, groupable, alignment, type) {
        return {
            name: name,
            width: width,
            groupable: groupable,
            alignment: alignment,
            type: type
        }
    },	

    getColumnIndex: function(column) {
        for (var i = 0; i < this.COLUMNS.ALL.length; i++) {
			if (this.COLUMNS.ALL[i].name == column.name) {
				return i;
			}
		}
		return -1;
    },
	
    // Summary:
    //
    getNumColumns: function(content)
    {
        return this.COLUMNS.ALL.length;
    },
    
    // Summary:
    //
    getColumnHeading: function(content, index)
    {
        return this.COLUMNS.ALL[index].name;
    },

    // Summary:
    //    Optionally, returns an image src URL to be placed in the column header. If both heading
    //    and image are specified, both are shown.
    //    
    getColumnHeadingImage: function(content, atColumnIndex)
    {
        return null;
    },
	 
    // Summary:
    //    Returns the column header horizontal alignment. By default, the columns are
    //    aligned to the left.
    //
    //    Client optionally can indicate the alignment for the column header.
    //
    getColumnHeadingAlignment: function(atColumnIndex)
    {
        return this.COLUMNS.ALL[atColumnIndex].alignment;
    },	 
    
    getIsColumnGroupable: function(content, atColumnIndex)
    {
        return this.COLUMNS.ALL[atColumnIndex].groupable;     
    },

    getIsColumnSortable: function(content, atColumnIndex)
    {
        return this.COLUMNS.ALL[atColumnIndex].name != this.COLUMNS.TYPE.name;     
    },
    
    getColumnWidth: function(content, atColumnIndex)
    {
        return this.COLUMNS.ALL[atColumnIndex].width;
    },
    
    // Summary:
    //    The only assumptions the table have on the data that it displays
    //    is that it's an array contents. These are the data content/object that
    //    will be displayed in the table rows.
    //
    getContentObjects: function(content)
    {
        // our content is already an array of objects
        return content;
    },
    
    getRowDataId: function(rowData)
    {
        return "_table_row_for_report_" + rowData.reportUUID;
    },
    
    // Summary:
    //    Returns the type of the cell to be displayed. By default, the TableViewer
    //    will support a set of cell types. The type of the cell will map to a specific
    //    cell renderer (to be discussed).
    getCellDisplayType: function(rowData, atColumnIndex)
    {
        return this.COLUMNS.ALL[atColumnIndex].type;
    },

    getCellAlignment: function(rowData, atColumnIndex)
    {
        return this.COLUMNS.ALL[atColumnIndex].alignment;
    },
    
    // Summary:
    //    Returns the display data of the cell. For cell renderer type of TEXT, this will
    //    be the string. Others cell render type would be Date, or integer value, etc. If the 
    //    cell renderer type is image, the client can also supply cell data that can be sortable/groupable. 
    //    The TableViewer's default sorter, filter operates on this returned data. For "complex" 
    //    cell types such as image where it could have multiple values for different states, the 
    //    client also needs to implement the next method.
    //    
    //    Client must implement this method to provide the cell data.
    //    
    getCellDisplayData: function(rowData, atColumnIndex)
    {
        var val = "data";
        switch(this.COLUMNS.ALL[atColumnIndex].name) {
            case this.COLUMNS.TYPE.name: 
                val = "";
                break;
            case this.COLUMNS.NAME.name: 
                val = rowData.name;
                break;
            case this.COLUMNS.DESCRIPTION.name: 
                val = rowData.description;             
                break;
			case this.COLUMNS.THEME.name: 
                val = rowData.theme;
                break;
            case this.COLUMNS.SHARED.name: 
                val = rowData.shared ? Constants.getMessage("labelYes") : Constants.getMessage("labelNo");
                break;				
        }
//		console.log("column: "  + atColumnIndex + " " + val);
        return val;
    },

    
    // Summary:
    //    If applicable, returns the extended display information of the cell. The return extended
    //    info must be an object with the following attributes:
    //    
    //    {
    //      /* attribute common for all cell renderer types */
    //      "tooltip": <string>,
    //
    //      /* attributes for IMAGE and ACTION cell renderer types */
    //      "isHidden": <true, false> the image is hidden if the row is not on the mouse over
    //      "image": base image URL,  
    //      "mouseOverImage": image URL, (optional)
    //      "mouseOutImage": image URL,  (optional) (overrides "image")
    //
    //      /* attributes for SELECT cell renderer type */
    //      "select" : 'value', the selection in the combo
    //      "choices" : [{'text':'display text', 'value':'value'}, ] array of objects with 'text' and 'value' attributes
    //
    //      /* attributes for CHECKBOX cell renderer type */
    //      "state" : <true, false> initial state of the check box
    //
    //      /* attributes for LINK cell renderer type */
    //      "href" : <string> the href string for the link
    //    }
    //
    //    Client can implement this method to provide extended display infor for the cell.
    //    Certain cell renderer type will require the client to implement this method to
    //    provide needed display information, such as an image ACTION.
    //
    getCellDisplayExtendedInfo: function(rowData, atColumnIndex)
    {  
        var val = {};
        switch(this.COLUMNS.ALL[atColumnIndex].name) {
            case this.COLUMNS.TYPE.name: 
                val.image = ReportIcon;
                break;
            case this.COLUMNS.NAME.name: 
                val.href = ReportUtil.getReportURI(rowData);
                break;
            case this.COLUMNS.DESCRIPTION.name: 
            case this.COLUMNS.THEME.name: 
            case this.COLUMNS.SHARED.name: 
                break;              
        }
        return val;
    } 
    
});

dojo.declare("com.ibm.rqm.reports.web.ui.internal.view.ReportsTableView", 
    [dijit._Widget, dijit._Templated],
{    
    templateString: '<div class="com-ibm-rqm-reports-table-view" tabindex="-1">' +
					'   <div dojoAttachPoint="_tableViewerNode"></div>' +
				    '</div>',

    editable: true,
    widgetsInTemplate: false,
	labelCurrentQuery: Constants.getMessage("labelCurrentQuery"),
	
    constructor: function() {
	 	this.provider = null;
		this.reports = null;		
		this.refreshCallback = null;
        this.onRowSelected = null;
	 },

    postCreate: function() 
    {
        this.inherited("postCreate", arguments);
		          
        //content and title				  
        var content = this.reports ? this.reports : [];
        
		/*
		if (this.queryName) {
            this._queryName.innerHTML = this.queryName;
            dojo.removeClass(this._querySection, "hidden");
        } else {
            dojo.addClass(this._querySection, "hidden");
        }
		*/
		//set up table widget
        this.provider = new ReportsTableView.TableProvider();
		var viewerArgs = {
            groupByColumn: this.provider.getColumnIndex(this.provider.COLUMNS.THEME),
            allowAdjustSettings: true,
            autoSaveSettings: true,
            tableId: "RQM-REPORT-TABLEVIEWER"                             
            
		}
        var tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer(viewerArgs);        
        this._tableViewerNode.appendChild(tableViewer.domNode);
	    
        tableViewer.setContent(content);
        tableViewer.setContentProvider(this.provider);
        tableViewer.setTableListener(new ReportsTableView.TableListener(this));
        tableViewer.setCellModifier(new ReportsTableView.CellModifier());
        tableViewer.setUnspecifiedGroupTitle(Constants.getMessage("labelUnassigned"));
		tableViewer.setTableSummary(Constants.getMessage("labelTableSummary", this.queryName ? this.queryName : Constants.getMessage("labelReports")));
		
        if (this.refreshCallback && dojo.isFunction(this.refreshCallback)) {
            tableViewer.addAction("refresh", Constants.getMessage("labelRefreshReports"), this, "refresh-icon-image", this.refreshCallback);
        }
        
        tableViewer.addAction("reportimport", Constants.getMessage("labelReportImport"), this, "add-icon-image", dojo.hitch(this, this._onImportReportClick));
        
                   
        for (var i = 0; i < (this.actions ? this.actions.length : 0); i++) {
            var action = this.actions[i];
            var button = tableViewer.addAction(action.label, action.label, null, action.iconClass, action.actionHandler);
			if (action.requiresSelection) {
				button.setDisabled(true);
				tableViewer.setTableRowsCheckable(true, /* isSingleSelect */ true);
			}
			button.actionObject = action;
        }
        tableViewer.show();
        tableViewer.setEditable(this.editable);
		  
		this.tableViewer = tableViewer;
    },
    
    getAction: function(id) {
        for (var i = 0; i < (this.actions ? this.actions.length : 0); i++) {
            if (this.actions[i].label == id) {
                return this.actions[i];
            }
        }
    },
    
    _onImportReportClick: function() {            
    	new com.ibm.rqm.reports.cognos.web.ui.internal.UploadCognosReport( {reportsTableView: this } );   
    },
    
     // API
     getSelected: function() {
         return this.tableViewer.getCheckedRows();
     },
     
     refresh: function(content) {
         if (content) {
		 	this.tableViewer.setContent(content);
            
            // reloading the content will clear the selection, disable the actions
            var actionButtons = this.tableViewer.getActionButtons();
            for (var i = 0; i < actionButtons.length; i++) {
                var actionObj = actionButtons[i].actionObject;
                if (actionObj && actionObj.requiresSelection == true) {
                    actionButtons[i].setDisabled(true);
                }
            }
		 }
         this.tableViewer.refresh();   
     }
});

})();
