/*HomePageView***************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.reports.web.ui.internal.view.ReportsView");

dojo.require("dojox.data.dom");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.team.repository.web.client.init.InitializationData");
dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");

dojo.require("com.ibm.rqm.reports.web.ui.internal.util.ReportUtil");
dojo.require("com.ibm.rqm.reports.web.ui.internal.view.ReportsTableView");
dojo.require("com.ibm.rqm.reports.web.ui.internal.view.SaveReport");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");

(function()
{   
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	var InitializationData = com.ibm.team.repository.web.client.init.InitializationData;
	
	var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
	var ReportsTableView = com.ibm.rqm.reports.web.ui.internal.view.ReportsTableView;
	var ReportUtil = com.ibm.rqm.reports.web.ui.internal.util.ReportUtil;
	var ViewHeaderLayout = com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout; 
	var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	var REPORT_SYSTEM_FOLDER = "System";
	
	dojo.declare("com.ibm.rqm.reports.web.ui.internal.view.ReportsView",  [dijit._Widget, dijit._Templated], 
	{        		           
		templateString: '<div class="com-ibm-rqm-reports">' +
	                    '    <div dojoAttachPoint="viewHeaderNode"></div>' +
                        '    <div dojoAttachPoint="_reportsTableNode"></div>' +
                        '</div>',
			
		constructor: function() {
			this._view = null;
            this._canDeployQuery = false;
            this._canManageFolder = false;
			this._folders = null;
			this._queries = null;
			this._reports = null;			
		},
		
		// initialization      
		postCreate: function() {
	        var headerArg = {
				title: Constants.getMessage("labelViewReports"),
	            contextHelpId: "rqm_view_reports.html"
	        };
			this.viewHeader = new com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout(headerArg);
			this.viewHeader.setCSHelpButtonLabel(Constants.getMessage("labelReportsHelp"));
			this.viewHeaderNode.appendChild( this.viewHeader.domNode );

            this.viewHeader.setProgressStart(Constants.getMessage("labelLoadingReports"));
			
			//load the reports	
			this.initialize();		
		},
		
        initialize: function(){
            this._loadReportArtifacts();            		  
		}, 
		
        setView: function(view) {
            this._view = view;
            this._updateView();
        },		
		
		_updateView: function() {
            if (this._view && this._view.controlButton) {
                //apply the tab class
                dojo.addClass(this._view.controlButton.containerNode, "report-icon-tab");
                //TODO update the tab label
                //this._view.controlButton.setLabel(name);
            }
		},
		
	    _userIsAdmin: function() {
	       var permissionInitializer = InitializationData.get("com.ibm.team.repository.service.canGrantPermissionInitializer");
	       return (permissionInitializer != null && permissionInitializer.canGrantPermission);
	    },		
			
	   _loadReportArtifacts: function() {
            //reset the cached report artifacts
			this._folders = null;
			this._queries = null;
			this._reports = null;
			
            var args = {
                // Project Area will be added by ReportsClient
                flags: ReportsClient.ALL
            };
			
			var folderArgs = {
                // Project Area will be added by ReportsClient
                flags: ReportsClient.SHARED
            };
			
            var foldersHandler = new ServiceResponseHandler(this, "_setSharedFolders", "_error");
            ReportsClient.getFolderDescriptors(foldersHandler, folderArgs);

            var queriesHandler = new ServiceResponseHandler(this, "_setQueries", "_error");
            ReportsClient.getQueryDescriptors(queriesHandler, args);

            var reportsHandler = new ServiceResponseHandler(this, "_setReports", "_error");
            ReportsClient.getReportDescriptors(reportsHandler, args);			
        },
		
        _isInitialized: function() {
            return (this._folders != null && this._queries != null && this._reports != null);
        },		
		
		_setSharedFolders: function(folders) {
			console.debug(this.declaredClass + ":_setSharedFolders --> entry.");
			this._folders = folders ? folders : [];
			var folderArgs = {
                // Project Area will be added by ReportsClient
                flags: 4,  // private folder
				ownerUUID: com.ibm.rqm.planning.web.ui.internal.util.EditorUtil.getCurrentContributor().itemId
            };
			var foldersHandler = new ServiceResponseHandler(this, "_setFolders", "_error");
            ReportsClient.getFolderDescriptors(foldersHandler, folderArgs);
		},
		
		_setFolders: function(privateFolders) {
			console.debug(this.declaredClass + ":_setFolders --> entry.");
			for (var i = 0; i < (privateFolders ? privateFolders.length : 0); i++) {
				this._folders.push(privateFolders[i]);
			} 
			if (this._isInitialized()) {
				this._showReportTable();
			}		
		},
        
		_setQueries: function(queries) {
			console.debug(this.declaredClass + ":_setQueries --> entry.");
            this._queries = queries ? queries : [];
            if (this._isInitialized()) {
                this._showReportTable();
            }
        },
		
        _setReports: function(reports) {
            console.debug(this.declaredClass + ":_setReports --> entry.");
            this._reports = reports ? reports : [];
            if (this._isInitialized()) {
                this._showReportTable();
            }
        },		
		
		/*
		 * Create or Refresh the ReportsTableView widget 
		 */
		_showReportTable: function() {
			console.debug(this.declaredClass + ":_showReportTable --> entry.");
			this._organizedReports = this._organizeReports();
            if (!this._reportsTable) {
				dojox.data.dom.removeChildren(this._reportsTableNode);
				var tArgs = {
					queryName: Constants.getMessage("labelReportsTitle"),
					reports: this._organizedReports,
					actions: this._getReportTableActions(),
					refreshCallback: dojo.hitch(this, "_loadReportArtifacts"),
                    onRowSelected: dojo.hitch(this, function(){
                        this.viewHeader.clearStatus();
                    })
				}
				this._reportsTable = new ReportsTableView(tArgs);				
                this._reportsTableNode.appendChild(this._reportsTable.domNode);			     
			} else {
				this._reportsTable.refresh(this._organizedReports);
			}
            this.viewHeader.setProgressEnd();
		},
		
        _getReportTableActions: function(){
            var actions = [];
            actions.push({
                actionId: "edit",
                label: Constants.getMessage("labelEditReport"),
                actionHandler: dojo.hitch(this, "_showEditReport"),
                iconClass: "edit-icon-image",
				requiresSelection: true
            });			        
            return actions;
        },
		
        _showEditReport: function() {
			var selected = this._reportsTable.getSelected();
            if (!dojo.isArray(selected) || selected.length == 0) {
                alert(Constants.getMessage("msgReportMissing", Constants.getMessage("labelEdit")));
                return;
            } else if (selected.length > 1) {
                alert(Constants.getMessage("msgReportSingle", Constants.getMessage("labelEdit")));
                return;
            }	
			var report = selected[0];		
            var saveWidget = new com.ibm.rqm.reports.web.ui.internal.view.SaveReport({
				isExistingReport: true,
				onOk: dojo.hitch(this, "_saveReportOk"),
				onError: dojo.hitch(this, "_saveReportError"),
				title: Constants.getMessage("labelEditReport"),
                reportUUID: report.reportUUID,
                queryUUID: report.queryUUID           
            });
        },
		
        _saveReportOk: function(QueryUUID)
        {
            this._loadReportArtifacts();            
        },
		
		_saveReportError: function(message)
		{
            // The save report action has error. Show it to the user. 
            if(message) {
                this.viewHeader.setStatus(Constants.getMessage("msgErrorModifyReport"), Constants.HEADER_STATUS_ERROR, message);
            }

		},
        
		_showReport: function() {			
            var selected = this._reportsTable ? this._reportsTable.getSelected() : [];
			if (!selected || selected.length == 0) {
				alert(Constants.getMessage("msgSelectReport"));
				return;
			} else if (selected.length > 1) {
				alert(Constants.getMessage("msgSelectSingleReport"));
				return;
			}
			window.location = ReportUtil.getReportURI(selected[0]);
		},

        _findQueryForReport: function(reportUUID) {
            //find the query for the given reportUUID
            for (var i = 0; i < (this._queries ? this._queries.length : 0); i++) {
                if (this._queries[i].reportUUID == reportUUID) {
					return this._queries[i];
				}
            }
			return null;
        },	
		
        _findReport: function(reportUUID) {
            //find the report for the given queryUUID
            for (var i = 0; i < (this._reports ? this._reports.length : 0); i++) {
                if (this._reports[i].reportUUID == reportUUID) {
                    return this._reports[i];
                }
            }
            return null;
        },			
		
		_organizeReports: function() {
			//add the theme (folder) information to the report dtos
			var organized = [];
			var allReports = this._getAllReports();
			for (var i = 0; i < (allReports ? allReports.length : 0); i++) {				
				var report = allReports[i];
				var folder = report.folderUUID ? this._getFolder(report.folderUUID) : null;
				//only show system reports (non categorized reports / no folder) to admin users
				if (folder || this._userIsAdmin()) {  
                    report.theme = folder ? this._getFolderDisplayName(folder) : Constants.getMessage("labelSystem");
				    report.shared = folder ? folder.shared : true;
	                
                    //if its a query, get the reportId
	                if (!report.reportId) {
	                    var foundReport = this._findReport(report.reportUUID);
	                    if (foundReport != null) {
	                        report.reportId = foundReport.reportId;
	                    }
	                }
			    	organized.push(report); 
				}
			}
			return organized;
		},
		
		_getAllReports: function() {
			var allReports = this._queries.concat();  //clones the array
			for (var i = 0; i < this._reports.length; i++) {
				if (!this._contains(this._queries, this._reports[i], 'reportUUID')) {
					allReports.push(this._reports[i]);
				}
			}
			return allReports;
		},	
		
        _getFolder: function(folderUUID) {
            for (var i = 0; i < (this._folders ? this._folders.length : 0); i++) {
                if (this._folders[i].folderUUID == folderUUID) {
                    return this._folders[i];
                }
            }
            return null;
        },		
		
		_getFolderDisplayName: function(folder) {
			if (folder) {
				var ancestors = [folder];
				while (folder.parentUUID != null) {
					folder = this._getFolder(folder.parentUUID);
					ancestors.push(folder);
				}
				var name = "";
				for (var i = (ancestors.length - 1); i >= 0; i--) {
				    name += ancestors[i].name + (i != 0 ? "/" : "");
				}
				return name;
			}
			return null;
		},
		
        _contains: function(array, value, propertyName) {
            for (var i = 0; i < array.length; i++) {
                if (array[i][propertyName] == value[propertyName]) {
                    return true;
                }
            }
            return false;
        },		
		
		_error: function(e) {
            this.viewHeader.setProgressEnd();
            this.viewHeader.setStatus(Constants.getMessage("labelReportsLoadingFailed"),
                                      Constants.HEADER_STATUS_ERROR, e.message);
			console.error("ReportsView: Error: " + e.message);
		}	

	});
})();
