/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.administration.AdministrationEditor");

dojo.require("com.ibm.asq.common.web.client.internal.option");

dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.AbstractPropertyEditor");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.BuildProviderContentArea");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.ReqproHostnameContentArea");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.ResourceEditorContentArea");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.ReportPropertiesContentArea");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.ReportServerContentArea");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.CPUNumberContentArea");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.StateTransitionConstraintContentArea");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.ExecutionStatesContentArea");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.CategoryAdminContentArea");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.QualityObjectivesContentArea");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.RelatedLinksContentArea");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.ManualScriptConverterDictionaryContentArea");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.ExecutionSystemPreferenceContentArea");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.ArtifactOptionsContentArea");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.CatalogEditor");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.WhitchCatalogEditor");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.RiskProfilesContentArea");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.RisksContentArea");

dojo.require("com.ibm.rqm.planning.web.client.internal.PreferenceClient");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.util.GlobalEvents");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");

(function(){
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    var Constants 		= com.ibm.rqm.planning.web.client.internal.Constants;
	var AlertDialog 	= com.ibm.asq.common.web.ui.internal.widgets.AlertDialog;
	var EditorUtil		= com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
	var GlobalEvents    = com.ibm.asq.common.web.ui.internal.util.GlobalEvents;
	var ViewHeaderLayout = com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout;
    var PermissionsClient = com.ibm.asq.common.web.client.internal.PermissionsClient;
    var Administration    = com.ibm.rqm.planning.web.ui.internal.view.administration;
    
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.AdministrationEditor", com.ibm.rqm.planning.web.ui.internal.view.administration.AbstractPropertyEditor, {
        
		dirty: false,
		savingParticipant: 0,
		
		constructor: function(){
            this.headerTitle = Constants.getMessage("labelAdministration");
            this.defaultProperty = Constants.getMessage("labelRequisiteProHostProperties");
            this.editable = true;
            this._checkPermission();
        },
		
		postCreate: function(){
//			var time = "timing postCreate AdministrationEditor";
//			console.time(time);
			this.inherited("postCreate", arguments);
			this._initHeader();
//			console.timeEnd(time);
		},

        isEditable: function() {
            return this.editable;  
        },
        
        _checkPermission: function() {
            var permission = PermissionsClient.getStoredPermission(PermissionsClient.OP_ADMIN_OPTIONS);                
            if (permission) {
                this.editable = permission[PermissionsClient.ACTION_EDIT];
            }
        },
		
		_initHeader: function(){
			var viewHeaderNode = this.viewHeaderNode;
			dojox.data.dom.removeChildren(viewHeaderNode.getRightActionsContainer());
			dojox.data.dom.removeChildren(viewHeaderNode.getRightActionsBottomContainer());
			var cancelButton = viewHeaderNode.addRightViewAction(Constants.getMessage("labelDiscardChanges"), dojo.hitch(this, "onCancel"));
			cancelButton.setDisabled(true);
			this._cancelButton = cancelButton;
			var saveButton = viewHeaderNode.addRightViewAction(Constants.getMessage("labelSave"), dojo.hitch(this, "onSave"));
			saveButton.setDisabled(true);
			this._saveButton = saveButton;

            if(!this.isEditable()) {
                var text = Constants.getCommonMessage("statusNoEditPerm");
                var details = Constants.getCommonMessage("msgNoEditPermDetails", Constants.getMessage("labelSystemProperties"));
                viewHeaderNode.setStatus(text, ViewHeaderLayout.WARNING, details);    
            }
            
		},
        
        setNameContentPairs: function(){
			var widgetArgs = {parent: this};
			
			this._nameContentPairs = [];
			this._addContent(Constants.getMessage("labelArtifactOptions"), Administration.ArtifactOptionsContentArea, widgetArgs);
			this._addContent(Constants.getMessage("labelBuildProvider"), Administration.BuildProviderContentArea);
			this._addContent(Constants.getMessage("labelRequisiteProHostProperties"), Administration.ReqproHostnameContentArea, widgetArgs);
			this._addContent(Constants.getMessage("labelResources"), Administration.ResourceEditorContentArea, widgetArgs);
			this._addContent(Constants.getMessage("labelReportProperties"), Administration.ReportPropertiesContentArea, widgetArgs);
			this._addContent(Constants.getMessage("labelReportServer"), Administration.ReportServerContentArea, widgetArgs);
			this._addContent(Constants.getMessage("labelCPUNumber"), Administration.CPUNumberContentArea, widgetArgs);
			this._addContent(Constants.getMessage("labelStateTransitionConstraint"), Administration.StateTransitionConstraintContentArea, widgetArgs);
			this._addContent(Constants.getMessage("labelExecutionStates"), Administration.ExecutionStatesContentArea, widgetArgs);
			this._addContent(Constants.getMessage("labelQualityObjectives"), Administration.QualityObjectivesContentArea, widgetArgs);
			this._addContent(Constants.getMessage("labelTypeCategories", Constants.TYPE_PLAN), Administration.CategoryAdminContentArea, {parent: this, artifactType: Constants.MODEL_TYPE_PLAN, csHelpId:"rqm_admin_tp_cat.html"});
			this._addContent(Constants.getMessage("labelTypeCategories", Constants.TYPE_CASE), Administration.CategoryAdminContentArea, {parent: this, artifactType: Constants.MODEL_TYPE_CASE, csHelpId:"rqm_admin_tc_cat.html"});
			this._addContent(Constants.getMessage("labelCustomizeCatalog"), Administration.CatalogEditor, widgetArgs);
            this._addContent(Constants.getMessage("labelWhitchCustomizeCatalog"), Administration.WhitchCatalogEditor, widgetArgs);
			this._addContent(Constants.getMessage("labelExecutionPrefs"), Administration.ExecutionSystemPreferenceContentArea, widgetArgs);
			this._addContent(Constants.getMessage("labelManualScriptDictionary"), Administration.ManualScriptConverterDictionaryContentArea, widgetArgs);
			this._addContent(Constants.getMessage("labelRelatedSites"), Administration.RelatedLinksContentArea, widgetArgs);
			this._addContent(Constants.getMessage("labelRiskProfiles"), Administration.RiskProfilesContentArea, widgetArgs);
			this._addContent(Constants.getMessage("labelRisks"), Administration.RisksContentArea, widgetArgs);
        },
		
		_addContent: function(name, jsClass, widgetArgs) {
			this._nameContentPairs.push({name: name, content: null, jsClass: jsClass, widgetArgs: widgetArgs});
		},
        
        _onOK: function(){
            this._getProjectPreference();
        },
		
		_getProjectPreference: function(){
			var keyValues = this.getCurrentContentArea().getValues();
            var self = this;
            for (var i = 0; i < keyValues.length; i++) {
				var keyValue = keyValues[i];
				var handler = {
	                success: function(property){
	                    if(property == null){
							self._createProjectPreference(keyValue.key, keyValue.value);
						}else{
							self._updateProjectPreference(property.itemId, keyValue.value);
						}
	                },
	                
	                error: function(e){
	                    console.debug(e);
	                }
	            }
	            var responseHandler = new ServiceResponseHandler(handler, "success", "error");
				
				var option = new com.ibm.asq.common.web.client.internal.option();
				option.getProjectPreferences(responseHandler, com.ibm.team.process.web.projectArea.itemId, keyValues[i].key);
            }
		},
		
		_createProjectPreference: function(key, value){
            var responseHandler = new ServiceResponseHandler(this, "_savedSuccess", "_error");
			
			var option = new com.ibm.asq.common.web.client.internal.option();
			option.createProjectPreferences(responseHandler, com.ibm.team.process.web.projectArea.itemId, key, dojo.toJson(value));
		},
		
		_updateProjectPreference: function(preferenceItemId, value){
            var responseHandler = new ServiceResponseHandler(this, "_savedSuccess", "_error");
			
			var option = new com.ibm.asq.common.web.client.internal.option();
			option.updateProjectPreferences(responseHandler, com.ibm.team.process.web.projectArea.itemId, preferenceItemId, dojo.toJson(value));
		},
		
		_savedSuccess: function(){
			var alertDialog = new AlertDialog({
                'message': Constants.getMessage("msgSavedSuccessfully")
            });
            alertDialog.setSecondaryTitle("");
            alertDialog.open();
			
			//if we are updating the artifact constraints then reload the cache
			if (this.getCurrentContentArea().declaredClass == "com.ibm.rqm.planning.web.ui.internal.view.administration.StateTransitionConstraintContentArea")
				EditorUtil.loadStateTransitionConstraintSettings(null, true);
		},
		
		setDirty: function(){
            if(!this.isEditable())
                return;
                
			var _dirty = false;
			var dirtyContentNames = "";
			var nameContentPairs = this._nameContentPairs;
			var length = nameContentPairs.length;
			for(var i = 0; i < length; i++){
				var nameContentPair = nameContentPairs[i];
				var content = nameContentPair.content;
				if(content && content.isDirty()){
					_dirty = true;
					dirtyContentNames += nameContentPair.name + ",";
				}
			}
			dirtyContentNames = dirtyContentNames.substring(0, dirtyContentNames.length - 1);
			
			if(this.isDirty() == false && _dirty == true){
				dojo.publish(GlobalEvents.UpdateTabLabel, [window.location.hash, "", null, true]);
			}else if(this.isDirty() == true && _dirty == false){
				dojo.publish(GlobalEvents.UpdateTabLabel, [window.location.hash, "", null, false]);
			}
			this.dirty = _dirty;
			if(this.dirty){
				this.viewHeaderNode.setStatus(Constants.getCommonMessage("statusDirty"), ViewHeaderLayout.INFO, Constants.getMessage("labelUnsavedChanges", dirtyContentNames));
				this._saveButton.setDisabled(false);
				this._cancelButton.setDisabled(false);
			}else{
				this.viewHeaderNode.clearStatus();
				this._saveButton.setDisabled(true);
				this._cancelButton.setDisabled(true);
			}
		},
		
		isDirty: function(){
			return this.dirty;
		},
		
		onCancel: function(){
			//set current property as default
			this.defaultProperty = this.getName();
            
            // clean up existing widgets first, before recreating them.
            this._destroyAllcontentsWidgets();
			this.postCreate();
			this.setDirty();
			//set it back
			this.defaultProperty = Constants.getMessage("labelRequisiteProHostProperties");
		},
		
		onSave: function(){
			//clean Header Status: 
			this.viewHeaderNode.clearStatus();
			
			//validation
			var nameContentPairs = this._nameContentPairs;
			var length = nameContentPairs.length;
			for(var i = 0; i < length; i++){
				// only need to validate when the content is dirty
				var content = nameContentPairs[i].content;
				if(content && content.isDirty && !content.isDirty()) {
			         continue;
				}
				if(content && content.onValidate){
					var result = content.onValidate();
					if(result.pass == false){
						this.viewHeaderNode.setStatus(result.message, ViewHeaderLayout.WARNING);
						return;
					}
				}
			}
			
			//invoke the global save transaction     
			this._cancelButton.setDisabled(true);
			this._saveButton.setDisabled(true);
			
			this.viewHeaderNode.setProgressStart(Constants.getMessage("labelSavingArtifact", this.headerTitle));	
			
			for(var i = 0; i < length; i++){
                // only need to save when the content is dirty
				var content = nameContentPairs[i].content;
                if(content && content.isDirty && !content.isDirty()) {
                     continue;
                }
				if (content && content.onSave) {
					content.onSave();
				}
			}
		},
		
		addSaveParticipantNum: function() {
			this.savingParticipant++;
		},
		
		saveFinished: function() {
			this.savingParticipant--;
			if(this.savingParticipant == 0) {
				this.viewHeaderNode.setProgressEnd();
				this.setDirty();
			}
		},
		
        onActivated: function(){
            var currContent = this._currentContentArea;
            if(currContent && currContent.needRefreshOnActivate) {
                currContent.refresh();
            } 
        },
        
		_error: function(e){
			console.debug(e);
		}
    });
})();
