//BPC
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.administration.BuildProviderContentArea");

dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("jazz.core.loader");
dojo.require("com.ibm.team.repository.web.foundation.Status");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.AbstractContentArea");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");
dojo.require("com.ibm.team.repository.web.foundation.NullProgressMonitor");

(function(){

var Status							= com.ibm.team.repository.web.foundation.Status;
var Constants 						= com.ibm.rqm.planning.web.client.internal.Constants;
var AlertDialog 					= com.ibm.asq.common.web.ui.internal.widgets.AlertDialog;
var NullProgressMonitor				= com.ibm.team.repository.web.foundation.NullProgressMonitor;
var getCurrentProjectArea			= com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;

/*
 * FIXME this administartion page is not planning related and must be contributed by the defects plugin 
 */
dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.BuildProviderContentArea", 
			 [com.ibm.rqm.planning.web.ui.internal.view.administration.AbstractContentArea], {

	csHelpId: "buildprovider_admin_prop.html",
	_contentWidget: null,

    setName: function(){
		this._setTextNls("labelBuildProvider", this._name);
    },
    
    setText: function(){
		this._setTextNls("labelBuildProviderText", this._text);
    },
    
    setContent: function() {
		this._contentWidget= new com.ibm.rqm.planning.web.ui.internal.view.administration.BuildProviderContentArea._Content({}, this._content);
    },

    getValues: function(){
		return this._contentWidget.getValues();
    },
    
    isDirty: function() {
    	return false;
    },

	_setTextNls: function(labelKey, node) {
        dojox.data.dom.removeChildren(node);
        node.appendChild(document.createTextNode(Constants.getMessage(labelKey)));
	},

	__sentinel: null // terminates this class definition
});

dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.BuildProviderContentArea._Content",
			 [dijit._Widget, dijit._Templated], { 

	labelLoading: Constants.getMessage("labelLoading"),
	labelBuildProvider: Constants.getMessage("labelBuildProvider"),
	labelProvider: Constants.getMessage("labelProvider"),
	labelSummaryWithColon: Constants.getMessage("labelSummaryWithColon"),
	labelStatusWithColon: Constants.getMessage("labelStatusWithColon"),
	labelConfigure: Constants.getMessage("labelConfigure"),
	lableClear: Constants.getMessage("lableClear"),
	_contentWrapper: null,
	_provider: null,
	_serverSummary: null,
	_serverDetails: null,
	_statusIcon: null,
	_statusMessage: null,
	_configureButton: null,
	_currentProvider: null,
	
	constructor: function(args){
	},

	getValues: function() {
		return [];
	}, 

	_rtcIconSrc: dojo.moduleUrl("com.ibm.rqm.planning.web", "RTC_logo.png"),
	templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/administrator/BuildProviderContentArea.html"),

	postCreate: function() {
		var className = "com.ibm.rqm.buildintegration.web.client.internal.build.BuildsManager";
		var loaded = function(){
			this._loadOptions();
			this.connect(this._configureButton, "onclick", this._configureClicked);
			this.connect(this._provider, "onchange", this._providerChanged);
			this.connect(this._deleteButton, "onclick", this._deleteClicked);
		};
		jazz.core.loader.load_async("com.ibm.rqm.buildintegration.web.ui.internal.wizard.WizardDialog", function(){});
        jazz.core.loader.load_async(className, dojo.hitch(this, loaded));
	},

	_loadOptions: function() {
		var buildsManager = com.ibm.rqm.buildintegration.web.client.internal.build.BuildsManager.getInstance();
		buildsManager.ensureInitialized(new NullProgressMonitor(), dojo.hitch(this, function(status) {
			
			var buildsProviders= buildsManager.getBuildProviders();
			
			if (buildsProviders) {
				for (var providerId in buildsProviders) {
					var providerName= buildsProviders[providerId].name;
					var providerOption= document.createElement("option");
					providerOption.appendChild(document.createTextNode(providerName));
					providerOption.setAttribute("value", providerId);
					if (providerId == buildsManager.getCurrentBuildProviderId()) {
						providerOption.setAttribute("selected", "selected");
					}
					this._provider.appendChild(providerOption);
				}
				this._updateConnectionState(buildsManager.getBuildProvider(buildsManager.getCurrentBuildProviderId()));
			} else {
                var providerOption = document.createElement("option");
                providerOption.appendChild(document.createTextNode(Constants.getMessage("labelNoneProvidersAvailable")));
                providerOption.setAttribute("value", "");
                providerOption.setAttribute("selected", "selected");
                this._provider.appendChild(providerOption);
				this._configureButton.setAttribute("disabled", "");
			}
			dojo.removeClass(this._container, "loading");
			dojo.addClass(this._container, "loaded");
			this._isLoading= false;
		}));
	},

	_updateConnectionState: function(buildsProvider) {
		if (!buildsProvider)
			return; 
		this._setText(this._serverSummary, buildsProvider.getConnectionName());
		this._setText(this._serverDetails, buildsProvider.getConnectionDetails());
		this._checkProviderStatus(buildsProvider, true);
		this._currentProvider= buildsProvider;
	},
	
	_checkProviderStatus: function(buildsProvider, cancelIfNotLoggedIn) {
		this._status.className= "checking";
		this._setText(this._statusMessage, Constants.getMessage("labelChecking"));
		dojo.toggleClass(this._configureButton, "enabled", buildsProvider.needsConfiguration());
        buildsProvider.checkProviderStatus(dojo.hitch(this, function(status){
            this._checkProviderStatusHandler(status, buildsProvider)
        }), cancelIfNotLoggedIn);
	},
	
	_checkProviderStatusHandler: function(status, buildsProvider) {
		var statusClass= "ok";
		var statusMessage= Constants.getMessage("labelOk");
		if (status.getSeverity() == Status.CANCEL) {
			var statusClass= "unconfirmed";
			var statusMessage= Constants.getMessage("labelClickToVerify");
			var handle= this.connect(this._statusMessage, "onclick", function(evt) {
				evt.stopPropagation();
				this.disconnect(handle);
				this._checkProviderStatus(buildsProvider, false);
			});
		} else if (!status.isOK()) {
			statusClass= "error";
			statusMessage= status.getMessage();
		}
		this._status.className= statusClass;
		this._setText(this._statusMessage, statusMessage);
	},
	
	_setText: function(node, text) {
		dojox.data.dom.removeChildren(node);
		if (text != null) {
			node.appendChild(document.createTextNode(text));
		}
	},

	_configureClicked: function(evt) {
		var provider= this._currentProvider;
		if (provider != null && provider.needsConfiguration()) {
			var wizardDialog= com.ibm.rqm.buildintegration.web.ui.internal.wizard.WizardDialog.create(provider.getConfigurationWizard());
	        this.connect(wizardDialog, "onFinish", dojo.hitch(this, function(){
	           this._updateConnectionState(provider);
	        }));
		}
	},
	
	_providerChanged: function(evt) {
		var buildsManager= com.ibm.rqm.buildintegration.web.client.internal.build.BuildsManager.getInstance();
		buildsManager.ensureInitialized(new NullProgressMonitor(), dojo.hitch(this, function(status) {
			var buildProvider= buildsManager.getBuildProvider(this._provider.value);
			this._updateConnectionState(buildProvider);
		}));
	},

	_deleteClicked: function(evt){
		var provider= this._currentProvider;
		if (provider != null && provider.isConfigured()) {
			provider.clearConnectionDetails(dojo.hitch(this, function(status, response) {
					var buildsManager = com.ibm.rqm.buildintegration.web.client.internal.build.BuildsManager.getInstance();
					buildsManager.setUninitialized(); 
					this._providerChanged();
			}));
		}
	},

	__sentinel: null // terminates this class definition
});

})();
