/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.administration.CategoriesEditor");

dojo.require("dojox.data.dom");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.util.GlobalEvents");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.Button");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");

dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

dojo.require("com.ibm.asq.common.web.ui.internal.util.SelectUtility" );
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");

dojo.require("com.ibm.rqm.planning.web.client.internal.CategoryClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.rqm.planning.web.ui.internal.model.SaveParticipant");
dojo.require("com.ibm.rqm.planning.web.ui.internal.model.SaveContext");


(function(){
	var PermissionsClient = com.ibm.asq.common.web.client.internal.PermissionsClient;
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    var CSHelp = com.ibm.asq.common.web.ui.internal.util.CSHelp;
    var GlobalEvents = com.ibm.asq.common.web.ui.internal.util.GlobalEvents;
	var ImageCache = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;

   	var SelectUtility = dojo.getObject( "com.ibm.asq.common.web.ui.internal.util.SelectUtility" );

	var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	var EditorUtil = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
	var SaveParticipant = com.ibm.rqm.planning.web.ui.internal.model.SaveParticipant;
	var SaveContext = com.ibm.rqm.planning.web.ui.internal.model.SaveContext;
	var CategoryClient = com.ibm.rqm.planning.web.client.internal.CategoryClient;
	var UNASSIGNED = Constants.getMessage("labelUnassigned");
    var UNKNOWN = Constants.getMessage("labelUnknown");
	
	dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.CategoriesEditor",
	[dijit._Widget, dijit._Templated],
	{
        //		
	    // Provides dynamic selection of Categories 
	    //
	    templateString: '<div class="com-ibm-rqm-categories-editor">' +
		                '  <div dojoAttachPoint="_loadingNode" class="hidden">${labelLoading}</div>' +
                        '  <div class="action-option-bar">' +
                        '      <div class="actions" dojoAttachPoint="_actionsNode"></div>' +
						'      <div class="clear"></div>' +
						'  </div>' +
						'  <div dojoAttachPoint="_categoryTypesNode" class="category-types"></div>' +
						'  <div class="options">' +
						'      <input name="show-archived-checkbox" id="show-archived-checkbox" type="checkbox" dojoAttachPoint="_showArchivedCheckbox" dojoAttachEvent="onclick: _onArchivedClick"></input>' +
						'      <label for="show-archived-checkbox" class="" dojoAttachPoint="_showArchivedLabel"></label>' +
						'  </div>' +	                
		                '</div>',
		
        dialogWidth: "650px",
		nonDialogWidth: "425px",   //Smaller to fit in 1024x768

		constructor: function() {
			this.labelLoading = Constants.getMessage("labelLoading");
			this.isDialog = true;
			this.artifactType = null;
            this.categoryTypes = [];
			this.onClose = null;  //optional callback fired on editor close
			this.onSaved = null;  //optional callback fired on successful save
			this._editors = [];  //CategoryTypeEditor cache
			this._showArchived = false;			
			this._permissions = {};
			this._actionButtons = {};
			this._contributors = [];			
		},		
	    
	    postCreate: function() {
			//ensure there is an artifact type specified
			if (!this.artifactType) {
                throw new Error("Parameter 'artifactType' is REQUIRED!");
            }
			
			//show the 'archived' option
			this._showArchivedLabel.innerHTML = Constants.getMessage("labelShowArchivedCategories");	
            if (!this.isDialog) {
				this.domNode.style.width = this.nonDialogWidth;
			}
			this._newCategoryTypeEditors = [];
			this._newCategoryEditors = [];
			//determine the current user's permissions
            var operations = [];
            operations.push({
                id: PermissionsClient.OP_CATEGORY_TYPE, 
                actions: PermissionsClient.DEFAULT_ACTIONS
            });
            operations.push({
                id: PermissionsClient.OP_CATEGORY, 
                actions: PermissionsClient.DEFAULT_ACTIONS
            });
			
			var responseHandler = new ServiceResponseHandler(this, "_preinitPermissions", "_error");
            PermissionsClient.checkAllPermissions(responseHandler, operations);				
	    },
		
		_preinitPermissions: function(permissions) {
			this._permissions = permissions;			
	        EditorUtil.getContributors(this, "_preinitContributors", "_error");  			
		},
		
        _preinitContributors: function(contributors) {
            this._contributors = contributors;
            this.initialize();          
        },		

        initialize: function() {			
			this._showActions();
            this._showArchivedCheckbox.checked = this._showArchived;
			dojox.data.dom.removeChildren(this._categoryTypesNode);
			dojo.removeClass(this._loadingNode, "hidden");
			
			//retrieve the categoryTypes for display/edit			
			var self = this;
			var handlerObj = {
				success: function(categoryTypes) {
                    dojo.addClass(self._loadingNode, "hidden");
					self.categoryTypes = dojo.isArray(categoryTypes) ? categoryTypes : [];
					self.showCategoryTypes();
					//if this is being shown as a dialog, init the dialog
		            if (self.isDialog && !self._dialog) {
		                self.show();
		            }
					if(self.doPublish){
						//publish event 
						var type;
						if (self.getArtifactType() == Constants.MODEL_TYPE_CASE) {
			                type = Constants.TYPE_TC_CATEGORY;
						} else if (self.getArtifactType() == Constants.MODEL_TYPE_PLAN) {
			                type = Constants.TYPE_TP_CATEGORY;
			            }
			            if (type) {
			                dojo.publish(GlobalEvents.ArtifactsUpdated, [type, self.categoryTypes]);
			            }
						self.doPublish = false;
					}
				},
				
				failure: function(e) {
					self.doPublish = false;
					self._error(e);
				}
			}
            var responseHandler = new ServiceResponseHandler(handlerObj, "success", "failure");
            CategoryClient.getCategoryTypes(responseHandler, this.artifactType, this._showArchived);
        },
		
        show: function() {		
            if (!this._dialog) {           
                var dialogArgs = {                    
					primaryTitle: Constants.getMessage("labelManageCategories", this._getTypeDisplayName()),
					isClosable: true,
					okCallback: dojo.hitch(this, "doSave"),
					cancelCallback: dojo.hitch(this, "hide"),
					showActions: true,
					_truncate: function() {}  //disable the Dialog's '_truncate'
                };                
                this._dialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(dialogArgs);
				var csHelp = null;
				if (this.artifactType == Constants.MODEL_TYPE_CASE) {
	                csHelp = new CSHelp({contextID: "rqm_manage_tc_categories.html"});
				} else if (this.artifactType == Constants.MODEL_TYPE_PLAN){
    	            csHelp = new CSHelp({contextID: "rqm_manage_tp_categories.html"});
				}
                csHelp.setButtonLabel(Constants.getMessage("labelManageCategoriesCSHelp"));
                this._dialog.setSecondaryTitleNode(csHelp.domNode);
				
                this._dialog.setWidth(this.dialogWidth);
                this._dialog.contentContainer.appendChild(this.domNode);                     
            }
            this._dialog.open();    						
		},		
		
		hide: function() {
			//fire onClose callback, if specified
			if (dojo.isFunction(this.onClose)) {
				this.onClose(this.categoryTypes);
			}
            // we could be called from OK, which does not close the dialog
            // while we save the content. Close the dialog if it's 
            // still open.
            if (this._dialog && this._dialog.opened) {
                this._dialog.close();
            }
			return;
		},
	
		showCategoryTypes: function() {
			this._editors = [];
			dojox.data.dom.removeChildren(this._categoryTypesNode);	
			var categoryTypes = this.categoryTypes;
			var length = categoryTypes.length;
			var newCategoryEditors = this._newCategoryEditors;
			var newCategoryTypeEditors = this._newCategoryTypeEditors;		
			for (var i = 0; i < length; i++) {
				var categoryType = categoryTypes[i];
				var editor = new  com.ibm.rqm.planning.web.ui.internal.view.administration.CategoryTypeEditor({
					parent: this,
					index: this._editors.length,
					categoryType: categoryType
				});
				this._editors.push(editor);   //cache the editor
				this._categoryTypesNode.appendChild(editor.domNode);    //show the editor
				var newCategoryEditor = newCategoryEditors[categoryType.itemId];
				if(newCategoryEditor){
					var length = newCategoryEditor.length;
					for(var k=0;k<length;k++){
						var categoryEditor = newCategoryEditor[k];
						if(!categoryType.categories)
							categoryType.categories = [];
						editor._editors.push(categoryEditor);
						categoryType.categories.push(categoryEditor.getCategory());
						editor._categoryTypesNode.appendChild(categoryEditor.domNode);
					}
				}
			}
			if(newCategoryTypeEditors) {
				var length = newCategoryTypeEditors.length;
				if(length){
					for(var j=0;j<length;j++){
						var newCategoryTypeEditor = newCategoryTypeEditors[j];
						this._editors.push(newCategoryTypeEditor);
						this._categoryTypesNode.appendChild(newCategoryTypeEditor.domNode);
					}
				}
			}
		},
		
        addCategoryType: function(newCategoryType) {
            this.categoryTypes.push(newCategoryType);
        },		
		
		_getTypeDisplayName: function() {
            if (this.artifactType == Constants.MODEL_TYPE_PLAN) {
                return Constants.TYPE_PLAN;
            } else if (this.artifactType == Constants.MODEL_TYPE_CASE) {
                return Constants.TYPE_CASE;
            }
			return "";
        },
		
        _showActions: function() {
            dojox.data.dom.removeChildren(this._actionsNode);
            var actions = this._getActions();
			var length = actions ? actions.length : 0;
            for (var i = 0; i < length; i++) {
                this._addAction(actions[i]);   
            }            
        },  		
		
        _getActions: function() {
            var actions = []; 
            if (this.hasCategoryTypePermission(PermissionsClient.ACTION_CREATE)) {
                actions.push({
					id: "new", //$NON-NLS$
                    label: Constants.getMessage("labelAddCategoryType"),
                    actionHandler: dojo.hitch(this, "_handleAddNew"),
                    enabledIcon: ImageCache.ADD_ICON_ENABLED,
                    disabledIcon: ImageCache.ADD_ICON_ENABLED
                });
            }
            return actions; 
        },      
        
        _addAction: function(action) {                
            var hover = action.disabledIcon && action.enabledIcon;
            var button = document.createElement("img");
            dojo.addClass(button, "action-button");
            button.src = hover ? action.disabledIcon : action.enabledIcon;
            button.title = action.label;
			button.tabIndex = "0";
			this.connect(button, "onclick", function(event){
				dojo.stopEvent(event);
				action.actionHandler(event);
			});
			this.connect(button, "onkeypress", function(e){
//				var charCode = (e.which) ? e.which : e.keyCode
				if (e.keyCode == dojo.keys.ENTER) {
					dojo.stopEvent(e);
					action.actionHandler(e);
				}
			});
            if (hover) {
            	this.connect(button, "onmouseover", function() { button.src = action.enabledIcon; });
                this.connect(button, "onmouseout", function() { button.src = action.disabledIcon; });
            }			
			
            this._actionsNode.appendChild(button);					
        },
					
        //////////////////////////////////////////////////////////////////////////////////////////
        ///                                        API                                         ///
        //////////////////////////////////////////////////////////////////////////////////////////		
		isShowArchived: function() {
			return this._showArchived;
		},    
		
        hasPermission: function(operationId, actionId) {
            var permission = this._permissions[operationId];
            return (permission && permission[actionId] && permission[actionId] == true);         
        },		
		
		hasCategoryTypePermission: function(actionId) {
			return this.hasPermission(PermissionsClient.OP_CATEGORY_TYPE, actionId);
		},

        hasCategoryPermission: function(actionId) {
            return this.hasPermission(PermissionsClient.OP_CATEGORY, actionId);
        },
		
        getCategoryType: function(name) {
			var editors = this._editors;
			var length = editors ? editors.length : 0;
            for (var i = 0; i < length; i++) {
                var categoryType = editors[i].getCategoryType();
                if (categoryType.name.toUpperCase() == name.toUpperCase()) {
                    return categoryType;
                }            
            }
            return null;
        },		
		
		getArtifactType: function() {
			return this.artifactType;
		},	
		
		getContributors: function() {
			return this._contributors;
		},
		
        removeEditor: function(editor) {
            if (editor) {
                this._categoryTypesNode.removeChild(editor.domNode);
                this._editors.splice(editor.index, 1);
                editor.destroy();
            }
        },
		
		isDirty: function() {
			return this._dirty;
		},
		
		setDirty: function(dirty) {
			this._dirty = dirty;
			if (this._dialog) {
				this._dialog.getOkButton().disabled = !dirty;
			}
		},		
		
		doSave: function() {
			console.debug("doSave: entry.");
			//init save context
			var context = new SaveContext();
			
			//for each CategoryType editor
			var editors = this._editors;
			var typeLength = editors.length;
			for (var i = 0; i < typeLength; i++) {
				//add save participant to context (if its dirty)
				var typeEditor = editors[i];
				var typeAdded = false;
				if (typeEditor.isDirty()) {
					typeAdded = true;
					context.addParticipant(typeEditor);
				}
				//for each Category editor
				var categoryEditors = typeEditor.getEditors() || [];
				var catLength = categoryEditors.length;
				for (var j = 0; j < catLength; j++) {
					//add dirty save participants to context
					var categoryEditor = categoryEditors[j];
					if (categoryEditor.isDirty()) {
						context.addParticipant(categoryEditor);
						//create new reference if this is a new category item within a new category type item
						if (categoryEditor.isNew() && typeEditor.isNew()) {
							//add relationship to parent Category Type	
							context.createReference(categoryEditor, "typeItemId", typeEditor);
						}
					}
				}
				
			}
			
			//commit the save context
			context.setOnSaveFinished(dojo.hitch(this, "_saveFinished"));
			var status = context.commit();
			
			//if return code from commit != SAVING, return false so dialog stays open
			if (status != Constants.STATUS_SAVING) {
				//TODO add some message
				alert("Bad Status from SaveContext.commit() ==> " + status);
			}
			return false;
		},
		
		//INTERNALS
		
		_saveFinished: function(status) {
			this._newCategoryTypeEditors = [];
			this._newCategoryEditors = [];
			if (this._dialog) {
				this._updateCategories();
			} else {
				this.doPublish = true;
				this.initialize();
			}		
			//fire onSaved callback, if specified
			if (dojo.isFunction(this.onSaved)) {
				this.onSaved(status);
			}
		},
		_updateCategories:function(){
			var self = this;
			var handlerObj = {
				success: function(categoryTypes) {
					self.categoryTypes = dojo.isArray(categoryTypes) ? categoryTypes : [];
					//publish event 
					var type;
					if (self.getArtifactType() == Constants.MODEL_TYPE_CASE) {
		                type = Constants.TYPE_TC_CATEGORY;
					} else if (self.getArtifactType() == Constants.MODEL_TYPE_PLAN) {
		                type = Constants.TYPE_TP_CATEGORY;
		            }
		            if (type) {
		                dojo.publish(GlobalEvents.ArtifactsUpdated, [type, self.categoryTypes]);
		            }
					self.hide();
				},
				
				failure: function(e) {
					self._error(e);
					self.hide();
				}
			}
            var responseHandler = new ServiceResponseHandler(handlerObj, "success", "failure");
            CategoryClient.getCategoryTypes(responseHandler, this.artifactType, this._showArchived);
		},
		
		_onArchivedClick: function(event) {						
			//reinit the view, changing the archive option			
			if (this._showArchived != this._showArchivedCheckbox.checked) {
				this._showArchived = this._showArchivedCheckbox.checked;
				this.initialize();
			}
		},
		
        _handleAddNew: function() {
			//protect against concurrent adds
			if (this._isAdding) {
				return;
			} else {
				this._isAdding = true;
			}

            var newNum = 1;
            var newName = Constants.getMessage("labelNewCategoryType");            
            while (this.getCategoryType(newName) != null) {
                newName = Constants.getMessage("labelNewCategoryType") + " (" + newNum + ")"; 
				newNum++;
            }
            var editor = new  com.ibm.rqm.planning.web.ui.internal.view.administration.CategoryTypeEditor({
				parent: this,
				index: this._editors.length,
                categoryTypeName: newName
            });			
            this._editors.push(editor);   //cache the editor
            this._newCategoryTypeEditors.push(editor);
            this._categoryTypesNode.appendChild(editor.domNode);    //show the editor
            editor.setInitialFocus();
			
            //this._actionButtons["new"].setDisabled(false);	      
			this._isAdding = false;
        },
		
		_capFirst: function(str) {
			var value = str + "";
			return value.charAt(0).toUpperCase() + value.substring(1).toLowerCase()
		},
		
        _sortValues: function(val1, val2) {
            if (val1.name < val2.name) {
				return -1;
			} else if (val1.name > val2.name) {
				return 1;
			}
			return 0;
        },		
		
        _error: function(e) {
            console.error(this.declaredClass + ": Error: " + e.message);
        }
	});  
    
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.CategoryTypeEditor", [dijit._Widget, dijit._Templated, SaveParticipant],
    {                   
        templateString: '<table dojoAttachPoint="_categoryTypeEditorNode" class="category-type-editor-table">' +
		                '    <tr class="category-type-row" dojoAttachPoint="_categoryTypeEditorRow" dojoAttachEvent="onclick:_handleRowClick">' + 
		                '        <th dojoAttachPoint="_nameNode" class="content-column">' + 
                        '            <span dojoAttachPoint="_nameText"></span>' +
                        '            <input dojoAttachPoint="_name" dojoAttachEvent="onblur:_doEdit,onkeyup:_doEditKeypress" type="text" size="${nameInputSize}" maxlength="${nameInputMax}" class="hidden"></input>' +						
						'        </th>' +
						'        <th dojoAttachPoint="_ownerNode" class="content-column">${labelOwner}</th>' +
						'        <th dojoAttachPoint="_actionsNode" class="actions-column"></th>' +
		                '    </tr>' +  
						'    <tbody dojoAttachPoint="_categoryTypesNode">' +
		                '        <!-- individual category editor rows will be appended here -->' +
						'    </tbody>' +
						'</table>',	
						
	    labelOwner: Constants.getMessage("labelOwner"),
		nameInputSize: 25,			
        nameInputMax: 50,
				
		constructor: function() {
			this.parent = null;
			this.categoryType = null;
			this.editable = false;
			this.inEditMode = false;
			this._editors = [];  //CategoryEditor array; index is rowNum , element is editor instance
		},
		
		postCreate: function() {
			if (!this.categoryType) {
				if (!this.categoryTypeName) {
					throw new Error("Parameter 'categoryTypeName' REQUIRED for new CategoryTypes!");
				}
				this.inEditMode = true;
				this.categoryType = {
					newCategoryType: true,
					name: this.categoryTypeName	
				}
			} else {
				//existing category type
				this.setItemId(this.categoryType.itemId);
				this.setStateId(this.categoryType.stateId);
				this.setDirty(false); //don't save unless modified
			}
			this.setItemType(Constants.MODEL_TYPE_CATEGORY_TYPE);
			
			this.editable = this.hasCategoryTypePermission(PermissionsClient.ACTION_EDIT);
			this._initialize();
		},

        _initialize: function() {
            this._showName();
            this._showCategories();
			this._showActions();
		},
		
        hasCategoryTypePermission: function(actionId) {
            return this.parent.hasCategoryTypePermission(actionId);
        },

        hasCategoryPermission: function(actionId) {
            return this.parent.hasCategoryPermission(actionId);
        },
		
		getEditors: function() {
			return this._editors;
		},	
			
        getCategoryType: function() {
            return this.categoryType;
        },		
		
		getCategories: function() {
			if (this.categoryType && this.categoryType.categories) {
				return this.categoryType.categories;
			}
			return null;
		},
		
		getCategory: function(name) {
			var editors = this._editors;
			var length = editors ? editors.length : 0;
			for (var i = 0; i < length; i++) {
                var category = editors[i].getCategory();
				if (category.name.toUpperCase() == name.toUpperCase()) {
					return category;
				}			 
			}
			return null;
		},		
		
        addCategory: function(newCategory) {
			var categoryType = this.categoryType;
            if (categoryType) {
                if (!categoryType.categories) {
                    categoryType.categories = [];
                }
                categoryType.categories.push(newCategory);
            }           
        },		
		
		getContributors: function() {
			return this.parent.getContributors();
		},
		
        isShowArchived: function() {
            return this.parent.isShowArchived();
        },	
		
		setInitialFocus: function() {
			if (this.inEditMode && this._name) {
				this._name.focus();
			}
		},
		
        isNew: function() {
            return (this.categoryType.newCategoryType == true);
        },		
		
		_showName: function() {		
            //name
			if (this.editable && this.inEditMode) {
                this._name.value = this.categoryType.name;                
                dojo.removeClass(this._name, "hidden");
                dojo.addClass(this._nameText, "hidden");				
                dojo.removeClass(this._categoryTypeEditorRow, "editable");					
            } else {
                this._nameText.innerHTML = this.categoryType.name;                
                dojo.addClass(this._name, "hidden");
                dojo.removeClass(this._nameText, "hidden");	
                dojo.addClass(this._categoryTypeEditorRow, "editable");					
            }
            dojo.toggleClass(this._nameText, "archived", this.categoryType.archived);            
		},
		
        _showActions: function() {
            dojox.data.dom.removeChildren(this._actionsNode);
            var actions = this._getActions();
			var length = actions ? actions.length : 0;
            for (var i = 0; i < length; i++) {
                this._addAction(actions[i]);   
            }            
        },		
		
        _showCategories: function() {
			if (!this.categoryType ||!this.categoryType.categories) {
				return;
			}
			//initialize the category editors and display them
			this._editors = [];
			dojox.data.dom.removeChildren(this._categoryTypesNode);	
			var categories = this.categoryType.categories;		
			categories.sort(this._sortCategories);
			var length = categories.length;
            for (var i = 0; i < length; i++) {
				//create an editor for the CategoryType
				var category = categories[i];
                if (category.archived == false || this.isShowArchived()) {
				    this._appendCategory(category);	
				}				
			}			
        },

        _appendCategory: function(category) {
            var editor = new com.ibm.rqm.planning.web.ui.internal.view.administration.CategoryEditor({
                parent: this,
				index: this._editors.length,
                category: category
            });
			this._editors.push(editor); //cache the editor
            this._categoryTypesNode.appendChild(editor.domNode);			
		},
		
		removeEditor: function(editor) {
			var index = editor.index; //this._getCategoryIndex();
			this._editors.splice(index, 1);
			this._categoryTypesNode.deleteRow(index);
		},	
		
        _sortCategories: function(val1, val2) {
            if (val1.name < val2.name) {
                return -1;
            } else if (val1.name > val2.name) {
                return 1;
            }
            return 0;
        },  	
		
        _getActions: function() {
            var actions = []; 
            if (!this.isNew() && this.hasCategoryTypePermission(PermissionsClient.ACTION_ARCHIVE)) {
	            if (!this.getCategoryType().archived) {
	                actions.push({
	                    label: Constants.getMessage("labelArchiveCategoryType"),
	                    actionHandler: dojo.hitch(this, "_doArchive"),
	                    enabledIcon: ImageCache.REMOVE_ENABLED,
	                    disabledIcon: ImageCache.REMOVE_ENABLED,
	                    disabledOnReadOnly: true
	                });
	            } else {
	                actions.push({
	                    label: Constants.getMessage("labelUnarchiveCategoryType"),
	                    actionHandler: dojo.hitch(this, "_doUnarchive"),
	                    enabledIcon: ImageCache.RESTORE_ICON_ENABLED,
	                    disabledIcon: ImageCache.RESTORE_ICON_ENABLED,
	                    disabledOnReadOnly: true
	                });             
	            }
            }			
			if (this.hasCategoryPermission(PermissionsClient.ACTION_CREATE)) {
				actions.push({
					label: Constants.getMessage("labelAddCategory", this.getCategoryType().name),
					actionHandler: dojo.hitch(this, "_handleAddNew"),
					enabledIcon: ImageCache.ADD_ICON_ENABLED,
					disabledIcon: ImageCache.ADD_ICON_ENABLED,
					disabledOnReadOnly: true
				});
			}
            return actions; 
        },		
		
        _addAction: function(action) {			
            var hover = action.disabledIcon && action.enabledIcon;
            var button = document.createElement("img");
            dojo.addClass(button, "action-button");
            button.src = hover ? action.disabledIcon : action.enabledIcon;
            button.title = action.label;
			button.tabIndex = "0";
			this.connect(button, "onkeypress",  function(e){
///\				var charCode = (e.which) ? e.which : e.keyCode
				if (e.keyCode == dojo.keys.ENTER) {
					dojo.stopEvent(e);
					action.actionHandler(e);
				}
			});
			this.connect(button, "onclick", function(event) {
				dojo.stopEvent(event);
				action.actionHandler();
			});      
			if (hover) {
				this.connect(button, "onmouseover", function() { button.src = action.enabledIcon; });
                this.connect(button, "onmouseout", function() { button.src = action.disabledIcon; });
            }
            this._actionsNode.appendChild(button);  			     
        },		
		
		_handleAddNew: function() {
            //protect against concurrent adds
            if (this._isAdding) {
                return;
            } else {
                this._isAdding = true;
            }
			
            var newNum = 1;
			var newName = Constants.getMessage("labelNewCategory", this.getCategoryType().name);			
			while (this.getCategory(newName) != null) {
				newName = Constants.getMessage("labelNewCategory", this.getCategoryType().name) + " (" + newNum + ")";
				newNum++;
			}
            var editor = new com.ibm.rqm.planning.web.ui.internal.view.administration.CategoryEditor({
                parent: this,
				index: this._editors.length,
                categoryName: newName
            });
            this._editors.push(editor); //cache the editor
            if(!this.parent._newCategoryEditors[this.getItemId()])
				this.parent._newCategoryEditors[this.getItemId()] = [];
            this.parent._newCategoryEditors[this.getItemId()].push(editor);
            this._categoryTypesNode.appendChild(editor.domNode);
			editor.setInitialFocus();
			
			this._isAdding = false;
		},
		
        _handleRowClick: function(event) {
            dojo.stopEvent(event);
            if (!this.inEditMode) {
                this._toggleEditMode();
            }
        },		
		
		_handleTitlePress:function(e) {
			if (e.keyCode == dojo.keys.ENTER) {
				dojo.stopEvent(e);
				if (!this.inEditMode) {
					this._toggleEditMode();
				}
			} 
        },		
		
        _toggleEditMode: function() {
            this.inEditMode = !this.inEditMode;
            this._showName();
			this.setInitialFocus();
        },
        
        _doEditKeypress: function(e) {       
            if (e.keyCode == dojo.keys.ENTER) {
                //perform the edit on ENTER pressed 
				this._doEdit();
				
            } else if (e.keyCode == dojo.keys.ESCAPE) {
                //on ESCAPE - cancel out if existing
                if (!this.categoryType.newCategoryType) {
                    this._toggleEditMode();
                } else {
                    //'un-create' if new 
	                this.parent.removeEditor(this);
                }              
				dojo.stopEvent(e); //prevent Dialog from closing 
            } else {
				var type = this.categoryType;
				var name = dojo.trim(this._name.value);
				if (type.newCategoryType || name != type.name) {
					this.setDirty(true);
				}
			}
        },
        
        _doEdit: function() {
			var name = this._name;
            if (name.value != null && dojo.trim(name.value).length > 0) {
                if (!this._validate()) {
                    //revert if invalid for edit
                    name.value = this.categoryType.name;
                    return;
                }
                this.categoryType.name = name.value;
				//this._save();			
				this.setDirty(true);
				if (this.inEditMode) this._toggleEditMode();	              
            } else {
                //don't allow empty name - reset name
                name.value = this.categoryType.name;
                name.focus();				
			}
        },
		
		//@Override for SaveParticipant
		setDirty: function(dirty) {
			this.inherited('setDirty', arguments);

			//otherwise, it is not dirty
			if (dirty) {
				this.parent.setDirty(dirty);
			}
		},
		
		checkChildrenDirty: function() {
			//if any of the contained editors are dirty, this is dirty
			var editors = this._editors;
			var length = editors.length;
			for (var i = 0; i < length; i++) {
				if (editors[i].isDirty()) {
					this.parent.setDirty(true);
					return;	
				}
			}		
			this.parent.setDirty(false);	
		},
		
		//@Override for SaveParticipant
		getAttributes: function() {
			var categoryType = this.categoryType;
            var attributes = {
                newCategoryType: categoryType.newCategoryType ? true : false, 
				itemType: this.parent.getArtifactType(),             
                name: categoryType.name,
				archive: categoryType.archived ? true : false             
            }
			return attributes;
		},
		
        //@Override for SaveParticipant				
        validate: function() {
			//validate the name
			if (this.inEditMode) this._doEdit();
            return this._validate();
        },
		
		_validate: function() {
			var categoryType = this.categoryType;
			var name = this.inEditMode ? this._name.value : categoryType.name;
			var existing = this.parent.getCategoryType(name);
            if (name != categoryType.name && name.toUpperCase() == categoryType.name.toUpperCase()) {
				//allow renaming of same category type with change to case only
				return true; 
			} else if (existing && existing.itemId && existing.itemId != categoryType.itemId) {
                //disallow naming/renaming of category type to already existing category type
                alert(Constants.getMessage("msgCategoryTypeExists", name));
                return false;               
            }			
			return true;
		},
        
        _showSaving: function() {
            dojox.data.dom.removeChildren(this._nameNode);
            this._nameNode.innerHTML = "Saving...";
        },
        
        _doArchive: function(event) {
			this.categoryType.archived = true;
            this._showName();
			this.setDirty(true);
			//this._save();
        },
        
        _doUnarchive: function() {
            this.categoryType.archived = false;
            this._showName();
			this.setDirty(true);
			//this._save();
        },
				
        _error: function(e) {
            console.error(this.declaredClass + ": Error: " + e.message);
        }			       
    });    
	
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.CategoryEditor", [dijit._Widget, dijit._Templated, SaveParticipant],
    {                   
        templateString: '    <tr dojoAttachPoint="_categoryEditorNode" class="category-row">' +                        
                        '        <td dojoAttachPoint="_nameNode" dojoAttachEvent="onclick:_handleRowClick" class="content-column">' +
                        '            <span dojoAttachPoint="_nameText"></span>' +
                        '            <input dojoAttachPoint="_name" dojoAttachEvent="onblur:_doEdit,onkeyup:_doEditKeypress" type="text" size="${nameInputSize}" maxlength="${nameInputMax}" class="hidden"></input>' +
                        '        </td>' +												
						'        <td dojoAttachPoint="_ownerNode" class="content-column" dojoAttachEvent="onclick:_clickOwner, onkeypress:_clickOwner">' +
						'            <span dojoAttachPoint="_ownerText"></span>' +
						'            <select name="ownerSelect" dojoAttachPoint="_owner" dojoAttachEvent="onchange:_doEditOwner,onblur:_doEditOwner,onkeypress:_doEditOwnerKeypress" class="owner-select hidden"><option value="">${labelLoading}</option></select>' +
						'        </td>' +
                        '        <td dojoAttachPoint="_actionsCell" class="actions-column">' +
						'            <div dojoAttachPoint="_actionsNode" class="hidden"></div>' +
						'        </td>' +						
                        '    </tr>',
						
		labelLoading: Constants.getMessage("labelLoading"),	
		nameInputSize: 25,			
        nameInputMax: 50,
		
        constructor: function() {
			this.parent = null;
            this.category = null;
			this.editable = false;
			this.inEditMode = false;
			this._isSaving = false;
//			this.editOwnerFinish = false;
			this.clickingOwner = false;
        },
        
        postCreate: function() {
            if (!this.parent) {
                throw new Error("Parameter 'parent' REQUIRED!");
            }
            if (!this.category) {
                if (!this.categoryName) {
                    throw new Error("Parameter 'categoryName' REQUIRED for new Categories!");
                }
				this.inEditMode = true;
                this.category = {
                    newCategory: true,
                    name: this.categoryName 
                }
			} else {
				//existing category type
				this.setItemId(this.category.itemId);
				this.setStateId(this.category.stateId);
				this.setDirty(false); //don't save unless modified
			}
			this.setItemType(Constants.MODEL_TYPE_CATEGORY);
			
			this.editable = this.parent.hasCategoryPermission(PermissionsClient.ACTION_EDIT);
			this._initialize();
        },
		
        getCategory: function() {
            return this.category;
        },
		
		isNew: function() {
			return (this.category.newCategory == true);
		},
		
        setInitialFocus: function() {
            if (this.inEditMode && this._name && !this.clickingOwner) {
                this._name.focus();
            }else if(this.clickingOwner)
				this._owner.focus();
        },		

        _initialize: function() {
            this._showInfo();
            this._showActions();
            this._setupHover();
		},
				
        _showInfo: function() {
			//if in edit mode
            if (this.editable && this.inEditMode) {
				//name
                this._name.value = this.category.name;                
                dojo.removeClass(this._name, "hidden");
                dojo.addClass(this._nameText, "hidden");
				
				//owner
                dojo.removeClass(this._owner, "hidden");
                dojo.addClass(this._ownerText, "hidden");
                dojox.data.dom.removeChildren(this._owner);
                var contributors = this.parent.getContributors();
				var length = contributors.length;
                for (var i = 0; i < length; i++) {
					var contributor = contributors[i];
					var o = new Option(contributor.name, contributor.itemId);
					var selected = null;
					if (this.category && this.category.ownerItemId && this.category.ownerItemId == contributor.itemId) {
						selected = contributor;
						o.selected = true;						
					} else if (selected == null && contributor.name == UNASSIGNED) {
						o.selected = true;
					} else if(selected == null && com.ibm.asq.common.web.ui.internal.util.Utility.isUnassignedUserId(contributor.itemId)){
						// if the server is on different locale than the client, unassigned name will be different
						// so use the item id which is fixed for unassigned user
						//defect 16837
						o.text = UNASSIGNED;
						o.selected = true;
					}
					SelectUtility.add( this._owner, o );
				}
				
				//toggle row's editable style
                dojo.removeClass(this._categoryEditorNode, "editable");
								
            } else {
				//name
//                this._nameText.innerHTML = this.category.name;
				dojox.data.dom.removeChildren(this._nameText);
				var text= document.createTextNode(this.category.name);
				this._nameText.appendChild(text);
				this._nameText.tabIndex = "0";
				this.connect(this._nameText, "onkeypress", "_handleNamePress");
				
                dojo.addClass(this._name, "hidden");
				dojo.removeClass(this._nameText, "hidden");
                if (this.category.archived) {
                    dojo.addClass(this._nameText, "archived");
                }
				else //Not archived.  Remove the style if applied
                   dojo.removeClass(this._nameText, "archived");
 				
				//owner
				if (this.category.ownerItemId) {
					var owner = this._getContributor(this.category.ownerItemId);
					this._ownerText.innerHTML = owner ? owner.name : UNKNOWN;
				} else {
					this._ownerText.innerHTML = UNASSIGNED;
				}                
                dojo.addClass(this._owner, "hidden");
				dojo.removeClass(this._ownerText, "hidden");
				
				this._ownerText.tabIndex = "0";				
                //toggle row's editable style
                dojo.addClass(this._categoryEditorNode, "editable");				 				  
            }
        },
		
		_getContributor: function(itemIdOrName) {
            var contributors = this.parent.getContributors();
			var length = contributors ? contributors.length : 0;
            for (var i = 0; i < length; i++) {
				var contributor = contributors[i];
                if (contributor.itemId == itemIdOrName) {
                    return contributor;
                } else if (contributor.name == itemIdOrName) {
					return contributor;
				}
            }
			return null;
		},
		
        _showActions: function() {
            dojox.data.dom.removeChildren(this._actionsNode);
			var actions = this._getActions();
			var length = actions.length;
			for (var i = 0; i < length; i++) {
			 	this._addAction(actions[i]);   
			}            
        },		
		
		_setupHover: function() {
			this.connect(this.domNode, "onmouseover", function() {
				dojo.removeClass(this._actionsNode, "hidden")
			});
            this.connect(this.domNode, "onmouseout", function() {
				dojo.addClass(this._actionsNode, "hidden");
			});
		},
		
		_getActions: function() {
            var actions = [];   
			if (this.category.newCategory) {
				return actions;
			}   
			if (!this.isNew() && this.parent.hasCategoryPermission(PermissionsClient.ACTION_ARCHIVE)) {
				if (!this.category.archived) {
					actions.push({
						label: Constants.getMessage("labelArchiveCategory", this.parent.getCategoryType().name),
						actionHandler: dojo.hitch(this, "_doArchive"),
						enabledIcon: ImageCache.REMOVE_ENABLED,
						disabledIcon: ImageCache.REMOVE_ENABLED,
						disabledOnReadOnly: true
					});
				}
				else {
					actions.push({
						label: Constants.getMessage("labelUnarchiveCategory", this.parent.getCategoryType().name),
						actionHandler: dojo.hitch(this, "_doUnarchive"),
						enabledIcon: ImageCache.RESTORE_ICON_ENABLED,
						disabledIcon: ImageCache.RESTORE_ICON_ENABLED,
						disabledOnReadOnly: true
					});
				}
			}
			return actions;	
		},
		
		_addAction: function(action) {
            var hover = action.disabledIcon && action.enabledIcon;
			var button = document.createElement("img");
			dojo.addClass(button, "action-button");
			button.src = hover ? action.disabledIcon : action.enabledIcon;
			button.title = action.label;
			button.tabIndex = "0";
			this.connect(button, "onkeypress",  function(e){
//				var charCode = (e.which) ? e.which : e.keyCode
				if (e.keyCode == dojo.keys.ENTER) {
					dojo.stopEvent(e);
					action.actionHandler(e);
				}
			});
			this.connect(button, "onclick", function(event) {
                dojo.stopEvent(event);
                action.actionHandler();
            });
			if (hover) {
				this.connect(button, "onmouseover",  function() { button.src = action.enabledIcon; });
				this.connect(button, "onmouseout", function() { button.src = action.disabledIcon; });
			}			
			this._actionsNode.appendChild(button);			
		},
		
        _handleRowClick: function(event) {
            dojo.stopEvent(event);
			if (!this.inEditMode) {
				this._toggleEditMode();
			}			
		},
		
		_handleNamePress: function(e) {
			if (e.keyCode == dojo.keys.ENTER) {
				dojo.stopEvent(e);
				if (!this.inEditMode) {
					this._toggleEditMode();
				}
			} 
		},
		
		_handleOwnerPress: function(e) {
			if (e.keyCode == dojo.keys.ENTER) {
				dojo.stopEvent(e);
				if (!this.inEditMode) {
					this.inEditMode = !this.inEditMode;
					this._showInfo();
					this._owner.focus();
				}
			} 
		},
		
		_toggleEditMode: function() {
			this.inEditMode = !this.inEditMode;
			this._showInfo();
			this.setInitialFocus();
		},
		
		_doEditKeypress: function(e) {
            if (e.keyCode == dojo.keys.ENTER || e.keyCode == dojo.keys.TAB) {
			    //perform the edit on ENTER or TAB pressed
			    this._doEdit();
				this._nameText.focus();
            } else if (e.keyCode == dojo.keys.ESCAPE) {
                //on ESCAPE - cancel out if existing
                if (!this.category.newCategory) {
                    this._toggleEditMode();
                } else {
                    //'un-create' if new 
                    this.parent.removeEditor(this);
                }				
				this._nameText.focus();
				dojo.stopEvent(e); //prevent Dialog from closing
			} else {
				var category = this.category;
				var name = dojo.trim(this._name.value);
				if (category.newCategory || name != category.name) {
					this.setDirty(true);
				}				
			}
		},
		
		_doEditOwnerKeypress: function(e) {
            if (e.keyCode == dojo.keys.ENTER || e.keyCode == dojo.keys.TAB) {
			    //perform the edit on ENTER or TAB pressed
			    this._doEditOwner(e);
            } else if (e.keyCode == dojo.keys.ESCAPE) {
                //on ESCAPE - cancel out if existing
                if (!this.category.newCategory) {
                    this._toggleEditMode();
                } else {
                    //'un-create' if new 
                    this.parent.removeEditor(this);
                }				
				this._nameText.focus();
				dojo.stopEvent(e); //prevent Dialog from closing
			}
		},
		
		_doEdit: function() {		
			if (this._name.value && dojo.trim(this._name.value).length > 0) {
				if (!this._validate()) {
					//revert if invalid for edit
					this._name.value = this.category.name;
					return;
                }
                this.category.name = this._name.value;
				//this._save();			
				this.setDirty(true);
				if (this.inEditMode) this._toggleEditMode();	
			} else {
                //don't allow empty name - reset name
                this._name.value = this.category.name;
                this._name.focus(); 				
			}
		},	
		
		_doEditOwner: function(e) {
			dojo.stopEvent(e);
			this.category.ownerItemId = this._owner.value;
			this.clickingOwner = false;
			this._doEdit();
		},
		
		_clickOwner: function(e) {
			this.clickingOwner = true;
			dojo.stopEvent(e);
			if (!this.inEditMode) this._toggleEditMode();
		},

		//@Override for SaveParticipant
		setDirty: function(dirty) {
			this.inherited('setDirty', arguments);
			this.parent.checkChildrenDirty();
		},
				
		//@Override for SaveParticipant
		getAttributes: function() {
            var attributes = {
                newCategory: this.category.newCategory ? true : false,              
                name: this.category.name,
				archive: this.category.archived ? true : false                  
            }
            if (this.category.itemId) {
                attributes.itemId = this.category.itemId;
            }
			if (this.isNew() && !this.parent.isNew()) {
				attributes.typeItemId = this.parent.getItemId();
			}
			if (this._owner.value != null && dojo.trim(this._owner.value).length > 0) {
				var contributor = this._getContributor(dojo.trim(this._owner.value));
				var unassignedContributor = this._getContributor(com.ibm.asq.common.web.ui.internal.util.Utility.getUnassignedUserId());
				if (contributor) {
				    var hasOwner = (this.category.ownerItemId != null);
					if ((this.isNew() || !hasOwner) && contributor.itemId == unassignedContributor.itemId) {
						//do not specify Unassigned owner for new categories or for categories that don't 
						//already have an owner.  In other words, force user to explicitly specifiy an owner. 
					} else {
						attributes.ownerItemId = contributor.itemId;
					}
				}				
			}			
			return attributes;
		},
		
        //@Override for SaveParticipant				
        validate: function() {
            //validate the name
			if (this.inEditMode) this._doEdit();
			return this._validate();
		},
		
		_validate: function() {
			var categoryName = this.category.name;
			var name = this.inEditMode ? this._name.value : categoryName;
			
			var existing = this.parent.getCategory(name);
            if (name == categoryName && name.toUpperCase() != categoryName.toUpperCase()) {
                //allow renaming of same category with change to case only
                return true; 
            } else if (existing && existing.itemId && existing.itemId != this.category.itemId) {
                //disallow naming/renaming of category to already existing category
                alert(Constants.getMessage("msgCategoryExists", this.parent.getCategoryType().name, name));
                return false;               
            }
			return true;			
		},
		
		_setSaving: function() {
			this._isSaving = true;
			dojox.data.dom.removeChildren(this._nameText);
			this._nameText.appendChild(document.createTextNode(Constants.getMessage("labelSavingArtifact")));
		},
		
		_endSaving: function() {
			this._isSaving = false;
		},
		
        _doArchive: function() {
			//check for references
			if (this.category.itemId != null) {
				//check for artifacts in approved state only
				var states = [Constants.STATETRANSITION_STATE_APPROVED];

				var handler = new ServiceResponseHandler(this, "_handleArchiveWarning", "_error");
				CategoryClient.getCategoryReferences(handler, this.parent.parent.artifactType, this.category.itemId, states);
			
			} else {
				this._performArchive();
			}
        },	
		
        _doUnarchive: function() {
            this.category.archived = false;
            this._showInfo();
			this.setDirty(true);
			//this._save();
        },					
		
		_handleArchiveWarning: function(references) {
			//if no references found, archive the category
			if (!references || references.length == 0) {
				this._performArchive();
				return;
			}
			
			//references found; warn the user that the category is in use
			var dialogArgs = {
				primaryTitle: Constants.getMessage("labelConfirmation"),
				isClosable: true,
				okCallback: dojo.hitch(this, "_performArchive"),
				cancelCallback: function() {} //archive cancelled; do nothing
			};			
		
			var fragment = document.createDocumentFragment();
			var stateName = Constants.getMessage(Constants.STATETRANSITION_STATE_APPROVED);
			var refMsg = Constants.getMessage("msgWarnCategoryInUse_1", stateName, this.parent.parent._getTypeDisplayName()) //$NON-NLS$
			fragment.appendChild(document.createTextNode(refMsg));
			fragment.appendChild(document.createElement("br"));
			fragment.appendChild(document.createElement("br"));
			var refLen = references.length;
			for (var i = 0; i < refLen; i++) {
				fragment.appendChild(document.createTextNode(references[i]));
				fragment.appendChild(document.createElement("br"));
			}	
			fragment.appendChild(document.createElement("br"));		
			fragment.appendChild(document.createTextNode(Constants.getMessage("msgWarnCategoryInUse_2"))); //$NON-NLS$
			
			var dlg = new com.ibm.asq.common.web.ui.internal.widgets.ConfirmationDialog(dialogArgs);
	 		dlg.contentContainer.appendChild(fragment);
			dlg.open();			
		},
		
		_performArchive: function() {
            this.category.archived = true;
			this._showInfo();
			this.setDirty(true);
			//this._save();			
		},		

		_error: function(e) {
			console.error(this.declaredClass + ": Error: " + e.message);
		}
		
    });		
})();


