/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.administration.CategoryAdminContentArea");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.AbstractPropertyContentArea");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.CategoriesEditor");


(function(){
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	var CategoriesEditor = com.ibm.rqm.planning.web.ui.internal.view.administration.CategoriesEditor;
	
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.CategoryAdminContentArea", com.ibm.rqm.planning.web.ui.internal.view.administration.AbstractPropertyContentArea, {
        
		artifactType: null,
		
		postCreate: function() {
			//ensure there is an artifact type specified
            if (!this.artifactType) {
                throw new Error("Parameter 'artifactType' is REQUIRED!");
            }			
			this.inherited('postCreate', arguments);
			
			this.csHelp.setButtonLabel(Constants.getMessage("labelCategoryHelp", this._getTypeDisplayName()));
		},
		
		setName: function(){
            this.name = Constants.getMessage("labelTypeCategories", this._getTypeDisplayName());
            dojox.data.dom.removeChildren(this._name);
            var name = document.createTextNode(this.name);
            this._name.appendChild(name);
        },
        
        setText: function(){
            this.text = Constants.getMessage("labelManageCategoriesText", this._getTypeDisplayName());
			dojox.data.dom.removeChildren(this._text);
            var text = document.createTextNode(this.text);
            this._text.appendChild(text);
        },
        
        setContent: function(){
			this._showEditor();
        },

        _showEditor: function() {
            dojox.data.dom.removeChildren(this._content);
            this._editor = new CategoriesEditor({
				artifactType: this.artifactType,
				isDialog: false,
				nonDialogWidth: "95%",
				onSaved: dojo.hitch(this, "_onSaveFinished")
			});
			dojo.connect(this._editor, "setDirty", this, "_setDirty");
            this._content.appendChild(this._editor.domNode); 
		},
		
		_setDirty: function() {
			this.setDirty(this._editor.isDirty());
		},
		
		_getTypeDisplayName: function() {
			if (this.artifactType == Constants.MODEL_TYPE_PLAN) {
				return Constants.TYPE_PLAN;
			} else if (this.artifactType == Constants.MODEL_TYPE_CASE) {
                return Constants.TYPE_CASE;
            }
		},
		
		onSave: function() {
			if (this._editor && this._editor.isDirty()) {
				this.parent.addSaveParticipantNum();
				this._editor.doSave();
			}
		},
		
		_onSaveFinished: function(status) {
			this.setDirty(false);
			this.parent.saveFinished();
			//append error message if necessary
			if (!status.ok) {
				this.setErrorMessage(Constants.getMessage("msgCategorySaveError"), status.errorMessage);
			}
		},
        
        getValues: function(){
        	return [];
        }
    });
})();
