/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.administration.FeedViewsContentArea");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("ibm_atom.widget.FeedViewer");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.AbstractContentArea");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");

(function(){

var Constants 						= com.ibm.rqm.planning.web.client.internal.Constants;
var YesNoDialog 					= com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog;
var AlertDialog 					= com.ibm.asq.common.web.ui.internal.widgets.AlertDialog;
var ImageCache 						= com.ibm.asq.common.web.ui.internal.view.common.ImageCache;

dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.FeedViewsContentArea", 
			 [com.ibm.rqm.planning.web.ui.internal.view.administration.AbstractContentArea], {

	csHelpId: "rqm_pref_feed_views.html",
	_contentWidget: null,
	_feedViewer: null,
	_pagingControl: null,
	_loadingWidget: null,
	_projectAlias: null,
	_currentResourceType: null,
	_noItemsFoundTxt: null,
	
	postCreate: function(){
        this.inherited("postCreate", arguments);
        this._projectAlias = com.ibm.asq.common.web.ui.internal.util.Utility.getCurrentProjectAreaAlias();
	},
	
	// Overridden
	setName: function(){
	    this.name = Constants.getMessage("labelFeedViews");
	    dojox.data.dom.removeChildren(this._name);
	    var name = document.createTextNode(this.name);
	    this._name.appendChild(name);
	},
        
	// Overridden
     setText: function(){
         this.text = Constants.getMessage("labelFeedViewsText");
         dojox.data.dom.removeChildren(this._text);
         var text = document.createTextNode(this.text);
         this._text.appendChild(text);
     },

	// Overridden
    setContent: function(){
    	var setFeedUrlCallback = dojo.hitch(this, this.setFeedUrl);
		this._pagingControl = new FeedViewsContentArea.PagingControl({'callback': setFeedUrlCallback});
		this._content.appendChild(this._pagingControl.domNode);
		
		this._feedViewer = new ibm_atom.widget.FeedViewer();
		this._content.appendChild(this._feedViewer.domNode);
		this._feedViewer.startup();
		
		this._loadingWidget = new FeedViewsContentArea.LoadingWidget();
		this._loadingWidget.setVisible(false);
		this._content.appendChild(this._loadingWidget.domNode);
		
	    this._noItemsFoundTxt = document.createElement("div");
	    this._noItemsFoundTxt.innerHTML = Constants.getMessage("feedViews_noItemMsg");
	    this._content.appendChild(this._noItemsFoundTxt);
	    dojo.style(this._noItemsFoundTxt, "display", "none");
		
        dojo.connect(this._feedViewer,"onEntrySelected", dojo.hitch(this, this._showEntry));
    },
    
	_createFeedUrl: function(resourceType) {
       	var feedPrefix = "/jazz/service/com.ibm.rqm.integration.service.IIntegrationService/resources/"
       	               + this._projectAlias + "/";
       	return feedPrefix + resourceType;
	},
	
	setFeedType: function(resourceType) {
		this._pagingControl.resetPageNav();
		this._pagingControl.resetPageNums();
		this._currentResourceType = resourceType;
		var feedUrl = this._createFeedUrl(resourceType);
		this.setFeedUrl(feedUrl);
	},

	setFeedUrl: function(feedUrl) {
		this._feedViewer.clear();
		this._pagingControl.resetPageNav();
		this._loadingWidget.setVisible(true);
	    dojo.style(this._noItemsFoundTxt, "display", "none");
		
		var atomIO = this._feedViewer.atomIO;
		var onFeedOk = dojo.hitch(this, this._handleFeedOk);
		var onFeedError = dojo.hitch(this, this._handleFeedError);
		atomIO.getFeed(feedUrl, onFeedOk, onFeedError);		
	},
	
	_handleFeedOk: function(feed) {
		if (feed!=null && feed.entries && feed.entries.length>=1) {
			this._feedViewer.setFeed(feed);
		}
		this._postProcessFeed(feed);
	},
	
	_handleFeedError: function(err, responseObj) {
		var x = responseObj;  // don't remove this assignment. it works around a weird problem in the js engine
		 
		if (responseObj.xhr!=null && responseObj.xhr.status==410) {
			// status code 410 means the feed cursor has expired
			var message =  Constants.getMessage("feedViews_feedCursorExpiredMsg");
			var reloadHandler = dojo.hitch(this, this.setFeedType, this._currentResourceType);
			var yesnoDialog = new YesNoDialog({
				'message': message,
				'yesCallback': reloadHandler,
				'yesCallbackData': null
			});
			yesnoDialog.setSecondaryTitle(Constants.getMessage("labelError"));
			yesnoDialog.open();
		}
		else if (responseObj.xhr!=null && responseObj.xhr.status==404) {
			// feed not found
			var message =  Constants.getMessage("feedViews_feedNotFound");
			var alertDialog = new AlertDialog({
				'message': message
			});
			alertDialog.setSecondaryTitle(Constants.getMessage("labelError"));
			alertDialog.open();
		}
		else {
			// unknown error
			console.error(err);
		}
		
		this._postProcessFeed(null);
	},
	
	_showEntry: function(entry) {
		var links = entry.links;
		for (var i=0; i<links.length; i++) {
			var link = links[i];
			if (link.rel == 'enclosure') {
				var matches = /\#(.+)/.exec(link.href);
				if (matches && matches.length>0) {
					window.location.hash = matches[1];
				}
			}
		}
	},
	
	_postProcessFeed: function(feed) {
		this._loadingWidget.setVisible(false);

		if (feed==null || feed.links==null || feed.links.length==0) {
			// no links
			dojo.style(this._noItemsFoundTxt, "display", "inline");
			return;
		}
		else if (feed.entries==null || feed.entries.length==0) {
			// no entries
			dojo.style(this._noItemsFoundTxt, "display", "inline");
			return;
		}
		else {
			dojo.style(this._noItemsFoundTxt, "display", "none");
		}
		
		this._pagingControl.setVisible(true);	
		var currentPage = 1;
		var lastPage = 1;
		
		for (var i=0; i<feed.links.length; i++) {
			var link = feed.links[i];
			var href = link.href;
			if (link.rel == "first") {
				this._pagingControl.setFirst( href );
			}
			else if (link.rel == "previous") {
				this._pagingControl.setPrevious( href );
			}
			else if (link.rel == "next") {
				this._pagingControl.setNext( href );
			}
			else if (link.rel == "last") {
				this._pagingControl.setLast( href );
				var matches = /page=(\d+)/.exec(href); // hack to figure out the last page number
				if (matches && matches.length>0) {
					lastPage = new Number(matches[1]) + 1;
				}
			}
			else if (link.rel == "self") {
				var matches = /page=(\d+)/.exec(href); // hack to figure out the current page number
				if (matches && matches.length>0) {
					currentPage = new Number(matches[1]) + 1;
				}
			}
		}
		
		if (currentPage>lastPage) {
			lastPage = currentPage;
		}
		
		this._pagingControl.setCurrentPage(currentPage, lastPage);
	},

	__sentinel: null // terminates this class definition
});

dojo.declare("FeedViewsContentArea.PagingControl", 
			 [dijit._Widget, dijit._Templated], {

    templateString: '<div class="com-ibm-rqm-test-feedviews" dojoAttachPoint="_pageControlNode">' +
                    '<table>' +
                    '  <tr>'+
                    '     <td>' +
                    '         <span dojoAttachPoint="_firstNode">&lt;&lt; ${labelFirstPage}</span>&nbsp;' +
                    '         <span dojoAttachPoint="_previousNode">&lt; ${labelPreviousPage}</span>&nbsp;&nbsp;&nbsp;|' +
                    '         <span dojoAttachPoint="_pageNumberText" align="center"></span> | ' +
                    '         &nbsp;&nbsp;&nbsp;<span dojoAttachPoint="_nextNode">${labelNextPage} &gt;</span>&nbsp;' +
                    '         <span dojoAttachPoint="_lastNode">${labelLastPage} &gt;&gt;</span>' +
                    '     </td>' +
                    '  </tr>' +
                    '</table>' +
                    '</div>',
                    
    callback: "",   // function to call back with a feed URL
    
	labelFirstPage: Constants.getMessage("labelFirstPage"),
	labelPreviousPage: Constants.getMessage("labelPreviousPage"),
	labelNextPage: Constants.getMessage("labelNextPage"),
	labelLastPage: Constants.getMessage("labelLastPage"),

    constructor: function() {
    },

    postCreate: function() { 
        this.inherited("postCreate", arguments);
	},
	
	resetPageNav: function() {
		var nodes = [ this._firstNode, this._previousNode, this._nextNode, this._lastNode ];
		for (var x in nodes) {
			nodes[x].onclick="";
			nodes[x].className="deactivated";
		}
	},
	uninitialize: function() {
		this.resetPageNav(); 
	},	
	resetPageNums: function() {
		var pageNumberText = Constants.getMessage("labelPageOfPages", 0, 0);
		dojox.data.dom.removeChildren(this._pageNumberText);
		var tmp = document.createTextNode(pageNumberText);
		this._pageNumberText.appendChild(tmp);
	},

	setFirst: function(url) {
		this._activate(this._firstNode, url);
	},
	
	setPrevious: function(url) {
		this._activate(this._previousNode, url);
	},
	
	setNext: function(url) {
		this._activate(this._nextNode, url);
	},
	
	setLast: function(url) {
		this._activate(this._lastNode, url);
	},
	
	setCurrentPage: function(currentPage, lastPage) {
		var pageNumberText = Constants.getMessage("labelPageOfPages", currentPage, lastPage);
		dojox.data.dom.removeChildren(this._pageNumberText);
		var tmp = document.createTextNode(pageNumberText);
		this._pageNumberText.appendChild(tmp);
	},
	
	setVisible: function(isVisible) {
		if (isVisible) {
			dojo.style(this._pageControlNode, "display", "inline");
		} else {
			dojo.style(this._pageControlNode, "display", "none");
		}
	},
	
	_activate: function(node, url) {
		node.onclick=dojo.hitch(this, this.callback, url);
		node.className="activated";
	},
	
	
	__sentinel: null // terminates this class definition
});

dojo.declare("FeedViewsContentArea.LoadingWidget", 
			 [dijit._Widget, dijit._Templated], {

    templateString: '  <div dojoAttachPoint="loadingProgressNode" style="display:none">' +
                    '    <div class="progress-image">' +
                    '      <img dojoAttachPoint="loadingImgNode"></img>' +
                    '      <span dojoAttachPoint="loadingLabelNode">${loadingLabel}</span>' +
                    '    </div>' +
                    '  </div>',
                    
		
	loadingLabel: Constants.getMessage("labelLoadingContent"),
	
    postCreate: function() {
        this.inherited("postCreate", arguments);
		this.loadingImgNode.src = ImageCache.VIEW_PROGRESS;
	},
	
	setVisible: function(isVisible) {
		if (isVisible) {
			dojo.style(this.loadingProgressNode, "display", "inline");
		} else {
			dojo.style(this.loadingProgressNode, "display", "none");
		}
	},
		
	__sentinel: null // terminates this class definition
});

})();
