/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.administration.ReportServerContentArea");

dojo.require("com.ibm.team.process.web.client.projectarea");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.client.internal.option");
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.AbstractPropertyContentArea");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.ReportServerTableViewer");

(function(){
	
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	var ReportServerTableViewer = com.ibm.rqm.planning.web.ui.internal.view.administration.ReportServerTableViewer;
	var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	var getCurrentProjectArea  = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
	var AlertDialog = com.ibm.asq.common.web.ui.internal.widgets.AlertDialog;

    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.ReportServerContentArea", com.ibm.rqm.planning.web.ui.internal.view.administration.AbstractPropertyContentArea, {
        
		constructor: function(){
			this.reportServer = [];			
			this.csHelpId = "rqm_admin_report_server.html";
			
			//internal
			this._preference = null;
			this._optionsClient = new com.ibm.asq.common.web.client.internal.option();
		},
		
		postCreate: function(){
            this.inherited("postCreate", arguments);
			this.csHelp.setButtonLabel(Constants.getMessage("labelReportServerHelp"));
        },
        
        setName: function(){
            this.name = Constants.getMessage("labelReportServer");
            dojox.data.dom.removeChildren(this._name);
            var name = document.createTextNode(this.name);
            this._name.appendChild(name);
        },
        
        setText: function(){
            this.text = Constants.getMessage("labelSetReportServer");
            dojox.data.dom.removeChildren(this._text);
            var text = document.createTextNode(this.text);
            this._text.appendChild(text);
        },
        
        setContent: function(){
			this._showReportServer();
			this._loadReportServer();
        },
		
		_showReportServer: function(){
			var args = {
				reportServer: [],
				onChangeListener: dojo.hitch(this, "_onReportServerChange"),
				actions: this._getTableActions(),
                editable: this.isEditable(),
                enableRowSelection: false		
			}
			this._linksTable = new ReportServerTableViewer(args);			
			this._content.appendChild(this._linksTable.domNode);
		},
		
		_loadReportServer: function() {
			var args = {
				projectAreaItemId: getCurrentProjectArea().itemId,
				key: Constants.REPORT_SERVER_PREFERENCE_KEY
			}
			var handler = new ServiceResponseHandler(this, "_setPreference", "_error");
			this._optionsClient.getProjectPreferences(handler, args.projectAreaItemId, args.key);
		},
		
		_setPreference: function(preference) {
            if (preference != null) {
                this._preference = preference;
                this.reportServer = dojo.fromJson(preference.value);
            } else {
                this.reportServer = Constants.REPORT_SERVER_PREFERENCE_DEFAULTS.concat([]);                     
            }
            this.setDirty(false);
            this._refreshTable();			
		},
		
		_refreshTable: function() {
			this._linksTable.refresh(this.reportServer);
		},
		
        //actions to be displayed in the table header
        _getTableActions: function(){
            var actions = [];        
            return actions;
        },
		
		_onReportServerChange: function(rowData) {
			this.setDirty(true);
			for (var i = 0; i < this.reportServer.length; i++) {
				//if (this.reportServer[i].id == (rowData.id)) {
					this.reportServer[i] = rowData;
				//} 
			}
		},
		
        /*_sortLinks: function(link1, link2) {
            if (link1.address < link2.address) {
                return -1;
            } else if (link1.address > link2.address) {
                return 1;
            }
            return 0;
        },*/
		
		_displayAlertDialog: function(title, message){
            var alertDialog = new AlertDialog({
                'message': message
            });
            alertDialog.setSecondaryTitle(title);
            alertDialog.open();
        },
        
        getValues: function(){
            return [];
        },
			
		onValidate: function() {
            var result = {
				message: null,
				pass: false
			}
            for (var i = 0; i < this.reportServer.length; i++) {
                var link = this.reportServer[i];
                if (link.address == null || dojo.trim(link.address) == 0) {                    
                    result.message = Constants.getMessage("msgRequiredField", Constants.getMessage("labelURL"), Constants.getMessage("labelURL"));
					break;
                }
            }
            if (result.message != null) {                
                result.pass = false;
            } else {
                result.pass = true;
            }            	
			return result;		
		},
		
		onSave: function(){
            if (this.isDirty()) {				
				this._doSave();          
            }
		},
		
        _doSave: function(){
			this.parent.addSaveParticipantNum();
			
            var key = Constants.REPORT_SERVER_PREFERENCE_KEY;
            var value = dojo.toJson(this.reportServer, true, "   ");			

            var areaItemId = getCurrentProjectArea().itemId;
			var self = this;
			var handler = {
				success: function(results){
					self.parent.saveFinished();
					self._setPreference(results);
				},
				error: function(e){
					self.parent.saveFinished();
					self._error(e);
				}
			}
		
            var responsehandler = new ServiceResponseHandler(handler, "success", "error");
            if (this._preference != null) {
                this._optionsClient.updateProjectPreferences(responsehandler, areaItemId, this._preference.itemId, value);
            } else {
                this._optionsClient.createProjectPreferences(responsehandler, areaItemId, key, value);
            }
			this.setDirty(false);            
        },
		
		_error: function(error) {
			var message = error && error.message ? error.message : Constants.getMessage("msgSysPropErrorNoDetails");
			this.setErrorMessage(Constants.getMessage("msgSysPropError"), message);
		}
    });
    
})();
