/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Load: com.ibm.rqm.planning.web.ui.internal.view.administration.RequirementConnectionTableViewer");
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.administration.RequirementConnectionTableViewer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.layout.FormTextField");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function(){
	var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
	var ImageCache     = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
	var AlertDialog = com.ibm.asq.common.web.ui.internal.widgets.AlertDialog;
	var YesNoDialog = com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog;
	var RENDER_TYPE_INPUTTEXT = "RenderType.InputText";
	
	dojo.declare("RequirementConnectionTableViewer.InputTextBox", com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer, {
        // Summary:
        //    Renders the cell content. For renderer that can modify the cell value.
		//    Invoke cellValueChanged when input is blured
        //    
        renderCell: function(viewer, cellData, extendedDisplayInfo, /*table.CellModifier*/ cellModifier, rowObject, columnIndex, cellWidthHint){
            var node;
            var contentProvider = viewer.getContentProvider();
            var editable = viewer.isEditable() && (contentProvider ? contentProvider.getIsCellEditable(rowObject, columnIndex) : false);
            
            if (editable) {
				node = new com.ibm.asq.common.web.ui.internal.widgets.layout.FormTextField({
					required: false,
					disabled: false,
//					labelWidthPercent: 1,
					textFieldInvalidFormatMsg: Constants.getMessage("msgRequirementURIInvalid"),
					currentValue: cellData,
					regExp:"http(s)?:\/\/(.)+"
				});
				
                var widthStyled = false;
                
                if (extendedDisplayInfo && dojo.isObject(extendedDisplayInfo)) {
                    if (extendedDisplayInfo['textWidth']) {
                        node.domNode.style.width = extendedDisplayInfo['textWidth'];
                        widthStyled = true;
                    }
                }
                // by default, if not IE always add style to stretch the field
                // to the table cell width. 
//                if (!widthStyled && !dojo.isIE) {
//                    dojo.addClass(node.domNode, "editText-cell");
//                }
                
                node.cellModifier = cellModifier;
                node.rowObject = rowObject;
                node.columnIndex = columnIndex;
                node.oldValue = cellData;
                
                node._valueChangedHandler = function(){
					this.inherited("_valueChangedHandler", arguments);
                    var oldValue = this.oldValue;
					var newValue = this.getValue();
                    this.oldValue = newValue;
                    
                    if (oldValue != newValue && this.cellModifier) 
                        this.cellModifier.cellValueChanged(oldValue, newValue, this.rowObject, this.columnIndex);
                }
                if (extendedDisplayInfo && dojo.isObject(extendedDisplayInfo)) 
                    node.tooltip = this._attachTooltipToCell(node.domNode, extendedDisplayInfo['tooltip']);
				
				node.fieldLabelHolderNode.style.display = "none";
				node.fieldHolderNode.style.borderBottom = "0px";
				dojo.removeClass(node.fieldHolderNode.lastChild, "text-field");
				node.fieldHolderNode.lastChild.style.width = "100%";
				
                this.cellNodes.push(node);
                return node.domNode;
            }
            else {
                node = document.createElement("span");
                this._renderCellText(viewer, cellData, extendedDisplayInfo, node, cellWidthHint);
                return node;
            }
        },
    
        disconnectAll: function()
        {
            //console.debug("================RequirementConnectionTableViewer.InputTextBox disconnectAll");
            this.inherited(arguments);
            var cellNodes = this.cellNodes;
            var len = cellNodes.length;
            for(var i = 0; i < len; i++) {
                var node = cellNodes[i];
                if(node.destroy)
                    node.destroy();
                var tooltip = node.tooltip;    
                if(tooltip && tooltip.destroy) {
                    tooltip.destroy();
                }
            } 
            this.cellNodes = [];
        }
    });
	
    dojo.declare("RequirementConnectionTableViewer.TableListener", com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener, {
        constructor: function(parentView) {
	        this.parentView = parentView;
	    },
    
    	tableContentActionInvoked: function(tableViewer, actionId, callbackData, callback)
    	{
        	if (callback != null && dojo.isFunction(callback)) {
		        callback();
		   	}  
    	},
		
		// Summary:
	    //   If the table's rows are checkable, when a row is checked, or unchecked, this method is
	    //   called.
	    //
	    rowCheckStateChanged: function(tableViewer, rowObject, state)
	    {
			var selectionCount = tableViewer.getCheckedRows().length;
			var parentView = this.parentView;
	        if (parentView.removeActionNode)
	            parentView.removeActionNode.setDisabled(selectionCount == 0);
			if (parentView.testActionNode)
				parentView.testActionNode.setDisabled(selectionCount != 1);
			if (parentView.synchronizeActionNode)
				parentView.synchronizeActionNode.setDisabled(selectionCount != 1 || rowObject.itemId < 0);
			if (parentView.startSyncNowNode)
				parentView.startSyncNowNode.setDisabled(selectionCount != 1 || rowObject.itemId < 0);
	    }
    });
    
    dojo.declare("RequirementConnectionTableViewer.CellModifier", com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier, {
        tableViewer: null,
		
		constructor: function(parentView)	{
	        	this.parentView = parentView;
	    },
        
        cellValueChanged: function(oldValue, newValue, rowObject, columnIndex){
            switch (columnIndex) {
                case 0:
					var values = newValue.split(":");
					var port = values[2];
					if (!port) {
						port = 80;
					}
                    rowObject.hostname = values[0]+":"+values[1];
					rowObject.port = port;
					this.parentView.parent.setDirty(true);
                    break;
                default:
                    break;
            }
        },
		
		cellActionInvoked: function(rowObject, columnIndex)
	    {
//			if(columnIndex == 2) {
//				this.parentView._handleStopSynchronization(rowObject);
//			}
	    }
    });
    
    dojo.declare("RequirementConnectionTableViewer.TableProvider", com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider, {
        getNumColumns: function(content){
            return 2;
        },
        
        getColumnHeading: function(content, index){
            var headings = [Constants.getMessage("labelReqProConnURI"), Constants.getMessage("labelReqProSynStatus")];
            return headings[index];
        },
        
        getIsColumnSortable: function(content, atColumnIndex){
            return true;
        },
		
        getColumnWidth: function(content, atColumnIndex){
            var width = 2;
            switch (atColumnIndex) {
                case 0:
                    width = 85;
                    break;
				case 1:
                    width = 15;
                    break;
            }
            return width;
        },
		
        getIsCellEditable: function(rowData, atColumnIndex){
			if(atColumnIndex == 0){
				return true;
			}else{
				return false;
			}
        },
        
        // Summary:
        //    The only assumptions the table have on the data that it displays
        //    is that it's an array contents. These are the data content/object that
        //    will be displayed in the table rows.
        //
        getContentObjects: function(content){
            // our content is already an array of objects
            return content;
        },
        
        getRowDataId: function(rowData){
            return rowData.itemId;
        },
        
        // Summary:
        //    Returns the type of the cell to be displayed. By default, the TableViewer
        //    will support a set of cell types. The type of the cell will map to a specific
        //    cell renderer (to be discussed).
        getCellDisplayType: function(rowData, atColumnIndex){
			if(rowData.itemId < 0 && atColumnIndex == 0){
				return RENDER_TYPE_INPUTTEXT;
			}else{
				return TableConstants.RENDER_TYPE_TEXT;
			}
        },
        
        // Summary:
        //    Returns the display data of the cell. For cell renderer type of TEXT, this will
        //    be the string. Others cell render type would be Date, or integer value, etc. If the 
        //    cell renderer type is image, the client can also supply cell data that can be sortable/groupable. 
        //    The TableViewer's default sorter, filter operates on this returned data. For "complex" 
        //    cell types such as image where it could have multiple values for different states, the 
        //    client also needs to implement the next method.
        //    
        //    Client must implement this method to provide the cell data.
        //    
        getCellDisplayData: function(rowData, atColumnIndex){
            var val = "data";
            switch (atColumnIndex) {
                case 0:
                    val = rowData.hostname + ":" + rowData.port;
                    break;
				case 1:
                    if(rowData.syncStopped === ""){
						val = rowData.syncStopped;
					}else if(rowData.syncStopped === true){
						val = Constants.getMessage("labelReqProStartSyn");
					}else{
						val = Constants.getMessage("labelReqProStopSyn");;
					}
                    break;
            }
            return val;
        },
		
		getCellDisplayExtendedInfo: function(rowData, atColumnIndex)
	    {  
	        var val = null;
	        return val;
	    }  
    });
    
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.RequirementConnectionTableViewer", [dijit._Widget, dijit._Templated], {
        templateString: '<div dojoAttachPoint="domNode" tabindex="-1"></div>',
        widgetsInTemplate: true,
        
        constructor: function(args){
            this.content = null;
			this.newItemId = -1;
			this.newConnections = [];
        },
        
        postCreate: function(){
            this.inherited("postCreate", arguments);
        },
        
        initialize: function(args){
//			this.parentController = args.parentController;
            var tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer();
			tableViewer.actionsTD.setAttribute("nowrap", "nowrap");
            this.domNode.appendChild(tableViewer.domNode);
            this.tableViewer = tableViewer;
			
            var contentProvider = new RequirementConnectionTableViewer.TableProvider();
            
            tableViewer.setContentPageSize(20);
			tableViewer.addAction("add", Constants.getMessage("labelReqProAddConn"), null, "add-icon-image", dojo.hitch(this, "_handleAddNewClick"));
    		this.removeActionNode = tableViewer.addAction("remove", Constants.getMessage("labelReqPeoRemoveConn"), null, "delete-icon-image", dojo.hitch(this, "_handleDeleteClick"));
			this.removeActionNode.setDisabled(true);
			this.testActionNode = tableViewer.addAction("test", Constants.getMessage("labelReqProTestConn"), null, "test-connection-icon-image", dojo.hitch(this, "_handleTestClick"));
			this.testActionNode.setDisabled(true);
			this.synchronizeActionNode = tableViewer.addAction("synchronize", Constants.getMessage("labelEnableSynch"), null, "sychronize-icon-image", dojo.hitch(this, "_handleStopSynchronization"));
			this.synchronizeActionNode.setDisabled(true);
			this.startSyncNowNode = tableViewer.addAction("startSyncNow", Constants.getMessage("labelSyncNow"), null, "syncnow-icon-image", dojo.hitch(this, "_handleSynchronizeNow"));
			this.startSyncNowNode.setDisabled(true);
			tableViewer.addAction("refresh", Constants.getMessage("labelReqProRefreshConn"), null, "refresh-icon-image", dojo.hitch(this.parent, "loadConnections"));
            
			tableViewer.setContent(this.content);
            tableViewer.setContentProvider(contentProvider);
			tableViewer.setTableRowsCheckable(this.parent.isEditable(), /* isSingleSelect */ false);  
			tableViewer.setTableListener(new RequirementConnectionTableViewer.TableListener(this));
			tableViewer.setCellModifier(new RequirementConnectionTableViewer.CellModifier(this));
			tableViewer.registerCellRenderer(RENDER_TYPE_INPUTTEXT, new RequirementConnectionTableViewer.InputTextBox());
            tableViewer.show();
            tableViewer.setEditable(this.parent.isEditable());
        },
		
		refresh: function(connections) {
	      this.tableViewer.setContent(connections);
		  this.removeActionNode.setDisabled(true);
		  this.testActionNode.setDisabled(true);
		  this.synchronizeActionNode.setDisabled(true);
		  this.startSyncNowNode.setDisabled(true);
	      this.tableViewer.show();
		  this.newItemId = -1;
		  this.newConnections = [];
		  this._refreshActionStatus();
	    },
		
		_refreshActionStatus: function() {
			this.removeActionNode.setDisabled(true);
			this.testActionNode.setDisabled(true);
			this.synchronizeActionNode.setDisabled(true);
			this.startSyncNowNode.setDisabled(true);
		},
        
        getSelectedConnections: function() {
	    	return this.tableViewer.getCheckedRows();
		},
		
		removeSelectedConnections: function(){
			var connections = this.getSelectedConnections();
			var connectionsLength = connections.length;
			var newConnections = [];
			var deletedConnections = [];
			for(var i = 0; i < connectionsLength; i++){
				var connection = connections[i];
				deletedConnections.push(connection);
				var isNewConnectionRemoved = false;
				for(var j = 0; j < this.newConnections.length; j++){
					if(connection.itemId < 0 && connection.itemId == this.newConnections[j].itemId){
						isNewConnectionRemoved = true;
					}
				}
				if(isNewConnectionRemoved == false){
					newConnections.push(connection);
				}
			}
			this.newConnections = newConnections;
			this.tableViewer.setContent(this._deleteItems(this.tableViewer.getContent(), deletedConnections));
			this.tableViewer.refresh();
			this._refreshActionStatus();
		},
		
		validateNewConnections: function(){
			var connections = this.tableViewer.getContent();
			var connectionsLength = connections.length;
			for(var i = 0; i < connectionsLength; i++){
				var connection = connections[i];
				if(connection.itemId < 0 && connection.hostname != "" && connection.port != ""){
					for(var j = 0; j < connectionsLength; j++){
						if(this._checkDuplication(connection, connections[j])){
							return Constants.getMessage("msgReqProSaveConn", connection.hostname+":"+connection.port);
						}
					}
				}
			}
			return "";
		},
		
        _handleAddNewClick: function(){
			var obj = {
                hostname: "",
                port: "",
				syncStopped: "",
				itemId: this.newItemId
            };
			this.newItemId--;
			var content = this.tableViewer.getContent();
			content.push(obj);
            this.tableViewer.setContent(content);
            this.tableViewer.refresh();
            this._refreshActionStatus();
        },
		
		_handleDeleteClick: function(){
			var connections = this.getSelectedConnections();
			if(connections.length == 0){
				this._displayAlertDialog("", Constants.getMessage("msgReqProRemoveConn"));
				return;
			}
            var confirmMsg = "";
            if (connections.length > 1) {
                confirmMsg = Constants.getMessage("msgReqProPromoptRemoveConn", connections.length);
            } else {
                confirmMsg = Constants.getMessage("msgReqProPromoptRemoveConn1", connections[0].hostname);
            }
			var args = {
				title: "",
				message: confirmMsg,
				yesCallback: dojo.hitch(this.parent, "deleteConnection"),
				yesCallbackData: connections,
				noCallback: function(){
					return;	
				}
			}
			this._displayYesNoDialog(args);
		},
		
		_handleTestClick: function(){
			var connections = this.getSelectedConnections();
			if(connections.length > 1){
				this._displayAlertDialog("", Constants.getMessage("msgReqProTestConn"));
				return;
			}else if(connections.length == 0){
				this._displayAlertDialog("", Constants.getMessage("msgReqProTestConn1"));
				return;
			}
			if(dojo.isArray(connections) && connections.length == 1){
				this.parent.testConnection(connections[0]);
			}
		},
		
		_handleSaveClick: function(){
			var connections = this.tableViewer.getContent();
			this.newConnections = [];
			var connectionsLength = connections.length;
			for(var i = 0; i < connectionsLength; i++){
				var connection = connections[i];
				if(connection.itemId < 0 && connection.hostname != "" && connection.port != ""){
					this.newConnections.push(connection);
				}
			}
			this.parent.saveConnection(this.newConnections);
		},
		
		_handleStopSynchronization: function(){
			var connections = this.getSelectedConnections();
			if(connections.length > 1){
				this._displayAlertDialog("", Constants.getMessage("msgSelectOnlyOneConn"));
				return;
			}else if(connections.length == 0){
				this._displayAlertDialog("", Constants.getMessage("msgSelectOneConn"));
				return;
			}else if(connections.length == 1 && connections[0].syncStopped !== ""){
				var message = "";
				if(connections[0].syncStopped === true){
					message = Constants.getMessage("msgReqProStartSyn", connections[0].hostname);
				}else if(connections[0].syncStopped === false){
					message = Constants.getMessage("msgReqProStopSyn", connections[0].hostname);
				}
				var args = {
					title: "",
					message: message,
					yesCallback: dojo.hitch(this.parent, "stopSynchronization"),
					yesCallbackData: connections[0],
					noCallback: function(){
						return;
					}
				}
				this._displayYesNoDialog(args);
			}else{
				this._displayAlertDialog("", Constants.getMessage("msgReqProSaveConn1"));
			}
		},
		
		_handleSynchronizeNow: function() {
			var connections = this.getSelectedConnections();
			
			if(connections.length == 0) {
				this._displayAlertDialog("", Constants.getMessage("msgSelectOneConnToSync"));
				return;
			}else if(connections.length > 1) {
				this._displayAlertDialog("", Constants.getMessage("msgSelectOnlyOneConnToSync"));
				return;
			}else if(connections.length == 1) {
				var args = {
					title: "",
					message: Constants.getMessage("msgReqProSyncNow", connections[0].hostname),
					yesCallback: dojo.hitch(this.parent, "startSynchronization"),
					yesCallbackData: connections[0],
					noCallback: function() {
						return;
					}
				}
				this._displayYesNoDialog(args);
			}else{
				this._displayAlertDialog("", Constants.getMessage("msgReqProSaveConn1"));
			} 
		},
		
		_checkDuplication: function(firstConn, secondConn){
			if(firstConn.hostname == secondConn.hostname && firstConn.port == secondConn.port && firstConn.itemId != secondConn.itemId){
				return true;
			}else{
				return false;
			}
		},
		
		_deleteItems: function(items, deletedItems){
			var newItems = [];
			var length = items.length;
			var length2 = deletedItems.length;
			for(var i = 0; i < length; i++){
				var deleted = false;
				for(var j = 0; j < length2; j++){
					if(items[i].itemId == deletedItems[j].itemId){
						deleted = true;
					}
				}
				if(!deleted){
					newItems.push(items[i]);
				}
			}
			return newItems;
		},
		
		_displayAlertDialog: function(title, message){
            var alertDialog = new AlertDialog({
                'message': message
            });
            alertDialog.setSecondaryTitle(title);
            alertDialog.open();
        },
		
		_displayYesNoDialog: function(args){
            var dialog = new YesNoDialog(args);
            dialog.setSecondaryTitle(args.title);
            dialog.open();
        },
		
        _error: function(errorObj){
            alert(errorObj.message);
        }
    });
    
})();
