/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading: com.ibm.rqm.risk.web.ui.internal.view.assessment.RiskDiscussionWidget");
dojo.provide("com.ibm.rqm.risk.web.ui.internal.view.assessment.RiskDiscussionWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Slider");

// Require the common widgets needed
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");

// Require local widgets
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");

(function(){ // module container
    /* Jazz objects */
    var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    /* QM objects */
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var ImageCache = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
	var Utility = com.ibm.rqm.planning.web.ui.internal.util.Utility;
	var DATE_FORMAT_NO_TIME = "%b %e, %Y";
	
	dojo.declare("com.ibm.rqm.risk.web.ui.internal.view.assessment.RiskCommentWidget", [dijit._Widget, dijit._Templated], {
        templateString: '<div class="com-ibm-rqm-risk-assessment">' +
					'    <div dojoAttachPoint="_comment" class="comment">' +
                    '      <div dojoAttachPoint="header"> '+
					'         <span dojoAttachPoint="_commentNumber" class="commentNumberSpan"></span>' + 
					'         <span dojoAttachPoint="_commentOwner" class="commentOwnerSpan"></span>' + 
					'         <span dojoAttachPoint="_commentDate"></span>' +
					'      </div>' +
                    '      <div dojoAttachPoint="_content" class="commentContentDiv"></div>' +
					'    </div>' +
                    '</div>',
        
        // object initializer
        constructor: function(args){
            this.comment = args.comment;
			this.commentNumber = args.commentNumber;
			this.isCollapsible = args.isCollapsible;
        },
        
        postCreate: function(){
            this._initialize();
        },
        
        _initialize: function(){
			if(!this.comment)
				return;
			this.isCollapsed = false;
            this._commentNumber.appendChild(document.createTextNode(this.commentNumber + "."));
            this._commentOwner.appendChild(document.createTextNode(this.comment.creator.name));
			this._commentDate.appendChild(document.createTextNode(this._getCommentDate(this.comment.creationDate)));
			if(this.comment.content){
				var span = document.createTextNode(this.comment.content);
				dojox.data.dom.removeChildren(this._content);
				this._content.appendChild(span);
//				dojox.data.dom.replaceChildren(this._content, span);
			}
			this.header.className = "commentHeader";	
			
			if(this.isCollapsible) {
				this.connect(this.header, "onmouseover", "handleMouseOver");
				this.connect(this.header, "onmouseout", "handleMouseOut");
				this.connect(this.header, "onclick", "handleClick");
			}
        },
        
        handleMouseOver: function(e) {
			this.header.className  = "commentHeaderMouseOver";
		},
		
		handleMouseOut: function(e) {
			this.header.className = "commentHeader";	
		},
        
		handleClick: function(e) {
			if(!e || !e.target) return;
			
			if(!this.isCollapsible)
				return;
			e.preventDefault();
			if(this.isCollapsed) {
				this.expand();
			} else {
				this.collapse();
			}
		},
		
		expand: function() {
			this.isCollapsed = false;
				this._content.style.display="block";
				this._comment.className = "comment";
		},
		collapse: function() {
			this.isCollapsed = true;
				this._comment.className = "commentCollapsed";
				this._content.style.display="none";
		},
		
		_getCommentDate: function(value) {
			var date;
			var timeValue = value;
			if(value && value.length > 0) {
				date = Utility.dateFromISOString(value);					
			}
				
			if(date != null ) {
				value = this._getDateString(date, false);
				timeValue = this._getDateString(date, true);
				if(!dojo.isString(timeValue)) {
					timeValue = value;
				}
			} 
			return timeValue;
		},
		
		//copy from com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil
		_getDateString: function(dateObject, includeTime) {
			var dateStr;
			if (dojo.i18n.normalizeLocale() == "en-us") {
				dateStr = dojox.date.posix.strftime(dateObject, DATE_FORMAT_NO_TIME);
				if (includeTime) {
					dateStr += " ";
					var hour = dojox.date.posix.strftime(dateObject, "%I");
					dateStr += hour.charAt(0) !== "0" ? hour : hour.substring(1);
					dateStr += ":" + dojox.date.posix.strftime(dateObject, "%M") + " ";
					dateStr += dojox.date.posix.strftime(dateObject, "%p").toUpperCase();
				}
			}
			else {
				if (includeTime) {
					dateStr = dojo.date.locale.format(dateObject, {formatLength: 'medium' })
				} else {
					dateStr =  dojo.date.locale.format(dateObject, {selector:'date', formatLength: 'medium'});
				}
			}
			return dateStr;
		},
		
        _error: function(e){
            alert("An error occurred: [" + e.message + "] @ RiskDiscussionWidget#_error");
        }
    });
    
    dojo.declare("com.ibm.rqm.risk.web.ui.internal.view.assessment.RiskDiscussionWidget", [dijit._Widget, dijit._Templated], {
        templatePath: dojo.moduleUrl("com.ibm.rqm.risk.web", "ui/internal/view/templates/assessment/RiskDiscussionWidget.html"),
        
        textDefaultTitle: Constants.getMessage("textDefaultTitle"),
        textCollapseAll: Constants.getMessage("textCollapseAll"),
        textExpandAll: Constants.getMessage("textExpandAll"),
		labelNoComments: Constants.getMessage("labelNoComments"),
		textAddComment: Constants.getMessage("textAddComment"),
		textHideComment: Constants.getMessage("textHideComment"),
		
		//allow add new comment
        editable: true,
		
		//allow collapse/expand comments
		isCollapsible: true,
		
		//when clicking title, allow collapse/expand the whole comments
		isHideAllComments: true,
        
		isDefaultOpen: true,
		
        // object initializer
        constructor: function(args){
            this.comments = args.comments;
			this.commentsNum = args.commentsNum;
            this.editable = args.editable;
        },
        
        postCreate: function(){
			this.commentsWidget = [];
			this.isShowAddNewComment = false;
			this.isShowDiscussionDetail = false;
            this._initialize();
        },
        
        _initialize: function(){
            if (this.comments && this.comments.length > 0) {
                var message =  Constants.getMessage("msgCommentNumber", this.commentsNum);
                dojox.data.dom.replaceChildren(this._sectionMessage, document.createTextNode(message));
            }
            
			if (!this.comments || this.comments.length == 0) {
				this._expandCollapseDiv.style.visibility = "hidden";
				var noComments = document.createElement("div");
				noComments.className = "NoComments";
				noComments.appendChild(document.createTextNode(this.labelNoComments));
				this.discussion.appendChild(noComments);
			}
			else {
				this._expandCollapseDiv.style.visibility = "visible";
				for (var i = 0; i < this.comments.length; i++) {
					var commentWidget = new com.ibm.rqm.risk.web.ui.internal.view.assessment.RiskCommentWidget({
						isCollapsible: this.isCollapsible,
						comment: this.comments[i],
						commentNumber: i+1
					});
					this.commentsWidget.push(commentWidget);
					this.discussion.appendChild(commentWidget.domNode);
				}
			}
			
			if(!this.isDefaultOpen) 
				this._sectionDetailDIV.style.display = "none";
			if(!this.editable) 
				this.addCommentLinkBottomDiv.style.display = "none";		
			if(!this.isCollapsible)	
				this._expandCollapseDiv.style.display = "none";
			if(this.isHideAllComments)
				dojo.addClass(this._sectionTitleBarDIV, "SectionTitleBarCollapsible");	
        },
		
		_onTitleClick: function(e) {
			e.preventDefault();
			if(!this.isHideAllComments)
				return;
			if(this.isShowDiscussionDetail) {
				//now collapse the discussion
				this._sectionDetailDIV.style.display = "none";
				this.domNode.className = "SectionWithoutBorder";
			}
			else {
				//expand the discussion
				this._sectionDetailDIV.style.display = "block";
				this.domNode.className = "SectionWithBorder";
			}
			this.isShowDiscussionDetail = !this.isShowDiscussionDetail;
		},
		
		_handleNewCommentClick:function(e) {
			e.preventDefault();
			var text;
			if(!this.isShowAddNewComment) {
				//add new textarea
				if(!this._newCommentTextArea) {
					this._newCommentTextArea = document.createElement("textarea");
					this._newCommentTextArea.style.width = "98%";
					this.newCommentDiv.appendChild(this._newCommentTextArea);
				}
				this.newCommentDiv.style.display = "block";
				text = this.textHideComment;
			} else {
				//hide new textarea
				this.newCommentDiv.style.display = "none";
				text = this.textAddComment;
			}
			this.isShowAddNewComment = !this.isShowAddNewComment;
			dojox.data.dom.replaceChildren(this.createCommentCmdBottom, document.createTextNode(text));
		},
		
        _handleCollapseAll: function(e){
            dojo.stopEvent(e);
            this._doExpandCollapse(false);
        },
        
        _handleExpandAll: function(e){
            dojo.stopEvent(e);
            this._doExpandCollapse(true);
        },
        
        _doExpandCollapse: function(doExpand){
			for(var i = 0; i < this.commentsWidget.length; i++) {
                if (doExpand) {
                    this.commentsWidget[i].expand();
                }
                else {
                    this.commentsWidget[i].collapse();
                }
			}
        },
        
        _error: function(e){
            alert("An error occurred: [" + e.message + "] @ RiskDiscussionWidget#_error");
        }
    });
    
})();
