/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.buildintegration.web.ui.internal.view.build.SelectItemTableDialog");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
//dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");
dojo.require("com.ibm.rqm.buildintegration.web.client.internal.build.BuildClient");
dojo.require("com.ibm.rqm.buildintegration.web.ui.internal.view.table.BuildTableView");
(function() {

var ServiceResponseHandler	= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var Constants               = com.ibm.rqm.planning.web.client.internal.Constants;
//var CSHelp                  = com.ibm.asq.common.web.ui.internal.util.CSHelp;
var BuildClient				= com.ibm.rqm.buildintegration.web.client.internal.build.BuildClient;
var BuildTableView			= com.ibm.rqm.buildintegration.web.ui.internal.view.table.BuildTableView;


// cap max number of query results at 1000
var MAX						= 1000;

var RUNNING_SEARCH_MSG		= Constants.getMessage("labelRunningSearch");
var FETCH_ERR_MSG			= Constants.getMessage("msgErrorRunningQuery");

dojo.declare("com.ibm.rqm.buildintegration.web.ui.internal.view.build.SelectItemTableDialog",[dijit._Widget, dijit._Templated],
	{
        templateString: '<div class="com-ibm-rqm-test-common-select-item-table">' +
		                '    <div dojoAttachPoint="_msgArea" class="NormalStatus"></div>' +
						'    <div dojoAttachPoint="_itemListArea" class="ItemList"></div>' +
                        '</div>',
		
        width: "600px",   //default dialog width
		
        constructor: function(params){
            this._items = [];
			dojo.mixin(this, params);
            this.setExclusionList(this.exclusionList ? this.exclusionList : []);
			
			if(dojo.isIE) {
				//for defect 9494
				MAX = 50;
			}
        },		
		
		postCreate: function() {
			this._initItemList();
		},
		
        _showAsDialog: function() {
            var dialogArgs = {
                primaryTitle: this.title ? this.title : Constants.getMessage("labelTitle"),
                okCallback: dojo.hitch(this, "_onOK"),
                cancelCallback: dojo.hitch(this, "_onCancel"),
				_truncate: function() {},  //disable the Dialog's '_truncate'
				zIndex:998,
            	forceZIndex:true
            };
            this._dialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(dialogArgs);
//            var cshelp = new CSHelp({contextID: "rqm_add_test_scripts.html"});
//            cshelp.setButtonLabel(Constants.getMessage("labelAddExistingCSHelp"));
//            this._dialog.setSecondaryTitleNode(cshelp.domNode);
            this._dialog.setWidth(this.width);
            this._dialog.contentContainer.appendChild(this.domNode);
			this._dialog.open(false);
			this._dialog.updatePosition();
        },		
		
        repositionDialog: function() {
            if (this._dialog) {
                // set timeout to allow the browser to render dialog
                setTimeout(dojo.hitch(this._dialog, "updatePosition"), 100);
            }
        },
        
		_initItemList: function() {
            var tableArgs = {
                builds: [],
				tableRowsCheckable:true,
				min_profile:true,
				editable:true
            };
			tableArgs.rowCheckedListener = dojo.hitch(this, "_onTableRowsChecked");
			this._list = new BuildTableView(tableArgs);
			this._list.show();
		},
		
        show: function(items){
			if (!this._dialog) {
                this._showAsDialog();
            }
            this.refresh(items);
        },		

		refresh: function(items) {
			this._dialog.getOkButton().disabled = true;
			if (this._list == null) {
				this._initItemList();
			}
			if (items) {
				this._setItems(items);
			} else {
                this._setStatusMessage(RUNNING_SEARCH_MSG);
                var srh = new ServiceResponseHandler(this, "_setItems", "_queryError");
                this._runningRequest = BuildClient.getBuildRecordsDTO(srh);
            }
		},
                
        setExclusionList: function(exclusionList) {
            this.exclusionMap= {};
            for (var i=0; i<exclusionList.length; i++) {
                var id= exclusionList[i].buildRecord?exclusionList[i].buildRecord.id:exclusionList[i].id;
                this.exclusionMap[id]= true;
            }
        },		
		
        _onCancel: function() {
            //this.reset();
        },
        
        _onOK: function() {
			if (dojo.isFunction(this.onOk)) {
				this.onOk(this._list.getSelectedRows());
			}
        },		
		
		_onTableRowsChecked: function() {
			//if any rows in the table are selected, make sure the Ok button is enabled
			var selectedRows = this._list.getSelectedRows();
            this._dialog.getOkButton().disabled = !(selectedRows && selectedRows.length > 0);
		},
		
		_setItems: function(items) {
			items = this._filterExcluded(items);
            this._list.refreshTable(items);          
            dojox.data.dom.removeChildren(this._itemListArea);
            this._itemListArea.appendChild(this._list.domNode);
            this._clearStatusMessage();
            this._dialog.getOkButton().disabled = true;
            this._dialog.updatePosition();			
		},
		
		_filterExcluded: function(items) {
            items = items || [];
			var filtered = [];
            for (var i = 0; i < items.length; i++) {
                // don't add an entry for the item if it is explicitly excluded
                if (!this.exclusionMap[items[i].id]) {
                    filtered.push(items[i]);
                }
            }
			return filtered;
		},
		
		_queryError: function(errorObj) {
			this._setStatusMessage(FETCH_ERR_MSG, true);
		},
		
        _clearStatusMessage: function() {
            this._msgArea.innerHTML= "";
        },
        
        _setStatusMessage: function(msg, isError) {
            this._msgArea.innerHTML= msg;
            this._msgArea.className= isError ? "ErrorStatus" : "NormalStatus";
        }
		
	}
);

})();
