/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading com.ibm.rqm.planning.web.ui.internal.view.common.ActionBar");
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.common.ActionBar");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Toolbar");

dojo.require("com.ibm.team.repository.web.client.init.InitializationData");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

function userIsAdmin()
{
	var InitializationData    = com.ibm.team.repository.web.client.init.InitializationData;
	var permissionInitializer = InitializationData.get("com.ibm.team.repository.service.canGrantPermissionInitializer");
	if (permissionInitializer != null && permissionInitializer.canGrantPermission)
	{
		return true;
	}
	else
	{
		return false;
	}
}

(function() 
{
	var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	
	dojo.declare(
		"com.ibm.rqm.planning.web.ui.internal.view.common.ActionBar",
		[dijit._Widget, dijit._Templated],
		{
			templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/common/ActionBar.html"),		
	
            // If asMenuBar is true the action bar will be horizontal in the global
            // task bar area as a menu bar.  Otherwise it will be vertical at the
            // left of the screen.
            //
            asMenuBar: false,
            
            tabIndex: 1,
            
			constructor: function()
			{
                // keep track of all entries to support select with arrow keys traversal
                this.actionEntryCount = 0;
                this.allEntries = [];
			},
	
			postCreate: function(args)
			{
			},
			
			initialize: function(contentArea, popupMenuArea, actionDispatcherName, asMenu)
			{
				this._contentArea = contentArea;
				this._popupMenuArea = popupMenuArea;
                this.actionDispatcherName = actionDispatcherName;
                
                this.asMenuBar = asMenu;

                this._isMenuShowing = false;
                
				this._actionBarRegistry = dojo.getObject(Constants.ACTION_BAR_REGISTRY_OBJECT_ID);
				if (!this._actionBarRegistry) {
					throw new Error("ActionBarRegistry has not been properly initialized!");
				}
				//set callback on registry; called when registry initialization is completed
				//will be immediately called synchronously in most cases unless loading 
				//of contributed ActionBarEntry widgets is slow
				this._actionBarRegistry.setInitCallback(dojo.hitch(this, "_initialize"));
			},
            
            getMenuBar: function() {
                return this._menuBar;
            },
            
			_initialize: function()
			{
				var args =
				{
					contentArea: this._contentArea,
					popupMenuArea: this._popupMenuArea,
                    actionDispatcherName: this.actionDispatcherName,
                    asMenuBarEntry: this.asMenuBar,
                    parent: this
				};
				
				var entries = this._actionBarRegistry.getActionBarEntries();
				for (var i = 0; i < (entries.length || 0); i++) {
	                var entry = entries[i];       
					args.actionMenuDTO = entry;             
                    //wrap widget creation in a try/catch to prevent any *bad*
					//contributions do not affect the overall UI	                   
	                try {
	                    var entryWidgetConstructor = dojo.getObject(entry.jsClass);
	                    var entryWidget = new entryWidgetConstructor();
	                    entryWidget.initialize(args);
	                    if (entry.hidden !== true) {
	                        this.appendActionEntry(entryWidget);
	                    }   
						console.debug(this.declaredClass + ": loaded ActionBarEntry widget: " + entry.jsClass);
	                } catch(e) {
	                    console.error(this.declaredClass + ": failed to load widget: [" + entry.jsClass + "] due to: " + e.message || "Unknown");
	                }
				}
				return;
			},
            uninitialize: function() {
            	//debugger;
            	var allEntries = this.allEntries;
            	var n = allEntries.length;
            	for (var i = 0; i < n; ++i) {
            		allEntries[i].destroyRecursive();
            	}
            },
            
            appendActionEntry: function(entryWidget)
            {
                if (this.asMenuBar) {
                    // realize the menu entry widget in the global nav bar of the common banner
                    entryWidget.topDomElement.realize();
                } else {
                    // Set tab index to 1 (default) for the action entry as first element to tab into.
                    // Elements with the same index will have order based on the order of their creation.
                    entryWidget.domNode.tabIndex = this.tabIndex;
                    // add entry to vertical action bar
                    this._actionBar.appendChild(entryWidget.topDomElement);
                    // keep track of the entries for keyboard traversal
                    this.allEntries.push(entryWidget);
                    entryWidget.entryIndex = this.actionEntryCount++;
                }
            },
            
            setCurrentActionEntry: function(currentWidget) {
                this.currentActionEntry = currentWidget;
            },
            
            clearCurrentActionEntry: function() {
                this.currentActionEntry = null;
            },
            
            resetCurrentActionEntry: function(requestWidget) {
                if (this.currentActionEntry && (this.currentActionEntry != requestWidget)) {
                    this.currentActionEntry._setHoverImage(false);
                    this.currentActionEntry = null;
                } else console.debug("SKIPPING");
            },
                
            selectPrevActionEntry: function(currentWidget) {
                // check previous entry index
                var prevIndex = currentWidget.entryIndex - 1;
                if (prevIndex < 0) {
                    // loop around to the last index
                    prevIndex = this.actionEntryCount - 1;
                }
                this.allEntries[prevIndex].topDomElement.focus();
            },

            selectNextActionEntry: function(currentWidget) {
                // check next entry index
                var nextIndex = currentWidget.entryIndex + 1;
                if (nextIndex >= this.actionEntryCount) {
                    // loop around to the first index
                    nextIndex = 0;
                }
                this.allEntries[nextIndex].topDomElement.focus();
            }
		}
	);
}
)();
