/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.execution.web.ui.internal.view.common.BlockingDefectsPopUp");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.execution.web.ui.internal.view.table.BlockingPopUpDefectTableView");
dojo.require("com.ibm.rqm.defects.web.client.internal.CmIntegrationClient");
dojo.require("com.ibm.rqm.defects.web.ui.view.common.ProgressDialog");
dojo.require("com.ibm.rqm.defects.web.client.ChangeRequestManager");
dojo.require("com.ibm.team.repository.web.foundation.Status");
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");

(function() {
	
var Constants						= com.ibm.rqm.planning.web.client.internal.Constants;
var BlockingPopUpDefectTableView	= com.ibm.rqm.execution.web.ui.internal.view.table.BlockingPopUpDefectTableView;
var AlertDialog 					= com.ibm.asq.common.web.ui.internal.widgets.AlertDialog;
var ProgressDialog              	= com.ibm.rqm.defects.web.ui.view.common.ProgressDialog;
var Status                      	= com.ibm.team.repository.web.foundation.Status;
var CmIntegrationClient 			= com.ibm.rqm.defects.web.client.internal.CmIntegrationClient;
var ChangeRequestManager			= com.ibm.rqm.defects.web.client.ChangeRequestManager;
var CSHelp 							= com.ibm.asq.common.web.ui.internal.util.CSHelp;

dojo.declare("com.ibm.rqm.execution.web.ui.internal.view.common.BlockingDefectsPopUp",[dijit._Widget, dijit._Templated], 
{
	templateString: '<div class= "com-ibm-asq-common-tableViewer" dojoAttachPoint="_fileContainer">' +
					'    <div class="" dojoAttachPoint="_messageNode"></div>' +
					'    <div class="com-ibm-asq-common-tableViewer table-content-node" dojoAttachPoint="_table">' +
					'        <table><thead><th width="10%"></th><th width="10%">${labelid}</th><th width="40%">${labelsummary}</th><th width="30%">${labelstate}</th></thead>' +
					'            <tbody dojoAttachPoint="_tableBody"></tbody>' +
					'        </table>'+
					'    </div>'+
					'</div>',
	
	labelok: Constants.getMessage("labelOk"),
	labelcancel: Constants.getMessage("labelCancel"),
	labeltitle: Constants.getMessage("editBlockingDefects"),
	labelsummary: Constants.getMessage("labelSummary"),
	labelid: Constants.getMessage("labelId"),
	labelstate: Constants.getMessage("labelState"),
	
	postCreate: function(){
		this._messageNode.innerHTML = Constants.getMessage("labelSelectNonBlockingDefect");
	},

	initialize: function(args){
        this.ewis = args.ewis;
		this.defects = args.defects;
		this.okCallback = args.okCallback;
		this.cancelCallback = args.cancelCallback;
		this.addedDefectIds = [];
		this.removedDefectIds = [];
		this._showAsDialog();
		this.showDefects();
	},
	
	_showAsDialog: function() {
        var dialogArgs = {
			primaryTitle:this.labeltitle,
            okCallback: dojo.hitch(this, "_okClick"),
            cancelCallback: dojo.hitch(this, "_cancelClick"),
			style: "z-index: 990;",
			zIndex: 990
        };
        
        this._dialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(dialogArgs);
        this._dialog.setWidth("800px");
		this._dialog.setHeight("400px");
		var cshelp = new CSHelp({contextID: "rqm_manage_blocking_defs.html"});
		cshelp.setButtonLabel(Constants.getMessage("labelManageBlockingDefectsCSHelp"));
		this._dialog.setSecondaryTitleNode(cshelp.domNode);
        this._dialog.getOkButton().disabled = false;
		this._dialog.open(true);		
	},
	
	showDefects: function() {
		if (ChangeRequestManager.INIT_INITIALIZED) {
			ProgressDialog.doRunWithProgress(Constants.getMessage("labelQueryAssociatedBlockingDefects"), dojo.hitch(this, function(monitor) {
				this.ewiItemIdArray = [];
				var ewis = this.ewis;
				for (var ewi in ewis){
					this.ewiItemIdArray.push(ewis[ewi].itemId);
				}
				var workItemIdentification = { 
							linkType: CmIntegrationClient.EWI_BLOCKING_DEFECT,
							isExecutionResultRelated: false, 
							itemIdOrUriArray: this.ewiItemIdArray 
					};
				CmIntegrationClient.postAssociatedWorkItems(workItemIdentification, dojo.hitch(this, function(status, response) {
					monitor.done();
					if	(status.isOK()){
						var ewiItemIdArr = this.ewiItemIdArray;
						var temp = [];
						if (dojo.isArray(response)){
							var responseLength = response.length;
							var ewiItemIdArrLength = ewiItemIdArr.length;
							for(var i = 0; i < ewiItemIdArrLength; i++){
								for(var j = 0; j < responseLength; j++){
									var t = temp[response[j].uri];
									if(t && response[j].sourceItemId == ewiItemIdArr[i] )
										t.count++;
									else if (response[j].sourceItemId == ewiItemIdArr[i]){
										temp[response[j].uri] = response[j];
										t = temp[response[j].uri];
										t.count = 1;
										t.isBlocking = true;
									}
								}
							}
						}
						var defects = [];
						for (var defect in temp) {
							if(temp[defect].count == ewiItemIdArr.length){
								defects.push(temp[defect]);
							}
						}
						var args= {
		                    ewis: this.ewis,
							defects: defects,
							parentEditor: this,
							pageEditor: this.parentEditor,
							okCallback: dojo.hitch(this, this._onOk),
							cancelCallback: dojo.hitch(this, this._onCancel)
						}
						this.defectTableView = new BlockingPopUpDefectTableView(args);
						dojox.data.dom.removeChildren(this._dialog.contentContainer);
						this._dialog.contentContainer.appendChild(this.defectTableView.domNode);
					} else if(!status.isOK() && status.getSeverity() != Status.CANCEL) {
						status.log();
						var alertDialog = new AlertDialog({
			                'message': Constants.getMessage("labelUnableCreateWorkItem", status.getMessage())
			            	});
		            	alertDialog.open();
					}
				}));
			}));
		} else {
			var args = {
				defects: this.defects, 
				parentEditor: this,
                pageEditor: this.parentEditor,
				ewis: this.ewis
			};
			
			this.defectTableView = new BlockingPopUpDefectTableView(args);
			this._dialog.contentContainer.appendChild(this.defectTableView.domNode);
		}
	},
	_appendInDefectList: function(data,type){
		var dataLength = data.length;
		var removedDefectIds = this.removedDefectIds;
		var defectsList = this.defects;
		for(var i=0;i<dataLength;i++){
			if(data[i].statusId == 5){//dont bother with closed defects
				var alertDialog = new AlertDialog({
                'message': Constants.getMessage("msgClosedDefect")
            	});
            	alertDialog.open();
				continue;
			}
			var found = false;
			var defectsLength = defectsList.length;
			for(var j=0;j<defectsLength;j++){
				if((!data[i].isRTCDefect && defectsList[j].id == data[i].id) || (data[i].isRTCDefect && defectsList[j].draftUri == data[i].draftUri)){
					found = true;
					break;
				}
			}
			if(!found){
				var defect = {id:data[i].id,status:data[i].status,summary:data[i].summary,isBlocking:true};
				if (data[i].isRTCDefect) {
					defect.draftUri = data[i].draftUri;
				}
				defectsList.push(defect);
				if(!data[i].isRTCDefect) 
					this.addedDefectIds.push(data[i].id);
				//if removed and then added again
				var len = removedDefectIds.length;
				for(var j=0;j<len;j++){
					if (removedDefectIds[j] == data[i].id) {
						removedDefectIds.splice(j, 1);
						break;
					}
				}
			}
		}
		this.defectTableView.setContent(defectsList);
	},
	_updateDefectStatus:function(defectId,blocking){
		if(!blocking){
			var defects = this.defects;
			var defectsLen = defects.length;
			for(var i=0;i<defectsLen;i++){
				if (defects[i].id == defectId) {
					defects.splice(i, 1);
					break;
				}
			}
			//if added and then removed
			var addedDefectIds = this.addedDefectIds;
			var addedDefectIdsLength = addedDefectIds.length;
			for(var j=0;j<addedDefectIdsLength;j++){
				if (addedDefectIds[j] == defectId) {
					addedDefectIds.splice(j, 1);
					break;
				}
			}
			this.removedDefectIds.push(defectId);
			this.defectTableView.setContent(this.defects);
		}
	},
	_okClick: function(){
		if(this.okCallback)
			this.okCallback(this.ewis,this.addedDefectIds, this.removedDefectIds );
	},
	
	_cancelClick: function(){
		if(this.cancelCallback)
			this.cancelCallback();
	},
	
	_error: function(errorObj){
		console.error("Error in BlockingDefectsPopUp "+errorObj.message);
	},
	
	_closePopupDialog: function(){
		if(this._dialog)
			this._dialog.close();
	}
});

})();
	
