/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 /*
 * Author: M Brown
 *
 * Summary:
 *   A panel with a header and content sections that can be expanded and collapsed.  
 *   This is a minimal subset of what's provided by the CollapsibleSection widget
 *   
 *     /-----------+---------------+---------------------------------\    \
 *     | titleNode | titleHelpNode                                   |    |  title
 *     +-----------+---------------+---------------------------------+    |
 *     |                                                             |    |
 *     | containerNode                                               |     > content
 *     |                                                             |    |
 *     \-------------------------------------------------------------/    /
 *
 *   Expand and collapse is controlled by clicking the titleNode or pressing return
 *   key when the titleNode is in focus.
 */
dojo.provide("com.ibm.asq.common.web.ui.internal.view.common.CollapsibleSubSection");

dojo.require("dijit.TitlePane");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.InlineEditor");

(function() {
dojo.declare("com.ibm.asq.common.web.ui.internal.view.common.CollapsibleSubSection",
dijit.TitlePane,
{
    templateString: '<div id="${id}" dojoAttachPoint="domNode" class="com-ibm-asq-common-web-ui-collapsible-sub-section">' +
                    '  <div class="sub-header" dojoAttachPoint="focusNode">' +
                    '     <div class="sub-top-border" dojoAttachPoint="topNode">' +
                    '     <div class="sub-title-container">' +
                    '        <span dojoAttachPoint="titleNode" class="sub-title"' +
                    '              dojoAttachEvent="onclick:_onTitleClick,onkeypress:_onTitleKey" tabindex="0" waiRole="button">' +
                    '        </span>' +
                    '        <span dojoAttachPoint="viewEditableTitle" dojoType="com.ibm.asq.common.web.ui.internal.widgets.InlineEditBox" value="${title}" class="sub-title"></span>' +                    
                    '        <span dojoAttachPoint="titleHelpNode" class="sub-title-help"></span>' +                    
                    '     </div>' +
                    '     <div class="sub-content-outer-container" dojoAttachPoint="hideNode">' +
                    '        <div class="sub-content-inner-container" dojoAttachPoint="wipeNode">' +                    
                    '           <div dojoAttachPoint="containerNode" waiRole="region" tabindex="-1"></div>' +
                    '        </div>' +
                    '     </div>' +
                    '  </div>' +
                    '</div>',

    titleEditable: false, //optionally make the title editable
    open: false,  // default section to be collapsed
    widgetsInTemplate: true,

    postCreate: function() {
        this.arrowNodeInner = document.createElement("div");
        this.inherited('postCreate', arguments);
        this.setTitleMode(this.titleEditable);
    },
    
    // change handler for editable title
    editableTitleChange: function(value) {
        this.setTitle(value);
        if (this.onTitleEdited && dojo.isFunction(this.onTitleEdited)) {
            this.onTitleEdited(value);
        }
    },  
	
	setTitleMode: function(editable) {
        this.titleEditable = editable;
        if (this.titleEditable) {
            this.viewEditableTitle.onChange = dojo.hitch(this, "editableTitleChange");
        }
        this.viewEditableTitle.domNode.style.display = this.titleEditable ? "inline" : "none";
       	this.titleNode.style.display = this.titleEditable ? "none" : "inline";         
    },   
    
    getTitle: function() {
        return this.title;
    },    
    
    setTitle: function(title) {
        this.title = title;
        if (this.titleNode)
            this.titleNode.innerHTML = this.title ? this.title : "";
        if (this.viewEditableTitle)
            this.viewEditableTitle._setDisplayValue(this.title ? this.title : "");
    },
    
    setContent: function(contentNode) {
        dojox.data.dom.removeChildren(this.containerNode);
        this.content = contentNode;
        this.containerNode.appendChild(contentNode);
    },
    
    _onTitleClick: function() {
        this.toggle();
    },
    
    _setCss: function() {
        this.inherited('_setCss', arguments);
        
        // Need to toggle the background style to clear when collapsed.
        // Tried to append the class dijitOpen or dijitClosed to the
        // node and toggle between them, but the class for repeat-x
        // background image always got ignored.
        //
        if (this.open) {
            dojo.removeClass(this.containerNode, "sub-content-area-closed");
            dojo.addClass(this.containerNode, "sub-content-area-open");
            dojo.removeClass(this.titleHelpNode, "sub-title-help-closed");
            dojo.addClass(this.titleHelpNode, "sub-title-help-open");            
            //this.bottomNode.style.display = "none";
        } else {
            dojo.removeClass(this.containerNode, "sub-content-area-open");
            dojo.addClass(this.containerNode, "sub-content-area-closed");
            dojo.removeClass(this.titleHelpNode, "sub-title-help-open");
            dojo.addClass(this.titleHelpNode, "sub-title-help-closed");            
            //this.bottomNode.style.display = "block";
        }
    }
       
});
})();