/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
// Summary:
//    The configuration cell renderer provides an clickable text to list configuration detail infomation.
//
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.common.ConfigurationCellRenderer");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.TooltipBuilder");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function(){
	var Constants = com.ibm.rqm.planning.web.client.internal.Constants;

    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.common.ConfigurationCellRenderer", null, {
        // Summary:
        //    Renders the cell content. For renderer that can modify the cell value,
        //    
        renderCell: function(viewer, cellData, extendedDisplayInfo, /*table.CellModifier*/ cellModifier, rowObject, columnIndex, cellWidthHint){
            var node;
            
            var doc = document;
            node = doc.createElement("a");
            var displayNode = doc.createElement("div");
            var text = cellData;
            if(viewer.isCompactMode && typeof cellWidthHint != "undefined") {
                text = this._handleCompactCellText(viewer, text, extendedDisplayInfo, displayNode, cellWidthHint);
            }
            displayNode.appendChild(doc.createTextNode(text));
            
            node.appendChild(displayNode);
            node.className = "action-link";
            
            node.cellModifier = cellModifier;
            node.rowObject = rowObject;
            node.columnIndex = columnIndex;
            var displayDetails = false;
            this.connect(node, "onclick", function(e){
                if (displayDetails == false) {
					var catalog = new com.ibm.asq.common.web.client.internal.Catalog();
                    var configuration = new com.ibm.asq.common.web.client.internal._catalog.Configuration(extendedDisplayInfo.configuration);
                    var attributeIds = catalog.getDisplayTypes();
                    var attributeVals = [];
                    for (var i = 0; i < attributeIds.length; i++) {
                        var attribute = configuration.getValue(attributeIds[i]);
                        if (!attribute) {
                            attributeVals.push(Constants.getMessage("labelNotConfig"));
                        }
                        else 
                            attributeVals.push(attribute)
                    }
                    var table = doc.createElement("table");
                    for (var i = 0; i < attributeIds.length; i++) {
                        var tr = doc.createElement("tr");
                        var typeTD = doc.createElement("td");
                        
                        // catalog types are returned in an object identifying a type
                        // It has these fields: name, path, template.
                        var typeName = Constants.getLmMessage(attributeIds[i].name);
                        var typeTextNode = doc.createTextNode(typeName + ":");
                        typeTD.appendChild(typeTextNode);
                        tr.appendChild(typeTD);
                        var valueTD = doc.createElement("td");
                        var valueTextNode = doc.createTextNode(attributeVals[i]);
                        valueTD.appendChild(valueTextNode);
                        tr.appendChild(valueTD);
                        table.appendChild(tr);
                    }
                    e.target.parentNode.appendChild(table);
                    displayDetails = true;
                }
                else {
                    e.target.parentNode.removeChild(e.target.parentNode.lastChild);
                    displayDetails = false;
                }
            });
            return node;
        }
    });
    
})();
