/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading: com.ibm.rqm.planning.web.ui.internal.view.common.EditableItemList");
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.common.EditableItemList");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

// Require the common widgets needed
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

// Require local widgets
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.TooltipBuilder");

dojo.require("com.ibm.rqm.planning.web.client.internal.construction.TestGroupClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.TestScriptClient");

dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.CollapseExpandArea");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.SelectItemDialog");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.SelectItemTableDialog");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.ImageCache");

dojo.require("com.ibm.rqm.planning.web.ui.internal.model.SaveContext");
dojo.require("com.ibm.rqm.planning.web.ui.internal.model.SaveParticipant");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.EditableItemSaveParticipant");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function() { // module container

/* Jazz widgets */
var getAuthenticatedContributor	= com.ibm.team.repository.web.client.session.getAuthenticatedContributor;

/* QM widgets */
var Constants    			   = com.ibm.rqm.planning.web.client.internal.Constants;
var Utility				      = com.ibm.rqm.planning.web.ui.internal.util.Utility;
var EditorUtil     			= com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
var TooltipBuilder   		= com.ibm.rqm.planning.web.ui.internal.util.TooltipBuilder;

var ServiceResponseHandler	= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var CollapseExpandArea 		= com.ibm.rqm.planning.web.ui.internal.view.common.CollapseExpandArea;
var SelectItemDialog 		= com.ibm.rqm.planning.web.ui.internal.view.common.SelectItemDialog;
var SelectItemTableDialog	= com.ibm.rqm.planning.web.ui.internal.view.common.SelectItemTableDialog;
var ImageCache				   = com.ibm.rqm.planning.web.ui.internal.view.common.ImageCache;
var Constants				= com.ibm.rqm.planning.web.client.internal.Constants;

//QM Clients
var TestGroupClient		    = com.ibm.rqm.planning.web.client.internal.construction.TestGroupClient;
var TestCaseClient		    = com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient;
var TestScriptClient		    = com.ibm.rqm.planning.web.client.internal.construction.TestScriptClient;

// global save context
var SaveContext				= com.ibm.rqm.planning.web.ui.internal.model.SaveContext;
var SaveParticipant			= com.ibm.rqm.planning.web.ui.internal.model.SaveParticipant;

var ItemSaveParticipant    = com.ibm.rqm.planning.web.ui.internal.view.common.EditableItemSaveParticipant;

var EMPTY_MESSAGE_ROW_ID           = "emptyMessageRow";

/**
 * 
 */
dojo.declare(
	"com.ibm.rqm.planning.web.ui.internal.view.common.EditableItemList",
	[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/common/EditableItemList.html"),
//		templateCssPath: dojo.uri.dojoUri("../com.ibm.rqm.planning.web/ui/internal/view/templates/common/EditableItemList.css"),

		labelName: Constants.getMessage("labelName"),
		labelDescription: Constants.getMessage("labelDescription"),
		labelOwner: Constants.getMessage("labelOwner"),
		labelLoading: Constants.getMessage("labelLoading"),
		
		constructor: function(args) {		
			//widget level attributes
			this.parentController = null;
			this.editorFactory = null;
			this._currentView = null; 
			this._itemType = null;
			this._collapsable = false;
			this._showTitleRow = true;
			this._itemSectionWidgets = []; //{index: position in array, widget: collapsableAreaWidget}
			this._contributors = []; 
			this._items = []; 
			this._itemsAdded = [];
			this._itemsRemoved = [];
			this._addEnabled = false;	
			this._newEnabled = false;
			this._linkedRows = false;
			this._childCollapseExpandArea = [];
			this._excludeFromAdd = [];
			this._modelParent = null;
			this._referencesAddedType = null;
			this._referencesRemovedType = null;
			this._initializing = false;
			this._saveParticipants = {};
		},
		
		// populate the template with data, connect events, and show initial
		// sub-tab
		postCreate: function() {
		},		
		
		initialize: function(args) {
			this._initializing = true;
			
			//initialize widget
			if (args.parentController == null) {
				throw new Error("EditableItemList#postCreate: Missing required parameter 'args.parentController'");
			} else if (!args.parentController["getEditorFactory"]) {
				throw new Error("EditableItemList#postCreate: parentController MUST provide access to " 
					+ "an EditorFactory by implementing the 'getEditorFactory' function.");
			}
			if (args.itemType == null) {
				throw new Error("EditableItemList#postCreate: Missing required parameter 'args.itemType'");
			}		
			this.parentController = args.parentController;
			this.editorFactory = this.parentController.getEditorFactory();
			
			this.parent = args.parent;
			this._itemType = args.itemType;
			this._modelParent = args.modelParent;
			this._referencesAddedType = args.referencesAddedType;
			this._referencesRemovedType = args.referencesRemovedType;
			this.parentEditor = args.parentEditor ? args.parentEditor : null;
			this._addEnabled = args.addEnabled != null && args.addEnabled == true;
			this._newEnabled = args.newEnabled != null && args.newEnabled == true;
			this._collapsable = args.collapsable != null && args.collapsable == true;
			this._showTitleRow = args.showTitleRow != null && args.showTitleRow == false ? false : true;
			this._showTwistie = args.showTwistie != null && args.showTwistie == false ? false : true;
			
			if ( args.itemList != null && args.itemList.length > 0 ) {
				this._items = [];
				for (var i = 0; i < args.itemList.length; i++) {
					this._items.push(args.itemList[i]);
				}
				this._items.sort(function(a,b) { return a.name.toLowerCase() < b.name.toLowerCase() ? -1 : 1; } );
			}
			
			if (args.excludeFromAdd && args.excludeFromAdd.length > 0) {
				for (var i = 0; i < args.excludeFromAdd.length; i++) {
					this._excludeFromAdd.push(args.excludeFromAdd[i]);
				}
			}			
			
			//show the title and items			
			if (this._showTitleRow) {
				this._appendTitleRow();
			} else {
				this._hideTitleRow();
			}
			this._initItems();
			this._initializing = false;
		},		
		
		////////////////////////////////
		// API
		////////////////////////////////
		getItems: function() {
			return this._items;
		},
			
		getItemsAdded: function() {
			return this._itemsAdded;
		},		

		getItemsRemoved: function() {
			return this._itemsRemoved;
		},
		
		clearItemsAdded: function() {
			this._itemsAdded = [];
		},
		
		clearItemsRemoved: function() {
			this._itemsRemoved = [];
		},
		
		showItems: function() {							
			//clear the appropriate section of the widget
			if ( this._collapsable ) {
				dojox.data.dom.removeChildren(this._itemList);	
			} else {
				dojox.data.dom.removeChildren(this._itemListTableBody);
				if ( this._items.length == 0 ) {
					this._appendNoItemsMessageRow();
				}		
			}
			//populate the widget with the items
			if ( this._items.length > 0 ) {						
				for(var i = 0; i < this._items.length; i++) {
					this._appendItem(this._items[i]);
				}				
			}
		},
				
		collapseAll: function() {
			if (this._itemSectionWidgets != null && this._itemSectionWidgets.length > 0) {
				for (var i = 0; i < this._itemSectionWidgets.length; i++) {
					this._itemSectionWidgets[i].collapse();
				}
			}
		},
		
		expandAll: function() {
			if (this._itemSectionWidgets != null && this._itemSectionWidgets.length > 0) {
				for (var i = 0; i < this._itemSectionWidgets.length; i++) {
					this._itemSectionWidgets[i].expand();
				}
			}
		},		
		
		
		////////////////////////////////
		// Internal
		////////////////////////////////					
		_appendTitleRow: function() {
			//dojox.data.dom.removeChildren(this._itemListTitleRow);
         dojox.data.dom.removeChildren(this._actions);
			if ( this._newEnabled ) {
				var newLink = document.createElement("a");
				newLink.className = "actionLink";
				newLink.title="New " + this._itemType;
				newLink.appendChild( document.createTextNode(newLink.title) );				
				this.connect(newLink, "onclick", this._handleAddNewClick);	
				this._actions.appendChild(newLink);//td.appendChild(newLink);								
			}	
			
			if ( this._newEnabled && this._addEnabled ) {
				//td.appendChild(document.createTextNode("   "));
				this._actions.appendChild(document.createTextNode("   "));
			}			
			if ( this._addEnabled ) {				
				var addLink = document.createElement("a");
				addLink.className = "actionLink";
				addLink.title="Add Existing";
				addLink.appendChild( document.createTextNode(addLink.title) );				
				this.connect(addLink, "onclick", this._handleAddClick);	
				this._actions.appendChild(addLink);//td.appendChild(addLink);
			}
			//this._title.appendChild(links);			
         this._title.innerHTML = this._itemType + "s";
		},
		
		_hideTitleRow: function() {
			dojo.addClass(this._itemListTitleRow, "hidden");
         //this._appendTitleRow();
		},
		
		_appendNoItemsMessageRow: function() {
			var row = this._getEmptyRow();
			var cell = this._getEmptyCell();				
			cell.colSpan = dojo.byId(this._itemListTableHeaderRow).childNodes.length - 1;			
			cell.appendChild(document.createTextNode("No associated " + this._itemType + "s."));
         dojo.addClass(cell, "item-table-empty-message");
			row.appendChild( cell );
			row.id = "emptyMessageRow";
			this._itemListTableBody.appendChild(row);
		},		
		
		_initItems: function() {
			//show items after appropriately initializing view			
			if ( this._collapsable ) {
				dojox.data.dom.removeChildren(this._itemTableContainer);
			} else {
				dojox.data.dom.removeChildren(this._itemListContainer);
			}
			this.showItems();		
		},		
		
		_getOwnerName: function(item) {
			if ( item != null) { 
				if (item.owner != null ) {
					return item.owner;
				} else if ( item.workItems != null && item.workItems[0] != null ) {
					return item.workItems[0].ownerName;
				} 
			} 
			return Constants.getMessage("labelUnassigned");
		},	

		_appendItem: function(item) {
			if (this._collapsable) {
				this._appendItemSection(item);
			} else {
				this._appendItemRow(item);
			}
		},
		
		_appendItemRow: function(item) {
			if ( this._items.length == 1 ) {
				dojox.data.dom.removeChildren(this._itemListTableBody);
			}
			var tr = document.createElement("tr");
			tr.className = "item-table-row";	
			var uri = Utility.getArtifactURI(item, Utility.getCurrentPerspectiveByURI());
			this.connect(tr, "onclick", function() { window.location.href=uri});				
			
			this._addColumn(tr, item.id, "id-column" );
			this._addColumn(tr, item.name + (item.snapshotInfo ? "(Snapshot: " + item.snapshotInfo.name + ")" : "") );
			this._addColumn(tr, item.description);
			this._addColumn(tr, this._getOwnerName(item));
			
			// SaveParticipant for the item
			var versionableItemId = item.versionableItemId ? item.versionableItemId : null;
			var itemId = item.versionableItemId ? item.versionableItemId : item.itemId;
			var saveParticipant = SaveContext.findParticipant(itemId);
			if (saveParticipant == null)
			{
				saveParticipant = new SaveParticipant();
				var itemType = Utility.getModelType(item);
				saveParticipant.setDirty(false);
				saveParticipant.getItemId = function () { return itemId; };
				saveParticipant.getVersionableItemId = function() { versionableItemId };
				saveParticipant.getItemType = function() { return itemType; };
				SaveContext.addParticipant(saveParticipant);
			}
			var parent = this._modelParent;
			// treat it as a new reference if the parent is a copy
			if (parent!= null && parent.isCopy!=null && parent.isCopy.call(parent)) {
				SaveContext.createReference(parent, this._referencesAddedType, saveParticipant);
			}
			else if (this._containsItemId(this._itemsAdded, itemId) && parent!=null)
			{
				SaveContext.createReference(parent, this._referencesAddedType, saveParticipant);
			}
			
			if (!this._modelParent.isReadOnly()) {
				var removeImg = document.createElement("img");
				removeImg.title = "Remove " + this._itemType;
				removeImg.alt= "Remove " + this._itemType;
				removeImg.src = ImageCache.REMOVE_DISABLED;
				removeImg.className = "type-image";	
				
				this.connect(removeImg, "onclick", this._onRemoveItem);	
				this.connect(removeImg, "onmouseover", this._overRemove);
				this.connect(removeImg, "onmouseout", this._offRemove);	
			   this._addElementColumn(tr, removeImg, "right"); 			
			} else {
				this._addElementColumn(tr, document.createElement("span"), "right");   
			}
						
			this._itemListTableBody.appendChild(tr);			
		},		
		
		_appendItemSection: function(item) {
			//var itemEditor = this._getArtifactEditor(this._getItemById(item.id), this._itemSectionWidgets.length);
			var listener = {index: this._itemSectionWidgets.length, obj: this, method: this.notifyOfUpdatedItem};
			var itemEditor = this.editorFactory.getArtifactEditor(item, this._itemType, this, listener);
			if (itemEditor.isASaveParticipant)
			{
				SaveContext.addParticipant(itemEditor);
				var itemId = item ? (item.versionableItemId ? item.versionableItemId : item.itemId) : null;
				var parent = this._modelParent;
				// treat it as a new reference if the parent is a copy
				if (parent!= null && parent.isCopy!=null && parent.isCopy.call(parent)) {
					SaveContext.createReference(parent, this._referencesAddedType, itemEditor);
				}
				else if (parent!= null && (itemId == null || this._containsItemId(this._itemsAdded, itemId)))
				{
					SaveContext.createReference(parent, this._referencesAddedType, itemEditor);
				}
			}
			var itemTooltipHTML = item ? TooltipBuilder.getArtifactTipHTML(item) : null;
			
			var label;
			if (item!=null) {
				label = item.name;
				if (item.snapshotInfo != null) {
					label += " (Snapshot " + item.snapshotInfo.name + ")";
				}
			} else {
				label = "New " + this._itemType;
			}
			var args = {
				/*id: item.id,*/ 
				label: label, 
				labelTooltip: itemTooltipHTML ? itemTooltipHTML : null, 
				icon: Utility.getArtifactIconURL(this._itemType), 
				sectionWidget: itemEditor, 
				//if null item (new), section should be open
				open: item ? false : true,			
				editorOpen: item ? false : true,
				showTwistie: this._showTwistie, 
				flagStatus: null
			}
			 
			var collapsableAreaWidget = new com.ibm.rqm.planning.web.ui.internal.view.common.CollapseExpandArea();
			collapsableAreaWidget.initialize(args);
			this._appendActionsToItemSection(collapsableAreaWidget, itemEditor, item);
			this._itemSectionWidgets.push(collapsableAreaWidget);
			this._itemList.appendChild(collapsableAreaWidget.domNode);
			
			if (!item) {
				setTimeout(dojo.hitch(this,this._setFocus),200);
			}		
		},	
		
		//set focus to the editor in the most recently added section
		_setFocus: function() {
			if (this._itemSectionWidgets.length > 0) {
				var section = this._itemSectionWidgets[this._itemSectionWidgets.length - 1];
				var editor = section ? section.sectionWidgets[0] : null;
				if (editor) {
					editor._name.focus();
				}
			}
		},
		
		_appendActionsToItemSection: function(sectionWidget, itemEditor, item) {
         var self = this;
		 
			 var itemId = null;
			 if (item!=null) {
				 itemId = item.versionableItemId!=null ? item.versionableItemId : item.itemId;
			 }
			 var removeObj = { remove: function() { self._onRemoveItem(itemId); } };
			
			if(this._itemType == Constants.TYPE_GROUP || this._itemType==Constants.TYPE_CASE) {				
				var ewiActionObj = {
					generateEWI: function() {
						self._generateEWI(item);
					}
				}
				sectionWidget.registerAction(
				  Constants.getMessage("labelGenEWIs"), null, ewiActionObj, ewiActionObj.generateEWI
            );
			}
			if (itemEditor.isReadOnly!=null && !(itemEditor.isReadOnly.call(itemEditor))) {
				sectionWidget.registerAction("Discard Changes", null, itemEditor, itemEditor.onCancel);
			}
			var parent = this._modelParent;
			if (parent!=null && parent.isReadOnly!=null && !(parent.isReadOnly.call(parent))) {
				sectionWidget.registerAction("Remove", null, removeObj, removeObj.remove);
			}			
		},
		
	   _generateEWI: function(item) { 
	      if (item && item.id) {                         
	         var self = this;
	         var args = {
	            title: Constants.getMessage("labelGenEWIs"),		// CML: term
	            parentController: self.parentController,
	            callback: {context: self, method: self.update},
	            artifact: item
	         }  
	         this._ewiWizard = new GenerateEWIWizard(args);
	
	      } else {
	            alert(Constants.getMessage("msgSaveToGenerateConfigurations", this._itemType));
	      }                    
	   },		
		
		_findSectionForItem: function(item) {
			if ( this._itemSectionWidgets != null && this._itemSectionWidgets.length > 0 ) {
				for ( var i = 0; i < this._itemSectionWidgets.length; i++ ) {
					var sectionWidget = this._itemSectionWidgets[i];
					var itemId = item.versionableItemId!=null ? item.versionableItemId : item.itemId;
					var sectionId = sectionWidget.versionableItemId!=null ? sectionWidget.versionableItemId : sectionWidget.itemId;
					if ( sectionId == itemId || sectionWidget.label == item.name) {
						return sectionWidget;
					}
				}
			}
		},		
		
		_overRemove: function(e)
		{
			e.target.src = ImageCache.REMOVE_ENABLED;
		},
		
		_offRemove: function(e)
		{
			e.target.src = ImageCache.REMOVE_DISABLED;
		},		
		
		_addColumn:function(row, data) {
			var td = document.createElement("td");			
			td.className="item-table-cell";		
			td.appendChild( document.createTextNode(data) );
			row.appendChild(td);
		},
			
		_addElementColumn:function(row, element, align) {
			var td = document.createElement("td");
			td.appendChild(element);
			if ( align != null ) {
				td.align = align;
			}			
			row.appendChild(td);
		},			
		
		_handleRowClick: function(e) {
			e.preventDefault();
			var row = e.currentTarget;
			var tbody = row.parentNode; 
			var rowId = row.sectionRowIndex;
			var artifactId = e.currentTarget.firstChild.childNodes[0].data;
			
			var editorRow = tbody.insertRow(rowId + 1);
			var editorCell = editorRow.insertCell(0);
			editorCell.colSpan = row.childNodes.length;
			var editor = this.editorFactory.getArtifactEditor(this._getItemById(artifactId), this._itemType, this);
			if (editor != null) {
				editorCell.appendChild(editor.domNode);
				if (editor.isASaveParticipant)
				{
					SaveContext.addParticipant(editor);
					if (this._initializing==false && this._modelParent!=null)
					{
						SaveContext.createReference(this._modelParent, this._referencesAddedType, editor);
					}
				}
			} else {
				alert(Constants.getMessage("labelNoEditorConfigured"));
			}
		},
		
		notifyOfUpdatedItem: function(item, itemIndex) {
			//update the title of the collapsable section			
			var itemId = item.versionableItemId ? item.versionableItemId : item.itemId;
			var index = this._getItemIndexById(itemId);
			if ( index > -1 ) {
				this._items[index] = item;	
			} else {
				this._items.push(item);
				this._itemsAdded.push(itemId);	
			}			
			var itemSection = this._itemSectionWidgets[itemIndex];
			if ( itemSection != null ) {
				itemSection.versionableItemId = itemId;
				itemSection.label = item.name;
				if(this._itemType == Constants.TYPE_PLAN_SIZING)
					itemSection.open = true;
				itemSection.open = itemSection.isExpanded();
				itemSection._initializeWidget();
			}
		},
		
		_getEmptyRow: function() {
			var row = document.createElement("tr");
			row.className = "item-table-row";
			return row;
		},
		
		_getEmptyCell: function(span, align) {
			var cell = document.createElement("td");
			cell.className = "item-table-cell";
			if ( span != null ) {
				cell.colSpan = span;
			}
			if ( align != null ) {
				cell.align = align;
			}
			return cell;
		},		
		
		/*
		 * SECTION FOR ADDING ITEMS (NEW OR EXISTING) TO THE LIST
		 */		
		_appendNewItemLinkRow: function() {
			var row = this._getEmptyRow();
			this.connect(row, "onclick", this._handleAddNewClick);	
			var cell = this._getEmptyCell(5);
			cell.appendChild(document.createTextNode(Constants.getMessage("labelNewItemClick", this._itemType)));	
			row.appendChild(cell);
			this._itemListTableBody.appendChild(row);
		},		
		
		_createNewItemSaveParticipant: function(index, nameInput, descInput, ownerInput) {
         // SaveParticipant for the item
			var args = {
            parent: this,
            itemType: Utility.getModelTypeByItemType(this._itemType),
            //artifact: null,				
				index: index,
				name: nameInput,
				description: descInput,
				owner: ownerInput
			}
			var itemParticipant = new ItemSaveParticipant(args);
			SaveContext.addParticipant(itemParticipant);
			SaveContext.createReference(this._modelParent, this._referencesAddedType, itemParticipant);
		},
		
      _documentSaveSupported: function() {
         var type = this._itemType;
         return (type == Constants.TYPE_PLAN || type == Constants.TYPE_GROUP || type == Constants.TYPE_CASE);
      },
				
		_appendNewItemRow: function() {
			var tr = this._itemListTableBody.insertRow(this._itemListTableBody.rows.length);
			var index = tr.sectionRowIndex;
			tr.className = "item-table-row";
			
			//add a div to the id column (will show id when generated)
			var newIdCol = document.createElement("div"); 
			newIdCol.id = "_newItemId" + index;
			this._addElementColumn(tr, newIdCol );
			
			//name entry column
			var inputLabel = document.createElement("label");
			inputLabel.setAttribute("for", "_newItemName" + index);
			var newNameCol = document.createElement("input");
			newNameCol.type = "text";
			newNameCol.size = "30";
			newNameCol.maxLength = "50";
			newNameCol.id = "_newItemName" + index;
			newNameCol.title = "New Name";
			inputLabel.appendChild(newNameCol);
			//this.connect(newNameCol, "onchange", this._doSave);
			this._addElementColumn(tr, inputLabel );
			
			//description entry column
			var inputLabel = document.createElement("label");
			inputLabel.setAttribute("for", "_newItemDesc" + index);
			var newDescCol = document.createElement("input");
			newDescCol.type = "text";
			newDescCol.size = "50";
			newDescCol.maxLength = "255";								
			newDescCol.id = "_newItemDesc" + index;
			newDescCol.title = "New Description";
			inputLabel.appendChild(newDescCol);
			//this.connect(newDescCol, "onchange", this._doSave);
			this._addElementColumn(tr, inputLabel );
						
			//owner entry column	
			var ownerSelect = document.createElement("select");
			ownerSelect.id = "_newItemOwnerId" + index;
			ownerSelect.title = "Owner";
			EditorUtil.initContributors(ownerSelect);
			this._addElementColumn(tr, ownerSelect);
			
			//actions column (save/delete)
			//var saveButton = document.createElement("img");
			//saveButton.src = ImageCache.SAVE;
			//saveButton.alt = "Save";
			//saveButton.title = "Save";		
			//this.connect(saveButton, "onclick", this._doSave);
			this._addElementColumn(tr, document.createElement("div"), "right");											
								
			this._itemListTableBody.appendChild(tr);	
			this._createNewItemSaveParticipant(index, newNameCol, newDescCol, ownerSelect);
			//this._appendNewItemLinkRow();
		},
				
		_handleAddClick: function(e) {						
			this._openItemsDialog();
		},
		
		_isEmpty: function() {
			var tbody = this._itemListTableBody;
			return (tbody && tbody.rows.length == 1 && tbody.rows[0].id == EMPTY_MESSAGE_ROW_ID);
		},

		_handleAddNewClick: function(e) {
			if ( this._collapsable ) {
				this._appendItemSection();
			} else {
				if (this._isEmpty()) {
					dojox.data.dom.removeChildren(this._itemListTableBody);
				}
				this._appendNewItemRow();	
			}		
		},
		
		_openItemsDialog: function() {
			var self= this;
			if (!this.itemListWidget) {
				var params = {
					title: Constants.getMessage("labelAddItems", this._itemType),
					parentElement: this.domNode,
					exclusionList: this._getAddExclusionList(),
					itemType: this._itemType,				
					onOk: function(selectedItems) {
						for (var i = 0; i < selectedItems.length; i++) {
							var item = selectedItems[i];
							var itemId = item.versionableItemId ? item.versionableItemId : item.itemId;
							if ( self._containsItemId(self._itemsRemoved, itemId) ) {
								self._itemsRemoved = self._removeItemId(self._itemsRemoved, itemId);
								if (self._modelParent!=null)
								{
									SaveContext.unremoveReference(self._modelParent, self._referencesRemovedType, itemId);
								}
							} else if ( !self._contains(item) ) {
								self._items.push(item);
								self._itemsAdded.push(itemId);								
							} else {
								alert(item.name + " is already part of the list!");
							}
							//append the item to the list
							self._appendItem(item);
						}						
					},
					
					onAddNew: function() {
						self._appendNewItem();
					}
				};
				if (this._itemType==Constants.TYPE_GROUP || this._itemType==Constants.TYPE_CASE) {
					this.itemListWidget = new SelectItemTableDialog(params);
				} else {
					this.itemListWidget = new SelectItemDialog(params);
				}
			}
			this.itemListWidget.setExclusionList(this._getAddExclusionList());
			this.itemListWidget.show();			
		},		
		
		_getAddExclusionList: function() {
			var excludedItems = [].concat(this.getItems(), this._excludeFromAdd);
			var excludedItemIds = [];
			
			for ( var i = 0; i < excludedItems.length; i++) {
				var excludedItem = excludedItems[i];
				var itemId = excludedItem.versionableItemId!=null ? excludedItem.versionableItemId : excludedItem.itemId;
				excludedItemIds.push(itemId);
			}
			return excludedItemIds;	
		},

		_removeItemByIndex: function(index) {			
			var updatedItems = [];			
			for ( var i = 0; i < this._items.length; i++ ) {
				if ( i != index ) {
					updatedItems.push(this._items[i]);
				} else {
					//deal with the itemId in 'added' and removed'
					var item = this._items[i];
					var itemId = item.versionableItemId ? item.versionableItemId : item.itemId;					
					//if item was added this session, just remove from 'added'
					if ( this._containsItemId(this._itemsAdded, itemId) ) {
						this._itemsAdded = this._removeItemId(this._itemsAdded, itemId);						
						if (this._modelParent!=null)
						{
							SaveContext.uncreateReference(this._modelParent, this._referencesAddedType, itemId);
						}
					} else {
						//otherwise, add to 'removed'
						this._itemsRemoved.push(itemId);
						if (this._modelParent!=null)
						{
							SaveContext.removeReference(this._modelParent, this._referencesRemovedType, itemId);
						}
					}
				}
			}
			this._items = updatedItems;						
			this.showItems();
		},
		
		_onRemoveItem: function(e) {
			var itemIndex = null;
			if (this._collapsable == false) {
				var row = e.currentTarget.parentNode.parentNode;
				itemIndex = row.rowIndex - 1;
			} else {
				itemIndex = this._getItemIndexById(e);
			}
			if ( confirm(Constants.getMessage("msgRemoveItem", this._itemType, this._items[itemIndex].name)) == false ) {
				return;
			};			
			this._removeItemByIndex(itemIndex);
		},
		
		_doSave: function() {
			//name MUST be present
			var newId = dojo.byId("_newItemId").innerHTML;
			var newName = dojo.byId("_newItemName").value;
			var newDesc = dojo.byId("_newItemDesc").value;
			var newOwnerId = dojo.byId("_newItemOwnerId").value;
			if ( newName != null && newName.length > 0 ) {
				var itemObj = { name: newName, description: newDesc, ownerId: newOwnerId };
				if (newId == null || newId.length == 0 ) {
					//alert("save new item: " + newName );
					itemObj.newItem = true;		
					itemObj.workItemData	= {
						state: "1",
						owner: newOwnerId,
						projectAreaName: EditorUtil.getCurrentProjectArea().name						
					} 	
				} else {
					itemObj.id = newId;
				}
				var responseHandler = new ServiceResponseHandler(this, "_itemSaved", "_error");	
				
				//perform the save depending on the itemType
				switch(this._itemType) {
					case Constants.TYPE_GROUP:
						//save item as a test group
						TestGroupClient.saveTestGroup(responseHandler, itemObj);	
						break;
					case Constants.TYPE_CASE:
						//save item as a test case
						TestCaseClient.saveTestCase(responseHandler, itemObj);
						break;											
					case Constants.TYPE_SCRIPT:
						//save item as a test script
						TestScriptClient.saveTestScript(responseHandler, itemObj);
						break;		
					default:
						throw new Error("Unsupported item type: " + this._itemType + " ! ");			
				}								
			}
						
		},
		
		_itemSaved: function(item)
		{		
			//retrieve the new item's DTO
			var responseHandler = new ServiceResponseHandler(this, "_itemRetrieved", "_error");	
			
			//perform the save depending on the itemType
			switch(this._itemType) {
				case Constants.TYPE_GROUP:
					//save item as a test group
					TestGroupClient.getTestGroup(responseHandler, item.id);	
					break;
				case Constants.TYPE_CASE:
					//save item as a test case
					TestCaseClient.getTestCase(responseHandler, item.id);
					break;											
				case Constants.TYPE_SCRIPT:
					//save item as a test script
					TestScriptClient.getTestScript(responseHandler, item.id);
					break;		
				default:
					throw new Error("Unsupported item type: " + this._itemType + " ! ");			
			}			
					
		},		
		
		_itemRetrieved: function(item, index) {
			if ( dojo.isArray(item) ) {
				item = item[0];
			}
			//alert("Successfully saved " + this._itemType +  ": " + item.name );
			if ( item.id ) {
				var node = dojo.byId("_newItemId" + index);
				if (node) {
               node.innerHTML = item.id;	
				}
			}	
			//indicate a new test case added	
			var itemId = item.versionableItemId ? item.versionableItemId : item.itemId;			
			this._itemsAdded.push(itemId);
			this._items.push(item);								
//			this.showItemTable();				
			this.showItems();				
		},
		
		_contains: function(item) {
			var itemId = item.versionableItemId ? item.versionableItemId : item.itemId;			
			return this._getItemById(itemId) != null;
		},	
		
		_getItemById: function(id) {
			for (var i = 0; i < this._items.length; i++) {
				var item = this._items[i];
				var itemId = item.versionableItemId ? item.versionableItemId : item.itemId;			
				if (itemId == id) {
					return this._items[i];
				}
			}
			return null;
		},		
		
		_getItemIndexById: function(id) {
			for (var i = 0; i < this._items.length; i++) {
				var item = this._items[i];
				var itemId = item.versionableItemId ? item.versionableItemId : item.itemId;			
				if (itemId == id) {
					return i;
				}
			}
			return -1;
		},		
		
		_containsItemId: function(array, itemId) {
			for (var i = 0; i < array.length; i++) {
				if (array[i] == itemId) {
					return true;
				}
			}
			return false;
		},	
		
		_removeItemId: function(array, itemId) {
			var updatedArray = [];
			for (var i = 0; i < array.length; i++) {
				if (array[i] != itemId) {
					updatedArray.push(array[i]);
				}
			}
			return updatedArray;
		},	
		
		collapseAllSections: function(area){
		  dojo.forEach(this._itemSectionWidgets,function(section){
		  	section.collapse();
		  });
		},
		
		_error: function(errorObj) {
			alert("Error: (message: '" + errorObj.message + "') from EditableItemList#_error");
		}
	}
);

})();
