/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

EditorSectionContainer = dojo.provide("com.ibm.asq.common.web.ui.internal.view.common.EditorSectionContainer");
EditorSectionContainer.actionIconOnly = 0;
EditorSectionContainer.actionLabelOnly = 1;
EditorSectionContainer.actionLabelAndIcon = 2;

dojo.require("dojox.data.dom");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.CollapsibleSection");

(function() {

var Utility = com.ibm.asq.common.web.ui.internal.util.Utility;

dojo.declare("com.ibm.asq.common.web.ui.internal.view.common.EditorSectionContainer",
com.ibm.asq.common.web.ui.internal.view.common.CollapsibleSection,
{
    editable: false,
    actionDisplay: EditorSectionContainer.actionIconOnly,
    
    summary: "",
    summaryEditable: false,
    _summaryTextNode: null,
    _summaryEditableNode: null,
    _summaryEditableNodeContainer: null,
    
    hideToolbarOnReadOnly: true,
    _editableAwareActions: [],
    _actions: [],
    
    _infoListNode: [],
    
    postCreate: function() {
        this.inherited('postCreate', arguments);

        // create read-only the summary text node
        this._summaryTextNode = document.createElement("span");
        dojo.addClass(this._summaryTextNode, "com-ibm-asq-common-web-ui-editor-container-summary-text");
        this.summaryNode.appendChild(this._summaryTextNode);

        // create the editable summary text node if it can be editable
        if (this.summaryEditable) {
            this._summaryEditableNodeContainer = document.createElement("span");
            dojo.addClass(this._summaryEditableNodeContainer, "com-ibm-asq-common-web-ui-editor-container-summary-text");

            // store data in the dom node
			var inputLabel = document.createElement("label");
			this._summaryEditableNode = document.createElement("input");
            dojo.addClass(this._summaryEditableNode, "com-ibm-asq-common-web-ui-editor-container-summary-editable");
            this._summaryEditableNode.value = this.summary;
            this._summaryEditableNode.oldValue = this.summary;
            this._summaryEditableNode.container = this;
            this._summaryEditableNode.changeHandler = this.summaryOnChange;
            this._summaryEditableNode.twinTextNode = this._summaryTextNode;
            
            this.connect(this._summaryEditableNode, "onchange", function() {
                var oldValue = this.oldValue;
                this.oldValue = this.value;
                this.twinTextNode.innerHTML = this.value;
                
                if (this.changeHandler) {
                    this.changeHandler(oldValue, this.value);
                }
            });
			inputLabel.appendChild(this._summaryEditableNode);
            this._summaryEditableNodeContainer.appendChild(inputLabel);
            this.summaryNode.appendChild(this._summaryEditableNodeContainer);
        }
            
        this.setEditable(this.editable);
    },
    
    setSummary: function(summary) {
        this.summary = summary;
        if (this._summaryTextNode) {
            this._summaryTextNode.innerHTML = summary;
        }
        if (this.summaryEditable && this._summaryEditableNode) {
            this._summaryEditableNode.value = this.summary;
        }
    },
    
    // The container will create the summary text box but the consumer can override this
    // method to pass additional data in the args.  Note that the attribute 'container'
    // for this container widget must be provided in the args since.  This method is
    // invoke from within the summary input text dom node so this.container is the container
    // widget that is stored in the dom node.
    //
    summaryOnChange: function(oldValue, newValue) {
        this.container.headerChangedHandler(oldValue, newValue, {'container': this.container});
    },
    
    clearAllInfoAttributes: function() {
        Utility.destroyWidgetsFromNode(this.infoNode);
        dojox.data.dom.removeChildren(this.infoNode);

        this._infoListNode = [];
    },
    
    addInfoAttribute: function(label, value, icon, domNodeType, editable, onChange) {
        var args = {
                       'domControlType': domNodeType,
                       'label': label,
                       'imgSrc': icon,
                       'attributeValue': value,
                       'attributeEditable': editable,
                       'container': this,
                       'onChange': onChange
                   };
        var attributeNode = new com.ibm.asq.common.web.ui.internal.view.common.EditorSectionInfoAttribute(args);
        
        this.infoNode.appendChild(attributeNode.domNode);
        this._infoListNode.push(attributeNode);
        return attributeNode;
    },
    
    addAction: function(label, actionHandler, callbackData, changeHandler, iconClass, disabledOnReadOnly) {
        var arg = {
            label: label,
            showLabel: !iconClass ||       // must show label if no icon is specified
                       this.actionDisplay, // label only or both icon and label
            onClick: function( e ) {
                this.clientCallback(this.callbackData, e);
            },
            'class': "com-ibm-asq-common-web-ui-editor-container-action"
        };
        
        if (iconClass) {
            arg["iconClass"] = iconClass;
        }
        var button = new dijit.form.Button(arg);
        
        // store the client callback and data into the button
        button.parent = this;
        button.clientCallback = actionHandler;
        button.callbackData = callbackData;
        button.changeHandler = changeHandler;
        if (disabledOnReadOnly) {
            button.setDisabled(!this.editable);
        }
        this.containerHeaderNode.appendChild(button.domNode);
        
        if (disabledOnReadOnly) {
            this._editableAwareActions.push(button);
        } else {
            this._actions.push(button);
        }
        return button;
    },

    setEditable: function(editable) {
        this.editable = editable;
        
        // set appropriate summary node to visible
        if (this.summaryEditable) {
            this._summaryTextNode.style.display = editable ? "none" : "block";
            this._summaryEditableNodeContainer.style.display = editable ? "block" : "none";
        }
        
        // set the enablement of actions that is aware of editable mode
        for (var i = 0; i < this._editableAwareActions.length; i++) {
            this._editableAwareActions[i].setDisabled(!editable);
        }

        // set the tool bar display only if it care to be hidden when read-only
        if (this.hideToolbarOnReadOnly)
            this.containerHeaderNode.style.display = editable ? "block" : "none";

        for (var j = 0; j < this._infoListNode.length; j++) {
            this._infoListNode[j].setEditable(this.editable);
        }
    },
    
    getIsEditable: function() {
        return this.editable;
    },
    
    // This handler is invoke from within a change handler of a dom node such as the summary.
    // The context for "this" is the dom node and not the container.  The consumer is required
    // to add the attribute 'container' in the args to provide the appropriate context.
    //
    contentChangedHandler: function(oldValue, newValue, args) {
        // get the container widget from the args
        var container = args['container'];
        if (!container) {
            // can't proceed without the context of the container widget
            return;
        }
        args['changeSrc'] = "CONTENT";
        for (var i = 0; i < container._editableAwareActions.length; i++) {
            if (container._editableAwareActions[i].changeHandler) {
                container._editableAwareActions[i].changeHandler(oldValue, newValue, args);
            }
        }
        for (var j = 0; j < this._actions.length; j++) {
            if (container._actions[j].changeHandler) {
                container._actions[j].changeHandler(oldValue, newValue, args);
            }
        }
    },
    
    // This handler is invoke from within a change handler of a dom node as well so
    // the requirement is the same as in contentChangedHandler.
    //
    headerChangedHandler: function(oldValue, newValue, args) {
        // get the container widget from the args
        var container = args['container'];
        if (!container) {
            // can't proceed without the context of the container widget
            return;
        }
        // notify the actions for the changes
        args['changeSrc'] = "HEADER";
        for (var i = 0; i < container._editableAwareActions.length; i++) {
            if (container._editableAwareActions[i].changeHandler) {
                container._editableAwareActions[i].changeHandler(oldValue, newValue, args);
            }
        }
        for (var j = 0; j < container._actions.length; j++) {
            if (container._actions[j].changeHandler) {
                container._actions[j].changeHandler(oldValue, newValue, args);
            }
        }
    }
});

dojo.declare("com.ibm.asq.common.web.ui.internal.view.common.EditorSectionInfoAttribute",
[dijit._Widget, dijit._Templated],
{
    // A widget contains a label, an image icon, and a dom control.  The dom control is
    // control is created based on the specified dom element type.  The onChange handler
    // of the dom control is set to notify the container of the change besides invoking
    // the onChange handler that is specified by the consumer.
    // 
    // After this widget is created, the consumer is responsible to "tie in" the desired
    // behavior with the dom control (access through the "domControlNode" attribute) such
    // as updating the icon on value changed.  The consumer can access the image icon
    // throught the "iconNode" attribute.  The consumer is responsible to populate additional
    // data to the dom control as well such as specifying the options to a select dom control.
    //
    templateString: '<span dojoAttachPoint="domNode" class="com-ibm-asq-common-web-ui-editor-container-info">' +
                    '  <span dojoAttachPoint="labelNode" class="info-label"></span>' +
                    '  <img dojoAttachPoint="iconNode" class="info-icon"></img>' +
                    '  <span dojoAttachPoint="controlNode" class="info-control"></span>' +
                    '</span>',
    editable: false,
    label: "",
    imgSrc: null,
    container: null,
    domControlNode: null,
    domControlType: 'span',
    onChange: null,
    attributeEditable: false,
    attributeValue: null,
    _attributeTextNode: null,
    
    postCreate: function() {
        this.inherited('postCreate', arguments);
        
        this.labelNode.innerHTML = this.label;
        if (this.imgSrc) {
            this.iconNode.src = this.imgSrc;
        } else {
            this.iconNode.style.display = "none";
        }
        this.initControl(this.domControlType);
        
        this.setAttributeValue(this.attributeValue);
        this.setEditable(this.editable);
    },
    
    // Determine if the control is read only or editable.  The default is to honor the
    // the parent container's setting.
    //
    getIsEditable: function() {
        return this.editable;
    },
    
    setEditable: function(editable) {
        this.editable = editable;
        if (this.attributeEditable) {
            this._attributeTextNode.style.display = editable ? "none" : "inline";
            this.domControlNode.style.display = editable ? "inline" : "none";
        }
    },
    
    setAttributeValue: function(value) {
        this.attributeValue = value;
        this._attributeTextNode.innerHTML = this.attributeValue;
        
        // check the type of dom node to set value accordingly
        if (this.attributeEditable) {
            var nodeName = this.domControlType.toUpperCase();
                if ((nodeName == "SELECT") || (nodeName == "INPUT") || (nodeName == "OPTION"))
                    this.domControlNode.value = this.attributeValue;
                else
                    this.domControlNode.innerHTML = this.attributeValue;
        }
    },
    

    initControl: function() {
        this._attributeTextNode = document.createElement("span");
        dojo.addClass(this._attributeTextNode, "info-label");
        this.controlNode.appendChild(this._attributeTextNode);

        if (this.attributeEditable) {
            this.domControlNode = document.createElement(this.domControlType);
            
            // store data in the dom node
            this.domControlNode.value = this.attributeValue;
            this.domControlNode.oldValue = this.attributeValue;
            this.domControlNode.container = this.container;
            this.domControlNode.containerChangeHandler = this.container.headerChangedHandler;
            this.domControlNode.onChangeHandler = this.onChange;
            this.domControlNode.twinTextNode = this._attributeTextNode;
            
            this.connect(this.domControlNode, "onchange", function(){
                var oldValue = this.oldValue;
                this.oldValue = this.value;
                this.twinTextNode.innerHTML = this.value;
                
                // invoke container change handler
                this.containerChangeHandler(oldValue, this.value, { 'container': this.container });
                
                // invoke the attribute onChange handler (consumer supplied)
                if (this.onChangeHandler) {
                    this.onChangeHandler();
                }
            });
            this.controlNode.appendChild(this.domControlNode);
        }
    }
    
});

})();

