/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading com.ibm.rqm.planning.web.ui.internal.view.common.NewTestCasePanel");
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.common.NewTestCasePanel");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.template.TemplateClient");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.CategorySelection");
dojo.require("com.ibm.asq.common.web.ui.internal.util.SelectUtility");
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

(function(){
    var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    var CSHelp = com.ibm.asq.common.web.ui.internal.util.CSHelp;
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var EditorUtil = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
    var TemplateClient = com.ibm.rqm.planning.web.client.internal.template.TemplateClient;
    var SelectUtility = dojo.getObject("com.ibm.asq.common.web.ui.internal.util.SelectUtility");
	var RequirementSectionID = "com.ibm.rqm.requirements.editor.section.caseRequirements";
	
    /**
     *
     */
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.common.NewTestCasePanel", [dijit._Widget, dijit._Templated], {
    
        templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/common/NewTestCasePanel.html"),
        
        labelname: Constants.getMessage("labelname"),
        labeldescription: Constants.getMessage("labeldescription"),
        labelweight: Constants.getMessage("labelweight"),
        labelTemplate: Constants.getMessage("labelTemplate"),
        labelowner: Constants.getMessage("labelowner"),
        labelplan: Constants.getMessage("labelTestPlan"),
		labelOpenNewEditor: Constants.getMessage("labelNewCaseEditor"),
		labelSaveArtifact: Constants.getMessage("labelSaveArtifact"),
		
        constructor: function(args){
            this._templates = null;
			this.testPlans = null;
			this.isDisabled = false;
			this.isInSection = false;
        },
        
        postCreate: function(){
			if (this.defaultSummary) {
				this._name.value = this.defaultSummary;
			}
			if (this.defaultDesc) {
				this._description.value = this.defaultDesc;
			}
            this._weight.value = Constants.TEST_CASE_DEFAULT_WEIGHT;
            this._loadTemplates();
			if (this.isInPicker) {
				this._planNode.style.display = "none";
				this._rightColumn0.style.display = "none";
			}
			else if(!this.isInSection) {
				this._loadTestPlans();
			} 
			else {
				this._planNode.style.display = "none";
				this._saveArtifactNode.style.display = "";
			}
			
            this._categorySelect = new com.ibm.rqm.planning.web.ui.internal.view.construction.CategorySelection({
                artifactType: Constants.MODEL_TYPE_CASE,
				afterInitialize: dojo.hitch(this, function(){
					try{
						if(dojo.isIE){
							var selectElements = this._categorySelect._categoriesNode.getElementsByTagName("SELECT");
							dojo.forEach(selectElements, dojo.hitch(this, function(select){
								select.style.width = "250px";
							}));
						}
						var rowElements = this._categorySelect._categoriesNode.getElementsByTagName("TR");
						while(rowElements.length > 0){
							this._upperTable.tBodies[0].appendChild(rowElements[0]);
						}
					} catch (e) {
						console.error(e);
					}
				})
            });
            EditorUtil.initContributors(this._owner, null);
            
        },
        
        _loadTemplates: function(){
            //load the templates
            SelectUtility.add(this._template, new Option(Constants.getMessage("labelLoading"), ""));
            var srh = new ServiceResponseHandler(this, "_setTemplates", "_error");
            TemplateClient.getTemplates(srh, {
                editorId: Constants.EDITOR_ID_CASE
            });
        },
        
        _setTemplates: function(templates){
            this._templates = templates;
            dojox.data.dom.removeChildren(this._template);
            for (var i = 0; i < (this._templates ? this._templates.length : 0); i++) {
				if(this._templateContainReq(this._templates[i]) || this.isInPicker) { // if the widget is contained in the picker, all templates should display
                	var option = new Option(Constants.getExtensionDisplayMessage(this._templates[i].name), this._templates[i].itemId);
                	option.selected = this._templates[i].isDefault;
                	SelectUtility.add(this._template, option);
				}
            }
            if (this._name && !this._name.disabled) 
                dijit.focus(this._name);
        },
		
		/*
		 * Check given template contains requirement section. Return true if it contains. 
		 */  
		_templateContainReq: function(template) {
			for(var i = 0; i < (template.sections ? template.sections.length : 0); i++) {
				if(template.sections[i].id == RequirementSectionID) {
					return true;
				}
			}
			return false;
		},
        
        _getTemplate: function(itemId){
            for (var i = 0; i < (this._templates ? this._templates.length : 0); i++) {
                if (this._templates[i].itemId == itemId) {
                    return this._templates[i];
                }
            }
            return null;
        },
        
		_loadTestPlans: function() {
			dojox.data.dom.removeChildren(this._plan);
			var option = new Option(Constants.getMessage("labelUnassigned"), "-1");
			option.selected = true;
			SelectUtility.add(this._plan, option);
			if (this.testPlans && this.testPlans.length > 0) {
				for (var i = 0; i < this.testPlans.length; i++) {
					var planoption = new Option(this.testPlans[i].name, this.testPlans[i].id);
					SelectUtility.add(this._plan, planoption);
				}
			}
		},
		
        setDisabled: function(isDisabled){
			this.isDisabled = isDisabled;
            this._name.disabled = isDisabled;
            this._description.disabled = isDisabled;
            this._template.disabled = isDisabled;
            this._weight.disabled = isDisabled;
            this._owner.disabled = isDisabled;
			this._plan.disabled = isDisabled;
            if (this._categorySelect && this._categorySelect._selections && this._categorySelect._selections.length > 0) {
                var selections = this._categorySelect._selections
                for (var i = 0; i < selections.length; i++) {
                    selections[i].disabled = isDisabled;
                }
            }
        },
        
        getAttributes: function(){
            var item = {
                names: new Array(this._name.value),
                descriptions: new Array(this._description.value),
                weights: new Array(this._weight.value),
				isSingleTestCase: true,
				isOpenNewTab: this._openNewEditor.checked ? true : false
            }
            var planId = null;
            if (this.isInSection && this.artifact) {
				item.isSaveTestPlan = this._saveArtifactEditor.checked ? true : false;
                planId = this.artifact.id+"";
            }
            else 
                planId = this._plan.value;
            if (planId && planId != "-1") {
               item.planId = planId;
            }
            var selections = this._categorySelect.getSelectedItemIds();
            if (selections) {
                item.categories = new Array(selections.join());
            }
            if (this._owner && this._owner.textContent != Constants.getMessage("labelUnassigned")) {
                //HACK - force owner into summary section's workitem until
                //we decide on an official artifact ownership strategy				
                var wiList = [{
                    state: "1",
                    section: "com.ibm.rqm.planning.editor.section.testCaseSummary",
                    owner: this._owner.value,
                    projectAreaName: EditorUtil.getCurrentProjectArea().name
                }];
                item.workItemsJsonStrings = new Array(dojo.toJson(wiList));
				item.ownerIds = new Array(this._owner.value);
            }
			else {
				item.workItemsJsonStrings = new Array("-1");
				item.ownerIds = new Array("-1");
			}
            if (this._template.value != null) {
                var template = this._getTemplate(this._template.value);
                item.templateItemId = template.itemId;
                item.templateStateId = template.stateId;
            }
            return item;
        },
        
		setAlign: function() {
			var width = dojo.marginBox(this._planTD).w + "px";
			this._categorySelect.setLabelWidth(width);
		},
		
        _error: function(e){
            alert(this.declaredClass + ": Error: " + e.message);
        }
        
    });
})();
