/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.common.SimpleWorkItemPane");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.workitem.web.client.internal.ImageClient");

dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.util.TooltipBuilder");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.DatePicker");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");

(function(){ // module container
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var EditorUtil = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
    var ImageClient = com.ibm.team.workitem.web.client.internal.ImageClient;
    var ImageCache = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
	var TooltipBuilder = com.ibm.asq.common.web.ui.internal.util.TooltipBuilder;
	var CSHelp = com.ibm.asq.common.web.ui.internal.util.CSHelp;
		    
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.common.SimpleWorkItemPane", [dijit._Widget, dijit._Templated], {
    
        templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/common/SimpleWorkItemPane.html"),
        
		width: "600px", //STYLE = "position: absolute; top: 25%; left: 25%; width: 600px; height: 200px";
		labelSummary: Constants.getMessage("labelSummary"),
		labelOwnedBy: Constants.getMessage("labelOwnedBy"),
		labelDue: Constants.getMessage("labelDue"),
		labelstatus: Constants.getMessage("labelstatus"),
		labelOk: Constants.getMessage("labelOk"),
		labelCancel: Constants.getMessage("labelCancel"),
		labelUnassigned: Constants.getMessage("labelUnassigned"),
        ///////////////////////////////////////////////////////////////////////////////////
        // Called on creation
        ///////////////////////////////////////////////////////////////////////////////////				
        constructor: function(){
            this.editable = false;
			this._wiDialog = null;
			this.parent = null;
			this.workItemInfo = null;
        },
        
        postCreate: function(args){
            if (!this.sectionId) {
                throw new Error("Argument 'sectionId' is REQUIRED!");
            }
			if (!this.parent) {
                throw new Error("Argument 'parent' is REQUIRED!");
            }
            this.initialize();
        },
        
        initialize: function(){
            this._displayWorkItemInfo();
            this.setEditable(this.editable);
			this._showAsDialog();
        },
		
		_showAsDialog: function() {
            if (!this._wiDialog) {
                var dialogArgs = {
					primaryTitle: Constants.getMessage("labelNewWorkItem"),
                    okCallback: dojo.hitch(this, "_onOK"),
                    cancelCallback: dojo.hitch(this, "_onCancel")
                };                
                this._wiDialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(dialogArgs);
				var cshelp = new CSHelp({contextID: "createworkitem.html"});
            	cshelp.setButtonLabel(Constants.getMessage("labeCreatWorkItemCSHelp"));
            	this._wiDialog.setSecondaryTitleNode(cshelp.domNode);
                this._wiDialog.setWidth(this.width);
                this._wiDialog.contentContainer.appendChild(this.domNode);
                this._wiDialog.getOkButton().disabled = false;                                
            }
			 //setup date picker
            dojox.data.dom.removeChildren(this._dueDate);
            var args = {isDate: true, containerToggle: "fade", isInDialog: true, fieldLabel: this.labelDue };
            
            this._dueDatePicker = new com.ibm.asq.common.web.ui.internal.widgets.DatePicker(args, this._dueDate);  
            if(this.workItemInfo && this.workItemInfo.dueDate) {
                var dueDate = new Date(this.workItemInfo.dueDate);
                this._dueDatePicker.setValue(dueDate);
            }
            this._wiDialog.open(false);			
            this._popup = true;
		},
        
        ///////////////////////////////////////////////////////////////////////////////////
        // API
        ///////////////////////////////////////////////////////////////////////////////////    		
        
        getWorkItemInfo: function(){
            // return the last captured work item info.
            if(!this._popup) {
                return this.workItemInfo;
            }
            var wiInfo = {};
            var owner = this.getSelectedOwner();
            wiInfo.state = EditorUtil.getWIStateForAction(this._stateSelect.value).stateId;
            wiInfo.section = this.sectionId;
            wiInfo.owner = owner.id;
			wiInfo.ownerName = owner.name;
            wiInfo.projectAreaName = EditorUtil.getCurrentProjectArea().name;
			wiInfo.summary = this._summary.value;
			wiInfo.isNew = true;
			wiInfo.dueDate = this._dueDatePicker.getValue() != null ? this._dueDatePicker.getValue().getTime() : null;
			this.workItemInfo = wiInfo;
            return wiInfo;
        },
        
        getWorkflowAction: function(){
            return this._stateSelect.value;
        },
		
        ///////////////////////////////////////////////////////////////////////////////////
        // Internal
        ///////////////////////////////////////////////////////////////////////////////////    		
        _displayWorkItemInfo: function(){
            var stateName = EditorUtil.getWIStateForAction(this.getWorkflowAction()).name;
            var stateIconURL = EditorUtil.getWIStateForAction(this.getWorkflowAction()).icon;

			this._summary.value = this.workItemInfo != null ? this.workItemInfo.summary : this.defaultSummary;
			
            //Select default owner if defaultOwnerId specified and new artifact
            var selectedItemId = this.workItemInfo != null ? this.workItemInfo.owner : null;
            EditorUtil.initContributors(this._ownerSelect, selectedItemId);
			this.connect(this._ownerSelect, "onmouseover", "_onmouseover");
            EditorUtil.initWorkItemStates(this._stateSelect, stateName);
            this._stateSelect.disabled = true;
            
			dojox.data.dom.removeChildren(this._ownerSpan);
            dojox.data.dom.removeChildren(this._stateSpan);
            this._stateSpan.innerHTML = stateName;
            this._stateIcon.src = ImageClient.getImageUrl(stateIconURL);
        },
        
		_onmouseover: function() {
			if(this._ownerSelect) {
				this._ownerSelect.focus();
			}
		},
		
        selectOwner: function(ownerItemId) {
            if (this._ownerSelect && this._ownerSelect.options) {
                for (var i = 0; this._ownerSelect.options.length; i++) {
                    var option = this._ownerSelect.options[i];
                    if (option.value == ownerItemId) {
                        this._ownerSelect.options[i].selected = true;
                    }
                }
            }
        },
        
        getSelectedOwner: function() {
            var val = {};
            if (this._ownerSelect && this._ownerSelect.options) {
                for (var i = 0; i < this._ownerSelect.options.length; i++) {
                    var option = this._ownerSelect.options[i];
                    if (option.selected) {
                        val['id'] = option.value;
                        val['name'] = option.text;
                        break;
                    }
                }
            }
            else {
                val['id'] = "";
                val['name'] = "";
            }
            return val;
        },
        
        setEditable: function(editable){
            this.editable = editable;
            if (!this.editable) {
                dojo.addClass(this._ownerSelect, "hidden");
                dojo.addClass(this._stateSelect, "hidden");
                dojo.removeClass(this._ownerSpan, "hidden");
                dojo.removeClass(this._stateSpan, "hidden");
            }
            else {
                dojo.addClass(this._ownerSpan, "hidden");
                dojo.addClass(this._stateSpan, "hidden");
                dojo.removeClass(this._ownerSelect, "hidden");
                dojo.removeClass(this._stateSelect, "hidden");
            }
        },
        
		_validate: function() {
			//summary is required
            if (!this._summary || dojo.trim(this._summary.value).length == 0) {
                alert(Constants.getMessage("msgRequiredField", Constants.getMessage("labelSummary")));
				return false;
			}
			//owner cannot be Unassigned
			if (this._ownerSelect.options && this._ownerSelect.options.length > 0) {
				var owner = this._ownerSelect.options[this._ownerSelect.selectedIndex];
				if (owner.text != this.labelUnassigned) {
					return true;
				}
			}
			alert(Constants.getMessage("msgAssignmentRequired"));
			return false;
		},
		
		_onOK: function() {
			if (!this._validate()) {
				return false;
			}
			var now = new Date();
			var time = dojo.string.pad(now.getHours(), 2, '0') + ":" 
			 + dojo.string.pad(now.getMinutes(), 2, '0') + ":" 
			 + dojo.string.pad(now.getSeconds(), 2, '0');
			this.parent._updateWorkItemLink(time); 
            
            // save the workitem info. Once the dialog
            // is popped down on IE, the data would be
            // null when we try to read the dom nodes.
            this.getWorkItemInfo();
            
            // let the dialog popdown the dialog.
            this._popup = false;
            return true;
		},
		
		_onCancel: function() {
            this._popup = false;
			this.parent._updateWorkItemLink(null); 
		},
        
        _error: function(e){
            alert("An error occurred: [" + e.message + "] @ SectionWorkItem#_error");
        }
    });
    
})();
