/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.common.properties.TestPlanTestCaseConfigSelector");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.parser");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.util.SelectUtility");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.layout.FormTextField");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.CategoryClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.CategorySelection");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.properties.FilterListPanel");

(function(){
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var CategoryClient = com.ibm.rqm.planning.web.client.internal.CategoryClient;
    var TestCaseClient = com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient;
    var EditorUtil = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
    var FilterListPanel = com.ibm.rqm.planning.web.ui.internal.view.common.properties.FilterListPanel;
    
    var SelectUtility = dojo.getObject("com.ibm.asq.common.web.ui.internal.util.SelectUtility");
    var FormNumericField = com.ibm.asq.common.web.ui.internal.widgets.layout.FormNumericField;
    var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    
    // cap max number of query results at 1000
    var MAX = 1000;
    var UNASSIGNED_CATEGORY = Constants.getMessage("labelUnassigned");
    
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.common.properties.TestPlanTestCaseConfigSelector", [dijit._Widget, dijit._Templated], {
        templateString: '<div dojoAttachPoint="_content">' +
        '<div dojoAttachPoint="_area"></div>' +
        '</div>',
        
        height: "450px",
        width: "250px",
        invokeTimes: 0,
        returnedNum: 0,
        
        labelWeight: Constants.getMessage("labelweight"),
        labelPoints: Constants.getMessage("labelPoints"),
        labelowner: Constants.getMessage("labelowner"),
        
        constructor: function(params){
            this._allTestCases = []; //all TestCaseDTO
            this._allTCSummaries = [];
            this._categoryTypeSummaries = [];
            this._categoryType = null;
            this._action = null;
            this.select = null;
            if (dojo.isIE) {
                //for defect 9494
                MAX = 50;
            }
        },
        
        postCreate: function(){
        },
        
        initialize: function(args){
            this._allTCSummaries = args.testcases ? args.testcases : [];
            this.okCallback = args.onOkCallback;
            this.actionId = args.actionId;
            
            if (args.actionId == Constants.getMessage("labelWeight")) {
                this._action = "weight";
                dojox.data.dom.removeChildren(this._content);
                this.weightWidget = new FormNumericField({
                    fieldLabel: this.labelWeight,
                    required: false,
                    disabled: false
                });
                this.weightWidget.setEdited(true);
                this.weightWidget.setErrorMessage(Constants.getMessage("msgNumericWeightRequired"));
                this.weightWidget.setPromptMessage(Constants.getMessage("msgWeightRequired"));
                this._content.appendChild(this.weightWidget.domNode);
                this.weightWidget.onValueChanged = dojo.hitch(this, "_validateWeight");
				
            }else if (args.actionId == Constants.getMessage("labelState")) {
                this._action = "state";
                dojox.data.dom.removeChildren(this._content);
				var table = document.createElement("table");
				table.style.width = "100%";
				var tbody = document.createElement("tbody");
				var tr = document.createElement("tr");
				var label = document.createElement("td");
				label.innerHTML = Constants.getMessage("labelState")+Constants.getMessage("labelColen");
				label.width = "20%";
				tr.appendChild(label);
				var td = document.createElement("td");
				td.width = "80%";
				this.stateSelect = document.createElement("select");
				SelectUtility.add(this.stateSelect,new Option(Constants.getMessage("labelLoading"), ""));
				EditorUtil.initArtifactStates(dojo.hitch(this,"_loadStates"));
				td.appendChild(this.stateSelect);
				tr.appendChild(td);
				tbody.appendChild(tr);
				table.appendChild(tbody);
				this._content.appendChild(table);
            }else {
                this._action = "category";
                for (var i = 0; i < (args.categoryTypes ? args.categoryTypes.length : 0); i++) {
					var categoryName = Constants.getMessage(args.categoryTypes[i].name);
                    if (categoryName == args.actionId) {
                        this._categoryType = args.categoryTypes[i];
                        break;
                    }
                }
                if (!this._categoryType) {
                    console.error("error!!!!!!!!!!!!!!!!!!!!");
                    return;
                }
                
                this.filterListPanel = new FilterListPanel();
				this.filterListPanel.addChangeLisner(dojo.hitch(this, "_validateCategory"));
                this.filterListPanel.addOption(UNASSIGNED_CATEGORY, UNASSIGNED_CATEGORY, UNASSIGNED_CATEGORY);
                for (var j = 0; j < (this._categoryType.categories ? this._categoryType.categories.length : 0); j++) {
                    this.filterListPanel.addOption(this._categoryType.categories[j].itemId, this._categoryType.categories[j].name, this._categoryType.categories[j].name);
                }
				
                dojox.data.dom.removeChildren(this._content);
                this._content.appendChild(this.filterListPanel.domNode);
            }
            this._showAsDialog();
        },
        _loadStates:function(){
			dojox.data.dom.removeChildren(this.stateSelect);
			var artifactStates = EditorUtil.artifactStates;
			for(var i=0;i<artifactStates.length;i++){
				SelectUtility.add(this.stateSelect,new Option(artifactStates[i].name,artifactStates[i].id));
			}
		},
        _showAsDialog: function(){
            if (!this._dialog) {
                var dialogArgs = {
                    okCallback: dojo.hitch(this, "_okAction")
                };
                this._dialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(dialogArgs);
                this._dialog.setWidth(this.width);
                this._dialog.setTitle(Constants.getMessage("labelChangeProperties", this.actionId));
                this._dialog.contentContainer.appendChild(this.domNode);
            }
            this._dialog.open(false);
			if(this._action == "state"){
				this._dialog._yesButton.disabled = false;
			}
        },
        
        _okAction: function(){
            var itemIds = [];
            for (var i = 0; i < this._allTCSummaries.length; i++) {
                if (this._allTCSummaries[i].itemId && !this._allTCSummaries[i].stateId) {
                    itemIds.push(this._allTCSummaries[i].itemId);
                }
            }
            if (itemIds.length == 0) {
                this._loadTestCaseDTOs(null);
            }
            else {
                this.invokeTimes = Math.ceil(itemIds.length / MAX);
                for (var j = 0; j < this.invokeTimes; j++) {
                    var params = {
                        itemUUIDs: []
                    };
                    var startIndex = j * MAX;
                    var tmpLen = (j == this.invokeTimes - 1) ? (itemIds.length - startIndex) : MAX;
                    for (var i = 0; i < tmpLen; i++) {
                        params.itemUUIDs.push(itemIds[startIndex + i]);
                    }
                    var srh = new ServiceResponseHandler(this, "_loadTestCaseDTOs", "_error");
                    TestCaseClient.getTestCases(srh, params);
                }
            }
        },
        
        _loadTestCaseDTOs: function(testCaseDTOs){
            if (testCaseDTOs) 
                this.returnedNum++;
            for (var j = 0; j < (testCaseDTOs ? testCaseDTOs.length : 0); j++) {
                this._allTestCases.push(testCaseDTOs[j]);
            }
            if (this.returnedNum == this.invokeTimes) {
                var args = {};
                var testCases = [];
                for (var i = 0; i < this._allTCSummaries.length; i++) {
                    var tcSummary = this._allTCSummaries[i];
                    if (tcSummary.itemId && !tcSummary.stateId) {
                        //existing test case
                        var stateId = this._getTestCaseStateId(tcSummary.itemId);
                        tcSummary.stateId = stateId;
                    }
                    if (this._action == "category") {
                        tcSummary = this._reCaculateCategories(tcSummary);
                    }
                    testCases.push(tcSummary);
                }
                args.testCases = testCases;
                if (this._action == "weight") {
                    if (dojo.trim(this.weightWidget.getValue()).length != 0) 
                        args.weight = this.weightWidget.getValue();
                }
				if (this._action == "state") {
					args.state = this.stateSelect.value;
                }
                if (this.okCallback) {
                    this.okCallback(args);
                }
            }
        },
        
        _getTestCaseStateId: function(itemId){
            for (var i = 0; i < this._allTestCases.length; i++) {
                if (this._allTestCases[i].itemId == itemId) {
                    return this._allTestCases[i].stateId;
                }
            }
            return null;
        },
        
        _reCaculateCategories: function(tcSummary){
            var nowCategory = this.filterListPanel.getFirstSelectedValue();
			if(nowCategory == UNASSIGNED_CATEGORY)
				nowCategory = null;
            var oldCategories = tcSummary.categories;
            var newCategories = [];
			var isReplaced = false;
			
            for (var i = 0; i < (oldCategories ? oldCategories.length : 0); i++) {
                var newCategory = oldCategories[i];
                for (var j = 0; j < (this._categoryType.categories ? this._categoryType.categories.length : 0); j++) {
                    if (oldCategories[i] == this._categoryType.categories[j].itemId) {
						newCategory = nowCategory;
						isReplaced = true;
					}
                }
				if(newCategory)
                	newCategories.push(newCategory);
            }
			if(!isReplaced)
				newCategories.push(nowCategory);
				
			if(newCategories.length == 0)
				newCategories.push("  ");
            tcSummary.categories = newCategories;
            return tcSummary;
        },
        
        _validateWeight: function(){
            if (dojo.trim(this.weightWidget.getValue()).length != 0) {
                var weight = Number(dojo.trim(this.weightWidget.getValue()));
                if (weight != NaN && weight >= 0 && this._dialog) {
                    this._dialog._yesButton.disabled = false;
					return;
                }
            }
            if (this._dialog) 
	            this._dialog._yesButton.disabled = true;
        },
        
        _validateCategory: function(){
            var currentValues = this.filterListPanel ? this.filterListPanel.getSelectedItems() : [];
            if (currentValues.length != 1) 
                this._dialog._yesButton.disabled = true;
            else 
                this._dialog._yesButton.disabled = false;
        },
        
        _error: function(errorObj){
            console.error("Error in TestCaseConfigSelector " + errorObj.message);
        }
        
    });
})();
