/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.configuration.web.ui.internal.view.configuration.ConfigurationPopup");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.rqm.configuration.web.ui.internal.view.configuration.WeightingPanel");
dojo.require("com.ibm.rqm.configuration.web.ui.internal.view.configuration.ExclusionPanel");

dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.ContentArea");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");

(function() {
var WeightingPanel 				= com.ibm.rqm.configuration.web.ui.internal.view.configuration.WeightingPanel;
var ExclusionPanel 				= com.ibm.rqm.configuration.web.ui.internal.view.configuration.ExclusionPanel;
var Constants 					= com.ibm.rqm.planning.web.client.internal.Constants;
var CSHelp 						= com.ibm.asq.common.web.ui.internal.util.CSHelp;
var ContentAreas 				= com.ibm.rqm.planning.web.ui.internal.view.common.ContentArea;
var ViewHeaderLayout 			= com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout;

dojo.declare(
	"com.ibm.rqm.configuration.web.ui.internal.view.configuration.ConfigurationPopup",
	[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.rqm.configuration.web", "ui/internal/view/templates/configuration/ConfigurationPopup.html"),
		
		widgetsInTemplate: true,
		
		constructor: function(args) {
			this.excludesIncludesEnv = args.excludesIncludesEnv;
			this.weightingEnv  = args.weightingEnv;
			this.setExclusions = args.setExclusions;
			this.setInclusions = args.setInclusions;
			this.setWeightings = args.setWeightings;
			this.csHelpId = "rqm_ex_advanced_prop.html";
			
		},
		
		postCreate: function(){
			this._setTabContainer();
		},
		
		createCSHelpID: function(){
			var csHelp = new CSHelp();
			csHelp.contextID = this.csHelpId;
			csHelp.setButtonLabel(Constants.getMessage("labelEWIAdvProCSHelp"));
			return csHelp;
		},
		
		refresh: function(excludesIncludesEnv,weightingEnv,refreshExIn,refreshWeightings){
			this.excludesIncludesEnv = excludesIncludesEnv;
			this.weightingEnv = weightingEnv;
			if(refreshExIn){
				this._inclusionsPane = new ExclusionPanel({
						env: this.excludesIncludesEnv, 
						label: Constants.getMessage("labelInclusionsText"),
						panelType: "inclusions"
					});
			    this._exclusionsPane = new ExclusionPanel({
						env: this.excludesIncludesEnv,
						label: Constants.getMessage("labelExclusionsText"),
						panelType: "exclusions"
					});
				this._inclusionsTab = this._contentArea._setTabContent(this._inclusionsTab, this._inclusionsPane);
				this._exclusionsTab = this._contentArea._setTabContent(this._exclusionsTab, this._exclusionsPane);
			}
			if(refreshWeightings){
				this._weightingPanel = new WeightingPanel({
						env: this.weightingEnv
					});
				this._weightingTab = this._contentArea._setTabContent(this._weightingTab, this._weightingPanel);
			}
		},
		
		_setTabContainer: function() {
			dojox.data.dom.removeChildren(this._ConfigurationPopupContent);
			this._contentArea = new ContentAreas();
			this._contentArea.initialize({tabContainerStyle: "min-height: 100px"});
			
		    this._ConfigurationPopupContent.appendChild(this._contentArea.domNode);
			
			//add Inclusions Tab
			this._inclusionsPane = new ExclusionPanel({
						env: this.excludesIncludesEnv, 
						label: Constants.getMessage("labelInclusionsText"),
						panelType: "inclusions"
					});
			var args = {
				name: Constants.getMessage("labelInclusions"),
				noSelect: true,
				content: this._inclusionsPane
			};
			this._inclusionsTab = this._contentArea.addPage(args);
			
			//add exclusions Tab
			this._exclusionsPane = new ExclusionPanel({
						env: this.excludesIncludesEnv,
						label: Constants.getMessage("labelExclusionsText"),
						panelType: "exclusions"
					});
			var args = {
				name: Constants.getMessage("labelExclusions"),
				noSelect: true,
				content: this._exclusionsPane
			};
			this._exclusionsTab = this._contentArea.addPage(args);
			
			
			//add weightings Tab
			this._weightingPanel = new WeightingPanel({
						env: this.weightingEnv
					});
			var args = {
				name: Constants.getMessage("labelWeightings"),
				noSelect: true,
				content: this._weightingPanel
			};
			this._weightingTab = this._contentArea.addPage(args);
		},
		
       	_getExclusions: function(){
			if(this._exclusionsPane){
				return (this._exclusionsPane.getItems());
			}
		},
		
		_getInclusions: function(){
			if(this._inclusionsPane){
				return (this._inclusionsPane.getItems());
			}
		},
		
		_getWeightings: function(){
			if(this._weightingPanel){
				return (this._weightingPanel.getWeightings());
			}
		},
		
		_onOK: function(){
			var exclusions = this._getExclusions();
			var inclusions = this._getInclusions();
			this.setExclusions(exclusions);
			this.setInclusions(inclusions);
			this.setWeightings(this._getWeightings());
		},
		
		_compareExclusionsAndInclusions: function(){
			var exclusions = this._getExclusions();
			var inclusions = this._getInclusions();
			for(var i=0; i<exclusions.length; i++){
				for(var j=0; j<inclusions.length; j++){
					if(exclusions[i].join() == inclusions[j].join()){
						this._viewHeaderNode.setStatus(Constants.getMessage("msgSameCombinations"), ViewHeaderLayout.WARNING);
						return false;
					}
				}
			}
			this._viewHeaderNode.clearStatus();
			return true;
		},
		
		_error: function(e) {
			if(e && e.message){
				console.error(e.message);
			}else{
				console.error(e);
			}
		}			
	}
);

})();
