/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading: com.ibm.rqm.configuration.web.ui.internal.view.configuration.EnvironmentSection");
dojo.provide("com.ibm.rqm.configuration.web.ui.internal.view.configuration.EnvironmentSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.planning.web.client.internal.planning.PlanningPageClient");
dojo.require("com.ibm.asq.common.web.client.internal.Catalog");
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.Button");
dojo.require("com.ibm.rqm.configuration.web.ui.internal.view.configuration.HiddenableDialog");

dojo.require("com.ibm.rqm.configuration.web.ui.internal.view.configuration.ConfigurationPopup");
dojo.require("com.ibm.rqm.configuration.web.util.TestConfigurationUtil");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function(){
    var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    var ConfigurationPopup = com.ibm.rqm.configuration.web.ui.internal.view.configuration.ConfigurationPopup;
    var CSHelp = com.ibm.asq.common.web.ui.internal.util.CSHelp;
    var TestConfigurationUtil = com.ibm.rqm.configuration.web.util.TestConfigurationUtil;
    var PlanningPageClient = com.ibm.rqm.planning.web.client.internal.planning.PlanningPageClient;
	var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	var HiddenableDialog = com.ibm.rqm.configuration.web.ui.internal.view.configuration.HiddenableDialog;
    
    dojo.declare("com.ibm.rqm.configuration.web.ui.internal.view.configuration.EnvironmentSection", [dijit._Widget, dijit._Templated], {
        templatePath: dojo.moduleUrl("com.ibm.rqm.configuration.web", "ui/internal/view/templates/configuration/EnvironmentSection.html"),
        
		labelEnvironment: Constants.getMessage("labelEnvironment"),
		
        constructor: function(args, frag){
            if (args.controller == null) {
                throw new Error("Create EnvironmentSection#constructor: Missing required parameter 'args.controller'");
            }
            this.disableCoverageOptions = (args.disableCoverageOptions != null) ? args.disableCoverageOptions : null;
            this.enableCoverageOptions = (args.enableCoverageOptions != null) ? args.enableCoverageOptions : null;
			this.enableNextButton = (args.enableNextButton != null) ? args.enableNextButton : null;
            this.controller = args.controller;
            this.csHelpId = "rqm_ex_ewi_environment.html";
			this.advancedPropertiesEnabled = false;
			this._inintVariables();
        },
        
        initialize: function(){
			this._initEnvironmentSection();
			this._initAdvancedProperties();
			this._loadCatalogEnvironment();
			this._topic = dojo.subscribe("TESTPLAN_CHANGED", this, "_reloadEnvironment");
        },
		
		uninitialize: function(){
//	    	console.debug("===============EnvironmentSection Uninitialize");
			dojo.unsubscribe(this._topic);
			if(this.advancePropertiesDialog && dojo.isFF){
				this.advancePropertiesDialog.close();
				this.advancePropertiesDialog = null;
			}
		},
		
		_reloadEnvironment: function(id){
			this._inintVariables();
			this._loadTestPlanEnvironment(id);
			this.advancedPropertiesButton.setDisabled(true);
			this.checkEnviromentAndProperties();
            if(this.advancePropertiesPopupWindow){
				this.advancePropertiesPopupWindow.refresh(this.newInfoForExIn, this.newInfoForWeightings, true, true);
			}
			if(this.enableNextButton != null){
				this.enableNextButton(true, true);
			}
		},
		
		_inintVariables: function(){
			this.tables = [];
			this.exclusions = null;
            this.inclusions = null;
            this.weightings = null;
            this.refreshExIn = false;
            this.refreshWeightings = false;
            this.oldInfoForExIn = [];
            this.oldInfoForWeightings = [];
            this.newInfoForExIn = [];
            this.newInfoForWeightings = [];
		},
        
        _loadTestPlanEnvironment: function(id){
			if(id == TestConfigurationUtil.NO_ASSOCIATED_TEST_PLAN){
				this._loadCatalogEnvironment();	
			}else{
				var responseHandler = new ServiceResponseHandler(this, "_initTestPlanEnvironment", "_error");
	            PlanningPageClient.getTestPlans(responseHandler, {
	                id: id
	            })
			}
        },
        
        _initEnvironmentSection: function(){
            var csHelp = new CSHelp();
            csHelp.contextID = this.csHelpId;
			csHelp.setButtonLabel(Constants.getMessage("labelEWIEnvironmentCSHelp"));
            this._cshelp.appendChild(csHelp.domNode);
        },
        
        _loadCatalogEnvironment: function(){
            var responseHandler = new ServiceResponseHandler(this, "_initCatalogEnvironment", "_error");
            this.catalog = new com.ibm.asq.common.web.client.internal.Catalog();
            if (!com.ibm.asq.common.web.client.internal.Catalog.theCatalog) {
                this.catalog.initializeTemplates(responseHandler);
            }
            else {
                this._initCatalogEnvironment();
            }
        },
        
        _initTestPlanEnvironment: function(testPlans){
            this.catalogTypes = [];
            this.catTypeValues = [];
            if (testPlans && testPlans.length > 0) {
				if(!testPlans[0].catalogTypes){
					this._loadCatalogEnvironment();
				}else{
					for (var i = 0; i < testPlans[0].catalogTypes.length; i++) {
                        // convert the test plan's catalog type back from json string
                        // to object.
                        if(dojo.isString(testPlans[0].catalogTypes[i].name))
                            testPlans[0].catalogTypes[i].name = dojo.fromJson(testPlans[0].catalogTypes[i].name);
	                    this.catalogTypes.push(testPlans[0].catalogTypes[i].name);
	                    var arr = [];
	                    for (var j = 0; j < testPlans[0].catalogTypes[i].values.length; j++) {
	                        arr.push(testPlans[0].catalogTypes[i].values[j].name);
	                    }
	                    this.catTypeValues.push(arr);
	                }
				}
            }
			this._initEnvironmentTable();
        },
        
        _initCatalogEnvironment: function(){
            this.catalogTypes = this.catalog.getDisplayTypes();
            this.catTypeValues = [];
            for (var i = 0; i < this.catalogTypes.length; i++) {
                this.catTypeValues.push(this.catalog.getValuesForDisplayType(this.catalogTypes[i]));
            }
			this._initEnvironmentTable();
        },
		
		_initEnvironmentTable: function(){
			var attributeNum = this.catalogTypes.length / 3;
            
            dojox.data.dom.removeChildren(this._tables);
			var tables = document.createElement("table");
			var tbody = document.createElement("tbody");
			tables.appendChild(tbody);
//			tables.style = "font-size:small";
			tables.cellspacing = 3;
			tables.width = "100%";
            for (var i = 0; i < attributeNum; i++) {
                var trHead = document.createElement("tr");
                this._createLineHead(trHead, i);
                tbody.appendChild(trHead);
                
                var trTable = document.createElement("tr");
                this._createLineBody(trTable, i);
                tbody.appendChild(trTable);
            }
			
			this._tables.appendChild(tables);
		},
        
        _initAdvancedProperties: function(){
            /*
             * add _advancedP
             */
			var self = this;
			var arg = {
	            label: Constants.getMessage("labelAdvancedProperties"),
	            showLabel: true,
	            onClick: function(e) {
	                self._handleAdvProEnvClick();
	            }
	        };
			this.advancedPropertiesButton = new com.ibm.asq.common.web.ui.internal.widgets.Button(arg);
	        this.advancedPropertiesButton.setDisabled(true);
			this._advancedP.appendChild(this.advancedPropertiesButton.domNode);
        },
        
        _createLineHead: function(tr, index){
            var types = this.catalogTypes;
            for (var i = 0; i < 3; i++) {
                if (3 * index + i < types.length) {
                    var tdHead = document.createElement("td");
                    tdHead.className = "tdHead";
                    var typeName = types[3 * index + i].name;
                    var hDate = document.createTextNode(Constants.getLmMessage(typeName));
                    tdHead.appendChild(hDate);
                    tr.appendChild(tdHead);
                }
            }
        },
        
        _createLineBody: function(tr, index){
            var types = this.catalogTypes;
			var values = this.catTypeValues;
            for (var i = 0; i < 3; i++) {
                if (3 * index + i < types.length) {
                    var tdTable = document.createElement("td");
                    tdTable.className = "tdTable";
                    var div = document.createElement("div");
					div.className = "tableDiv";
					div.style.height = "100px";
					div.style.border = "1px solid";
					
					div.style.overflow = "auto";
					if (dojo.isIE) {
						div.style.overflowX = "hidden";
					}
                    this._createTable(div, types[3 * index + i], values[3 * index + i]);
                    tdTable.appendChild(div);
                    tr.appendChild(tdTable);
                }
            }
        },
        
        _getAttributeTypeById: function(id){
            console.debug("searching for matching type for path: " + id)
            for(var i = 0; i < this.catalogTypes.length; i++) {
                if(this.catalogTypes[i].id == id) {
                    return this.catalogTypes[i];
                }
            }
            return null;
        },
        
        _createTable: function(tdTable, type, typeValues){
            var table = document.createElement("table");
			var tbody = document.createElement("tbody");
			table.appendChild(tbody);
            table.className = "tableInEnv";
            
            // store the identifier of the catalog type as attributes. 
            table.setAttribute("dojoattachpoint", type.id); 
            table.setAttribute("id", "table" + type.name);//should change this id with tmp
            
			var btr = document.createElement("tr");
			this._addCheckBoxAndValue(btr, Constants.getMessage("labelAll"));
			this._addUnchangedColumn(btr, Constants.getMessage("labelAll"));
			tbody.appendChild(btr);
			
            for (var i = 0; i < typeValues.length; i++) {
                var btr = document.createElement("tr");
				this._addCheckBoxAndValue(btr, typeValues[i]);
                this._addUnchangedColumn(btr, typeValues[i]);
                tbody.appendChild(btr);
            }
            tdTable.appendChild(table);
            this.tables.push(table);
        },
		
		_checkAllAttributes: function(e){
			var inputs = e.currentTarget.parentNode.parentNode.getElementsByTagName("input");
			if(inputs && inputs.length > 0){
				var checked = e.target.checked;
				if(e.target.value == Constants.getMessage("labelAll")){
					for(var i = 0; i < inputs.length; i++){
						inputs[i].checked = checked;
					}
				}else if(checked == false){
					inputs[0].checked = false;
				}
			}
			this._checkAdvancedPropertiesEnable();
		},
        
        _addCheckBoxAndValue: function(row, value){
            var td = document.createElement("TD");
			var inputLabel = document.createElement("label");
			inputLabel.setAttribute("for", value);
            var input = document.createElement("input");
            input.type = "checkbox";
            input.id = value;
			input.name = value;
            input.value = value;
            td.appendChild(inputLabel);
			td.appendChild(input);
            td.className = "checkBox";
            row.appendChild(td);
            this.connect(td, "onclick", this._checkAllAttributes);
        },
        
        _clearResourceInfo: function(){
            this.setExclusions(null);
            this.setInclusions(null);
            this.setWeightings(null);
        },
        
        _checkAdvancedPropertiesEnable: function(){
			var numberOfSelectedTypes = 0; // Number of selected environment types
			var maxNumberOfSelectedAttritute = 0;// Number of selected attributes of a environment type
            for (var i = 0; i < this.tables.length; i++) {
                inputs = this.tables[i].getElementsByTagName("input");
                var num = 0;
                if (inputs) {
                    for (var j = 0; j < inputs.length; j++) {
                        var input = inputs[j];
                        if (input.checked && input.value != Constants.getMessage("labelAll")) {
                            num++;
                        }
                    }
					maxNumberOfSelectedAttritute = maxNumberOfSelectedAttritute > num ? maxNumberOfSelectedAttritute : num;
					if (num > 0) {
		                numberOfSelectedTypes++;
		            }
                }
            }
			if(maxNumberOfSelectedAttritute > 0 && this.enableNextButton != null){
				this.enableNextButton(null, false);
			}else{
				this.enableNextButton(null, true);
			}
			if(maxNumberOfSelectedAttritute > 1){
				this.advancedPropertiesEnabled = true;
                this.enableCoverageOptions(numberOfSelectedTypes);
                this.advancedPropertiesButton.setDisabled(false);
				return;
			}else{
				this.advancedPropertiesEnabled = false;
			}
            this.disableCoverageOptions();
            this._clearResourceInfo();
            if (this.advancePropertiesPopupWindow) {
                this.checkEnviromentAndProperties();
                this.advancePropertiesPopupWindow.refresh(this.newInfoForExIn, this.newInfoForWeightings, this.refreshExIn, this.refreshWeightings);
                this.oldInfoForExIn = this.newInfoForExIn;
                this.oldInfoForWeightings = this.newInfoForWeightings;
            }
            this.advancedPropertiesButton.setDisabled(true);
        },
        
        _addUnchangedColumn: function(row, data){
            var td = document.createElement("TD");
            var text = document.createTextNode(data);
            td.appendChild(text);
            row.appendChild(td);
        },
        
        getResourceInfo: function(){
            var list = [];
            for (var i = 0; i < this.tables.length; i++) {
                var values = [];
                inputs = this.tables[i].getElementsByTagName("input");
                if (inputs) {
                    for (var j = 0; j < inputs.length; j++) {
                        var input = inputs[j];
                        if (input.checked && input.value != Constants.getMessage("labelAll")) {
                            values.push(input.value);
                        }
                    }
                }
                if (values.length > 0) 
                    list.push(values);
            };
            return list;
        },
        
        _getResourceInfoForExIn: function(){
            var list = [];
            for (var i = 0; i < this.tables.length; i++) {
                var args = {};
                var id = this.tables[i].getAttribute("dojoattachpoint");
                args.key = this._getAttributeTypeById(id).name;
                args.values = new Array();
                inputs = this.tables[i].getElementsByTagName("input");
                if (inputs) {
                    var flag = false;
                    for (var j = 0; j < inputs.length; j++) {
                        var input = inputs[j];
                        if (input.checked) {
                            flag = true;
                        }
						if (input.value != Constants.getMessage("labelAll")){
							args.values.push(input.value);
						}
                    }
                    if (flag == true) {
                        list.push(args);
                    }
                }
            };
            return list;
        },
        
        _getResourceInfoForWeightings: function(){
            var list = [];
            for (var i = 0; i < this.tables.length; i++) {
                var args = {};
                var id = this.tables[i].getAttribute("dojoattachpoint");
                args.key = this._getAttributeTypeById(id).name;
                args.values = new Array();
                inputs = this.tables[i].getElementsByTagName("input");
                if (inputs) {
                    for (var j = 0; j < inputs.length; j++) {
                        var input = inputs[j];
                        if (input.checked && input.value != Constants.getMessage("labelAll")) {
                            args.values.push(input.value);
                        }
                    }
                }
                if (args.values.length > 0) 
                    list.push(args);
            };
            return list;
        },
        
        _handleAdvProEnvClick: function(){
			if(this.advancedPropertiesEnabled == false){
				return;
			}
            if (this.advancePropertiesPopupWindow == null) {
                this.advancePropertiesPopupWindow = this._createAdvancePropertiesPopupWindow();
				this.advancePropertiesDialog = new HiddenableDialog({
					primaryTitle: Constants.getMessage("labelAdvancedProperties"),
                	okCallback: dojo.hitch(this.advancePropertiesPopupWindow, "_onOK"),
					preCheck: dojo.hitch(this.advancePropertiesPopupWindow, "_compareExclusionsAndInclusions")
				});
				this.advancePropertiesDialog.getOkButton().disabled = false;
				this.advancePropertiesDialog.contentContainer.appendChild(this.advancePropertiesPopupWindow.domNode);
				this.advancePropertiesDialog.setSecondaryTitleNode(this.advancePropertiesPopupWindow.createCSHelpID().domNode);
				this.advancePropertiesDialog.open();
                this.oldInfoForExIn = this._getResourceInfoForExIn();
                this.oldInfoForWeightings = this._getResourceInfoForWeightings();
            }
            else {
                this.checkEnviromentAndProperties();
                this.advancePropertiesPopupWindow.refresh(this.newInfoForExIn, this.newInfoForWeightings, this.refreshExIn, this.refreshWeightings);
				this.advancePropertiesDialog.show();
                this.oldInfoForExIn = this.newInfoForExIn;
                this.oldInfoForWeightings = this.newInfoForWeightings;
            }
        },
        
        _createAdvancePropertiesPopupWindow: function(){
            var self = this;
            var excludesIncludesEnv = this._getResourceInfoForExIn();
            var weightingEnv = this._getResourceInfoForWeightings();
            var setExclusions = function(exclusions){
                self.setExclusions(exclusions);
            }
            
            var setInclusions = function(inclusions){
                self.setInclusions(inclusions);
            }
            
            var setWeightings = function(weightings){
                self.setWeightings(weightings);
            }
            this.dialogContent = new ConfigurationPopup({
                excludesIncludesEnv: excludesIncludesEnv,
                weightingEnv: weightingEnv,
                setExclusions: setExclusions,
                setInclusions: setInclusions,
                setWeightings: setWeightings
            });
            return this.dialogContent;
        },
		
		checkEnviromentAndProperties: function(){
            this.refreshExIn = false;
            this.refreshWeightings = false;
            this.newInfoForExIn = this._getResourceInfoForExIn();
            this.newInfoForWeightings = this._getResourceInfoForWeightings();
            if (this.newInfoForExIn.length != this.oldInfoForExIn.length) {
                this.refreshExIn = true;
            }
            else {
                for (var i = 0; i < this.newInfoForExIn.length; i++) {
                    if (this.newInfoForExIn[i].key != this.oldInfoForExIn[i].key) {
                        this.refreshExIn = true;
                        break;
                    }
                }
            }
            
            if (this.newInfoForWeightings.length != this.oldInfoForWeightings.length) {
                this.refreshWeightings = true;
            }
            else {
                for (var i = 0; i < this.newInfoForWeightings.length; i++) {
                    var newValues = this.newInfoForWeightings[i].values.toString();
                    var oldValues = this.oldInfoForWeightings[i].values.toString();
                    if (this.newInfoForWeightings[i].key != this.oldInfoForWeightings[i].key || newValues != oldValues) {
                        this.refreshWeightings = true;
                        break;
                    }
                }
            }
            return (this.refreshExIn || this.refreshWeightings);
        },
        
        getResourceInfomation: function(){
            var env = this.getResourceInfo();
            return dojo.toJson(env);
        },
        
        getExecutionEnvTypes: function(){
            var list = [];
            for (var i = 0; i < this.tables.length; i++) {
                inputs = this.tables[i].getElementsByTagName("input");
                if (inputs) {
                    var flag = false;
                    for (var j = 0; j < inputs.length; j++) {
                        var input = inputs[j];
                        if (input.checked && input.value != Constants.getMessage("labelAll")) {
                            flag = true;
                            break;
                        }
                    }
                    if (flag == true) {
                        var id = this.tables[i].getAttribute("dojoattachpoint");
                        var envType = this._getAttributeTypeById(id);
                        list.push(envType);
                    }
                }
            };
            return list;
        },
        
        
        getExclusions: function(){
            if (this.exclusions) {
                return (dojo.toJson(this.exclusions));
            }
            else {
                return dojo.toJson([]);
            }
        },
        
        getInclusions: function(){
            if (this.inclusions) {
                return (dojo.toJson(this.inclusions));
            }
            else {
                return dojo.toJson([]);
            }
        },
        
        getWeightings: function(){
            if (this.weightings) {
                return (dojo.toJson(this.weightings));
            }
            else {
                return dojo.toJson([]);
            }
        },
        
        setExclusions: function(exclusions){
            this.exclusions = exclusions;
        },
        
        setInclusions: function(inclusions){
            this.inclusions = inclusions;
        },
        
        setWeightings: function(weightings){
            this.weightings = weightings;
        },
        
        _error: function(e) {
			if(e && e.message){
				console.error(e.message);
			}else{
				console.error(e);
			}
		}
    });
    
})();
