/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 //
 // Summary:
 //    AttachmentContainer encapsulates GenericAttachmentViewer to provide attachments support for artifact.
 //
console.debug("Loading com.ibm.rqm.planning.web.ui.internal.view.construction.AttachmentContainer");
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.construction.AttachmentContainer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

dojo.require("com.ibm.rqm.planning.web.client.internal.AttachmentRestClient");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.GenericAttachmentViewer");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function(){
    var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    var AttachmentClient       = com.ibm.rqm.planning.web.client.internal.AttachmentRestClient;
    var Constants              = com.ibm.rqm.planning.web.client.internal.Constants;
	var CommonUtility          = com.ibm.asq.common.web.ui.internal.util.Utility;
    var LabelUnknown           = Constants.getMessage("labelUnknown");
	
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.construction.AttachmentContainer", [dijit._Widget, dijit._Templated], {
        templateString: '<div dojoAttachPoint="Attachment">' +
						' <div dojoAttachPoint="_attachments">' +
						' </div>' +
				        '</div>',
		
        constructor: function(){
            this.artifact = null;
            this.artifactType = null;
			this.parentEditor = null;
			this.isPreviewMode = false;
			this.isNonEditableMode = false;
        },
        
        postCreate: function(){
        },
        /*
		 * This widget associate attachments with certain artifact
		 * The widget has 2 public methods
		 *
		 * onSave()
		 * This can be called to add/delete attachments to certain artifact
		 * 
		 * initialize()
		 * First function to be called when the view is created.
		 * artifact: the artifact that needs attachments, such as test plan or test case
		 * artifactType: this artifactType being edited
		 * parentEditor: artifact editor, such as Test Plan Editor or Test Case Editor
		 * isPreviewMode: boolean
		 * isNonEditableMode: boolean
		 */
        initialize: function(args){
            this.artifact = args.artifact;
			this.parentSection = args.parentSection ? args.parentSection : null;
			this.parentEditor = args.parentEditor;
			this.isPreviewMode = (args.isPreviewMode)?args.isPreviewMode:this.isPreviewMode;
			this.isNonEditableMode = (args.isNonEditableMode)?args.isNonEditableMode:this.isNonEditableMode;
			this._artifactType = this._getArtifactType(this.parentEditor.declaredClass);
            if(!this.artifact){
				this._initialAtta();
			} else if (this.artifact && this._artifactType) {
				var responseHandler = new ServiceResponseHandler(this, "_initialAtta", "_initError");
				var param = {
					artifactUUID: this.artifact.itemId,
					artifactType: this._artifactType
				}
				AttachmentClient.getAttachmentsWithArtifact(responseHandler, param);
			}
        },
        
        // The response from the server on the call to getAttachmentsWithArtifact uses a hard
        // coded <section name> + " Section" as description for the path if the attachment is
        // added from other section.  The section name is keyword that needs to be looked up
        // to appropriate display string.
        //
        _lookupDescriptionPath: function(path) {
            // check if tha path is actually a section description
            var sectionStart = path.indexOf(" Section");
            if (sectionStart > 0) {
                // get the section id string
                var sectionKey = path.substring(0, sectionStart);
                // lookup the display string for the specific id key
                return Constants.getMessage("labelTitleSection", Constants.getExtensionDisplayMessage(sectionKey));
            } else {
                // return the path as is if it is not a section description
                return path;
            }
        },
        
        _initialAtta: function(results){
			var elements = [];
            if (results && results.length > 0) {
                for (var i = 0; i < results.length; i++) {
                    elements.push({
                        isNew: false,
                        fileName: results[i].name,
                        id: results[i].id + "", // fix 11251
                        uuid: results[i].itemId,
                        size: results[i].content.contentLength,
						path: this._lookupDescriptionPath(results[i].description),
						noRemovable: this._noRemovable(results[i].description)
                    });
                }
            }
            // destroy any old widgets first before creating the new one.
            CommonUtility.destroyWidgetsFromNode(this._attachments);
            dojox.data.dom.removeChildren(this._attachments);

            this._attachmentWidget = new com.ibm.asq.common.web.ui.internal.widgets.GenericAttachmentViewer();
            this._attachmentWidget.initialize({
				parentSection : this.parentSection != null ? this.parentSection : null,
				isPreviewMode: this.isPreviewMode,
                isNonEditableMode: this.isNonEditableMode,
                elements: elements
            });
            this._attachments.appendChild(this._attachmentWidget.domNode);
        },
        
        onCancel: function(){
        
        },
        onSave: function(artifact){
			if(artifact) {
				this.artifact = artifact;
			}
            var commands = this._attachmentWidget.getNewAttachments();
            var added = [];
            if (commands && commands.length > 0) {
                for (var i = 0; i < commands.length; i++) {
                    added.push(commands[i].uuid);
                }
            }
            var commands1 = this._attachmentWidget.getRemovedAttachments();
            var removed = [];
            if (commands1 && commands1.length > 0) {
                for (var i = 0; i < commands1.length; i++) {
                    removed.push(commands1[i].uuid);
                }
            }
			if (this._artifactType) {
				var responseHandler = new ServiceResponseHandler(this, "_success", "_saveError");
				var param = {
					artifactUUID: this.artifact.itemId,
					addedAttachments: added,
					deletedAttachments: removed,
					artifactType: this._artifactType
				}
				AttachmentClient.postAttachmentsWithArtifact(responseHandler, param);
			}
        },
        
        _getArtifactType: function(declareClass){
            if(declareClass == "com.ibm.rqm.planning.web.ui.internal.view.planning.TestPlanEditor"){
                return "TestPlan";
            }
            if(declareClass == "com.ibm.rqm.planning.web.ui.internal.view.construction.TestCaseEditor"){
                return "TestCase";
            }
            return null;
        },        
		
        _success: function(){
            this.initialize({
                artifact: this.artifact,
				parentEditor: this.parentEditor,
				parentSection: this.parentSection
            });
        },
		
		//Only attachments in "Attachments Section" can be removed. 
		//Those in RichText Section can't be removed in "Attachments Section".
		_noRemovable: function(path) {
			// check if tha path is actually a section description
            var sectionStart = path.indexOf(" Section");
            if (sectionStart > 0) {
                // get the section id string
                var sectionKey = path.substring(0, sectionStart);
				if(sectionKey == "Attachments")
					return false;
            } 
			return true;
		},
        
        _initError: function(e){
            this._initialAtta(null);
            var detail = (e && e.message) ? e.message : LabelUnknown;            
            this._attachmentWidget.showContentError(Constants.getMessage("msgLoadAttachError", detail));              
        },
        
        _saveError: function(e){
			var msg = Constants.getMessage("msgSaveAttachError");
			var detail = (e && e.message) ? e.message : null;
			
            if (this.parentEditor) {
				this.parentEditor.setHeaderStatus(msg, 1, detail);
			}
			else {
				// last resort, show an dialog to notify about the error.
				// Attachments should really be a part of the document save!
				var args = {
					width: "40%",
					primaryTitle: Constants.getCommonMessage("labelAlert"),
					message: (msg + '<br/><br/>' + detail)
				}
				var alertDialog = new com.ibm.asq.common.web.ui.internal.widgets.AlertDialog(args);
				alertDialog.open(true);
			}
        }
    });
})();
