/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading com.ibm.rqm.planning.web.ui.internal.view.construction.CategorySelection");
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.construction.CategorySelection");

dojo.require("dojox.data.dom");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.rqm.planning.web.client.internal.CategoryClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require( "com.ibm.asq.common.web.ui.internal.util.SelectUtility" );

(function(){
    var CategoryClient = com.ibm.rqm.planning.web.client.internal.CategoryClient;
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	var ImageCache = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
	var SelectUtility = dojo.getObject( "com.ibm.asq.common.web.ui.internal.util.SelectUtility" );

	var UNASSIGNED_CATEGORY = Constants.getMessage("labelUnassigned");
	
	dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.construction.CategorySelection",
	[dijit._Widget, dijit._Templated],
	{
        //		
	    // Provides dynamic selection of Categories 
	    //
	    templateString: '<div class="com-ibm-rqm-category-selection">' +
		                '  <table width="100%" class= "category-table" dojoAttachPoint="_categoriesNode"></table>' +
		                '</div>',
		
		
		constructor: function() {
			this.parent == null;
			this.artifact = null;
			this.artifactType = null;
			this._allTypes = [];     //CategoryTypeDTO
			this._selections = [];
            this._disabled = false;
		},		
	    
	    postCreate: function() {
			if (!this.artifactType) {
				throw new Error("Parameter 'artifactType' is REQUIRED!");
			}
            this.initialize([]);	      
	    },
		
		initialize: function(selections) {
			var self = this;
			var handlerObj = {
				success: function(allCategoryTypes) {
		            self.setCategories(allCategoryTypes);
		            self.showSelections(selections);
					
					//Fix defect 17403
					if(dojo.isFunction(self.afterInitialize)){
						self.afterInitialize();	
					}
				},
				failure: function(e) {
					self._error(e);
				}
			}		
			
            var responseHandler = new ServiceResponseHandler(handlerObj, "success", "failure");
            CategoryClient.getCategoryTypes(responseHandler, this.artifactType, false);     			
		},
		
		refresh: function() {
			//get selections
			var selectedMap = this.getSelectedByTypes();
            this.initialize(selectedMap);		  
		},
		
		showSelections: function(/* type: name, optional */selections) {
			this._selections = [];
			dojox.data.dom.removeChildren(this._categoriesNode);
			
			//if no categories exist, show message
			if (!dojo.isArray(this._allTypes) || this._allTypes.length == 0) {
				this._showEmptyMessage();
				return;
			}
			
			//show selects for ecah category type			
			for (var i = 0; i < (this._allTypes ? this._allTypes.length : 0); i++) {
				var type = this._allTypes[i]; 
				var values = type.categories ? type.categories.sort(this._sortValues) : [];
				var selected = this.getCurrentCategory(type);
                if (selections && selections[type.itemId] != null) {
                    selected = selections[type.itemId];
                }
				
				//create contanier & label
				var container = document.createElement("div");
				dojo.addClass(container, "category-container");
				
				var label = document.createElement("label");	
				dojo.addClass(label, "category-label");
				label.innerHTML = type.name + ":";
				label.setAttribute("for", "category-select" + type.name);
				
				//create selection list				
				var select = document.createElement("select");
				var warning = null;       
                dojo.addClass(select, "category-select");
				if(dojo.isIE) {
					select.style.width = "200px";
				}
				
				select.typeId = type.itemId;
				SelectUtility.add(select, new Option(UNASSIGNED_CATEGORY, UNASSIGNED_CATEGORY));
				for (var j = 0; j < (values ? values.length : 0); j++) {	                
						var option = new Option(values[j].name, values[j].itemId);
						if (selected && selected.itemId == values[j].itemId) {
							option.selected = true;
						}
						if (!values[j].archived || option.selected) {
							if (values[j].archived) {
								dojo.addClass(option, "archived");
								warning = this._getArchivedWarning(type, values[j]);
							}
							SelectUtility.add(select, option);                        
						}
				}              
                this.connect(select, "onchange", dojo.hitch(this, "_selectChanged", type.itemId));
                select.disabled = this.isDisabled();
				select.id = "category-select" + type.name;
				select.name = "category-select" + type.name;
				this._selections.push(select);				
				this._addCategoryRow(label, select, warning);
			}
		},
		
		_showEmptyMessage: function() {
            //show message indicating there are no categories available
            var table = this._categoriesNode;
            var row = table.insertRow(table.rows.length);
            dojo.addClass(row, "category-row");			
						
            var labelTD = row.insertCell(row.cells.length);
            dojo.addClass(labelTD, "category-label-col");
			labelTD.colSpan = 2; 
            labelTD.appendChild(document.createTextNode(Constants.getMessage("labelNoCategories")));						
		},
 		
		_getArchivedWarning: function(type, category) {
			var img = document.createElement("img");
			img.src = ImageCache.WARNING_ICON;
			img.title = Constants.getMessage("msgCategoryArchived", type.name, category.name);
			return img;
		},
		
        //////////////////////////////////////////////////////////////////////////////////////////
        ///                                        API                                         ///
        //////////////////////////////////////////////////////////////////////////////////////////
        /*
         * Summary:
         *      Set the types (all CategoryTypeDTOs with complete CategoryDTO set) 
         */         
        setCategories: function(allCategoryTypes) {           
            this._allTypes = allCategoryTypes;
        }, 
           
        /*
         * Summary:
         *      Returns an array of ItemIds as Strings representing the currently selected values 
         *      for each of the CategoryType selection boxes
         */          
        getSelectedItemIds: function() {
            var selections = [];
			for (var i = 0; i < this._selections.length; i++) {
				if (this._selections[i].value != UNASSIGNED_CATEGORY) {
					selections.push(this._selections[i].value);
				}
			}            
            return selections;
        },	

        /*
         * Summary:
         *      Returns an array of Category objects representing the currently selected values 
         *      for each of the CategoryType selection boxes
         */          
        validate: function() {
			var categories = this.getSelectedCategories();
			for (var i = 0; i < categories.length; i++) {
				if (categories[i].archived) {
					//message
					var type = this._getCategoryTypeByItemId(categories[i].categoryType.itemId);
					var msg = Constants.getMessage("msgCategoryArchived", type.name, categories[i].name);
					if (this.parent && dojo.isFunction(this.parent.getParentEditor)) {
						this.parent.getParentEditor().setHeaderStatus(Constants.VALIDATION_ERROR, Constants.HEADER_STATUS_WARNING, msg);
					} else {
						alert(msg);
					}
					return false;
				}
			}
			return true;
		},        
        
		/*
         * Summary:
         *      Returns an array of Category objects representing the currently selected values 
         *      for each of the CategoryType selection boxes
         */          
        getSelectedCategories: function() {
            var categories = [];
            for (var i = 0; i < this._selections.length; i++) {
                var itemId = this._selections[i].value;
                if (itemId != UNASSIGNED_CATEGORY) {
                    categories.push(this._getCategoryByItemId(itemId));
                }
            }            
            return categories;
        },  
		
		getSelectedByTypes: function() {
            var selectedMap = {};
            for (var i = 0; i < this._selections.length; i++) {
                var itemId = this._selections[i].value;
				if (itemId != UNASSIGNED_CATEGORY) {
					var category = this._getCategoryByItemId(itemId);
                    selectedMap[category.categoryType.itemId] = category;
                }
            }            
            return selectedMap;
		},
        
        /*
         * Summary:
         *      Returns the current Category (on the test case) for the specified CategoryType, 
         *      or null if this is a new Test Case. 
         * 
         */        
        getCurrentCategory: function(type) {
            var persisted = this.artifact ? this.artifact.categories : [];
            for (var i = 0; i < (persisted ? persisted.length : 0); i++) {
                if (persisted[i].categoryType.itemId == type.itemId) {
                    return persisted[i];
                }
            }
            return null;
        },          
		
		setDisabled: function(/* boolean */ disabled) {
			this._disabled = disabled;
            for (var i = 0; i < this._selections.length; i++) {
                this._selections[i].disabled = this._disabled;
            }            
		},
        
		isDisabled: function() {
			return this._disabled;
		},
        
		setLabelWidth: function(width) {
			this.tdWidth = width;
			var children = this._categoriesNode.getElementsByTagName("TD");
			for(var i=0; i<(children ? children.length : 0); i++) {
				if(children[i].className == "category-label-col")
					children[i].style.width = this.tdWidth;
			}
		},
        
        // Summary:
        //    Returns the width of category fields' label width in integer pixel value.
        //
        getLabelWidth: function() {
            var labels = dojo.query(".category-label-col", this._categoriesNode);
            if(labels && labels.length > 0) {
                return labels[0].clientWidth;
            }
            return this.tdWidth || 0;
        },
        
        //////////////////////////////////////////////////////////////////////////////////////////
        ///                                      Internal                                      ///
        //////////////////////////////////////////////////////////////////////////////////////////           
        _getCategoryTypeByItemId: function(itemId) {
            for (var i = 0; i < (this._allTypes ? this._allTypes.length : 0); i++) {
                if (this._allTypes[i].itemId == itemId) {
                    return this._allTypes[i];
                }
            }
            return null;            
        },
		        
		_getCategoryByItemId: function(itemId) {
            for (var i = 0; i < (this._allTypes ? this._allTypes.length : 0); i++) {
				var categories = this._allTypes[i].categories;
				for (var j = 0; j < (categories ? categories.length : 0); j++) {
					if (categories[j].itemId == itemId) {
						return categories[j];
					}
				}
			}
            return null;            
        },		
		
		_getSelectByTypeItemId: function(itemId) {
            for (var i = 0; i < (this._selections ? this._selections.length : 0); i++) {
                if (this._selections[i].typeId == itemId) {
				    return this._selections[i];
				}			 
			}
			return null;
        },
        
        _selectChanged: function(typeItemId) {
			if (this.parent && dojo.isFunction(this.parent.setDirty)){
				this.parent.setDirty();
			}
			//TODO remove the warning if necessary
			var select = this._getSelectByTypeItemId(typeItemId);
			if (select.nextSibling) {
				select.parentNode.removeChild(select.nextSibling);
			}
			//add the warning if necessary
			var category = this._getCategoryByItemId(select.value);
			if (category && category.archived) {
				select.parentNode.appendChild(this._getArchivedWarning(category));
			}
			//validate
			this.validate();
        }, 
		
		_addCategoryRow: function(label, select, warning) {
			var table = this._categoriesNode;
			var row = table.insertRow(table.rows.length);
			dojo.addClass(row, "category-row");
			
			var labelTD = row.insertCell(row.cells.length);
			dojo.addClass(labelTD, "category-label-col");
			if(this.tdWidth)
				labelTD.style.width = this.tdWidth;
			labelTD.appendChild(label);
			
			var selectTD = row.insertCell(row.cells.length);
            dojo.addClass(selectTD, "category-select-col");
			selectTD.appendChild(select);	
			if (warning != null) {
				selectTD.appendChild(warning);
			}
			return row;		
		},
		
        _sortValues: function(val1, val2) {
            if (val1.name < val2.name) {
				return -1;
			} else if (val1.name > val2.name) {
				return 1;
			}
			return 0;
        },		
		
		_error: function(e) {
			throw new Error("An error occurred: [" + e.message + "] @ CategorySelection#_error")
		}
	});    
})();


