/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading com.ibm.rqm.planning.web.ui.internal.view.construction.NewSectionDialog");
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.construction.NewSectionDialog");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.util.SelectUtility");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.Button");
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function(){
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var CSHelp = com.ibm.asq.common.web.ui.internal.util.CSHelp;
	var SelectUtility = com.ibm.asq.common.web.ui.internal.util.SelectUtility;

    /**
     *
     */
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.construction.NewSectionDialog", [dijit._Widget, dijit._Templated], {
        templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/construction/NewSectionDialog.html"),
        
		labelSectionName: Constants.getMessage("labelSectionName"),
		labelDescription: Constants.getMessage("labeldescription"),
		labelApplyToTemplate: Constants.getMessage("labelApplyToTemplate"),
		labelOk: Constants.getMessage("labelOk"),
        labelCancel: Constants.getMessage("labelCancel"),
        labelTypeOfSection: Constants.getMessage("labelTypeOfSection"),
		
		//grid labels
		labelNumOfCol: Constants.getMessage("labelNumOfCol"),
		labelColumnNames: Constants.getMessage("labelColumnNames"),
		labelEnterColumnNames: Constants.getMessage("labelEnterColumnNames"),
		
        constructor: function(args) {
			this._typeIdMap = {};
			this._typeIdMap[Constants.DYNAMICSECTION_GRID] = Constants.DYNAMICSECTION_GRID_ID;
			this._typeIdMap[Constants.DYNAMICSECTION_RICHTEXT] = Constants.DYNAMICSECTION_RICHTEXT_ID;
		},
        
        postCreate: function(){
			if (!dojo.isFunction(this.onOk)) {
				throw new Error("Parameter 'onOk' is REQUIRED and must be a Function.");
			}
			
			//hide not-yet-needed ui elements
			this._applyTemplateCheckbox.style.display = "none";
			this._applyTemplateLabel.style.display = "none";
			this._messages.style.display = "none";
			
            var dialogArgs = {
				primaryTitle: Constants.getMessage("labelNewSection"),
                okCallback: dojo.hitch(this, "_onOk"),
                cancelCallback: dojo.hitch(this, "_onCancel"),
				_truncate: function() {}  //disable the Dialog's '_truncate'
            };      
            this._dialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(dialogArgs);
            this._dialog.setWidth("500px");
			var cshelp = new CSHelp({contextID: "rqm_new_custom.html"});
			cshelp.setButtonLabel(Constants.getMessage("labelNewSectionCSHelp"));
			this._dialog.setSecondaryTitleNode(cshelp.domNode);
			this._dialog.contentContainer.appendChild(this.domNode);
            this._dialog.getOkButton().disabled = false;
            this._dialog.open();		
			
			var opt = new Option(Constants.getMessage("labelRichTextEditor"), Constants.DYNAMICSECTION_RICHTEXT);
			var opt1 = new Option(Constants.getMessage("labelGrid"), Constants.DYNAMICSECTION_GRID);
			SelectUtility.add(this._typeCombo, opt);
			SelectUtility.add(this._typeCombo, opt1);
			this.connect(this._typeCombo, "onchange", "_handleTypeChange");
			
			setTimeout(dojo.hitch(this, function() {this._name.focus()}), 250);		
        },
        
        _onCancel: function() {
			if (this._dialog) {
				this._dialog = null;
			}			
        },
        
        _onOk: function() {
			this._messages.style.display = "none";
            if (this._name.value == null || dojo.trim(this._name.value).length == 0) {
				dojox.data.dom.removeChildren(this._messages);
				this._messages.style.display = "inline";
				this._messages.appendChild(document.createTextNode(Constants.getMessage("labelSectionNameRequired")));
				return false;
            } else {
				var type = this._typeCombo.value;
				var isGrid = (type == Constants.DYNAMICSECTION_GRID);
				var numColumns = isGrid ? this._getNumColumns() : null;
				
				if (isGrid && numColumns == null) {
					//invalid grid configuration - must have some columns
					dojox.data.dom.removeChildren(this._messages);
					this._messages.style.display = "inline";
					this._messages.appendChild(document.createTextNode(Constants.getMessage("msgColumnNumberInvalid")));
					return false;
				}
				if (dojo.isFunction(this.onOk)) {
					var params = {
						type: type
					};
					if (isGrid){
						//ensure all columns have a name; collect and add them to configuration data for new section
						var colNames = [];
						var colLen = numColumns != null ? numColumns : 0;
						for(var i=0; i < colLen; i++){
							 var col = document.getElementById("col_"+ i);
							 if (!col || !col.value || dojo.trim(col.value).length == 0) {
							 	dojox.data.dom.removeChildren(this._messages);
								this._messages.style.display = "inline";
								this._messages.appendChild(document.createTextNode(Constants.getMessage("msgColumnNameInvalid", (i+1) ) ) );
								return false;
							 }
							 colNames[i] = col.value;
						}
						params.configurationData = dojo.toJson(colNames);
					}
					
					var uid = this._typeIdMap[type] + new Date().getTime();
	                this.onOk(uid, this._name.value, this._description.value, params);
	            }		
            }
        },
        
        _onKeyup: function(e){
/*
enter key on any one of the text fields should not close the dialog 
            if (e.keyCode == dojo.keys.ENTER) {
                if (this._dialog) {
					this._dialog.getOkButton().onclick();
				}
            }
*/
        },

		_handleTypeChange: function() {
			var isGrid = (this._typeCombo.value == Constants.DYNAMICSECTION_GRID);
			if (isGrid) {
				dojo.removeClass(this._gridConfigRow, "hidden");
				this._addEditButton();
			} else {
				dojo.addClass(this._gridConfigRow, "hidden");
			}
		},
        
		_updateNumColumns: function(){
			var numColumns = this._getNumColumns();
			this._editButton.setDisabled(numColumns == null);
			if (numColumns == null) {
				dojox.data.dom.removeChildren(this._messages);
				this._messages.style.display = "inline";
				this._messages.appendChild(document.createTextNode(Constants.getMessage("msgColumnNumberInvalid")));
				return;
			} else {
				this._messages.style.display = "none";
			}
		},
		
		_updateTableRows: function() {
			var numColumns = this._getNumColumns();
			if (!numColumns) {
				return;
			}
			dojo.removeClass(this._columnsNode, "hidden");
			this._messages.style.display = "none";
			dojox.data.dom.removeChildren(this._colTbody);
			for (var i=0; i < numColumns; i++) {
				var tr = document.createElement('tr');
				var td = document.createElement('td');
				var inputBox = document.createElement('input');
				inputBox.value = "Column " + (i+1);
				inputBox.id = "col_"+i; 
				td.appendChild(inputBox);
				tr.appendChild(td);
				this._colTbody.appendChild(tr);
			}
		},
		
		_getNumColumns: function() {
			var numColumns = parseInt(this._numColumnsInputBox.value);
			return (!numColumns || isNaN(numColumns)) ? null : numColumns;
		},		
		
		_addEditButton: function() {
			if (this._editButton != null) {
				return;
			}
	        var arg = {
	            label: this.labelEnterColumnNames,
	            showLabel: false,
				iconClass: "edit-icon-image",
	            onClick: dojo.hitch(this, "_updateTableRows")
	        };

	        this._editButton = new com.ibm.asq.common.web.ui.internal.widgets.Button(arg);
			this._editButton.setDisabled(true);
			this._numColumnsNode.appendChild(this._editButton.domNode);
		}
    });
})();
