/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading com.ibm.rqm.planning.web.ui.internal.view.construction.QueryNavigator");
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.construction.QueryNavigator");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function() { // module container

var Constants    			= com.ibm.rqm.planning.web.client.internal.Constants;
var Utility    				= com.ibm.rqm.planning.web.ui.internal.util.Utility;

/**
 * Widget to display list of link elements in the LeftNavigation.
 */
dojo.declare(
	"com.ibm.rqm.planning.web.ui.internal.view.construction.QueryNavigator",
	[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/construction/Navigator.html"),
//		templateCssPath: dojo.uri.dojoUri("../com.ibm.rqm.planning.web/ui/internal/view/templates/construction/Navigator.css"),

		constructor: function() {
			this.controller = null;
			this._queries = [];
		},

		postCreate: function() { //args) {
//			this.controller = args.controller;			
		},
		
		_queries: [],
		
		/**
		 * Adds a collection of queries (with 'name' property) to the widget
		 */
		addQueries: function(newQueries) {
			if (newQueries != null) {
				// Sort them first.
				//var sorted = this._sort(newQueries);
				for (var i = 0; i < newQueries.length; i++) {
					//this.addElement(sorted[i]);
					this.addQuery(newQueries[i]);
				}
			}
		},

		/**
		 * Adds an query to the widget.
		 */
		addQuery: function(query) {
			//if this is the first query being added, clear the list div
			if ( this._queries == null || this._queries.length == 0 ) {
				dojox.data.dom.removeChildren(this.elementListDiv);	
			}
						
			this._queries.push(query);
			query.id = this._queries.length;
             
            var queryDiv = document.createElement("div");
			queryDiv.className= "element-div-unselected";

    		query.div = queryDiv;
            queryDiv.appendChild(this._buildQueryLink(query));
				
            this.elementListDiv.appendChild(queryDiv);
		},

		/**
		 * Adds a link for the query to the list.
		 */
		_buildQueryLink: function (query) {
			var linkText = query.name;
			var anchor= document.createElement("a");
			// Construct the link here
			anchor.title = linkText;
			if ( query.type == Constants.TYPE_GROUP ) {
				anchor.href = Utility.getTestGroupsURI(query.name);
			} else if ( query.type == Constants.TYPE_CASE ) {
				anchor.href = Utility.getTestCasesURI(query.name);
			} else if ( query.type == Constants.TYPE_SCRIPT ) {
				anchor.href = Utility.getTestScriptsURI(query.name, Utility.CONSTRUCTION_PERSPECTIVE);
			}
			//anchor.href = this._linkPrefix + "&name=" + escape(element.name);
			//'xojo.event.connect(anchor, "onclick", this, this._handleElementClick);
			
			var text = document.createTextNode(linkText);
			anchor.appendChild(text);
			
			return anchor;
		},
			
		selectQuery: function(name) 
		{
			/*
			 * Highlight the selected option
			 */
			var queryLinks = this.elementListDiv.childNodes;
			for (var i=0; i < queryLinks.length; i++) 
			{
				var anchorText = queryLinks[i].textContent;
				if(anchorText == name) 
				{
					queryLinks[i].className = "element-div-selected";
				} 
				else
				{
					queryLinks[i].className = "element-div-unselected";
				}
			}
			
		},
		
		/**
		 * Helper function to sort a list of report descriptors
		 * Note: Assume that report names are unique
		 */
		_sort: function(elements) {
			var names = [];
			for (var i = 0; i < elements.length; i++) {
				names[i] = elements[i].name;
			}
			names = names.sort();
			var result = [];
			for (var i = 0; i < names.length; i++) {
				result[i] = this._find(names[i], elements);
			}
			return result;
		},
		
		_find: function(name, elements) {
			for (var i = 0; i < elements.length; i++) {
				if (elements[i].name == name) {
					return elements[i];
				}
			}
			return null;
		}
	}
);

})();
