/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.construction.SnapshotEnabledEditor");

dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.SnapshotPane");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.SnapshotDialog");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");

(function(){    
    var SnapshotDialog = com.ibm.rqm.planning.web.ui.internal.view.construction.SnapshotDialog;
    var SnapshotPane = com.ibm.rqm.planning.web.ui.internal.view.construction.SnapshotPane;
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var Utility = com.ibm.rqm.planning.web.ui.internal.util.Utility;
    
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.construction.SnapshotEnabledEditor", null, {
    
        _overviewElements: new Array(),
        
		constructor: function() {
			this._snapshotClient = null;
		},
		
		setSnapshotClient: function(snapshotClient) {
			this._snapshotClient = snapshotClient;
		},
		
        getSnapshotPane: function() {
            // create snapshots pane if necessary
            if (this._snapshotPane == null) {
                this._snapshotPane = new SnapshotPane();
                this._snapshotPane.initialize({
                    editor: this
                });
                //this._snapshotPaneArea.appendChild(this._snapshotPane.domNode);
            }
            // give the server a second to catch up before refreshing
            //this._snapshotPane.refreshSnapshots();
            return this._snapshotPane;           
        },
	   
        refreshSnapshots: function() {
			this.getSnapshotPane().refreshSnapshots();
		},       
               
        onSnapshot: function(){
            this.snapshotDialog = new SnapshotDialog({
				onOk: dojo.hitch(this, "snapshotDialogOnOk"),
                onCancel: dojo.hitch(this, "snapshotDialogOnCancel")
            });
        },
        
        snapshotDialogOnCancel: function(){
            //this._snapshotFloatingPane.hide();
        },
        
        snapshotDialogOnOk: function(name, description){
            // set the snapshot properties on the editor
            this._createSnapshot = true;
            this._snapshotName = name;
            this._snapshotDescription = description;
            this.setDirty(true);
			
			// show the Loading message
			//this._snapshotPane.showLoadingMessage(true);
            this.onSave();
        },

		getSnapshotInfo: function()	{
			if (this.getArtifact()!=null && !this.isCopy()) {
				return this.getArtifact().snapshotInfo;
			} else {
				return null;
			}
		},
		        
        getSnapshots: function(responseHandler){
			if (this._snapshotClient == null) {
	            throw new Error("SnapshotEnabledEditor#getSnapshots snapshot client not provided for editor " + this);
			}
			if (!this.isNew() && !this.isCopy()) {
				this._snapshotClient.getSnapshots(responseHandler, {uuid: this.getItemId()});
			} else {
				// no snapshots until it has been saved at least once
				responseHandler.doCallback({});
			}
        },
        
        onCurrentRevision: function(artifactType, artifactId) {
            if (artifactId) {
                var currentPerspective = Utility.getCurrentPerspectiveByURI();
                var href = null;
                if (artifactType == Constants.TYPE_PLAN) {
                    href = Utility.getTestPlanURI({id: artifactId}, currentPerspective);
                } else if (artifactType == Constants.TYPE_CASE) {
                    href = Utility.getTestCaseURI(artifactId, null, currentPerspective);
                } else if (artifactType == Constants.TYPE_SCRIPT) {
                    href = Utility.getTestScriptURI(artifactId, currentPerspective);
                } else if (artifactType == Constants.TYPE_SUITE) {
                    href = Utility.getTestSuiteURI(artifactId, currentPerspective);
                }

                if (href != null) {
                    window.location.href = href; //Utility.reuniquifyUrl(href);
                }
            }           
        }
        
    });
    
})();
