/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.construction.extension.TestCasesSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.core.loader");

dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.mapper.TestCaseMapper");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient");

dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");
dojo.require("com.ibm.rqm.planning.web.ui.internal.model.SaveParticipant");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.table.TestCaseTableView");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.SelectItemTableDialog");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.NewItemPopup");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.properties.TestPlanTestCaseConfigSelector");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

//xojo.require("com.ibm.rqm.configuration.web.ui.internal.view.configuration.GenerateEWIWizard");

(function(){ // module container   
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var Utility = com.ibm.rqm.planning.web.ui.internal.util.Utility;
	var TestCaseMapper = com.ibm.rqm.planning.web.client.internal.mapper.TestCaseMapper;
    var AbstractArtifactEditorSection = com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection;
	var TestCaseClient = com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient;
    var TestCaseTableView = com.ibm.rqm.planning.web.ui.internal.view.table.TestCaseTableView;
    var SelectItemTableDialog = com.ibm.rqm.planning.web.ui.internal.view.common.SelectItemTableDialog;
    var NewItemPopup = com.ibm.rqm.planning.web.ui.internal.view.construction.NewItemPopup;
    var EditorUtil = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
	var AlertDialog = com.ibm.asq.common.web.ui.internal.widgets.AlertDialog;
    var YesNoDialog = com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog;
	
    // global save context
    var SaveParticipant = com.ibm.rqm.planning.web.ui.internal.model.SaveParticipant;
    
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.construction.extension.TestCasesSection", [dijit._Widget, dijit._Templated, AbstractArtifactEditorSection], {
        templateString: "<div dojoAttachPoint=\"_testCasesSection\" class=\"com-ibm-rqm-planning-web-ui-internal-view-construction-extension-TestCasesSection\"></div>",
        
        constructor: function(args){
            this._items = [];
            this._itemType = Constants.TYPE_CASE;
            this._itemsAdded = [];
            this._itemsRemoved = [];
            this._referencesAddedType = "testCasesAdded";
            this._referencesRemovedType = "testCasesRemoved";
            this._initialized = false;
			this.ewiSaveContextGroup = [];
        },
        
        postCreate: function(){
            this.inherited('postCreate', arguments);
            this.addWorkItemSupport();
        },
        
        initialize: function(){
            this._itemsAdded = [];
            this._itemsRemoved = [];	
			this.showTestCaseTable();
			this.refreshTestCases("_setTestCases", "_error");		
			this.setSectionDirty(false);
        },
		
		refreshTestCases: function(successCB, errCB) {
            if (this.getParentEditor().isNew() && !this.getParentEditor().isCopy()) {
                return false;			 
			}
			var srh = new TestCaseMapper(this, successCB, errCB);
			TestCaseClient.getTestPlanTestCases(srh, {
				versionedTestPlanItemId: this.getArtifact().versionableItemId
			});
		},
		
		_setTestCases: function(testCases) {
            
            this._items = testCases ? testCases : [];			
            this.showTestCaseTable();
            this._initialized = true;			
			//if new item was clicked before save, open it
			this._openNewlySavedItem();
		},
        
        showTestCaseTable: function(){
            var artifact = this.getArtifact();
            
            if (!this._caseTableViewer) {
                dojox.data.dom.removeChildren(this._testCasesSection);
                
                this._caseTableViewer = new TestCaseTableView({
					parent: this,
                    editable: this.isEditable(),
					printable: this.isPrintView(),
                    enableRowSelection: true,
					enableRowEditable: this.isRolePermitsEdit(),
                    testCases: this.getItems(),
                    actions: this.getTableActions(),
					refreshCallback: dojo.hitch(this, "refreshTestCases"),
                    newItemLinkCallback: dojo.hitch(this, "_handleNewItemLink")
                });
                this._testCasesSection.appendChild(this._caseTableViewer.domNode);
            }
            else {
                this._caseTableViewer.refresh(this.getItems());
            }
			
			if (this._caseTableViewer.deleteActionNode){
				this._caseTableViewer.deleteActionNode.setDisabled(true);
			}
			if (this._caseTableViewer.changeSuspectActionNode){
				this._caseTableViewer.changeSuspectActionNode.setDisabled(true);
			}
			if (this._caseTableViewer.genEWIActionNode){
				this._caseTableViewer.genEWIActionNode.setDisabled(true);
			}
			if (this._caseTableViewer.executeActionNode){
				this._caseTableViewer.executeActionNode.setDisabled(true);
			}
        },
        
        getItems: function(){
            return this._items;
        },
        
        setItems: function(items){
            this._items = items;
        },
        
        //actions to be displayed in the table header
        getTableActions: function(){
            var actions = [];
            
            if (this.isRolePermitsEdit()) {
                actions.push({
                    label: Constants.getMessage("labelAddExistingTestCases"),
                    actionHandler: dojo.hitch(this, "_handleAddClick"),
                    iconClass: "add-icon-image",
                    disabledOnReadOnly: true
                });
            }
            var tc_type = com.ibm.asq.common.web.client.internal.PermissionsClient.OP_TEST_CASE;
            var tc_perm = com.ibm.asq.common.web.client.internal.PermissionsClient.getStoredPermission(tc_type);
            if (!tc_perm || tc_perm.create) {
                actions.push({
                    label: Constants.getMessage("labelAddNewTestCase"),
                    actionHandler: dojo.hitch(this, "_handleAddNewClick"),
                    iconClass: "add-new-testcase-image",
                    disabledOnReadOnly: true
                });
            }
            if (this.isRolePermitsEdit()) {
                actions.push({
                    label: Constants.getMessage("labelRemoveTestCases"),
                    actionHandler: dojo.hitch(this, "_handleRemoveClick"),
                    iconClass: "remove-enabled-icon-image",
                    disabledOnReadOnly: true
                });
            }
			var tc_exec_permission = com.ibm.asq.common.web.client.internal.PermissionsClient.getStoredPermission(
											com.ibm.asq.common.web.client.internal.PermissionsClient.OP_TEST_CASE_EXE);
			var hasExePermission = !tc_exec_permission ? true : tc_exec_permission 
														   		&& tc_exec_permission[com.ibm.asq.common.web.client.internal.PermissionsClient.ACTION_EXECUTE];
			if (hasExePermission) {
                actions.push({
                    label: Constants.getMessage("labelExeTest"),
                    actionHandler: dojo.hitch(this, "_onExecuteActionClick"),
                    iconClass: "execute-icon-image",
                    disabledOnReadOnly: true
                });
            }
            var ewi_type = com.ibm.asq.common.web.client.internal.PermissionsClient.OP_EXECUTION_EWI;
            var ewi_perm = com.ibm.asq.common.web.client.internal.PermissionsClient.getStoredPermission(ewi_type);
            if (!ewi_perm || ewi_perm.create) {
                actions.push({
                    label: Constants.getMessage("labelGenEWIs"),
                    actionHandler: dojo.hitch(this, "_handleGenerateEWIClick"),
                    iconClass: "generate-ewis-icon",
                    disabledOnReadOnly: true
                });
            }
            return actions;
        },
        
        //@Overridden from AbstractArtifactEditorSection
        getAttributes: function(){
            return null;
        },
        
        //@Overridden from AbstractArtifactEditorSection
        saveFinished: function(){
            this.inherited('saveFinished', arguments);
            
            // if we have not been shown, don't need to
            // create the content
            if (this._initialized) {
                this.initialize();
            }
            this.addWorkItemSupport();			
			this.updateRefreshable();
			
			this._createGeneratedEwis();
        },
		
		//@Overridden from AbstractArtifactEditorSection
		setSectionDirty: function(dirty) {
			this.inherited('setSectionDirty', arguments);
			this.updateRefreshable();
		},
		
		updateRefreshable: function() {
            if (this._caseTableViewer && this._caseTableViewer.refreshAction) {
                this._caseTableViewer.refreshAction.setDisabled(this.isSectionDirty());
            }			
		},
		       
        _handleAddClick: function(){
            this._showItemsDialog();
        },
        
        _handleAddNewClick: function(){
            var args = {
                itemType: Constants.MODEL_TYPE_CASE,
                onOkCallback: dojo.hitch(this, "_handleAddNewItem")
            }
            this._newItemDialog = new NewItemPopup(args);
        },
        
        _handleRemoveClick: function(){
            var items = this._caseTableViewer.getSelectedItems();
            this._removeItems(items);
            if (items && items.length != 0) {
                this.setDirty();
            }
        },
		
		_handleBatchUpdateClick: function(columnIndex, actionId, categoryTypes) {
			var args = {
				testcases: this._caseTableViewer.getSelectedItems(),
				categoryTypes: categoryTypes, 
				actionId: actionId,
                onOkCallback: dojo.hitch(this, "_handleBatchUpdateItem")
            }
			this._updatedDialog = new com.ibm.rqm.planning.web.ui.internal.view.common.properties.TestPlanTestCaseConfigSelector();
			this._updatedDialog.initialize(args);
		},
        
        _handleNewItemLink: function(participantId){
			var self = this;
			var args = {
				title: Constants.getMessage("labelConfirmation"),
				message: Constants.getMessage("msgSaveRequiredToView", Constants.TYPE_PLAN, Constants.TYPE_CASE),
				yesCallback: function(){
					self._newItemToOpen = self._getItemIndexByParticipantId(participantId);
            		self.getParentEditor().onSave();
					return true;          
				},
				noCallback: function() {
					return;
				}
			};
			var dialog = new YesNoDialog(args);
	        dialog.setSecondaryTitle(args.title);
	        dialog.open();
        },
		
		_openNewlySavedItem: function() {
            //if a new item was clicked, display it
            if (this._newItemToOpen) {
                var savedItem = this._items[this._newItemToOpen];
                this._newItemToOpen = null;
                if (savedItem != null) {
                    window.location = Utility.getTestCaseURI(savedItem.id);
                }
            }			
		},
        
		_onExecuteActionClick: function(){
			var selectedItems = this._caseTableViewer.getSelectedItems();
			var selectedCase = selectedItems[0];
			if(this.isSectionDirty() || selectedCase.id == ""){
				var self = this;
				var args = {
					title: Constants.getMessage("labelConfirmation"),
					message: Constants.getMessage("msgSaveRequiredToRunTestCase", Constants.TYPE_PLAN),
					yesCallback: function(){
	            		self.getParentEditor().onSave();
						return;
					},
					noCallback: function() {
						return;
					}
				};
				var dialog = new YesNoDialog(args);
		        dialog.setSecondaryTitle(args.title);
		        dialog.open();
			}else
				this._caseTableViewer._onExecuteActionClick();
		},
		
        _handleGenerateEWIClick: function(){
            var selected = this._caseTableViewer.getSelectedItems();
            if (!selected || selected.length == 0) {
 				this._displayAlertDialog(Constants.getMessage("msgGenEWITestCaseWarning"))
                //this.getParentEditor().setHeaderStatus(Constants.OPERATION_WARNING, Constants.HEADER_STATUS_WARNING, Constants.getMessage("msgGenEWITestCaseWarning"));
                return;
            }
            //HACK until we get contributed actions enabled  
            var wizardClassName = "com.ibm.rqm.configuration.web.ui.internal.view.configuration.GenerateEWIWizard";
            jazz.core.loader.load_async(wizardClassName, dojo.hitch(this, "_showEWIWizard", wizardClassName, selected));
            //            this._showEWIWizard(selected);     
        },
        
        _showEWIWizard: function(wizardClassName, testCases) {
			if (dojo.isFunction(this.getArtifact) && this.getArtifact() && this.getArtifact().id && this.getParentEditor() && !this.getParentEditor().isDirty()) {
	            var self = this;
	            var args = {
	                title: Constants.getMessage("labelGenEWIs"),
	                parentController: self,
	                callback: {context: self, method: function(arguments){
						var jsClassName = "com.ibm.rqm.configuration.web.ui.internal.view.configuration.GenerateEWISaveContext";
						jazz.core.loader.load_async(jsClassName, dojo.hitch(this, function(jsClassName, args){
							var constructor = dojo.getObject(jsClassName);
							var saveContext = new constructor(args);
							this.ewiSaveContextGroup.push(saveContext);
							this.showTestCaseTable();
							this.setDirty();
						}, jsClassName, arguments));
					}},
	                testCases: testCases,
	                parentTestPlan: this.getArtifact()
	            }
	            var constructor = dojo.getObject(wizardClassName);
	            this._ewiWizard = new constructor(args);
			} else {
				var msg = Constants.getMessage("msgSaveRequiredToGenEWI", this.getArtifactType());
				this._displayAlertDialog(msg);
            }
        },
		
		_createGeneratedEwis: function(){
			if(this.ewiSaveContextGroup) {
				var ewiSaveContextGroup = this.ewiSaveContextGroup;
				var length = ewiSaveContextGroup.length;
				var counter = 0;
				if (length){
					for(var i = 0; i < length; i++){
						ewiSaveContextGroup[i].commit({
							callback: dojo.hitch(this, function(){
								counter++;
								if(counter == length) {
									this.ewiSaveContextGroup = [];
								}
							})
						});
					}
				} 
			}
		},
        
        _showItemsDialog: function(){
            var params = {
                title: Constants.getMessage("labelAddItems", this._itemType),
                exclusionList: this._getAddExclusionList(),
                showQueryFilters: true,
                itemType: this._itemType,
                onOk: dojo.hitch(this, "_handleAddedItems"),
                onAddNew: function(){
                    alert(Constants.getMessage("labelNotSupported")); //self._appendNewItem();
                }
            };
            var itemListWidget = new SelectItemTableDialog(params);
            itemListWidget.setExclusionList(this._getAddExclusionList());
            itemListWidget.show();
        },
        
        _getAddExclusionList: function(){
            var excludedItems = this.getItems(); //[].concat(this.getItems(), this._excludeFromAdd);
            var excludedItemIds = [];
            
            for (var i = 0; i < excludedItems.length; i++) {
                var excludedItem = excludedItems[i];
                //var itemId = excludedItem.versionableItemId != null ? excludedItem.versionableItemId : excludedItem.itemId;
                excludedItemIds.push(excludedItem.versionableItemId);
                excludedItemIds.push(excludedItem.itemId);
            }
            return excludedItemIds;
        },
        
        _handleAddNewItem: function(){
            var newItemParticipant = this._newItemDialog.getItemSaveParticipant();
            this.getSaveContext().addParticipant(newItemParticipant);
            this.getSaveContext().createReference(this.getParentEditor(), this._referencesAddedType, newItemParticipant);
            
            var newItem = this._newItemDialog.getItemForDisplay();
            newItem.participantId = newItemParticipant.getParticipantId();
            this._items.push(newItem);
            this.showTestCaseTable();
            this.setDirty();
        },
        
        _handleAddedItems: function(addedItems){
            for (var i = 0; i < addedItems.length; i++) {
                var item = addedItems[i];
                var itemId = item.versionableItemId ? item.versionableItemId : item.itemId;
                if (this._containsItemId(this._itemsRemoved, itemId)) {
                    this._itemsRemoved = this._removeItemId(this._itemsRemoved, itemId);
                    this.getSaveContext().unremoveReference(this.getParentEditor(), this._referencesRemovedType, itemId);
                    this._items.push(item);
                }
                else 
                    if (!this._contains(item)) {
                        this._items.push(item);
                        this._itemsAdded.push(itemId);
                    }
                    else {
                        alert(item.name + " is already part of the list!");
                    }
                //append the item to the list
                this._addItem(item);
            }
            this.showTestCaseTable();
            this.setDirty();
        },
		
		_handleBatchUpdateItem: function(updatesData) {
			var items = updatesData ? updatesData.testCases : [];
			for(var i = 0; i < (items ? items.length : 0); i++) {
				var item = items[i];
				this._updateItem(item, updatesData.weight, updatesData.state);
			}
            if (items && items.length != 0) {
				this.setDirty();
            }
//			this.showTestCaseTable();
			//only change table content, won't change checkboxs' state.
			this._caseTableViewer.tableViewer.content = this.getItems();
			this._caseTableViewer.tableViewer.refresh();
		},
        
        _addItem: function(item){
            //create a SaveParticipant for the item
            var versionableItemId = item.versionableItemId ? item.versionableItemId : null;
            var itemId = item.versionableItemId ? item.versionableItemId : item.itemId;
            var saveParticipant = this.getSaveContext().findParticipant(itemId);
            if (saveParticipant == null) {
                saveParticipant = new SaveParticipant();
                var itemType = Utility.getModelType(item);
                saveParticipant.setDirty(false);
                saveParticipant.getItemId = function(){
                    return itemId;
                };
                saveParticipant.getVersionableItemId = function(){
                    versionableItemId
                };
                saveParticipant.getItemType = function(){
                    return itemType;
                };
                this.getSaveContext().addParticipant(saveParticipant);
            }
            var parent = this.getParentEditor();
            // treat it as a new reference if the parent is a copy
            if (parent != null && parent.isCopy != null && parent.isCopy.call(parent)) {
                this.getSaveContext().createReference(parent, this._referencesAddedType, saveParticipant);
            }
            else 
                if (this._containsItemId(this._itemsAdded, itemId) && parent != null) {
                    this.getSaveContext().createReference(parent, this._referencesAddedType, saveParticipant);
                }
        },
        
        _removeItems: function(items){
            if (!items || items.length == 0) {
                //                alert("Please select one or more " + this._itemType + "s to remove.");
                var msg = Constants.getMessage("msgDeleteTypeRequired", this._itemType);
                this.getParentEditor().setHeaderStatus(Constants.OPERATION_WARNING, Constants.HEADER_STATUS_WARNING, msg);
                return;
            }
            var confirmMsg = "";
            if (items.length > 1) {
                confirmMsg = Constants.getMessage("msgRemoveItem2", items.length, this._itemType);
            }
            else {
                confirmMsg = Constants.getMessage("msgRemoveItem", this._itemType, items[0].name);
            }
			var self = this;
			var	args = {
				message: confirmMsg,
				yesCallback: function(){
					for (var i = 0; i < (items ? items.length : 0); i++) {
                		self._removeItem(items[i]);
            		}
            		self.showTestCaseTable();
					self.setDirty();      
					return true;          
				},
				noCallback: function() {
					return;
				}
			}
			var dialog = new com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog(args);
        	dialog.setSecondaryTitle(Constants.getMessage("labelConfirmation"));
			dialog.open();
        },
        
        _removeItem: function(item){
            var updatedItems = [];
            var items = this.getItems();
            for (var i = 0; i < items.length; i++) {
                //if this item is not new && its itemId doesn't match the item to be removed's itemId
                if (items[i].itemId && items[i].itemId != item.itemId) {
                    updatedItems.push(items[i]);
                    //if this item is new and its participantId doesn't match the item to be removed's participantId
                }
                else 
                    if (items[i].newItem && items[i].participantId != item.participantId) {
                        updatedItems.push(items[i]);
                    }
                    else {
                        //handle item removal from save context   
                        var saveContext = this.getSaveContext();
                        var itemId = items[i].versionableItemId ? items[i].versionableItemId : items[i].itemId;
                        if (items[i].newItem) {
                            //handle new item CREATED during this saveContext session
                            saveContext.uncreateReference(this.getParentEditor(), this._referencesAddedType, items[i].participantId);
                            saveContext.removeParticipant(saveContext.findParticipant(items[i].participantId));
                        }
                        else 
                            if (this._containsItemId(this._itemsAdded, itemId)) {
                                //handle existing items ADDED during this saveContext session
                                this._itemsAdded = this._removeItemId(this._itemsAdded, itemId);
                                saveContext.uncreateReference(this.getParentEditor(), this._referencesAddedType, itemId);
								saveContext.removeParticipant(saveContext.findParticipant(itemId));
                            }
                            else {
                                //handle existing references, add them 'removed'
                                this._itemsRemoved.push(itemId);
                                saveContext.removeReference(this.getParentEditor(), this._referencesRemovedType, itemId);
								//handle batch updated participant
								saveContext.removeParticipant(saveContext.findParticipant(itemId));
                            }
                    }
            }
            this.setItems(updatedItems);
        },
        
		_updateItem: function(item, weight,state) {
			var categories = item.categories;
			if(item.newItem) {
				//if this item is new
				var participant = this.getSaveContext().findParticipant(item.participantId);
				var oldAttribute = participant.getAttributes();
				participant.getAttributes = function(){
            		var attributes = {
                		newItem: true,
                		name: oldAttribute.name,
                		description: oldAttribute.description,                               
                		state: state?state:oldAttribute.state,
						newTestCase: true,
						templateItemId: oldAttribute.templateItemId,
						templateStateId: oldAttribute.templateStateId,
						//changed
						categories: categories.join(),
						weight: weight ? weight : oldAttribute.weight
            		};          
					if (oldAttribute.workItemsJsonString) {
						attributes.workItemsJsonString = oldAttribute.workItemsJsonString;
					}
					return attributes;
            	}; 
			}
			else {
				var versionableItemId = item.versionableItemId ? item.versionableItemId : null;
            	var itemId = item.versionableItemId ? item.versionableItemId : item.itemId;
				var stateId = item.stateId;
				var itemType = Utility.getModelType(item);
				//existing item, then add saveParticipant
				var saveParticipant = this.getSaveContext().findParticipant(itemId);
				if(!saveParticipant) {
					//handle existing test case
					saveParticipant = new SaveParticipant();
					this.getSaveContext().addParticipant(saveParticipant);
				}
				saveParticipant.setDirty(true);
				saveParticipant.getItemId = function(){
                 	return itemId;
               	};
               	saveParticipant.getVersionableItemId = function(){
                	return versionableItemId;
               	};
               	saveParticipant.getItemType = function(){
                	return itemType;
               	};
				saveParticipant.getStateId = function(){
					return stateId;
				};
				saveParticipant.getAttributes = function(){
					var attributes = {
						itemId: itemId,
						//changed
						weight: weight ? weight : item.weight,
						state : state? state : item.state,
						categories: categories ? categories.join() : ""
					};
					return attributes;
				};
				saveParticipant.saveFinished = function() {
					saveParticipant.setDirty(false);
				};
			}
				
			//refresh items
			var oldItems = this.getItems();
			var updatedItems = [];
			for (var j = 0; j < oldItems.length; j++) {
				var tmpItem = oldItems[j];
				if ((tmpItem.newItem && tmpItem.participantId == item.participantId) ||
					(tmpItem.itemId && item.itemId && tmpItem.itemId == item.itemId)) {
						if(weight)
							tmpItem.weight = weight;
						if(state)
							tmpItem.state = state;
						tmpItem.categories = categories;
                   }
				updatedItems.push(tmpItem);
			}
			this.setItems(updatedItems);
		},
				
        _containsItemId: function(array, itemId){
            for (var i = 0; i < array.length; i++) {
                if (array[i] == itemId) {
                    return true;
                }
            }
            return false;
        },
        
        _removeItemId: function(array, itemId){
            var updatedArray = [];
            for (var i = 0; i < array.length; i++) {
                if (array[i] != itemId) {
                    updatedArray.push(array[i]);
                }
            }
            return updatedArray;
        },
        
        _contains: function(item){
            var itemId = item.versionableItemId ? item.versionableItemId : item.itemId;
            return this._getItemById(itemId) != null;
        },
        
        _getItemById: function(id){
            for (var i = 0; i < this._items.length; i++) {
                var item = this._items[i];
                var itemId = item.versionableItemId ? item.versionableItemId : item.itemId;
                if (itemId == id) {
                    return this._items[i];
                }
            }
            return null;
        },
        
        _getItemIndexByParticipantId: function(participantId){
            for (var i = 0; i < this._items.length; i++) {
                if (this._items[i].participantId == participantId) {
					return i;
                }
            }
            return null;
        },
        
        _getItemByName: function(name){
            for (var i = 0; i < this._items.length; i++) {
                if (this._items[i].name == name) {
                    return this._items[i];
                }
            }
            return null;
        },
        
		_displayAlertDialog: function(message, handler)
		{
			var args = {
                'message': message
            };
			if(handler){
				args.okCallback = handler;
			}
			var alertDialog = new AlertDialog(args);
			
            alertDialog.setSecondaryTitle("");
            alertDialog.open();
		},
	
        validateStateTransition: function(initialState, action, endState, editable, callback){
            //The only transiton we are interested in is to approved
            if (action == Constants.STATETRANSITION_ACTION_APPROVE && EditorUtil.applyTransactionConstraint(Constants.STATETRANSITION_PREFERENCE_ALLCHILDCASESAPPROVED)) {
                // We have not been initialized yet. Need to fetch the test cases
                // to do the validtion.
                if (!this._initialized && callback) {
                    this.validateCB = callback;
                    this.validateEndState = endState;
                    var fetched = this.refreshTestCases("_validateFetchSuccess", "_validateFetchError");
                    
                    // not a valid artifact, just return null
                    if(fetched == false) {
                        return null;
                    }
                    // return a delay flag.
                    return Constants.EDITORSECTION_STATE_VALIDATION_FETCHING;
                }
                // we have loaded the data, just do the validation.    
                return this._validateTestCasesApproval(endState);
            }
            //If we got here than nothing offensive happened so return null which indicates that it is ok to transition
            return null;
            
        },

        _validateFetchSuccess: function(testcases){
            this._items = testcases;
            var isValid = this._validateTestCasesApproval(this.validateEndState);
            
            // return the validation result and cleanup
            this.validateCB(isValid);
            delete this._items;
            delete this.validateCB;
            delete this.validateEndState;
        },
        
        _validateFetchError: function(errObj){
            var msg = (errObj && errObj.message) ? errObj.essage : null;
            this.validateCB(msg);
            delete this.validateCB;
            delete this.validateEndState;
        },
                		
        _validateTestCasesApproval: function(endState) {
            //Now loop over all the items (test cases) if any to ensure that they are all approved
            if (this._items && this._items.length > 0) {
                for (var i = 0; i < this._items.length; i++) {
                    if (this._items[i].state != Constants.STATETRANSITION_STATE_APPROVED) //Not approved
                    {
                        return Constants.getMessage("msgCannotTransitionNotAllTestCasesApproved", EditorUtil.getStateNameFromID(endState));
                    }
                }
            }
            return null;
        },
        
        _error: function(error) {
            console.error(this.declaredClass + ": Error occurred: " + error.message);
        }		
        
    })
    
})();
