/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading: com.ibm.rqm.datapool.web.ui.internal.view.datapool.DatapoolEditor");
dojo.provide("com.ibm.rqm.datapool.web.ui.internal.view.datapool.DatapoolEditor");

dojo.require("dojox.data.dom");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

// Require local widgets
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.util.GlobalEvents");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.EditorSection");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.datapool.web.client.internal.datapool.DatapoolClient");

dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.AbstractArtifactEditor");
dojo.require("com.ibm.team.process.web.client.projectarea");

(function(){ // module container
    /* QM objects */
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var Utility = com.ibm.rqm.planning.web.ui.internal.util.Utility;
    var AbstractArtifactEditor = com.ibm.rqm.planning.web.ui.internal.view.construction.AbstractArtifactEditor;
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	var EditorSection = com.ibm.asq.common.web.ui.internal.view.common.EditorSection;
    var GlobalEvents = com.ibm.asq.common.web.ui.internal.util.GlobalEvents;
    var ViewHeaderLayout = com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout;
	var YesNoDialog = com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog;
	var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
	var DatapoolClient = com.ibm.rqm.datapool.web.client.internal.datapool.DatapoolClient;
	var DATAFILESECTIONID = "com.ibm.rqm.datapool.editor.section.csvAttachment";
    var CommonUtility = com.ibm.asq.common.web.ui.internal.util.Utility;
	
    dojo.declare("com.ibm.rqm.datapool.web.ui.internal.view.datapool.DatapoolEditor", [dijit._Widget, dijit._Templated, AbstractArtifactEditor], {
        templateString: "<div dojoAttachPoint='_contentNode' class='com-ibm-rqm-editor'></div>",
        
        // object initializer
        constructor: function(args){
            this.initialized = false;
        },
        
        // populate the template with data, connect events, and show initial
        // sub-tab
        postCreate: function(){
            this.inherited('postCreate', arguments);
            this.setArtifactType(Constants.TYPE_DATAPOOL);
        },
        
        initialize: function(args){
            //check for re-initialization hack; needs to be cleaned up
            if (this.getArtifact() && this.parentController) {
                args.datapool = this.getArtifact();
                args.parentController = this.parentController;
            }
            
            if (args != null) {
                if (args.parentController === null) {
                    throw new Error("DatapoolEditor#initialize: Missing required parameter 'args.parentController'");
                }
                
                if (args.datapool != null) {
                    if (args.isCopy != null && args.isCopy == true) {
                        this.setCopy(true);
                    }
                    else {
                        this.setNew(false);
                    }
                    this.setArtifact(args.datapool);
                }
                
                this.parentController = args.parentController;
                this.setArtifact(args.datapool);
                this.initialized = true;
                
            }
            else 
                if (!this.initialized) {
                    throw new Error("DatapoolEditor#initialize: Missing required parameter 'args'");
                    return;
                }
            
            //create the layout                
            this.createLayout();
            if (this.isNew() && !this.isCopy()) {
                this._defaultName = this.getHeader().getTitle();
                this.setLastSaved(Constants.STATUS_NEW);
            }
            this.getHeader().setSummaryEditable(true);
            this.getHeader().setSummaryLabel(Constants.getMessage("labeldescription"));
			this.getHeader().onSummaryEdited = dojo.hitch(this, this._setEditorDirty);
						
			dojox.data.dom.removeChildren(this.getHeader().getInfoAttributesNode());
            if (this.getArtifact() != null) {
                if (this.getArtifact().description != null) {
                    this.getHeader().setSummary(this.getArtifact().description);
                }
                else {
                    this.getHeader().setSummary("");
                }
            }
            else {
                this.setNew(true);
                this.getHeader().setSummary("");
            }
            
            this.getHeader().setCSHelpId("rqm_datapool_overview.html");
			this.getHeader().setCSHelpButtonLabel(Constants.getMessage("labelDatapoolCSHelp"));
            this.updateHeader();
            if (this.isNew() || this.isCopy()) {
                if (this.getHeader()) {
                    this.getHeader().setEditableTitleFocus();
                }
            }
            
            if(args.datapool == null){
            	this.setDirty(false);
            }
        },
        
        createLayout: function(){
            // clean up any widgets before we reload.
            if(this._sectionsNode) {
                CommonUtility.destroyWidgetsFromNode(this._sectionsNode);                
            }
            //content and alternate content nodes for view header layout
            this._sectionsNode = document.createElement("div");
            
            //create the common header layout
            this.createHeader("datapool-icon-48", this._sectionsNode);
            dojox.data.dom.removeChildren(this._contentNode);
            this._contentNode.appendChild(this.getHeader().domNode);
            
            //load sections                      
            this.loadSections(this._sectionsNode);
        },
		
		// @override for AbstractArtifactEditor
		onCancel: function() {
			var self = this;
			var artifact = this.getArtifact();
			self.deteleFile = dojo.hitch(this, "_deleteCurrentFile");
			var args = {
				title: Constants.getMessage("labelConfirmation"),
				message: Constants.getMessage("msgConfirmCancel"),
				yesCallback: function(){   
					//delete file
					self.deteleFile();
					
				    //reset to default template (if templated)
                    self.setTemplateId(null);
                    self.setTemplate(null);	
					//reset savecontext
					if (self.isEditable()) {
						self.getArtifactModel().discardChanges();
						self.getArtifactModel().getSaveContext().addParticipant(self);
                        self.setDirty(false);
					}	
					//re-initialize	         	                  
					self.initialize({
                    	isCopy: self.isCopy(),
						artifact: artifact
					});
					return true;
				},
				noCallback: function() {
					return false;
				}
			}
			var dialog = new YesNoDialog(args);
	        dialog.setSecondaryTitle(args.title);
	        dialog.open();
		},
		
		/*
		 * when user close current tab, this function will be invoked 
		 */
		rollBack: function() {
			this._deleteCurrentFile();
		},
		
		_deleteCurrentFile: function() {	
			var initUUID = this.getArtifact() ? this.getArtifact().dataAttachment.uuid : null;
			var sections = this.getSections(); 
			var currentUUID = null;
			for(var i = 0; i < (sections ? sections.length:0); i++) {
				if(sections[i].getConfig().id == DATAFILESECTIONID){
					var attrs = sections[i]._attView.getAllAttachments();
					if(attrs && attrs.length > 0 && attrs[0].uuid)
						currentUUID = attrs[0].uuid;
					break;
				}
			}
			if(currentUUID != initUUID) {
				//delete current attachment
				var handler = {
					success: function(){
						console.debug("Remove attachment success");
					},
					failure: function(e){
						console.error("Unable to remove attachment");
					}
				}
				var responseHandler = new ServiceResponseHandler(handler, "success", "failure");
            	DatapoolClient.deleteDataFile(responseHandler, {dataItemUUID: currentUUID});
			}
		},
		
		// @override for AbstractArtifactEditor
		_displaySections: function() {                                                        			   
            //create and append the section widgets to the editor           
            var sectionDefinitions = this._getDisplayable();

            // Ensure that we clean up any previously created sections widgets
            // before we unload them and recreate the sections.
            this.cleanUpSectionsBeforeReload();
            
			dojox.data.dom.removeChildren(this._sectionsNode);     
            this._sections = [];
			if(sectionDefinitions && dojo.isArray(sectionDefinitions) && sectionDefinitions.length > 0 ){
				for (var i = sectionDefinitions.length - 1; i > -1; i--) {
	                this.appendSection(sectionDefinitions[i], null);
	            }
			}       
		},
		
		// @override for AbstractArtifactEditor
		_createSectionWidget: function(/* AbstractArtifactEditorSection */ sectionProvider){
            var containerArgs = {
                title: sectionProvider.getTitle(),
				sectionProvider: sectionProvider,
				collapsible: false
            };
            return new com.ibm.rqm.datapool.web.ui.internal.view.datapool.DatapoolEditor.EditorSection(containerArgs);    
        },
        
        _saveError: function(status){
            status.caughtAt = "DatapoolEditor#_saveError";
            this.setLastSaved(Constants.STATUS_FAILURE, status);
        },
        
        // @override for SaveParticipant
        validate: function(){
            //validate fields
            var name = this.getHeader().getTitle();
            if (dojo.trim(name).length == 0) {
                if (this.isNew()) {
					this.setHeaderStatus(Constants.VALIDATION_ERROR, Constants.HEADER_STATUS_ERROR,
						Constants.getMessage("msgRequiredField", Constants.getMessage("labelName"), Constants.getMessage("labelDatapool")));
					return false;
                }
                else {
                    this.getHeader().setTitle(this.getArtifact().name);
                }
            }
            else 
                if (this._defaultName && name == this._defaultName) {
					this.setHeaderStatus(Constants.VALIDATION_ERROR, Constants.HEADER_STATUS_ERROR,
						Constants.getMessage("msgRequiredField", Constants.getMessage("labelName"), Constants.getMessage("labelDatapool")));
                    return false;
                }
            return true;
        },
        
        // @override for SaveParticipant
        getAttributes: function(){
            var datapoolObj = this.inherited('getAttributes', arguments);
            datapoolObj.externalId = this.isNew() ? null : this.getArtifact().id;
            datapoolObj.name = this.getHeader().getTitle();
            datapoolObj.description = this.getHeader().getSummary();
			datapoolObj.processArea = getCurrentProjectArea().itemId;
            return datapoolObj;
        },
        
        // @override for SaveParticipant
        getItemType: function(){
            return Constants.MODEL_TYPE_DATAPOOL;
        },
        
        // @override for SaveParticipant
        saveFinished: function(datapool){
            var wasNew = this.isNew();
            
            this.setArtifact(datapool);
            this.setNew(false);
            this.setCopy(false);
            this.getHeader().setProgressEnd();
			this.getHeader().clearStatus();
            
            // this was a new artifact. Need to update its URI to a saved artifact one.
            // This allows any new navigation to this artifact to reuse the same editor page. 
            if (wasNew) {
                this._publishArtifactUriChanged();
            }
            if(!wasNew){
            	this._publishArtifactUpdated();
            }
			
			this.setSaving(false);
			this.setDirty(false);
			this.setLastSaved(Constants.STATUS_SUCCESS);
			
			// notify the attached sections
			var allsections = this.getSections();
			for(var i = 0; i < (allsections ? allsections.length:0); i++) {
				allsections[i].saveFinished();
			}
        },
        
        getOwnerItemId: function(){
            return null;
        },
        
        // handler on artifacts changed
        onArtifactChanged: function(changeType, artifactType, artifactArray) {
            // handle test plans deleted
            if ((changeType == GlobalEvents.ArtifactsDeleted) && (artifactType == Constants.TYPE_DATAPOOL)) {
                for (var i = 0; i < artifactArray.length; i++) {
                    if (this.getArtifact() && (this.getArtifact().itemId == artifactArray[i].itemId)) {
                        // it is in the deleted artifacts list, close itself
//                        this._publishSelfClosing();
                        // mark editor with warning of associated test case is changed
                        this.getHeader().setStatus(Constants.getMessage("msgEditorDatapoolDeleted"), ViewHeaderLayout.ERROR);
                        // done updating the editor if it is deleted
                        return;
                    }
                }
            }
        },
        
		_setEditorDirty: function(value){
			if(value == "")
				value = true;
			if(value != true && value != false){
				value = true;
			}
			if(!this.isDirty())
				this.setDirty(value);
		},
		
        _error: function(e){
            alert("An error occurred: [" + e.message + "] @ DatapoolEditor#_error");
        }
    });
	
	
	dojo.declare("com.ibm.rqm.datapool.web.ui.internal.view.datapool.DatapoolEditor.EditorSection", [EditorSection], {
		
		// @override for EditorSection
        _onTitleClick: function(){
			//
		}
    });
    
})();
