/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading: com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");

dojo.require("dojox.data.dom");
dojo.require("dijit._Widget");

dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");

dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.SectionWorkItem");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function(){ // module container
    var SectionWorkItem = com.ibm.rqm.planning.web.ui.internal.view.common.SectionWorkItem;
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var Utility = com.ibm.asq.common.web.ui.internal.util.Utility;
	
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection", null, {
        constructor: function(){
            this.parentEditor = null;
			this.config = null;
            this._section = null;
            this._sectionWI = null;
            this._sectionContainer = null;
            this._visible = true;
			this._dirty = false;
        },
        
        postCreate: function(){
            //ensure parentEditor was mixed in during construction
            if (!this.parentEditor) {
                throw new Error(this.declaredClass + ": argument 'parentEditor' REQUIRED!");
            }
            //ensure config was mixed in during construction
            if (!this.config) {
                throw new Error(this.declaredClass + ": argument 'config' REQUIRED!")
            }
            this.refreshSection();
        },
        
        getParentEditor: function(){
            return this.parentEditor;
        },	
        
        getSaveContext: function(){
            return this.getParentEditor().getSaveContext();
        },
        
        getArtifact: function(){
            return this.getParentEditor().getArtifact();
        },

        getArtifactType: function(){
            return this.getParentEditor().getArtifactType();
        },
                
        isPrintView: function() {
            return this.getParentEditor().isPrintView();
        },		
		
		isSnapshot: function() {
			return this.getParentEditor().isSnapshot();			
		},
        
        isRolePermitsEdit: function() {
            return this.getParentEditor().isRolePermitsEdit();    
        },
        
        setVisible: function(visible) {
           this._visible = visible; 
        },
        
        isVisible: function() {
            return this._visible;    
        },
		
		isSelected: function() {
			return (this.getParentEditor().getSelectedSection() == this.config.id);
		},
		
        getConfig: function(){
            return this.config;
        },		

        getSection: function() {
            return this._section;                
        },

        setSection: function(section) {
            this._section = section;                
        }, 
		
        setDirty: function(){
			//only set parent editor dirty
			if(!this.isSectionDirty()){
				console.debug(this.declaredClass + ": setDirty.");
				this.parentEditor.setDirty(true);
				this.setSectionDirty(true);
			}
        },
		
		isSectionDirty: function(){
			return this._dirty;
		},
		
		setSectionDirty: function(dirty){
			this._dirty = dirty;
		},
        
        refreshSection: function() {
            var persistedSection = this.parentEditor.getArtifactSection(this.getConfig().id);
            if (persistedSection) {
                this.setSection(persistedSection);
            } else {
                var description = this.getConfig().description;
                if (description == null || description.match(/^\s*$/)) {
                    description = Constants.getMessage("labelDefine", this.getConfig().name, this.getArtifactType());
                }
                //create a new section object for JSON 
                this.setSection({
                    id: this.getConfig().id,
                    name: this.getConfig().name,
                    description: description
                    //content: persistedSection ? persistedSection.content : ""
                });
            }        
        },
        
        getContainer: function() {
            return this._sectionContainer;                
        },

        setContainer: function(container) {
            this._sectionContainer = container;                
        },        
		
        //////////////////////////////////////////////////////////////////////////////////////
        //API for general section functionality, to be overridden where appropriate
        //////////////////////////////////////////////////////////////////////////////////////				     			
		//@Abstract: Override to add any non-default conditions where the implementing
		//section should be editable when the overall editor is not.  This will result in the
		//action buttons being enabled but will not trigger the save of the overall artifact.
        isEditable: function() {
            return this.getParentEditor().isEditable();
        },
        
		//@Abstract: Override to specify section actions
        getActions: function(){
            return null;
        },
        
        //@Abstract: Override to specify section actions
        getHelpContextID: function(){
            var section = this.getSection();
            if (section != null) {
				return section.id + ".html";
            }
            return null;
        },
        
        //@Abstract: Override to contribute attributes to the parent's save
        getAttributes: function(){
            return null;
        },
               
        //@Abstract: Override - called upon successful save of parent
        saveFinished: function(){
            this.refreshSection();
			this.setSectionDirty(false);
        },

        //Summary: provide section level validation; this is a hack since any section
        //requiring validation should be using the global save's validation mechanism 
        //@Deprecated: WILL BE REMOVED IN M7B - See RQM WI 2562
        //@Abstract: Override to provide section level validation
        validate: function(){
            return true;
        },
		
		//Summary: specifies whether or not any changes/actions taken by the user in this
		//section have necessitated the need for an electronic signture.  If any section
		//returns true, user will be prompted to re-authenticate upon attempt to save.
        //@Abstract: Override - called to determine if save requires electronic signature
        saveRequiresSignature: function(){
            return false;
        },		
		
        //////////////////////////////////////////////////////////////////////////////////////
        //API for section look & feel, should be overridden in most cases
        //////////////////////////////////////////////////////////////////////////////////////	
		//@Abstract: Override
		initialize: function(isEditable) {
			this.initialized = true;
		},

        //@Abstract: Override if desired; Default: show no Summary
        getTitle: /* String */ function() {
            if (this.getSection()) {
                return this.getSection().name;
            }
            return this.getConfig().name;
        },
        					
        getDisplayTitle: /* String */ function() {
            var title = this.getTitle();
            return Constants.getExtensionDisplayMessage(title);
        },
        
        /*
         * Default Info behavior is to provide the minimal WorkItem support
         */
		getInfoContent: /* String || DOM Node */ function(isEditable) {
            var workItemEditor = this.getWorkItemEditor();
			if (workItemEditor) {
				workItemEditor.setEditable(isEditable);
				return workItemEditor.domNode; //this.getWorkItemEditor().domNode; //"Some extra Info here."
			}			
			return null; //document.createTextNode("Section does NOT override getInfoContent!");
		},
		
		//@Abstract: Override if desired; Default: show no Summary
        getSummaryContent: /* String */ function(isEditable) {
            if (this.getConfig() && this.getConfig().description) {
                return this.getConfig().description;
                
            } else if (this.getSection() && this.getSection().description) {
                return this.getSection().description;
            }
            return null;
        },		
		
        getDisplaySummaryContent: /* String */ function() {
            var summary = this.getSummaryContent();
            if (summary) {
                return Constants.getExtensionDisplayMessage(summary);
            } else {
                return null;
            }
        },
        
		//@Abstract: Override if desired; Default: no Summary, so not editable
        isSummaryEditable: /* String */ function(isEditable) {
            return false;
        },			

        //@Abstract: Override if desired; Default: no Summary, so no change handler required
        getSummaryChangeHandler: /* Function */ function() {
            return null; //dojo.hitch(this, function(oldValue, newValue){alert("Section does NOT override getSummaryChangeHandler!")});
        },
				
        //@Abstract: Override if desired; Default: no header information			
        getHeaderContent: /* String || DOM Node */ function(isEditable) {
            if (!this.isEditable()) {
                return null;
            }
            var sectionId = this.getConfig().id;
            var anchor = document.createElement("a");
            this.connect(anchor, "onclick", dojo.hitch(this.getParentEditor(), "removeSection", sectionId));
            var image = document.createElement("div");            
            image.title = Constants.getMessage("labelRemoveSectionFrom", this.getArtifactType());
            dojo.addClass(image, "remove-link");
            dojo.addClass(image, "remove-disabled-icon-image");
            this.connect(image, "onmouseover", dojo.hitch(image, function() {
                dojo.removeClass(this, "remove-disabled-icon-image");
                dojo.addClass(this, "remove-enabled-icon-image");
            }));
            this.connect(image, "onmouseout", dojo.hitch(image, function() {
                dojo.removeClass(this, "remove-enabled-icon-image");
                dojo.addClass(this, "remove-disabled-icon-image");
            }));          
            anchor.appendChild(image);  
            return anchor; //"Add header here."; //"Section does NOT override getHeaderContent!";
        },
			
		/*
		 * Note: Main content will be lazily loaded, so this method will not be called until the
		 * section is expanded for the first time.
		 */
        getMainContent: /* DOM Node */ function(isEditable) {
            //lazily initialize main content
			if (!this.initialized) {
                this.initialize(isEditable);
                this.initialized = true;
            }
            return this.domNode;
        },	
		
		//@Abstract: Override if desired; Called when user expands Section; Default: no handler
        getExpandHandler: /* Function */ function() {
            return null; // dojo.hitch(this, function(){alert("Section does NOT override getExpandHandler!")});
        },	
		
		//@Abstract: Override if desired; Called when user expands Section; Default: no handler
        getCollapseHandler: /* Function */ function() {
            return null; // dojo.hitch(this, function(){alert("Section does NOT override getCollapseHandler!")});
        },		
		
        //@Abstract: Override to contribute section details to the parent artifact's sections
        getSectionInfo: function() {
            //allow any section to edit its name
            var section = this.getSection();
            if (section == null) {
                //HACK - this is to account for contributed sections that don't properly
                //initialize themselves during postCreate                    
                return null; //this.refreshSection();
            }
            var name = this.getContainer() ? this.getContainer().getTitle() : null;
            if (this.isEditable() && name != null && name != this.getConfig().name) {
                section.name = name;
            }
            //HACK - need an attribute in here since rest service expects one, but
            //the service will ignore the value unless it's 'richText'
            if (!section.attributes) {
                section.attributes = [];
                section.attributes.push({
                    type: "dummy",
                    name: "dummy",
                    value: "dummy"
                });    
            }
             
            this.setSection(section);
            return this.getSection();
        },				
				
        //////////////////////////////////////////////////////////////////////////////////////
        //API for Managing section's 'mini' WorkItem editor (Overriding discouraged)
        //////////////////////////////////////////////////////////////////////////////////////
        //  Added 'isEditable' param - implementors can override the default editability check
		//@DO NOT OVERRIDE
        addWorkItemSupport: function(defaultOwnerId, isEditable){
			
            //workitems not supported for custom sections            
            var id = this.getConfig().id;
//            if (this.getParentEditor().isCustomSection(id)) {
//                return;
//            }
            
            // destroy the old widget before building a new one.
            if(this._sectionWI) {
                var sectionWI = this._sectionWI;
                sectionWI.parentSection = null;
                sectionWI.artifact = null;
                sectionWI.workItem = null;
                if(sectionWI.destroy) {
                    sectionWI.destroy();
                }
            }
            
            //set up the workitem editor
			var artifact = this.getArtifact();
            this._sectionWI = new SectionWorkItem({
				parentSection: this,
                workItem: this._findWorkItemForSection(),
                artifact: this.getArtifact() ? this.getArtifact() : null,
                sectionId: this.config.id,
				editable: isEditable != null ? (isEditable === true) : this.isEditable(),
                defaultOwnerId: defaultOwnerId
            });
        },
        
        _findWorkItemForSection: function() {
            var artifact = this.getArtifact();
            var workItems = artifact && artifact.workItems ? artifact.workItems : []      
            for (var i = 0; i < workItems.length; i++) {
                if (workItems[i].section == this.config.id) {
                    return workItems[i];
                }
            }            
            return null;                    
        },        
		
        //@DO NOT OVERRIDE
        getWorkItemEditor: function(){
            return this._sectionWI ? this._sectionWI : null;
        },
        //@DO NOT OVERRIDE		      
        getWorkItem: function(){
            return this._sectionWI ? this._sectionWI.getWorkItem() : null;
        },
        
        //@Abstract: Override ONLY if default behavior is insufficient
        getWorkItemInfo: function(){
            return this._sectionWI ? this._sectionWI.getWorkItemInfo() : null;
        },
		
		//////////////////////////////////////////////////////////////////////////////////////
        //API for state transitions.  Override to add constraints on state transitions
        //////////////////////////////////////////////////////////////////////////////////////
		//@Abstract: override to validate a transition to a new state
		//return null if the transition is ok
		//return a USER CONSUMABLE error string if the transition is not permissable
        //return planning.Constants.EDITORSECTION_STATE_VALIDATION_FETCHING if the section needs
        //to fetch the data first to do the validation. Call the callback with the 
        //validation result (null/error message) when done.
		validateStateTransition:function(initialState, action, endState, callback)
		{
			return null;
		},
        
        // API to get notification when the section is being shown/activated in the parent editor.
        //@Abstract: override to do extra processing when the section is being shown from hidden state.
        onActivated: function()
        {
            // the section is being activated. Search its descendant widgets for
            // table viewers. If one is found refresh its content. The table viewer
            // could be updated when it's not shown by changes in other area. Those
            // updates would cause the table trying to recalculate its cell width, but
            // would get cell width of zero and thus clip all contents if it's in 
            // compact mode. When it's being show again, trigger a refresh on the table
            // would get the content to display properly.
            // 
            var containerWidget = this.getContainer();
            if(containerWidget && containerWidget instanceof dijit._Widget) {
                var descendants = containerWidget.getDescendants();
                dojo.forEach(descendants, function(widget) {
                    if(widget instanceof com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer) {
                        widget.refresh();
                    }
                });
            }
        },
        
        // Summary:
        //    Destroy all descendant widgets archored in node first and remove all contents
        //    in node. Subclasses must use this method to clear contents from a particular node
        //    to ensure all widgets are properly freed. 
        unloadContent: function(node)
        {
            Utility.destroyWidgetsFromNode(node);
            dojox.data.dom.removeChildren(node);                 
        }	
    })
    
})();
