/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading com.ibm.rqm.planning.web.ui.internal.view.extension.DynamicGridSection");
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.extension.DynamicGridSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.richtext.RichTextControl");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.grid.GridContainer");
(function(){
    var AbstractArtifactEditorSection = com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection;
    var RichTextControl = com.ibm.asq.common.web.ui.internal.widgets.richtext.RichTextControl;
	var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	var GridContainer = com.ibm.asq.common.web.ui.internal.widgets.grid.GridContainer;

    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.extension.DynamicGridSection", [dijit._Widget, dijit._Templated, AbstractArtifactEditorSection], {
        templateString: '<div dojoAttachPoint="_dynamicContainer">' +
                        '   <div style="text-align: right" dojoAttachPoint="_actionsNode"></div>' +
                        '   <div dojoAttachPoint="_gridNode" style="height: 30em; width: 100%;"></div>' +
						'</div>',
        
        constructor: function(){
			this._grid = null;	
			this._isInit = false;
        },
        
        postCreate: function(){	
            this.inherited('postCreate', arguments);
            var id = this.getConfig().id;
			this._data = dojo.fromJson(this.getSection().content);
            this.addWorkItemSupport();			
        },
        
		//will be lazily loaded by getMainContent
		//@Overridden from AbstractArtifactEditorSection
        initialize: function(isEditable) {
            //create the rich text control
			if (this._data) {
				var data = dojo.fromJson(this._data);
				//From the backend after the first save
				this.columnNames = data.columnNames ? data.columnNames : [];
			} else {
				//from the UI new section added but data is not saved yet.
				//This will also be the case if the section was declared in the template
				if (!this.getConfig().configurationData) {
					new Error("Cannot create Grid without Configuration data");
				}
				this.columnNames = this.getConfig().configurationData ? dojo.fromJson(this.getConfig().configurationData) : [];
			}
			setTimeout(dojo.hitch(this, "_initializeGridEditor", null), 100);
        },
		
		//@Overridden from AbstractArtifactEditorSection
        getExpandHandler: /* Function */ function() {
            return dojo.hitch(this, "showGrid");
        },
		
        _initializeGridEditor: function(){
            //Sample comparator to be used for sorting
            var data = this._data ? (dojo.fromJson(this._data).value) : [];
            var compare2ndLetter = function(inA, inB){
                // Sample Comparator for second alphabet
                // return <0, 0, >0
                return inA.charCodeAt(1) - inB.charCodeAt(1);
            }
            
            var model = new dojox.grid.data.Table([{}, {}, {}, {
                compare: compare2ndLetter
            }], data);
            
            // Layout
			var colNames = this.columnNames;
			var tableCells = [];
			var containerWidth = 700;
			var defaultStaticRow = [];
			var colWidth = 700; 
			
			var colLen = colNames ? colNames.length : 0;
			if (colLen > 0) {
				colWidth = ((1.0 / colLen) * containerWidth) + "px";
			} 
			
			for (var i=0; i < colLen; i++) {
				tableCells[i] = {name: colNames[i], width: colWidth, styles: 'text-align: left;'};
				defaultStaticRow.push("&nbsp;");//Need to add a space else the row height is improper
				if (this.isEditable()) {
					tableCells[i].editor = dojox.grid.editors.Input;
				}
			}
            var gridLayout = [{
                defaultCell: {
                    styles: 'text-align: left;'
                },
                rows: [tableCells]
            }];
			
			dojox.data.dom.removeChildren(this._actionsNode);
            dojox.data.dom.removeChildren(this._gridNode);
            if (!this._grid) {
                this._grid = new com.ibm.asq.common.web.ui.internal.widgets.grid.GridContainer({
					rowsPerPage: 20,
                    model: model,
                    structure: gridLayout,
                    rowCount: "10",
                    elasticView: "2",
                    defaultStaticRow: defaultStaticRow,
                    onchange: dojo.hitch(this, "setDirty"),
					singleClickEdit: true,
					actionsNode: this._actionsNode
                });
            }
			/*
			this._grid.viewsHeaderNode.style.maxWidth = "100%";
            this._grid.viewsNode.style.maxWidth = "100%";
			
			var gridViews = this._grid.views;
			var views = gridViews.views;
			var viewLen = views ? views.length : 0;
			for (var i = 0; i < viewLen; i++) {
				var view = views[i];
				view.domNode.style.maxHeight = "140px";
				view.contentNode.style.minHeight = "1px";
			}
			*/

            this._gridNode.appendChild(this._grid.domNode);
			this.showGrid();
        },

		
        showGrid: function() {
            if (this._grid) {
                this._grid.render();
                if (!this._isInit && this.isEditable()) {
                    this._grid.initialize();
                    this._isInit = true;
                }
            }
        },
               
        //@Overridden from AbstractArtifactEditorSection
        getHelpContextID: function(){
			// cs help id will be a custom attribute with id 'csHelpId'
			var config = this.getConfig();
            if (config && config.properties) {
				for (var index in config.properties) {
					if(config.properties[index].id == "csHelpId") {
						return config.properties[index].value + ".html";
					}
				}  
			}
			if(config.id.indexOf("dynamicGridSection") == 0 || 
				config.id.indexOf("com.ibm.rqm.planning.editor.section.dynamicGridSection") == 0) {
				// for new added section, no Help button
				//new added section's id = dynamicGridSection_****
				return null;
			}
			return this.inherited('getHelpContextID', arguments);
        },
        
        //returns the persistent section DTO
		//@Overridden from AbstractArtifactEditorSection
        getSectionInfo: function() {      
            this.inherited("getSectionInfo", arguments);
			var section = this.getSection();
			var config = this.getConfig();
			var grid = this._grid;
            if (this._grid || section.content) {
                //Grid is instnatiated, content may be present, update it.
				var strValue = grid ? dojo.toJson({columnNames: this.columnNames, value: grid.getAllRows()}) : section.content;
                section.attributes = [];
                section.attributes.push({
                    type: Constants.DYNAMICSECTION_GRID,
                    name: "content",
                    value: strValue
                });
				
            }else if (config.configurationData){
				//Section is from the template and not initialized.
				//First time if a user does not open the section and saves the artifact, 
				//we need to store the column info in the content.
				var columnNames = dojo.fromJson(config.configurationData);
				var strValue = dojo.toJson({columnNames: columnNames}) ;
                section.attributes = [];
                section.attributes.push({
                    type: Constants.DYNAMICSECTION_GRID,
                    name: "content",
                    value: strValue
                });
			}
			section.propertiesJsonString = dojo.toJson([{key: "type", value: "GRID", propertyType: "string"}]);
			return section;
        },
		
        //@Overridden from AbstractArtifactEditorSection: 
		validate: function () {
			return true;
		},
		
        //@Overridden from AbstractArtifactEditorSection
        saveFinished: function(savedItem){
            this.inherited('saveFinished', arguments);
            this.initialized = false;
            this.addWorkItemSupport();
        },
        
		//@Overriden from AbstractArtifactEditorSection
        isSummaryEditable: function(isEditable) {
            return isEditable;
        }, 	
		
		//@Overriden from AbstractArtifactEditorSection
        getSummaryChangeHandler: /* Function */ function() {
            return dojo.hitch(this, "_summaryChanged");
        },	
		
		_summaryChanged: function(oldValue, newValue) {
			this.getSection().description = newValue;
		},	
        
        _error: function(e){
            console.error("An error occurred: [" + e.message + "] @ DynamicGridSection#_error");
        }
    });
})();
