/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading com.ibm.rqm.planning.web.ui.internal.view.extension.DynamicSection");
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.extension.DynamicSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.richtext.RichTextControl");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function(){
    var AbstractArtifactEditorSection = com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection;
    var RichTextControl = com.ibm.asq.common.web.ui.internal.widgets.richtext.RichTextControl;
	var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	
    /**
     * Widget to display list of link elements in the LeftNavigation.
     */
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.extension.DynamicSection", [dijit._Widget, dijit._Templated, AbstractArtifactEditorSection], {
        templateString: '<div dojoAttachPoint="_dynamicContainer">' +
		                '    <div style="width: 99%" dojoAttachPoint="_richTextNode"></div>' +
						'</div>',
        
        constructor: function(){
			this._richText = null;	
        },
        
        postCreate: function(){	
            this.inherited('postCreate', arguments);
            var id = this.getConfig().id;
            this.addWorkItemSupport();			
			if (!this.isPrintView()) {
				this._richTextNode.style.overflow = "auto";
			}
        },
        
		//will be lazily loaded by getMainContent
		//@Overridden from AbstractArtifactEditorSection
        initialize: function(isEditable) {
            //create the rich text control
			dojox.data.dom.removeChildren(this._richTextNode);
			if (!this._richText) {
				this._richText = new RichTextControl({
                    editable: this.isEditable(),
					inEditMode: false,  //default to preview mode
					contentHTML: this.getSection().content ? this.getSection().content : "",
					save: dojo.hitch(this, function(){
						var editor = this.getParentEditor();
						if(dojo.isFunction(editor.onSave)){
							editor.onSave();
						}
					})
				});
				dojo.connect(this._richText, "onChange", this, "setDirty");
			} else {
				this._richText.setContentHTML(this.getSection().content);
			}			
			this._richTextNode.appendChild(this._richText.domNode);                 
        },
		
		//@Overridden from AbstractArtifactEditorSection
        getExpandHandler: /* Function */ function() {
            return dojo.hitch(this, "_showRichText");
        },
		
        _showRichText: function() {
            if (this._richText) {
                this._richText.show();
            }
        },
               
        //@Overridden from AbstractArtifactEditorSection
        getHelpContextID: function(){
			// cs help id will be a custom attribute with id 'csHelpId'
			var config = this.getConfig();
            if (config && config.properties) {
				for (var index in config.properties) {
					if(config.properties[index].id == "csHelpId") {
						return config.properties[index].value + ".html";
					}
				}  
			}
			if(config.id.indexOf("dynamicSection") == 0 || 
				config.id.indexOf("com.ibm.rqm.planning.editor.section.dynamicSection") == 0) {
				// for new added section, no Help button
				//new added section's id = dynamicSection_****
				return null;
			}
			return this.inherited('getHelpContextID', arguments);
        },
        
        //returns the persistent section DTO
		//@Overridden from AbstractArtifactEditorSection
        getSectionInfo: function() {      
            this.inherited("getSectionInfo", arguments);
            if (this._richText || this.getSection().content) {
                var strValue = this._richText ? this._richText.getContentHTML() : this.getSection().content;
                this.getSection().attributes = [];
                this.getSection().attributes.push({
                    type: "richtext",
                    name: "content",
                    value: strValue ? dojo.trim(strValue) : ""
                });
            }
			return this.getSection();
        },
		
        //@Overridden from AbstractArtifactEditorSection: 
		validate: function () {
			//Remove length limitation. Now model support large content, but HTTPRequest has Content-Length
			//limitation. Jetty has default max-length of 200000.
//			if (this._richText && this._richText.getContentHTML() != null) {
//                var content = this._richText.getContentHTML();
//				alert(content.length);
//				if(content && content.length > 32768) {
//					var msg = Constants.getMessage("msgInvalidField", "Content", "less than 32768 bytes", this.getTitle() + " Section");
//					this.getParentEditor().setHeaderStatus(Constants.VALIDATION_ERROR, Constants.HEADER_STATUS_ERROR, msg);
//					return false;
//				}
//            }
			return true;
		},
		
        //@Overridden from AbstractArtifactEditorSection
        saveFinished: function(savedItem){
            this.inherited('saveFinished', arguments);
            this.initialized = false;
            this.addWorkItemSupport();
        },
        
		//@Overriden from AbstractArtifactEditorSection
        isSummaryEditable: function(isEditable) {
            return isEditable;
        }, 	
		
		//@Overriden from AbstractArtifactEditorSection
        getSummaryChangeHandler: /* Function */ function() {
            return dojo.hitch(this, "_summaryChanged");
        },	
		
		_summaryChanged: function(oldValue, newValue) {
			this.getSection().description = newValue;
		},	
        reset:function(){
			if(this._richText){
				var richText = this._richText;
				
				richText._doEdit(null, true);
				
				richText._doPreview();
			}
		},
		
        _error: function(e){
            alert("An error occurred: [" + e.message + "] @ DynamicSection#_error");
        }
    });
})();
