/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.execution.web.ui.internal.view.extension.EWISection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Dialog");

dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");

dojo.require("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");
dojo.require("com.ibm.rqm.execution.web.ui.internal.view.execution.EWIList");
dojo.require("com.ibm.rqm.configuration.web.ui.internal.view.configuration.GenerateEWIWizard");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

(function(){ // module container
    var AbstractArtifactEditorSection = com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection;
    var GenerateEWIWizard = com.ibm.rqm.configuration.web.ui.internal.view.configuration.GenerateEWIWizard;
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	var AlertDialog = com.ibm.asq.common.web.ui.internal.widgets.AlertDialog;
    
    dojo.declare("com.ibm.rqm.execution.web.ui.internal.view.extension.EWISection", [dijit._Widget, dijit._Templated, AbstractArtifactEditorSection], {
        templateString: "<div dojoAttachPoint=\"_ewiSection\" class=\"com-ibm-rqm-execution-web-ui-internal-view-extension-EWISection\"></div>",
        
		_ewiSectionInitialized: false,
		
        constructor: function(args){
            this.dialogIndex = 0;
        },
        
        postCreate: function(){
            this.inherited('postCreate', arguments);
            this.addWorkItemSupport();			
        },
        
		//@Overriden from AbstractArtifactEditorSection
        initialize: function(isEsitable) {
            if(!this._ewiSectionInitialized){
				var self = this;
	            dojox.data.dom.removeChildren(this._ewiSection);
	            var listArgs = {
	                isEditorSection: true,
	                showTestCase: false,
					showArchived: this.getParentEditor().isSnapshot(),
					archiveEnabled: false,
	                editable: this.isEditable() && !this.isPrintView(),             
	                testCase: self.getArtifact(),
	                tableActions: this.getTableActions(),
					parentSection: this
	            }
	            this._listWidget = new com.ibm.rqm.execution.web.ui.internal.view.execution.EWIList(listArgs);
	            this._ewiSection.appendChild(this._listWidget.domNode);
				this._ewiSectionInitialized = true;
				
			} else{
				// was new.. now its saved.. so update the listwidget with the testcase
				this._listWidget.testCase = this.getArtifact();
				this._listWidget.initialize();
			}	
			
        },
		
        getTableActions: function(){
            var actions = [];
            var ewi_type = com.ibm.asq.common.web.client.internal.PermissionsClient.OP_CONFIG_EWI;
            var ewi_gen_perm = com.ibm.asq.common.web.client.internal.PermissionsClient.getStoredPermission(ewi_type);
           
            if (!ewi_gen_perm || ewi_gen_perm.create) {
				var action = {
                    actionId: "generate",
                    label: Constants.getMessage("labelGenEWIs"),
                    actionHandler: dojo.hitch(this, "_generateEWI"),
                    iconClass: "generate-ewis-icon",
                    enabledOnReadOnly: true
                }
				if(!this.canGenerateEWI()){
					action.disabled = true;
				}
				actions.push(action);
            }			
            return actions;
        },
        
        //@Overridden from AbstractArtifactEditorSection
        saveFinished: function(savedItem){
            this.inherited('saveFinished', arguments);
            this.addWorkItemSupport();
			if(this._ewiSectionInitialized){
				this._listWidget.createGeneratedEwis(dojo.hitch(this, "initialize"));
			}	
        },
        
        update: function(){
            this._listWidget.initialize();
			this.setDirty(true);
        },
		
		_getTestCase: function() {
			var testCase = dojo.clone(this.getArtifact());
			var snapInfo = testCase.snapshotInfo;
			if (snapInfo != null) {
				testCase.name = testCase.name + Constants.getMessage("labelRevisionInfo", snapInfo.revision, snapInfo.name);
			}	
			return testCase;		
		},
        
        _generateEWI: function(){
			
            if (dojo.isFunction(this.getArtifact) && this.getArtifact() && this.getArtifact().id) {
                var self = this;
                var args = {
                    title: Constants.getMessage("labelGenEWIs"),
                    parentController: self.parentController,
                    callback: {
                        context: self,
                        method: dojo.hitch(self, "_handleGeneratedEWIs")
                    },
                    testCases: [self._getTestCase()]
                }
                this._ewiWizard = new GenerateEWIWizard(args);
            } else {
                var msg = Constants.getMessage("msgSaveBeforeGenTER", this.getArtifactType());
				this._displayAlertDialog(msg);
            }
        },
		
		_handleGeneratedEWIs: function(generated) {	
			if (this.getParentEditor().isSnapshot()) {
				/*
				//confirm user wants to create new TERs
				var dialogArgs = {
					primaryTitle: Constants.getMessage("labelConfirmation"),
					isClosable: true,
					okCallback: dojo.hitch(this, "_createGeneratedEWIs"),
					cancelCallback: function() {} //cancelled; do nothing
				};	
				var dlg = new com.ibm.asq.common.web.ui.internal.widgets.ConfirmationDialog(dialogArgs);
		 		dlg.contentContainer.appendChild(document.createTextNode(generated.ewis.length + " new TERs will be saved immediately.  Continue?"));
				dlg.open();
				*/	
				this._createGeneratedEWIs(generated);	
			} else {
				//add new TERs to the list & mark the editor dirty	
				this._listWidget.addEwiSaveContext(generated);
				this._listWidget.addNewGeneratedEwis(generated.ewis);					
				this.setDirty(true);
			}	
		},
		
		_createGeneratedEWIs: function(generated) {			
			//add newly generated EWIs to the list
			this._listWidget.addEwiSaveContext(generated);
			this._listWidget.addNewGeneratedEwis(generated.ewis);	
			//explicitly create them outside of the global save 
			this._listWidget.createGeneratedEwis(dojo.hitch(this, function() {
				this._listWidget.onRefresh();
			}));				
		},
		
		_contains: function(item) {
            var itemId = item.versionableItemId ? item.versionableItemId : item.itemId;         
            return this._getItemById(itemId) != null;
        },
		
		 _getItemById: function(id) {
		 	this._items = [];
			if(this._listWidget && this._listWidget.ewis)
				this._items = this._listWidget.ewis;
				var _items = this._items;
				var _itemsLength = _items.length;
			for (var i = 0; i < _itemsLength; i++) {
                var item = _items[i];
                var itemId = item.versionableItemId ? item.versionableItemId : item.itemId;         
                if (itemId == id) {
                    return _items[i];
                }
            }
            return null;
        },
		
		_displayAlertDialog: function(message, handler){
			var args = {
                'message': message
            };
			if(handler){
				args.okCallback = handler;
			}
			var alertDialog = new AlertDialog(args);
			
            alertDialog.setSecondaryTitle("");
            alertDialog.open();
		},
		
		validate: function(){
			if(this._listWidget && this._listWidget._isBeingEdited()){
				this._displayAlertDialog(Constants.getMessage("msgSaveDiscardTERChanges"));
				this.getParentEditor().setSectionFocus("com.ibm.rqm.execution.editor.section.ewis");
				return false;
			}
			return true;
		},
		canGenerateEWI:function(){
			if (this.parentEditor && this.parentEditor.isSaveableState) {
				return this.parentEditor.isSaveableState();
			}
			return true;
		}
        
    });
    
})();
