/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.extension.QualityObjectivesSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.grid.GridContainer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.ObjectiveClient");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.QualityObjectivesGrid");

(function() {
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    var PermissionsClient = com.ibm.asq.common.web.client.internal.PermissionsClient;
	
	var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	var ObjectiveClient = com.ibm.rqm.planning.web.client.internal.ObjectiveClient;
	var AbstractArtifactEditorSection = com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection;
	
	var COMMAND_CREATE = "create"; //$NON-NLS$
    var COMMAND_UPDATE = "update"; //$NON-NLS$
    var COMMAND_REMOVE = "remove"; //$NON-NLS$
    
	var STATE_SUCCESS = 	"com.ibm.rqm.planning.objectives.success"; //$NON-NLS$
	var STATE_EXCEPTION = 	"com.ibm.rqm.planning.objectives.success_exception"; //$NON-NLS$
	var STATE_FAILURE = 	"com.ibm.rqm.planning.objectives.unsuccessful"; //$NON-NLS$
	var STATE_NOT_STARTED = "com.ibm.rqm.planning.objectives.not_started"; //$NON-NLS$
	var STATE_UNRESOLVED = 	"com.ibm.rqm.planning.objectives.unresolved"; //$NON-NLS$	

    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.extension.QualityObjectivesSection", [dijit._Widget, dijit._Templated, com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection], {
        templateString: '<div id="${id}">' +
						'   <div style="text-align: right" dojoAttachPoint="_actionsNode"></div>' +
						'   <div dojoAttachPoint="_gridNode" style="height: auto; width: auto;"></div>' +
						'</div>',
        
        postCreate: function(){
            this.inherited('postCreate', arguments);
            this._objStatuses = [];     
			this._evalEnabled = false;               
    
            //console.debug(this.getSection().content);
            //this._data = dojo.fromJson(this.getSection().content);
            this.addWorkItemSupport();
			this._states = [
                {id: STATE_SUCCESS, name: Constants.getMessage(STATE_SUCCESS)},
				{id: STATE_EXCEPTION, name: Constants.getMessage(STATE_EXCEPTION)},
				{id: STATE_FAILURE, name: Constants.getMessage(STATE_FAILURE)},
				{id: STATE_NOT_STARTED, name: Constants.getMessage(STATE_NOT_STARTED)},
				{id: STATE_UNRESOLVED, name: Constants.getMessage(STATE_UNRESOLVED)}
			];
        },
        
        //will be lazily loaded by getMainContent
        //@Overridden from AbstractArtifactEditorSection    
        initialize: function(isEditable){
            console.debug(this.declaredClass + ":initialize: entry.");
			this.inherited("initialize", arguments);
			var artifact = this.getArtifact();
			if (artifact != null) {
	            var args = {
	                planItemId: artifact.versionableItemId,
					groupId: this.getConfig().id
	            }
	            var respHandler = new ServiceResponseHandler(this, "_initialize", "_error");
	            ObjectiveClient.getObjectiveStatuses(respHandler, args);				
			} else {
				// the section has not yet ben open, try again in half a second
                // unless it is open
				setTimeout(dojo.hitch(this, "_initialize", null), 300);	
			}
            console.debug(this.declaredClass + ":initialize: exit.");
        },
		
		_initialize: function(statuses) {
			this._objStatuses = statuses || [];
			if (this._grid) {
				this._grid = null;
			}            
			this._initializeGrid();
            this._showGrid();		
			this._evalEnabled = this._hasEvalObjectives();
			this._updateActionStates();
		},
        
        _initializeGrid: function(){
            var dataAsGridModel = this._toGridModel(this._objStatuses);
            
            var model = new dojox.grid.data.Table([{}, {}, {}, {}, {}], dataAsGridModel);
            
            // Layout
            var objectiveCell = {
                name: Constants.getMessage("labelObjective"),
                width: '200px'
            };
            var expectedCell = {
                name: Constants.getMessage("labelExpected"),
                width: '75px',
                styles: 'text-align: center;'
            };          
            var actualCell = {
                name: Constants.getMessage("labelActual"),
                width: '75px',
                styles: 'text-align: center;'
            };
            var stateCell = {
                name: Constants.getMessage("labelStatus"),
                width: '100px',
                styles: 'text-align: center;'
            };
            var commentCell = {
                name: Constants.getMessage("labelComment"),
                width: '200px',
                styles: 'text-align: left;'
            };          
            if (this.isEditable()) {
                //idCell.editor =
                actualCell.editor = commentCell.editor = dojox.grid.editors.Input;
                stateCell.editor = dojox.grid.editors.Select;
                stateCell.options = this._getStateNames();
            }
            var gridLayout = [{
                defaultCell: {
                    styles: 'text-align: left;'
                },
                rows: [[objectiveCell, expectedCell, actualCell, stateCell, commentCell]]
            }];
            
			dojox.data.dom.removeChildren(this._actionsNode);
			dojox.data.dom.removeChildren(this._gridNode);
            if (!this._grid) {
                this._grid = new com.ibm.asq.common.web.ui.internal.widgets.grid.GridContainer({
                    rowsPerPage: 5,
                    model: model,
                    structure: gridLayout,
                    rowCount: 10,
                    elasticView: "2",
                    defaultStaticRow: ["", "", "", this._getStateName(STATE_NOT_STARTED), ""],
                    onchange: dojo.hitch(this, "_onGridChange"),
					onAdd: dojo.hitch(this, "_showSelectExisting"),
					onRemoveRows: dojo.hitch(this, "_onRemoveRows"),
                    singleClickEdit: false,
					actionsNode: this._actionsNode
                });
				this._addGridActions();
				if (this.isEditable()) {
					this._grid.initialize();
				}
				this._updateActionStates();
				this.connect(this._grid.selection, "onSelected", "_onGridSelection");
            }
            
            this._gridNode.appendChild(this._grid.domNode);
        },
		
		_addGridActions: function() {
		    if (this._grid) {
				//add the evaluate action
				var evaluateAction = {
					id: "evaluate", //$NON-NLS$
					label: Constants.getMessage("labelEvaluateObjectives"), //$NON-NLS$
					callback: dojo.hitch(this, "_doEvaluate"),
					iconClass: "evaluate-icon-image", //$NON-NLS$
					disabled: false
				}				
				this._gridEvalButton = this._grid.addAction(evaluateAction);
				
				//add the 'manage' action if the section is editable
				if (this.isEditable()) {
					var permission = PermissionsClient.getStoredPermission(PermissionsClient.OP_TEST_OBJECTIVE);
					if (permission && (permission.create || permission.edit)) {
						var manageAction = {
							id: "manage", //$NON-NLS$
							label: Constants.getMessage("labelManageObjectives"), //$NON-NLS$
							callback: dojo.hitch(this, function(e){
								var objectivesView = new com.ibm.rqm.planning.web.ui.internal.view.construction.QualityObjectivesGrid({
									isDialog: true,
									editable: true
								});
							}),
							iconClass: "objectives-icon-image",
							disabled: false
						}
						this._grid.addAction(manageAction);
					}
				}
			}
		},
		
		_showSelectExisting: function() {
            var objectivesView = new com.ibm.rqm.planning.web.ui.internal.view.construction.QualityObjectivesGrid({
                isDialog: true,
				dialogTitle: Constants.getMessage("labelSelectObjectives"),
                editable: false,
                excluded: this._getExcludedFromAdd(),
                okCallback: dojo.hitch(this, "_doAddExisting")
            });			
		},
		
		_doAddExisting: function(selected) {
			var newStatuses = [];
			for (var i = 0; i < (selected || []).length; i++) {
				//o.name, (o.operator + " " + o.targetValue), s.actualValue, s.status, s.comment, o.itemId, o.stateId
				var newObjStatus = {
					actualValue: 0,
				    state: STATE_NOT_STARTED, 
					comment: "",
					objective: selected[i]
				};
				newStatuses.push(newObjStatus);
				//this._objStatuses.push(newObjStatus);
			}
			//add the new status objects to the grid
			if (newStatuses.length > 0) {
				this._grid.appendRows(this._toGridModel(newStatuses));
				this._evalEnabled = this._hasEvalObjectives();
				this._onGridChange();
			}
		},
		
		_doEvaluate: function() {
			console.debug(this.declaredClass + ": _doEvaluate!");
			var artifact = this.getArtifact();
			if (artifact != null) {
				var respHandler = new ServiceResponseHandler(this, "_handleEvalUpdates", "_error");
				var statuses = this._fromGridModel(false);
				if (statuses && statuses.length > 0) {
					ObjectiveClient.evaluateObjectives(respHandler, artifact.versionableItemId, statuses);
				}
			}
		},
		
		_handleEvalUpdates: function(statuses) {			
			statuses = statuses || [];
			var statusLen = statuses.length;
			var gridRows = this._fromGridModel(false);
			var grid = this._grid;
			var changesMade = false;
			for (var i = 0; i < statusLen; i++) {
				//update the grid rows with new evaluations, if they've changed
				var updated = statuses[i];
				var gridIndex = this._findGridRowIndex(gridRows, updated);
				var original = gridRows[gridIndex];
				var newActual = updated.actualValue;
				if (original.actualValue != newActual) {
					changesMade = true;
					grid.doApplyCellEdit(newActual, gridIndex, 2);
				}
				var newState = updated.state;
				if (original.state != newState) {
					changesMade = true;
					grid.doApplyCellEdit(this._getStateName(newState), gridIndex, 3);
				}
				if (changesMade) {
					grid.doApplyEdit(gridIndex);
					this._onGridChange();
				} else {
					//show info dialog indicating everything is up-to-date
					var args = {
						primaryTitle: Constants.getMessage("labelEvaluateObjectives"),
		                message: Constants.getMessage("msgEvaluateNoChanges")
		            };
					var alertDialog = new com.ibm.asq.common.web.ui.internal.widgets.AlertDialog(args);			
		            alertDialog.setSecondaryTitle("");
		            alertDialog.open();
				}
			}			
		},
		
		_findGridRowIndex: function(statuses, status) {
			var statusLen = (statuses || []).length;
			for (var i = 0; i < statusLen; i++) {
				if (statuses[i].groupOrder == status.groupOrder) return i;
			}						
			return -1;
		},
		
		_hasEvalObjectives: function() {
			//determine whether or not this section contains any objectives that can
			//be automatically evaluated by the system
			var statuses = this._fromGridModel();
			var objLen = (statuses || []).length;
			for (var i = 0; i < objLen; i++) {
				var evaluatorId = statuses[i].evaluatorId;
				if (evaluatorId != null && dojo.trim(evaluatorId).length > 0) {
					return true;
				}
			}
			return false;
		},
		
		_onGridSelection: function() {
			this._updateActionStates();
		},
		
		_onRemoveRows: function() {
			this._evalEnabled = this._hasEvalObjectives();
			this._onGridChange();
		},
		
        _onGridChange: function() {
            //TODO add edit validation
			this.setDirty();
			this._updateActionStates();
        },		
		
		_updateActionStates: function() {
			if (this._grid && this._gridEvalButton) {
				this._gridEvalButton.setDisabled(!this._evalEnabled);
			}			
		},
		
        //@Overridden from AbstractArtifactEditorSection
        saveFinished: function(savedItem){
            this.inherited('saveFinished', arguments);
            this.addWorkItemSupport();
			//remove temp commands on the editor
			if (this.getParentEditor()._tempQualObjCommands) {
				delete this.getParentEditor()._tempQualObjCommands;
			}	
			if (this.isSelected()) {
				this.initialize();
			} else {
				this.initialized = false;
			}
        },
		
		//Override
		validate: function() {
            if (this.initialized) {
				var updates = this._fromGridModel();
				for (var i = 0; i < (updates || []).length; i++) {
					var actualValue = updates[i].actualValue;
		            if (actualValue === "") {
						var msg = Constants.getMessage("msgRequiredGridField", Constants.getMessage("labelActual"), (i + 1));
		                this.getParentEditor().setHeaderStatus(Constants.VALIDATION_ERROR, Constants.HEADER_STATUS_ERROR, msg);
						return false;
					} else if (this._isNonNumeric(actualValue)) {
						var msg = Constants.getMessage("msgInvalidGridField", Constants.getMessage("labelActual"), Constants.getMessage("msgValidNumber"), (i + 1));
						this.getParentEditor().setHeaderStatus(Constants.VALIDATION_ERROR, Constants.HEADER_STATUS_ERROR, msg);
		                return false;
		            }
				}
			}
			return true;
		},
		
		_isNonNumeric: function(value) {
			var asNum = new Number(value);
			return isNaN(asNum);
		},		
		
        //@Overriden from AbstractArtifactEditorSection
        getAttributes: function(){
            if (this.initialized) {
				var commands = this.getParentEditor()._tempQualObjCommands || [];
				var updates = this._fromGridModel();
				//handle creates & updates 				
				for (var i = 0; i < (updates || []).length; i++) {
					var u = updates[i];
					commands.push({
						cmd: u.id != null ? COMMAND_UPDATE : COMMAND_CREATE,
						statusObj: u
					});
				}
				//handle removals
				var originals = this._objStatuses;
				for (var i = 0; i < (originals || []).length; i++) {
					var o = originals[i];
					if (!this._findIn(updates,o)) {
	                    commands.push({
	                        cmd: COMMAND_REMOVE,
	                        statusObj: {id: o.id}
	                    });		
					}
				}
				//temporarily store commands on the editor
				this.getParentEditor()._tempQualObjCommands = commands;
				
                return {
                    updateObjectivesJson: dojo.toJson(commands)
                }               
            }
            return null;
        },		
		
        //@Overriden from AbstractArtifactEditorSection
        isSummaryEditable: function(isEditable){
            return isEditable;
        },
        
        //@Overriden from AbstractArtifactEditorSection
        getSummaryChangeHandler: /* Function */ function(){
            return dojo.hitch(this, "_summaryChanged");
        },		
		
        _findIn: function(statuses, status) {
            for (var i = 0; i < (statuses || []).length; i++) {
                if (statuses[i].id != null && statuses[i].id == status.id) {
                    return statuses[i];
                }
            }
            return null;
        },
		
		_getStateNames: function() {
			var namesOnly = [];
			for (var i = 0; i < this._states.length; i++) {
				namesOnly.push(this._states[i].name);
			}
			return namesOnly;
		},

		_getStateName: function(id) {
			var state = this._getState("id", id); //$NON-NLS$
			return (state != null ? state.name : null);
		},
				
		_getStateId: function(name) {
			var state = this._getState("name", name);  //$NON-NLS$
			return (state != null ? state.id : null);
		},
				
		_getState: function(property, value) {
			for (var i = 0; i < this._states.length; i++) {
				if (this._states[i][property] == value) {
					return this._states[i];
				}
			}
			return null;
		},	
        
        _summaryChanged: function(oldValue, newValue){
            this.description = newValue;
        },
        
        _showGrid: function(){
            if (this._grid) {
                this._grid.render();
            }
        },
		
		_getExcludedFromAdd: function() {
            var statuses = this._objStatuses || [];
			statuses = statuses.concat(this._fromGridModel());
			var excluded = {};
            for (var i = 0; i < (statuses || []).length; i++) {
                var s = statuses[i];
                if (s.objective != null) {
					excluded[s.objective.itemId] = true;
				} else if (s.objectiveItemId != null) {
					excluded[s.objectiveItemId] = true;
				}
            }
            return excluded;			
		},
		
        _fromGridModel: function(selectedOnly) {
            var asModelled = [];
            if (this._grid) {
                //each row is an array of values
                var rows = selectedOnly === true ? this._grid.getSelectedRows() : this._grid.getAllRows();
                for (var i = 0; i < rows.length; i++) {
                    asModelled.push({
                        name: rows[i][0] || "",
                        expected: rows[i][1] || "",
                        actualValue: rows[i][2] >= 0 ? rows[i][2] : "",
                        state: this._getStateId(rows[i][3]) || "",
                        comment: rows[i][4] || null,
                        objectiveItemId: rows[i][5] || null,
						objectiveStateId: rows[i][6] || null,
						id: rows[i][7] != null ? rows[i][7] : null,
						evaluatorId: rows[i][8] != null ? rows[i][8] : null,
						group: this.getSection().id,
						groupOrder: i
                    });
                }
            }
            return asModelled;
        },
        
        _toGridModel: function(statuses) {
            var asGridModel = [];
            for (var i = 0; i < (statuses || []).length; i++) {
                var s = statuses[i];
				var o = s.objective;
				var target = o.targetValue != "" ? new String(o.targetValue) : "0";
                asGridModel.push([
                    o.name, (o.operator + " " + target), s.actualValue, this._getStateName(s.state), s.comment||"", o.itemId, o.stateId, s.id, o.evaluatorId
                ]);
            }
            return asGridModel;     
        },
		
        _error: function(e){
            console.error(this.declaredClass + ": An error occurred: \n[" + e.message + "]");
        }		
    });
})();
