/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.itemtemplates.TemplateEditor");

dojo.require("dojox.data.dom");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.SortableOptionChooser");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.template.TemplateClient");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.NewSectionDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");
dojo.require("com.ibm.asq.common.web.ui.internal.util.AutoWordWrapping");

(function(){
	var Utility = com.ibm.asq.common.web.ui.internal.util.Utility;
	var PermissionsClient = com.ibm.asq.common.web.client.internal.PermissionsClient;
	var EditorRegistry = null;
    var TemplateClient = com.ibm.rqm.planning.web.client.internal.template.TemplateClient;
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	var SortableOptionChooser = com.ibm.asq.common.web.ui.internal.widgets.SortableOptionChooser;
	var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	var NewSectionDialog = com.ibm.rqm.planning.web.ui.internal.view.construction.NewSectionDialog;
	var CSHelp = com.ibm.asq.common.web.ui.internal.util.CSHelp;
	var wrapText  = com.ibm.asq.common.web.ui.internal.util.wordWrapping.wrapText;
	
	var DIALOG_STYLE = "position: absolute; top: 15%; left: 15%; width: 650px; height: 550px; z-index: 999";
	
	dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.itemtemplates.TemplateEditor",
	[dijit._Widget, dijit._Templated],
	{
        //		
	    // Provides editing capability for artifact templates 
	    //
        templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/itemtemplates/TemplateEditor.html"),			    
	
        width: "600px",
		selectionSize: 15,
		
		constructor: function() {
			//optional params
			this.isCopy = false;
			this.isDialog = true;	
			this.dialogTitle = null;		
			this.onOK = null;
			this.editorId = null;
			//internal
			this.templateId = null;
            this._template = null;			
			this._allSections = null;
			this._setDefault = false;
			this._isAdvanced = false;			
			this._permissions = {};		
			
			EditorRegistry = dojo.getObject(Constants.EDITOR_REGISTRY_OBJECT_ID);
			
			this.EDITOR_MAP = {};
			this.EDITOR_MAP[Constants.EDITOR_ID_PLAN] = Constants.TYPE_PLAN;
			this.EDITOR_MAP[Constants.EDITOR_ID_CASE] = Constants.TYPE_CASE;
		},		
	    
	    postCreate: function() {
			//ensure there is either an existing template OR an editorId
			if (this.template) {
				this._template = this.template;
				this.editorId = this._template.editorId;
				delete this.template;				
			} else if (!this.editorId && !this.templateId) {
                throw new Error("One of 'editorId' or 'templateId' is REQUIRED!");
            }
			this._artifactType = this.EDITOR_MAP[this.editorId];		
			if (this.dialogTitle == null) {
				this.dialogTitle = Constants.getMessage("labelManageTemplate", this._artifactType);
			}	
			
            //if this is being shown as a dialog, init the dialog
            if (this.isDialog) {
                this.showAsDialog();
            }
						
            //show 'loading' message
            dojo.removeClass(this._loadingNode, "hidden");			
						
			//determine the current user's permissions
            var operations = [];
            operations.push({
                id: PermissionsClient.OP_TEMPLATE, 
                actions: PermissionsClient.DEFAULT_ACTIONS
            });
            operations.push({
                id: PermissionsClient.OP_TEMPLATE_SECTION, 
                actions: PermissionsClient.DEFAULT_ACTIONS
            });			
			
			var responseHandler = new ServiceResponseHandler(this, "_preinit", "_error");
            PermissionsClient.checkAllPermissions(responseHandler, operations);
			
	    },
		
		_preinit: function(permissions) {
			this._permissions = permissions;
            //initialize the editor 
            dojo.removeClass(this._loadingNode, "hidden");  //show 'loading' message
            if (this.templateId) {
                this._loadTemplate();
            } else {
                this.initialize();                  
            }
		},
		
        isNew: function() {
            return !(this._template && this._template.itemId);
        },

        isEditable: function() {
			var hasPermission = false;
			if (this.isNew()) {
				hasPermission = this.hasPermission(PermissionsClient.OP_TEMPLATE, PermissionsClient.ACTION_CREATE);
			} else {
				hasPermission = this.hasPermission(PermissionsClient.OP_TEMPLATE, PermissionsClient.ACTION_EDIT);
			}
            return (this.isNew() || (hasPermission && this._template && this._template.modifiable));
        },
							
		_loadTemplate: function() {
            //retrieve the template to be edited (using this.templateId)       
            var self = this;
            var handlerObj = {
                success: function(template) {
					self._template = dojo.isArray(template) ? template[0] : template;
					self.editorId = self._template.editorId;
					self._dialog.titleNode.innerHTML = self.getDialogTitle();
                    self.initialize();
                },
                
                failure: function(e) {
                    self._error(e);
                }
            }			
            var responseHandler = new ServiceResponseHandler(this, "success", "failure");
            TemplateClient.getTemplate(responseHandler, {editorId: this.editorId});			
		},

        initialize: function() {
            //hide 'loading' message
            dojo.addClass(this._loadingNode, "hidden");  
								
			//static labels
            this._idLabel.innerHTML = Constants.getMessage("labelId") + ": ";
            this._nameLabel.innerHTML = wrapText(
                        Constants.getMessage("labelName"), 
                        "", 
                        85, // hard code the max width since the dom node has not been added to the tree. 
                        true,
                        "<BR/>") + ": ";
			this._descriptionLabel.innerHTML = wrapText(
                        Constants.getMessage("labelDescription"), 
                        "", 
                        85, // hard code the max width since the dom node has not been added to the tree. 
                        true,
                        "<BR/>") + ": ";
            this._setDefaultLabel.innerHTML = Constants.getMessage("labelSetDefaultTemplate");
			this._setAdvancedLabel.innerHTML = Constants.getMessage("labelAdvancedOptions");
            
            //form input 
            if (this.isNew()) {
                this._id.value = this._generateTemplateId();
				if (this.isCopy) {
                    var templateName = Constants.getExtensionDisplayMessage(this._template.name);
					this._name.value = Constants.getMessage("labelCopyOf", templateName);
					this._description.value = Constants.getMessage("labelCopyOf", templateName);
				} else {
				    this._name.value = Constants.getMessage("labelNewTemplateName", this._artifactType);	
				}
                this._setDefaultCheckbox.checked = false;
            } else {
                var templateName = Constants.getExtensionDisplayMessage(this._template.name);
                var templateDesc = Constants.getExtensionDisplayMessage(this._template.description);
				this._toggleVisibility();
				this._id.value = this._template.id;
				this._idReadOnly.innerHTML = this._template.id;				
				this._name.value = templateName;
				this._nameReadOnly.innerHTML = templateName;				
				this._description.value = templateDesc || "";
				this._descriptionReadOnly.innerHTML = templateDesc || "";				
				this._setDefaultCheckbox.checked = false; //TODO - model isDefault property                            
            } 
			
			//retrieve all sections for this editorId			
			dojox.data.dom.removeChildren(this._sectionChoiceContainerNode);
			var self = this;
			var handlerObj = {
				success: function(sections) {
                    dojo.addClass(self._loadingNode, "hidden");
					self._allSections = self._aggregateSections(sections);
					self.showSectionChoice();
				},
				
				failure: function(e) {
					self._error(e);
				}
			}
            var responseHandler = new ServiceResponseHandler(handlerObj, "success", "failure");
            TemplateClient.getSections(responseHandler, {editorId: this.editorId});
        },
		
		_aggregateSections: function(persistedSections) {
			//aggregate the sections persisted in the repository with any that have been added
			//to the server's registry (via extensions) but have not yet been persisted
			var allSections = [].concat(persistedSections);
			var editor = EditorRegistry.getEditorById(this.editorId);
			var registrySections = editor && editor.sections ? editor.sections : [];
			var secLen = registrySections.length;
			for (var i = 0; i < secLen; i++) {
				var section = registrySections[i];
				if (!this._contains(persistedSections, section.id)) {
					//there is a section in the registry that has not yet been persisted
					//add it to the list and indicate that it is a registry section
					section.isNewItem = true;
					section.isRegistrySection = true;
					section.allowedEditors = (section.editorsAllowed || []).join(","); //mismatch in DTO & json post object
					allSections.push(section);
				}
			}
			return allSections;
		},
		
		_contains: function(sections, sectionId) {
			sections = sections || [];
			var secLen = sections.length;
			for (var i = 0; i < secLen; i++) {
				var s = sections[i];
				if (s.id == sectionId) {
					return true;
				}
			}
			return false;
		},
		
		_toggleVisibility: function() {			
			var editable = this.isEditable();						
            dojo.toggleClass(this._idReadOnly, "hidden", editable);
            dojo.toggleClass(this._nameReadOnly, "hidden", editable);
            dojo.toggleClass(this._descriptionReadOnly, "hidden", editable);

			dojo.toggleClass(this._id, "hidden", !editable);
			dojo.toggleClass(this._name, "hidden", !editable);
			dojo.toggleClass(this._description, "hidden", !editable);
			
			dojo.toggleClass(this._setDefaultContainer, "hidden", !editable);
		},
		
        showAsDialog: function() {
			if (!this._dialog) {			
	            var dialogArgs = {
					primaryTitle: this.getDialogTitle(),
	                okCallback: dojo.hitch(this, "_onOK"),
	                cancelCallback: dojo.hitch(this, "_close"),
					_truncate: function() {}  //disable the Dialog's '_truncate'
	            };
	            
	            this._dialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(dialogArgs);
	            if (this._artifactType == Constants.TYPE_CASE) {
		            var cshelp = new CSHelp({contextID: "rqm_manage_tc_template.html"});
		            this._dialog.setSecondaryTitleNode(cshelp.domNode);
				} else if (this._artifactType == Constants.TYPE_PLAN) {
		            var cshelp = new CSHelp({contextID: "rqm_manage_tp_template.html"});
		            this._dialog.setSecondaryTitleNode(cshelp.domNode);
			    }	
	            cshelp.setButtonLabel(Constants.getMessage("labelManageTemplateCSHelp"));
	            this._dialog.setWidth(this.width);
	            this._dialog.contentContainer.appendChild(this.domNode);
	            //this._dialog.setSecondaryTitle(this.getDialogTitle());
	            this._dialog.getOkButton().disabled = false;
			}
            this._dialog.open();			
		},
		
		getDialogTitle: function() {
			return this.dialogTitle;			
		},	
	
		showSectionChoice: function() {			
            //create & show the option selector widget		  
			Utility.destroyWidgetsFromNode(this._sectionChoiceContainerNode);	
			this._sectionChoice = new SortableOptionChooser({
				titleMaxWidth: 160,
				selectionSize: this.selectionSize,
				leftTitle: Constants.getMessage("labelAvailableSections"), 
				rightTitle: Constants.getMessage("labelSelectedSections"),
				enableCreate: true
			});
			this._sectionChoiceContainerNode.appendChild(this._sectionChoice.domNode);		
			if (this.isEditable()) {
				if (this.hasPermission(PermissionsClient.OP_TEMPLATE_SECTION, PermissionsClient.ACTION_CREATE)) {
					this._sectionChoice.addNewListener(dojo.hitch(this, this._showNewSectionDialog));
				} else {
					this._sectionChoice.newButton.style.display = "none";
				}
				//this._sectionChoice.enableOkCancel();				
			} else {
				this._sectionChoice.setDisabled(true);
				this._id.disabled = true;
				this._name.disabled = true;
				this._description.disabled = true;
				this._setDefaultCheckbox.disabled = true;
			}
			
			//load existing sections (in order)
			var existingSections = this._template && this._template.sections ? this._template.sections : [];
			var secLength = existingSections.length;
			for (var i = 0; i < secLength; i++) {
				var s = existingSections[i];
				this._sectionChoice.addOption(Constants.getExtensionDisplayMessage(s.name), s, true, s.required);
			}
			//load the remaining selections
			for (var i = 0; i < this._allSections.length; i++) {
                var s = this._allSections[i];
				if (!this.hasSection(s)) {
	                if (this.isNew && s.required) {
	                    this._sectionChoice.addOption(Constants.getExtensionDisplayMessage(s.name), s, true, true);
	                } else {
	                    this._sectionChoice.addOption(Constants.getExtensionDisplayMessage(s.name), s, false);
	                }				
				}
			}		
			if (this._dialog) {
				this._dialog.updatePosition();
			}	
		},   
		
		getSelectedSections: function() {
			var selected = [];
			if (this._sectionChoice) {
				var objects = this._sectionChoice.getSelectedOptions();
				var objLen = objects ? objects.length : 0;
				for (var i = 0; i < objLen; i++) {
					var obj = objects[i];
					selected.push({
						name: obj.name,
						itemId: obj.itemId,
						itemType: obj.itemType,
						isRegistrySection: (obj.isRegistrySection === true)
					});
				}
			}
			return objects;
		},
			
        //////////////////////////////////////////////////////////////////////////////////////////
        ///                                        API                                         ///
        //////////////////////////////////////////////////////////////////////////////////////////		
		isSetDefault: function() {
			return this._setDefault;
		},    
		
        hasPermission: function(operationId, actionId) {
            var permission = this._permissions[operationId];
            return (permission && permission[actionId] && permission[actionId] == true);         
        },	
		
		hasSection: function(section) {
			var sections = this._template ? this._template.sections : [];
			var secLen = sections ? sections.length : 0;
			for (var i = 0; i < secLen; i++) {
				if (sections[i].itemId == section.itemId) {
					return true;
				}
			}
			return false;
		},		
		
        //////////////////////////////////////////////////////////////////////////////////////////
        ///                                      Internal                                      ///
        //////////////////////////////////////////////////////////////////////////////////////////  
		_validate: function() {            
			if (!this._id.value || dojo.trim(this._id.value).length == 0) {
				if (this.isNew()) {
					alert(Constants.getMessage("msgRequiredField", "ID", "Template"));
					return false;
				} else {
					this._id.value = this._template.id;
				}
			} 
			if (!this._name.value || dojo.trim(this._name.value).length == 0) {
                if (this.isNew()) {
                    alert(Constants.getMessage("msgRequiredField", "Name", "Template"));
                    return false;
                } else {
                    this._name.value = this._template.name;
                }				
			}
			return true;
		},
				
		_onOK: function() {
			//if new or modifiable, save the template
			if (this.isNew() || (this._template && this._template.modifiable)) {
				this._save();
			}			
		},
		
		_save: function() {
			//validate the template
			if (!this._validate()) {
				if (this._sectionChoice) {
				    this._sectionChoice.okNode.disabled = true;	
				} 
				return false;
			}
			
            //disable the actions
            this._sectionChoice.okNode.disabled = true;
            this._sectionChoice.cancelNode.disabled = true;
            
			//save the template
            var templateObj = {
                isNewItem: this.isNew(),     
				isDefault: this._setDefaultCheckbox.checked,           
                id: this._id.value,
				name: this._name.value,
				editorId: this.editorId,
				modifiable: this.isNew() ? true : this._template.modifiable,
                sectionsJsonString: dojo.toJson(this.getSelectedSections())
            }
			if (this._description && this._description.value) {
				templateObj.description = this._description.value;
			}
			if (!this.isNew()) {
				templateObj.itemId = this._template.itemId;
				templateObj.stateId = this._template.stateId;
			}
            var handler = new ServiceResponseHandler(this, "_saveCompleted", "_error");
            TemplateClient.postTemplate(handler, templateObj);			
		},
		
        _saveCompleted: function(savedTemplate) {            
			if (dojo.isFunction(this.onOK)) {
				this.onOK();
			}
			this._close();
        },		
		
        _saveFailed: function(error) {            
            alert("Unable to save Template: " + error);
            this._sectionChoice.okNode.disabled = true;
            this._sectionChoice.cancelNode.disabled = true;			
        },		
		
        _close: function() {
            if (dojo.isFunction(this.destroy)) {
                this.destroy();                    
            };
        },
		
        _showNewSectionDialog: function() {
            var dialogArgs = {      
                onOk: dojo.hitch(this, "_newSectionAdded")
            }
            this._newSectionDialog = new NewSectionDialog(dialogArgs);          
        },
        
        _newSectionAdded: function(sectionId, sectionName, sectionDescription, args) {
            var newSection = {
				isNewItem: true,
                id: this._fullyQualifySectionId(sectionId),
                name: sectionName,
				description: sectionDescription,
				allowedEditors: this.editorId,
				itemType: (args.type == Constants.DYNAMICSECTION_GRID ? "com.ibm.rqm.planning.web.ui.internal.view.extension.DynamicGridSection": null),
				attributes: [{ attributeType: args.type, name: "content"}]
            };
			if(args.type == Constants.DYNAMICSECTION_GRID){
				newSection.configurationData = args.configurationData;
				newSection.attributes = [{type: "GRID", name: "GRID", id: "dynamicGrid"}];
			}
            if (this._sectionChoice) {
                this._sectionChoice.addOption(sectionName, newSection, true, true);  
            }            
        },			
		
		_fullyQualifySectionId: function(id) {
			return Constants.EDITOR_PREFIX + ".section." + id;
		},
		
        _generateTemplateId: function(id) {
			var uniqueId = "template_" + new Date().getTime();
			var typeName =this._artifactType.split(" ").join("").toLowerCase();
            return Constants.EDITOR_TEMPLATE_PREFIX + "." + typeName + "." + uniqueId;
        },		
		
		_onDefaultChange: function() {
			this._setDefault = this._setDefaultCheckbox.checked;
		},
		
        _onAdvancedChange: function() {
            this._isAdvanced = this._setAdvancedCheckbox.checked;
			dojo.toggleClass(this._idContainer, "hidden", !this._isAdvanced);
        },
		
        _sortByName: function(val1, val2) {
            if (val1.name < val2.name) {
				return -1;
			} else if (val1.name > val2.name) {
				return 1;
			}
			return 0;
        },		
		
        _error: function(e) {
            console.error(this.declaredClass + ": Error: " + e.message);
        }
	});  
	
})();


