/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

dojo.provide("com.ibm.asq.common.web.ui.internal.view.layout.ImportPageLayout");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");

(function() {

dojo.declare("com.ibm.asq.common.web.ui.internal.view.layout.ImportPageLayout",
com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout,
{
	isDisplayBottomButton: false,

    postCreate: function() {
        this.inherited('postCreate', arguments);
        
        // hide the summary that is not used but leave the blank attribute
        // container DIV in place to allocate space for the status message
        this.summaryNode.style.display = "none";
        
        // create the import button
        this.importButton = this.addRightViewAction(
            MessageUtil.getMessage(MessageBundle.COMMON, "uploaderLabelUpload"),
            dojo.hitch(this, "onImport"), null, null);
        this.importButton.disabled = true;
    },
    
    // consumer should override this method to perform the import
    onImport: function() {
        
    },
    
    // return the import button for the consumer to control the enablement
    getImportButton: function() {
        return this.importButton;
    }

});

dojo.declare("com.ibm.asq.common.web.ui.internal.view.layout.SimpleSection",
[dijit._Widget, dijit._Templated],
{
    templateString: '<div id="${id}" dojoAttachPoint="domNode" class="com-ibm-asq-common-web-ui-simple-section">' +
                    '  <div class="header-left">' +
                    '    <div class="header-right">' +
                    '      <div class="header-mid">' +
                    '      </div>' +
                    '    </div>' +
                    '  </div>' +
                    '  <div class="title-area" dojoAttachPoint="titleContainerNode">' +
                    '    <span dojoAttachPoint="sectionTitle" class="section-title"></span>' +
                    '    <span dojoAttachPoint="sectionHelpButtonNode" class="help-button-node"></span><p></p>' +
                    '  </div>' +
                    '  <div class="content-area" dojoAttachPoint="containerNode,contentNode" waiRole="region">' +
                    '  </div>' +
                    '  <div class="content-bottom-left">' +
                    '    <div class="content-bottom-right">' +
                    '      <div class="content-bottom-mid">' +
                    '      </div>' +
                    '    </div>' +
                    '  </div>' +
                    '</div>',

    postCreate: function(){
        this.inherited('postCreate', arguments);
        this.setTitle(this.title);
        this.setCSHelpId(this.contextHelpId, this.csHelpProviderId, this.csHelpEngine);
        if (this.contentPaddingBottom) {
            this.containerNode.style.paddingBottom = this.contentPaddingBottom;
        }
    },

    setTitle: function(title) {
        this.title = title;
        if (this.sectionTitle)
            this.sectionTitle.innerHTML = this.title ? this.title : "";
    },

    setCSHelpId: function(helpId, csHelpProviderId, csHelpEngine){
        this.contextHelpId = helpId;
        this.csHelpProviderId = csHelpProviderId;
        this.csHelpEngine = csHelpEngine;
        if (this.sectionHelpButtonNode) {
            dojox.data.dom.removeChildren(this.sectionHelpButtonNode);
            if (this.contextHelpId) {
                this.sectionHelpButtonNode.style.display = "inline";
                var args = {
                    'contextID': helpId,
                    'csHelpEngine': this.csHelpEngine,
                    'csHelpProviderId': this.csHelpProviderId
                };
                this.cshelp = new com.ibm.asq.common.web.ui.internal.util.CSHelp(args);
                this.sectionHelpButtonNode.appendChild(this.cshelp.domNode);
            } else {
                this.sectionHelpButtonNode.style.display = "none";
            }
        }
    },
    
    setCSHelpButtonLabel: function(/* REQUIRED */ cshelpLabel) {
        if (this.cshelp)
            this.cshelp.setButtonLabel(cshelpLabel);
    }

});

})();