/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading com.ibm.rqm.planning.web.ui.internal.view.planning.PlanSummarySection");
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.planning.PlanSummarySection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.Button");

dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.CategorySelection");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.CategoriesEditor");


(function(){
    var EditorUtil = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
    var Utility = com.ibm.rqm.planning.web.ui.internal.util.Utility;
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var AbstractArtifactEditorSection = com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection;
    
    /**
     * Widget to display list of link elements in the LeftNavigation.
     */
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.planning.PlanSummarySection", [dijit._Widget, dijit._Templated, AbstractArtifactEditorSection], {
        templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/planning/PlanSummarySection.html"),
        
		labeldescription: Constants.getMessage("labeldescription"),
		
        constructor: function(){
            this._workItem = null;
        },
        
        postCreate: function(args){
            this.inherited('postCreate', arguments);
            //set up workitem support for this section
            this.addWorkItemSupport(EditorUtil.getCurrentContributor().itemId);
        },
        
		//@Overriden from AbstractArtifactEditorSection - lazily loaded by getMainContent on first expansion
        initialize: function(isEditable) {
            var plan = this.getArtifact();
			
			//category labels
            this._categoriesTitle.innerHTML = Constants.getMessage("labelCategories");			
	        var args = {
	            label: Constants.getMessage("labelManageCategories", Constants.TYPE_PLAN),
	            onClick: dojo.hitch(this, "_handleCategoriesClick"),
				iconClass: "categories-icon-image",
				showLabel: false
	        };
	        this._manageCategoriesActionButton = new com.ibm.asq.common.web.ui.internal.widgets.Button(args);			
			this._manageCategoriesActionNode.appendChild(this._manageCategoriesActionButton.domNode);
			this._manageCategoriesActionButton.setDisabled(!this.isEditable());			
			
            //setup category selection
            this.unloadContent(this._categoriesNode);
            this._categorySelect = new com.ibm.rqm.planning.web.ui.internal.view.construction.CategorySelection({
                artifactType: Constants.MODEL_TYPE_PLAN,
                artifact: plan,             
                parent: this
            });
            this._categoriesNode.appendChild(this._categorySelect.domNode);
            this._categorySelect.setDisabled(!this.isEditable());			
        },
               
        //@Overridden from AbstractArtifactEditorSection
        getAttributes: function(){
            var attributes = {};
            if (this.getWorkItemEditor()) {            
                attributes.ownerId = this.getWorkItemEditor().getOwnerId();                
            }
            if (this._categorySelect) {
                //TODO send up an array instead of this comma delimited string
                attributes.categories = this._categorySelect.getSelectedItemIds().join();
            }			
            return attributes;
        },
        
        //@Overridden from AbstractArtifactEditorSection
        saveFinished: function(savedItem){
            this.inherited('saveFinished', arguments);
//            this.initialize();
			this.initialized = false;
			this.addWorkItemSupport(true);
        },
        
        //@Overridden from AbstractArtifactEditorSection --> disable 'remove' capability for Summary section          
        getHeaderContent: /* String || DOM Node */ function(isEditable) {
            return null;
        },
        
		//@Overridden from AbstractArtifactEditorSection: 
		validate: function () {
            //categories
            if (this._categorySelect && !this._categorySelect.validate()) {
                return false;
            }			
			return true;
		},
		
	    _handleCategoriesClick: function() {
	        this._categoriesEditor = new com.ibm.rqm.planning.web.ui.internal.view.administration.CategoriesEditor({
	            artifactType: Constants.MODEL_TYPE_PLAN,
				onClose: dojo.hitch(this, "_updateCategories")
	        });
	    },		
		
		_updateCategories: function() {
			//refresh the category selection widget; refreshes the list
			if (this._categorySelect) {
				this._categorySelect.refresh();
			}
		},
		
        _onKeyUp: function(e){
            /*
             if (this._description.value != this._plan.description)
             {
             dojo.event.topic.publish("TPE-widget-event", {dirty: true});
             }
             */
        },
        
        _error: function(e){
            alert("An error occurred: [" + e.message + "] @ PlanSummarySection#_error");
        }
    });
})();
