/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading: com.ibm.rqm.process.web.ui.internal.view.sizing.SizingPopup");
dojo.provide("com.ibm.rqm.process.web.ui.internal.view.sizing.SizingPopup");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.NumberTextBox");

// Require the common widgets needed
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.team.repository.web.client.internal.RepositoryClient");
dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");

// Require local widgets
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.TooltipBuilder");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

dojo.require("com.ibm.rqm.process.web.client.internal.sizing.SizingClient");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.layout.FormTextField");

(function(){ // module container
    /* Jazz objects */
    var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    var RepositoryClient = com.ibm.team.repository.web.client.internal.RepositoryClient;
    /* QM objects */
    var EditorUtil = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;    
    var TooltipBuilder = com.ibm.rqm.planning.web.ui.internal.util.TooltipBuilder;
    
    var SizingClient = com.ibm.rqm.process.web.client.internal.sizing.SizingClient;
	var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
	var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    
	var FormDecimalField = com.ibm.asq.common.web.ui.internal.widgets.layout.FormDecimalField;
	
    dojo.declare("com.ibm.rqm.process.web.ui.internal.view.sizing.SizingPopup", 
        [dijit._Widget, dijit._Templated], {
        templatePath: dojo.moduleUrl("com.ibm.rqm.process.web", "ui/internal/view/templates/sizing/SizingPopup.html"),
        
		labelPlanningEffort: Constants.getMessage("labelPlanningEffort"),
		labelPersonHours: Constants.getMessage("labelPersonHours"),
		labelPersonDays: Constants.getMessage("labelPersonDays"),
		labelPersonMonths: Constants.getMessage("labelPersonMonths"),
		labelPersonYears: Constants.getMessage("labelPersonYears"),
		labelExecutionEffort: Constants.getMessage("labelExecutionEffort"),
		
        // object initializer
        constructor: function(args) {
			this.artifact = args.artifact;
			this.sizingRecord = null;
			this.type = args.type;
			this.dialog = null;
			this.effortWidget = null;
			this.exeEffortWidget = null;
			this.isEditable = args.isEditable;
		},
   
   		postCreate: function() {
			this.initialize();
		},
        
        initialize: function(){
			dojox.data.dom.removeChildren(this._planEffortTD);
			dojox.data.dom.removeChildren(this._executionEffortTD);
			
			this.effortWidget = this._createFormField(this.labelPlanningEffort);
			this._planEffortTD.appendChild(this.effortWidget.domNode);
			this.exeEffortWidget = this._createFormField(this.labelExecutionEffort);
			this._executionEffortTD.appendChild(this.exeEffortWidget.domNode);
				
			var responseHandler = new ServiceResponseHandler(this, "_doInitSizing", "_error");
			//this.artifact  this.parentEditor
			if(this.artifact != null){
				SizingClient.getArtifactWithSizing(responseHandler, {type: this.type, uid: this.artifact.versionableItemId});
     		}
	    },
		
		_createFormField: function(label) {
			var node = new FormDecimalField({
				fieldLabel: label,
				labelWidthPercent: (dojo.isIE) ? 33 : 30,
				required: true});
			dojo.removeClass(node.fieldHolderNode.lastChild, "text-field");
			node.fieldHolderNode.lastChild.style.width = "93%";	
			node.fieldHolderNode.lastChild.style.padding = "2px 0";
			return node;
		},
		
		_doInitSizing: function(result) {	
			if (result.sizingRecord) {
					this.sizingRecord = result.sizingRecord;
					this.effortWidget.setValue(this.sizingRecord.effort);
					this.effortWidget.parent = this;
					this.exeEffortWidget.setValue(this.sizingRecord.estimatedEffort);
					this._planUnit.value = this.sizingRecord.unit;
					this._executionUnit.value = this.sizingRecord.description;
				}
			this.effortWidget._valueChangedHandler = dojo.hitch(this, "_onChange");
			this.exeEffortWidget._valueChangedHandler = dojo.hitch(this, "_onChange");
			if(!this.isEditable) {
				this.effortWidget.textField.setDisabled(true);
				this.exeEffortWidget.textField.setDisabled(true);
				this._planUnit.setAttribute("disabled","");
				this._executionUnit.setAttribute("disabled","");
				
			}
		},
		
		setDialog: function(dialog) {
			this.dialog = dialog; 	
		},
		
		_handleSaveClick: function(owner){
			var params = this.getAttributes(owner);
			var responseHandler = new ServiceResponseHandler(this, "_saveFinish", "_error");
            SizingClient.saveSizingRecord(responseHandler, params);
			return;
		},	
		
		getAttributes: function(owner){
            var sizingObj = {
				type: this.type,
                unit: this._planUnit.value,
				description: this._executionUnit.value,
                owner: owner,
                effort: dojo.trim(this.effortWidget.getValue()),//dojo.trim(this._planEffort.value) == "" ? "0" : dojo.trim(this._planEffort.value),
				estimatedEffort: dojo.trim(this.exeEffortWidget.getValue()),//dojo.trim(this._executionEffort.value) == "" ? "0" : dojo.trim(this._executionEffort.value),
                artifactItemId: this.artifact!=null ? this.artifact.versionableItemId : null,
//				projectAreaName: EditorUtil.getCurrentProjectArea().name,
				externalId: "sizing"
            };
            return sizingObj;
        },
		
		_saveFinish: function(){
			this.initialize();
			
		},
		_onChange: function() {
			if(this.parentSection){
				this.parentSection.setDirty();
			}
		},
        _error: function(e){
            alert("An error occurred: [" + e.message + "] @ SizingEditor#_error");
        }
    });
    
})();
