/******************************SuiteExecutionLog************************************************ 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
/*
 * ExecutionLogView for Suite
 * Author: vasrivas
 */
dojo.provide("com.ibm.rqm.execution.web.ui.internal.view.suite.SuiteExecutionLog");

dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.execution.web.client.internal.execution.ExecutionResultGroupClient");
dojo.require("com.ibm.rqm.execution.web.client.internal.execution.ExecutionResultStateClient");
dojo.require("com.ibm.rqm.execution.web.client.internal.mapper.ExecutionResultGroupMapper");
dojo.require("com.ibm.rqm.execution.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");
dojo.require("com.ibm.rqm.execution.web.ui.internal.view.table.TestSuiteInternalResultTableView");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.CollapsibleSection");
dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");
dojo.require("com.ibm.rqm.execution.web.ui.internal.view.suite.SuiteWeightSection");
dojo.require("com.ibm.asq.common.web.ui.internal.util.GlobalEvents");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.util.SelectUtility");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

(function(){ 
	
	var ServiceResponseHandler			= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	var ERGClient 						= com.ibm.rqm.execution.web.client.internal.execution.ExecutionResultGroupClient;
	var ExecutionResultStateClient		= com.ibm.rqm.execution.web.client.internal.execution.ExecutionResultStateClient;
	var Constants                       = com.ibm.rqm.planning.web.client.internal.Constants;
	var Utility 						= com.ibm.rqm.execution.web.ui.internal.util.Utility;
	var TSIResultTableView				= com.ibm.rqm.execution.web.ui.internal.view.table.TestSuiteInternalResultTableView;
	var CollapsibleSection				= com.ibm.asq.common.web.ui.internal.view.common.CollapsibleSection;
	var WeightSection					= com.ibm.rqm.execution.web.ui.internal.view.suite.SuiteWeightSection;
	var GlobalEvents 					= com.ibm.asq.common.web.ui.internal.util.GlobalEvents;
	var ViewHeaderLayout 				= com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout;
	var YesNoDialog      				= com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog;
	var EditorUtil						= com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
	var SelectUtility 					= dojo.getObject( "com.ibm.asq.common.web.ui.internal.util.SelectUtility" );
	var ERGMapper						= com.ibm.rqm.execution.web.client.internal.mapper.ExecutionResultGroupMapper;
	
	dojo.declare("com.ibm.rqm.execution.web.ui.internal.view.suite.SuiteExecutionLog", [dijit._Widget, dijit._Templated], {
		
		templateString: "<div>"+
							"<div dojoType=\"com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout\" dojoAttachPoint=\"viewHeaderNode\">" +
							"<div dojoAttachPoint=\"_topSection\"></div>"+
							"<div dojoAttachPoint=\"_weightSection\"></div>"+
							"<div dojoAttachPoint=\"_table\"></div>"+
						"</div>",
		
		widgetsInTemplate: true,
		initialized: false,
		verdicts: [],
		
		postCreate: function(){
			var args = {}
			if(this.suiteId){
				args.id = this.suiteId;
			}else if(this.itemId){
				args.itemId = this.itemId
			}
			
			args.includeResult = true;
			args.includeFullResult = false;
			args.includeRequest = false;
			args.includeTestcase = false;
			var responseHandler = new ERGMapper(this, "initialize", "_error");
			ERGClient.getExecutionResultGroupDTOs(responseHandler, args);
		},
		
		initialize: function(erGroup){
			if(erGroup && erGroup.length > 0){
				this.erGroup = erGroup[0];
			}
			this.fillDetails();
			if(!this.initialized){
				this.viewHeaderNode.setCSHelpId("rqm_suite_results_overview.html");
				this.viewHeaderNode.setIconClass("ewiresult-icon-48");
				this.viewHeaderNode.setTypeLabel(Constants.getMessage("labelTestSuite"));
				this.viewHeaderNode.setTitle(Constants.getMessage("labelTestSuiteResultExecution", this.erGroup.name));
			
				if(this._isPermitted()){
					this.cancelButton = this.viewHeaderNode.addRightViewAction(Constants.getMessage("labelDiscardChanges"), dojo.hitch(this, "onCancel"), null, null);
					this.saveButton = this.viewHeaderNode.addRightViewAction(Constants.getMessage("labelSave"), dojo.hitch(this, "onSave"), null, null);
					this.saveButton.setDisabled(true);
					this.cancelButton.setDisabled(true);
					var registryResponseHandler = new ServiceResponseHandler(this, "_setResultStates", "_error");
					ExecutionResultStateClient.getExecutionResultStates(registryResponseHandler, {includeHidden: true});
				}
				this.initialized = true;
			}
			
			this._updateBrowserTitle();
		},
		
		fillDetails: function(){
			var doc = document;
			var table = doc.createElement('table');
			table.width = "50%";
			dojox.data.dom.removeChildren(this._topSection);
			this._topSection.appendChild(table);
			
			//Verdict row
			var verdictRow = this._getEmptyRow(doc);
			verdictRow.width = "100%";
			table.appendChild(verdictRow);
			var verdictLabelCell = this._getEmptyCell(doc);
			verdictLabelCell.width = "45%";
			var verdictEmptyCell = this._getEmptyCell(doc);
			verdictEmptyCell.width = "5%";
			var verdictValueCell = this._getEmptyCell(doc);
			verdictValueCell.width = "50%";
			verdictRow.appendChild(verdictLabelCell);
			verdictRow.appendChild(verdictEmptyCell);
			verdictRow.appendChild(verdictValueCell);
			var verdictLabel = doc.createElement('label');
			verdictLabel.innerHTML = Constants.getMessage("labelOverallVerdict")+Constants.getMessage("labelColen");
			this.verdictImg = doc.createElement('img');
			this.verdictImg.src = Utility.getImageForState(this.erGroup.state.id);
			verdictLabelCell.appendChild(verdictLabel);
			verdictEmptyCell.appendChild(this.verdictImg);
			if(!this._isPermitted()){
				this.verdictValue = doc.createElement('label');
				this.verdictValue.innerHTML = this.erGroup.state.name;
				dojo.style(this.verdictValue, "fontWeight", "bold");
				verdictValueCell.appendChild(this.verdictValue);
			}else{
				this.verdictCombo = doc.createElement('select');
				verdictValueCell.appendChild(this.verdictCombo);
				dojo.style(this.verdictCombo, "fontWeight", "bold");
				if(this.verdicts.length > 0)
					this._setResultStates(this.verdicts)
				this.connect(this.verdictCombo, "onchange", this._updateVerdictIcon);
			}
			
			//StartTime row
			var startTimeRow = this._getEmptyRow(doc);
			startTimeRow.width = "100%";
			table.appendChild(startTimeRow);
			var stLabelCell = this._getEmptyCell(doc);
			stLabelCell.width = "45%";
			var stEmptyCell = this._getEmptyCell(doc);
			stEmptyCell.width = "5%";
			var stValueCell = this._getEmptyCell(doc);
			stValueCell.width = "50%";
			startTimeRow.appendChild(stLabelCell);
			startTimeRow.appendChild(stEmptyCell);
			startTimeRow.appendChild(stValueCell);
			var stLabel = doc.createTextNode(Constants.getMessage("labelStartTime")+Constants.getMessage("labelColen"));
			var stValue = doc.createTextNode(Utility._convertLongToDate(this.erGroup.startTime));
			stLabelCell.appendChild(stLabel);
			stValueCell.appendChild(stValue);
			
			//EndTime row
			var endTimeRow = this._getEmptyRow(doc);
			endTimeRow.width = "100%";
			table.appendChild(endTimeRow);
			var endLabelCell = this._getEmptyCell(doc);
			endLabelCell.width = "45%";
			var endEmptyCell = this._getEmptyCell(doc);
			endEmptyCell.width = "5%";
			var endValueCell = this._getEmptyCell(doc);
			endValueCell.width = "50%";
			endTimeRow.appendChild(endLabelCell);
			endTimeRow.appendChild(endEmptyCell);
			endTimeRow.appendChild(endValueCell);
			var endLabel = doc.createTextNode(Constants.getMessage("labelEndTime")+Constants.getMessage("labelColen"));
			endLabelCell.appendChild(endLabel);
			if (this.erGroup.endTime > 0) {
				var endValue = doc.createTextNode(Utility._convertLongToDate(this.erGroup.endTime));
				endValueCell.appendChild(endValue);
			}
			
			//Total Run Time row
			var totalTimeRow = this._getEmptyRow(doc);
			totalTimeRow.width = "100%";
			table.appendChild(totalTimeRow);
			var totalLabelCell = this._getEmptyCell(doc);
			totalLabelCell.width = "45%";
			var totalEmptyCell = this._getEmptyCell(doc);
			totalEmptyCell.width = "5%";
			var totalValueCell = this._getEmptyCell(doc);
			totalValueCell.width = "50%";
			totalTimeRow.appendChild(totalLabelCell);
			totalTimeRow.appendChild(totalEmptyCell);
			totalTimeRow.appendChild(totalValueCell);
			var totalLabel = doc.createTextNode(Constants.getMessage("labelTotalRunTime")+Constants.getMessage("labelColen"));
			var duration = this.erGroup.endTime - this.erGroup.startTime;
			var totalValue = doc.createTextNode(Utility.getTimeDifferenceInHrMinSec(duration > 0 ? duration : 0));
			totalLabelCell.appendChild(totalLabel);
			totalValueCell.appendChild(totalValue);
			
			//Mode row
			var modeRow = this._getEmptyRow(doc);
			modeRow.width = "100%";
			table.appendChild(modeRow);
			var modeLabelCell = this._getEmptyCell(doc);
			modeLabelCell.width = "45%";
			var modeEmptyCell = this._getEmptyCell(doc);
			modeEmptyCell.width = "5%";
			var modeValueCell = this._getEmptyCell(doc);
			modeValueCell.width = "50%";
			modeRow.appendChild(modeLabelCell);
			modeRow.appendChild(modeEmptyCell)
			modeRow.appendChild(modeValueCell);
			var modeLabel = doc.createElement('label');
			modeLabel.innerHTML = Constants.getMessage("labelExecutionMode")+Constants.getMessage("labelColen");
			var mode = this.erGroup.sequentialExecution ? Constants.getMessage("labelSequential") : Constants.getMessage("labelParallel");
			var modeValue = doc.createTextNode(mode);
			modeLabelCell.appendChild(modeLabel);
			modeValueCell.appendChild(modeValue);
			
			this._createWeightSection();
			this._createTable();
		},
		
		_createWeightSection: function(){
			this.weightSection = new  WeightSection();
			this.weightSection.initialize(this.erGroup);
			var params ={
					title: Constants.getMessage("labelWeightDist"),
					summary: this.weightSection.domNode,
					helpContextId: "rqm_execution_weight_help.html"
				}

				var resultWeightSection = this._createCollapsibleSection(params)
				dojox.data.dom.removeChildren(this._weightSection);
				this._weightSection.appendChild(resultWeightSection.domNode);
		},
		
		_createTable: function(){
			var results = [];
			var len = this.erGroup.elements ? this.erGroup.elements.length: 0;
			var erGroup = this.erGroup;
			for(var i=0; i < len; i++){
				if (erGroup.elements[i].result) {
					results.push(erGroup.elements[i].result);
				}
			}
			this.results = results;
			if(!this._TSIResultTableView){
				var args = {
					results: results
				}
				this._TSIResultTableView = new TSIResultTableView(args);
				
				var params ={
					title: Constants.getMessage("labelResultDetails"),
					summary: this._TSIResultTableView.domNode,
					helpContextId: "rqm_test_suite.html"
				}

				var resultDetailsSection = this._createCollapsibleSection(params)
				this._table.appendChild(resultDetailsSection.domNode);
			}else
				this._TSIResultTableView.refresh(results);
			var elements = this.erGroup.elements;
			var len = this.results.length;	
			for(var j=0;j<len;j++){
				var args = null;
				var args = this.getProperty(elements[j]);//Order of Results need to be order of Elements
				if(args)
					this._TSIResultTableView.updateItem(this.results[j].itemId, args, j==len);
			}	
		},
		
		getProperty: function(element){
			var value = null;
			if(element){
				value = {
					ewiId: element.workitem.id,
					configName: element.ewiConfigurationName,
					configId: element.workitem.configuration,
					weight: element.workitem.weight
				}
			}
			return value;
		},
		
		_getEmptyRow: function(doc) {
			var row = doc.createElement("tr");
			row.className = "steps-table-row";
			return row;
		},
		
		_getEmptyCell: function(doc, span, align) {
			var cell = doc.createElement("td");
			cell.className = "steps-table-cell";
			if ( span != null ) {
				cell.colSpan = span;
			}
			if ( align != null ) {
				cell.align = align;
			}
			return cell;
		},
		
		_setResultStates: function(verdicts){	
			this.verdicts = [];
			var len=verdicts.length;
			for(var i=0;i<len; i++) {
				if (verdicts[i].visible == true) {
					this.verdicts.push({name: verdicts[i].name, id: verdicts[i].id});
					SelectUtility.add(this.verdictCombo, new Option(verdicts[i].name, verdicts[i].id));
				} else if (verdicts[i].id == this.erGroup.state.id) {
					this.verdicts.push({name: verdicts[i].name, id: verdicts[i].id, disabled: true});
					var option = new Option(verdicts[i].name, verdicts[i].id);
					option.disabled = true;
					SelectUtility.add(this.verdictCombo, option);
				}
			}
			SelectUtility.select(this.verdictCombo, this.erGroup.state.id);
		},

		_updateVerdictIcon: function(){
			this.verdictImg.src = Utility.getImageForState(this.verdictCombo.value);
			this._setDirty(true);
		},
		
		_createCollapsibleSection: function(args){
			var expando = new com.ibm.asq.common.web.ui.internal.view.common.CollapsibleSection({open: true, collapsible: false});
	        expando.domNode.name = "first-section";
	        expando.setTitle(args.title ? args.title : "");
	        expando.infoNode.innerHTML = args.info ? args.info: "" ;
			if(args.summary)
				expando.summaryNode.appendChild(args.summary);
			else
				expando.summaryNode.innerHTML = "";
	        if(args.headerRight)
				expando.headerRightNode.appendChild(args.headerRight);
			else
				expando.headerRightNode.innerHTML = "";
			expando.containerHeaderNode.innerHTML = "";
        	var content = document.createElement('div');
			content.innerHTML = "";
        	expando.containerNode.appendChild(content);
			expando.setCSHelpId(args.helpContextId);
			expando.headerCollapsNode.style.display = "none";
		   	return expando;
		},
		
		_isPermitted:function(){
            if(!this.erg_type)
                this.erg_type = com.ibm.asq.common.web.client.internal.PermissionsClient.OP_SUITE_RESULT;
            if(!this.erg_perm)
                this.erg_perm = com.ibm.asq.common.web.client.internal.PermissionsClient.getStoredPermission(this.erg_type);
			if (this.erg_perm && !this.erg_perm.edit)
                return false;
            return true;
        },
		
		onSave: function(){
			var params = {};
			params.id = this.erGroup.id;
			params.stateId = this.erGroup.stateId;
			params.state = this.verdictCombo.value;//OnSave will be enabled only if editable
			var responseHandler = new ServiceResponseHandler(this, "saveFinished", "saveError");
			ERGClient.postExecutionResultGroupDTO(responseHandler, params);
			this.setLastSaved(Constants.STATUS_SAVING);
		},
		
		saveFinished: function(summaryDTO){
			if (summaryDTO.elements!=null) {
				var len=summaryDTO.elements.length ? summaryDTO.elements.length : 0;
				for(var i=0;i<len;i++){
					var element = summaryDTO.elements[i];
					element.workitem = {};
					element.workitem.id = element.ewiId;
					element.workitem.name = element.ewiName;
					element.workitem.itemId = element.ewiitemId;
					element.workitem.testCase = element.testCase;
					element.workitem.weight = element.ewiWeight;
					if(element.ewiDefaultScriptId){
						element.workitem.defaultScript = {};
						element.workitem.defaultScript.id = element.ewiDefaultScriptId;
						element.workitem.defaultScript.name = element.ewiDefaultScriptName;
						element.workitem.defaultScript.itemId = element.ewiDefaultScriptItemId;
						element.workitem.defaultScript.stateId = element.ewiDefaultScriptStateId;
						element.workitem.defaultScript.scriptType = {};
						element.workitem.defaultScript.scriptType.id = element.ewiDefaultScriptStateType;
					}else
						element.workitem.defaultScript = null;	
					element.workitem.stateId = element.ewiStateId;
					element.workitem.configuration = element.ewiConfigurationItemId;
					
					if(element.ewiPlanName){
						element.workitem.testPlan = {};
						element.workitem.testPlan.name = element.ewiPlanName;
						element.workitem.testPlan.id = element.ewiPlanId;
						element.workitem.testPlan.versionableItemId = element.ewiPlanVersionableItemId;
					}
					if(element.ewiPhaseName){
						element.workitem.phase = {};
						element.workitem.phase.name = element.ewiPhaseName;
						element.workitem.phase.itemId = element.ewiPhaseItemId;
					}
					
					delete element.ewiId;
					delete element.ewiName;
					delete element.ewiitemId;
					delete element.testCase;
					delete element.ewiDefaultScriptId;
					delete element.ewiDefaultScriptName;
					delete element.ewiDefaultScriptItemId;
					delete element.ewiDefaultScriptStateId;
					delete element.ewiDefaultScriptStateType;
					delete element.ewiConfigurationItemId;
					delete element.ewiStateId;
					delete element.ewiPhaseItemId;
					delete element.ewiPhaseName;
					delete element.ewiPlanVersionableItemId;
					delete element.ewiPlanName;
					delete element.ewiPlanId;
					delete element.ewiWeight;
				}	
			}
			var ergObj = summaryDTO;
			this._setDirty(false);
			this.setLastSaved(Constants.STATUS_SUCCESS);
			this.saveButton.setDisabled(true);
			this.cancelButton.setDisabled(true);
			this.initialize([ergObj]);
		},
		
		saveError: function(status){
			if (status!=null && status.message!=null) {
				var re = /^\[(\d+)\]\s*:\s*/;
				status.message = status.message.replace(re, "");
			}
			this.setLastSaved(Constants.STATUS_FAILURE, status);
		},
		
		onCancel: function(){
			var self = this;
			var args = {
				title: Constants.getMessage("labelConfirmation"),
				message: Constants.getMessage("msgConfirmCancel"),
				yesCallback: function(){
					self.initialize(self.erGroup);
					return true;          
				},
				noCallback: function() {
					return false;
				}
			}
			var dialog = new YesNoDialog(args);
	        dialog.setSecondaryTitle(args.title);
	        dialog.open();
		},
		
		// handler on artifacts changed
        onArtifactChanged: function(changeType, artifactType, artifactArray) {
			if ((changeType != GlobalEvents.ArtifactsAdded) && (artifactType == Constants.TYPE_EXE_LOG)) {
                var ergElements = this.erGroup.elements;
				var changed = false;
				var artifactArrayLength=artifactArray.length;
				var ergElementsLength=ergElements.length;
				for (var i = 0; i < artifactArrayLength; i++) {
					for(var j=0;j<ergElementsLength;j++){
						if (ergElements[j] && ergElements[j].result && ergElements[j].result.id == artifactArray[i].id) {
							if(changeType == GlobalEvents.ArtifactsDeleted){
								this.viewHeaderNode.setStatus(Constants.getMessage("msgEROfSuiteDeleted"), ViewHeaderLayout.ERROR);
								changed = true;
							}
							if(changeType == GlobalEvents.ArtifactsUpdated){
								this.viewHeaderNode.setStatus(Constants.getMessage("msgEROfSuiteUpdated"), ViewHeaderLayout.INFO);
								changed = true;
							}	
                    	}
					}
                }
				if(changed && this.suiteId){
					var args = {id:this.suiteId, includeResult: true, includeFullResult: false, includeRequest: false, includeTestcase: false} //Fetch the ERG again, ERG result rolled up on update of ER
					var responseHandler = new ERGMapper(this, "initialize", "_error");
					ERGClient.getExecutionResultGroupDTOs(responseHandler, args);
				}
            }
        },
		
		setLastSaved: function(status, error) {   
            var st_code = -1; // none, no status icon
            var text = "";
            var detail_msg = null;
                                 
            if ( status == Constants.STATUS_SUCCESS ) {
                st_code = ViewHeaderLayout.SUCCESS;
                text = Constants.getCommonMessage("statusSaveSuccess", EditorUtil.getTimeString());  
            } 
            else if ( status == Constants.STATUS_FAILURE ) {
                if(error && error.message && error.message.toLowerCase().indexOf('license') > -1) {
                    var licenseMsg = Constants.getCommonMessage("serverLicenseErrMsg");
                    EditorUtil.showServerErrorMessage(licenseMsg);
                }
                st_code = ViewHeaderLayout.ERROR;
                text = Constants.getCommonMessage("statusSaveFail", EditorUtil.getTimeString());  
                
                if(error && error.message) {
                    detail_msg = "<b>" + Constants.getCommonMessage("labelError") + "</b><br/>" + error.message;
                }
            } 
            else if ( status == Constants.STATUS_SAVING ) {
                st_code = ViewHeaderLayout.INFO;
                text = Constants.getCommonMessage("statusSaving");
            } 
            else if ( status == Constants.STATUS_NEW ) {
                st_code = ViewHeaderLayout.INFO;
                text = Constants.getCommonMessage("statusNewNeedSave");
            } 
            else if ( status == Constants.STATUS_DIRTY ) {
                st_code = ViewHeaderLayout.INFO;
                text = Constants.getCommonMessage("statusDirty");
            }
            
            if(st_code > -1)
               this.viewHeaderNode.setStatus(text, st_code, detail_msg);
            else
                this.viewHeaderNode.clearStatus();
        },
		
		_setDirty:function(dirty){
			this.dirty = dirty;
			this.saveButton.setDisabled(!dirty);
			this.cancelButton.setDisabled(!dirty);
			dojo.publish(GlobalEvents.UpdateTabLabel, [window.location.hash, "", null, dirty]);
			if(dirty)
				this.setLastSaved(Constants.STATUS_DIRTY);
		},
		
		_error: function(errorObj){
			console.error("Error in Suite Execution Log"+ errorObj.message + " ");
		},
		
		_updateBrowserTitle: function(){
			if (this.page){
				var name = "";
				if (this.erGroup && this.erGroup.name) {
					name = this.erGroup.name;
				}
				this.page.browserTitle =  Constants.getMessage("labelTestSuiteResultExecution", Constants.getMessage("labelColen") +" " + name);
				net.jazz.ajax.ui.PlatformUI.getWorkbench().setTitle(this.page.browserTitle);
				net.jazz.ajax.ui.PlatformUI.getWorkbench().browserTitle = this.page.browserTitle;	
			}
		}          
    });
})();
