/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.execution.web.ui.internal.view.table.EngineTableView");
 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.rqm.execution.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");

(function() {

var TableConstants 		= com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
var ImageCache     		= com.ibm.rqm.execution.web.ui.internal.view.common.ImageCache;
var Constants      		= com.ibm.rqm.planning.web.client.internal.Constants;
var CommonImageCache	= com.ibm.asq.common.web.ui.internal.view.common.ImageCache;

dojo.declare("EngineTableView.TableProvider", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider,
{
    getNumColumns: function(content){
        return 4;
    },
    
    _columnHeadings: [
        Constants.getMessage("labelId"),
        Constants.getMessage("labelName"),
        Constants.getMessage("labelHostname"),
		Constants.getMessage("labelState")
    ],
    
    getColumnHeading: function(content, index){
        return this._columnHeadings[index];
    },

    getColumnHeadingImage: function(content, atColumnIndex){
        return null;
    },
    getColumnHeadingAlignment: function(atColumnIndex){
        return this.getCellAlignment(null, atColumnIndex);
    },	 
    
    getIsColumnGroupable: function(content, atColumnIndex){
        return (atColumnIndex >= 1 && atColumnIndex <= 3);     
    },

    getIsColumnSortable: function(content, atColumnIndex){
        return (atColumnIndex >= 1 && atColumnIndex <= 3);   
    },
    
    _columnWidths: [12,60,15,10],

    getColumnWidth: function(content, atColumnIndex)
    {
        return this._columnWidths[atColumnIndex];
    },
    setServerTime: function(serverTime){
		this.serverTime = serverTime;
	},
    getContentObjects: function(content){
        return content;
    },
    
    getRowDataId: function(rowData){
        return "_table_row_for_engine_" + rowData.id;
    },
    
    getCellDisplayType: function(rowData, atColumnIndex){
        if(atColumnIndex == 3) {
			return TableConstants.RENDER_TYPE_IMAGE;
        }
        return TableConstants.RENDER_TYPE_TEXT;;
    },

    getCellAlignment: function(rowData, atColumnIndex){
        if(atColumnIndex < 0)
            return TableConstants.ALIGN_CENTER;
        return TableConstants.ALIGN_LEFT;
    },
    
    getCellDisplayData: function(rowData, atColumnIndex){
        var val = "";
        switch(atColumnIndex) {
            case 0: 
                val = rowData.id;
                break;
            case 1: 
                val = rowData.name;
                break;
            case 2: 
                val = rowData.hostName;
                break;   
            case 3: 
				this.engineStatus = Constants.getMessage("labelAvailable");
				this.engineStatusIcon = CommonImageCache.ENGINE_AVAILABLE;
				this.engineToolTip = Constants.getMessage("msgEngAvailable");
				if( (this.serverTime - rowData.lastHeartBeat) > 10*(rowData.pollingInterval)){
					this.engineStatus = Constants.getMessage("labelUnAvailable");
					this.engineStatusIcon = CommonImageCache.ENGINE_NOTAVAILABLE;
					this.engineToolTip = Constants.getMessage("msgEngUnresponsive");
				}
				else if( (this.serverTime - rowData.lastHeartBeat) > 2*(rowData.pollingInterval)){
					this.engineStatus = Constants.getMessage("labelAvailable");
					this.engineStatusIcon = CommonImageCache.ENGINE_NOTRESPONDED;
					this.engineToolTip = Constants.getMessage("msgEngRespondLate");
				}
                val = this.engineStatus;
                break;  
        }
        return val;
    },

    getCellDisplayExtendedInfo: function(rowData, atColumnIndex){  
        var val = null;  
        if (atColumnIndex == 3) {
			val = {};
			val.image = this.engineStatusIcon;
			val.tooltip = this.engineToolTip;
		}		  
        return val;
    }
    
    
});

dojo.declare("com.ibm.rqm.execution.web.ui.internal.view.table.EngineTableView", 
    [dijit._Widget, dijit._Templated],
{    
    templateString: '<div dojoAttachPoint="_tableViewerNode" class="com-ibm-rqm-test-artifact-table-view" tabindex="-1"></div>',
    widgetsInTemplate: true,
    constructor: function() {
	 	this.results = null;
	 },

    postCreate: function(){
        this.inherited("postCreate", arguments);
        var tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer({
            allowAdjustSettings: true,
            autoSaveSettings: true,
            tableId: "RQM-EXE-ENGINE-TABLEVIEWER"                          
        });        
        this._tableViewerNode.appendChild(tableViewer.domNode);
        var content = this.engines ? this.engines : [];
        tableViewer.setContentPageSize(10);
        tableViewer.setContent(content);
		var tableProvider = new EngineTableView.TableProvider();
		tableProvider.setServerTime(this.serverTime);
        tableViewer.setContentProvider(tableProvider);
        tableViewer.setTableListener(new EngineTableView.TableListener());
		tableViewer.setTableRowsCheckable(this.executable, /* isSingleSelect */true);
		tableViewer.setTableSummary(Constants.getMessage("labelTableSummary", Constants.getMessage("labelEngines")));
        tableViewer.show();
		this.tableViewer = tableViewer;
    },
	
	setContent: function(content, serverTime){
		this.tableViewer.setContent(content);
		if(serverTime)
			this.tableViewer.getContentProvider().setServerTime(serverTime);
		this.tableViewer.refresh()
	},
	
	getSelectedRows: function() {
		return this.tableViewer.getCheckedRows();
	}
});

})();
