/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.table.TemplatesTableView");
 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer");

dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

dojo.require("com.ibm.rqm.planning.web.ui.internal.view.itemtemplates.TemplateEditor");

(function() {

var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
var TemplateEditor = com.ibm.rqm.planning.web.ui.internal.view.itemtemplates.TemplateEditor;
var ImageCache = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;

var EDITOR_MAP = {};
EDITOR_MAP[Constants.EDITOR_ID_PLAN] = Constants.TYPE_PLAN;
EDITOR_MAP[Constants.EDITOR_ID_CASE] = Constants.TYPE_CASE;
var ICON_MAP = {};
ICON_MAP[Constants.EDITOR_ID_PLAN] = ImageCache.TEMPLATE_PLAN_ICON;
ICON_MAP[Constants.EDITOR_ID_CASE] = ImageCache.TEMPLATE_CASE_ICON;

dojo.declare("TemplatesTableView.TableListener", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener,
{
    // Summary:
    //   If the table's rows are checkable, when a row is checked, or unchecked, this method is
    //   called.
    //
    rowCheckStateChanged: function(tableViewer, rowObject, state)
    {
        //enable/disable the archive action depending on the state
		var actionButtons = tableViewer.getActionButtons();
		var actionsLen = actionButtons.length;
		for (var i = 0; i < actionsLen; i++) {
			var actionObj = actionButtons[i].actionObject;
			if (actionObj && actionObj.requiresSelection == true) {
                var disabled = !state;
				disabled = disabled || (actionObj.id == "archive" && !rowObject.modifiable);
				disabled = disabled || (actionObj.id == "setDefault" && rowObject.isDefault);
				actionButtons[i].setDisabled(disabled);
			}
		}      
    },
    
    // Summary:
    //    If the client have registered table actions, this message will be fired when an action (button)
    //    is invoked by the user.
    //
    tableContentActionInvoked: function(tableViewer, actionId, callbackData, callback) 
	{		
        if (callback != null && dojo.isFunction(callback)) {
            if (callbackData != null) {
                callback(callbackData);
            } else {
                callback();
            }
        }	
	}    
});

dojo.declare("TemplatesTableView.CellModifier", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier,
{
    // Summary:
    //    Method to be called when a cell renderer changes the cell value. This notifies
    //    the cell's value change event. 
    //    
    //    
    cellValueChanged: function(oldValue, newValue, rowObject, columnIndex)
    {
    },
    
    cellActionInvoked: function(rowObject, columnIndex)
    {
    }
});

dojo.declare("TemplatesTableView.ListRowRenderer",
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer,
{    
    // Summary:
    //   Override the base list row renderer's create list content method
    //   to supply more custom contents.
    //
    _createListContents: function(viewer, rowObject, listWidget)
    {
    }
});

dojo.declare("TemplatesTableView.TableProvider", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider,
{
	
	constructor: function(args) {
		dojo.mixin(this, args);
		var COLUMNS = {
            TYPE: this.toColumn(Constants.getMessage("labelType"), 2, true, TableConstants.ALIGN_CENTER, TableConstants.RENDER_TYPE_IMAGE),
            NAME: this.toColumn(Constants.getMessage("labelName"), 40, false, TableConstants.ALIGN_LEFT, TableConstants.RENDER_TYPE_LINK),
			DESCRIPTION: this.toColumn(Constants.getMessage("labelDescription"), 45, false, TableConstants.ALIGN_LEFT, TableConstants.RENDER_TYPE_TEXT),
			ISDEFAULT: this.toColumn(Constants.getMessage("labelDefault"), 6, false, TableConstants.ALIGN_CENTER, TableConstants.RENDER_TYPE_TEXT),
			MODIFIABLE: this.toColumn(Constants.getMessage("labelModifiable"), 7, false, TableConstants.ALIGN_CENTER, TableConstants.RENDER_TYPE_TEXT)
        }	   
        COLUMNS.ALL = [COLUMNS.TYPE, COLUMNS.NAME, COLUMNS.DESCRIPTION, COLUMNS.ISDEFAULT, COLUMNS.MODIFIABLE];
		this.COLUMNS = COLUMNS;  
	},
	
    toColumn: function(name, width, groupable, alignment, type) {
        return {
            name: name,
            width: width,
            groupable: groupable,
            alignment: alignment,
            type: type
        }
    },	

    getColumnIndex: function(column) {
		var COLUMNS = this.COLUMNS;
		var colLen = COLUMNS.ALL.length;
        for (var i = 0; i < colLen; i++) {
			if (COLUMNS.ALL[i].name == column.name) {
				return i;
			}
		}
		return -1;
    },
	
    // Summary:
    //
    getNumColumns: function(content)
    {
        return this.COLUMNS.ALL.length;
    },
    
    // Summary:
    //
    getColumnHeading: function(content, index)
    {
        return this.COLUMNS.ALL[index].name;
    },

    // Summary:
    //    Optionally, returns an image src URL to be placed in the column header. If both heading
    //    and image are specified, both are shown.
    //    
    getColumnHeadingImage: function(content, atColumnIndex)
    {
        return null;
    },
	 
    // Summary:
    //    Returns the column header horizontal alignment. By default, the columns are
    //    aligned to the left.
    //
    //    Client optionally can indicate the alignment for the column header.
    //
    getColumnHeadingAlignment: function(atColumnIndex)
    {
        return this.COLUMNS.ALL[atColumnIndex].alignment;
    },	 
    
    getIsColumnGroupable: function(content, atColumnIndex)
    {
        return this.COLUMNS.ALL[atColumnIndex].groupable;     
    },

    getIsColumnSortable: function(content, atColumnIndex)
    {
        return this.COLUMNS.ALL[atColumnIndex].name != this.COLUMNS.TYPE.name;     
    },
    
    getColumnWidth: function(content, atColumnIndex)
    {
        return this.COLUMNS.ALL[atColumnIndex].width;
    },
    
    // Summary:
    //    The only assumptions the table have on the data that it displays
    //    is that it's an array contents. These are the data content/object that
    //    will be displayed in the table rows.
    //
    getContentObjects: function(content)
    {
        // our content is already an array of objects
        return content;
    },
    
    getRowDataId: function(rowData)
    {
        return "_table_row_for_template_" + rowData.id;
    },

    getRowDataName: function(rowData)
    {
        return Constants.getExtensionDisplayMessage(rowData.name);
    },
    
    // Summary:
    //    Returns the type of the cell to be displayed. By default, the TableViewer
    //    will support a set of cell types. The type of the cell will map to a specific
    //    cell renderer (to be discussed).
    getCellDisplayType: function(rowData, atColumnIndex)
    {
        return this.COLUMNS.ALL[atColumnIndex].type;
    },

    getCellAlignment: function(rowData, atColumnIndex)
    {
        return this.COLUMNS.ALL[atColumnIndex].alignment;
    },
    
    // Summary:
    //    Returns the display data of the cell. For cell renderer type of TEXT, this will
    //    be the string. Others cell render type would be Date, or integer value, etc. If the 
    //    cell renderer type is image, the client can also supply cell data that can be sortable/groupable. 
    //    The TableViewer's default sorter, filter operates on this returned data. For "complex" 
    //    cell types such as image where it could have multiple values for different states, the 
    //    client also needs to implement the next method.
    //    
    //    Client must implement this method to provide the cell data.
    //    
    getCellDisplayData: function(rowData, atColumnIndex)
    {
        var val = "data";
		var COLUMNS = this.COLUMNS;
        switch(COLUMNS.ALL[atColumnIndex].name) {
            case COLUMNS.TYPE.name: 
                val = EDITOR_MAP[rowData.editorId]; //"";
                break;
            case COLUMNS.NAME.name: 
                val = Constants.getExtensionDisplayMessage(rowData.name);
                break;
            case COLUMNS.DESCRIPTION.name: 
                val = Constants.getExtensionDisplayMessage(rowData.description) || "";             
                break;
            case COLUMNS.ISDEFAULT.name: 
                val = rowData.isDefault ? Constants.getMessage("labelYes") : "";             
                break;				
			case COLUMNS.MODIFIABLE.name: 
                val = rowData.modifiable ? Constants.getMessage("labelYes") : Constants.getMessage("labelNo");
                break;
        }
        return val;
    },

    
    // Summary:
    //    If applicable, returns the extended display information of the cell. The return extended
    //    info must be an object with the following attributes:
    //    
    //    {
    //      /* attribute common for all cell renderer types */
    //      "tooltip": <string>,
    //
    //      /* attributes for IMAGE and ACTION cell renderer types */
    //      "isHidden": <true, false> the image is hidden if the row is not on the mouse over
    //      "image": base image URL,  
    //      "mouseOverImage": image URL, (optional)
    //      "mouseOutImage": image URL,  (optional) (overrides "image")
    //
    //      /* attributes for SELECT cell renderer type */
    //      "select" : 'value', the selection in the combo
    //      "choices" : [{'text':'display text', 'value':'value'}, ] array of objects with 'text' and 'value' attributes
    //
    //      /* attributes for CHECKBOX cell renderer type */
    //      "state" : <true, false> initial state of the check box
    //
    //      /* attributes for LINK cell renderer type */
    //      "href" : <string> the href string for the link
    //    }
    //
    //    Client can implement this method to provide extended display infor for the cell.
    //    Certain cell renderer type will require the client to implement this method to
    //    provide needed display information, such as an image ACTION.
    //
    getCellDisplayExtendedInfo: function(rowData, atColumnIndex)
    {  
        var val = {};
        switch(this.COLUMNS.ALL[atColumnIndex].name) {
            case this.COLUMNS.TYPE.name: 
                val.image = ICON_MAP[rowData.editorId];;
                break;
            case this.COLUMNS.NAME.name:
			    val.onclick = dojo.hitch(this, "_editTemplate", rowData);
				break;
			case this.COLUMNS.DESCRIPTION.name:              
            case this.COLUMNS.MODIFIABLE.name:			    
                break;              
        }
        return val;
    },
	
	_editTemplate: function(template) {
		var editor = new TemplateEditor({
			template: template,
			onOK: this.onEdited
		});
	}
    
});

dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.table.TemplatesTableView", 
    [dijit._Widget, dijit._Templated],
{    
    templateString: 
					'<div class="com-ibm-rqm-templates-table-view" tabindex="-1">' +
                    '  <div dojoType="com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout" "noline"=true dojoAttachPoint="viewHeaderNode">' +
					'    <div dojoAttachPoint="_queryTableViewerNode" class="com-ibm-rqm-test-artifact-table-view" tabindex="-1"></div>' +
                    '  </div>' +
					'  <div dojoAttachPoint="_tableViewerNode" class="com-ibm-rqm-test-artifact-table-view" tabindex="-1"></div>' +
                    '</div>',
    editable: true,
    widgetsInTemplate: true,
	labelCurrentQuery: Constants.getMessage("labelCurrentQuery"),

    constructor: function() {
	 	this.provider = null;
		this.templates = null;		
		this.refreshCallback = null;
		this.archiveCallback = null;
	 },

    postCreate: function() 
    {
        this.inherited("postCreate", arguments);
		          
        //content and title				  
        var content = this.templates ? this.templates : [];
		
		this.viewHeaderNode.setCSHelpId("rqm_manage_templates.html");
		this.viewHeaderNode.setCSHelpButtonLabel(Constants.getMessage("labelTemplateViewHelp"));
		//set up table widget
        var tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer({
            allowAdjustSettings: true,
            autoSaveSettings: true,
            tableId: "RQM-TEMPLATE-TABLEVIEWER"                          
        });        
        this._tableViewerNode.appendChild(tableViewer.domNode);
		
		// the view is in a query page if there is a query name otherwise it is within
        // some container that don't need the header
        this.isQueryPage = this.queryName;
        
        if (this.isQueryPage) {
            // it is a query page so set header title and add table viewer as the header content
            this.viewHeaderNode.setTitle(this.queryName);
			this._tableViewerNode.style.display = "none";
            dojox.data.dom.removeChildren(this._queryTableViewerNode);
			this._queryTableViewerNode.appendChild(tableViewer.domNode);
        } else {
            // it is not a query page i.e. within a section without header
            this.viewHeaderNode.domNode.style.display = "none";
            dojox.data.dom.removeChildren(this._tableViewerNode);
			this._tableViewerNode.appendChild(tableViewer.domNode);
        }
				
        tableViewer.setContent(content);
		this.provider = new TemplatesTableView.TableProvider({
			onEdited: this.refreshCallback
		});
        tableViewer.setContentProvider(this.provider);
        tableViewer.setTableListener(new TemplatesTableView.TableListener());
        tableViewer.setCellModifier(new TemplatesTableView.CellModifier());
        tableViewer.setUnspecifiedGroupTitle(Constants.getMessage("labelUnassigned"));
        tableViewer.setTableRowsCheckable(true, /* isSingleSelect */ true);
		tableViewer.setTableSummary(Constants.getMessage("labelTableSummary", this.queryName ? this.queryName : Constants.getMessage("labelTemplate")));
		
        if (this.refreshCallback && dojo.isFunction(this.refreshCallback)) {
            tableViewer.addAction("refresh", Constants.getMessage("labelRefreshTemplates"), this, "refresh-icon-image", this.refreshCallback);
        }
		var actions = this.actions;
		var actionsLen = actions ? actions.length : 0;
        for (var i = 0; i < actionsLen; i++) {
            var action = actions[i];
            var button = tableViewer.addAction(action.id, action.label, null, action.iconClass, action.actionHandler);			
			if (action.requiresSelection) {
				button.setDisabled(true);
			}
			button.actionObject = action; //'button' is a widget
        }
        tableViewer.show();
        tableViewer.setEditable(this.editable);
		  
		this.tableViewer = tableViewer;
		  
    },
    
    getAction: function(id) {
        var actions = this.actions;
        var actionsLen = actions ? actions.length : 0;		
        for (var i = 0; i < actionsLen; i++) {
            if (actions[i].id == id) {
                return actions[i];
            }
        }
    },
    
    // API
    getSelected: function() {
        return this.tableViewer.getCheckedRows();
    },
     
    refresh: function(content) {
        if (content) {
            this.tableViewer.setContent(content);
		}
        this.tableViewer.refresh();   
    }
});

})();
