/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.tree.TestPlanTreeView");
 
dojo.require("dojox.data.dom");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Tree");
dojo.require("jazz.util.ResizeNotifier");

dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.tree.TestPlanTreeModel");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.horizontalTree.HorizontalTree");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.horizontalTree.ActionRegistry");

(function() {

var ImageCache     = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
var Utility        = com.ibm.asq.common.web.ui.internal.util.Utility;
var Constants      = com.ibm.rqm.planning.web.client.internal.Constants;
var TestPlanTreeModel = com.ibm.rqm.planning.web.ui.internal.view.tree.TestPlanTreeModel;
var HorizontalTree = com.ibm.asq.common.web.ui.internal.widgets.horizontalTree.HorizontalTree;

dojo.declare("Htree.ActionRegistry",
             com.ibm.asq.common.web.ui.internal.widgets.horizontalTree.ActionRegistry,
{   
    init: function()
    {
        this.contextAction = {
            actions: [{
                label: Constants.getMessage('labelOpenArtifact'),
                callback: dojo.hitch(this.parentView, "openItem")
            }]
        };
    },
    
    // API 
    getContextActions: function(model, item)
    {
        if (item.type == "testplan" || 
            item.type == "testcaseGroup" ||
            item.type == "requirementGroup" ||
            item.type == "testscriptGroup" ||
            item.type == "executionworkitemGroup" ||
            (item.artifact && item.artifact.enclosure)) {
            return this.contextAction;
        }
        return null;
    }
});

dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.tree.TestPlanTreeView", 
    [dijit._Widget, dijit._Templated],
{    
    templateString: '<div class="com-ibm-rqm-test-artifact-tree-view" tabindex="-1">' +
                    // attach our content node to containerNode as well, allowing our content to be destructable by destroyRecursive
                    '  <div dojoAttachPoint="_treeNode,containerNode" class="com-ibm-rqm-test-artifact-tree-view" tabindex="-1"></div>' +
                    '</div>',

	// variable used to keep track of unique identifier for tree nodes
    _uniqueId: 0,
    
    groups: null,  // provided in constructor args
    root: null,
    tree: null,
    isHtree: false,
    
    postCreate: function() 
    {
        this.inherited("postCreate", arguments);
    	this._initialize();
	},
	
	// load the tree with data
	_initialize: function() {
        
    	// create a new tree.
    	// The refresh function is sometimes spammed before the tree
    	// is actually ready to be displayed.  This causes some "flicker" in the
    	// tree viewer as an empty tree is drawn and then immediately replaced.
    	// In order to avoid this flicker the tree is only created when
    	// valid group data has been provided.
        if (this._hasGroups(this.groups)) {
			this.model = new TestPlanTreeModel({groups: this.groups});
			
            if (!this.isHtree) {
                this.tree = new dijit.Tree({
                    model: this.model,
                    getIconClass: dojo.hitch(this, this._getIconClass),
                    onClick: dojo.hitch(this, this._onClick),
                    onClose: dojo.hitch(this, this._onClose),
                    persist: false,
                    showRoot: true
                });
            }
            else {
                this.tree = new HorizontalTree({
                    model: this.model,
					allowAutoResize: false,
                    getIconClass: dojo.hitch(this, this._getIconClass),
                    nVisiblePanelsHint: 4,
                    actionRegistry: new Htree.ActionRegistry({parentView: this}),
                    width: "100%",
                    height: "400px"
                });
				// handle resizing the explorer when the window is resized. If
				// we allow the explorer to resize itself, this parent node would
				// allow a wider width. Have to constraint the width of the explorer
				// to the width of this container.
                this.resizeNotifierId = jazz.util.ResizeNotifier.register({context: this, callback: function(){
                    var contentBox = dojo.contentBox(this._treeNode);
                    if(contentBox.w > 0) {
                        contentBox.h = 400;
                        this.tree.resize(contentBox);
                    }
                }});
            }
			
	        // top level nodes are groups
	        if (this.groups[0].name != null) {
	        	// groups
		        for (var i=0; i<this.groups.length; i++) {
		        	var group = this.groups[i];
		        	var testplans = group.objects;
		        	var item = this._newItem(group, group.name, "group", this.root);
		        	// testplans
			        for (var j=0; j<testplans.length; j++) {
			        	var testplan = testplans[j];
			        	var child = this._newItem(testplan, testplan.name, "testplan", item);
			        }
			        
		        }
	        }
	        // no groups, top level nodes are test plans
	        else {
	        	var testplans = this.groups[0].objects;
		        for (var i=0; i<testplans.length; i++) {
		        	var testplan = testplans[i];
		        	var item = this._newItem(testplan, testplan.name, "testplan", this.root);
		        }
	        }
	        
            if(!this.isHtree)
               this.tree.rootNode.expand();
            else 
                this.tree.load();
			this._treeNode.appendChild(this.tree.domNode);
		}
		
    },

	_destroyUI: function() {
        dojox.data.dom.removeChildren(this._treeNode);
        if (this.tree!=null) {
            this.tree.destroy();
            this.tree=null;
        }
    	if (this.model!=null) {
	    	this.model.destroy();
	    	this.model=null;
    	}
        if (this.resizeNotifierId) {
            jazz.util.ResizeNotifier.unregister(this.resizeNotifierId);
            delete this.resizeNotifierId;
        }  
        if(this.groups)      
    	   delete this.groups;
	},
	
    refresh: function(groups) {
    	this._destroyUI();
        this.groups = groups;
    	this._initialize();      		
    }, 
	
    _newItem: function(artifact, name, type, parent, /*optional*/id) {
    	if (id==null) {
    		id = this._uniqueId++;
    	}
    	var tmp = {
    		artifact: artifact,
    		name: name,
    		type: type,
    		id: id,
    		children: [],
    		childrenLoaded: false
    	};
		var newItem = this.tree.model.newItem(tmp, parent);
    	return newItem;
    },

    // checks to see if the provided groups array
    // is ready for display.  it is ready for display
    // if it contains more than one group element or if
    // the first group element has more than one object.	    
    _hasGroups: function(groups) {
    	var hasGroups = false;
    	if (groups!=null && groups.length>0) {
    		if (groups.length>1) {
    			hasGroups = true;
    		} else if (groups[0].objects.length>0) {
    			hasGroups = true;
    		}
    	}
    	return hasGroups;
    },
    
    _iconMap: {
    	'project': 'project-icon-tree-16',
    	'testplan': 'testplan-icon-tree-16',
    	'testplanGroup': 'testplan-icon-tree-16',
    	'testcase': 'testcase-icon-tree-16',
    	'testcaseGroup': 'testcase-icon-tree-16',
    	'requirement': 'requirement-icon-tree-16',
    	'requirementGroup': 'requirement-icon-tree-16',
    	'testscript': 'testscript-icon-tree-16',
    	'remotescript': 'testscript-icon-tree-16',
    	'testscriptGroup': 'testscript-icon-tree-16',
    	'remotescriptGroup': 'testscript-icon-tree-16',
    	'manualscriptGroup': 'testscript-icon-tree-16',
    	'executionworkitem': 'ewi-icon-tree-16',
    	'executionworkitemGroup': 'ewi-icon-tree-16',
    	'executionresult': 'ewiresult-icon-tree-16',
    	'executionresultGroup': 'ewiresult-icon-tree-16',
    	'workitem': 'defect-icon-tree-16',
    	'workitemGroup': 'defect-icon-tree-16'
    },
    
	_getIconClass: function(item, opened)
	{
       	var iconClass;
       	
       	if (item == null) {
			iconClass = opened ? "dijitFolderOpened" : "dijitFolderClosed";
       	}
       	else if (this._iconMap[item.type]!=null) {
	       	iconClass = this._iconMap[item.type];
	    }
	    else {
			iconClass = opened ? "dijitFolderOpened" : "dijitFolderClosed";
		}
		return iconClass;
	},
	
    openItem: function(parentItem, item) {
        this._onClick(item, null);
    },
	
	// listen for tree node close.  remove the children
	// so refresh will be done next time it is opened
	_onClose: function(/* dojo.data */ item, /*TreeNode*/ node){
		
		// children of the root node (project) are provided
		// via the table query
		if (item.type=="project") {
			return;
		}
		
		// recursively destroy all the child nodes
		var children = node.getChildren();
		for (var i=0; i<children.length; i++) {
			var childNode = children[i];
			this._destroyNodeRecursive(childNode,0);
			node.removeChild(childNode);
		}
		
		// clear out the children
		item.children = [];
		item.mayHaveChildren=true;
		
		// reset the expando indicator since don't
		// know yet whether next query to the server
		// might return children
		node.isExpandable = true;
		node.state = "UNCHECKED";
	},
	
	// recursively destroy a tree node
	_destroyNodeRecursive: function(/*TreeNode*/ node, /*int*/ tooMuchRecursionCheck) {
		// safety check for too much recursion
		if (++tooMuchRecursionCheck > 100) {
			console.error("too much recursion while destroying tree nodes");
			return;
		}
		
		// destroy the node's children
		var children = node.getChildren();
		for (var i=0; i<children.length; i++) {
			var childNode = children[i];
			this._destroyNodeRecursive(childNode,tooMuchRecursionCheck);
			node.removeChild(childNode);
		}
		
		// destroy the node
		this.model.destroyRecursive(node.item);
		node.destroy();
	},
    
	// listen for tree node clicks
	_onClick: function(/* dojo.data */ item, /*TreeNode*/ node){
		if (item.type=="testplan") { // testplans came from the table view
			var hash = Utility.getTestPlanURI({id: item.artifact.id});
			window.location.hash = hash;
		}
		else if (item.type=="testcaseGroup") {
			var testplanId = item.parent.artifact.id;
			var hash = Utility.getTestPlanURI({id: testplanId});
			var hashWithSectionId = hash + "&sectionId=com.ibm.rqm.planning.editor.section.planTestCases"; 
			window.location.hash = hashWithSectionId;
		}
		else if (item.type=="requirementGroup") {
			var targetType = item.parent.type;
			if (targetType=="testplan") {
				var testplanId = item.parent.artifact.id;
				var hash = Utility.getTestPlanURI({id: testplanId});
				var hashWithSectionId = hash + "&sectionId=com.ibm.rqm.requirements.editor.section.planRequirements"; 
				window.location.hash = hashWithSectionId;
			} else if (targetType=="testcase") {
				var enclosure = item.parent.artifact.enclosure;
				var index = enclosure.indexOf('#');
				if (index>0) {
					var hash = enclosure.substring(index+1);
					var hashWithSectionId = hash + "&sectionId=com.ibm.rqm.requirements.editor.section.caseRequirements"; 
					window.location.hash = hashWithSectionId;
				}
			}
		}
		else if (item.type=="testscriptGroup") {
			var enclosure = item.parent.artifact.enclosure;
			var index = enclosure.indexOf('#');
			if (index>0) {
				var hash = enclosure.substring(index+1);
				var hashWithSectionId = hash + "&sectionId=com.ibm.rqm.planning.editor.section.testCaseScripts"; 
				window.location.hash = hashWithSectionId;
			}
		}
		else if (item.type=="remotescriptGroup") {
			var enclosure = item.parent.parent.artifact.enclosure;
			var index = enclosure.indexOf('#');
			if (index>0) {
				var hash = enclosure.substring(index+1);
				var hashWithSectionId = hash + "&sectionId=com.ibm.rqm.planning.editor.section.testCaseScripts"; 
				window.location.hash = hashWithSectionId;
			}
		}
		else if (item.type=="manualscriptGroup") {
			var enclosure = item.parent.parent.artifact.enclosure;
			var index = enclosure.indexOf('#');
			if (index>0) {
				var hash = enclosure.substring(index+1);
				var hashWithSectionId = hash + "&sectionId=com.ibm.rqm.planning.editor.section.testCaseScripts"; 
				window.location.hash = hashWithSectionId;
			}
		}
		else if (item.type=="executionworkitemGroup") {
			var enclosure = item.parent.artifact.enclosure;
			var index = enclosure.indexOf('#');
			if (index>0) {
				var hash = enclosure.substring(index+1);
				var hashWithSectionId = hash + "&sectionId=com.ibm.rqm.execution.editor.section.ewis"; 
				window.location.hash = hashWithSectionId;
			}
		}
		else if (item.artifact && item.artifact.enclosure) {
			var enclosure = item.artifact.enclosure;
			var index = enclosure.indexOf('#');
			if (index>0) {
				var hash = enclosure.substring(index+1);
				window.location.hash = hash;
			} else {
				window.location = enclosure;
			}
		}
		else if(node){
			/*   disabled node expansion to avoid confusing users
			     w.r.t. what action clicking a node will perform
			// toggle node expansion
			if (node.isExpanded) {
				this.tree._collapseNode(node);
			} else {
				this.tree._expandNode(node);
			}
			*/
		}
	},
    
    destroy: function(){
        this.inherited("destroy", arguments);
        
        if (this.resizeNotifierId) {
            jazz.util.ResizeNotifier.unregister(this.resizeNotifierId);
            delete this.resizeNotifierId;
        }
    },
	
    _error: function(e) {
        console.error(this.declaredClass + ": Error: " + e.message);
    }	
	
});

})();
