/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.process.web.ui.internal.view.viewlets.ReviewViewlet");

dojo.require("dojox.data.dom");

dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.util.GlobalEvents");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.table.WorkItemTableView");

dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.rqm.process.web.client.internal.approval.ProcessClient");
dojo.require("com.ibm.team.workitem.web.client.internal.QueryClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.QueryClient");
dojo.require("com.ibm.asq.common.web.ui.internal.view.viewlets.AbstractViewlet");

(function()
{
var Constants 					= com.ibm.rqm.planning.web.client.internal.Constants;
var ProcessClient				= com.ibm.rqm.process.web.client.internal.approval.ProcessClient;
var PlanClient					= com.ibm.rqm.planning.web.client.internal.QueryClient;

var Viewlet 					= com.ibm.team.dashboard.web.ui.Viewlet;
var DashboardConstants          = com.ibm.team.dashboard.web.ui.DashboardConstants;
var QueryClient                 = com.ibm.team.workitem.web.client.internal.QueryClient;
var ServiceResponseHandler 		= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;	
var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
var EditorUtil					= com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
var WorkItemTableView 			= com.ibm.rqm.planning.web.ui.internal.view.table.WorkItemTableView;
var GlobalEvents 				= com.ibm.asq.common.web.ui.internal.util.GlobalEvents;
var AbstractViewlet = 			com.ibm.asq.common.web.ui.internal.view.viewlets.AbstractViewlet;
	
dojo.declare("com.ibm.rqm.process.web.ui.internal.view.viewlets.ReviewViewlet",	AbstractViewlet,
{
	templateString: "<div dojoAttachPoint=\"_resultSection\"></div>",

	_actions: [ 
				"My open items",
				"Already approved for me", 
				"Already rejected for me",
				"Pending approvals for me"], 
	
	_isQuerying: false,
	
	/*
     * Called by the framework after the viewlet is instantiated, but before the
     * first refresh. Viewlets that need to initialize themselves shoudl override.
     */
	//@Override from com.ibm.team.dashboard.web.ui.Viewlet
    init: function() {
		this.inherited("init", arguments);
		this._projectArea = EditorUtil.getCurrentProjectArea();
		dojo.subscribe(GlobalEvents.DashboardFocus, this, "_refreshOnFocus");
		this._queries = [];
    },
	
	//@Override from com.ibm.team.dashboard.web.ui.Viewlet
	refresh: function() {
		console.debug(this.declaredClass + ": refresh --> entry.");
        this._showTable();		
		if (!this._queries || this._queries.length == 0) {
			this._getScopedQueries();
		} else {
			this._runQueries();
		}
	},
	
	//@Override from com.ibm.team.dashboard.web.ui.Viewlet
	update: function() {
		console.debug(this.declaredClass + ": update --> entry.");
		this._showTable(this._results);
	},
	
	_refreshOnFocus: function() {
		//TODO determine if a refresh *should* be performed
		//for now, always refresh
		this.refresh();  
	},
		
	_getScopedQueries: function() {
		var scope;
		switch (this.getScope()) {
			case DashboardConstants.SCOPE_CONTRIBUTOR:
				scope = QueryClient.SCOPE_PERSONAL;
				break;
			case DashboardConstants.SCOPE_TEAM_AREA:
				scope = QueryClient.SCOPE_TEAM_AREA;
				break;
			case DashboardConstants.SCOPE_PROJECT_AREA:
				scope = QueryClient.SCOPE_PROJECT_AREA;
				break;
		}
         
        this._showTable();		
		var responseHandler = new ServiceResponseHandler(this, "_setScopedQueries", "_error");
		QueryClient.getScopedQueries(responseHandler, scope, this.getScopeItem().itemId, true, true);		
	},
	
	_setScopedQueries: function(scopedQueriesDTO) {
		this._queries = new Array();
		var scopedQuery = null;
		if (scopedQueriesDTO && scopedQueriesDTO.length > 0) {
			for (var i=0; i<scopedQueriesDTO.length; i++) {
				if (this._projectArea.name == scopedQueriesDTO[i].scopeName)
					scopedQuery = scopedQueriesDTO[i];
			}
		}
		if (scopedQuery.queries && scopedQuery.queries.length > 0) {
			for (var i = 0; i < scopedQuery.queries.length; i++){
				var query = scopedQuery.queries[i];
				if(this._isInArray(this._actions, query.name))
					this._queries.push(query);
			}
		}
		this._runQueries();
  	},
	
	_runQueries: function() {
		if (this._isQuerying) {
			console.debug(this.declaredClass + ": _runQueries --> attempted to run query while a query was already running.");
			return;
		} else {
			this.getSite().setLoading(true);
			this._isQuerying = true;
		}
		
		var queryOps = new Array();
		for(var i = 0; i < this._queries.length; i++){
			var params = {
				jsonQuery:{
					itemId: this._queries[i].itemId
				},
				queryItemId: this._queries[i].itemId,
				name: this._queries[i].name,
				token: "",
				projectAreaItemId: this._projectArea.itemId
			}
			queryOps.push(params);
		}
		var param = dojo.toJson({querys: queryOps});
		
		var responseHandler = new ServiceResponseHandler(this, "_showResults", "_error");
		ProcessClient.postExecuteQuery(responseHandler, {operationJson: param});		
	},
	
	_showTable: function(summaries) {
		summaries = summaries || [];
		if (!this._tableViewer) {
			dojox.data.dom.removeChildren(this._resultSection);
			var itemCount = this.getPreference("items");
			this._tableViewer = new WorkItemTableView({
				summaries: summaries,
				numitems: itemCount,
				usePageRangeControl: false
			});
			this._resultSection.appendChild(this._tableViewer.domNode);
			this._resultSection.style.width = "100%"; // CML: hack: the table doesn't fill the viewlet otherwise, looks silly
		} else {
			this._tableViewer.refresh(summaries);
		} 		
	},
	
	_showResults: function(results) {
		results = results || [];
		this._isQuerying = false;
		
		this.getSite().setLoading(false);
		this.getSite().doneRefresh();
		
		//change workitem's summary to display what the user should do.  	
		for (var i = 0; i < results.length; i++) {
			var wiSummary = results[i].workItemSummary;
			if (results[i].linkTypeDTOs != null) {
				var linkType = results[i].linkTypeDTOs[0];
				var taskName = this._getTaskName(linkType);
				if (!taskName) {
					taskName = wiSummary.summary;
				}
				if (this._actions[0] != linkType.displayName) {
					//if the display name is not "My open items", this is an approval record
					//flag it as such so URI can point to the approvals ta directly
					wiSummary.isApproval = true;
				}
				wiSummary.summary = taskName;
			}
		}	
		this._results = results;		
        this._showTable(results);
	},
	
	_isInArray: function(array, item) {
		if(array != null && array.length > 0) {
			for(var i=0; i<array.length; i++){
				if(array[i]==item)
					return true;
			}
		}
		return false;
	},
		
	_getTaskName: function(linkType) {
		var name = null;
		if(linkType.displayName == this._actions[1]) {
			name = Constants.getMessage("labelApprovedReviewlet", linkType.linkDTOs[0].comment);
		}
		if(linkType.displayName == this._actions[2]) {
			name = Constants.getMessage("labelRejectedReviewlet", linkType.linkDTOs[0].comment);
		}
		if(linkType.displayName == this._actions[3]) {
			name = Constants.getMessage("labelApproveReviewReviewlet", linkType.linkDTOs[0].comment);
		}
		return name;
	},
		
	_error: function(errorObj){
        console.error(errorObj.message);
    }
});

})();
