
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
//

//
//
// Author: L. To
//
// Summary:
//

dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.machines.ActionCustomElementProvider");
 
dojo.require("com.ibm.asq.common.web.client.internal.Catalog");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

dojo.require("com.ibm.rational.test.lm.web.client.internal.RtlmInitializer");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.layout.SectionLayoutForm");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.layout.SectionHeader");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.machines.ArrayAttributeTitle");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.layout.SectionHeader");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.machines.DefaultCustomElementProvider");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.machines.TemplatedAssetContentProvider");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.ActionTitlePane");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.machines.DefaultAssetContentProvider");

(function() {

var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
var SectionHeader = com.ibm.rational.test.lm.web.ui.internal.widgets.layout.SectionHeader;
var DefaultAssetContentProvider = com.ibm.rational.test.lm.web.ui.internal.views.machines.DefaultAssetContentProvider;
var RtlmUiUtil_getMessage = RtlmUiUtil.getMessage;
var RtlmUiUtil_getCommonMessage = RtlmUiUtil.getCommonMessage;

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.machines.ActionCustomElementProvider", 
    [com.ibm.rational.test.lm.web.ui.internal.views.machines.DefaultCustomElementProvider],
{   
    constructor: function()
    {
        this.sectionCreatedState = {};
        this.subSectionCreatedState = {};
    },
    
    // Summary:
    //    Returns a widget that is the section header.
    //
    constructSectionHeader: function(sectionKey)
    {
        var sectionInfo = this.layoutForm.getContentProvider().getSectionInfo(this.layoutForm.getContent(), sectionKey);   
        var header = null;
        var isOpen = (typeof sectionKey.open != "undefined") ? sectionKey.open : true;
        
        // if we already have shown this section, this's not the initial section creation,
        // show it always.
        if(this.sectionCreatedState[sectionKey.id]) {
            isOpen = true;
        }
        
        // provide a help id for the section if one is defined in the template.
        var helpId = (sectionKey.contextHelpId) ? sectionKey.contextHelpId : (sectionKey.contextHelpIdNew) ? sectionKey.contextHelpIdNew : null;

        header = new com.ibm.rational.test.lm.web.ui.internal.widgets.layout.SectionHeader({
                title: RtlmUiUtil_getMessage(sectionInfo.title), 
                contextHelpId: helpId,
                csHelpProviderId: com.ibm.rational.test.lm.web.CSHelpID,
                open: isOpen,
                'sectionKey': sectionKey
            });
            
        this.sectionCreatedState[sectionKey.id] = header;
        
        var actions = sectionKey.actions;
        var actions_length = actions.length;
        for( var iAction = 0; iAction < actions_length; iAction++ )
        {
        	if(actions[ iAction ] == "add" )
            	header.addAction("add", dojo.hitch(this, "_handleAddAttributeValueFromSection"), sectionKey, "add-icon-image");
        }
        
        return header;
    },

    getSectionHeaderContentNodeName: function()
    {   
        return 'containerNode'; 
    },
    
    // Summary:
    //    Returns a widget that is the header for collection attributes.
    //
    constructCollectionAttributeHeader: function(sectionKey, attributeKey)
    {
        var isArrayVal = (attributeKey.type && attributeKey.type == DefaultAssetContentProvider.TYPE_ATTRIBUTE_MULTI_VALUE);
        
        // Create a sub-section for the attribute if it's declared in the section
        // header. But we need to handle the case where an attribute is an array
        // and it has sub-values with the same attribute name. Ensure that its
        // type is not a multi-value attribute before creating the header for it.
        // The condition (!isArrayVal).
        //
        // We probably should define that in the attribute key definition,
        //  i.e., to have a subsection or not and not decided here.
        
        if(typeof sectionKey.subsections != "undefined") {
            // check if the attribute is in the list of subsections
            // csHelpId as string (not just true) is returned if the subsection included
            // the CS Help id
            var csHelpId = this._inList(sectionKey.subsections, attributeKey.attributeName);
            
            // In addition, if the section can tolerate undeclared attributes, create the
            // the subsection too.
            if ((sectionKey.allowNonDeclaredSubsections || csHelpId) && !isArrayVal) {
                var isOpen = false;

                if(this.subSectionCreatedState[attributeKey.id]) {
                    isOpen = true;
                }
                // if the attribute doesn't have a translation for it, just use the 
                // untranslated name. This is a custom attribute.
                var sectTitle = RtlmUiUtil_getMessage(attributeKey.attributeName + "-section");
                if(sectTitle.indexOf("-section") > -1) {
                    sectTitle = attributeKey.attributeName;
                }
                var subsection_args = {
                    title: sectTitle,
                    open: isOpen
                };
                var subsection = new com.ibm.rational.test.lm.web.ui.internal.widgets.ActionTitlePane(subsection_args);
                
                // set subsection CS Help id if it is returned
                if (typeof csHelpId == 'string') {
                    this.setCollectionAttributeHeaderCSHelpId(subsection, csHelpId);
					var label = RtlmUiUtil_getMessage("labelHelp", RtlmUiUtil_getMessage(attributeKey.attributeName + "-section"), "");
					this.setCollectionAttributeHeaderCSHelpLabel(subsection, label);
                }
            
                var parentPart = attributeKey.parentPart;
                var attrName = attributeKey.attributeName;
                var attr = parentPart.getAttribute(attrName);
                var type = attr.getType();
            
                if (attr.isArray()) {
                    var label = RtlmUiUtil_getCommonMessage("add_button") + " " + RtlmUiUtil.getCatalogMessage(attributeKey.attributeName);
                    subsection.addTool(label, dojo.hitch(this, "_handleAddAttributeValue"), attributeKey, "add-icon-image");
                }
                this.subSectionCreatedState[attributeKey.id] = subsection;
                return subsection;
            }
        }
        
        // create a border pane for the attribute if it's declared in the section,
        // but only create the array value attribute. A collection attributes always
        // has a root and its values. Only declarate one. Otherwise, it would have two
        // sets of border pane, one for the outer root and a border pane for each of the
        // values.
        if (sectionKey.borderedCollectionAttributes && 
            this._inList(sectionKey.borderedCollectionAttributes, attributeKey.attributeName) && 
            isArrayVal) {
            var simpleBorderPane = new com.ibm.rational.test.lm.web.ui.internal.widgets.SimpleBorderPane({});
            return simpleBorderPane; 
        }
        // no special subsection. The layout form will provide an invisble
        // grouping for this container.        
        return null;
    },
    
    setCollectionAttributeHeaderCSHelpId: function(subsection, id)
    {
        subsection.setCSHelpId(id);
    },
    
	setCollectionAttributeHeaderCSHelpLabel: function(subsection, label)
    {
        subsection.setCSHelpButtonLabel(label);
    },
	
    _inList: function(list, value)
    {
        var list_length = list.length;
        for(var i = 0; i < list_length; i++) {
            // the subsection list entries may contain a CS help id
            // in the format of <title>?<CS help id>
            var csHelpPos = list[i].indexOf("?");
            var subsectionTitle = list[i];
            if (csHelpPos > 0) {
                subsectionTitle = list[i].substring(0, csHelpPos);
            }
            if (subsectionTitle == value) {
                if (csHelpPos > 0)
                    // return the CS help id, not just true, if there is one
                    return list[i].substring(csHelpPos + 1);
                else 
                    return true;
            }
        }
        return false;
    },
    
    _handleAddAttributeValueFromSection: function(sectionKey)
    {
        var groups = sectionKey.groups;
        var root = groups[0].root;
        var asset = this.layoutForm.getContent();
        var parentPart = this._getAttributeParent(asset, root);
        var attrName = root.slice(root.lastIndexOf(".") + 1);
        var attr = parentPart.getAttribute(attrName);
        var type = attr.getType();
        var value = "";
        
        if(sectionKey.actions[0].indexOf(".") > -1) {
            var subAttrName = sectionKey.actions[0].slice(sectionKey.actions[0].indexOf(".") + 1);
            return this._addSubAttributeToRoot(sectionKey, attr, subAttrName);
        }
        
        if (type == DefaultAssetContentProvider.TYPE_PART) {
            var rootPart = attr.getRootPart();
            value = rootPart.getName();
        }
        // Add a new empty value to the attribute
        // and invalidate the layout for this attribute
        // to pick up the new value.
        attr.addArrayValue(value);
        this.invalidateSectionLayout(sectionKey);
    },

    _addSubAttributeToRoot: function(sectionKey, parentAttr, attrName)
    {
        var parentPart;
        var attr;
        var type;
        var needUpdateOther = false;
        
        if (parentAttr.isArray()) {
            var values = parentAttr.getArrayValues();
            var parentPart;
            
            if (values && values.length > 0) {
                if (values.length == 1) {
                    parentPart = values[0];
                }
                else {
                    // bring up a dialog to ask the user which parent to add
                    // the sub-attribute to.
                    this._selectParentArrayValue(sectionKey, parentAttr, attrName, values);
                    return;
                }
            }
            else {
                // The parent attribute doesn't have any value,
                // auto add the root part to it.
                parentAttr.addArrayValue(parentAttr.getRootPart().getName());
                
                values = parentAttr.getArrayValues();
                parentPart = values[0];
                needUpdateOther = true;
            }
            attr = parentPart.getAttribute(attrName);
            var value = "";
            
            type = attr.getType();
            if (type == DefaultAssetContentProvider.TYPE_PART) {
                var rootPart = attr.getRootPart();
                value = rootPart.getName();
            }
            
            // Add a new empty value to the attribute
            // and invalidate the layout for this attribute
            // to pick up the new value.
            attr.addArrayValue(value);
            this.invalidateSectionLayout(sectionKey);
        }
        // we also changed the parent part. Update the other
        // section too to pickup the new value.
        if(needUpdateOther)
            this._updateOtherSectionAfterAction(sectionKey.updateOtherSections); 
    },
    
    _updateOtherSectionAfterAction: function(sections) 
    {
        if(sections) {
            var sections_length = sections.length;
            for(var i = 0; i < sections_length; i++) {
                this.invalidateSectionLayout(sections[i]);
            }
        }
    },

    _selectParentArrayValue: function(sectionKey, parentAttr, attrName, osValues) {
        this.osSelect = -1;
		var selectLabel = document.createElement("label");
		this.oslist = document.createElement("select");
        this.oslist.multiple = false;
        this.oslist.size = 6;
		
        this.oslist.className = "osSelect";
        this.connect(this.oslist, "onchange", "_choiceOSselected");
        
        var osValues_length = osValues.length;
        for(var i = 0; i < osValues_length; i++) {
            var an_option = new Option(osValues[i].getName());
            this.oslist.options[i] = an_option;
        }
        var callbackData = {};
        callbackData["sectionKey"] = sectionKey;
        callbackData["parentAttr"] = parentAttr;
        callbackData["attrName"] = attrName;
        callbackData["values"] = osValues;
        
        var args = {
            width: "30%",
            okCallback: dojo.hitch(this, "_selectOSConfirmed"),
            okCallbackData: callbackData
        };
        var message = document.createElement("div");
        message.className = "osSelectPrompt";
        message.innerHTML = RtlmUiUtil_getMessage("selectOSAddSWPrompt");

		selectLabel.appendChild(this.oslist);
        this.osSelectDialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(args);
        this.osSelectDialog.contentContainer.appendChild(message);
        this.osSelectDialog.contentContainer.appendChild(selectLabel);
        this.osSelectDialog.setTitle(RtlmUiUtil_getMessage("selectOSTitle"));
        this.osSelectDialog.open();
        
        return this.osSelect;
    },
    
    _selectOSConfirmed: function(callbackData)
    {
        if(this.osSelect < 0)
            return;
            
        var parentPart = callbackData.values[this.osSelect];
        var attr = parentPart.getAttribute(callbackData.attrName);
        var value = "";
        
        var type = attr.getType();
        if (type == DefaultAssetContentProvider.TYPE_PART) {
            var rootPart = attr.getRootPart();
            value = rootPart.getName();
        }
        
        // Add a new empty value to the attribute
        // and invalidate the layout for this attribute
        // to pick up the new value.
        attr.addArrayValue(value);
        
        this.invalidateSectionLayout(callbackData.sectionKey);
    },
    
    _choiceOSselected: function()
    {
        this.osSelect = -1;
        var options = this.oslist.options;
        var options_length = options.length;
        for(var i = 0; i < options_length; i++) {
            if(options[i].selected) {
                this.osSelect = i;
                break;
            }
        }
        this.osSelectDialog.getOkButton().disabled = (this.osSelect == -1);
    },
    
    _isAttributePathRoot: function(attrPath) 
    {
        return attrPath.indexOf(".") < 0;
    },
    
    _getAttributeParent: function(asset, attributePath)
    {
        var parentPos = attributePath.lastIndexOf(".");
        var parentPath = (parentPos > 0) ? attributePath.substr(0, parentPos) : attributePath;
        return this._getAttributeByPath(asset, parentPath);
    },
    
    _getAttributeByPath: function(asset, attrPath)
    {
        var part = asset.getRoot();
        var paths = attrPath.split(".");
        var i = 0;
        var attr;
        var paths_length = paths.length;
        
        while(i < paths_length - 1) {
            try {
                var attrName = paths[i + 1];
                
                attr = part.getAttribute(attrName);
                
                // we're not at the attribute we wanted yet
                // Ensure this is a valid part.
                if((i + 1) < paths_length) {
                    if(attr.getType() != DefaultAssetContentProvider.TYPE_PART) {
                        var err = new Error("Invalid attribute Path: " + attrPath);
                        throw err;
                    }
                    part = attr;
                }
                i++;
            }
            catch(error) {
                var err = new Error("Invalid attribute Path: " + attrPath);
                throw err;
            }
        }
        return part;
    },
    
    // Summary:
    //    convenient method to invalidate the layout of a section. This method
    //    tries to get the section's sub-attributes and invalid the sub-attributes
    //    content. If no section's attributes (very unlikely case), it will invalidate
    //    the entire section's layout.
    //
    invalidateSectionLayout: function(sectionKey) 
    {
        var rootAttrs = this.layoutForm.getSectionAttributes(sectionKey);
        
        if(rootAttrs) {
            var rootAttrs_length = rootAttrs.length;
            for(var i = 0; i < rootAttrs_length; i++) {
                this.layoutForm.invalidateAttributeLayout(rootAttrs[i]);
            }
        }
        else {
            this.layoutForm.invalidateSectionLayout(sectionKey);
        }
    }
});

})();
