
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
//

//
//
// Author: L. To
//
// Summary:
//
(function() {

dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.machines.AssetTemplateUtil");

dojo.require("com.ibm.asq.common.web.client.internal.option");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

var MACHINE_DETAIL_TEMPLATE = {
	assetType: "Machine",
	viewType: "details",
	needIsRequiredCheck: true,
    autoAddUnknownAttributes: true,
	
	sections: [
	    {
			id: "machine.details",
			title: "General Info",
            contextHelpIdNew: "rtlm_machine_sect_geninfo.html",
            contextHelpId: "rtlm_machine_sect_geninfo.html",
			nColumns: 2,
            open: false,
            
            // the general section is the catch all for custom attributes.
            allowNonDeclaredSubsections: true,
			subsections: ["RemoteConnectivity"],
			groups: [
			    {
					hidden: true,
					root: "Machine",
                    infoAttributes: ["Location", "Administrative Status"],
                    defaultDescription: "generalDesc",
					attributes: ["Machine","Location", "Administrative Status"],
                    excludeAttributes: ["model", "ipAddress","operatingsystem", "SoftwareInstall", "cpu", "L2Interface",
					  "PrimaryMACAddress", "SystemBuildUUID", "SystemID", "TotalDiskSize", "memory", "name",
					  "FreeDiskSpace", "Manufacturer", "ModelNumber", "SerialNumber", "Notes", "owner", 
                      "Description", "Operational Status", "IpInterface", "group"]
				}
			]
		},
	    {
			id: "machine.hardware",
			title: "Hardware",
            contextHelpIdNew: "rtlm_machine_sect_hw.html",
            contextHelpId: "rtlm_machine_sect_hw.html",
			nColumns: 2,
            open: false,
            subsections: ["cpu?rtlm_machine_sect_cpu.html","L2Interface?rtlm_machine_sect_l2i.html","IpInterface"],
			groups: [
			    {
					hidden: true,
					root: "Machine",
                    infoAttributes: ["Manufacturer", "ipAddress", "memory", "TotalDiskSize", "FreeDiskSpace"],
                    defaultDescription: "hardwareDesc",
					attributes: ["Manufacturer", "model", "ipAddress", "memory", "TotalDiskSize", "FreeDiskSpace",  
								 "PrimaryMACAddress", "SystemBuildUUID", "SystemID", "ModelNumber", 
                                 "SerialNumber", "Notes", "cpu", "L2Interface", "IpInterface"],
                    excludeAttributes: ["RTLM-ALL-ATTRIBUTES"]
				}
			]
		},
        {
            id: "os.details",
            title: "Operating System",
            contextHelpIdNew: "rtlm_machine_sect_os.html",
            contextHelpId: "rtlm_machine_sect_os.html",
            nColumns: 2,
            open: false,
            actions: ["add"],
            borderedCollectionAttributes: ["operatingsystem"],
            subsections: ["Version?rtlm_machine_sect_ver.html","SoftwarePatch"],
            groups: [
                {
                    hidden: false,
                    root: "Machine.operatingsystem",
                    infoAttributes: ["operatingsystem", "hostname"],
                    defaultDescription: "osDesc",
					attributes: ["operatingsystem","hostname"],
                    excludeAttributes: ["SoftwareInstall"] 
                }
            ]
        },
        {
            id: "software.details",
            title: "Software",
            contextHelpIdNew: "rtlm_machine_sect_sw.html",
            contextHelpId: "rtlm_machine_sect_sw.html",
            nColumns: 2,
            actions: ["add.SoftwareInstall"],
            updateOtherSections: ["os.details"],
            open: false,
            borderedCollectionAttributes: ["SoftwareInstall"],
            subsections: ["Version?rtlm_machine_sect_ver.html","SoftwarePatch"],
            groups: [
                {
                    hidden: false,
                    root: "Machine.operatingsystem",
                    rootQualifier: "sw",
                    rootIsReadOnly: true,
                    showOnlyWithValue: "SoftwareInstall",
                    defaultDescription: "swDesc",
                    infoAttributes: ["SoftwareInstall"],
                    attributes: ["SoftwareInstall","DisplayName","Description","username", "password","executableName"],
                    excludeAttributes: ["RTLM-ALL-ATTRIBUTES"] 
                }
            ]
        }
	]
};

var VIRTUALIMAGE_DETAIL_TEMPLATE = {
    assetType: "VirtualImage",
    viewType: "details",
    needIsRequiredCheck: true,
    autoAddUnknownAttributes: true,

    sections: [
        {
            id: "vm.details",
            title: "General Info",
            contextHelpIdNew: "rtlm_machine_sect_geninfo_vi.html",
            contextHelpId: "rtlm_machine_sect_geninfo_vi.html",
            open: false,
            nColumns: 2,
            
            // the general section is the catch all for custom attributes.
            allowNonDeclaredSubsections: true,
            subsections: ["ImageStorage", "Version?rtlm_machine_sect_ver.html"],
            groups: [
                {
                    hidden: true,
                    root: "VirtualImage",
                    infoAttributes: ['ImageType','CreatedWithVMTool'],
                    defaultDescription: "generalDesc",
                    attributes: ['ImageType', 
					             'CreatedWithVMTool', 'VMFile', 'Notes', 
								 'DateCreated', 'DateLastModified'],
                    excludeAttributes:["name", "Description", "owner", "operatingsystem", "group"]
                }
            ]
        },
        {
            id: "os.details",
            title: "Operating System",
            contextHelpIdNew: "rtlm_machine_sect_os.html",
            contextHelpId: "rtlm_machine_sect_os.html",
            open: false,
            nColumns: 2,
            borderedCollectionAttributes: ["operatingsystem"],
            subsections: ["Version?rtlm_machine_sect_ver.html","SoftwarePatch"],
            actions: ["add"],
            groups: [
                {
                    hidden: false,
                    root: "VirtualImage.operatingsystem",
                    defaultDescription: "osDesc",
                    infoAttributes: ["operatingsystem", "hostname"],
                    attributes: ["operatingsystem","hostname"],
                    excludeAttributes: ["SoftwareInstall"] 
                }
            ]
        },
        {
            id: "software.details",
            title: "Software",
            contextHelpIdNew: "rtlm_machine_sect_sw.html",
            contextHelpId: "rtlm_machine_sect_sw.html",
            nColumns: 2,
            actions: ["add.SoftwareInstall"],
            updateOtherSections: ["os.details"],
            open: false,
            borderedCollectionAttributes: ["SoftwareInstall"],
            subsections: ["Version?rtlm_machine_sect_ver.html","SoftwarePatch"],
            groups: [
                {
                    hidden: false,
                    root: "VirtualImage.operatingsystem",
                    rootQualifier: "sw",
                    rootIsReadOnly: true,
                    showOnlyWithValue: "SoftwareInstall",
                    defaultDescription: "swDesc",
                    infoAttributes: ["SoftwareInstall"],
                    attributes: ["SoftwareInstall","DisplayName","Description","username", "password","executableName"],
                    excludeAttributes: ["RTLM-ALL-ATTRIBUTES"] 
                }
            ]
        }
    ]
};

/***
 * NOT USED
 */
var MACHINE_SEARCH_TEMPLATE = {
    assetType: "Machine",
    viewType: "search",
    autoAddAttributeWithValue: true,
    autoAddUnknownAttributes: true,
	
    sections: [
        {
            id: "machine.details",
            title: "Machine",
            contextHelpId: "rtlm_search_sect_machine.html",
            nColumns: 2,
			open: false,
            actions: ["edit"],
            groups: [
                {
                    hidden: true,
                    root: "Machine",
                    infoAttributes: ["Manufacturer","Operational Status"],
                    summaryAttribute: ["Description"],
                    attributes: ["Manufacturer", "Operational Status"],
					hiddenAttributes:["name", "Machine", "Description", "owner", "Location", "ipAddress", 
                                      "Administrative Status"],
                    excludeAttributes:["operatingsystem", "SoftwareInstall", "PrimaryMACAddress", "SystemBuildUUID", "SystemID", 
								 "ModelNumber", "TotalDiskSize", "FreeDiskSpace", "memory",
                                 "SerialNumber", "Notes", "cpu", "L2Interface", "model"]
                }
            ]
        },
	    {
			id: "machine.hardware",
			title: "Hardware",
            contextHelpId: "rtlm_search_sect_hw.html",
			nColumns: 2,
			open: false,
            actions: ["edit"],
            subsections: ["cpu","L2Interface"],
			groups: [
			    {
					hidden: true,
					root: "Machine",
                    infoAttributes: ["memory", "FreeDiskSpace"],
					attributes: ["memory", "FreeDiskSpace"],
                    hiddenAttributes:["PrimaryMACAddress", "SystemBuildUUID", "SystemID", 
								 "Manufacturer", "ModelNumber", "TotalDiskSize", 
                                 "SerialNumber", "Notes", "cpu", "L2Interface", "model"],
                    excludeAttributes: ["RTLM-ALL-ATTRIBUTES"]
				}
			]
		},
        {
            id: "os.details",
            title: "Operating System",
            contextHelpId: "rtlm_search_sect_os.html",
            nColumns: 2,
			open: false,
            actions: ["add", "edit"],
            groups: [
                {
                    hidden: false,
                    root: "Machine.operatingsystem",
                    infoAttributes: ["operatingsystem", "hostname"],
                    attributes: ["operatingsystem","hostname"],
                    hiddenAttributes: []
               }
            ]
        },
        {
            id: "software.details",
            title: "Software",
            contextHelpId: "rtlm_search_sect_sw.html",
            nColumns: 2,
			open: false,
            actions: ["add", "edit"],
            groups: [
                {
                    hidden: false,
                    root: "Machine.SoftwareInstall",
                    infoAttributes: ["SoftwareInstall"],
                    attributes: ["SoftwareInstall","executableName"],                   
                    hiddenAttributes: []
                }
            ]
        }
    ]
};

/***
 * NOT USED
 */
var MACHINE_SEARCH_CONFIG_TEMPLATE = {
    assetType: "Machine",
    viewType: "configuration",
    autoAddAttributeWithValue: true,
    autoAddUnknownAttributes: true,

    sections: [
        {
            id: "machine.details",
            title: "Machine",
            contextHelpIdNew: "rtlm_search_sect_machine.html",
            nColumns: 2,
            open: false,
			actions: ["edit"],
            groups: [
                {
                    hidden: true,
                    root: "Machine",
                    infoAttributes: ["Manufacturer","Operational Status"],
                    summaryAttribute: ["Description"],
                    attributes: ["Manufacturer", "Operational Status"],
					hiddenAttributes: ["name", "Machine", "Description", "owner", "Location", "ipAddress", 
                                 "Administrative Status"],
                    excludeAttributes: ["operatingsystem", "SoftwareInstall", "cpu", "L2Interface",
					  "PrimaryMACAddress", "SystemBuildUUID", "SystemID", "TotalDiskSize", "memory",
					  "FreeDiskSpace", "model", "ModelNumber", "SerialNumber", "Notes"]
                }
            ]
        },
	    {
			id: "machine.hardware",
			title: "Hardware",
            contextHelpIdNew: "rtlm_search_sect_hw.html",
			nColumns: 2,
            open: false,
			actions: ["edit"],
            subsections: ["cpu","L2Interface"],
			groups: [
			    {
					hidden: true,
					root: "Machine",
                    infoAttributes: ["memory", "FreeDiskSpace"],
					attributes: ["memory", "FreeDiskSpace"],
                    hiddenAttributes:["PrimaryMACAddress", "SystemBuildUUID", "SystemID", 
								 "Manufacturer", "ModelNumber", "TotalDiskSize", 
                                 "SerialNumber", "Notes", "cpu", "L2Interface", "model"],
                    excludeAttributes: ["RTLM-ALL-ATTRIBUTES"]
				}
			]
		},
        {
            id: "os.details",
            title: "Operating System",
            contextHelpIdNew: "rtlm_search_sect_os.html",
            nColumns: 2,
            open: false,
            actions: ["add", "edit"],
            subsections: ["Version"],
            groups: [
                {
                    hidden: false,
                    root: "Machine.operatingsystem",
                    infoAttributes: ["operatingsystem", "hostname"],
                    attributes: ["operatingsystem","hostname"],
                    hiddenAttributes: []
               }
            ]
        },
        {
            id: "software.details",
            title: "Software",
            contextHelpIdNew: "rtlm_search_sect_sw.html",
            nColumns: 2,
            open: false,
            actions: ["add", "edit"],
            subsections: ["Version"],
            groups: [
                {
                    hidden: false,
                    root: "Machine.SoftwareInstall",
                    infoAttributes: ["SoftwareInstall"],
                    attributes: ["SoftwareInstall","executableName"],                   
                    hiddenAttributes: []
                }
            ]
        }
    ]
};

/***
 * NOT USED
 */
var VIRTUALIMAGE_SEARCH_TEMPLATE = {
    assetType: "VirtualImage",
    viewType: "search",
    autoAddAttributeWithValue: true,
    autoAddUnknownAttributes: true,

    sections: [
        {
            id: "vm.details",
            title: "Virtual Image",
            contextHelpId: "rtlm_search_sect_virtual_machine.html",
            nColumns: 2,
            open: false,
            actions: ["edit"],
            groups: [
                {
                    hidden: true,
                    root: "VirtualImage",
                    infoAttributes: ['ImageType'],
                    attributes: ['ImageType'],
                    hiddenAttributes:['name','CreatedWithVMTool', 'VMFile', 'Notes', 
                                      'Description', 'DateCreated', 'DateLastModified', 'owner'],
                    excludeAttributes:["operatingsystem", "SoftwareInstall"]
                }
            ]
        },
        {
            id: "os.details",
            title: "Operating System",
            contextHelpId: "rtlm_search_sect_os.html",
            nColumns: 2,
            open: false,
            actions: ["add", "edit"],
            groups: [
                {
                    hidden: false,
                    root: "VirtualImage.operatingsystem",
                    infoAttributes: ["operatingsystem", "hostname"],
                    attributes: ["operatingsystem","hostname"],
                    hiddenAttributes: []
                }
            ]
        },
        {
            id: "software.details",
            title: "Software",
            contextHelpId: "rtlm_search_sect_sw.html",
            nColumns: 2,
            open: false,
            actions: ["add", "edit"],
            groups: [
                {
                    hidden: false,
                    root: "VirtualImage.SoftwareInstall",
                    infoAttributes: ["SoftwareInstall"],
                    attributes: ["SoftwareInstall","executableName"],                   
                    hiddenAttributes: []
                }
            ]
        }
    ]
};

/***
 * NOT USED
 */
var DEFAULT_SEARCH_TEMPLATES = {
    'Machine' : MACHINE_SEARCH_TEMPLATE,
    'VirtualImage' : VIRTUALIMAGE_SEARCH_TEMPLATE
}

var DEFAULT_DETAIL_TEMPLATES = {
	'Machine' : MACHINE_DETAIL_TEMPLATE,
	'VirtualImage' : VIRTUALIMAGE_DETAIL_TEMPLATE
};

com.ibm.rational.test.lm.web.ui.internal.views.machines.AssetTemplateUtil = {
    
	initializeTemplates: function( secondaryMethodObject, secondaryMethodName )
	{
		var options = new com.ibm.asq.common.web.client.internal.option();
        var virtualAttr = {
			projectAreaItemId: com.ibm.team.process.web.projectArea.itemId,
			category: "USER-VirtualImage-VirtualImageSearchTemplate",
			userId: com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR.userId
		};
			   
        var machineAttr = {
			projectAreaItemId: com.ibm.team.process.web.projectArea.itemId,
			category: "USER-Machine-MachineSearchTemplate",
			userId: com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR.userId
		};
					   
		var virtualHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper
		(
			{	"outerthis": this,
				_success: function( option )
				{
					if( option && option.length == 1 )
					{
						VIRTUALIMAGE_SEARCH_TEMPLATE = dojo.eval( '(' + option[0].value + ')' );
						VIRTUALIMAGE_SEARCH_TEMPLATE.itemId = option[0].itemId;
						VIRTUALIMAGE_SEARCH_TEMPLATE.stateId = option[0].stateId;
						DEFAULT_SEARCH_TEMPLATES[ "VirtualImage" ] = VIRTUALIMAGE_SEARCH_TEMPLATE;
					}
					
					secondaryMethodObject[ secondaryMethodName ]();
				},
				_failure: function( errObj )
				{
					secondaryMethodObject[ secondaryMethodName ]();
				}
			},
			"_success",
			"_failure"
		);
		
		var machineHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper
		(
			{	"outerthis": this,
				_success: function( option )
				{
					if( option && option.length == 1 )
					{
						MACHINE_SEARCH_TEMPLATE = dojo.eval( '(' + option[0].value + ')' );
						MACHINE_SEARCH_TEMPLATE.itemId = option[0].itemId;
						MACHINE_SEARCH_TEMPLATE.stateId = option[0].stateId;
						DEFAULT_SEARCH_TEMPLATES[ "Machine" ] = MACHINE_SEARCH_TEMPLATE;
					}
					
					options.search( virtualHandler, virtualAttr );
				},
				_failure: function( errObj )
				{
                    // this call is disabled as it no longer matched the user options service
					//options._search( virtualHandler, virtualAttr );					
				}
			},
			"_success",
			"_failure"
		);
				
		//options._search( machineHandler, machineAttr );
	},
	
	// Summary:
	//    Returns the asset's detail view template. The template is use to
	//    construct the asset's view display.
	//
    getAssetDetailsTemplate: function(assetType)
    {
		// TODO: use the local templates for now. When we have
		// the B/E service, we'll get it from there...
		// Or we should load the templates during our appl initialization to
		// avoid the async-part..
		//
		var template = DEFAULT_DETAIL_TEMPLATES;
		if(!DEFAULT_DETAIL_TEMPLATES[assetType]) {
            // Map the subtype to its root's template.
            var map = {
                "PhysicalMachine" : "Machine",
                "VirtualMachine" : "Machine"
            }
            var mappedType = map[assetType];
            if (mappedType && DEFAULT_DETAIL_TEMPLATES[mappedType]) {
                assetType = mappedType;
            }
            else {
                throw new Error("The asset type is not known. Cannot get detail template for it: " + assetType);
            }
		}
        return DEFAULT_DETAIL_TEMPLATES[assetType];
    },

    // Summary:
    //    Returns the asset's search view template. The template is use to
    //    construct the asset's search view display.
    //
    getAdvancedSearchTemplate: function(assetType)
	{
        if(!DEFAULT_SEARCH_TEMPLATES[assetType]) {
            throw new Error("The asset type is not known. Cannot get search template for it: " + assetType);
        }
        return DEFAULT_SEARCH_TEMPLATES[assetType];
	},

    // Summary:
    //    Returns the asset's configuration template. The template is use to
    //    construct the asset's configuration view display.
    //
    getConfigurationTemplate: function(assetType)
    {
        // TODO: use the local templates for now. When we have
        // the B/E service, we'll get it from there...
        // Or we should load the templates during our appl initialization to
        // avoid the async-part..
        //
        return MACHINE_SEARCH_CONFIG_TEMPLATE;      
    },
	
    // Summary:
    //    Saves the template to our B/E service. This allows a per-user preference
	//    of the asset's form.
    //
	saveTemplate: function(template)
	{
		var templateJson = dojo.toJson(template);
		var options = new com.ibm.asq.common.web.client.internal.option();
        var attributes = { projectAreaItemId: com.ibm.team.process.web.projectArea.itemId,
                       category: "USER-" + template.assetType + "-" + template.assetType + "SearchTemplate",
                       userId: com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR.userId,
                       value: templateJson };
					   
		var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper
		(
			{	
				"template": template,
				_success: function( option )
				{
					if( option && option.length == 1 )
					{
						attributes.optionUuid = option[0].itemId;
						attributes.stateId = option[0].stateId;
						var handler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper
						(
							{
								"template": this.template,
								_success: function( option )
								{
									if( option )
									{
										this.template.itemId = option.itemId;
										this.template.stateId = option.stateId;
									}
								},
								_failure: function( errObj )
								{
								}
							}, 
							"_success",
							"_failure"
						);
						
						// this call is disabled as it no longer matched the user options service
						//options._save( handler, attributes );
					}
					else
					{
						var handler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper
						(
							{
								"template": this.template,
								_success: function( option )
								{
									if( option )
									{
										this.template.itemId = option.itemId;
										this.template.stateId = option.stateId;
									}
								},
								_failure: function( errObj )
								{
								}
							}, 
							"_success",
							"_failure"
						);
                        // this call is disabled as it no longer matched the user options service
						//options._save( handler, attributes );
					}
				},
				_failure: function( errObj )
				{
				}
			},
			"_success",
			"_failure"
		);
		
        // this call is disabled as it no longer matched the user options service
		//options._search( responseHandler, attributes );		
	}
};

})();
