/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
//

//
//
// Author: L. To
//
// Summary:
//



dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.machines.DefaultAssetContentProvider");
 
dojo.require("com.ibm.asq.common.web.client.internal.Catalog");
dojo.require("com.ibm.rational.test.lm.web.client.internal.RtlmInitializer");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.layout.SectionLayoutForm");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.pages.GroupUtil");

(function() {
var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
var RtlmUiUtil_getMessage = RtlmUiUtil.getMessage;

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.machines.DefaultAssetContentProvider", 
    [com.ibm.rational.test.lm.web.ui.internal.widgets.layout.SectionLayoutForm.ContentProvider,
	 com.ibm.rational.test.lm.web.ui.internal.widgets.layout.FormListener],
{   
	layoutForm: null,
	showPartTitle: false,
	showArrayAttributeTitle: true,
	attributesToShow: {},
	chainedListener: null,
	
    constructor: function(layoutForm, /* optional */ formListener)
	{
		this.layoutForm = layoutForm;
		this.layoutForm.setListener(this);
		this.chainedListener = formListener;
		this.attributesToShow = {};
	},
	
	// Summary:
	//    Indicate to the provider to show the attribute's title if it's a PART type.
	//    PART attributes can contain other attributes. If this is set to true, a title
	//    is created for the collection of attributes that is under a PART attribute. By
	//    default, PART type attribute title are not shown.
	//
	setShowPartTypeTitle: function(show)
	{
		this.showPartTitle = show; 
	},
	
    // Summary:
    //    Indicate to the provider to show the attribute's title if it's an array type.
    //    Array attributes holds mulitple values for a single attribute. If this is set to true, a title
    //    is created for the collection of values for array attributes. By default, array attribute
	//    title are set to show.
	//
	//    NOTE: Please note that array attributes are most likely a PART attribute as well. Subclass of this 
	//    provider should only enable one title display. 
    //
    setShowArrayAttributeTitle: function(show)
    {
        this.showArrayAttributeTitle = show; 
    },

    // Summary:
    //    Returns the layout information about the form. The following is the info format:
    //    {
    //         nColumns: integer > 0,  the number of sections across the form layout. 
    //         tileDirection: value,   SectionLayoutForm.TILE_VERTICAL, or SectionLayoutForm.TILE_HORIZONTAL
    //         sections: ["key",...]   the array of section keys** (please see below)
    //    }
    //    If the tiling direction is vertical,
    //    the sections obtained from the "getSections()" method are filled from
    //    column 1 first. Otherwise, for horizontal tiling, the sections are filled from
    //    row 1 first.
    //
    //    ** The section keys can be a simple string tag, or an opique object. If it's an opique
    //    object, it must have an id attribute, like key.id that is a string value. The key (or id) must be
    //    unique amoung section ids. The section's key (id) can then be used to reference and access the
    //    section layout.
    //
    getFormInfo: function(/* the content object */content) 
    {   
        var sectionKeys = ["Details"];
        var info = {nColumns: 1, tileDirection: SectionLayoutForm.TILE_VERTICAL, sections: sectionKeys};
        return info;
    },
    
    // Summary:
    //    Returns information about a section given its key value. The following is the info format:
    //    {
    //         title: "string",          the title for the section
    //         nColumns: integer > 0,    the number of columns of attributes across the section. 
    //         attributes: ["key",...]   the array of attribute keys** (please see below)
    //    }
    //
    //    ** The attribute keys can be a simple string tag, or an opique object. If it's an opique
    //    object, it must have an id attribute, like key.id that is a string value. The key (or id) must be
    //    unique amoung attribute ids. The attribute's key (id) can then be used to reference and access the
    //    attribute layout.
    //
    getSectionInfo: function(/* the content object */content, sectionKey)
    {
		// we only have one section, get the asset's attributes.
        var assetRoot = content.getRoot();
        var attributeKeys = this._createAttributeList(assetRoot);
		var sectionInfo = {title: sectionKey, nColumns: 2, attributes: attributeKeys};
		
		return sectionInfo;
    },
    
    // Summary:
    //    Returns information about the attribute given its key value. The attribute info is not static for
    //    all attributes being described. Certain attribute types will have additional data 
    //    associated with them.
    //
    //    The following is the info format:
    //    {
    //         /* 
    //          * Required Info Data 
    //          */
    //         type: value,              the type of attribute, one of the SectionLayoutForm.ATTRIBUTE_*
    //
    //         /* 
    //          * Optional Info Data (NOT YET IMPLEMENTED)
    //          */
    //         colSpan: <1..section.nCol> Optionally overrides the parent section layout's nCols attribute.
    //                                    By default, the attribute will occupy one column of the section. This
    //                                    allows the attribute to occupy more then one column in the section.
    //                                    For attribute type ATTRIBUTE_COLLECTION, the default is section.nCols. 
    //                                    It occupies the entire row in the section.
    //
    //         /* 
    //          * Common Info Data for "atomic" attribute types
    //          */
    //         title: "string",          the title for the attribute
    //         readOnly: <true,false>,  
    //
    //         /* 
    //          * Specific Info Data for the SectionLayoutForm.ATTRIBUTE_COLLECTION type
    //          */
    //         collection: [keys...],    the array of sub-attribute keys** (please see below)
    //         title: "string",          Optional, the title for the collection
    //
    //         /* 
    //          * Specific Info Data for the SectionLayoutForm.ATTRIBUTE_ENUM type
    //          */
    //         enumChoices:[values...]
    //    }
    //
    //    ** The attribute keys can be a simple string tag, or an opique object. If it's an opique
    //    object, it must have an id attribute, like key.id that is a string value. The key (or id) must be
    //    unique amoung ALL attribute ids. The attribute's id can then be used to reference and access the
    //    attribute layout.
    //
    getAttributeInfo: function(/* the content object */content, sectionKey, attributeKey)
    {
		var parentPart = attributeKey.parentPart;
		var attrName = attributeKey.attributeName;
        var attr = parentPart.getAttribute(attrName);
        var attrInfo = {};
		
		// handle the special pseudo attribute type for the part selection
		if(attributeKey["type"] && attributeKey["type"] == DefaultAssetContentProvider.TYPE_PART_SELECTOR) {
			return this._constructPartTypeSelectorAttrInfo(attributeKey, parentPart, attr);
        }

        // handle multi-value attribute intances. Create the 
        // concrete attribute info for an array value in the attribute.
        if(attributeKey["type"] && attributeKey["type"] == DefaultAssetContentProvider.TYPE_ATTRIBUTE_MULTI_VALUE) {
            attrInfo = this._constructAttributeInfo(sectionKey, attributeKey, parentPart, attr);
            attrInfo["canDelete"] = true;
            return attrInfo;
        }
       
        var isMultiValueCollection = attr.isArray();
        
		// Create a collection to hold the array values for this attribute.
        if(isMultiValueCollection) {
			return this._constructMultiValueAttrInfo(sectionKey, attributeKey, parentPart, attr);
        }
        
        return this._constructAttributeInfo(sectionKey, attributeKey, parentPart, attr);
	   
    },
	
	_constructPartTypeSelectorAttrInfo: function(attributeKey, parentPart, attr)
	{
        var attrInfo = {};
		var title = RtlmUiUtil_getMessage(attr.getName() + "Selector");
		
        attrInfo["type"] = SectionLayoutForm.ATTRIBUTE_ENUM;
        attrInfo["title"] = title;
        attrInfo["enumChoices"] = attr.getAllPartTypeNames();
        attrInfo["defaultSelection"] = attributeKey.selectedPartName;
        attrInfo["readOnly"] = false;
        
        // always create an empty choice
        attrInfo["createFirstEmpty"] = true;
        return attrInfo;
	},
	
	_constructMultiValueAttrInfo: function(sectionKey, attributeKey, parentPart, attr)
	{
        var attrInfo = {};
        var attrName = attributeKey.attributeName;
        var title = (this.showArrayAttributeTitle) ? RtlmUiUtil_getMessage(attrName) : null;
        
        attrInfo["type"] = SectionLayoutForm.ATTRIBUTE_COLLECTION;
        attrInfo["collection"] = this._createMultiValueList(sectionKey, parentPart, attr);
        attrInfo["title"] = title;
        attrInfo["readOnly"] = false;
        return attrInfo;
	},
	
    _constructAttributeInfo: function(sectionkey, attributeKey, parentPart, attr) 
    {
        var attrName = attr.getName();
		var type = attr.getType();
		var subAttributeNames = [];
		var layoutType = SectionLayoutForm.ATTRIBUTE_LABEL;
        var attrInfo = {};
		var showTitle = true;
		
		if(type == DefaultAssetContentProvider.TYPE_PART) {
			var part;
			var selectedPartName = null;
			var subAttrs = [];
            var index = attributeKey["index"];
			
			if(attr.isArray()) {
				var values = attr.getArrayValues();
				part = values[index];
			}
			else {
				part = attr.getValue();
			}
			
			// an instance part is set. Grab its value.
		    if(part) {
				selectedPartName = part.getName();
			}
			else {
				// there's no value set in the attribute, ask its parent
				// to create a temporary root part for now. We'll show
				// the root part's attributes. 
				part = parentPart.getAttributeValue(attrName, true);
			}
			
			// create a pseudo sub-attribute for this PART attribute. It's
			// an ENUM attribute that shows the part's instance types. This
			// allows the user to select the instance types of the part
			//
			var index_val = (index != "undefined") ? ("." + index) : "";
			//var uniqueId = attrName + "." + "partSelector" + index_val;
            var uniqueId = parentPart.getAttribute( attrName ).getPathName() + "-id." + index_val;

			var partSelAttr = {};
			partSelAttr["id"] = uniqueId;
            partSelAttr["type"] = DefaultAssetContentProvider.TYPE_PART_SELECTOR;
            partSelAttr["parentKey"] = attributeKey;
            partSelAttr["parentPart"] = parentPart;
            partSelAttr["attributeName"] = attrName;
            partSelAttr["selectedPart"] = selectedPartName;
			
            if(attr.isArray()) {
                partSelAttr["index"] = index;
            }
				           
            var subAttrs = null;
			
			// Use the specific part instance, or the root part to get its attributes.
			// By default, we'll always show the root part's attribute even if no instance
			// was specified.
			if(part) {
				subAttrs = this._createAttributeList(part);
				subAttrs.unshift(partSelAttr);				
			}
			
            var subAttrs_length = subAttrs.length;
			for( var iSub = 0; iSub < subAttrs_length; iSub++ )
			{
				if( !this.isAttributeVisible( sectionkey, subAttrs[ iSub ].id ) )
				{
					var before = ( iSub > 0 ) ? subAttrs.slice( 0, iSub ) : [];
                    var before_length = before.length;
					var after = ( iSub + 1 < subAttrs_length ) ? subAttrs.slice( iSub + 1 ) : [];
                    var after_length = after.length;
					subAttrs = [];
					for( var iJoin = 0; iJoin < before_length; iJoin++ )
					{
						subAttrs.push( before[ iJoin ] );
					}
					
					for( var iJoin = 0; iJoin < after_length; iJoin++ )
					{
						subAttrs.push( after[ iJoin ] );
					}
					
					iSub--;
				}				
			}

	        layoutType = SectionLayoutForm.ATTRIBUTE_COLLECTION;
		    attrInfo["collection"] = subAttrs;
			
			// The CPU attribute is a PART with only no additional attributes
			// Make it to occupy the a single cell. (not span across the row)
			if(attrName == "cpu") {
				attrInfo["colSpan"] = 1;
                attrInfo["nColumns"] = 1;
			}
			
			// turn on/off title based on the provider setting
			showTitle = this.showPartTitle;
		}
        else if(attr.isEnum()) {
            layoutType = SectionLayoutForm.ATTRIBUTE_ENUM;
            attrInfo["enumChoices"] = attr.getEnumValues();
			
			// always create an empty choice
			attrInfo["createFirstEmpty"] = true;
        }
        else if(type == DefaultAssetContentProvider.TYPE_STRING) {
			// special case to create an IP field 
			// for the ip address attribute.
			if(attrName == "ipAddress") {
				layoutType = SectionLayoutForm.ATTRIBUTE_TEXT_IP;
			}
			else {
                layoutType = SectionLayoutForm.ATTRIBUTE_TEXT;				
			}
        }
        else if(type == DefaultAssetContentProvider.TYPE_NUMBER) {
            layoutType = SectionLayoutForm.ATTRIBUTE_TEXT_NUMERIC;
        }
        else if(type == DefaultAssetContentProvider.TYPE_POINTER) {
			// Special case for the owner and group attribute. We'll
			// have to generate the name list for its dropdown.
			if(attrName == "owner") {
                layoutType = SectionLayoutForm.ATTRIBUTE_ENUM;
				var userNames = com.ibm.rational.test.lm.web.client.internal.RtlmInitializer.getUserNames();
                attrInfo["enumChoices"] = userNames;
            
			    // always create an empty choice
                attrInfo["createFirstEmpty"] = true;
			}
            else if(attrName == "group") {
                layoutType = SectionLayoutForm.ATTRIBUTE_ENUM;
                var groupNames = GroupUtil.getAllGroupNames();
                attrInfo["enumChoices"] = groupNames;
            
                // always create an empty choice
                attrInfo["createFirstEmpty"] = true;
            }
        }
        attrInfo["title"] = (showTitle) ? RtlmUiUtil_getMessage(attrName) : null;
        attrInfo["type"] = layoutType;
        attrInfo["readOnly"] = false;
	
        return attrInfo;

    },
    
    // Summary:
    //    Returns the value of the attribute given the attrubute key. 
    //
    getAttributeValue: function(/* the content object */content, attributeKey)
    {
        var parentPart = attributeKey.parentPart;
        var attrName = attributeKey.attributeName;
        var attr = parentPart.getAttribute(attrName);
        var type = attr.getType();
        var value = null; 
		
		try {
	        if(attributeKey["type"] && attributeKey["type"] == DefaultAssetContentProvider.TYPE_PART_SELECTOR) {
				var partInst = this._internalGetAttributeValue(attributeKey);
				
	            if(partInst) {
					// We've "auto" instantiated the parent part to default to
					// show the root's attributes. Check to see if the part value
					// is set to the root, if so, don't show it.
					// 
					var rootPart = attr.getRootPart();
					
					if(rootPart.getName() != partInst.getName())
	                    value = partInst.getName();				
				}
	        }
			else {
				value = this._internalGetAttributeValue(attributeKey);
				
				// a pointer object has two components. Use
				// the display string for value.
	            if(type == DefaultAssetContentProvider.TYPE_POINTER && value) {
	                var pointer = value;
					value = pointer.getDisplayString();
	            }
			}
		}
	    catch(error) {
			console.debug("Error occurred: " + error);
			value = null;
		}
		return value;
    },

    
    // Summary:
    //    Gets the value of the specified attribute. If the attribute is
    //    an array attribute, returns the array value, or if it
    //    is a single value attribute, returns the value. In either case
    //    if the value doesn't have value set, it returns null.
    //
    _internalGetAttributeValue: function(attributeKey)
    {
        var parentPart = attributeKey.parentPart;
        var attrName = attributeKey.attributeName;
        var attr = parentPart.getAttribute(attrName);
        var value = null;
        
        if(attr.isArray()) {
            var index = attributeKey["index"];
            var values = attr.getArrayValues();
			
			// sanity check. We built the key with the attribute array value;
			// it *should* have the value in there.
			if(values && index < values.length)
                value = values[index];
        }
        else {
            value = attr.getValue();
        }
        return value;
    },

    // Summary:
    //    Set the value of the specified attribute. Call the proper attribute
    //    method to set the value based on whether the attribute is an array
    //    attribute.
    //
    _internalSetAttributeValue: function(attributeKey, value)
    {
        var parentPart = attributeKey.parentPart;
        var attrName = attributeKey.attributeName;
        var attr = parentPart.getAttribute(attrName);
        
        if(attr.isArray()) {
            var index = attributeKey["index"];
			attr.setArrayValue(index, value);
        }
        else {
            attr.setValue(value);
        }
    },

    attributeValueChanged: function(attributeKey, oldValue, newValue)
    {
        var parentPart = attributeKey.parentPart;
        var attrName = attributeKey.attributeName;
        var attr = parentPart.getAttribute(attrName);
        var value = (newValue && newValue.length) ? newValue : null;
		
		// handle the owner and group attribute differently. We have to
		// pass in the uuid as well.
        if(attrName == "owner" && value) {
            var user = {"ds":newValue, "uuid": this._getUserIdByName(newValue)};
            value = user;
        }
        else if(attrName == "group" && value) {
            var group = {"ds":newValue, "uuid": this._getGroupIdByName(newValue)};
            value = group;
        }
		
		// update the attribute with the new value.
		this._internalSetAttributeValue(attributeKey, value);
		
		// This is a part attribute and it has a new instance, 
		// invalidate its layout.
        if(attributeKey["type"] && attributeKey["type"] == DefaultAssetContentProvider.TYPE_PART_SELECTOR) {
            this.layoutForm.invalidateAttributeLayout(attributeKey.parentKey);
        }
		
		// if we have another listener wanting to know the event, 
		// forward the event to it.
		if(this.chainedListener) {
			this.chainedListener.attributeValueChanged(attributeKey, oldValue, newValue);
		}
		// return the action to affect the field
		var action = {edited:true};
		return action;
    },
	
	// Summary:
	//    Given a PART type, build a list of keys for each of the attributes
	//    that the part holds. 
	//
	_createAttributeList: function(part)
	{
		var attrKeys = [];
		var attributeNames = part.getAttributeNames();
        var attributeNames_length = attributeNames.length;
		
		for(var i = 0; i < attributeNames_length; i++) {
			//var uniqueId = part.getName() + "." + attributeNames[i];
			var uniqueId = part.getAttribute( attributeNames[i] ).getPathName();
			//if( this._shouldIncludeAttribute( part, part.getAttribute( attributeNames[i] ) ) ){
			//if (!this.attributesToShow || ( this.attributesToShow && this.attributesToShow[uniqueId] ) ) {
				var aKey = {
					id: uniqueId,
					parentPart: part,
					attributeName: attributeNames[i]
				};
				attrKeys.push(aKey);
			//}
		}
		return attrKeys;
	},
	
	_shouldIncludeAttribute: function( part, attribute )
	{
		if( !this.attributesToShow ||  this.attributesToShow && this.attributesToShow[attribute.getPathName()] )
		{
			return( true );
		}
		
		if( attribute.getType() == "ConfigurationPartType" )
		{
			var childParts = part.getChildParts();
			
			if (childParts && childParts.length > 0) {
				alert(childParts[0].getAttribute("hostname").getPathName());
			}
	
			var names = attribute.getRootPart().getAttributeNames();
			var attrPart = part.getAttributeValue( attribute.getName(), true );
			var names_length = names.length;
			for( var i = 0; i < names_length; i++ )
			{
				var subattr = attrPart.getAttribute( names[ i ] );
				if( subattr )
				{
					var id = subattr.getPathName();
					if( this.attributesToShow[ id ] )
					{
						return( true );
					}
				}
			}
		}
		
		return( false );
	},

	isAttributeVisible: function( sectionKey, attributeId )
	{
		var keyName = sectionKey.parentPart.getName() + "." + sectionKey.attributeName;
		var id = attributeId;
		var index = 0;
		if( ( index = attributeId.indexOf( "-id" ) ) != -1 )
		{
			id = attributeId.substr( 0, index );
		}
		
		if( !this.attributesToShow[ keyName ] ||  this.attributesToShow[ keyName ] && this.attributesToShow[keyName][id] )
		{
			return( true );
		}
		
		return( false );
	},

    // Summary:
    //    Given an array attribute, build a list of keys for each of the 
	//    attribute's value.
    //
    _createMultiValueList: function(sectionkey, parentPart, attr)
    {
        var attrKeys = [];
        var attrName = attr.getName();
        var values = attr.getArrayValues();

//		var catalog = new com.ibm.asq.common.web.client.internal.Catalog();
//		var template = catalog.getTemplate( "Machine" );
//		var os = template.getAttribute( "operatingsystem" );
        
        if(values) {
            var values_length = values.length;
            for(var i = 0; i < values_length; i++) {
                var uniqueId = parentPart.getAttribute( attrName ).getPathName() + "-id." + i;//parentPart.getName() + "." + attrName + i;
                //var uniqueId = parentPart.getName() + "." + attrName + i;
                var aKey = {};
                
                aKey["type"] = DefaultAssetContentProvider.TYPE_ATTRIBUTE_MULTI_VALUE;
                aKey["id"] =  uniqueId;
                aKey["parentPart"] =  parentPart;
                aKey["attributeName"] = attrName;
                aKey["index"] = i;
				aKey["catPath"] = uniqueId;
				aKey["showAttribute"] = this.isAttributeVisible( sectionkey, uniqueId );
                attrKeys.push(aKey);
            }           
        }
        return attrKeys;
    },

    _getUserIdByName: function(name)
    {
        var users = com.ibm.rational.test.lm.web.client.internal.RtlmInitializer.getUsers();
        if(users) {
            var users_length = users.length;
            for(var i = 0; i < users_length; i++) {
                if(users[i].name == name)
                    return users[i].itemId;
            }
        }
        return null;
    },
	
	_getGroupIdByName: function(name)
    {
        var groups = GroupUtil.getAllGroups();
        if(groups) {
            var groups_length = groups.length;
            for(var i = 0; i < groups_length; i++) {
                if(groups[i].name == name)
                    return groups[i].itemId;
            }
        }
        return null;
    },
    
	setAttributesToShow: function( id, listOfAttributes )
	{
		this.attributesToShow[ id ] = listOfAttributes;
	},
	
	shouldAttributeShow: function( attribute, attrInfo )
	{
		if( this.attributesToShow )
		{
			if (attribute.id && this.attributesToShow[attribute.id]) {
				return (true);
			}
			else 
				if (attribute.attributeName) {
					for (var elem in this.attributesToShow) {
						var shortname = elem.substr(elem.indexOf(".") + 1);
						if (shortname == attribute.attributeName) {
							return (true);
						}
					}
					
                    var attr_collection = attrInfo.collection;
                    var attr_collection_length = attr_collection ? attr_collection.length : 0;
					for( var i = 0; i < attr_collection_length; i++ )
					{
						for (var elem in this.attributesToShow) {
							var shortname = elem.substr(elem.indexOf(".") + 1);
							if (shortname == attr_collection[ i ].attributeName) {
								return (true);
							}
						}													
					}
					return (false);
				}
				else if (attribute.id && !this.attributesToShow[attribute.id]) 
				{
                    var attr_collection = attrInfo.collection;
                    var attr_collection_length = attr_collection ? attr_collection.length : 0;
					if (attr_collection_length > 1) {
						for( var i = 0; i < attr_collection_length; i++ )
						{
							var shortname = elem.substr(elem.indexOf(".") + 1);
							if (shortname == attr_collection[ i ].attributeName) {
								return (true);
							}							
						}
					}
						return (false);
				}
		}
		
		return( true );
	}
	
});

/**
 * TEST CODE BELOW
 */
dojo.declare("AssetDetailView.TestProvider", 
             com.ibm.rational.test.lm.web.ui.internal.widgets.layout.SectionLayoutForm.ContentProvider,
{
    
    // Summary:
    //    Returns the layout information about the form. The following is the info format:
    //    {
    //         nColumns: integer > 0,  the number of sections across the form layout. 
    //         tileDirection: value,   SectionLayoutForm.TILE_VERTICAL, or SectionLayoutForm.TILE_HORIZONTAL
    //         sections: ["key",...]   the array of section keys (tags)
    //    }
    //    If the tiling direction is vertical,
    //    the sections obtained from the "getSections()" method are filled from
    //    column 1 first. Otherwise, for horizontal tiling, the sections are filled from
    //    row 1 first.
    //
    getFormInfo: function(/* the content object */content) 
    {
        var sectionKeys = ["Section 1", "Section 2", "Section 3", "Section 4", "Section 5"];
        var info = {nColumns: 2, tileDirection: SectionLayoutForm.TILE_VERTICAL, sections: sectionKeys};
        return info;
    },
    
    // Summary:
    //    Returns information about the section. The following is the info format:
    //    {
    //         section: object,          the opique object that's the section
    //         title: "string",          the title for the section
    //         tileDirection: value,     SectionLayoutForm.TILE_VERTICAL, or SectionLayoutForm.TILE_HORIZONTAL
    //         nColumns: integer > 0,    the number of columns of attributes across the section. 
    //         attributes: ["key",...]   the array of attribute keys (tags)
    //    }
    //
    getSectionInfo: function(/* the content object */content, sectionKey)
    {
        var attrKeys = [sectionKey+"Attribute 1:", sectionKey+"Attribute 2:", sectionKey+"Attribute 3:", sectionKey+"Attribute 4:"];
        var sectioninfo = {title: sectionKey, nColumns: 2, attributes: attrKeys};
        return sectioninfo;
    },
    
    // Summary:
    //    Returns information about the attribute given its key value. The attribute info is not static for
    //    all attributes being described. Certain attribute types will have additional data 
    //    associated with them.
    //
    //    The following is the info format:
    //    {
    //         /* 
    //          * Required Info Data 
    //          */
    //         type: value,              the type of attribute, one of the SectionLayoutForm.ATTRIBUTE_*
    //
    //         /* 
    //          * Optional Info Data (NOT YET IMPLEMENTED)
    //          */
    //         colSpan: <1..section.nCol> Optionally overrides the parent section layout's nCols attribute.
    //                                    By default, the attribute will occupy one column of the section. This
    //                                    allows the attribute to occupy more then one column in the section.
    //                                    For attribute type ATTRIBUTE_COLLECTION, the default is section.nCols. 
    //                                    It occupies the entire row in the section.
    //
    //         /* 
    //          * Common Info Data for "atomic" attribute types
    //          */
    //         title: "string",          the title for the attribute
    //         readOnly: <true,false>,  
    //
    //         /* 
    //          * Specific Info Data for the SectionLayoutForm.ATTRIBUTE_COLLECTION type
    //          */
    //         collection: [keys...],    the array of sub-attribute keys** (please see below)
    //         title: "string",          Optional, the title for the collection
    //
    //         /* 
    //          * Specific Info Data for the SectionLayoutForm.ATTRIBUTE_ENUM type
    //          */
    //         enumChoices:[values...]
    //    }
    //
    //    ** The attribute keys can be a simple string tag, or an opique object. If it's an opique
    //    object, it must have an id attribute, like key.id that is a string value. The key (or id) must be
    //    unique amoung ALL attribute ids. The attribute's id can then be used to reference and access the
    //    attribute layout.
    //
    getAttributeInfo: function(/* the content object */content, sectionKey, attributeKey)
    {
        var attrTitle = attributeKey;
        
        if(attributeKey.indexOf("Attribute") > 0) {
            attrTitle = attributeKey.slice(attributeKey.indexOf("Attribute"));
        }
        
        if(sectionKey == "Section 3") {
            if(attributeKey.indexOf("Attribute 3") > 0) {
                var attr_info = {type: SectionLayoutForm.ATTRIBUTE_COLLECTION,  title: "Operating System", collection:["Linux", "Windows"]};
                return attr_info;
            }
        }
        var attrInfo = {title: attrTitle, type: SectionLayoutForm.ATTRIBUTE_LABEL, readOnly: true};

        if(attributeKey == "Linux") {
            attrInfo = {title: "Linux OS", type:SectionLayoutForm.ATTRIBUTE_COLLECTION, collection:["OsType:", "Version:", "Root Password:"]};          
        }
        if(attributeKey == "Windows") {
            attrInfo = {title: "Windows OS", type:SectionLayoutForm.ATTRIBUTE_COLLECTION, collection:["OsType:", "Version:", "Registry Location:", "Service pack:"]};          
        }

        if(attributeKey.indexOf("2") > 0 || attributeKey.indexOf("3") > 0 ||
           attributeKey == "Root Password:" || attributeKey == "Version:")
            attrInfo = {title: attrTitle, type: SectionLayoutForm.ATTRIBUTE_TEXT, readOnly: false};

        if(attributeKey.indexOf("Attribute 4") > 0 || attributeKey == "OsType:")
            attrInfo = {title: attrTitle, type: SectionLayoutForm.ATTRIBUTE_ENUM, readOnly: false, enumChoices:["test 1", "test 2", "test 3"]};
          
        return attrInfo;
    },
    // Summary:
    //    Returns the value of the attribute given the attrubute key.
    //
    getAttributeValue: function(/* the content object */content, attributeKey)
    {
        return "Testing!"
    }
    
});

//
// Summary:
//
dojo.declare("AssetDetailView.TestTemplateProvider", 
             com.ibm.rational.test.lm.web.ui.internal.widgets.layout.SectionLayoutForm.ContentProvider,
{
    
    constructor: function()
    {   
    },
    
    // Summary:
    //    Returns the layout information about the form. The following is the info format:
    //    {
    //         nColumns: integer > 0,  the number of sections across the form layout. 
    //         tileDirection: value,   SectionLayoutForm.TILE_VERTICAL, or SectionLayoutForm.TILE_HORIZONTAL
    //         sections: ["key",...]   the array of section keys (tags)
    //    }
    //    If the tiling direction is vertical,
    //    the sections obtained from the "getSections()" method are filled from
    //    column 1 first. Otherwise, for horizontal tiling, the sections are filled from
    //    row 1 first.
    //
    getFormInfo: function(/* the content object */content) 
    {
        var sectionKeys = ["Details"];
        var info = {nColumns: 1, tileDirection: SectionLayoutForm.TILE_VERTICAL, sections: sectionKeys};
        return info;
    },
    
    // Summary:
    //    Returns information about the section. The following is the info format:
    //    {
    //         section: object,          the opique object that's the section
    //         title: "string",          the title for the section
    //         tileDirection: value,     SectionLayoutForm.TILE_VERTICAL, or SectionLayoutForm.TILE_HORIZONTAL
    //         nColumns: integer > 0,    the number of columns of attributes across the section. 
    //         attributes: ["key",...]   the array of attribute keys (tags)
    //    }
    //
    getSectionInfo: function(/* the content object */content, sectionKey)
    {
        var template = content.rootTemplate;
        var rootPart = template.getRoot();
        var attrNames = rootPart.getAttributeNames();
        
        var sectionInfo = {
            title: sectionKey,
            tileDirection:SectionLayoutForm.TILE_VERTICAL,
            nColumns: 1,
            attributes: attrNames
        };
        return sectionInfo;
    },
    
    // Summary:
    //    Returns information about the attribute given its key value. The attribute info is not static for
    //    all attributes being described. There's a common set of info data for any attributes. But certain 
    //    attribute types will have additional data associated with them.
    //
    //    The following is the info format:
    //    {
    //         /* 
    //          * Required Info Data 
    //          */
    //         type: value,              the type of attribute, one of the SectionLayoutForm.ATTRIBUTE_*
    //
    //         /* 
    //          * Common Info Data for "atomic" attribute types
    //          */
    //         attribute: object,        the opique object that's the attribute
    //         title: "string",          the title for the attribute
    //         readOnly: <true,false>,  
    //
    //         /* 
    //          * Specific Info Data for the SectionLayoutForm.ATTRIBUTE_COLLECTION type
    //          */
    //         collection: [keys...],
    //         title: "string"           Optional, the title for the collection
    //    }
    //
    getAttributeInfo: function(/* the content object */content, sectionKey, attributeKey)
    {
        var catalog = content.theCatalog;
        var attrName = "booboo!";
        var attr;
        var type;
        var collection = null;
        var attrInfo = {};
        var layoutType = SectionLayoutForm.ATTRIBUTE_LABEL;
        
        // this is the first level attribute for the asset's root part.
        // use the content object to fetch the info.
        if(dojo.isString(attributeKey)) {
            var template = content.rootTemplate;
            var rootPart = template.getRoot();
            attr = rootPart.getAttribute(attributeKey);
            type = attr.getType();
            attrName = attr.getName();
        }
        else if(dojo.isObject(attributeKey)){
            if(attributeKey.type == "subPartAttribute") {
                attr = attributeKey.subPart.getAttribute(attributeKey.attributeName);
                type = attr.getType();
                attrName = attr.getName();
            }
            else {
                type = attributeKey.type;
            }
        }
        
        if(type == "partInstance") {
            layoutType = SectionLayoutForm.ATTRIBUTE_COLLECTION;
            var part = attributeKey.partTemplate.getPart(attributeKey.partName);
            var subpartAttrNames = part.getAttributeNames();
            collection = [];
            for(var i = 0; i < subpartAttrNames.length; i++) {
                var subPartKey = {id: subpartAttrNames[i], type: "subPartAttribute", subPart: part, attributeName: subpartAttrNames[i]};
                collection.push(subPartKey);                
            }
            attrName = attributeKey.partName;
        }
        else if(type == DefaultAssetContentProvider.TYPE_PART) {
            layoutType = SectionLayoutForm.ATTRIBUTE_COLLECTION;
            var partTypeName = attr.getPartTypeName();
            var partTemplate = catalog.getTemplate(partTypeName);
            collection = [];
            
            var partNames = partTemplate.getPartNames();        
            for (var i = 0; i < partNames.length; i++) {
                var partKey = {id: partNames[i], type: "partInstance", partTemplate: partTemplate, partName: partNames[i]};
                collection.push(partKey);
            }
        }
        else if(type == DefaultAssetContentProvider.TYPE_STRING) {
            layoutType = SectionLayoutForm.ATTRIBUTE_TEXT;
            attrInfo["attribute"] = attr;
        }
        else if(type == DefaultAssetContentProvider.TYPE_NUMBER) {
            layoutType = SectionLayoutForm.ATTRIBUTE_TEXT_NUMERIC;
            attrInfo["attribute"] = attr;
        }
        else if(attr.isEnum()) {
            layoutType = SectionLayoutForm.ATTRIBUTE_ENUMERATE;         
            attrInfo["attribute"] = attr;
        }
        
        attrInfo["title"] = attrName;
        attrInfo["type"] = layoutType;
        attrInfo["readOnly"] = false;
        
        if(collection != null) {
            attrInfo["collection"] = collection;
        }
        return attrInfo;
    },
    // Summary:
    //    Returns the value of the attribute given the attrubute key.
    //
    getAttributeValue: function(/* the content object */content, attributeKey)
    {
        return "Testing!"
    }
    
});
//	Create some definitions that will be used in other files
//	Note that the DefaultAssetContentProvider variable is used locally to this file as a shortcut.
	var DefaultAssetContentProvider = com.ibm.rational.test.lm.web.ui.internal.views.machines.DefaultAssetContentProvider;
	DefaultAssetContentProvider.TYPE_PART 		= 'ConfigurationPartType';
	DefaultAssetContentProvider.TYPE_NUMBER 	= 'NumericType';
	DefaultAssetContentProvider.TYPE_STRING 	= 'StringType';
	DefaultAssetContentProvider.TYPE_POINTER	= 'PointerType';

	// Pseudo type for selecting an instance part for an TYPE_PART attribute
	DefaultAssetContentProvider.TYPE_PART_SELECTOR = "PartSelector";

	// Pseudo type for a sub-value of an attribute of the Array type.
	DefaultAssetContentProvider.TYPE_ATTRIBUTE_MULTI_VALUE = 'AttributeArrayValue';
})();
