
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
//

//
//
// Author: L. To
//
// Summary:
//

dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.machines.DefaultCustomElementProvider");
 
dojo.require("com.ibm.asq.common.web.client.internal.Catalog");
dojo.require("com.ibm.rational.test.lm.web.client.internal.RtlmInitializer");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.layout.SectionLayoutForm");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.layout.SectionHeader");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.machines.ArrayAttributeTitle");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.machines.DefaultAssetContentProvider");

(function() {

// Convenient local shorthand variables
//
var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
var SectionHeader = com.ibm.rational.test.lm.web.ui.internal.widgets.layout.SectionHeader;
var ArrayAttributeTitle = com.ibm.rational.test.lm.web.ui.internal.views.machines.ArrayAttributeTitle;
var DefaultAssetContentProvider = com.ibm.rational.test.lm.web.ui.internal.views.machines.DefaultAssetContentProvider;
var RtlmUiUtil_getCommonMessage = RtlmUiUtil.getCommonMessage;

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.machines.DefaultCustomElementProvider", 
    [com.ibm.rational.test.lm.web.ui.internal.widgets.layout.SectionLayoutForm.CustomElementProvider],
{   
    layoutForm: null,
    
    constructor: function(layoutForm)
    {
        this.layoutForm = layoutForm;
    },
    // Summary:
    //    Returns a widget that is the section header.
    //
    constructSectionHeader: function(sectionKey)
    {
        var title = {title: sectionInfo.title};
        var sectionInfo = this.layoutForm.getContentProvider().getSectionInfo(this.layoutForm.getContent(), sectionKey);       
        var sectHeader = new SectionHeader(title);
        
        sectHeader.makeCollapsible(RtlmUiUtil_getCommonMessage("actionLabel_Less"), RtlmUiUtil_getCommonMessage("actionLabel_More"));
        return sectHeader;
    },
    
    // Summary:
    //    Returns a widget that is the header for collection attributes.
    //
    constructCollectionAttributeHeader: function(sectionKey, attributeKey)
    {
        var parentPart = attributeKey.parentPart;
        var attrName = attributeKey.attributeName;
        var attr = parentPart.getAttribute(attrName);
        var title = null;
        
        // only construct a "custom" attribute title for Array
        // attributes. The specialized title will have an add action
        // to insert a new value to the array attribute. If we return null
        // the form will create the default title for the collection
        //
        if(attr.isArray()) {
            var attrInfo = this.layoutForm.getContentProvider().getAttributeInfo(this.layoutForm.getContent(), sectionKey, attributeKey);       
            title = new ArrayAttributeTitle({title: attrInfo.title, callbackData: attributeKey});
    
            title.onAddValueClick = dojo.hitch(this, "_handleAddAttributeValue");
        }
        return title;
    },
    
    // Summary:
    //    Returns a widget for the attribute. The widget must be a subclass of the AbstractFormField.
    // 
    constructCustomField: function(atttributeKey)
    {
        return null;
    },
    
	_getHostname: function(osParts)
	{
		var i = 0;
        var part_length = osParts.length;
		for (i = 0; i < part_length; i++) {
			var hostname = osParts[i].getAttributeValue("FullyQualifiedDomainName");
			if (hostname != null) 
				return hostname;
		}
		for (i = 0; i < part_length; i++) {
			var hostname = osParts[i].getAttributeValue("hostname");
			if (hostname != null) 
				return hostname;
		}
		return "";
	},
	
	_populateRemoteConnection: function(attr, parentPart)
	{
		var hostname = "";
		var osParts = parentPart.getAttributeArrayValues("operatingsystem");
		if(osParts != null)
			hostname = this._getHostname(osParts);

		var allTypes =  ["Remote Desktop", "ftp","VNC", "telnet", "http"];
		var allURLs =   ["rdp://" + hostname, "ftp://" + hostname, "http://" + hostname + ":5800", "telnet://" + hostname, "http://"];
		var found = false;
		var values = attr.getArrayValues();
		var pos = values.length;
		var current = null;
		var part = null;
        var allTypes_length = allTypes.length;
        var values_length = values.length;
		for (var i = 0; i < allTypes_length; i++) {
			found = false;
			current = allTypes[i];
			for (var j = 0; j < values_length; j++) {
				if (values[j].getName() == current) {
					found = true;
					break;
				}
			}
			if (!found) {
				attr.addArrayValue(current);
				part = attr.getArrayValue(pos);
				part.setAttributeValue("URL", allURLs[i]);
				return;
			}
		}
		// One of each already exists, create a new one that has some value
		attr.addArrayValue(allTypes[0]);
		part = attr.getArrayValue(pos);
		part.setAttributeValue("URL", allURLs[0]);
		
	},
	
    _handleAddAttributeValue: function(attributeKey)
    {
        var parentPart = attributeKey.parentPart;
        
		var attr = parentPart.getAttribute(attributeKey.attributeName);
        var type = attr.getType();
        var value = "";
    	
		if (type == DefaultAssetContentProvider.TYPE_PART) {
			var rootPart = attr.getRootPart();
			value = rootPart.getName();
			if (value == "RemoteConnection") {
				this._populateRemoteConnection(attr, parentPart);
				this.layoutForm.invalidateAttributeLayout(attributeKey);
				return;
			}
		}
				
        	// Add a new empty value to the attribute
        	// and invalidate the layout for this attribute
        	// to pick up the new value.
        if(attr.isArray())
            attr.addArrayValue(value);
        else
            attr.setValue(value);
        this.layoutForm.invalidateAttributeLayout(attributeKey);
    }
});

})();


