/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.machines.MachineListBaseView");
 
dojo.require("com.ibm.asq.common.web.client.internal.Catalog");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.util.GlobalEvents");
 
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rational.test.lm.web.client.internal.config");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.pages.InventoryUriUtil");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");

dojo.require("com.ibm.rational.test.lm.web.client.internal.reservations");
dojo.require("com.ibm.rational.test.lm.web.client.internal.inventory" );

//
// Summary: Base class for common functionality for views that show lists of lab
// resources. Currently these views are the "all machines" view, the simple search
// view, and the advanced search view. They share some common behavior which should 
// be pushed down into this class. 
// Currently the code used to delete/hide lab resources is implemented in the 
// base class.
(function() {
    var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
    var GlobalEvents = com.ibm.asq.common.web.ui.internal.util.GlobalEvents;
    var ViewHeaderLayout = com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout;
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    var RtlmUiUtil_getMessage = RtlmUiUtil.getMessage;
    var SUCCESS = ViewHeaderLayout.SUCCESS;
    var ERROR = ViewHeaderLayout.ERROR;
    
dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.machines.MachineListBaseView", 
    null,	
{
    _archiveSelected: function(archiveType)
    {
        var msgId;
        
        if(archiveType == "delete")
            msgId = "msgConfirmDeleteResources";
        else if(archiveType == "hide")
            msgId = "msgConfirmHideResources";
        else 
            msgId = "msgConfirmRestoreResources";

        var args = {
            'message': RtlmUiUtil_getMessage(msgId),
            'yesCallback': dojo.hitch(this, "_performArchiveActionPrecondition"),
            'yesCallbackData': archiveType,
            'noCallback': null,
            'noCallbackData': null
        };
        var confirmDialog = new com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog(args);
        confirmDialog.setSecondaryTitle(RtlmUiUtil_getMessage("msgConfirmTitle"));
        confirmDialog.open();
    },
    
    _performArchiveActionPrecondition: function(archiveType)
    {
        var selectedItems = this.tableViewer.getCheckedRows();      
        var assetIds = [];        
        // package up the selected item ids     
        var selectedItems_length = selectedItems.length;
        for( var i = 0; i < selectedItems_length; i++ ) {
            var id = this.tableContentProvider.getRowDataId(selectedItems[i]);
            assetIds.push(id);                  
        }       
        var responseHandler = new ServiceResponseHandler(
            {
                parent: this,
                action : archiveType,   
                removeAssets : assetIds,    
                _success: function(response)
                {
                    if (response && (response == "true")) {
                        var alert_dlg_args = {message:RtlmUiUtil_getMessage("msgErrorDeleteOrHideReservedResource")};
                        var alert_dlg = new com.ibm.asq.common.web.ui.internal.widgets.AlertDialog(alert_dlg_args);
                        alert_dlg.open();
                    } else {
                        this.parent._performArchiveAction(this.action, this.removeAssets);
                    }
                },
                _failure: function(errObj)
                {
                    // If we error out trying to determine if there is a reservation
                    // just go ahead and do the archive operation. 
                    this.parent._performArchiveAction(this.action, this.removeAssets);
                }
            },
            "_success",
            "_failure"
        );            
        if (archiveType == "delete" || archiveType == "hide") {
            var reservations = new com.ibm.rational.test.lm.web.client.internal.reservations();
            reservations.checkReservations(responseHandler, assetIds);
        }
        else {
            this._performArchiveAction(archiveType, assetIds);
        }               
    },
    
    _performArchiveAction: function(archiveType, assetIds)
    {    
        var responseHandler = new ServiceResponseHandler(
            {
                parent: this,
                action: archiveType,
                removeAssets: assetIds,
                _success: function(response)
                {
                    // If we are deleting the resource call in to the LM provider integration code
                    // so that we can notify integrations to remove the asset as well (if applicable).
                    if(this.action == "delete") {
                        this.parent._performProviderDeleteAction(this.action, this.removeAssets);
                    }
                    this.parent.viewHeaderNode.setProgressEnd();
                    this.parent._updatePostDelete();
                    
                    var msgId;
                    if(this.action == "hide")
                        msgId = "msgSuccessHidingResources";
                    else
                        msgId = "msgSuccessRestoringResources";
                        
                    this.parent.viewHeaderNode.setStatus(RtlmUiUtil_getMessage(msgId), SUCCESS);
                    if (this.removeAssets.length > 0) {
                        var assetType = this.parent.assetType;
                        var type = (assetType == "Machine") ? RtlmUiUtil.TYPE_MACHINE : (assetType == "VirtualImage") ? RtlmUiUtil.TYPE_VM : assetType;
                        // notify all pages (including itself) resources are deleted
                        var eventType = (this.action == "restore") ? GlobalEvents.ArtifactsUndeleted : GlobalEvents.ArtifactsDeleted;
                        dojo.publish(eventType, [type, this.removeAssets]);
                    }
                },
                _failure: function(errObj)
                {
                    var msgId;
                    
                    if(this.action == "delete")
                        msgId = "msgErrorDeletingResources";
                    else if(this.action == "hide")
                        msgId = "msgErrorHidingResources";
                    else
                        msgId = "msgErrorRestoringResources";
                    var details = (errObj && errObj.message) ? errObj.message : null;                  
                    this.parent.viewHeaderNode.setProgressEnd();
                    this.parent.viewHeaderNode.setStatus(RtlmUiUtil_getMessage(msgId), ERROR, details);
                }
            },
            "_success",
            "_failure"
        );
        var msgId;
                
        if(archiveType == "delete")
            msgId = "msgDeletingResources";
        else if(archiveType == "hide")
            msgId = "msgHidingResources";
        else
            msgId = "msgRestoringResources";

        this.viewHeaderNode.setProgressStart(RtlmUiUtil_getMessage(msgId));
        
        var catalog = new com.ibm.asq.common.web.client.internal.Catalog();        
        catalog.updateAssetsArrayVisibility(responseHandler, this.assetType, assetIds, archiveType);
    },
    
    _performProviderDeleteAction: function(archiveType, assetIds)
    {
        var responseHandler = new ServiceResponseHandler(
            {    
                parent: this,
                action: archiveType,
                removeAssets: assetIds,
                _success: function(response)
                {
                    this.parent.viewHeaderNode.setProgressEnd();
                    this.parent._updatePostDelete();
                    
                    var msgId = "msgSuccessDeletingResources";
                    var isError = new Boolean(false);
                    var response_length = response.length;
                    for (var i=0; i<response_length; i++) {
                        if (response[i].rc != "SPIRC_NO_ERROR") {
                            isError = new Boolean(true);
                            msgId = response[i].rc;
                            break;
                        }
                    }
                    if (isError == true) {
                        this.parent.viewHeaderNode.setStatus(RtlmUiUtil_getMessage(msgId), ERROR);
                    } else {                                            
                        this.parent.viewHeaderNode.setStatus(RtlmUiUtil_getMessage(msgId), SUCCESS);
                    }
                    
                    if (this.removeAssets.length > 0) {
                        var assetType = this.parent.assetType;
                        var type = (assetType == "Machine") ? RtlmUiUtil.TYPE_MACHINE : (assetType == "VirtualImage") ? RtlmUiUtil.TYPE_VM : assetType;
                        // notify all pages (including itself) resources are deleted
                        var eventType = (this.action == "restore") ? GlobalEvents.ArtifactsUndeleted : GlobalEvents.ArtifactsDeleted;
                        dojo.publish(eventType, [type, this.removeAssets]);
                    }                
                },
                _failure: function(errObj)
                {
                    var msgId = "SPIRC_ASSET_DELETED_FOR_MULTIPLE_PROVIDERS_FAILED";
                    var details = (errObj && errObj.message) ? errObj.message : null;                  
                    this.parent.viewHeaderNode.setProgressEnd();
                    this.parent.viewHeaderNode.setStatus(RtlmUiUtil_getMessage(msgId), ERROR, details);                
                }
            },
            "_success",
            "_failure"
        );     
        var inven = new com.ibm.rational.test.lm.web.client.internal.inventory();
        inven.deleteAssetFromProviders(responseHandler, assetIds);
    },
    
    // Summary:
    // Override and implement necessary logic to refresh lab resource    
    // list, potentially after a delete, etc.
    _updatePostDelete: function()
    {
        //Intentionally left blank
    }
});		 
})();
