/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.providers.DefaultTableProvider"); 

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");

(function() {

	var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.providers.DefaultTableProvider", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider,
{
	constructor: function()
	{
	},
	
	setParameters: function( params )
	{
		this.column_headings = params[ "column_headings" ];
		this.id_fields = params[ "id_fields" ];
		this.columns_section = params[ "column_section" ];
		this.column_types = params[ "column_types" ];
		this.iconmap = params[ "iconmap" ];
	},
	
    // Summary:
    //
    getNumColumns: function(content)
    {
        return( ( this.columns_section ) ? this.column_headings[ this.columns_section ].length : this.column_headings.length );
    },
    
    // Summary:
    //
    getColumnHeading: function(content, index)
    {
		return( ( this.columns_section ) ? this.column_headings[ this.columns_section ][ index ][ "title" ] : this.column_headings[ index ][ "title" ] );
    },
    
    getIsColumnGroupable: function(content, atColumnIndex)
    {
		return( false );
    },

    getIsColumnSortable: function(content, atColumnIndex)
    {
        return true;     
    },
    
    getColumnWidth: function(content, atColumnIndex)
    {   	
        return( ( this.columns_section ) ? this.column_headings[ this.columns_section ][ atColumnIndex ][ "width" ] : this.column_headings[ atColumnIndex ][ "width" ] );
    },
    
    // Summary:
    //    The only assumptions the table have on the data that it displays
    //    is that it's an array contents. These are the data content/object that
    //    will be displayed in the table rows.
    //
    getContentObjects: function(content)
    {		
		return( content );
    },
    
    getCellAlignment: function(rowData, atColumnIndex)
    {
    	var alignment = ( this.columns_section ) ? 
    							this.column_headings[ this.columns_section ][ atColumnIndex ][ "alignment" ] : 
    							this.column_headings[ atColumnIndex ][ "alignment" ];
    							
        return( ( alignment ) ? alignment : this.inherited( "getCellAlignment", arguments ) );
    },
    
    
    // Summary:
    //    Returns the type of the cell to be displayed. By default, the TableViewer
    //    will support a set of cell types. The type of the cell will map to a specific
    //    cell renderer (to be discussed).
    getCellDisplayType: function(rowData, atColumnIndex)
    {    	
        return( ( this.columns_section ) ? this.column_types[ this.columns_section ][ atColumnIndex ] : this.column_types[ atColumnIndex ] );
    },
    
    // Summary:
    //    Returns the display data of the cell. To support type like 'dropdown' select,
    //    additional data would need to be passed as parameters.
    //
    getCellDisplayData: function(rowData, atColumnIndex)
    {
    	var data = null;
    	var property = ( this.columns_section ) ? 
    						this.column_headings[ this.columns_section ][ atColumnIndex ][ "native_column" ] : 
    						this.column_headings[ atColumnIndex ][ "native_column" ];
    						
    	if( property && typeof( property ) == "function" )
    	{
    		return( property( rowData, atColumnIndex ) );
    	}
    	else
    	{
	    	var properties = property.split( "." );
	    	
	    	//Glue back together items that should not have been split.
	    	for( var iProp = 0; iProp < properties.length; iProp++ )
	    	{
	    		if( properties[ iProp ].charAt( properties[ iProp ].length - 1 ) == '\\' )
	    		{
	    			properties[ iProp ] = properties[ iProp ].substr( 0, properties[ iProp ].length - 1 ) + "." + properties[ iProp + 1 ];
	    			properties.splice( iProp + 1, 1 );
	    			iProp--;
	    		}
	    	}
	    	
	    	data = rowData;
	    	
	    	for( var iProp = 0; data && iProp < properties.length; iProp++ )
	    	{
	    		data = data[ properties[ iProp ] ];
	    	}
    	}
    	
    	return( ( data ) ? data : "" );
    },
    
    getRowDataId: function(rowData)
    {
    	var data = null;
    	var id = ( ( this.columns_section ) ? this.id_fields[ this.columns_section ] : this.id_fields );
    	
    	if( id && typeof( id ) == "function" )
    	{
    		data = id( rowData );
    	}
    	else
    	{
	    	var properties = id.split( "." );
	    	
	    	//Glue back together items that should not have been split.
	    	for( var iProp = 0; iProp < properties.length; iProp++ )
	    	{
	    		if( properties[ iProp ].charAt( properties[ iProp ].length - 1 ) == '\\' )
	    		{
	    			properties[ iProp ] = properties[ iProp ].substr( 0, properties[ iProp ].length - 1 ) + "." + properties[ iProp + 1 ];
	    			properties.splice( iProp + 1, 1 );
	    			iProp--;
	    		}
	    	}
	    	
	    	var data = rowData;
	    	
	    	for( var iProp = 0; iProp < properties.length; iProp++ )
	    	{
	    		data = data[ properties[ iProp ] ];
	    	}
		}
		
		return( data );
    },
    
    getCellDisplayExtendedInfo: function(rowData, atColumnIndex)
    {  
    	return( null );
    },
    
    getColumnIndex: function( nativecolumn )
    {
		var columns = ( this.columns_section ) ? 
    						this.column_headings[ this.columns_section ] : this.column_headings ;
    						
		for( var iCol = 0; iCol < columns.length;  iCol++ )
		{
			if( columns[ iCol ][ "native_column" ] == nativecolumn )
				return( iCol );
		}
		
		return( -1 );   						
    } 
});

})();
