/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.providers.InventoryOperationsTableProvider"); 

dojo.require("dojo.date.locale");

dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache")
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.pages.AssetUriUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.providers.DefaultTableProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.util.DomConstructor");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");

(function() {

	var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
	var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
    var ImageCache     = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
	var getDisplayDateTimeString = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil.getDisplayDateTimeString;
	var AssetUriUtil = com.ibm.rational.test.lm.web.ui.internal.pages.AssetUriUtil;
	var column_types = [
						TableConstants.RENDER_TYPE_TEXT,
						TableConstants.RENDER_TYPE_LINK,
						TableConstants.RENDER_TYPE_TEXT,
						TableConstants.RENDER_TYPE_TEXT,
						TableConstants.RENDER_TYPE_TEXT ];
	var new_column_types = [
						TableConstants.RENDER_TYPE_ACTION,
						TableConstants.RENDER_TYPE_TEXT,
						TableConstants.RENDER_TYPE_TEXT,
						TableConstants.RENDER_TYPE_TEXT,
						TableConstants.RENDER_TYPE_TEXT ];
						
dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.providers.InventoryOperationsTableProvider", 
             com.ibm.rational.test.lm.web.ui.internal.views.providers.DefaultTableProvider,
{
    AssetAddedLabel: RtlmUiUtil.getMessage("inventoryAssetAdded"),
    AssetUpdatedLabel: RtlmUiUtil.getMessage("inventoryAssetUpdated"),
    
	constructor: function( column_headings, id_fields )
	{
		this.setParameters( { "column_headings" : this.getColumnHeadings(),
							  "id_fields" : this.getIdFields(),
							  "column_types" : column_types } );
	},
	
	getColumnHeadings : function()
	{
		return( [
		  {title: RtlmUiUtil.getMessage("Operation"), native_column: "group", isSortable: true, includeInFilter: true, width: 25}, 
		  {title: RtlmUiUtil.getMessage("Machine Name"), native_column: "machine", isSortable: true, includeInFilter: true, width: 7  },
		  {title: RtlmUiUtil.getMessage("Machine Status"), native_column: "machinestate", isSortable: true, includeInFilter: true, width: 8  },
		  {title: RtlmUiUtil.getMessage("Provider"), native_column: "provider", isSortable: true, includeInFilter: true, width: 25  },
		  {title: RtlmUiUtil.getMessage("Provider Instance"), native_column: "providerInstance", isSortable: true, includeInFilter: true, width: 60 }
		] );
	},
	
	getIdFields: function()
	{
		return( "machineid" );
	},
    
    // Summary:
    //    Returns the type of the cell to be displayed. By default, the TableViewer
    //    will support a set of cell types. The type of the cell will map to a specific
    //    cell renderer (to be discussed).
    getCellDisplayType: function(rowData, atColumnIndex)
    {    
		var type = this.inherited( "getCellDisplayType", arguments );
		//Ensure that blank machine rows do not have links for blank machines
		
        // If the operation is still running show the "in progress" image.
        // If the lab resource was deleted then don't display a clickable
        // link for the lab resource.  
        if(atColumnIndex == 1 && rowData['status'] == "IN_PROGRESS") {
            return TableConstants.RENDER_TYPE_IMAGE;
        } else if (atColumnIndex == 1 && rowData['machinestate'] == "Asset Removed") {
            return TableConstants.RENDER_TYPE_TEXT;
        }
        return( ( ( atColumnIndex == 1 ) && rowData.nomachine ) ? TableConstants.RENDER_TYPE_TEXT : type );
    },
	
	getContentObjects: function(content)
    {
		var rows = [];
		
		for( var iData = 0; iData < content.length; iData++ )
		{
            // Use a medium time format to have the seconds in the string
            // Otherwise, different inventory runs occurred within the minute
            // would be lumped into the same entry.
			var content_iData = content[ iData ];
			var startTime = content_iData.startTime;
			var date = RtlmUiUtil.convertTimeStampToDate( content_iData.startTime );
			var dateString = dojo.date.locale.format(date, {formatLength:"medium"});
			var group = dateString + " - " + content_iData.operation + " " + content_iData.status;
			var noAssetDiscovered = RtlmUiUtil.getMessage("noAssetDiscovered");
			
			if (content_iData.assets && content_iData.assets.length > 0) 
			{
				for (var iAsset = 0; iAsset < content[iData].assets.length; iAsset++) 
				{
					var row = {};

					row[ "startTime" ] = startTime;
			        var date1 = RtlmUiUtil.convertTimeStampToDate(startTime);
					row[ "_startTimeCmp"] = date1.getTime();
					
					row["group"] = group;
					var asset = content_iData.assets[ iAsset ];
					var machine = asset.name;
					row[ "machine" ] =  machine;

					row[ "machinestate" ] =  RtlmUiUtil.getMessage( asset.status );
					row[ "machineid" ] = asset.assetId;
					row[ "providerInstance" ] = asset.providerInstance;
					row[ "provider" ] = asset.providerName;
					row[ "virtual" ] = ( ( content_iData.operation == "IMAGE_IMPORT" ) ? true : false );
					row[ "status" ] =  content_iData.status;
					row[ "operation" ] = content_iData.operation;
					row[ "operationId" ] = content_iData.operationId;
					rows.push( row );
				}
			}
			else
			{
				var row = {};

				row[ "startTime" ] = startTime;
		        var date1 = RtlmUiUtil.convertTimeStampToDate(startTime);
				row[ "_startTimeCmp"] = date1.getTime();

				row["group"] = group;

				row[ "machine" ] =  noAssetDiscovered;

				row[ "machinestate" ] = "\u00a0";
				row[ "provider" ] = "\u00a0";
				row[ "providerInstance" ] = "\u00a0";
				row[ "nomachine" ] = true;
				row[ "status" ] =  content[ iData ].status;
				row[ "operation" ] = content[ iData ].operation;
				row[ "operationId" ] = content[ iData ].operationId;
				rows.push( row );
			}
		}
		
		return( rows );
    },
    
	getGroupColumnExtraData: function( grouprow, atColumnIndex )
	{
		var button = null;
		
		if( ( grouprow[ "operation" ] == "DISCOVER" ) && ( grouprow[ "status" ] == "IN_PROGRESS" ) )
		{
	        var arg = {
	            label: RtlmUiUtil.getMessage( "Cancel" ),
	            showLabel: false,
				iconClass: "cancel-icon-image",
	            onClick: function( e ) {
					this.clientCallback(this.callbackData, e);
	            }
	        };
			button = new dijit.form.Button( arg );
			
			button.callbackData = grouprow;
			button.clientCallback = this.cancelCallback;
		}
		
		return( ( button ) ? button.domNode : null );
	},

    getCellDisplayExtendedInfo: function(rowData, atColumnIndex)
    {  
        var val = null;
        if(atColumnIndex == 1) {
            val = {};
            if(rowData['status'] == "IN_PROGRESS") {
                val['image'] = ImageCache.PROGRESS;
            }
			else if (rowData["operation"] == "VIRTUAL_IMAGE_COLLECTION_IMPORT") {
				val['href'] = "#action=com.ibm.rqm.planning.home.actionDispatcher&subAction=viewTestCell&cellID=" + this.getRowDataId( rowData );
			}
			else if( rowData["operation" ] == "VIRTUAL_MACHINE_COLLECTION_IMPORT" )
			{
				val['href'] = "#action=com.ibm.rqm.planning.home.actionDispatcher&subAction=viewTestCell&cellID=" + this.getRowDataId( rowData );				
			}
			else {
				var type = (((rowData.virtual != undefined) && (rowData.virtual)) ? "VirtualImage" : "Machine");
				
				var id = this.getRowDataId(rowData);
				
				if (id) {
					val['href'] = AssetUriUtil.getViewAssetDetailURI(type, id);
				}
			}
        }

        return val;
    },
    
    getInitialSortDirection: function()
    {
    	return( TableConstants.SORT_DIR_DESCEND );
    },
    
    getInitialSortByColumn: function()
    {
    	return( 1 );
    },
	
	setCancelCallback: function( method )
	{
		this.cancelCallback = method;
	}      
});

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.providers.InventoryOperationsTableSorter",
				com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableSorter,
{
    sortBy: function(viewer, objects, column, sortDir)
    {
        var sortContainer = [];
        sortContainer = sortContainer.concat(objects);
        
        this.viewer = viewer;
        this.groupCol = -1;
        this.sortCol = column;
        this.sortDir = sortDir;
        sortContainer.sort(dojo.hitch(this, "_sortComparator"));
        
        var collate = [];
        collate.push(sortContainer);
        return collate;
    },
    
    groupSortBy: function(viewer, objects, /*primary key (group by)*/column1, /*secondary key*/column2, sortDir)
    {
        var sortContainer = [];
        sortContainer = sortContainer.concat(objects); 
        
        this.viewer = viewer;
        this.groupCol = column1;
        this.sortCol = column2;
        this.sortDir = sortDir;
        this.groupSortDir = TableConstants.SORT_DIR_DESCEND;
        sortContainer.sort(dojo.hitch(this, "_groupComparator"));
        
        var collate = [];
        var groupVal = null;
        var aGroup = null;
        
        // collate the objects into groups bins
        for(var i = 0; i < sortContainer.length; i++) {
            var val = this.viewer.getContentProvider().getCellDisplayData(sortContainer[i], this.groupCol);
            if(val != groupVal) {
                if(aGroup != null) {
                    collate.push(aGroup);
                    aGroup = null;
                }
                if(aGroup == null)
                    aGroup = [];
                groupVal = val;
            }
            aGroup.push(sortContainer[i]);
        }
        if(aGroup)
            collate.push(aGroup);
        return collate;
    },
    
    _groupComparator: function(obj1, obj2)
    {
        var val1 = obj1[ "startTime" ];
        var val2 = obj2[ "startTime" ];

        if (val1 && val2) {
	        val1 = obj1["_startTimeCmp"];
	        val2 = obj2["_startTimeCmp"];
        }
        
        // same group, try secondary sorting.
        if(val1 == val2) {
            // there's a secondary sort and it's not the same as the group
            // sort column, sort by it. 
            if(this.sortCol > -1 && this.groupCol != this.sortCol)
                return this._sortComparator(obj1, obj2);
                
            return 0;
        }
        
        if(this.groupSortDir== TableConstants.SORT_DIR_ASCEND) {
            return (val1 > val2) ? 1 : -1;
        }
        else {
            return (val1 > val2) ? -1 : 1;            
        }
    },
    
    _sortComparator: function(obj1, obj2)
    {
       var val1 = this.viewer.getContentProvider().getCellDisplayData(obj1, this.sortCol);   
        var val2 = this.viewer.getContentProvider().getCellDisplayData(obj2, this.sortCol);
        
        // this is a time stamp type. Convert the timestamp value to a Date
        // object for the comparison.
        //
        if(val1 && val2) {
            if(this.viewer.getContentProvider().getCellDisplayType(obj1, this.sortCol) == TableConstants.RENDER_TYPE_TIMESTAMP) {
                var date1 = RtlmUiUtil.convertTimeStampToDate(val1);
                val1 = date1.getTime();
            }
            else if(dojo.isString(val1)) {
                val1 = val1.toLowerCase();
            }
            if(this.viewer.getContentProvider().getCellDisplayType(obj2, this.sortCol) == TableConstants.RENDER_TYPE_TIMESTAMP) {
                var date2 = RtlmUiUtil.convertTimeStampToDate(val2);
                val2 = date2.getTime();
            }
            else if(dojo.isString(val2)) {
                val2 = val2.toLowerCase();
            }
        }        
        if(val1 == val2)
            return 0;
        
        if(this.sortDir == TableConstants.SORT_DIR_ASCEND) {
            return (val1 > val2) ? 1 : -1;
        }
        else {
            return (val1 > val2) ? -1 : 1;            
        }
    } 
});	

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.providers.NewInventoryOperationsTableProvider", 
             com.ibm.rational.test.lm.web.ui.internal.views.providers.DefaultTableProvider,
{
	constructor: function( column_headings, id_fields )
	{
		this.setParameters( { "column_headings" : this.getColumnHeadings(),
							  "id_fields" : this.getIdFields(),
							  "column_types" : new_column_types } );
	},
	
	getColumnHeadings : function()
	{
		return( [
		  {title: RtlmUiUtil.getMessage("Operation"), native_column: "operation", isSortable: true, includeInFilter: true, width: 25}, 
		  {title: RtlmUiUtil.getMessage("Status"), native_column: "status", isSortable: true, includeInFilter: true, width: 7  },
		  {title: RtlmUiUtil.getMessage("Integration"), native_column: "provider", isSortable: true, includeInFilter: true, width: 8  },
		  {title: RtlmUiUtil.getMessage("Server"), native_column: "provider", isSortable: true, includeInFilter: true, width: 25  },
		  {title: RtlmUiUtil.getMessage("User"), native_column: "owner", isSortable: true, includeInFilter: true, width: 60 }
		] );
	},
	
	getIdFields: function()
	{
		return( "machineid" );
	},
    
    // Summary:
    //    Returns the type of the cell to be displayed. By default, the TableViewer
    //    will support a set of cell types. The type of the cell will map to a specific
    //    cell renderer (to be discussed).
    getCellDisplayType: function(rowData, atColumnIndex)
    {    
		var type = this.inherited( "getCellDisplayType", arguments );
		//Ensure that blank machine rows do not have links for blank machines
		
        return( ( ( atColumnIndex == 1 ) && rowData.nomachine ) ? TableConstants.RENDER_TYPE_TEXT : type );
    },
	
	getContentObjects: function(content)
    {
		var rows = [];
		
		for( var iData = 0; iData < content.length; iData++ )
		{
			var date = RtlmUiUtil.convertTimeStampToDate( content[ iData ].startTime );
			var dateString = getDisplayDateTimeString( date );
			var group = dateString + " - " + content[ iData ].operation + " " + content[ iData ].status;
			
//			if (content[iData].assets && content[iData].assets.length > 0) 
//			{
//				for (iAsset = 0; iAsset < content[iData].assets.length; iAsset++) 
//				{
//					var row = {};
//					row[ "startTime" ] = content[ iData ].startTime;
//					row["group"] = group;
//					row[ "machine" ] =  content[iData].assets[ iAsset ].name;
//					row[ "machinestate" ] =  ( content[iData].assets[ iAsset ].isNew ) ? "Asset Added" : "Asset Updated";
//					row[ "machineid" ] = content[iData].assets[iAsset].assetId;
//					row[ "providerInstance" ] = content[iData].assets[iAsset].providerInstance;
//					row[ "provider" ] = content[iData].assets[iAsset].providerName;
//					row[ "virtual" ] = ( ( content[ iData ].operation == "IMAGE_IMPORT" ) ? true : false );
//					row[ "status" ] =  content[ iData ].status;
//					row[ "operation" ] = content[ iData ].operation;
//					row[ "operationId" ] = content[ iData ].operationId;
//					row[ "owner" ] = content[ iData ].owner.name;
//					rows.push( row );
//					
//				}
//			}
//			else
//			{
				var row = {};
				row[ "startTime" ] = content[ iData ].startTime;
				row["group"] = group;
				row[ "machine" ] = "\u00a0";
				row[ "machinestate" ] = "\u00a0";
				row[ "provider" ] = "\u00a0";
				row[ "providerInstance" ] = "\u00a0";
				row[ "nomachine" ] = true;
				row[ "status" ] =  content[ iData ].status;
				row[ "operation" ] = content[ iData ].operation;
				row[ "operationId" ] = content[ iData ].operationId;
				row[ "owner" ] = content[ iData ].owner.name;
				rows.push( row );
//			}
		}
		
		return( rows );
    },
    
	getGroupColumnExtraData: function( grouprow, atColumnIndex )
	{
		var button = null;
		
		if( ( grouprow[ "operation" ] == "DISCOVER" ) && ( grouprow[ "status" ] == "IN_PROGRESS" ) )
		{
	        var arg = {
	            label: RtlmUiUtil.getMessage( "Cancel" ),
	            showLabel: false,
				iconClass: "cancel-icon-image",
	            onClick: function( e ) {
					this.clientCallback(this.callbackData, e);
	            }
	        };
			button = new dijit.form.Button( arg );
			
			button.callbackData = grouprow;
			button.clientCallback = this.cancelCallback;
		}
		
		return( ( button ) ? button.domNode : null );
	},

    getCellDisplayExtendedInfo: function(rowData, atColumnIndex)
    {  
        var val = null;
        if(atColumnIndex == 0) {
            val = {};
			var type = ( ( ( rowData.virtual != undefined ) && ( rowData.virtual ) ) ? "VirtualImage" : "Machine" );
			
			var id = this.getRowDataId( rowData );
			
			if (id) {
				val['href'] = AssetUriUtil.getViewAssetDetailURI(type, id);
			}
        }

        return val;
    },
    
    getInitialSortDirection: function()
    {
    	return( TableConstants.SORT_DIR_DESCEND );
    },
    
    getInitialSortByColumn: function()
    {
    	return( 1 );
    },
	
	setCancelCallback: function( method )
	{
		this.cancelCallback = method;
	}      
});

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.providers.InventoryOperationsCellModifier", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier,
{
	constructor: function( listener )
	{
		this.listener = listener;
	},
	
    cellActionInvoked: function(rowData, columnIndex)
    {
    	if( ( columnIndex == 0 ) )
    	{
	    	var panel =	'<div dojoType="dijit.layout.LayoutContainer" style="width: auto; height: 400px">' +
					'<div dojoType="dijit.layout.ContentPane" layoutAlign="client" dojoAttachPoint="textNode">' +
					' <pre dojoAttachPoint="preNode"></pre>' +
					'</div>' +
				'</div>';
	    	
			var dom = new com.ibm.asq.common.web.ui.internal.util.DomConstructor( panel );
	    	var dlg = new dijit.Dialog( { title: "Operation Details" } );
	    	
			var tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer();
			tableViewer.setContentProvider( new com.ibm.rational.test.lm.web.ui.internal.views.providers.OperationsTableProvider() );
			tableViewer.setContentPageSize(15);
			tableViewer.setCellModifier( new com.ibm.rational.test.lm.web.ui.internal.views.providers.OperationDetailsCellModifier( dlg ) );
			tableViewer.setContent( [{resource: "Test Resource", type: "Test", status: "A OK"}] );
			tableViewer.setTableSummary(RtlmUiUtil.getMessage("labelTableSummary", "Operation Details"));
			tableViewer.show();

	    	dom.preNode.appendChild( tableViewer.domNode );
	    	dlg.containerNode.appendChild( dom.domNode );
	    	dlg.show();
	    }
    }
});


dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.providers.OperationsTableProvider", 
             com.ibm.rational.test.lm.web.ui.internal.views.providers.DefaultTableProvider,
{
	constructor: function( column_headings, id_fields )
	{
		this.setParameters( { "column_headings" : this.getColumnHeadings(),
							  "id_fields" : this.getIdFields(),
							  "column_types" : column_types } );
	},
	
	getColumnHeadings : function()
	{
		return( [
		  {title: RtlmUiUtil.getMessage("Resource Name"), native_column: "resource", isSortable: true, includeInFilter: true, width: 33}, 
		  {title: RtlmUiUtil.getMessage("Type"), native_column: "type", isSortable: true, includeInFilter: true, width: 33  },
		  {title: RtlmUiUtil.getMessage("Status"), native_column: "status", isSortable: true, includeInFilter: true, width: 34  }
		] );
	},
	
	getIdFields: function()
	{
		return( "machineid" );
	},
    
    // Summary:
    //    Returns the type of the cell to be displayed. By default, the TableViewer
    //    will support a set of cell types. The type of the cell will map to a specific
    //    cell renderer (to be discussed).
    getCellDisplayType: function(rowData, atColumnIndex)
    {    
        return( TableConstants.RENDER_TYPE_ACTION );
    },

    getCellDisplayExtendedInfo: function(rowData, atColumnIndex)
    {  
        var val = null;
        if(atColumnIndex == 0) {
            val = {};
			var type = ( ( ( rowData.virtual != undefined ) && ( rowData.virtual ) ) ? "VirtualImage" : "Machine" );
			
			var id = this.getRowDataId( rowData );
			
			if (id) {
				val['href'] = AssetUriUtil.getViewAssetDetailURI(type, id);
			}
        }

        return val;
    } 
});

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.providers.OperationDetailsCellModifier", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier,
{
	constructor: function( dialog )
	{
		this.dialog = dialog;
	},
	
    cellActionInvoked: function(rowData, columnIndex)
    {
    	if( ( columnIndex == 0 ) )
    	{
			this.dialog.destroy();
	    }
    }
});

})();