/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.requests.RequestUriUtil");
(function() {

var Root             = "com.ibm.rqm.planning.home.actionDispatcher";
var BaseActionHref   = "#action=" + Root;

// Summary:
//    Utility object to compose URIs for request page's actions.
//
com.ibm.rational.test.lm.web.ui.internal.views.requests.RequestUriUtil = {

    // Summary:
    //     Creates the URI to forward to the request query view. Optionally, 
    //     caller can specify whether to refresh the current query result, and/or
    //     removing a cached view in page.
    //     
    // params:
    //     args = {refresh:[true|false], removeViewId:id}
    //
    getQueryRequestURI: function(args)
    {
        var base = BaseActionHref + "&subAction=manageRequest";
        var uri = base; 
        
        if(args) {
            if(args.refresh)
                uri += "&refresh=true";
            if(args["removeViewId"] != null)
                uri += ("&removeViewId=" + args.removeViewId);
        }
        console.debug(uri);
        return uri;
    },
     
    // Summary:
    //    Creates the URI to forward to the new request view. Optionally, the caller
    //    can pass in extra argument name value pairs.
    //        
    getNewRequestURI: function(args)
    {
        var timestamp = dojo.date.locale.format(new Date(), {selector:"time", timePattern:"hh:mm:ss"});
        var moreArgs = "";
        
        if(args) {
            for(var argName in args) {
                moreArgs += ("&" + argName + "=" + args[argName]); 
            }
        }
        return BaseActionHref + "&subAction=createRequest" + "&id=" + timestamp + moreArgs;
    },

    // Summary:
    //    Creates the URI to forward to the edit request view for the specified request id.  
    //        
    getEditRequestURI: function(id) 
    {
        return BaseActionHref + "&subAction=viewRequest" + "&id=" + id;
    }
    
};
})();
