/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.reservations.QueryReservationWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.QueryResultWidget");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rational.test.lm.web.client.internal.reservations");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.requests.RequestUriUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.providers.ReservationTableProvider"); 


(function() 
{
	var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
    var RequestUriUtil = com.ibm.rational.test.lm.web.ui.internal.views.requests.RequestUriUtil;
    var RtlmUiUtil_getMessage = RtlmUiUtil.getMessage;

    dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.reservations.QueryReservationWidget", 
	      com.ibm.rational.test.lm.web.ui.internal.widgets.QueryResultWidget, 
    {

	ALL_RESERVATION_QUERY_ID: 1,
	ACTIVE_RESERVATION_QUERY_ID: 2,
	FUTURE_RESERVATION_QUERY_ID: 3,
	defaultQuery: null,
	runQueryCallback: null,
	
	constructor: function() {
        this.label = RtlmUiUtil_getMessage("reservations_view_label");
        this.tableId = "RQM-LM-RESERVE-QUERY-TABLEVIWER";		
	},
	
	postCreate: function()
    {
		this.inherited("postCreate", arguments);

		//Refresh Action
		this.refreshButton = this.resultTable.addAction("Refresh", RtlmUiUtil_getMessage("Refresh"), null, "refresh-icon-image", dojo.hitch( this, this.runSelectedQuery ) );
		this.resultTable.setTableListener(this);
        this.resultTable.setContent( [] );
        this.resultTable.show();
        
	},
	
	tableContentActionInvoked: function(tableViewer, actionId, callbackData, callback) 
	{
		callback(callbackData);
	},
    /**
    *   Load the combobox of "Views" with the list of views supplied by the
    *   reservations.getViewIdsForCurrentUser() method. 
    **/
    loadQueryCombo: function()
    {
        //Setup and use the reservations service
        var reservations = new com.ibm.rational.test.lm.web.client.internal.reservations();
        var responseHandler= new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_loadCombo", "_errorHandler");
        reservations.getViewIdsForCurrentUser(responseHandler);                     
    },
	
    _loadCombo: function(response)
    {
		if(response) {
			var selectors = [];
            var response_length = response.length;
            for(var i = 0; i < response_length; i++) {
				var selector = {name:RtlmUiUtil_getMessage(response[i].name), value:response[i].id, def:response[i].id == this.defaultQuery};
				selectors.push(selector);
	        }
			this.resultTable.setTableContentSelectors(selectors);
			this.resultTable.setTableListener(this);

			if(!this.defaultQuery)
			     this.resultTable.contentSelect.options[0].selected = true;
				 
	        // initialize the page to display the default view.
	        this.runSelectedQuery();
			if(this.processEndCallback)
			    this.processEndCallback();
        }    
    },
    
    _errorHandler: function(errorType, errorMessage)
    {

    },
    
    getTableContentProvider: function()
    {
        var provider = new com.ibm.rational.test.lm.web.ui.internal.views.providers.ReservationTableProvider();
		return(provider);
    },    
    
	//Callback from the table when the view selector is changed
	tableContentSelectionChanged : function(table, selectedId)
	{
		this.runSelectedQuery(selectedId)
	},
	
	setRowCheckStateListener: function(listener)
	{
		this.rowCheckStateListener = listener;
	},
    
	//Callback from the table when a row is checked	
	rowCheckStateChanged : function(tableViewer, rowObject, state)
    {
		if (this.rowCheckStateListener)
        	this.rowCheckStateListener.rowCheckStateChanged(tableViewer, rowObject, state);
    },

    runSelectedQuery: function(id)
    {
		//Call the callback if any so the client can update the user that we are starting to load
		if (this.progressStartCallback)
			this.progressStartCallback();
			
        var reservations = new com.ibm.rational.test.lm.web.client.internal.reservations();
        var responseHandler= new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_loadTable", "_errorFunc");
            
        this.resultTable.showLoadingProgress();
    
        //Get the ID of the view that we should use to fill the table
        if (id == null)
			id = this.getQuerySelectionId();
      
        reservations.getViewData(responseHandler, id);
		
		if(this.runQueryCallback)
		  this.runQueryCallback();
    },  
    
    _loadTable: function(response)
    {			
        this.resultTable.endLoadingProgress();
        this.resultTable.setContent( ( response ) ? response : [] );
        this.resultTable.refresh();
		
		//Call the callback if any so the client can update the user that we are done loading
		if (this.progressEndCallback)
			this.progressEndCallback();
    },
    
    _errorFunc: function(errorType, errorMessage) 
    {
		//Call the callback if any so the client can update the user that we are done loading
		if (this.progressEndCallback)
			this.progressEndCallback();
    }

});

})();
