/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.table.AbstractFilteredTableView");
 

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.client.internal.CommonPreferenceClient");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentFilterProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
dojo.require("com.ibm.rational.test.lm.web.client.internal.RtlmInitializer");

(function() {

var ServiceResponseHandler      = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var CommonPreferenceClient		= com.ibm.asq.common.web.client.internal.CommonPreferenceClient
var TableContentFilterProvider  = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentFilterProvider;

var FILTER_CACHE_OBJ_ID			= "com.ibm.rational.test.lm.tableViews.filter.cache";
var RtlmUiUtil 					= com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
var RtlmUiUtil_getMessage       = RtlmUiUtil.getMessage;
var RtlmUiUtil_getRQMMessage    = RtlmUiUtil.getRQMMessage;

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.table.AbstractFilteredTableView", null,
{
	constructor: function() {
		this._typeLabel = "";
		this._searchFunction = null;
		this._searchCallback = null;
		this._searchFilterCallback = null; //allow caller to provide additional filter
		this._searchResponseHandlerType = ServiceResponseHandler;
		
		//get the required user preferences
		var initOpenPref = CommonPreferenceClient.getStoredUserPreferences(
			RtlmUiUtil.USER_UI_CUSTOMISE_QUERYLISTS_FILTER_INITOPEN_CATEGORY.key
		);
		this._initiallyOpen = initOpenPref && !dojo.isArray(initOpenPref) && initOpenPref.value == true;
		
		var onlyMinePref = CommonPreferenceClient.getStoredUserPreferences(
			RtlmUiUtil.USER_UI_CUSTOMISE_QUERYLISTS_ONLY_MY_ASSETS_CATEGORY.key
		);
		this._onlyMineByDefault = onlyMinePref && !dojo.isArray(onlyMinePref) && onlyMinePref.value == true;

		var runOnOpenPref = CommonPreferenceClient.getStoredUserPreferences(
			RtlmUiUtil.USER_UI_CUSTOMISE_QUERYLISTS_AUTORUN_MYQUERY_CATEGORY.key
		);
		this._runOnOpen = runOnOpenPref && !dojo.isArray(runOnOpenPref) && runOnOpenPref.value == true;
	},
	
	isInitiallyOpen: function() {
		return this._initiallyOpen;
	},
	
	isDefaultsToMine: function() {
		return this._onlyMineByDefault;
	},
	
    setTypeLabel: function(/* String */ label) {
        this._typeLabel = label;
    },  
		
	setSearchFunction: function(/* Function */ func) {
		this._searchFunction = func;
	},	
	
    setSearchCallback: function(/* Function */ func) {
        this._searchCallback = func;
    }, 

    setSearchFilterCallback: function(/* Function */ func) {
        this._searchFilterCallback = func;
    },
	
    setResponseHandlerType: function(/* TransportServiceResponseWrapper */ handlerType) {
        this._searchResponseHandlerType = handlerType || ServiceResponseHandler;
    },  
		
    setInProgress: function(msg) {
        if (this.viewHeaderNode) {
			this.viewHeaderNode.setProgressStart(msg);
		}
    },
    
    stopInProgress: function() {
        this.showInitLoadProgress = false;
		if (this.viewHeaderNode) {
			this.viewHeaderNode.setProgressEnd();
		}
		if (this.tableViewer) {
			this.tableViewer.endLoadingProgress();
		}
    },	
	
	cacheFilters: function(filterArgs) {
		var filterCache = dojo.getObject(FILTER_CACHE_OBJ_ID);
		if (filterCache == null) {
			filterCache = {};
		}
		filterCache[this.declaredClass] = filterArgs;
		dojo.setObject(FILTER_CACHE_OBJ_ID, filterCache);
	},
	
	getCachedFilters: function() {
		var filterCache = dojo.getObject(FILTER_CACHE_OBJ_ID);
		if (filterCache != null) {
			return filterCache[this.declaredClass];
		}	
		return null;	
	},
	
	initFilterProvider: function(provider) {
		var cache = this.getCachedFilters();
		if (cache != null) {
			var filters = provider.getFilters() || [];
            var filters_length = filters.length;
			for (var i = 0; i < filters_length; i++) {
				var f = filters[i];
				if (cache[f.name] == null) {
					continue;
				}
				if (f.type == TableContentFilterProvider.FIELD_TOGGLELIST) {
						//set the 'checked' value = true for each selected option
						var defaults = cache[f.name] || [];
                        var f_options = f.params ? f.params.options : null;
                        var f_options_len = f_options ? f_options.length : 0;
						if (defaults.length > 0 && f_options_len > 0) {
							for (var j = 0; j < f_options_len; j++) {
								if (this._findIn(defaults, f_options[j].value) != null) {
									f_options[j].checked = true;
								}
							}
						}
                } else {
						//set the default directly
						f.defaultValue = cache[f.name];
				}
			}
		}
		
	},
	
	_findIn: function(array, value) {
        var array_length = array ? array.length : 0;
		for (var i = 0; i < array_length; i++) {
			if (array[i] == value) {
				return array[i];
			}
		}
		return null;
	},
    
    // method invoked by the filter "Run" action to query with server side filtering. 
    runQuery: function(filterArgs) {
		if (filterArgs != null) {
			this.filterArgs = filterArgs;
		}
		
		//add the current project area to the args
    	this.filterArgs.projectAreaItemId = com.ibm.team.process.web.projectArea.itemId;		

		//cache current filter settings (session persistence of filters)
		this.cacheFilters(this.filterArgs);

        var responseHandler = new this._searchResponseHandlerType(this, "showQueryResults", "showQueryError");          
        
        this.showInitLoadProgress = true;
        this.tableViewer.setContent([]);
        this.tableViewer.refresh();
        this.tableViewer.showLoadingProgress();
        
		this.setInProgress(RtlmUiUtil_getRQMMessage("msgQueryTableList", this._typeLabel));
        this._searchFunction(responseHandler, this.filterArgs);   
    },
	
	// run the query automatically if the preference dictates
    initRunQuery: function() {	
        if (this._runOnOpen) {
            var filterArgs = this.getCachedFilters();
			if (filterArgs == null) {
				filterArgs = {
	                myItemsOnly: this._onlyMineByDefault,
	                includeArchived: false
            	}; 
			}
            this.runQuery(filterArgs);
        }       
    },	
	
	toggleQueryRunning: function(isRunning) {
		
	},
	
	// refresh by simply running the query with the current settings
	refreshQuery: function() {
		if (this.filterArgs && !this._isRunningQuery) {
			this.runQuery(null);
		}
	},
	
	showQueryResults: function(resultsDTO) {
        this.stopInProgress();
		if (resultsDTO != null && resultsDTO.status == RtlmUiUtil.ARTIFACT_SEARCH_STATUS_OK) {
            //search ok. show results
			if (this._searchFilterCallback && dojo.isFunction(this._searchFilterCallback)) {
				resultsDTO.results = this._searchFilterCallback(resultsDTO.results);
			}
			this._doSearchCallback(resultsDTO.results);		  	
		} else if (resultsDTO != null && resultsDTO.status == RtlmUiUtil.ARTIFACT_SEARCH_STATUS_MAX_EXCEEDED) {
            //too many results, show warning dialog
            this.showQueryThresholdWarning(resultsDTO);			
		} else {
            //some error occurred
			this.showQueryError({
                message: resultsDTO ? resultsDTO.message : "Unknown Error"  
            });			
		}
	},
	
    _doSearchCallback: function(searchResults) {
        if(!this.tableViewer || this.tableViewer.uninitialized){
			console.error(this.declaredClass + ": tableviewer not found");
			return;
		}
		if (dojo.isFunction(this._searchCallback)) {
            this._searchCallback(searchResults);
        } else {
            console.error(this.declaredClass + ": no search callback secified!");
        }
    },

    showQueryError: function(error) {
        this._error(error);
        if(!this.tableViewer || this.tableViewer.uninitialized){
			console.error(this.declaredClass + ": tableviewer not found");
			return;
		}
		this.stopInProgress();		
	},
	
	showQueryThresholdWarning: function(resultsDTO) {
        var dialogArgs = {
            primaryTitle: RtlmUiUtil_getRQMMessage("labelSearchLimitWarning", resultsDTO.totalSize),
            yesLabel: RtlmUiUtil_getRQMMessage("labelRefine"),
			yesCallback: dojo.hitch(this, function() {
                //set proper policy; user can refine search
				this.filterArgs.policy = RtlmUiUtil.ARTIFACT_SEARCH_POLICY_ENFORCE;
            }),
			noLabel: RtlmUiUtil_getRQMMessage("labelLoadFirst", resultsDTO.size),
            noCallback: dojo.hitch(this, function() {
				//truncate the search to the threshold
                this.filterArgs.policy = RtlmUiUtil.ARTIFACT_SEARCH_POLICY_TRUNCATE;
				this.runQuery(this.filterArgs);
            }),
			cancelLabel: RtlmUiUtil_getRQMMessage("labelLoadAll"),
            cancelCallback: dojo.hitch(this, function() {
                //run the query anyway, getting all results
                this._dialog.close();
				this.filterArgs.policy = RtlmUiUtil.ARTIFACT_SEARCH_POLICY_IGNORE;
                this.runQuery(this.filterArgs);
            }),		
			showCancel: true,	
            isClosable: false
        };                
        var dlg = new com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog(dialogArgs);
        this._dialog = dlg;
        dlg.setWidth("600px");
        // note that this module is a mixin to dijits, so it is ok to use the 
        // dijit version of connect.
        dlg.connect(dlg._cancelButton, "onclick", dialogArgs.cancelCallback);
		
        var messageNode = document.createElement("span");
        messageNode.innerHTML = RtlmUiUtil_getRQMMessage("msgSearchLimitWarning", resultsDTO.totalSize);
		  	
		dlg.contentContainer.appendChild(messageNode);
        dlg.open();		
	},

	addContributorFilter: function(filterObj, filtersCollection, callback) {
        if (dojo.isArray(this.contributors) && this.contributors.length > 0) {
			this._doAddContributorFilter(this.contributors, filterObj, filtersCollection, callback);
			return;
		}
		//creates a filter with the list of contributors

        var self = this;
        var serviceObj = {
            success: function(contributors) {
				self._doAddContributorFilter(contributors, filterObj, filtersCollection, callback);
            },
            failure: function(e) {
                self._error(e);
				callback();
            }
        };
		var getUsersHandler = new ServiceResponseHandler(serviceObj, "success", "failure");
        com.ibm.rational.test.lm.web.client.internal.RtlmInitializer.updateAllUsers(getUsersHandler);
    },
	
    _doAddContributorFilter: function(contributors, filterObj, filtersCollection, callback) {
        contributors = contributors || [];
		this.contributors = contributors;
        var contributors_length = this.contributors.length;
		for (var i = 0; i < contributors_length; i++) {
            if (contributors[i].name == "Unassigned") {
                contributors[i].name = RtlmUiUtil_getMessage("cfgUnassigned");
            }
            contributors[i].value = contributors[i].itemId;
            contributors[i].checked = false;
        }
        
        var contributorFilter = {
            name: filterObj.name,  // must match key in the arg list of the API
            title: filterObj.label,
            promptTitle: filterObj.prompt,
            type: filterObj.type,
            defaultValue: [],
            params: {options: contributors}
        };
        filtersCollection.push(contributorFilter);
        callback();		
	},
	_error:function(errorObj){
		console.error("Error in AbstractFilteredTableView."+errorObj.message);
	}
});

})();
