/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //   The Directory pane displays a list of client specified entries that are the
 //   "bookmarks" to different locations on the current page. When an entry in the pane is
 //   clicked, the user is scrolled to the location on the page. It's subclassed from the
 //   StationaryPane which can track the current scroll location of the page and always keeps
 //   the pane in view (by auto sliding up or down). The user can disable the tracking. 
 //   
 //    
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.DirectoryPane");

dojo.require("dijit._base.scroll");
dojo.require("dojox.data.dom");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.StationaryPane");

dojo.require("com.ibm.asq.common.web.client.internal.Constants");

( function() {

var Constants = com.ibm.asq.common.web.client.internal.Constants;

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.DirectoryPane",
             com.ibm.asq.common.web.ui.internal.widgets.StationaryPane,
{   
    // initializes the parent class's attributes 
    title: Constants.getMessage("titleTOC"),
    trackMessage: Constants.getMessage("msgUnpinView"),
    untrackMessage: Constants.getMessage("msgPinView"),
	skipto: Constants.getMessage("skipto"),
	labelEmpty: Constants.getMessage("Empty"),
	        
    constructor: function() {
        this.entries = [];
        this.showAll = false;
    },
    
    postCreate: function() 
    {
        this.inherited("postCreate", arguments);
        
        if(this.entries) {
            this.setEntries(this.entries);
        }        
    },
    
    // Summary:
    //    loads the table of contents entries in the pane. This method expects
    //    an array of entries info to be passed. The valid data is as follows:
    //
    //    var entries = [
    //       {title: "name of the entry",
    //        image: "image src ref", (Optional to create an image for the entry)
    //
    //     provide one or the other (either a dom node for the entry, or an anchor's tag)
    //
    //        node: domNodRef, (Optional) the dom node of the entry to scroll to
    //        tag: "anchorTag" (Optional) the anchor's name to scroll to   
    //    ];
    //    
    //
    setEntries: function(entries)
    {
        this.entries = entries;           
        dojox.data.dom.removeChildren(this.containerNode);
        var doc=document;
        var entriesNode = doc.createElement("ul");
        this.entriesNode=entriesNode;
        entriesNode.className = "entries";
        
        if(entries) {
        	var length=entries.length;            
            for(var i = 0; i < length; i++) {
            	var entry=entries[i];               
                var aLink = doc.createElement("a");   
				aLink.setAttribute("href", "javascript:{}");             
                if(entry.image) {
                    var img = doc.createElement("img");
                    img.src = entry.image;
                    aLink.appendChild(img);
                }
                if(entry.title) {
					var node = doc.createElement("span");
					var style=node.style;
					style.visibility = "hidden";
					style.position = "absolute";
					style.display = "inline";
					node.innerHTML = this.skipto;
					aLink.appendChild(node);
                    aLink.appendChild(doc.createTextNode(this.getWordWrappedString( this.containerNode, this.rulerNode, entry.title )));
                }
                if(entry.tag) {
                    //aLink.href = window.location + "#" + this.entries[i].tag;
                    this.connect(aLink, "onclick", dojo.hitch(this, "selectEntry", i));
                }
                /*
                if(entry.node) {
                    aLink.refNode = entry.node;
                    this.connect(aLink, "onclick", function() {
                        dijit.scrollIntoView(this.refNode);
                    });
                }
                */
                var li = doc.createElement("li");
                li.appendChild(aLink);
                entriesNode.appendChild(li);
            }                        
        } else {
            var li = doc.createElement("li");
            li.innerHTML = this.labelEmpty;
            entriesNode.appendChild(li);
        }
        this.containerNode.appendChild(entriesNode);
        
        //add the show all option if its enabled
        if (this.showAllCallback != null) {
            this.addShowAllOption();    
        }        
    },
    
    selectEntry: function(index) {
    	var winEvent=window.event;  
		if( winEvent )
		{
			//This code will only run under IE.  It prevents the link from propagating to perform
			//href action for the link.  In this way it will prevent the dialog from coming up telling
			//the user that they are about to leave the site.  
	    	winEvent.cancelBubble = true;
			winEvent.returnValue = false;
		} 
		
        this.selectedIndex = index;  
        this.highlightSelected(index);
        if (this.entries && this.entries[index] && this.entries[index].callback) {
            this.entries[index].callback();
        }
    },
    
    selectEntryByTag: function(tag) {
    	var entries=this.entries;  
        if (entries) {
        	var length=entries.length;
        	for (var i=0; i<length; i++) {
        		var entry = entries[i];
        		if (entry.tag==tag) {
        			this.selectEntry(i);
        		}
        	}
        }
    },
    
    highlightSelected: function(selectedIndex) {
        this.selectedIndex = selectedIndex;
        if (!this.entriesNode || !this.entriesNode.childNodes) {
            return;
        }
        var childNodes=this.entriesNode.childNodes;
        var length=childNodes.length;
        for (var i = 0; i < length; i++) {
            if (i == selectedIndex) {
                dojo.addClass(childNodes[i], "selected");
            } else {
                dojo.removeClass(childNodes[i], "selected");
            }
        }
    },
    
    addShowAllOption: function() {
		this._showAllOption = document.createElement("input");
        this._showAllOption.type = "checkbox";
		this._showAllOption.setAttribute("tabindex", "0");
		this._showAllOption.id = "showAllCheckbox";
		this.connect(this._showAllOption, "onclick", "toggleShowAll");
		
        var ul = document.createElement("ul");
        dojo.addClass(ul, "entries");        
        var li = document.createElement("li");
		var inputLabel = document.createElement("label");
		inputLabel.innerHTML = Constants.getMessage("labelShowAllSections");
		inputLabel.setAttribute("for", "showAllCheckbox");
        
        var container = document.createElement("table");
        dojo.style(container, "margin", "0px");
        dojo.style(container, "padding", "0px");
        var row = container.insertRow(0);
        var toggleCol = row.insertCell(0);
        toggleCol.appendChild(this._showAllOption);
        var labelCol = row.insertCell(1);
        labelCol.appendChild(inputLabel);

        li.appendChild(container);
        dojo.addClass(li, "show-all-option");
        ul.appendChild(li);
        
        this.containerNode.appendChild(ul);
		
		// fixing defect 9893
		this._showAllOption.checked = this.showAll;
    },
    
    toggleShowAll: function(event) {
        this.showAll = !this.showAll;
        this.showAllCallback(this.showAll, this.entries[this.selectedIndex].tag);
        dojo.publish("com-ibm-asq.common-CollapsibleSection-sectionSizeChanged", null);
    }
});

})();
