/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
//

//
// author: Mark Victory
//
// Summary: This is a widget to choose options.
//

dojo.provide("com.ibm.rational.test.lm.web.ui.internal.widgets.FilteredListChooser");

dojo.require( "dijit._Widget" );
dojo.require( "dijit._Templated" );
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");

var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
var RtlmUiUtil_getMessage = RtlmUiUtil.getMessage;

(function() {
dojo.declare(
    "com.ibm.rational.test.lm.web.ui.internal.widgets.FilteredListChooser",
    [dijit._Widget, dijit._Templated],
{
	
	templateString: '<div id="${id}" dojoAttachPoint="domNode" tabindex="-1">' +
					' <table border="0">' +
					'  <tr>' +
					'    <td>'+
					'       <label><input type="text" dojoAttachPoint="filterBox" size="64" dojoAttachEvent="onkeyup:_onFilterChange"></label>'+
					'    </td>' +
					'  </tr>' +
					'  <tr>' +
					'    <td>'+
					'      <label><select dojoAttachPoint="selectionList" name="selectionList" class="optionSelectWide" multiple="true" size="20" dojoAttachEvent="onchange: _onChange">' +
					'      </select></label>' +
					'    </td>' +
					'  </tr>'+
					'</table>'+
					' <div align="center">' +
					' <table class="chooser-button-area" dojoAttachPoint="fsButtonTableNode" border="0">' +
					'  <tr>' +
					'   <td class="action-button-cell">' +
					'    <button dojoAttachPoint="fsOKNode" class="option-action-button" dojoAttachEvent="onclick: _okAction"></button>' +
					'   </td>' +
					'   <td class="action-button-cell">' +
					'    <button dojoAttachPoint="fsCancelNode" class="option-action-button" dojoAttachEvent="onclick: _cancelAction"></button>' +
					'   </td>' +
					'  </tr>' +
					' </table>' +
					' </div>' +					
	                '</div>',
					
											
		itemList:null,
		multiselect:true,
		okListener:null,
		cancelListener:null,
		
		postCreate :function()
		{
			this.inherited("postCreate", arguments);
			this.fsOKNode.innerHTML = RtlmUiUtil_getMessage("buttonLabelOK");
			this.fsCancelNode.innerHTML = RtlmUiUtil_getMessage("labelCancelButton");
			this.fsButtonTableNode.style.display = "none"
		},
		
		uninitialize: function()
        {
            if (this.options) {
                delete this.options;
            }
        },
        
		addOption: function(id, name, description)
		{
			if( !this.options )
			{
				this.options = new Array();
			}
            var option = new FilteredListChooser.Option( id, name, description );
			this.options.push(option);
			this._showOption(id, name)
		
		},
		
		_showOption :function(id, displayName)
		{
            var option = new Option( displayName, id );
			this.selectionList.options[ this.selectionList.options.length ] = option;
		},
		
		enableOkCancel: function()
		{
			this.fsButtonTableNode.style.display = "inline";
		},
		
		_okAction: function( event )
		{
			var temp = this.getSelectedItems();
			if( this.oklistener )
				this.oklistener( event );
		},
		
		_cancelAction: function( event )
		{
			if( this.cancellistener )
				this.cancellistener( event );
		},
		
		_onChange: function()
		{
			if (this.changeListener)
				this.changeListener();
		},
		
		addOkListener: function( listener )
		{
			if( listener )
			{
				this.oklistener = listener;
			}
		},
		
		addCancelListener: function( listener )
		{
			if( listener )
			{
				this.cancellistener = listener;
			}			
		},
		
		addChangeListener: function(listener)
		{
			if (listener)
				this.changeListener = listener;
		},
		
		_onFilterChange :function()
		{
			var str = this.filterBox.value;
			this._clearList();
			this._doFilter(str);
		},
		
		_clearList:function()
		{
            var options_length = this.selectionList.options.length;
			for (var i =0; i < options_length; i++)
				this.selectionList.remove(0);
		},
		
		_doFilter: function(filter)
		{
            var options_length = this.options.length;
			for (var i=0; i < options_length; i++)
			{
				var ItemLabel = this.options[i].name;
				if (ItemLabel.indexOf(filter) >= 0)
					this._showOption(this.options[i].id, ItemLabel);
			}
		},
		
		getSelectedItems: function()
		{
			var retVal = new Array();
			
            var options_length = this.selectionList.options.length;
			for (var i = 0; i < options_length; i++)
			{
				if (this.selectionList.options[i].selected == true)
				{
					var id = this.selectionList.options[i].value;
					var name = this.selectionList.options[i].text;
                    var option = new FilteredListChooser.Option(id, name);
					retVal.push(option);
				}
			}
			return retVal;
		},
		
		setMultiSelect :function(value)
		{
			this.selectionList.multiple=value;
		},
		
		selectItem : function(item)
		{
			this.selectionList.selectedIndex = item;
		}
}
);
		
dojo.declare( "FilteredListChooser.Option", null,
{
	id: null,
	name: "",
	description: "",	
	constructor: function(id, name, description )
	{
		this.name = name;
		this.id = id;
		this.description=description;
	} 	
}	
);
})();
