/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
//
// pc 09/08/09
// This file doesn't seem to be used by any JS file.
//
//
// Author: P. Chu
//
// Summary:
//   The MachineConfigPanel display the machine configuration in a table format.
//   Each row in the table contains a parameter with the attribute name, its
//   operator, and value.  Depending on the state of the MachineConfigPanel,
//   the user can add or remove the parameters and manipulate the attributes,
//   the operators, and values.
//
//   The keywords are fixed for definitions of the parameters but the contents
//   are not.  The consumer of this widget specifies as many or little parameters
//   in a configuration with as many or little attributes, operators, and values.
//   This widget will deermine the appropriate element to use such as using a
//   span for a single operator while using a select combo box for value of
//   enum type.
//
//   The purpose is to generalize the machine config panel as a simple field
//   which the consumer can simply initialize it, set it to some value, and
//   and get the current value.
//
// Usage (attributes):
//   dojoType - com.ibm.rational.test.lm.web.ui.internal.widgets.MachineConfigPanel
//   title - The title string in the header.
//   configOptions - The data to initialize the widgets i.e. the array of param
//                   attributes with their operations and possible values.
//                   The widget will be useless without this.  If not specified
//                   in the attribute, the method "setConfigOptions" must be called
//                   before using the widget.
//   configData - The array of config(s) to seed the panel.
//   state - The state of the config panel which can be EDITABLE (default),
//           READONLY, or SELECTABLE.
//   supportMulti - The default is false, not to support multiple configurations.
//   newConfigLabel - The label for the new configuration action.  If not specified,
//                    it will use the one defined in the message bundle.
//   newConfigEnableImgSrc - If specified, the icon will be used instead of the label
//                           for the new configuration action.
//   newConfigDisableImgSrc - If specified, hover over icon change will be supported
//                            for the new configuration action.
//   addParamLabel - The label for the add parameter action.  If not specified,
//                   it will use the one defined in the message bundle.
//   addParamEnableImgSrc - If specified, the icon will be used instead of the label
//                          for the add parameter action.
//   addParamDisableImgSrc - If specified, hover over icon change will be supported
//                           for the add parameter action.
//   removeParamLabel - The label for the remove parameter action.  If not specified,
//                      it will use the one defined in the message bundle.
//   removeParamEnableImgSrc - If specified, the icon will be used instead of the label
//                             for the remove parameter action.
//   removeParamDisableImgSrc - If specified, hover over icon change will be supported
//                              for the remove parameter action.
//   msgBundle - The message bundle to get messages from.  If not specified in the attribute,
//               the method "setMessageBundle" must be called in order to get proper message
//               string definition.
//   onChange - The event hook called when elements in the panel are changed. Container
//              of this widget will use this hook for listening to changes.
//   baseClass - A base class as prefix for group of classes for various elements in the widget.
//               Classes for various parts can be overridden with the following attributes.
//   headerClass - The header div area for title and new config action.
//   descriptionClass - The span under the header for description.
//   containerClass - The div for the content area i.e. the container div without the header.
//   actionLinkClass - The action as a link.
//   actionButtonClass - The action as a button.
//   configTableClass - The table that hold the config params.
//   attrNameSelectClass - The select combo within a config for the param name attribute.
//   selectClass - The select combo within a config param table.
//   labelClass - The span label within a config param table.
//   textFieldClass - The editable text field within a config param table.
//   checkboxClass - The checkbox within a config param table.
//   csHelpClass - the CSS class for the CS Help button
//   csHelpIconClass - the icon class for the CS Help button
//   csHelpContainerClass - the CSS class for the container DIV of the CS Help button
//   csHelpOnClick - must specify the CS help handler or it won't show the help button
//   tabIndex - tab index for the widget.
//
// Usage (methods):
//   setTitle(title) - Set the header title.
//   setConfigOptions(options) - Set the config parameters options.
//   setMessageBundle(bundle) - Set the message bundle to use for messages.
//   setConfigs(state, newConfigs) - Set the configs to be displayed in the
//                                   widget in specific state.  Use null to
//                                   clear the widget.
//   getConfigs() - Get the configs data from the widget.
//   isEdited() - Return true of false to check if there is any changes in
//                the config.
//   getError() - Check the widget for error and input validation for invalid
//                error input.  Empty string is return if there is no error.
//   getWarning() - Validate the config param attributes in the table and
//                  return the warning .  It is just warning of incomplete
//                  params that are being ignored.  Empty string is return
//                  if there is no warning.  Note that a config can have
//                  error but no warning.
//
// Known Issues:
//   The isEdited() method looks into the config data in the UI and compares
//   it with the original data.  It is possible that the user changed the
//   param attributes (rows) order in the config (table) so that the UI
//   elements display the edited CSS style while the isEdited() method returns
//   false for no change.
//   
  
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.widgets.MachineConfigPanel");

dojo.require("dijit.layout.ContentPane");
dojo.require("dijit.form.ValidationTextBox");
dojo.require("dijit.form.Button");
dojo.require("dijit._Templated");

dojo.declare(
  "com.ibm.rational.test.lm.web.ui.internal.widgets.MachineConfigPanel",
  [dijit.layout.ContentPane, dijit._Templated],
{
  templateString: '<div id="${id}" dojoAttachPoint="domNode" tabindex="-1">' +
                    '<table border="0" cellpadding="0" cellspacing="0" width="100%">' +
                      '<tr dojoAttachPoint="headerRowNode">' +
                        '<td align="left"><div dojoAttachPoint="headerNode">${title}</div></td>' +
                        '<td align="right" width="100%"><div dojoAttachPoint="actionsNode" align="right">&nbsp;</div></td>' +
                      '</tr>' +
                      '<tr><td colspan="2">' +
                      '<div dojoAttachPoint="containerNode" waiRole="region" style="overflow:visible" tabindex="-1">' +
                        '<div dojoAttachPoint="descriptionContainer" style="display: none;" tabindex="-1">' +
                          '<table border="0" cellpadding="0" cellspacing="0" width="100%">' +
                            '<tr>' +
                              '<td nowrap><div dojoAttachPoint="descriptionNode" tabindex="-1"></div></td>' +
                              '<td align="right"><div dojoAttachPoint="csHelpContainer"></div></td>' +
                            '</tr>' +
                          '</table>' +
                        '</div>' +
                        '<div dojoAttachPoint="configsNode" style="display: none;" tabindex="-1"></div>' +
                        '<div dojoAttachPoint="configsActionContainer" style="display: none;" tabindex="-1">' +
                          '<table border="0" cellpadding="0" cellspacing="0" width="100%">' +
                            /* wrap in a table to prevent the scroll bar showing up */
                            /* with size change from hiding and showing dynamically */
                            '<tr><td><div dojoAttachPoint="configsActionNode" tabindex="-1"></div></td></tr>' +
                          '</table>' +
                        '</div>' +
                      '</div>' +
                      '</td></tr>' +
                    '</table>' +
                  '</div>',

  /* the data to initialize the widgets i.e. the array of parameters with their operations and possible values */
  configOptions: null,
  
  /* the config(s) to seed the panel */
  configData: null,
  
  /* the state of the config panel */
  state: "EDITABLE",
  
  /* the state that the config can be in which affects the display */
  EDITABLE: "EDITABLE",
  READONLY: "READONLY",
  SELECTABLE: "SELECTABLE",  // read only with check boxes to select param

  /* the panel can support a single config or multiple configs */
  supportMulti: false,
  
  title: "",
  
  /* a main class as prefix for group of classes for various parts */
  baseClass: null,
  
  /* classes for various parts to allow override particular part of main class */
  headerClass: null,       // header div area for title and action
  headerLabelClass: null,  // header div area for title and action
  descriptionClass: null,  // span under the header for description
  containerClass: null,    // div for the content area i.e. the container div without the header
  actionLinkClass: null,   // action using link
  actionButtonClass: null, // action using button
  configTableClass: null,  // the table that hold the config params
  attrNameSelectClass: null,   // the select combo within a config for the param name attribute
  selectClass: null,       // the select combo within a config
  labelClass: null,        // the span label within a config
  textFieldClass: null,    // the editable text field within a config
  checkboxClass: null,     // the checkbox within a config
  
  /* CS Help button */
  _csHelp: null,
  csHelpClass: null,
  csHelpIconClass: null,
  csHelpContainerClass: null,
  csHelpOnClick: null,         // must specify the handler or it won't dhow the help button
  
  
  /* block of 500 tab index allocated for this widget only if it is specified to use tab index */
  tabIndex: 0,
  _currentTabIndex: 0,
  _allocatedTabIndexBlock: 500,
  
  /* action links for the config */
  _newConfigAction: null,
  _addParamAction: null,
  
  /* optional icons for buttons */
  newConfigLabel: "",
  newConfigEnableImgSrc: null,
  newConfigDisableImgSrc: null,
  addParamLabel: "",
  addParamEnableImgSrc: null,
  addParamDisableImgSrc: null,
  removeParamLabel: "",
  removeParamEnableImgSrc: null,
  removeParamDisableImgSrc: null,
  
  /* message bundle to get messages from */
  msgBundle: null,
  
  /* event hook called when elements in the panel are changed */
  onChange: function(evt) {},
  
  /* array of the parameter options for the config*/
  _paramsOptions: {},
  
  /* the array of parameters that are not chosen yet in editable state */
  _unsetParams: {},

  /* the array of parameters that are ordered to display in the combo box */
  _orderedParams: {},
  
  /* array of original values of elements in the config */
  _originalValues: {},
  
  /* array of validation text box widget that should be checked for invalid error input */
  _validationWidgets: [],
  
  /* track max label length to determine element width */
  _attrNameMaxLength: 0,
  _opMaxLength: 0,
  _valComboMaxLength: 0,
  _valTextMaxLength: 0,
  
  /* track if a unspecified param attribute is aleady on display to hide the "add param" action */
  _unspecifiedAttrParam: false,
  
  /* a unique id for composing the table row element id */
  _tableRowId: 0,
  
  /* error message of the widget */
  _errorMessage: "",
  
  postCreate: function(){
    com.ibm.rational.test.lm.web.ui.internal.widgets.MachineConfigPanel.superclass.postCreate.apply(this, arguments);
    
    // initialize the config parameters options array
    this.setConfigOptions(this.configOptions);

    
    // a main class as prefix for group of classes for various parts
    var baseClass = this.baseClass;
    if (baseClass != null) {
      this.headerRowNode.className = baseClass + "HeaderClass";
      this.headerNode.className = baseClass + "HeaderClass";
      this.containerNode.className = baseClass + "ContainerClass";
      this.descriptionNode.className = baseClass + "DescriptionClass";
      this.csHelpContainer.classname = baseClass + "CsHelpContainerClass";

      if (!this.actionLinkClass)
        this.actionLinkClass = baseClass + "ActionLinkClass";
      if (!this.actionButtonClass)
        this.actionButtonClass = baseClass + "ActionButtonClass";
      if (!this.configTableClass)
        this.configTableClass = baseClass + "ConfigTableClass";
      if (!this.selectClass)
        this.selectClass = baseClass + "SelectClass";
      if (!this.attrNameSelectClass)
        this.attrNameSelectClass = baseClass + "AttrNameSelectClass";
      if (!this.labelClass)
        this.labelClass = baseClass + "LabelClass";
      if (!this.textFieldClass)
        this.textFieldClass = baseClass + "TextFieldClass";
      if (!this.checkboxClass)
        this.checkboxClass = baseClass + "CheckboxClass";
    }

    // allow override the class of individual part from the main class
    if (this.headerClass != null) {
      this.headerRowNode.className = this.headerClass;
    }
    if(this.headerLabelClass != null) {
      this.headerNode.className = this.headerLabelClass;
    }
    if (this.containerClass != null) {
      this.containerNode.className = this.containerClass;
    }
    if (this.descriptionClass != null) {
      this.descriptionNode.className = this.descriptionClass;
    }
    if (this.csHelpContainerClass != null) {
      this.csHelpContainer.className = this.csHelpContainerClass;
    }
    
    // create the new config action in the header
    var label = this.newConfigLabel ? this.newConfigLabel : this._getMessageFromBundle(this.MSG_KEY_NEW_CONFIG);
    var doc = document;
    var _newConfigAction; 
    if (this.newConfigEnableImgSrc) {
      // if icon is specified, use image button
      _newConfigAction = doc.createElement("button");
      this._newConfigAction = _newConfigAction;
      var img = doc.createElement("img");

      if (this.newConfigDisableImgSrc) {
        // disable icon is specified too, do hovering
        img.src = this.newConfigDisableImgSrc;
        this.connect(_newConfigAction, "onmouseover", "_onMouseOverNewConfigHandler");
        this.connect(_newConfigAction, "onmouseout", "_onMouseOutNewConfigHandler");
      } else {
        // set icon for the image image
        img.src = this.newConfigEnableImgSrc;
      }
      
      img.alt = label;
      _newConfigAction.appendChild(img);
      if (this.actionButtonClass)
        _newConfigAction.className = this.actionButtonClass;
    } else {
      // use action link
      _newConfigAction = doc.createElement("a");
      this._newConfigAction = _newConfigAction;
      _newConfigAction.innerHTML = label;
      this.connect(_newConfigAction, "onkeypress", "_newConfigActionKeyHandler");
      if (this.actionLinkClass)
        _newConfigAction.className = this.actionLinkClass;
    }
    this.connect(_newConfigAction, "onclick", "_newConfigActionHandler");

    this.actionsNode.appendChild(_newConfigAction);
    
    // create the add param action below the config area
    var label = this.addParamLabel ? this.addParamLabel : this._getMessageFromBundle(this.MSG_KEY_ADD_PARAM);
    var _addParamAction;
    if (this.addParamEnableImgSrc) {
      // if icon is specified, use image button
      _addParamAction = doc.createElement("button");
      this._addParamAction = _addParamAction;
      var img = doc.createElement("img");
      if (this.addParamDisableImgSrc) {
        // disable icon is specified too, do hovering
        img.src = this.addParamDisableImgSrc;
        this.connect(_addParamAction, "onmouseover", "_onMouseOverAddParamHandler");
        this.connect(_addParamAction, "onmouseout", "_onMouseOutAddParamHandler");
      } else {
        img.src = this.addParamEnableImgSrc;
      }
      img.alt = label;
      _addParamAction.appendChild(img);
      if (this.actionButtonClass)
        _addParamAction.className = this.actionButtonClass;
    } else {
      // use action link
      _addParamAction = doc.createElement("a");
      this._addParamAction = _addParamAction;
      _addParamAction.innerHTML = label;
      this.connect(_addParamAction, "onkeypress", "_addParamActionKeyHandler");
      if (this.actionLinkClass)
        _addParamAction.className = this.actionLinkClass;
    }
    this.connect(_addParamAction, "onclick", "_addParamActionHandler");
    this.configsActionNode.appendChild(_addParamAction);

    // create the CS help button
    if (this.csHelpOnClick) {
      var params = {
        id: this.id + "_csHelpConfig",
        onClick: this.csHelpOnClick,
        iconClass: this.csHelpIconClass ? this.csHelpIconClass : ""
      };
      this._csHelp = new dijit.form.Button(params, this.csHelpContainer);
      
      if (this.csHelpClass)
        dojo.addClass(this._csHelp.domNode, this.csHelpClass);
    }

    // allocate a block of tab indexes for internal elements of this widget
    if (this.tabIndex > 0) {
      this._currentTabIndex = this.tabIndex + 1;
      
      // get all elements in the page
      var allElements = doc.getElementsByTagName("*");
      
      // loop through all elements to push out elements after this widget
      for (var i = 0; i < allElements.length; i++) {
        if (allElements[i].tabIndex > this.tabIndex) {
        var blah = allElements[i];
          allElements[i].tabIndex += this._allocatedTabIndexBlock;
        }
      }
      
      // the new config action is the first item of the widget
      this._newConfigAction.tabIndex = this.tabIndex;

      // the add param action is the last item of the widget
      _addParamAction.tabIndex = this.tabIndex + this._allocatedTabIndexBlock;
    }
  },
  
  // Set the title of the machine config panel.
  //
  setTitle: function(title) {
    this.titleNode.innerHTML = title;
  },
  
  // Set the config parameters options, i.e. the array of attributes for the parameters and
  // their possible array of operations and values choices.  For example:
  //    var options = [];
  //    var param = {};
  //    param["attrName"] = "OperatingSystem";
  //    param["displayName"] = "Operation System";
  //    param["type"] = "enum";
  //    param["opValues"] = ["="];
  //    param["choices"] = ["Windows XP", "Windows 2000", "Windows Sever 2003"];;
  //    options.push(param);
  //
  //    param["attrName"] = "Memory";
  //    param["displayName"] = "System Memory";
  //    param["type"] = "integer";
  //    param["opValues"] = ["=", ">="];
  //    options.push(param);
  //
  setConfigOptions: function(options) {
    if (options) {
      // make a copy of the input array
      this.configOptions = options.slice();
    
      this._attrNameMaxLength = 0;
      this._opMaxLength = 0;
      this._valComboMaxLength = 0;
      this._valTextMaxLength = 0;

      if (this.configOptions) {
        for (var i = 0; i < this.configOptions.length; i++) {
          // initialize the associative array of parameter id for all parameter options
          var attrId = this.configOptions[i][this.CONFIG_PARAM_OPTIONS_ATTR_NAME];
          this._paramsOptions[attrId] = this.configOptions[i];

          // get the max string length for the param attribute display name
          var name = this.configOptions[i][this.CONFIG_PARAM_OPTIONS_ATTR_DISPLAY_NAME];
          if (name.length > this._attrNameMaxLength) {
            this._attrNameMaxLength = name.length;
          }
          
          // get the max string length for the operator
          var opArray = this.configOptions[i][this.CONFIG_PARAM_OPTIONS_OP_OPTIONS];
          var opCombo = false;
          // operator can be really short like "=" and ">=" so there is no padding from the
          // extra character height than the width.
          if (opArray.length > 1) {
            opCombo = true;
          }
          for (var j = 0; j < opArray.length; j++) {
            if (opArray[j].length > this._opMaxLength) {
              this._opMaxLength = opArray[j].length;
            }
          }
          if (opCombo) {
            // add length for the dropdown icon for combo box
            this._opMaxLength += 1;
          }
          // get the max string length for the values if it is enum type
          if (this.configOptions[i][this.CONFIG_PARAM_OPTIONS_VALUE_TYPE] == this.CONFIG_PARAM_OPTIONS_VALUE_ENUM_TYPE) {
            var valArray = this.configOptions[i][this.CONFIG_PARAM_OPTIONS_VALUE_OPTIONS];
            for (var k = 0; k < valArray.length; k++) {
              if (valArray[k].length > this._valComboMaxLength) {
                this._valComboMaxLength = valArray[k].length;
              }
            }
          }
        }
        if (this._attrNameMaxLength > 5) {
          // character height is larger width, shorten for better fit
          this._attrNameMaxLength = this._attrNameMaxLength * 3 / 4;
        }
        if (this._valComboMaxLength > 5) {
          // it is the browsers... text field is a bit longer than combobox
          this._valComboMaxLength = this._valComboMaxLength * 3 / 4;
        }
        if (this._valComboMaxLength > 0) {
          // it is the browsers... text field is a bit longer than combobox
          this._valTextMaxLength = this._valComboMaxLength - 0.9;
        }
      }
    }
  },
  
  // The associative array _paramsOptions holds the actual data of the params and
  // is initialized in the method setConfigOptions() from user specified array.
  // The associative array _unsetParams serves as a list of params that have not
  // been set to use in the config param table.  When a new empty config is created,
  // the _unsetParams array is the entire list of params to create the combobox to
  // set a specific attribute.  Once the attribute of the param is set, the param
  // is removed from the _unsetParams array so the next row of param created for
  // the config will not be duplicated.  The associative array _orderedParams
  // maintains the orders of the param so the combobox options pull down menu
  // can have a consistent order when the options are dynamically added and removed.
  //
  _initConfigOptions: function() {
    this._unsetParams = {};
    
    var index = 0;
    for (var paramId in this._paramsOptions) {
      // set index as entry value to maintain order of the array
      this._orderedParams[paramId] = index++;
      // all param are default to be unset
      this._unsetParams[paramId] = paramId;
    }
  },
  
  // Handler for the new config action link on key press.
  //
  _newConfigActionKeyHandler: function(evt) {
    var charCode = (evt.which) ? evt.which : evt.keyCode
    // skip if not spacebar or enter key pressed
    if ((charCode == 32) || (charCode == 13)) {
      this._newConfigActionHandler();
    }
  },

  // Handler for the new config action link.
  //
  _newConfigActionHandler: function() {
    // set a empty config with editable state
    this.setConfigs(this.EDITABLE, null);
        
    // update the header action state and description message
    this._updateHeaderDesc(true);
    
    // create the config param table but will hide it until add param action is invoked
    var table = this._createConfigTable(this.id);
    this.configsNode.appendChild(table);
    
    // associate the table to the add param action
    this._addParamAction.data = table;

    // trigger the onChange event hook
    this.onChange();
    
    // set focus to the add param action
    this._addParamAction.focus();
  },
  
  // Handler for the add param action link on key press.
  //
  _addParamActionKeyHandler: function(evt) {
    var charCode = (evt.which) ? evt.which : evt.keyCode
    // skip if not spacebar or enter key pressed
    if ((charCode == 32) || (charCode == 13)) {
      this._addParamActionHandler();
    }
  },

  // Handler for the add param action link.
  //
  _addParamActionHandler: function() {
    // add a row of param in the config param table cached with the action
    var table = this._addParamAction.data;
    var rowIndex = table.rows.length;
    var row = table.insertRow(rowIndex);
    
    // single config only, use zero as config index to compose prefix
    var idPrefix = this._constructElemIdPrefix(0);
    // cache the prefix in the table row
    row.data = idPrefix;
    // create the param attribute elements
    this._createEditableParam(idPrefix, null, row);
    // display the confiog table
    dojo.style(this.configsNode, "display", "inline");
    // hide the "add param" action link until the user uses the newly created param
    dojo.style(this.configsActionContainer, "display", "none");
    // trigger the onChange event hook

    this.onChange();
  },
  
  _onMouseOverNewConfigHandler: function(evt) {
    // update the button icon on hover
    if (this.newConfigEnableImgSrc && this.newConfigDisableImgSrc) {
      var children = dojo.isIE ? this._newConfigAction.children : this._newConfigAction.childNodes;
      var img = (children.length > 0) ? children[0] : null;
      if (img && img.nodeName.toUpperCase() == "IMG")
        img.src = this.newConfigEnableImgSrc;
    }
  },
  
  _onMouseOutNewConfigHandler: function(evt) {
    // update the button icon on hover
    if (this.newConfigEnableImgSrc && this.newConfigDisableImgSrc) {
      var children = dojo.isIE ? this._addParamAction.children : this._newConfigAction.childNodes;
      var img = (children.length > 0) ? children[0] : null;
      if (img && img.nodeName.toUpperCase() == "IMG")
        img.src = this.newConfigDisableImgSrc;
    }
  },
  
  _onMouseOverRemoveParamHandler: function(evt) {
    // update the button icon on hover
    if (this.removeParamEnableImgSrc && this.removeParamDisableImgSrc) {
      // get the button... hate this IE and FF browsers differences
      var mouseEvent = evt ? evt : window.event;
      var button = mouseEvent.target ? mouseEvent.target : mouseEvent.srcElement;
      var children = dojo.isIE ? button.children : button.childNodes;
      var img = (children.length > 0) ? children[0] : null;
      if (img && img.nodeName.toUpperCase() == "IMG")
        img.src = this.removeParamEnableImgSrc;
    }
  },
  
  _onMouseOutRemoveParamHandler: function(evt) {
    // update the button icon on hover
    if (this.removeParamEnableImgSrc && this.removeParamDisableImgSrc) {
      // get the button... hate this IE and FF browsers differences
      var mouseEvent = evt ? evt : window.event;
      var button = mouseEvent.target ? mouseEvent.target : mouseEvent.srcElement;
      var children = dojo.isIE ? button.children : button.childNodes;
      var img = (children.length > 0) ? children[0] : null;
      if (img && img.nodeName.toUpperCase() == "IMG")
        img.src = this.removeParamDisableImgSrc;
    }
  },
  
  _onMouseOverAddParamHandler: function(evt) {
    // update the button icon on hover
    if (this.addParamEnableImgSrc && this.addParamDisableImgSrc) {
      var children = dojo.isIE ? this._addParamAction.children : this._addParamAction.childNodes;
      var img = (children.length > 0) ? children[0] : null;
      if (img && img.nodeName.toUpperCase() == "IMG")
        img.src = this.addParamEnableImgSrc;
    }
  },
  
  _onMouseOutAddParamHandler: function(evt) {
    // update the button icon on hover
    if (this.addParamEnableImgSrc && this.addParamDisableImgSrc) {
      var children = dojo.isIE ? this._addParamAction.children : this._addParamAction.childNodes;
      var img = (children.length > 0) ? children[0] : null;
      if (img && img.nodeName.toUpperCase() == "IMG")
        img.src = this.addParamDisableImgSrc;
    }
  },
  
  // Check if an element value is changed from its original value.
  //
  _isValueChanged: function(elementId) {
    var changed = true;
    var element = document.getElementById(elementId);
    // get the element from the id for current value
    if (element) {
      var currentValue = element.value;
      var originalValue = this._originalValues[elementId];
      if (currentValue && originalValue) {
        // compare to original value cached in the array
        changed = currentValue != originalValue;
      } else if (!currentValue && !originalValue) {
        changed = false;
      }
    }
    return changed;
  },
  
  // The handler for text field change.
  //
  _onTextFieldChangedHandler: function(evt) {
    // update the element style if style is set for select elements
    if (this.textFieldClass) {
      // get the text field... hate this IE and FF browsers differences
      var keyEvent = evt ? evt : window.event;
      var textField = keyEvent.target ? keyEvent.target : keyEvent.srcElement;
    
      if (textField) {
        textField.className = textField.className.replace(this.textFieldClass + "Edited", this.textFieldClass);
        if (this._isValueChanged(textField.id)) {
          // change style to reflect changed
          textField.className = textField.className.replace(this.textFieldClass, this.textFieldClass + "Edited");
        }
      }
    }
    // trigger the onChange event hook
    this.onChange();
  },
  
  // The handler for combo box change.
  //
  _onSelectChangedHandler: function(evt) {

    // get the param name combo... hate this IE and FF browsers differences
    var changeEvent = evt ? evt : window.event;
    var combo = changeEvent.target ? changeEvent.target : changeEvent.srcElement;
    
    // update the element style if style is set for select elements
    if (this.selectClass) {
      if (combo) {
        combo.className = combo.className.replace(this.selectClass + "Edited", this.selectClass);
        if (this._isValueChanged(combo.id)) {
          // change style to reflect changed
          combo.className = combo.className.replace(this.selectClass, this.selectClass + "Edited");
        }
      }
    }
    if (this.attrNameSelectClass) {
      if (combo) {
        combo.className = combo.className.replace(this.attrNameSelectClass + "Edited", this.attrNameSelectClass);
        if (this._isValueChanged(combo.id)) {
          // change style to reflect changed
          combo.className = combo.className.replace(this.attrNameSelectClass, this.attrNameSelectClass + "Edited");
        }
      }
    }

    // check if combo has the first empty option which should be removed once
    // there is a valid selection
    //
    if (combo.options[0].value == "") {
        combo.remove(0);
        this._unspecifiedAttrParam = false;
    }

    // trigger the onChange event hook
    this.onChange();
  },
  
  // Handler for param attribute name combobox changed.
  //
  _paramAttrChangedHandler: function(evt) {
    // get the param name combo... hate this IE and FF browsers differences
    var changeEvent = evt ? evt : window.event;
    var paramNameCombo = changeEvent.target ? changeEvent.target : changeEvent.srcElement;
    
    if (!paramNameCombo) {
    alert("I am gone!!");
    }
    
    // get the previous selected value before the change
    var oldSelection = paramNameCombo.data["selection"];
    
    // clear the 2nd and 3rd cell which are operator and value of the param attribute
    var paramTableRow = paramNameCombo.data["row"];
    if (oldSelection && paramTableRow) {
      paramTableRow.deleteCell(3);
      paramTableRow.deleteCell(2);
      paramTableRow.deleteCell(1);
    }
    
    // set the new selected value to the combo
    var newSelection = paramNameCombo.value;
    paramNameCombo.data["selection"] = newSelection;
    
    // get the table of the row
    var table = paramTableRow.parentNode;
    var currentRowIndex = paramTableRow.rowIndex;
    
    // loop through all the table rows to update the param
    // attribute name combo box
    for (var i = 0; i < table.rows.length; i++) {
      var row = table.rows[i];
      if (row.rowIndex == currentRowIndex) {
        continue;
      }
      paramNameCombo = dojo.byId(row.data + this.CONFIG_PARAM_NAME_COMBO_ID_EXT);
      
      // old selection is now allowed for all other combo boxes
      if (oldSelection && oldSelection != "") {
        // add the option
        this._addOption(paramNameCombo, oldSelection);
      }
      //new selection is not allowed for all other combo boxes
      if (newSelection != "") {
        // remove the option
        this._removeOption(paramNameCombo, newSelection);
      }
    }
    
    // the old selection can be added back to the unset params array if it is not null
    if (oldSelection && oldSelection != "") {
      this._unsetParams[oldSelection] = oldSelection;
    }

    if (newSelection != "") {
      // create the new operation and value widgets if it is a valid selection
      this._createEditableParamOpValue(paramTableRow.data, newSelection, null, paramTableRow);
      
      // remove the selection from the unset param array
      delete this._unsetParams[newSelection];
    }
    
    // trigger the select element onChange event handler
    this._onSelectChangedHandler(evt);

    // show the "add param" action link if there is more unset param available AND there isn't any
    // unspecified param attribute already there
    if (!this._unspecifiedAttrParam)
      dojo.style(this.configsActionContainer, "display", (this._getUnsetParamsLength() > 0) ? "inline" : "none");
  },
  
  // Add an option to the combobox option array.
  //
  _addOption: function(combobox, optionValue) {
    if (combobox.nodeName.toUpperCase() != "SELECT") {
      // on error case the combobox is replaced by a span to indicate error, just skip
      return;
    }
    
    var toBeAddedPos = this._orderedParams[optionValue];
    var op = document.createElement("option");
    op.text = this._getParamDisplayName(optionValue);
    op.value = optionValue;
    for (var i = 0; i < combobox.options.length; i++) {
      var optionPos = this._orderedParams[combobox.options[i].value];
      if (optionPos > toBeAddedPos) {
        if (dojo.isIE) {
          combobox.add(op, i);
        } else {
          combobox.insertBefore(op, combobox.options[i]);
        }
        return;
      }
    }
    combobox[combobox.options.length] = op;
  },
  
  // Remove an option from the combobox option array.
  //
  _removeOption: function(combobox, optionValue) {
    if (combobox.nodeName.toUpperCase() != "SELECT") {
      // on error case the combobox is replaced by a span to indicate error, just skip
      return;
    }
    
    for (var i = 0; i < combobox.options.length; i++) {

      if (combobox.options[i].value == optionValue) {
        combobox.remove(combobox.options[i].index);
        return;
      }
    }
  },
  
  // Handler to delete a row of param attribute.
  //
  _onDeleteClickHandler: function(evt) {
    // get the delete button... hate this IE and FF browsers differences
    var clickEvent = evt ? evt : window.event;
    var delButton = clickEvent.target ? clickEvent.target : clickEvent.srcElement;
    
    // get the table row information
    var row = delButton.data;
  
    // get param attribute elements using the id prefix from the row
    // which is cached in the row element
    var paramNameCombo = dojo.byId(row.data + this.CONFIG_PARAM_NAME_COMBO_ID_EXT);

    // get the current selected value before the change
    var selection = paramNameCombo.value;
    
    // the selection can be added back to the unset params array
    this._unsetParams[selection] = selection;

    // get the table of the row
    var table = row.parentNode;
    
    // clear the cells from the row
    row.deleteCell(3);
    row.deleteCell(2);
    row.deleteCell(1);
    row.deleteCell(0);
    
    // delete the row from the table
    table.deleteRow(row.rowIndex);
    
    // selection is now allowed for all other combo boxes
    for (var i = 0; i < table.rows.length; i++) {
      row = table.rows[i];
      paramNameCombo = dojo.byId(row.data + this.CONFIG_PARAM_NAME_COMBO_ID_EXT);
      // add the option
      this._addOption(paramNameCombo, selection);
    }
    
    // show "add param" action link
    dojo.style(this.configsActionContainer, "display", "inline");

    // trigger the onChange event hook
    this.onChange();
  },
  
  // Seed the config with specified config array.  Use null to clear the config.
  //
  setConfigs: function(state, newConfigs) {
    // update the configs state and data
    this.state = state.toUpperCase();
    this.configData = newConfigs;
    
    // reset the configs area
    var configs = dojo.isIE ? this.configsNode.children : this.configsNode.childNodes;
    for (var i = 0; i < configs.length; i++) {
      this.configsNode.removeChild(configs[i]);
    }
    this._currentTabIndex = this.tabIndex + 1;
    this._unspecifiedAttrParam = false;
    this._errorMessage = "";
    this._validationWidgets = [];
    
    // clear cached original values of the config
    this._originalValues = {};
    
    // create a new set of config
    this._initConfigOptions();
    
    if (this.configData) {
      if (this.supportMulti) {
        // create elements for multiple configs
        // TODO: probably collapsible div
      } else {
        // configs should be an array of a single config
        if (this.state.toUpperCase() == this.EDITABLE) {
          // create the editable elements for new set of config
          this._initConfigOptions();
          this._createSingleEditable(0 /* first config in the config data array */, this.configsNode);
        } else {
          // create read only elements
          this._createSingleReadOnly(0 /* first config in the config data array */, this.configsNode);
        }
      }
    }
    // update the header action state and description message
    this._updateHeaderDesc();
    
    // display the configs
    dojo.style(this.configsNode, "display", "inline");
  },
  
  // Sorting function for the param array by the name attribute.
  //
  _paramSorter: function(param1, param2) {
    var name1 = param1[this.CONFIG_PARAM_NAME_ID];
    var name2 = param2[this.CONFIG_PARAM_NAME_ID];
    return (name1 == name2) ? 0 : (name1 > name2) ? 1 : -1;
  },
  
  // Check if the confg is edited by comparing the set confogs with the current
  // configs in the config area.
  //
  isEdited: function() {
    // nothing can be changed if it is not editable
    if (this.state.toUpperCase() != this.EDITABLE)
      return false;
    
    // TODO: handle multiple configs
    var originalConfig = this.configData ? this.configData[0] : null;
    var currentConfig = this.getConfigs()[0]; // first config in the configs array
    
    if (originalConfig) {
      // quickly check the length of two configs
      if (originalConfig.length != currentConfig.length) {
        return true;  // must have changes if the number of params is different
      }
      
      // sort the two config by the name attribute of the param
      originalConfig.sort(dojo.hitch(this, "_paramSorter"));
      currentConfig.sort(dojo.hitch(this, "_paramSorter"));
      
      // loop through the params in the current config
      for (var i = 0; i < currentConfig.length; i++) {
        if (currentConfig[i][this.CONFIG_PARAM_NAME_ID] != originalConfig[i][this.CONFIG_PARAM_NAME_ID])
          return true;
        if (currentConfig[i][this.CONFIG_PARAM_OP_ID] != originalConfig[i][this.CONFIG_PARAM_OP_ID])
          return true;
        if (currentConfig[i][this.CONFIG_PARAM_VALUE_ID] != originalConfig[i][this.CONFIG_PARAM_VALUE_ID])
          return true;
      }
      return false;
    } else {
      // Original config is null meaning it is a new config.
      // Added changes in the config will be in the config.
      return currentConfig.length > 0;
    }
  },
  
  // Check the widget for error and input validation for invalid error input.
  // Empty string is return if there is no error.
  // 
  getError: function() {
    var errorMessage = "";
    
    // get widget level error message
    errorMessage += this._errorMessage;
    
    for (var i = 0; i < this._validationWidgets.length; i++) {
      if (!this._validationWidgets[i].isValid()) {
        // append error message of an invalid widget to the message string
        errorMessage += this._validationWidgets[i].getErrorMessage() + "\n";
      }
    }
    return errorMessage;
  },
  
  // Validate the config param attributes in the table and return the warning
  // message.  It is just warning of incompleted params that are being ignored.
  // Empty string is return if there is no warning.
  //
  getWarning: function() {
    var warningMessage = "";
    
    // handle obly one config for now
    
    // table id with only one config
    var id = this.id + this.CONFIG_TABLE_ID_EXT;
    // get the table
    var configTable = dojo.byId(this.id + this.CONFIG_TABLE_ID_EXT);
    if (configTable) {
      for (var i = 0; i < configTable.rows.length; i++) {
        // loop through all the rows in the table
        var row = configTable.rows[i];

        // get param attribute elements using the id prefix for each row
        // which is cached in the row element
        var name = dojo.byId(row.data + this.CONFIG_PARAM_NAME_COMBO_ID_EXT);
        var op = dojo.byId(row.data + this.CONFIG_PARAM_OP_ID_EXT);
        var value = dojo.byId(row.data + this.CONFIG_PARAM_VALUE_ID_EXT);
      
        // check the param attribute of the config
        if (name && op && value) {
          // if name is not specified (in editable state) there will be no op
          // and value elements so we can skip handling unspecified name.
          var nameValue = this._getElementValue(name);
          var opValue = this._getElementValue(op);
          var valueValue = this._getElementValue(value);
          if ((!opValue) || (!valueValue)) {
            // op and/or value is not specified
            warningMessage += this._getMessageFromBundle(this.MSG_KEY_NO_ATTR_WARNING,
                                 name.options[name.selectedIndex].text) + "\n";
          }
        }
      }
    }
    return warningMessage;
  },
  
  // Get the configs data from the widgets.
  //
  getConfigs: function() {
    var allConfigs = [];
    var config = [];
    
    if (this.supportMulti) {
      // TODO: multiple configs support would get the element containers of configs
    } else {
      // table id with only one config
      var id = this.id + this.CONFIG_TABLE_ID_EXT;
      // get the table
      var configTable = dojo.byId(this.id + this.CONFIG_TABLE_ID_EXT);
      if (configTable) {
        for (var i = 0; i < configTable.rows.length; i++) {
          var param = {};
          // loop through all the rows in the table
          var row = configTable.rows[i];

          // get param attribute elements using the id prefix for each row
          // which is cached in the row element
          var name = dojo.byId(row.data + this.CONFIG_PARAM_NAME_COMBO_ID_EXT);
          var op = dojo.byId(row.data + this.CONFIG_PARAM_OP_ID_EXT);
          var value = dojo.byId(row.data + this.CONFIG_PARAM_VALUE_ID_EXT);
          
          // for selectable state, only checked rows are returned
          if (this.state.toUpperCase() == this.SELECTABLE) {
            var checkbox = dojo.byId(row.data + this.CONFIG_PARAM_CHECKED_ID_EXT);
            if (!checkbox || !checkbox.checked)
              continue;
          }
          // add the param attribute to the config
          if (name && op && value && this._getElementValue(op) && this._getElementValue(value)) {
            // add the param attribute to the config
            param[this.CONFIG_PARAM_NAME_ID] = this._getElementValue(name);
            param[this.CONFIG_PARAM_OP_ID] = this._getElementValue(op);
            param[this.CONFIG_PARAM_VALUE_ID] = this._getElementValue(value);
            config.push(param);
          }
        }
      }
      allConfigs.push(config);
    }
    return allConfigs;
  },
  
  // Create the operator and value cells of the param with the specified name id.
  //
  _createEditableParamOpValue: function(idPrefix, nameId, param, parentTableRow) {
    // get the config param options for the specific name attribute
    var selectedParam = this._paramsOptions[nameId];
    
    // create the param operation widget
    var opCol = parentTableRow.insertCell(1);
    // center the operator
    opCol.align = "center";
    dojo.style(opCol, "width", this._opMaxLength + "em");

    var op;
    if (selectedParam[this.CONFIG_PARAM_OPTIONS_OP_OPTIONS].length > 1) {
	  var selectLabel = document.createElement("label");
	  selectLabel.setAttribute("for", idPrefix + this.CONFIG_PARAM_OP_ID_EXT);
      op = document.createElement("select");
	  
      var opIndex = 0;

      // add a blank if no param is specified to seed the combo boc
      if (!param)
        op[opIndex++] = new Option("", "");

      for (var i = 0; i < selectedParam[this.CONFIG_PARAM_OPTIONS_OP_OPTIONS].length; i++) {
        op[opIndex++] = new Option(selectedParam[this.CONFIG_PARAM_OPTIONS_OP_OPTIONS][i],
                                   selectedParam[this.CONFIG_PARAM_OPTIONS_OP_OPTIONS][i]);
      }
      // add the onChange event handler
      this.connect(op, "onchange", "_onSelectChangedHandler");
      op.id = idPrefix + this.CONFIG_PARAM_OP_ID_EXT;
      if (this.selectClass)
        op.className = this.selectClass;

      if (this.tabIndex > 0) {
        op.tabIndex = this._currentTabIndex++;
      }
    
      // seed initial value to the combo box and cache the original value
      if (param) {
        op.value = param[this.CONFIG_PARAM_OP_ID];
        this._originalValues[op.id] = op.value;
      } else {
        op.selectedIndex = 0;
      }
	  selectLabel.appendChild(op);
      opCol.appendChild(selectLabel);
    } else {
      op = this._createLabel(idPrefix + this.CONFIG_PARAM_OP_ID_EXT,
                             selectedParam[this.CONFIG_PARAM_OPTIONS_OP_OPTIONS][0],
                             opCol);
    }
    // set width to fill the column width
    dojo.style(op, "width", "100%");
    parentTableRow.appendChild(opCol);
    
    // create the param value widget
    var valCol = parentTableRow.insertCell(2);
    var val;
    var widget = null;
    if (selectedParam[this.CONFIG_PARAM_OPTIONS_VALUE_TYPE] == this.CONFIG_PARAM_OPTIONS_VALUE_ENUM_TYPE) {
      // use combobox for array values
	  var selectLabel = document.createElement("label");
	  selectLabel.setAttribute("for", idPrefix + this.CONFIG_PARAM_VALUE_ID_EXT);
      val = document.createElement("select");
      var valIndex = 0;
      
      // add a blank if no param is specified to seed the combo box
      if (!param)
        val[valIndex++] = new Option("", "");
      
      for (var i = 0; i < selectedParam[this.CONFIG_PARAM_OPTIONS_VALUE_OPTIONS].length; i++) {
        val[valIndex++] = new Option(selectedParam[this.CONFIG_PARAM_OPTIONS_VALUE_OPTIONS][i],
                                     selectedParam[this.CONFIG_PARAM_OPTIONS_VALUE_OPTIONS][i]);
      }
      // add the onChange event handler
      this.connect(val, "onchange", "_onSelectChangedHandler");
      val.id = idPrefix + this.CONFIG_PARAM_VALUE_ID_EXT;
      if (this.selectClass)
        val.className = this.selectClass;

      if (this.tabIndex > 0) {
        val.tabIndex = this._currentTabIndex++;
      }
    
      // set width corresponding to the max param value options length
      dojo.style(val, "width", this._valComboMaxLength + "em");

      // seed initial value to the combo box and cache the original value
      if (param) {
        val.value = param[this.CONFIG_PARAM_VALUE_ID];
        this._originalValues[val.id] = val.value;
      } else {
        val.selectedIndex = 0;
      }
	  selectLabel.appendChild(val);
      valCol.appendChild(selectLabel);
         
    } else if (selectedParam[this.CONFIG_PARAM_OPTIONS_VALUE_TYPE] == this.CONFIG_PARAM_OPTIONS_VALUE_MEMORY_TYPE) {
      // use dojo validation text box for memory
      widget = this._createMemoryWidget(idPrefix + this.CONFIG_PARAM_VALUE_ID_EXT,
                                        param ? param[this.CONFIG_PARAM_VALUE_ID] : null, valCol);
    } else if (selectedParam[this.CONFIG_PARAM_OPTIONS_VALUE_TYPE] == this.CONFIG_PARAM_OPTIONS_VALUE_IP_TYPE) {
      // use dojo validation text box for IP address
      widget = this._createIPWidget(idPrefix + this.CONFIG_PARAM_VALUE_ID_EXT,
                                    param ? param[this.CONFIG_PARAM_VALUE_ID] : null, valCol);
    } else if (selectedParam[this.CONFIG_PARAM_OPTIONS_VALUE_TYPE] == this.CONFIG_PARAM_OPTIONS_VALUE_STRING_TYPE) {
      // use dojo validation text box for hostname
      widget = this._createHostnameWidget(idPrefix + this.CONFIG_PARAM_VALUE_ID_EXT,
                                          param ? param[this.CONFIG_PARAM_VALUE_ID] : null, valCol);
    } else {
      // error - invalid/unknown value type
      this._createLabel(idPrefix + this.CONFIG_PARAM_VALUE_ID_EXT,
                        this._getMessageFromBundle(this.MSG_KEY_ERROR_LABEL),
                        valCol);
      this._errorMessage += this._getMessageFromBundle(this.MSG_KEY_INVALID_VAL_TYPE, nameId) + "\n";
      // remove the operator when the type is invalid
      opCol.removeChild(op);
    }
    if (widget && (this._valTextMaxLength > 0)) {
      // set width corresponding to the max param value options length if there is one
      dojo.style(widget.domNode, "width", this._valTextMaxLength + "em");
      if (this.tabIndex > 0) {
        widget.domNode.tabIndex = this._currentTabIndex++;
      }
    }
    parentTableRow.appendChild(valCol);
    
    // create the delete row button
    var delCol = parentTableRow.insertCell(3);
    var delButton = document.createElement("button");
    
    // if icon is specified, use icon
    var label = this.removeParamLabel ? this.removeParamLabel : this._getMessageFromBundle(this.MSG_KEY_REMOVE_PARAM);
    if (this.removeParamEnableImgSrc) {
      var img = document.createElement("img");
      if (this.removeParamDisableImgSrc) {
        // disable icon is specified too, do hovering
        img.src = this.removeParamDisableImgSrc;
        this.connect(delButton, "onmouseover", "_onMouseOverRemoveParamHandler");
        this.connect(delButton, "onmouseout", "_onMouseOutRemoveParamHandler");
      } else {
        img.src = this.removeParamEnableImgSrc;
      }
      img.alt = label;
      delButton.appendChild(img);
    } else {
      delButton.innerHTML = label;
    }
    if (this.actionButtonClass)
      delButton.className = this.actionButtonClass;
    
    if (this.tabIndex > 0) {
      delButton.tabIndex = this._currentTabIndex++;
    }

    // cache the row info
    delButton.data = parentTableRow;
    
    // add the onClick event handler
    this.connect(delButton, "onclick", "_onDeleteClickHandler");
    delButton.id = idPrefix + this.CONFIG_PARAM_DELETE_ID_EXT;

    delCol.appendChild(delButton);
    parentTableRow.appendChild(delCol);
  },
  
  // Create a table row of editable parameter i.e. a three column row
  // with name, operator, and value.
  //
  _createEditableParam: function(idPrefix, param, parentTableRow) {
    // create the param name combobox
	var selectLabel = document.createElement("label");
	selectLabel.setAttribute("for", idPrefix + this.CONFIG_PARAM_NAME_COMBO_ID_EXT);
    var combo = document.createElement("select");
    combo.data = {};
    var index = 0;
    
    // add a blank if no param is specified to seed the row
    if (!param) {
      combo[index++] = new Option("", "");
      this._unspecifiedAttrParam = true;
    }
    // iterate through the unset param array to create the attribute combobox
    for (var nameId in this._unsetParams) {
      // the unset param array has id tag only, the actual data of the param is in the _paramsOptions
      combo[index++] = new Option(this._getParamDisplayName(nameId), nameId);
    }
    // add the combobox to the row
    var nameCol = parentTableRow.insertCell(0);
	selectLabel.appendChild(combo);
    nameCol.appendChild(selectLabel);
    parentTableRow.appendChild(nameCol);
    combo.data["row"] = parentTableRow;

    combo.id = idPrefix + this.CONFIG_PARAM_NAME_COMBO_ID_EXT;
    this.connect(combo, "onchange", "_paramAttrChangedHandler");
    combo.data["selection"] = null;
    if (this.attrNameSelectClass)
      combo.className = this.attrNameSelectClass;

    // set width corresponding to the max param attribute name length
    dojo.style(combo, "width", this._attrNameMaxLength + "em");
    if (this.tabIndex > 0) {
      combo.tabIndex = this._currentTabIndex++;
    }
    
    if (param) {
      var selectedParam = this._paramsOptions[param[this.CONFIG_PARAM_NAME_ID]];
      if (selectedParam) {
        // seed initial value to the combo box and cache the original value
        combo.value = param[this.CONFIG_PARAM_NAME_ID];
        combo.data["selection"] = param[this.CONFIG_PARAM_NAME_ID];
        this._originalValues[combo.id] = combo.value;

        // move this param from unset param array to set param array so next param will not have duplicate
        delete this._unsetParams[param[this.CONFIG_PARAM_NAME_ID]];
      } else {
        // error - specified config param is not one of the parameters in the combo box
        nameCol.removeChild(combo);
        this._createLabel(idPrefix + this.CONFIG_PARAM_NAME_COMBO_ID_EXT,
                          this._getMessageFromBundle(this.MSG_KEY_ERROR_LABEL),
                          nameCol);
        this._errorMessage += this._getMessageFromBundle(this.MSG_KEY_INVALID_PARAM_ATTR,
                              param[this.CONFIG_PARAM_NAME_ID]) + "\n";
        return;
      }
    
      // create the param operation widget and set the value
      this._createEditableParamOpValue(idPrefix, param[this.CONFIG_PARAM_NAME_ID], param, parentTableRow);
    }
    
    // set combo to focus
    combo.focus();
  },
  
  // Create a table for a single editable config.
  //
  _createSingleEditable: function(configIndex, /* index of config in this.configData */
                                  parent /* the parent dom element */) {
    // create the table element to hold the config
    var table = this._createConfigTable(this.id);
    
    // get the config
    var config = this.configData[configIndex];
    
    // iterate through the parameters in the config
    for (var i = 0; i < config.length; i++) {
      var row = table.insertRow(i);
      // the current row of param within the config
      var param = config[i];
      
      // the id prefix for the elements in the table row
      var idPrefix = this._constructElemIdPrefix(configIndex);
      // cache the prefix in the table row
      row.data = idPrefix;
      
      this._createEditableParam(idPrefix, param, row);
    };
    
    // associate the table to the add param action
    this._addParamAction.data = table;
    
    parent.appendChild(table);
    
    // now that the table is setup remove options from comboboxes that are selected
    for (var i = table.rows.length - 1; i >= 0; i--) {
      // get selected option of current row from bottom row up
      var thisCombo = dojo.byId(table.rows[i].data + this.CONFIG_PARAM_NAME_COMBO_ID_EXT);
      var thisValue = thisCombo.value;
      // delete selected option from comboboxes in previous rows
      for (var j = i - 1; j >=0; j--) {
        var previousCombo = dojo.byId(table.rows[j].data + this.CONFIG_PARAM_NAME_COMBO_ID_EXT);
        this._removeOption(previousCombo, thisCombo.value);
      }
    }
  },
  
  // Create a table for a single read only config.
  //
  _createSingleReadOnly: function(configIndex, /* index of config in this.configData */
                                  parent /* the parent dom element */) {
    // create the table element to hold a config
    var table = this._createConfigTable(this.id);
    
    // get the config
    var config = this.configData[configIndex];
    
    // iterate through the parameters in the config
    for (var i = 0; i < config.length; i++) {
      var row = table.insertRow(i);
      var col;
      var colCount = 0;
      // the current row of param within the config
      var param = config[i];
      
      // the id prefix for the elements in the table row
      var idPrefix = this._constructElemIdPrefix(configIndex);
      // cache the prefix in the table row
      row.data = idPrefix;
      
      if (this.state.toUpperCase() == this.SELECTABLE) {
        col = row.insertCell(colCount++);
        this._createCheckbox(idPrefix + this.CONFIG_PARAM_CHECKED_ID_EXT,
                             true, col);
      }
      
      col = row.insertCell(colCount++);
      col.setAttribute("nowrap", "nowrap");
      var label= this._createLabel(idPrefix + this.CONFIG_PARAM_NAME_COMBO_ID_EXT,
                                   this._getParamDisplayName(param[this.CONFIG_PARAM_NAME_ID]), col);
      // cache the value of the attribute
      label.data = param[this.CONFIG_PARAM_NAME_ID];
      
      col = row.insertCell(colCount++);
      col.setAttribute("nowrap", "nowrap");
      col.setAttribute("align", "center" );
      // set width corresponding to the max operator length
      if (this._opMaxLength > 0) {
        dojo.style(col, "width", this._opMaxLength + "em");
      }
      this._createLabel(idPrefix + this.CONFIG_PARAM_OP_ID_EXT,
                        param[this.CONFIG_PARAM_OP_ID], col);
        
      col = row.insertCell(colCount++);
      col.setAttribute("nowrap", "nowrap");
      this._createLabel(idPrefix + this.CONFIG_PARAM_VALUE_ID_EXT,
                        param[this.CONFIG_PARAM_VALUE_ID], col);
    };
    parent.appendChild(table);
  },
  
  // Create a HTML table to hold the config.
  //
  _createConfigTable: function(id) {
    // create the table element to hold a config
    var table = document.createElement("table");
    table.cellpadding = 0;
    table.cellspacing = 4;
    table.border = 0;
    table.id = id + this.CONFIG_TABLE_ID_EXT;
    if (this.configTableClass) 
      table.className = this.configTableClass;
    return table;
  },
  
  // Create a HTML checkbox input field.
  //
  _createCheckbox: function(id, checked, parent) {
  	var inputLabel = document.createElement("label");
	inputLabel.setAttribute("for", id);
    var checkbox = document.createElement("input");
    checkbox.id = id; 
	checkbox.type = "checkbox";
    checkbox.checked = checked;
    checkbox.defaultChecked = checked;
    if (this.checkboxClass)
      checkbox.className = this.checkboxClass;
	inputLabel.appendChild(checkbox);
    parent.appendChild(inputLabel);
    return checkbox;
  },
  
  // Create a HTML span label.
  //
  _createLabel: function(id, label, parent) {
    var span = document.createElement("span");
    span.innerHTML = label;
    span.id = id;
    if (this.labelClass)
      span.className = this.labelClass;
    parent.appendChild(span);
    return span;
  },
  
  // Create a HTML text input field.
  //
  _createTextField: function(id, value, parent) {
  	var inputLabel = document.createElement("label");
	inputLabel.setAttribute("for", id);
    var text = document.createElement("input");
    text.type = "text";
    text.id = id;
    this.connect(text, "onkeyup", "_onTextFieldChangedHandler");
    
    // seed the value to the control and cache the initial value if it is specified
    if (value) {
      text.value = value;
      this._originalValues[id] = value;
    }
    if (this.textFieldClass)
      text.className = this.textFieldClass;
	inputLabel.appendChild(text);
    parent.appendChild(inputLabel);
    return text;
  },
  
  // Create a Dojo validation textbox for machine memory.
  //
  _createMemoryWidget: function(id, memory, parent) {
    // create a dojo validation text field
    var params = {
      id: id,
      maxlength: 8,
      // The memory in MB can be empty, zero, or any number without leading zero
      regExp: "|0|[1-9][0-9]*",
      invalidMessage: this._getMessageFromBundle(this.MSG_KEY_INVALID_MEMORY)
    };
    var memoryWidget = new dijit.form.ValidationTextBox(params);
    
    // seed the value to the control and cache the initial value if it is specified
    if (memory) {
      memoryWidget.setValue(memory);
      this._originalValues[id] = memory;
    }
	var memDomNode = memoryWidget.domNode;
    memDomNode.id = id;
    if (this.textFieldClass)
      memDomNode.className = this.textFieldClass;
    
    // fill the table cell
    dojo.style(memDomNode, "width", "100%");
        
    // add the onChange event handler to key up event - be sure not to override
    // the onkeyup of the widget which handle the validation
    this.connect(memDomNode, "onkeyup", "_onTextFieldChangedHandler");
    
    // add the dom element to the parent
    parent.appendChild(memDomNode);
    
    // save the widget to array to check for error
    this._validationWidgets.push(memoryWidget);
    
    return memoryWidget;

  },
  
  // Create a Dojo validation textbox for machine hostname.
  //
  _createHostnameWidget: function(id, hostname, parent) {
    // create a dojo validation text field
    var params = {
      id: id,
      maxlength: 255,
      // The hostname can be empty, 1 or 2 alphanumeric characters, or 3 or more alphanumeric
      // characters that allow dot (.), dash (-), or underscore (_) in between.
      //
      regExp: "[a-zA-Z0-9]{0,2}|[a-zA-Z0-9][a-zA-Z0-9_.-]*[a-zA-Z0-9]",
      invalidMessage: this._getMessageFromBundle(this.MSG_KEY_INVALID_HOSTNAME)
    };
    var hostnameWidget = new dijit.form.ValidationTextBox(params);
    
    // seed the value to the control and cache the initial value if it is specified
    if (hostname) {
      hostnameWidget.setValue(hostname);
      this._originalValues[id] = hostname;
    }
    var hostnameDomNode = hostnameWidget.domNode;
    hostnameDomNode.id = id;
    if (this.textFieldClass)
      hostnameDomNode.className = this.textFieldClass;

    // fill the table cell
    dojo.style(hostnameDomNode, "width", "100%");

    // add the onChange event handler to key up event - be sure not to override
    // the onkeyup of the widget which handle the validation
    this.connect(hostnameDomNode, "onkeyup", "_onTextFieldChangedHandler");
    
    // add the dom element to the parent
    parent.appendChild(hostnameDomNode);
    
    // save the widget to array to check for error
    this._validationWidgets.push(hostnameWidget);

    return hostnameWidget;
  },
  
  // Create a Dojo validation textbox for machine IP address.
  //
  _createIPWidget: function(id, ip, parent) {
    // The IP address can be empty, IPv4 or IPv6 format.
    // IPv4:
    // Regular expression works on text not numeric value so to check number range from 0 - 255
    // is to validate [0-9] for a single digit, or [1-9][0-9] for any two digits not starting with 0,
    // or 1[0-9][0-9] for 100-199, or 2[0-4][0-9] for 200-249, or 25[0-5] for 250-255.
    // IPv6:
    // The format are eight groups of four hexadecimal digits separated by colons, such
    // as 1234:0000:01F2:001B:1002:0000:0000:10CE.
    //
    var ipv4 = "((?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?))";
    var ipv6 = "(([0-9a-fA-F]{1,4}|0)(\:([0-9a-fA-F]{1,4}|0)){7})";
    var ipv6Abbr0 = "(::([0-9a-fA-F]{1,4}|0)(\:([0-9a-fA-F]{1,4}|0)){0,6})";
    var ipv6Abbr1 = "(([0-9a-fA-F]{1,4}|0)\:\:(|([0-9a-fA-F]{1,4}|0)(\:([0-9a-fA-F]{1,4}|0)){0,5}))";
    var ipv6Abbr2 = "((([0-9a-fA-F]{1,4}|0)\:)([0-9a-fA-F]{1,4}|0)\:\:(|([0-9a-fA-F]{1,4}|0)(\:([0-9a-fA-F]{1,4}|0)){0,4}))";
    var ipv6Abbr3 = "((([0-9a-fA-F]{1,4}|0)\:){2}([0-9a-fA-F]{1,4}|0)\:\:(|([0-9a-fA-F]{1,4}|0)(\:([0-9a-fA-F]{1,4}|0)){0,3}))";
    var ipv6Abbr4 = "((([0-9a-fA-F]{1,4}|0)\:){3}([0-9a-fA-F]{1,4}|0)\:\:(|([0-9a-fA-F]{1,4}|0)(\:([0-9a-fA-F]{1,4}|0)){0,2}))";
    var ipv6Abbr5 = "((([0-9a-fA-F]{1,4}|0)\:){4}([0-9a-fA-F]{1,4}|0)\:\:(|([0-9a-fA-F]{1,4}|0)(\:([0-9a-fA-F]{1,4}|0)){0,1}))";
    var ipv6Abbr6 = "((([0-9a-fA-F]{1,4}|0)\:){5}([0-9a-fA-F]{1,4}|0)\:\:(|([0-9a-fA-F]{1,4}|0)))";
    var ipv6Abbr7 = "((([0-9a-fA-F]{1,4}|0)\:){6}([0-9a-fA-F]{1,4}|0)\:\:)";
    var ipRegExp = "|" + ipv4 + "|" + ipv6 +
                   "|" + ipv6Abbr0 + "|" + ipv6Abbr1 + "|" + ipv6Abbr2 + "|" + ipv6Abbr3 +
                   "|" + ipv6Abbr4 + "|" + ipv6Abbr5 + "|" + ipv6Abbr6 + "|" + ipv6Abbr7;

    // create a dojo validation text field
    var params = {
      id: id,
      regExp: ipRegExp,
      invalidMessage: this._getMessageFromBundle(this.MSG_KEY_INVALID_IP)
    };
    var iPWidget = new dijit.form.ValidationTextBox(params);

    // seed the value to the control and cache the initial value if it is specified
    if (ip) {
      iPWidget.setValue(ip);
      this._originalValues[id] = ip;
    }
	var ipWidgetDomNode = iPWidget.domNode;
    ipWidgetDomNode.id = id;
    if (this.textFieldClass)
      ipWidgetDomNode.className = this.textFieldClass;

    // fill the table cell
    dojo.style(ipWidgetDomNode, "width", "100%");

    // add the onChange event handler to key up event - be sure not to override
    // the onkeyup of the widget which handle the validation
    this.connect(ipWidgetDomNode, "onkeyup", "_onTextFieldChangedHandler");
    
    // add the dom element to the parent
    parent.appendChild(ipWidgetDomNode);
    
    // save the widget to array to check for error
    this._validationWidgets.push(iPWidget);

    return iPWidget;
  },
  
  // Get the value of an element based on its type.
  //
  _getElementValue: function(element) {
    var nodeName = element.nodeName.toUpperCase();
    if (nodeName == "SPAN")
      if (element.data)
        // return the cache value if there is one
        return element.data;
      else
        // otherwise return the display text
        return element.firstChild.data;
    else if ((nodeName == "SELECT") || (nodeName == "INPUT") || (nodeName == "OPTION"))
      return element.value;
    else
      return element.innerHTML;
  },
  
  // Get the CS Help button.
  //
  getCsHelpButton: function() {
    return this._csHelp;
  },
  
  // Update the display of the actions and description message.
  //
  _updateHeaderDesc: function(explicitNewAction) {
    if (this.state.toUpperCase() == this.EDITABLE) {
      this.descriptionNode.innerHTML = this._getMessageFromBundle(this.MSG_KEY_EDITABLE_DESC);
      
      if (!this.configData) {
        if (explicitNewAction) {
          // invoking the "New Config" action does not have any saved config
          // data yet but 
          dojo.style(this._newConfigAction, "display", this.supportMulti ? "inline" : "none");
          dojo.style(this.descriptionContainer, "display", "inline");
          dojo.style(this.configsActionContainer, "display", "inline");
        } else {
          // show "New Config" action if no config data yet
          dojo.style(this._newConfigAction, "display", "inline");
          dojo.style(this.descriptionContainer, "display", "none");
          dojo.style(this.configsActionContainer, "display", "none");
        }
      } else {
        // there is already config data, don't show "New Config" action if only single
        // config is allow or always show the action if multiple configs is supported
        dojo.style(this._newConfigAction, "display", this.supportMulti ? "inline" : "none");
        dojo.style(this.descriptionContainer, "display", "inline");
        dojo.style(this.configsActionContainer, "display", (this._getUnsetParamsLength() > 0) ? "inline" : "none");
      }
    } else {
      // don't show "New Config" and Add param" actions if not editable
      dojo.style(this._newConfigAction, "display", "none");
      dojo.style(this.configsActionContainer, "display", "none");
      
      if (this.configData) {
        // set appropriate description message
        if (this.state.toUpperCase() == this.READONLY) {
          this.descriptionNode.innerHTML = this._getMessageFromBundle(this.MSG_KEY_READ_ONLY_DESC);
        } else if (this.state.toUpperCase() == this.SELECTABLE) {
          this.descriptionNode.innerHTML = this._getMessageFromBundle(this.MSG_KEY_SELECTABLE_DESC);
        }
      } else {
        // description for no config specified
        this.descriptionNode.innerHTML = this._getMessageFromBundle(this.MSG_KEY_READ_ONLY_BLANK_DESC);
      }
      dojo.style(this.descriptionContainer, "display", "inline");
    }
  },
  
  // Given the param attribute name id, get the corresponding display name if
  // it is defined.  Otherwise, the attribute name id is returned.
  //

  _getParamDisplayName: function(paramId) {
    if (!this._paramsOptions[paramId]) {
      // param attribute name is bogus
      this._errorMessage += this._getMessageFromBundle(this.MSG_KEY_INVALID_PARAM_ATTR, paramId) + "\n";
      return this._getMessageFromBundle(this.MSG_KEY_ERROR_LABEL);
    }
    var displayName = this._paramsOptions[paramId][this.CONFIG_PARAM_OPTIONS_ATTR_DISPLAY_NAME];
    return (displayName && displayName != "") ? displayName : paramId;
  },
  
  // Get the associative array (hash) length of the unset params.
  //
  _getUnsetParamsLength: function() {
    var length = 0;
    for (var i in this._unsetParams) {
      length++;
    }
    return length;
  },
  
  // construct an element id based on the index of the config.
  //
  _constructElemIdPrefix: function(configIndex /* index of the config */) {
    return this.id + "_CONFIG" + configIndex + "_PARAM" + this._tableRowId++ + "_";
  },

  // Set the message bundle to use for messages.
  //
  setMessageBundle: function(bundle) {
    this.msgBundle = bundle;
    
    // update the actions which were created before this method is called
    //
    if (this._newConfigAction.nodeName.toUpperCase() == "BUTTON") {
      var children = dojo.isIE ? this._newConfigAction.children : this._newConfigAction.childNodes;
      var img = (children.length > 0) ? children[0] : null;
      if (img && img.nodeName.toUpperCase() == "IMG")
        img.alt = this._getMessageFromBundle(this.MSG_KEY_NEW_CONFIG);
    } else {
      this._newConfigAction.innerHTML = this._getMessageFromBundle(this.MSG_KEY_NEW_CONFIG);
    }
    if (this._addParamAction.nodeName.toUpperCase() == "BUTTON") {
      var children = dojo.isIE ? this._addParamAction.children : this._addParamAction.childNodes;
      var img = (children.length > 0) ? children[0] : null;
      if (img && img.nodeName.toUpperCase() == "IMG")
        img.alt = this._getMessageFromBundle(this.MSG_KEY_ADD_PARAM);
    } else {
      this._addParamAction.innerHTML = this._getMessageFromBundle(this.MSG_KEY_ADD_PARAM);
    }
  },
  
  // Get the message of the specified key from the message bundle.
  //
  _getMessageFromBundle: function(keyword, tokens) {
    var message = this.msgBundle ? this.msgBundle[keyword] : null;
    if (message == null) {
      console.debug("Failed to get message from keyword \"" + keyword + "\"");
      return keyword;
    }
    if (arguments.length > 1) {
      for (var i = 1; i < arguments.length; i++) {
        message = message.replace("%" + i, arguments[i]);
      }
    }
    return message;
  },
  
  CONFIG_PARAM_OPTIONS_ATTR_NAME: "attrName",
  CONFIG_PARAM_OPTIONS_ATTR_DISPLAY_NAME: "displayName",
  CONFIG_PARAM_OPTIONS_OP_OPTIONS: "opValues",
  CONFIG_PARAM_OPTIONS_VALUE_OPTIONS: "choices",
  CONFIG_PARAM_OPTIONS_VALUE_TYPE: "type",
  CONFIG_PARAM_OPTIONS_VALUE_ENUM_TYPE: "enum",
  CONFIG_PARAM_OPTIONS_VALUE_INTEGER_TYPE: "integer",
  CONFIG_PARAM_OPTIONS_VALUE_MEMORY_TYPE: "memory",
  CONFIG_PARAM_OPTIONS_VALUE_IP_TYPE: "IP",
  CONFIG_PARAM_OPTIONS_VALUE_STRING_TYPE: "String",
  CONFIG_PARAM_NAME_ID: "name",
  CONFIG_PARAM_OP_ID: "op",
  CONFIG_PARAM_VALUE_ID: "value",
  
  // element id extensions
  CONFIG_PARAM_NAME_COMBO_ID_EXT: "NameCombobox",
  CONFIG_PARAM_OP_ID_EXT: "Op",
  CONFIG_PARAM_VALUE_ID_EXT: "Value",
  CONFIG_PARAM_CHECKED_ID_EXT: "Checkbox",
  CONFIG_PARAM_DELETE_ID_EXT: "Delete",
  CONFIG_TABLE_ID_EXT: "ConfigParamTable",
  
  // message bundle keys
  MSG_KEY_EDITABLE_DESC: "machConfigEditableDesc",
  MSG_KEY_READ_ONLY_DESC: "machConfigReadOnlyDesc",
  MSG_KEY_READ_ONLY_BLANK_DESC: "machConfigReadOnlyBlankDesc",
  MSG_KEY_SELECTABLE_DESC: "machConfigSelectableDesc",
  MSG_KEY_NEW_CONFIG: "machConfigAddConfig",
  MSG_KEY_ADD_PARAM: "machConfigAddParam",
  MSG_KEY_REMOVE_PARAM: "machConfigRemoveParam",
  MSG_KEY_ERROR_LABEL: "machConfigErrorLabel",
  MSG_KEY_NO_ATTR_WARNING: "machConfigAttrNotSpecifiedWarning",
  MSG_KEY_INVALID_VAL_TYPE: "machConfigInvalidAttrValueType",
  MSG_KEY_INVALID_PARAM_ATTR: "machConfigInvalidParamAttr",

  
  MSG_KEY_INVALID_IP: "MsgInvalidIpInput",
  MSG_KEY_INVALID_HOSTNAME: "MsgInvalidHostnameInput",
  MSG_KEY_INVALID_MEMORY: "MsgInvalidMemoryInput"
});
