/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
//

//
//
// Author: Mark Victory
//
// Summary:
//

dojo.provide("com.ibm.rational.test.lm.web.ui.internal.widgets.MachineHistoryPanel");

dojo.require("dijit.layout.ContentPane");
dojo.require("dijit._Templated");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.rational.test.lm.web.client.internal.audit");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.providers.HistoryTableProvider"); 
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.CollapsibleSection");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener");

(function() 
{
	var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
    var RtlmUiUtil_getMessage = RtlmUiUtil.getMessage;

dojo.declare(
  "com.ibm.rational.test.lm.web.ui.internal.widgets.MachineHistoryPanel",
  [dijit._Widget, dijit._Templated],
{
	widgetsInTemplate: true,
	templateString:
'<div id="${id}" dojoAttachPoint="domNode">' +
'   <!-- ActionTitlePanel allows opening and closing the panel --> '+
' <div dojoType="com.ibm.asq.common.web.ui.internal.view.common.CollapsibleSection"'+ 
'         dojoAttachPoint="historyArea" '+
'         mainClass="actionPanel" '+
'         open="false">'+
'   <!-- a progress area to use to display progress when loading data --> '+
'   <table dojoAttachPoint="progressIndicator" border="0" cellpadding="0" cellspacing="0" width="100%" style="display: none">' +
'     <tr align="left">' +
'       <td align="right" valign="middle" width="25">' +
'         <div class="viewProgressImage"><div>' +
'       </td>' +
'       <td valign="middle" width="100%">' +
'         <span dojoAttachPoint="progressLabel" class="viewProgressLabel"></span>' +
'       </td>' +
'     </tr>' +
'   </table>' +
'   <!-- table resultarea --> '+
'   <table border="0" cellpadding="0" cellspacing="0" width="100%">' +
'     <tr align="left">' +
'       <td>' +
'         <div dojoAttachPoint="resultDiv"  /div> '+
'       </td>' +
'     </tr>' +
'   <!-- A status bar in case there are problems --> '+
'     <tr align="left">' +
'       <td>' +
'         <div dojoAttachPoint="statusDiv">' +
'           <table dojoAttachPoint="statusContent" border="0" cellpadding="0" cellspacing="0" width="100%">' +
'             <tr align="left">' +
'               <td valign="middle" align="left">' +
'                 <span dojoAttachPoint="statusLineNode" class="viewStatusLine"></span>' +
'               </td>' +
'             </tr>' +
'           </table>' +
'         </div>' +
'       </td>' +
'     </tr>' +
'   </table>' +
' </div> '+
'</div>',

	//The ID of the machine we are showing reservations for
	machineID: null,
	
	//The label for the Panel
	panelLabel: null,

	postCreate: function()
	{
		var self = this;
		this.historyArea.toggle = function()
								  {
								  		if( !this.open )
										{
											//Load the table
											if (self.machineID != null)
											{
												self.loadTable();
											}
										}
										
										this.inherited( "toggle", arguments );
								  }
		
		//Set the title
		if (this.panelLabel != null)
		{
			this.setLabel(this.panelLabel);
		}
		else
		{
			this.setLabel(RtlmUiUtil_getMessage("MachLabelHistory"));
		}
        
        // hide areas not being used in the section but may want to have the spacing to match other sections
        if (this.compact) {
            dojo.style(this.historyArea.infoNode, "display", "none");
        }
        this.historyArea.summaryNode.innerHTML = RtlmUiUtil_getMessage("historyDesc");
        
		//Define the table
		this.createTable();
	},
	
	//Load the History for the machine specified by id
	load: function(id)
	{
		if (id == null)
			return;
		
		this.machineID = id;
		if( this.historyArea.open )
		{
			this.loadTable();
		}
	},
	
	createTable: function()
	{
		this.resultTable = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer({
            allowAdjustSettings: true,
            autoSaveSettings: true,
            tableId: "RQM-LM-ASSET-HISTORY-TABLEVIEWER"                                
        });

        this.refreshActionNode = this.resultTable.addAction(
                "refresh",                       // action id
                RtlmUiUtil_getMessage("labelRefreshButton"), // label
                null,                            // callback data
                "refresh-icon-image",            // icon class
                dojo.hitch(this, "loadTable"));  // callback

        this.resultTable.setContentPageSize( 10 );
        var contentProvider = new com.ibm.rational.test.lm.web.ui.internal.views.providers.HistoryTableProvider();
        this.resultTable.setContentProvider(contentProvider);
        var tableListener = new History.TableListener();
        this.resultTable.setTableListener(tableListener);
        this.resultTable.setContent( [] );
		this.resultTable.setTableSummary(RtlmUiUtil_getMessage("labelTableSummary", RtlmUiUtil_getMessage("MachLabelHistory")));
        this.resultTable.show();
		this.resultDiv.appendChild( this.resultTable.domNode );
	},
	
	loadTable: function()
	{
		var auditor = new com.ibm.rational.test.lm.web.client.internal.audit();
		var auditHandler= new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_loadTable", "_errorFunc");
        
		this.setProgressStart(RtlmUiUtil_getMessage("MsgHistoryRetrieval"));
		
        if( this.machineID )
        {
        	auditor.getMachineAuditTrail( auditHandler, this.machineID );
        }
		else
			this.setProgressEnd();  //This shouldn't happen but just in case, take down the progress message
	},
	
	_loadTable: function(response)
	{
		//Take down the progress image
		this.setProgressEnd();
		this.resultTable.setContent( ( response ) ? response : [] );
        this.resultTable.refresh();
	},
	
	_errorFunc: function(errorType, errorMessage) 
	{
		this.setProgressEnd();
		//Display the empty table otherwise the result is a funny looking blank area
		this.resultTable.setContent( [] );
        this.resultTable.refresh();
		//Not sure what errors should do here.
	},
	
	
    _getDateDisplay: function(dateValue, dateOnly)
    {
        // Constant that represents the date equal to "-1".  This is the
        // convention used to represent a null date.  This convention is needed
        // so that we can indicate that we want to remove a date from a date field.
        var _NODATE = "1969-12-31T18:59:59.999-0500";
        
        if(dateValue == null)
            return "";
            
        // Check if the date is -1.  If it is we have no date specified
        // and we should display a blank.
        if( dateValue == _NODATE ) {
            return "";
        }
        var date = RtlmUiUtil.convertTimeStampToDate(dateValue);
        
        if(dateOnly)
            return dojo.date.locale.format(date, {selector:'date', formatLength:"short"});
        return RtlmUiUtil.getDisplayDateTimeString(date);
    },
	
	//Set the Panel's label
	setLabel: function(labelText)
	{
		if (labelText != null) 
		{
			this.panelLable = labelText;
			this.historyArea.setTitle(labelText);
		}
	},
	
	// hide the status line from the display
	//
	hideStatusLine: function() 
	{
		this.statusLineNode.style.display = "none";
	},
    
	// Summary:
	//    set the status message in the header. 
	//
	setStatus: function(status) 
	{
		this.statusLineNode.innerHTML = status;
	},
	
	// show the in-progress indicator
	//
	setProgressStart: function(message) 
	{
		this.progressLabel.innerHTML = message;
		this.progressIndicator.style.display = "inline";
	},
    
	// Hide the in-progress indicator
	//
	setProgressEnd: function() 
	{
		this.progressLabel.innerHTML = "";
		this.progressIndicator.style.display = "none";
	},

	getCollapsibleSection: function()
	{
		return( dijit.byId( this.historyArea.id ) );
	}
});

dojo.declare("History.TableListener",
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener,
{    
    tableContentActionInvoked: function(tableViewer, actionId, callbackData, callback) {
        callback(callbackData);
    }
});

})();
