/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //    The ActionRegistry is an abstract model object for installing actions for
 //    items inside the horizontal tree explorer. However, it's generic enough that 
 //    it could provide actions for other view types as well. Client needs to implement
 //    several methods defined here to supply application specific actions for
 //    the model objects.
 //
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.horizontalTree.ActionRegistry");

dojo.require("dijit.Menu");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.Button");

(function() {

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.horizontalTree.ActionRegistry",
             null,
{   
    actionsLabel: MessageUtil.getMessage(MessageBundle.COMMON, "labelActions"),

    postscript: function(/*Object?*/params)
    {
        dojo.mixin(this, params);
        this.init();
    },
    
    init: function()
    {
        
    },
    
    // API - returns the action descriptor for the specified model item. The expected
    // descriptor structure is as follows. The specified callback will be invoked with
    // the item and its parent in context: callback(parentItem, item)
    //
    //  action = {
    //        actions: [
    //                {
    //                    label: <text label for the action>,
    //                    needSelectionEnablement: <true | false> to enable the action,
    //                    enablementCallback: <callback function to call to return action's selection state> enablementCallback(select state),
    //                    iconClass: <icon class name>,
    //                    callback: <callback function to receive action invoked event> callback(parentItem, item)
    //                }
    //                ...
    //        ]
    //    };
    
    getPanelActions: function(model, item)
    {
        return null;
    },
    
    getContextActions: function(model, item)
    {
        return null;
    },

    // Summary:
    // NOTE: Consume by view.
    //    Convenient method to construct a context menu from the item's action description.
    //    Only support single level context menu at the moment.
    //
    getContextMenu: function(actionDesc, dispatcher)
    {
        if(!actionDesc)
            return null;
            
        var menu = new dijit.Menu();
        var label = new dijit.MenuItem({
                label: this.actionsLabel,
                disabled: true
            });
        menu.addChild(label);
        menu.addChild(new dijit.MenuSeparator());

		var len = actionDesc.actions.length;             
        for(var i = 0; i < len; i++) {
            var actionItem = actionDesc.actions[i];
            var menuItem = new dijit.MenuItem({
                label: actionItem.label,
                iconClass: actionItem.iconClass,
                action: actionItem,
                dispatcher: dispatcher,
                onClick: function() {
                    this.dispatcher(this.action);
                }
            });
            menu.addChild(menuItem);
        }
        menu.startup();
        return menu;
    },
    
    destroy: function()
    {
        
    }
});

})();