/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //    
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.horizontalTree.Panel");

dojo.require("dojo.fx");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit._Container");

dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.horizontalTree.FxUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.horizontalTree.PanelItem");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.horizontalTree.PanelHeader");

(function() {

var PanelItem = com.ibm.asq.common.web.ui.internal.widgets.horizontalTree.PanelItem;
var ItemFadeInDelay = 80;
var AnimationDuration = 200;

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.horizontalTree.Panel",
             [dijit._Widget, dijit._Templated, dijit._KeyNavContainer],
{    
    templateString: '<div id="${id}" dojoAttachPoint="domNode" class="com-ibm-asq-common-widget-htree-panel">' +
                    '  <div dojoType="com.ibm.asq.common.web.ui.internal.widgets.horizontalTree.PanelHeader" dojoAttachPoint="header">' +
                    '  </div>' +
                    '  <div class="panel-container" dojoAttachPoint="container">' +
                    '    <div dojoAttachPoint="loadingProgressNode" class="progress-area" style="display:none">' +

// Not yet enable cancel loading, still need more work
//                    '      <div class="cancel-load-action" dojoAttachEvent="onclick:onLoadingCanceled" dojoAttachPoint="cancelLoad">' +
//                    '      </div>' +

                    '      <div class="progress-image">' +
                    '        <span dojoAttachPoint="loadingLabelNode" class="progress-label">${loadingLabel}</span>' +
                    '      </div>' +
                    '    </div>' +
                    
                    // our main children container node, expected by _Container
                    '    <ul tabindex="-1" dojoAttachPoint="containerNode">' +
                    '    </ul>' +
                    '  </div>' +
                    
                    /* TODO: add paging.
                     * 
                    '  <div class="panel-footer" dojoAttachPoint="footer">' +
                    '    <div class="panel-page-control" dojoAttachPoint="pagerNode"' +
                    '      <div>' +
                    '      </div>' +
                    '    </div>' +
                    '  </div>' +
                    */
                    '</div>',
                    
    widgetsInTemplate: true,
    loadingLabel: MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_loading"),

    selectionMode: "single", // or "multi"
    model: null,
    title: null,
    width: 100,
    identity: null,
    ruler: null,
    
    postCreate: function()
    {
        this.inherited('postCreate', arguments);
        dojo.style(this.domNode, "display", "none");
        
        this.pageCount = 0;
        this.currOpenItem = null;
        this.selection = null;
        this.itemMap = {};
        this.header.ruler = this.ruler;
        
        this.title = this.model.getLabel(this.modelItem);
        this.identity = this.model.getIdentity(this.modelItem);
        this._installActions();
        
        // attach the up and down keys to navigate the panel
        // items.
        this.connectKeyNavHandlers([dojo.keys.UP_ARROW], [dojo.keys.DOWN_ARROW, dojo.keys.TAB]);
    },

    isFocusable: function()
    {
        return true;
    },

    focus: function()
    {
        if (this.selection) {
            var uiItem = this.getUIForItem(this.selection);
            this.focusChild(uiItem);
        }
        else {
            this.focusFirstChild();
        }
    },
    
    update: function()
    {
        this.setTitle(this.model.getLabel(this.modelItem));
    },
    
    setModelItem: function(item)
    {
        this.modelItem = item;
        this.identity = this.model.getIdentity(item);
        this.clearAll();
        this.update();
        this._installActions();
    },
        
    _installActions: function()
    {
        if(this.actionRegistry) {
            var actionDesc = this.actionRegistry.getPanelActions(this.model, this.modelItem);
            if(actionDesc) {
                this.actionDesc = this.actionDesc;
                this.tools = this.header.buildActionTools(actionDesc, dojo.hitch(this, "_handlePanelAction"));
                this.updatePanelActionStates();
            }
        }        
    },
    
    updatePanelActionStates: function()
    {
        var tools = this.header.tools;
        if(!tools) return;
        
        var hasSelect = this.selection != null;
        var enabled;
        
        for(var i = 0; i  < tools.length; i++) {
            // reset enable flag for the new action.
            enabled = hasSelect;
            
            if (tools[i].action.needSelectionEnablement) {
                if (hasSelect && tools[i].action.enablementCallback && dojo.isFunction(tools[i].action.enablementCallback)) {
                    enabled = tools[i].action.enablementCallback(this.model, this.selection)
                }
                tools[i].setDisabled(!enabled);
            }
        }
    },
    
    _handlePanelAction: function(action)
    {
        var selectItem = (action.needSelectionEnablement) ? this.selection : null;
        this._invokeAction(action, selectItem);
    },
    
    _invokeAction: function(action, onItem)
    {
        if(action && action.callback && dojo.isFunction(action.callback)) {
            action.callback(this.modelItem, onItem);
        }        
    },
    
    _initUI: function()
    {
        dojo.style(this.domNode, "display", "");
        
        if(this.title) {
            this.setTitle(this.title);
        }
        
        // The container height clips the veritcal scrollbar 
        // on IE. Need to call resize again to get the correct
        // height on the container on initialize UI.
        if(dojo.isIE && this.currentDim) {
            this.width = 0;
            this.resize(this.currentDim);
        }
        this.startupKeyNavChildren();
    },
    
    resize: function(dim)
    {
        if(dim.w == this.width) {
            return;
        }
        this.width = dim.w;
        this.currentDim = dim;
        
        if(this.title) {
            this.setTitle(this.title);
        }

        dojo.marginBox(this.domNode, dim);
        
        if (!this.headerHeight) {
            var cs = dojo.getComputedStyle(this.header.domNode);
            var headerH = dojo._toPixelValue(this.header.domNode, cs.height);
            this.headerHeight = Math.round(headerH);
        }
        var margin = dojo.isIE ? 10 : 4;
        dojo.marginBox(this.container, {h: dim.h - this.headerHeight - margin});
        
        // update the item's drawable size too.
        for(var id in this.itemMap) {
            this.itemMap[id].resize(this.width);
        }
    },
    
    setTitle: function(title)
    {
        var labelClass = null;
        
        this.header.widthHint = this.width;
        this.title = title;

        if(this.getLabelClass && dojo.isFunction(this.getLabelClass)) {
            labelClass = this.getLabelClass(this.modelItem);
        }
        this.header.setTitle(this.title, labelClass);        
    },
    
    startup: function(callback, animated)
    {
        if(!callback) {
            return;
        }
        if (animated) {
            this.expand(0, callback);
        }
        else {
            this._initUI();
            callback();
        }
    },
    
    expand: function(delay, callback)
    {
        var style = this.domNode.style;
        style.width = 1;
        
        dojo.style(this.header, "display", "none");
        var wipeIn = dojo.fx.wipeInHorizontal({
            node: this.domNode,
            duration: AnimationDuration,
            width: this.width - 4,
            onEnd: dojo.hitch(this, function(){
                this._initUI();
                callback();
            })
        });
        wipeIn.play(delay);
    },
    
    collapse: function(delay, callback)
    {
        var wipeOut = dojo.fx.wipeOutHorizontal({
            node: this.domNode,
            duration: AnimationDuration,
            onEnd: function() {if(callback) callback();}
        });
        wipeOut.play(delay || 0);
    },
    
    setStartLoading: function(message)
    {
        var msg = (message || this.loadingLabel);
        this.loadingLabelNode.innerHTML = msg;

        dojo.style(this.loadingProgressNode, "display", "block");        
    },
    
    setEndLoading: function()
    {
        dojo.style(this.loadingProgressNode, "display", "none");        
    },
    
    clearAll: function()
    {
        this.containerNode.innerHTML = "";
        
        for(var item in this.itemMap) {
            var itemUI = this.itemMap[item];
            itemUI.destroy();
        }
        this.itemMap = {};
        this.clearSelection();
    },

    isSelected: function(item) 
    {
        // simple case for single selection for now...
        if(this.selection) {
            var iid = this.model.getIdentity(item);
            var sid = this.model.getIdentity(this.selection);
            
            return iid == sid;
        }
        return false;  
    },
    
    setSelection: function(item)
    {
        this.selection = item;
    },
    
    clearSelection: function()
    {
        this.selection = null;
    },
    
    getUIForItem: function(item)
    {
        var id = this.model.getIdentity(item);
        var uiItem = this.itemMap[id];
        
        if (uiItem) {
            return uiItem.domNode;
        }
        return null;
    },
    
    removeItem: function(item)
    {
        var id = this.model.getIdentity(item);
        var uiItem = this.itemMap[id];
        if (uiItem) {
            var clearSelection = this.isSelected(item);
            
            dojo.fx.wipeOut({
                node: uiItem.domNode,
                duration: AnimationDuration,
                onEnd: dojo.hitch(this, function(){
                    this.removeChild(uiItem);
                    uiItem.destroy();
                    delete this.itemMap[id];
                    
                    if(clearSelection) {
                        this.clearSelection();
                        this.updatePanelActionStates();
                    }
                })
            }).play();
        }
        else {
            console.debug("Item to remove does not exists: " + id + "!");
        }
    },
       
    addItem: function(item, pos, setFocus)
    {
        var panelItem = this._addItem(item, true, pos);
        dojo.fadeIn({
            node:panelItem.domNode,
            onEnd: function() {
                if(setFocus) {
                    panelItem.focus();
                }
            }
        }).play(0);
    },

    addItems: function(items)
    {
        this.setEndLoading();
        var delay = 0;
        var max = items.length;
        
        for(var i = 0; i < max; i++) {
            var panelItem = this._addItem(items[i], true);
            
            // increment the delay of fading in the items 
            // this gives the effect of items are added
            // one after another.
            delay += ItemFadeInDelay;
            dojo.fadeIn({node:panelItem.domNode}).play(delay);
        }
    },
    
    updateItem: function(item)
    {
        var uiItem;
        
        if (item) {
            uiItem = this.itemMap[this.model.getIdentity(item)];
            if (uiItem) {
                uiItem.update();
            }
            if(this.isSelected(item)) {
                this.updatePanelActionStates();
            }
        }
        else {
            // update all items in the panel.
            this.setTitle(this.title);
            for(var id in this.itemMap) {
                uiItem = this.itemMap[id];
                if(uiItem) {
                    uiItem.update();
                }
            }
        }
    },
    
    _addItem: function(item, isFadeIn, pos)
    {
        var panelItem = new PanelItem({
            model: this.model,
            modelItem: item,
            getIconClass: this.getIconClass,
            getLabelClass: this.getLabelClass,
            identity: this.model.getIdentity(item),
            actionRegistry: this.actionRegistry,
            ruler: this.ruler,
            widthHint: this.width,
            onOpen: dojo.hitch(this, "_handleItemOpen"),
            onContextActionInvoke: dojo.hitch(this, "_handleItemContextActions")
        });
        this.itemMap[panelItem.identity] = panelItem;
        
        if(isFadeIn)
            dojo.style(panelItem.domNode, "opacity", "0");
        
        this.addChild(panelItem, pos);                
        return panelItem;
    },
    
    _handleItemOpen: function(item)
    {
        if(this.currOpenItem === item)
            return;
        
        var uiItem;    
        if(this.currOpenItem) {
            uiItem = this.itemMap[this.model.getIdentity(this.currOpenItem)];
            if(uiItem) {
                uiItem.setOpenState(false);
            }
            this.currOpenItem = null;
        }
        this.currOpenItem = item;
        this.setSelection(this.currOpenItem);
        this.updatePanelActionStates();
        this.onItemOpen(this, item);
    },
    
    _handleItemContextActions: function(action, item)
    {
        this._invokeAction(action, item);
    },
    
    onItemSelect: function()
    {
    },
    
    onItemOpen: function(panel, item)
    {  
    },
    
    onLoadingCanceled: function(panel)
    {
    },
    
    dummy:null
});

})();
