/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
//

//
//
// Summary:
//      Subclassed from the dijit.form.ValidationTextBox to add notification
//      of value changed when the user is editing the field value.
//
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.layout.ASQValidateTextBox");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.ReTemplatedValidationTextBox");

dojo.declare(
    "com.ibm.asq.common.web.ui.internal.widgets.layout.ASQValidateTextBox",
    com.ibm.asq.common.web.ui.internal.widgets.ReTemplatedValidationTextBox,
{   
    // Set this attribute to true to receive onChange notification 
    // when the user is editing the field.
    reportChangeWhenEdit: false,
    
    postCreate: function()
    {
        this.inherited('postCreate', arguments);
        
        // avoid getting dijit.scrollIntoView being called when
        // this form field is getting focus. Auto scrolling when
        // a field has focus doesn't have too much value and
        // this creates a possible leak by the scroll code on the
        // node being scrolled.
        this._scroll = false;
        
        // the validation text box has a "dead" area to the right of
        // true edit area. It's the validation icon that would show up
        // when there's validation errors on the input. When the user 
        // click in that area, the focus is not put into the edit box
        // and ofter the focus goes to the outside area. Force the
        // focus into the edit box when this area is being clicked.
        //
        if(this.iconNode && this.iconNode.parentNode) {
            this.connect(this.iconNode.parentNode, "onclick", "focus");
        }
        this.connect(this.textbox, "onkeyup", "onkeyup");
    },

    // Summary:
    //     Override the base method that handles key up pressed event to
    //     add notification about interrim value changed.
    //
    onkeyup: function(evt)
    {
        //this.onfocus(evt);
        
        // if this is configured to report back change
        // while editing, notify the onChange method.
        if(this.reportChangeWhenEdit)
           this.onChange();
    },
    
    // Set this attribute to true to control the error state
    // explicitly and disable the build-in validation.
    externalValidation: false,
    
    // When external validation is used, this attribute identifies
    // if the text field entry is valid.
    _isValid: true,
    
    // Summary:
    //     Set the state of the text box when external validation is used.
    setIsValid: function(/* Boolean*/ valid)
    {
        this._isValid = valid;
    },
    
    // Summary:
    //     Override the base method that handles the validation to
    //     to check if external validation is used.
    isValid: function(/* Boolean*/ isFocused)
    {
        if (this.externalValidation)
            return this._isValid;
        else
            return this.inherited("isValid", arguments);
    }
});
