/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.widgets.layout.FormTextField");

dojo.require("dijit.form.TextBox");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.layout.ASQValidateTextBox");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.layout.AbstractFormField");

(function() {

var ValidateTextBox = com.ibm.asq.common.web.ui.internal.widgets.layout.ASQValidateTextBox;

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.widgets.layout.FormTextField", 
    com.ibm.rational.test.lm.web.ui.internal.widgets.layout.AbstractFormField,
{
    currentValue: null,
    
    textFieldInvalidFormatMsg: "",
    
    postCreate: function() {
        this.inherited("postCreate", arguments);
        this.textField = new ValidateTextBox({
                    name: "textField",
                    externalValidation: true,
                    reportChangeWhenEdit: true,
                    value: "",
                    'class': this._getBaseClassName()
                });
        this.fieldHolderNode.appendChild(this.textField.domNode);
        this.connect(this.textField, "onChange", "_valueChangedHandler");
        
        this.formField = this.textField.domNode;
        
        if (this.textField.textbox) {
            this._attachLabelToField(this.textField.textbox, this.fieldLabel + "-input");
        }    
    },

    _valueChangedHandler: function()
    {
        var oldValue = this.currentValue;
        this.currentValue = this.getValue(); 
        
        this.setHasError(!this.textField.isValid());
        this.onValueChanged(this, oldValue, this.currentValue);
        if (this.textField.externalValidation)
            // for external validation, use validate() will implicitly show the error tooltip
            this.textField.validate(true);
    },
    
    setHasError: function(hasError) {        
        if (!hasError && this.textFieldInvalidFormatMsg != "")
            // restore the original invalid format message on no error
            this.setErrorMessage(this.textFieldInvalidFormatMsg);
        
        // for internal validation, ignore/override the external set error. 
        // Need to preserve the actual error condition generated by
        // the self validation.
        hasError = (this.textField.externalValidation) ? hasError : !this.textField.isValid();
            
        this.inherited("setHasError", arguments);
        
        // only need to set the field's valid status if it's
        // external validation. Otherwise, it has its own valid/invalid state.
        if(this.textField.externalValidation)
            this.textField.setIsValid(!hasError);
    },
    
    setErrorMessage: function(message)
    {
        if (this.textFieldInvalidFormatMsg == "")
            // save the original invalid format message
            this.textFieldInvalidFormatMsg = this.textField.getErrorMessage();
            
        this.textField.invalidMessage = message;
    },
    
    setEdited: function(isEdited)
    {
        if (!this.textField.isValid()) {
            // use error condition for invalid entry 
            this.setHasError(true);
        } else {
            this.inherited("setEdited", arguments);
        }
    },
    
	setModified: function(isEdited)
	{
		if (!this.textField.isValid()) {
			// use error condition for invalid entry 
			this.setHasError(true);
		} else {
			this.inherited("setModified", arguments);
		}
    },
	
    setValue: function(value)
    {
        var val = (value) ? value : "";
        this.textField.setValue(val);
    },

    // Summary:
    //    Returns the value from the field. Subclass must override. A null is returned if
    //    no value is specified.
    //
    getValue: function()
    {
        var val = this.textField.getValue();
        return val;
    },

    // @override - If the field is 'self validate', we would need to return the error
    // up to our client.
    getHasError: function()
    {
        // external validation. Use our internal flag.
        if(this.textField.externalValidation)
            return this.hasError;
        
        // self validation, return the field's valid/invalid state.
        return !this.textField.isValid();
    },

    // Summary:
    //    This method is called when the field's value has been changed. Consumer of the widget 
    //    overrides this method to receive notification of the value changed event.
    //
    onValueChanged: function(oldValue, newValue)
    {
    },
    
    _getBaseClassName: function()
    {
        return "text-field";
    }
});

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.widgets.layout.FormNumericField", 
    com.ibm.rational.test.lm.web.ui.internal.widgets.layout.FormTextField,
{
    postCreate: function() {
        this.inherited("postCreate", arguments);
        this.textField.regExp = "|0|[1-9][0-9]*";
        this.textField.externalValidation = false;
    }
});

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.widgets.layout.FormIPAddressField", 
    com.ibm.rational.test.lm.web.ui.internal.widgets.layout.FormTextField,
{
    postCreate: function() {
        this.inherited("postCreate", arguments);

        var ipv4 = "((?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?))";
        var ipv6 = "(([0-9a-fA-F]{1,4}|0)(\:([0-9a-fA-F]{1,4}|0)){7})";
        var ipv6Abbr0 = "(::([0-9a-fA-F]{1,4}|0)(\:([0-9a-fA-F]{1,4}|0)){0,6})";
        var ipv6Abbr1 = "(([0-9a-fA-F]{1,4}|0)\:\:(|([0-9a-fA-F]{1,4}|0)(\:([0-9a-fA-F]{1,4}|0)){0,5}))";
        var ipv6Abbr2 = "((([0-9a-fA-F]{1,4}|0)\:)([0-9a-fA-F]{1,4}|0)\:\:(|([0-9a-fA-F]{1,4}|0)(\:([0-9a-fA-F]{1,4}|0)){0,4}))";
        var ipv6Abbr3 = "((([0-9a-fA-F]{1,4}|0)\:){2}([0-9a-fA-F]{1,4}|0)\:\:(|([0-9a-fA-F]{1,4}|0)(\:([0-9a-fA-F]{1,4}|0)){0,3}))";
        var ipv6Abbr4 = "((([0-9a-fA-F]{1,4}|0)\:){3}([0-9a-fA-F]{1,4}|0)\:\:(|([0-9a-fA-F]{1,4}|0)(\:([0-9a-fA-F]{1,4}|0)){0,2}))";
        var ipv6Abbr5 = "((([0-9a-fA-F]{1,4}|0)\:){4}([0-9a-fA-F]{1,4}|0)\:\:(|([0-9a-fA-F]{1,4}|0)(\:([0-9a-fA-F]{1,4}|0)){0,1}))";
        var ipv6Abbr6 = "((([0-9a-fA-F]{1,4}|0)\:){5}([0-9a-fA-F]{1,4}|0)\:\:(|([0-9a-fA-F]{1,4}|0)))";
        var ipv6Abbr7 = "((([0-9a-fA-F]{1,4}|0)\:){6}([0-9a-fA-F]{1,4}|0)\:\:)";
        this.textField.regExp = "|" + ipv4 + "|" + ipv6 +
                                "|" + ipv6Abbr0 + "|" + ipv6Abbr1 + "|" + ipv6Abbr2 + "|" + ipv6Abbr3 +
                                "|" + ipv6Abbr4 + "|" + ipv6Abbr5 + "|" + ipv6Abbr6 + "|" + ipv6Abbr7;
        this.textField.externalValidation = false;
    }
});

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.widgets.layout.FormMACAddressField", 
    com.ibm.rational.test.lm.web.ui.internal.widgets.layout.FormTextField,
{
    postCreate: function() {
        this.inherited("postCreate", arguments);

        var macAddr1 = "([0-9a-fA-F]{2}-[0-9a-fA-F]{2}-[0-9a-fA-F]{2}-[0-9a-fA-F]{2}-[0-9a-fA-F]{2}-[0-9a-fA-F]{2})";
        var macAddr2 = "([0-9a-fA-F]{2}:[0-9a-fA-F]{2}:[0-9a-fA-F]{2}:[0-9a-fA-F]{2}:[0-9a-fA-F]{2}:[0-9a-fA-F]{2})";
        this.textField.regExp = "|" + macAddr1 + "|" + macAddr2;
        this.textField.externalValidation = false;
    }
});

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.widgets.layout.GeneralValidateTextField", 
    com.ibm.rational.test.lm.web.ui.internal.widgets.layout.FormTextField,
{
    // specify a regex to enforce validation. If no regex is specified, no validation is done.
    regExp: null,
    
    postCreate: function() {
        this.inherited("postCreate", arguments);
        
        if (this.regExp) {
            this.textField.regExp = this.regExp;
            this.textField.externalValidation = false;
        }
    }
});

})();
