/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
// author: Paul Beaulieu
//
// Summary: This is a widget to choose options.
//

dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.propertyeditor.PropertyEditor");

dojo.require( "dijit._Widget" );
dojo.require( "dijit._Templated" );
dojo.require("dojox.data.dom");
dojo.require( "com.ibm.asq.common.web.ui.internal.view.common.MoreLessCollapsibleArea" );
dojo.require( "com.ibm.asq.common.web.ui.internal.util.PropertyAdder" );
dojo.require( "com.ibm.asq.common.web.ui.internal.widgets.propertyeditor.PropertyDialog" );
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");

dojo.require("com.ibm.asq.common.web.client.internal.Constants");




(function() {
	
var Constants = com.ibm.asq.common.web.client.internal.Constants;
	
dojo.declare(
    "com.ibm.asq.common.web.ui.internal.widgets.propertyeditor.PropertyEditor",
    [com.ibm.asq.common.web.ui.internal.util.PropertyAdder, dijit._Widget, dijit._Templated],
{
    templateString: '<div dojoAttachPoint="domNode">' +
                    ' <div dojoAttachPoint="actionArea" style="display:none;margin-bottom:12px;height:20px;width:98%;">' +
                    '   <div dojoAttachPoint="actionNode" style="float:right;"></div>' +
                    ' </div>' +
					' <div id="${id}" class="property-editor" dojoAttachPoint="widgetNode" tabindex="-1">' +
				 	'  <table class="property-editor-table" border="0" cellspacing="0">' +
					'   <thead>' +
					'    <th dojoAttachPoint="propertyNameNode" class="property-name"/>' +
					'    <th class="property-equals"/>' +
					'    <th dojoAttachPoint="propertyValueNode" class="property-value"/>' +
					'    <th class="property-edit"/>' +
					'    <th class="property-delete"/>' +
					'   </thead>' +
					'   <tbody dojoAttachPoint="rowsNode">' +
					'   </tbody>' +
					'  </table>' +
                    ' </div>' +
					'</div>',

    showFrame: true,
	postCreate: function()
	{
        if (this.showFrame) {
            var section = new com.ibm.asq.common.web.ui.internal.view.common.MoreLessCollapsibleArea({
                title: MessageUtil.getMessage(MessageBundle.COMMON, "Properties")
            });
            section.contentNode.appendChild(this.widgetNode);
            section.addAction(MessageUtil.getMessage(MessageBundle.COMMON, "add"), dojo.hitch(this, this.addPropertyRow), true, "add-icon-image")
            this.domNode.appendChild(section.domNode);
        }
        else {
            dojo.style(this.actionArea, "display", "");
            var arg = {
                label: MessageUtil.getMessage(MessageBundle.COMMON, "add"),
                showLabel: false,
                iconClass: "add-icon-image",
                onClick: dojo.hitch(this, function() {
                    this.addPropertyRow(true);
                })
            };
            var button = new dijit.form.Button(arg);
            this.actionNode.appendChild(button.domNode);            
        }
		var usedWidth = ( this.width ) ? this.width : "600px";
		
		this.domNode.style.width = "100%";
		this.nameEditors = [];
		this.valueEditors = [];
		this.sectionContainer = section;
	},
	
	setValueEditSize: function( size )
	{
		this._valueSize = size;
	},
	
	setNameEditSize: function( size )
	{
		this._nameSize = size;
	},
	
    // Add a property line in the editor. Optionally, specify whether the
    // property can be deleted by the user. If not specified, it's default 
    // to allow deletion, i.e., a delete action is create.
	addProperty: function( name, value, /*optional, true to allow deletion */canDelete )
	{
		var field = this.addPropertyRow(canDelete);
		field.nameNode.value = name;
		field.valueNode.value = value;
		field.nameNode.disabled = true;	
	},
	
	clear: function()
	{
		dojox.data.dom.removeChildren( this.rowsNode );
	},
	
	addPropertyRow: function( canDelete )
	{
		//debugger;
		var row = document.createElement( "tr" );
		row.className = "property-editor-row";
		var valueNode = {"node" : null };
		var nameNode = {"node": null };
		
		row.appendChild( this._constructNameCell( nameNode ) );
		row.appendChild( this._constructEqualCell() );			
		row.appendChild( this._constructValueCell( valueNode ) );
        
        /*
         * No real use to be able to edit the field in a dialog.
         * the edit box is nearly the same width.
		row.appendChild( this._constructEditCell( nameNode, valueNode ) );
        */
        if(typeof canDelete == "undefined" || canDelete)
		  row.appendChild( this._constructDeleteCell() );
          
		this.rowsNode.appendChild( row );
/*		nameNode[ "node" ].style.height = row.style.height;
		valueNode[ "node" ].style.height = row.style.height;*/
		return( new com.ibm.asq.common.web.ui.internal.widgets.propertyeditor.PropertyEditor.Field( nameNode[ "node" ], valueNode[ "node" ] ) );
	},
			
	_constructNameCell: function( nameNode )
	{
		var cell = document.createElement( "td" );
		cell.className = "property-name";
		nameNode[ "node" ] = this._constructEditControl( "property-name", ( ( this._nameSize ) ? this._nameSize : 20 ) );
		cell.appendChild( nameNode[ "node" ] );
		return( cell );			
	},
	
	_constructEqualCell: function()
	{
		var cell = document.createElement( "td" );
		cell.className = "property-equals";
		cell.setAttribute( "align", "center" );
		cell.appendChild( document.createTextNode( "=" ) );
		return( cell );			
	},
	
	_constructValueCell: function( valueNode )
	{
		var cell = document.createElement( "td" );
		cell.className = "property-value";
		valueNode["node"] = this._constructEditControl( "property-value-editor", 
													  ( ( this._valueSize ) ? this._valueSize : "75%" ),
													  {"onkeydown" : dojo.hitch( this, this.propertyValueChanged )} );			 
		cell.appendChild( valueNode["node"] );			
		
		return( cell );			
	},
	
	_constructDeleteCell: function()
	{
		var cell = document.createElement( "td" );
		cell.className = "property-delete";
		cell.appendChild( this._constructDeleteButton() );
		return( cell );			
	},
	
	_constructEditCell: function( nameNode, valueNode )
	{
		var cell = document.createElement( "td" );
		cell.className = "property-edit";
		cell.appendChild( this._constructEditButton( nameNode, valueNode ) );
		return( cell );					
	},
	
	_constructEditControl: function( classname, size, callbacks )
	{
		var input = document.createElement( "input" );
		var cls = " property-editor-textbox-border-all";
		input.className = classname + cls;
		input.type = "text";
		this.addProperties( input, callbacks );
		input.setAttribute( "size", size );
		return( input );
	},
	
	_constructDeleteButton: function()
	{
		var args = {"iconClass": "delete-property",
					"label": Constants.getMessage("labelDeleteProperty"), 
					"showLabel" : false,
					"onClick": function( e ) { this.clientCallback( e ); } };
		var button = new dijit.form.Button( args );
		button.clientCallback = dojo.hitch( this, this.deleteProperty );
		dojo.addClass( button.domNode, "property-editor" );
		
		return( button.domNode );
	},
	
	_constructEditButton: function( nameNode, valueNode )
	{
		var args = {"iconClass": "edit-property", 
					"label": Constants.getMessage("labelEditProperty"), 
					"showLabel" : false,
					"onClick": function( e ) { this.clientCallback( this.callbackData, e ); } };
		var button = new dijit.form.Button( args );
		button.clientCallback = dojo.hitch( this, this.editProperty );
		button.callbackData = { "name": nameNode["node"], "value": valueNode["node"] };
		dojo.addClass( button.domNode, "property-editor" );
		
		return( button.domNode );
	},

	deleteProperty: function( event )
	{
		var elem = null;
		
		if( window.event )
		{
			elem = window.event.srcElement;	
		}
		else
		{
			elem = event.target;
		}
		
		while( elem && elem.tagName.toLowerCase() != "tr" )
		{
			elem = elem.parentNode;
		}
		
		if( elem )
		{
			elem.parentNode.removeChild( elem );
		}
	},
	
	propertyValueChanged: function( event )
	{
		var target = null;
		
		if( window.event )
		{
			target = window.event.srcElement;
		}
		else
		{
			target = event.target;
		}
				
		return( true );
	},
	
	onKeyDownValue: function( event )
	{
		//debugger;
		var e = event;
	},
	
	editProperty: function( data )
	{
		var dialog = new com.ibm.asq.common.web.ui.internal.widgets.propertyeditor.PropertyDialog(
			{title: Constants.getMessage("labelEditProperty"), 
			 "callback": dojo.hitch( this, this.updateProperty ),
			 "propertyName": data["name"].value, 
			 "propertyValue": data["value"].value,
			 "callbackdata": data});
		dialog.show();
	},
	
	updateProperty: function( data, newValue )
	{
		data["value"].value = newValue;
	},
	
	getProperties: function()
	{
		var stack = [];
		var properties = {};
		
		for( var iRow = 0; iRow < this.rowsNode.rows.length; iRow++ )
		{
			var inputs = this.rowsNode.rows[ iRow ].getElementsByTagName( "input" );
			var name = "";
			var value = "";
			
			for( var iInput = 0; iInput < inputs.length; iInput++ )
			{
				if( inputs[ iInput ].className.indexOf( "property-name" ) != -1 )
				{
					name = inputs[ iInput ].value;
				}
				else if( inputs[ iInput ].className.indexOf( "property-value-editor" ) != -1 )
				{
					value = inputs[ iInput ].value;
				}
			}
			
			if( name && name.length > 0 && value && value.length > 0 )
			{
				properties[ name ] = value;
			}
		}
		
		return( properties );
	}
}
);

dojo.declare(
    "com.ibm.asq.common.web.ui.internal.widgets.propertyeditor.PropertyEditor.Field",
    null,
{
	constructor: function( nameNode, valueNode )
	{
		this.nameNode = nameNode;
		this.valueNode = valueNode;
	}
});

})();

