/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008,2009. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
//
// Summary:
//    RichTextEditor extends dojo's dijit.Editor widget to provide generic enhancements 
//     for rich text editing across the QM application.  If there are component-specific 
//     enhancements required, those components should subclass RichTextEditor.
//       
//
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.richtext.RichTextEditor");

dojo.require("dojox.data.dom");
dojo.require("dijit.Editor");

//must require the editor plugins to be used...
dojo.require("dijit._editor.plugins.AlwaysShowToolbar");
dojo.require("dijit._editor.plugins.TextColor");
dojo.require("dijit._editor.plugins.FontChoice");
dojo.require("dijit._editor.plugins.LinkDialog");

dojo.require("com.ibm.asq.common.web.client.internal.CommonPreferenceClient");

(function(){

    dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.richtext.RichTextEditor", [dijit.Editor], {
    
        //@Overridden from dijit.Editor
        constructor: function(){
            this.defaultPlugins = ['fontSize', 'fontName', 'foreColor', 'hiliteColor', 'dijit._editor.plugins.AlwaysShowToolbar'];
            
            //provide default plugins if none specified
            if (!arguments[0].extraPlugins) {
                arguments[0].extraPlugins = this.defaultPlugins;
            }
            else {
                //prepend default plugins to extraPlugins param
                var plugins = this.defaultPlugins;
                for (var i = 0; i < arguments[0].extraPlugins.length; i++) {
                    if (!this._contains(plugins, arguments[0].extraPlugins[i])) {
                        plugins.push(arguments[0].extraPlugins[i]);
                    }
                }
                arguments[0].extraPlugins = plugins;
            }
            com.ibm.asq.common.web.ui.internal.widgets.richtext.RichTextEditor.superclass.constructor.apply(this, arguments);
			this._editorCommandsLocalized = true;
        },
        
        //@Overridden from dijit.Editor
        //reduce the width of Font Size plugin
        addPlugin: function(/*String||Object*/plugin, /*Integer?*/ index){
            com.ibm.asq.common.web.ui.internal.widgets.richtext.RichTextEditor.superclass.addPlugin.apply(this, arguments);
            if (plugin == "fontSize") {
                var currentPlugin = this._plugins[index];
                currentPlugin.button.domNode.style.width = "100px";
            }
            else if (plugin == "fontName") {
                var currentPlugin = this._plugins[index];
                currentPlugin.button.domNode.style.width = "100px";
            }else if (plugin == "dijit._editor.plugins.EnterKeyHandling" && !dojo.isIE){
				var currentPlugin = this._plugins[index];
				this.contentDomPostFilters.push(dojo.hitch(currentPlugin, "singleLinePsToRegularPs"));
			}
			if(this._plugins[index].button){
				//FIX defect 4697. 
				//@Overridden: setDisabled function from dijit.form._FormWidget. 
				//When the domNode is disabled, the title should appear when the mouse is over the button. 
				this._plugins[index].button._setDisabledAttr = function(/*Boolean*/ value){
					// summary:
					//		Set disabled state of widget.
	
					this.disabled = value;
					//Fix: remove this line. 
//					dojo.attr(this.focusNode, 'disabled', value);
					dijit.setWaiState(this.focusNode, "disabled", value);

					if(value){
						//reset those, because after the domNode is disabled, we can no longer receive
						//mouse related events, see #4200
						this._hovering = false;
						this._active = false;
						// remove the tabIndex, especially for FF
						this.focusNode.removeAttribute('tabIndex');
					}else{
						this.focusNode.setAttribute('tabIndex', this.tabIndex);
					}
					this._setStateClass();
				};
			}
        },
        
        //@Overridden from dijit._editor.RichText
        //fix the bug of this.document.queryCommandValue("fontsize") in FF3
        queryCommandValue: function(command){
            // summary: check the value of a given command
            if (this.disabled) {
                return false;
            }
            command = this._normalizeCommand(command);
            if (dojo.isIE && command == "formatblock") {
                return this._local2NativeFormatNames[this.document.queryCommandValue(command)];
            }
			var val = "";
            if (command == "fontsize" || command == "fontname") {
                try {
                    val =  this.document.queryCommandValue(command);
                } 
                catch (e) {
                }
                if(command == "fontsize" && (val == null || val == ""))
					val = "2";
				if (command == "fontname") {
					if (val == null || val == "") 
						val = "Arial";
					else 
						if (dojo.isString(val) && val.indexOf(',') > -1)//for values like ""Arial",sans-serif"
							val = val.split(',')[0];
							val = val.replace(/"/g,'');
				}
				return val;
            }
            return this.document.queryCommandValue(command);
        },
		
		//@Overridden from dijit._editor.RichText
		//FIX 6963: Remove the style "p{ margin: 1em 0 !important; }" to fix the truncation on the left margin area.
		_getIframeDocTxt: function(/* String */ html){
			var _cs = dojo.getComputedStyle(this.domNode);
            
            // If the default content is empty, the base rich text
            // widget inserts a space. This creates a leading space in the
            // body of the editor and it would saved in to the content if
            // the user doesn't backspace to delete it.
            if(html == "&nbsp;") {
                html = "";
            }
            
			if(dojo.isIE || (!this.height && !dojo.isMoz)){
				html="<div style=\"height: 100%;\">"+html+"</div>";
			}
			try {
				var font = [_cs.fontWeight, _cs.fontSize, _cs.fontFamily].join(" ");
			}catch(e){
				//IE throws exception here for fontSize and fontFamily
			}
	
			var lineHeight = _cs.lineHeight;
			if(lineHeight.indexOf("px") >= 0){
				lineHeight = parseFloat(lineHeight)/parseFloat(_cs.fontSize);
			}else if(lineHeight.indexOf("em")>=0){
				lineHeight = parseFloat(lineHeight);
			}else{
				lineHeight = "1.0";
			}
			var userStyle = "";
			this.style.replace(/(^|;)(line-|font-?)[^;]+/g, function(match){ userStyle += match.replace(/^;/g,"") + ';' });
			return [
				this.isLeftToRight() ? "<html><head>" : "<html dir='rtl'><head>",
				(dojo.isMoz ? "<title>" + this._localizedIframeTitles.iframeEditTitle + "</title>" : ""),
				"<style>",
				"body,html {",
				"   background:transparent;",
				"   padding: 1em 0 0 0;",
				"   margin: -1em 0 0 0;", // remove extraneous vertical scrollbar on safari and firefox
				"   height: 100%;",
				"}",
				"body{",
				"   top:0px; left:0px; right:0px;",
				"   font:", font, ";",
					((this.height||dojo.isOpera) ? "" : "position: fixed;"),
				// FIXME: IE 6 won't understand min-height?
				"   min-height:", this.minHeight, ";",
				"   line-height:", lineHeight,
				"}",
//				"p{ margin: 1em 0 !important; }",
				(this.height ? // height:auto undoes the height:100%
					"" : "body,html{height:auto;overflow-y:hidden;/*for IE*/} body > div {overflow-x:auto;/*for FF to show vertical scrollbar*/}"
				),
				"li > ul:-moz-first-node, li > ol:-moz-first-node{ padding-top: 1.2em; } ",
				"li{ min-height:1.2em; }",
				"</style>",
				this._applyEditingAreaStyleSheets(),
				"</head><body style='"+userStyle+"'>"+html+"</body></html>"
			].join(""); // String
		},
        
		//@Overridden from dijit._editor.RichText
        //fix the bug of pasting formatted text from Word into rich text editor causes error:
		//"Warning: unresponsive script" 
        _postFilterContent: function(/*DomNode|DomNode[]|String?*/dom,/*Boolean?*/ nonDestructive){
            // summary:
            //		filter the output after getting the content of the editing area
            var ec;
            if (!dojo.isString(dom)) {
                dom = dom || this.editNode;
				if (dom) {
					dom = this._removeWordSpecificContent(dom);
					if (this.contentDomPostFilters.length) {
						if (nonDestructive && dom['cloneNode']) {
							dom = dom.cloneNode(true);
						}
						dojo.forEach(this.contentDomPostFilters, function(ef){
							dom = ef(dom);
						});
					}
					ec = this.getChildrenHtml(dom);
				}else{
					dom = "";
					ec = dom;
				}
            }
            else {
                ec = dom;
            }
			
			//separate \s|\xA0:  
			//hack: don't know why it doesn't work if using /^(?:\s|\xA0)+/g
            if (!ec.replace(/^(?:\s)+/g, "").replace(/(?:\s)+$/g, "").length) {
                ec = "";
            }
            if (!ec.replace(/^(?:\xA0)+/g, "").replace(/(?:\xA0)+$/g, "").length) {
				ec = "";
			}
            dojo.forEach(this.contentPostFilters, function(ef){
                ec = ef(ec);
            });
            return ec;
        },
		
        // @Overridden from dijit.Editor
        // For IE, the initial content is set after the Editor widget is created and opened.
        // See RichTextControl._createEditor().  The savedContent is empty and as the first
        // entry of the Undo steps stack, the last undo will wipe out the initial content.
        // RichTextControl is passing in the initialContent to be used as the first entry
        // of the Undo steps stack here.
        //
        _beginEditing: function(cmd){
            if(this._steps.length===0){
                if (dojo.isIE) {
                    this.savedContent = this.initialContent;
                }
                this._steps.push({'text':this.savedContent,'bookmark':this._getBookmark()});
             }
        },
				
        _removeWordSpecificContent:function(dom){
			var innerHtml = dom.innerHTML;
			if (innerHtml) {
				// Remove meta/link tags
				innerHtml = innerHtml.replace(/<(META|LINK)[^>]*>\s*/gi, '');
				//filter out the O:P tags introduced by MS Word. 
				//They cause nested P tags to appear in output of getChildrenHtml which isnt correctly handled by IE
				innerHtml = innerHtml.replace(/<o:p>[\s\S]*?<\/o:p>/g, '&nbsp;');
				//Remove smartTags
				innerHtml = innerHtml.replace(/<o:smarttagtype[^>]*>[\s\S]*?<\/o:smarttagtype>/g,'');
				//Remove Comments
				innerHtml = innerHtml.replace(/<\!--[\s\S]*?-->/g, '' );
				innerHtml = innerHtml.replace(/\n\n/g, '' );
				//Remove Style Tag
				innerHtml = innerHtml.replace( /<STYLE[^>]*>([\s\S]*?)<\/STYLE[^>]*>/gi, '' ) ;

			}
			dom.innerHTML = innerHtml;
			var children = dom.getElementsByTagName("a");
			var len = children?children.length:0;
			var doc = document;
			//remove OLE_LINK anchor elements
			for (var i = 0; i < len; i++) {
				var child = children[i];
				if (child.name && child.name.indexOf("OLE_LINK") > -1) {
					if (child.previousSibling === null && child.nextSibling === null) {
						child.parentNode.innerHTML = child.innerHTML;
					}else{
						var spanElem = doc.createElement('span');
						spanElem.innerHTML = child.innerHTML;
						child.parentNode.replaceChild(spanElem,child);
					}
					//the node is removed from the children's list since this is a live dom content, 
					//so reduce the length and counter
					len--;i--;
				}else{
					// If <a> element has no href attribute, set the text color to black. Fix the wi 7341
					if(!child.href){
						child.style.color = "#000";
					}
				}
			}
			return dom;
		},
         onKeyUp: function(e){
            this.inherited("onKeyUp", arguments);
				//For defect 7851
			var k = e.keyCode, ks = dojo.keys;
			if (e.ctrlKey && !e.altKey) {//undo and redo only if the special right Alt + z/y are not pressed IBM#79329
				if (k === 90 || k === 122 || k === 89 || k === 121) { //z & y
					dojo.stopEvent(e);
					this.onChange();
					return;
				}
			}
			switch (k) {
				case ks.ENTER:
				case ks.BACKSPACE:
				case ks.DELETE:
					this.onChange();
					break;
				case 88: //x
				case 86: //v
					if (e.ctrlKey && !e.altKey && !e.metaKey) {
						this.onChange();
						break;
					}
				//pass through
				default:
					if (!e.ctrlKey && !e.altKey && !e.metaKey && (e.keyCode < dojo.keys.F1 || e.keyCode > dojo.keys.F15)) {
						this.onChange();
						break;
					}
				//pass through
				case ks.ALT:
				case ks.UP_ARROW:
				case ks.DOWN_ARROW:
				case ks.LEFT_ARROW:
				case ks.RIGHT_ARROW:
				case ks.HOME:
				case ks.END:
				case ks.PAGE_UP:
				case ks.PAGE_DOWN:
				//maybe ctrl+backspace/delete, so don't set dirty when ctrl is pressed
				case ks.CTRL:
				case ks.SHIFT:
				case ks.TAB:
					break;
			}
        },
        
        execCommand: function(/*String*/command, argument){
            this.inherited("execCommand", arguments);
			if((command.toLowerCase() == "hilitecolor")&&(dojo.isMoz)){
				this.document.execCommand("styleWithCSS", false, true);
			}
            this.onChange();
        },

        _contains: function(array, value){
            for (var i = 0; i < (array ? array.length : 0); i++) {
                if (value == array[i]) {
                    return true;
                }
            }
            return false;
        },
        
        _onBlur: function(e){
			//do nothing
		},
		//only process one _endEditing at a time. Multiple calls can occur if the content is large and the browser is IE
		_endEditing: function(ignore_caret){
			if(!this._endingEdit){
				this._endingEdit = true;
				this.inherited(arguments);
				this._endingEdit = false;
			}
		},
		//copy from dijit._editor.html
        escapeXml: function(/*String*/str, /*Boolean*/ noSingleQuotes){
            //summary:
            //		Adds escape sequences for special characters in XML: &<>"'
            //		Optionally skips escapes for single quotes
            str = str.replace(/&/gm, "&amp;").replace(/</gm, "&lt;").replace(/>/gm, "&gt;").replace(/"/gm, "&quot;");
            if (!noSingleQuotes) {
                str = str.replace(/'/gm, "&#39;");
            }
            return str; // string
        },
        
		//copy from dijit._editor.html
        getNodeHtml: function(/* DomNode */node){
            var output;
            switch (node.nodeType) {
                case 1: //element node
                    output = '<' + node.nodeName.toLowerCase();
                    
                    //store the list of attributes and sort it to have the
                    //attributes appear in the dictionary order
                    var attrarray = [];
                    if (dojo.isIE && node.outerHTML) {
                        var s = node.outerHTML;
                        s = s.substr(0, s.indexOf('>'));
                        s = s.replace(/(['"])[^"']*\1/g, '');//to make the following regexp safe
                        var reg = /([^\s=]+)=/g;
                        var m, key;
                        while ((m = reg.exec(s))) {
                            key = m[1];
                            if (key.substr(0, 3) != '_dj') {
                                if (key == 'src' || key == 'href') {
                                    if (node.getAttribute('_djrealurl')) {
                                        attrarray.push([key, node.getAttribute('_djrealurl')]);
                                        continue;
                                    }
                                }
                                if (key == 'style') {
                                    attrarray.push([key, node.style.cssText.toLowerCase()]);
                                }
                                else {
                                    attrarray.push([key, key == 'class' ? node.className : node.getAttribute(key)]);
                                }
                            }
                        }
                    }
                    else {
                        var attr, i = 0, attrs = node.attributes;
                        while ((attr = attrs[i++])) {
                            //ignore all attributes starting with _dj which are
                            //internal temporary attributes used by the editor
                            var n = attr.name;
                            if (n.substr(0, 3) != '_dj'                        /*&&
                         (attr.specified == undefined || attr.specified)*/
                            ) {
                                var v = attr.value;
                                if (n == 'src' || n == 'href') {
                                    if (node.getAttribute('_djrealurl')) {
                                        v = node.getAttribute('_djrealurl');
                                    }
                                }
                                if (v != '') 
                                    attrarray.push([n, v]);
                            }
                        }
                    }
                    attrarray.sort(function(a, b){
                        return a[0] < b[0] ? -1 : (a[0] == b[0] ? 0 : 1);
                    });
                    i = 0;
					var tempStr = [];//reduce string size and concatentation
                    while ((attr = attrarray[i++])) {
						tempStr = tempStr.concat([' ',attr[0],'="',this.escapeXml(attr[1], true),'"']);
                    }
                    if (node.childNodes.length) {
						tempStr = tempStr.concat(['>',this.getChildrenHtml(node),'</',node.nodeName.toLowerCase(),'>']);
                    }
                    else {
                        tempStr = tempStr.concat([' />']);
                    }
					output += tempStr.join("");
                    break;
                case 3: //text
                    // FIXME:
                    output = this.escapeXml(node.nodeValue, true);
                    break;
                case 8: //comment
                    // FIXME:
					//defect 15073: remove comment node
					output = '';
//					output = '<!--' + this.escapeXml(node.nodeValue, true) + '-->';
                    break;
                default:
                    output = "Element not recognized - Type: " + node.nodeType + " Name: " + node.nodeName;
            }
            return output;
        },
        
		//copy from dijit._editor.html
        getChildrenHtml: function(/* DomNode */dom){
            // summary: Returns the html content of a DomNode and children
            var out = "";
            if (!dom) {
                return out;
            }
            var nodes = dom["childNodes"] || dom;
            var i = 0;
            var node;
			var temp = [];
            while ((node = nodes[i++])) {
                temp.push(this.getNodeHtml(node));
            }
			out = temp.join("");
            return out; // String
        },
		

        //@Overridden from dijit._editor.RichText
        close: function(/*Boolean*/save, /*Boolean*/force){
            this.inherited(arguments);
            
            // release the focus when we closed the editor. This fixed
            // errors appeared in IE when toggling between preview &
            // edit mode, but only do this in IE.
            if(dojo.isIE && this.blur && dojo.isFunction(this.blur))
                this.blur();
		},
        
		//@Overridden from dijit._editor.RichText
		onChange: function(newContent){
			// summary:
			//		this is fired if and only if the editor loses focus and
			//		the content is changed
	
			//console.debug("Change in " + this.declaredClass);
		},
		setupDefaultShortcuts: function(){
			/*
			 * There is a bug in the dijit Richtext setupDefaultShortcuts where
			 * the return value is returned as not of this.execCommand which causes the browser to handle the key event 
			 */
			var exec = dojo.hitch(this, function(cmd, arg){
				return function(){
					return this.execCommand(cmd, arg);
				};
			});
			
			var ctrlKeyHandlers = {
				b: exec("bold"),
				i: exec("italic"),
				u: exec("underline"),
				a: exec("selectall"),
				s: function(){
					if(dojo.isFunction(this.save)){
						this.save(true);
					}
				},
				m: function(){
					this.isTabIndent = !this.isTabIndent;
				},
				
				"1": exec("formatblock", "h1"),
				"2": exec("formatblock", "h2"),
				"3": exec("formatblock", "h3"),
				"4": exec("formatblock", "h4"),
				
				"\\": exec("insertunorderedlist")
			};
			
			if (!dojo.isIE) {
				ctrlKeyHandlers.Z = exec("redo"); //FIXME: undo?
			}
			
			for (var key in ctrlKeyHandlers) {
				this.addKeyHandler(key, true, false, ctrlKeyHandlers[key]);
			}
		},
		
		//@Overridden from dijit._Widget
		isLeftToRight: function(){
			return com.ibm.asq.common.web.client.internal.CommonPreferenceClient.getIsEditorTextLTRDirection();
		},
		//override and bypass digit editor's destroy since each plugin leaves behind its buttons
		//also fix IE7's issue about this.toolbar == null
		destroy: function(){
			dojo.forEach(this._plugins, function(p){
				if(p && p.button && p.button.destroyRecursive){
					p.button.destroyRecursive();
				}
			});
			dojo.forEach(this._plugins, function(p){
				if(p && p.destroy){
					p.destroy();
				}
			});
			this._plugins=[];
			if(this.toolbar)
				this.toolbar.destroy(); 
			this.toolbar = null;
			dijit.Editor.superclass.destroy.apply(this,arguments);
		}
    });
    
})();
