/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //    The action cell renderer provides an clickable text, or image to invoke an
 //    action.
 //
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ActionCellRenderer");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ImageCellRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");

(function() {

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ActionCellRenderer", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ImageCellRenderer,
{    
    // Summary:
    //    Renders the cell content. For renderer that can modify the cell value,
    //    
    renderCell: function(viewer, cellData, extendedDisplayInfo, /*table.CellModifier*/cellModifier, rowObject, columnIndex, cellWidthHint)
    {   
        var node;
        var hasExtendedInfo = extendedDisplayInfo && dojo.isObject(extendedDisplayInfo);
        
        // if the action has image create the image by calling our 
        // parent class method.
        //
        if(hasExtendedInfo && 
           (extendedDisplayInfo['image'] ||
            extendedDisplayInfo['mouseOutImage'] ||
            extendedDisplayInfo['overImage'])) {
            node = this.inherited("renderCell", arguments);
            node.className = "image-action";
        }
        else {
            node = document.createElement("a");
            var divNode=document.createElement('div');
        	node.appendChild(divNode);
            
            node.setAttribute("href", "javascript:{}");
            this._renderCellText(viewer, cellData, extendedDisplayInfo, divNode, cellWidthHint);
        }
        
        // keep the attribute associated with the action
        node.cellModifier = cellModifier;
        node.rowObject = rowObject;
        node.columnIndex = columnIndex;
        
        this.connect(node, "onclick", "_actionClicked");
        this.cellNodes.push(node);
        return node;
    },

    _actionClicked: function(e)
    {
        // local event don't propagate the event up.
        dojo.stopEvent(e);
        
        var node = e.target;
        if(node) {
            var cellModifier = node.cellModifier;
            
            // we have a div node inside the link node. The target
            // received the click is most likely the div. Resolve
            // the node to the link node.
            if(!cellModifier) {
                node = node.parentNode;
                cellModifier = node.cellModifier;
            }
            
            var tooltip = node.tooltip;
            if(tooltip)
                tooltip.close();
           
            if(cellModifier)
                cellModifier.cellActionInvoked(node.rowObject, node.columnIndex);
        }
    }
});

})();
