/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.CheckboxCellRenderer");

dojo.require("dijit.form.CheckBox");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");

(function() {

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.CheckboxCellRenderer", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer,
{    
    // Summary:
    //    Renders the cell content. For renderer that can modify the cell value,
    //    
    renderCell: function(viewer, cellData, extendedDisplayInfo, /*table.CellModifier*/cellModifier, rowObject, columnIndex)
    {
        var span = document.createElement("span");
        var isChecked = false;
        var tooltip = null;
        
        if(extendedDisplayInfo && (typeof extendedDisplayInfo['state'] != "undefined")) {
            isChecked = extendedDisplayInfo['state'];
            tooltip = extendedDisplayInfo['tooltip'];
        }
        
        var args = {'checked':isChecked};
        var button = new dijit.form.CheckBox(args);
        var contentProvider = viewer.getContentProvider();
        var editable = viewer.isEditable() && (contentProvider ? contentProvider.getIsCellEditable(rowObject, columnIndex) : false);
        button.setDisabled(!editable);
		
		var title = this.getCellLabel(viewer, rowObject, columnIndex);
		if(title){
			var label = document.createElement("label");
			label.appendChild(document.createTextNode(title));
			label.setAttribute("for", button.focusNode.id); 
			label.style.display = "none";
			button.domNode.appendChild(label);
		}
        
        span.appendChild(button.domNode);
        span.appendChild(document.createTextNode(cellData));
        
        // the parent class's attach tooltip will not create a tooltip
        // if the string is null, or undefined. Don't have to check it here.
        this._attachTooltipToCell(button.domNode, tooltip);    
            
        // keep the associated attributes in the button.
        // 
        button.cellModifier = cellModifier;
        button.rowObject = rowObject;
        button.columnIndex = columnIndex;
        button.onChange = function() {
            if(this.cellModifier)
                this.cellModifier.cellValueChanged(!this.checked, this.checked, this.rowObject, this.columnIndex);
        }
        this.cellNodes.push(button);
        return span;
    },
    
    disconnectAll: function()
    {
        //console.debug("================CheckboxCellRenderer disconnectAll");
        this.inherited(arguments);
        var cellNodes = this.cellNodes;
        var len = cellNodes.length;
        for(var i = 0; i < len; i++) {
            cellNodes[i].destroy();
        } 
        this.cellNodes = [];
    }

});

})();
