/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.EditableTextCellRenderer");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");

(function() {

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.EditableTextCellRenderer", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer,
{
    // Indicate what types of text input. Subclass can override the text input 
    // type of the text edit box. Default to plain text input. Other supported 
    // type is 'password'.
    inputType: "text",
        
    // Summary:
    //    Renders the cell content. 
    //    
    renderCell: function(viewer, cellData, extendedDisplayInfo, /*table.CellModifier*/cellModifier, rowObject, columnIndex, cellWidthHint)
    {
        var node;
        var contentProvider = viewer.getContentProvider();
        var editable = viewer.isEditable() && (contentProvider ? contentProvider.getIsCellEditable(rowObject, columnIndex) : false);
        var d = dojo;
        var doc = document;
        
        if (editable) {
			var span = doc.createElement("span");
			node = doc.createElement("input");
            node.type = this.inputType;
			node.value = cellData;
            var widthStyled = false;
            
            if(extendedDisplayInfo && dojo.isObject(extendedDisplayInfo)) {
                // Ignore sizing information if the cell size hint is provided.
                // the cell's size hint takes precedance.
    			if (!cellWidthHint && extendedDisplayInfo['textWidth']) {
                    node.style.width = extendedDisplayInfo['textWidth'];
                    widthStyled = true;
                }    
    			if (!cellWidthHint && extendedDisplayInfo['size']) {
                    node.size = extendedDisplayInfo['size'];
                    widthStyled = true;
                }
                if(extendedDisplayInfo['class'])
                    d.addClass(node, extendedDisplayInfo['class']);
            }
            // by default, if not IE always add style to stretch the field
            // to the table cell width. 
            if(!widthStyled && !d.isIE) {
                d.addClass(node, "editText-cell");
            }
            // restrict the size of the box to the width hint.
            if(cellWidthHint)
                node.style.width = (cellWidthHint - 4) + "px";
                
            node.cellModifier = cellModifier;
            node.rowObject = rowObject;
            node.columnIndex = columnIndex;
            node.oldValue = cellData;
                    
            this.connect(node, "onkeyup", "_handleValueChanged");
			
			var cellLabel = this.getCellLabel(viewer, rowObject, columnIndex);
			if(cellLabel){
				var label = document.createElement("label");
				label.appendChild(document.createTextNode(cellLabel));
				label.style.display = "none";
				span.appendChild(label);
			}
			
			span.appendChild(node);
			if(extendedDisplayInfo && d.isObject(extendedDisplayInfo))
            	this._attachTooltipToCell(node, extendedDisplayInfo['tooltip']);
			return span;
        } 
        else {
            node = doc.createElement("span");
            this._renderCellText(viewer, cellData, extendedDisplayInfo, node, cellWidthHint);
			return node;
        }
    },
    
    _handleValueChanged: function(e)
    {
        var node = e.target;
        if (node) {
            var oldValue = node.oldValue;
            node.oldValue = node.value;
            
            var cellModifier = node.cellModifier;
            if (oldValue != node.value && cellModifier) 
                cellModifier.cellValueChanged(oldValue, node.value, node.rowObject, node.columnIndex);
        }
    },
    
    disconnectAll: function()
    {
        //console.debug("================EditableTextCellRenderer disconnectAll");
        this.inherited(arguments);
        
        var cellNodes = this.cellNodes;
        var len = cellNodes.length;
        var d = dojo;
        for(var i = 0; i < len; i++) {
            var node = cellNodes[i];
            node.cellModifier = null;
            node.rowObject = null;
            node.columnIndex = null;
            node.oldValue = null;
            if(node.tooltip && node.tooltip.destroy) {
                node.tooltip.destroy();
            }
            node.tooltip = null;
            node.onmouseover = null;
            node.onmouseout = null;
            node.onhover = null;
            node.onunhover = null;
            d._destroyElement(node);
        }
        this.cellNodes = [];
    } 
});

})();
