/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //    The action cell renderer provides an clickable text, or image to invoke an
 //    action.
 //
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.FadeInOutCellRenderer");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("jazz.ui.HoverPopup");

(function() {

var ImageCache     = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
	
dojo.declare("FadeInOutCellRenderer.HoverPopup", 
             jazz.ui.HoverPopup,
{    

	titleImagePath: dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/view/templates/common/images/collapsible-section-header-back.gif"),
    // Summary:
    //    Reset popup's background,
    //    
    open: function(target) {
		this.inherited('open', arguments);
		var m = jazz.ui._getMasterPopup();
		m.content.style.backgroundImage = "url(" + this.titleImagePath + ")";
		m.content.style.backgroundColor = "white";
		m.content.style.backgroundRepeat = "repeat-x";
	},
	
	// Summary:
    //    Set popup's background back,
    //   
	close: function() {
		this.inherited('close', arguments);
		var m = jazz.ui._getMasterPopup();
		m.content.style.backgroundImage = "";
		m.content.style.backgroundColor = "#fbfbec";
	}
});	

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.FadeInOutCellRenderer", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer,
{    
    // Summary:
    //    Renders the cell content. For renderer that can modify the cell value,
    //    
    renderCell: function(viewer, cellData, extendedDisplayInfo, /*table.CellModifier*/cellModifier, rowObject, columnIndex, cellWidthHint)
    {   
        var img = document.createElement("img");
        var tooltipInfo = cellData;
        
        if(extendedDisplayInfo && dojo.isObject(extendedDisplayInfo)) {
            var imgSrc = (typeof extendedDisplayInfo['mouseOutImage'] != "undefined") ? extendedDisplayInfo['mouseOutImage'] : extendedDisplayInfo['image'];
            img.alt = cellData;
            img.src = imgSrc;
            img.extendData = extendedDisplayInfo;
            
            if(extendedDisplayInfo['tooltipInfo'])
                tooltipInfo = extendedDisplayInfo['tooltipInfo'];
        }
        else {
            img.src = cellData;
        }

        img.tooltip = this._attachTooltipToCell(img, tooltipInfo);		
		img.cellModifier = cellModifier;
		img.rowObject = rowObject;
		img.columnIndex = columnIndex;
        
        // use the viewer's connect as it will manage the lifetime of this connection.
        // use hitch to provide this cell renderer to the function.
		this.connect(img, "onclick", "_actionClicked");
		
        this.cellNodes.push(img);
        return img;
    },
	
	_attachTooltipToCell: function(node, tooltipInfo)
    {
        var hoverPopup = new FadeInOutCellRenderer.HoverPopup({
			around: [node],
			createContents: dojo.hitch(this, tooltipInfo.createContents),
			customClass: "hoverPopupContent",
			onOpen: dojo.hitch(this, tooltipInfo.onOpen),
			onClose: dojo.hitch(this, tooltipInfo.onClose)
		});
		return hoverPopup;
    },

    _actionClicked: function(e)
    {
        if(e.target) {
            if(e.target.tooltip)
                e.target.tooltip.close();
                
            if(e.target.cellModifier)
                e.target.cellModifier.cellActionInvoked(e.target.rowObject, e.target.columnIndex);
        }
    },
    
    disconnectAll: function()
    {
        this.inherited(arguments);
        //console.debug("================FadeInOutCellRenderer disconnectAll");
        var cellNodes = this.cellNodes;
        var len = cellNodes.length;
        var d = dojo;
        for(var i = 0; i < len; i++) {
            var node = this.cellNodes[i];
            node.src = null;
            node.cellModifier = null;
            node.rowObject = null;
            node.columnIndex = null;
            node.extendData = null;            
            var tooltip = node.tooltip;
            if(tooltip && tooltip.destroy) {
                tooltip.destroy();
            }
            node.tooltip = null;
            node.onclick = null;
            node.onmouseover = null;
            node.onmouseout = null;
            node.onhover = null;
            node.onunhover = null;
            d._destroyElement(node);
        } 
        this.cellNodes = [];
    }
    
});

})();
