/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.GridTableHeaderRenderer");

dojo.require("dojo.fx");

dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableHeaderRenderer");

(function() {

var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
var ImageCache     = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
var TOP_BORDER_CSS = "table-header-top-border-line";
var BOTTOM_BORDER_CSS = "table-header-bot-border-line";

var AscendSort_Text = MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_sortAscend");
var DescendSort_Text = MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_sortDescend");
var NoSort_Text = MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_nosort");
var OpenSortCtrl_Text = MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_openSortHeader");
var CloseSortCtrl_Text = MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_closeSortHeader");
var Utility = com.ibm.asq.common.web.ui.internal.util.Utility;
var WidthSlack = (dojo.isIE) ? 16 : 12;

dojo.declare("GridTableHeaderRenderer.SortActionGroup",
             [dijit._Widget, dijit._Templated],
{
    templateString: '<span class="sort-indicator-container" dojoAttachPoint="domNode">' +
                    '  <span class="msort-up-icon-image sort-indicator-up" dojoAttachPoint="upNode" dojoAttachEvent="onclick:_handleUpClick">' +
                    '    <span class="sort-indicator-text-up" dojoAttachPoint="upNodeText">' +
                    '    </span>' +
                    '  </span>' +
                    '  <span class="msort-down-icon-image sort-indicator-down" dojoAttachPoint="downNode" dojoAttachEvent="onclick:_handleDownClick">' +
                    '    <span class="sort-indicator-text-down" dojoAttachPoint="downNodeText">' +
                    '    </span>' +
                    '  </span>' +
                    '</span>',
    
    columnPos: null,
    sortDir: null,
    nSort: null,
    
    setSortAttributes: function(sortDir, nSort)
    {
        // clear all of the sort indications first. sort dir & nth sort
        // could be null.
        this.downNodeText.innerHTML = "";
        this.upNodeText.innerHTML = "";
        
        this.upNode.title = AscendSort_Text;
        this.downNode.title = DescendSort_Text;
        
        if(sortDir == TableConstants.SORT_DIR_ASCEND) {
            dojo.toggleClass(this.upNode, "msort-up-solid-icon-image", true);            
            dojo.toggleClass(this.downNode, "msort-down-solid-icon-image", false);
            this.upNodeText.innerHTML = nSort + "";
        }
        else if(sortDir == TableConstants.SORT_DIR_DESCEND){
            dojo.toggleClass(this.upNode, "msort-up-solid-icon-image", false);
            dojo.toggleClass(this.downNode, "msort-down-solid-icon-image", true);

            this.downNodeText.innerHTML = nSort + "";
        }
        else {
            dojo.toggleClass(this.upNode, "msort-up-solid-icon-image", false);            
            dojo.toggleClass(this.downNode, "msort-down-solid-icon-image", false);
        }
        this.sortDir = sortDir;
        this.nSort = nSort;
    },
    _handleUpClick: function()
    {
        if(this.sortDir == TableConstants.SORT_DIR_ASCEND) 
            this.onClearSort(this);
        else    
            this.onSort(this, TableConstants.SORT_DIR_ASCEND);
    },    
    _handleDownClick: function()
    {
        if(this.sortDir == TableConstants.SORT_DIR_DESCEND) 
            this.onClearSort(this);
        else    
            this.onSort(this, TableConstants.SORT_DIR_DESCEND);
    },
    onSort: function(sortWidget, whichDirClicked)
    {
    },
    onClearSort: function(sortWidget)
    {
    }  
});

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.GridTableHeaderRenderer", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableHeaderRenderer,
{
    tableHeaders: null,
	
    // Summary:
    //    Reset any states that the header may have cached. This is usually called when
    //    the table viewer's content has been changed and any old state may not be applicable.
    //
    resetStates: function()
    {
        this.checkAllState = false;
        this.setColumnActionEnablement();
    },
    
    createTableControls: function(pageRangeControlNode, pageNode, footerPageNode, filterControlNode, groupControlNode, viewModeControlNode)
    {   
		if(this.viewer.getIsShowingPagingRangeControl()) {
			this._createPageRangeControl(pageRangeControlNode);
		}
		
        if(this.viewer.getIsShowingPagingControl()) {
            this._createPageControls(pageNode);
            this._createPageControls(footerPageNode);
        }

        this._constructGroupControl(groupControlNode);
        
        var sep = document.createElement("span");
        sep.className = "controls-spacing";

        if(this.viewer.getIsShowingFilterControl()) {
            this._constructFilterControls(filterControlNode);            
        }   
        else {
//            var emptyNode = document.createElement("span");
//            emptyNode.innerHTML = "  ";
//            dojo.style(emptyNode, "width", "100%");
//            filterControlNode.appendChild(emptyNode);
			dojo.removeClass(filterControlNode, "filter-area");
        }

        if(this.viewer.getIsShowingViewModeControl()) {
            this._constructViewModeControl(viewModeControlNode);            
        }   
        else {
			dojo.removeClass(viewModeControlNode, "filter-area");
        }
    },

    createHeaderBorderLine: function(table, borderRow, isTop)
    {
        var line = document.createElement("th");
        var borderCss = (isTop) ? TOP_BORDER_CSS : BOTTOM_BORDER_CSS;

        line.innerHTML = "&nbsp;"
        dojo.addClass(line, borderCss);
        borderRow.appendChild(line);
    },
    
    destroyTableHeader: function()
    {
    	this.inherited(arguments);
        
        if (this.thead) {
            Utility.destroyWidgetsFromNode(this.thead);
        }
        if (this.tableHeaders) {
            for (var i in this.tableHeaders) {
                var th = this.tableHeaders[i].thead;
                var imgs = dojo.query("img", th);
                
                for (var j = 0; j < imgs.length; j++) {
                    imgs[j].src = null;
                }
            }
        }
        this.headerDisconnectAll();
        this.sortColCtrls = null;
        this.columnActions = null;
        this.tableHeaders = null;
        this.thead = null;
    },
    
    createTableHeader: function(table)
    {
        // not show table heading. 
        if(!this.viewer.getIsShowingTableHeading())
            return;
                    
        var provider = this.viewer.getContentProvider();
        var content = this.viewer.getContent();
        var nColumns = provider.getNumColumns(content);
        var thead = table.createTHead();
        var headerTopTr, headerBotTr;
        var headerContentTr
		var headerSortTr = null;                          
        var groupBy = this.viewer.getGroupByColumn();
        var atCol = 0;
        var slackWidth = 0;
        var nVisible = nColumns;
        var reduceWidth = 0;
        var tableHeaderStyle = "table-header-content";
        var showMultiSortHeader = false;
		var doc = document;
        
        // set table header style for CJK pictographic based locales
        if (dojo.locale.indexOf("zh") == 0 ||
            dojo.locale.indexOf("ja") == 0 ||
            dojo.locale.indexOf("ko") == 0) {
            tableHeaderStyle = "table-header-content-cjk";
        }

        // reset the header data
        this.columnActions = [];
        this.tableHeaders = {};
        
        // multi column sorting data.
        this.sortColCtrls = {};
        this.sortColumns = [];
        
        this.sortByColumn = this.viewer.getSortByColumn();
        this.sortDir = this.viewer.getSortDirection();

        // multi-column sorting. Get the array of sort columns.
        if (this.viewer.allowMultiSort) {
            this.sortColumns = this.viewer.getMultiColumSortInfo();
        }        
        for (var i = 0; i < nColumns; i++) {
            if(!this.viewer.isColumnVisible(i) || (groupBy == i)) {
                nVisible--;
            }
        }
        if(nVisible == 0)
            return;

        // we're going to remove the group column,
        // distribute its width amongst all columns
        if (groupBy > -1 && this.viewer.isColumnVisible(groupBy)) {
            slackWidth = Math.floor(this.viewer.getColumnWidth(groupBy) / nVisible);
        }    
        
		// create the header rows, top & bottom border rows
		// and content & sort rows in the middle.
        headerTopTr = table.insertRow(table.rows.length); 
        headerContentTr = table.insertRow(table.rows.length); 
		
		if(this.viewer.allowMultiSort) {
			headerSortTr = table.insertRow(table.rows.length);
		}
        headerBotTr = table.insertRow(table.rows.length); 
        
        // The rows in the table are checkable, insert
        // a column to provide a check box for the rows
        if(this.viewer.getTableRowsCheckable()) {
            this.createHeaderBorderLine(table, headerTopTr, true);
            
            var checkbox_th = doc.createElement("th");
            dojo.addClass(checkbox_th, tableHeaderStyle);
            
            // create a check box to select/deselect all rows in the table.
            if (!this.viewer.getIsSingleSelectionMode() && this.viewer.getIsAllowSelectAll()) {
                dojo.addClass(checkbox_th, "align-center");
                checkbox_th.vAlign = "center";
                checkbox_th.title = MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_selectAll");
                
                var checkState = (this.checkAllState) ? true : false;
                var button = new dijit.form.CheckBox({
                    'checked': checkState
                });
                button.headerRenderer = this;
                button.onChange = function() {
                    this.headerRenderer.checkAllState = this.checked;
                    this.headerRenderer.viewer.performSelectAll(this.checked);
                };
                checkbox_th.appendChild(button.domNode);
            }
            else {
                checkbox_th.width = 20;
            }                
            headerContentTr.appendChild(checkbox_th);
			if(headerSortTr) {
				checkbox_th = doc.createElement("th");
				dojo.addClass(checkbox_th, tableHeaderStyle);
				dojo.addClass(checkbox_th, TOP_BORDER_CSS);
				headerSortTr.appendChild(checkbox_th);
			}
            this.createHeaderBorderLine(table, headerBotTr);
        }
        
        // if there's a group by, insert a filler column for the
        // indentation.
        if(groupBy > -1) {            
            this.createHeaderBorderLine(table, headerTopTr, true);
            var gr_th = doc.createElement("th");
            dojo.addClass(gr_th, tableHeaderStyle);
            headerContentTr.appendChild(gr_th);
            gr_th.width = 8;            

            if(headerSortTr) {
				gr_th = doc.createElement("th");
                dojo.addClass(gr_th, tableHeaderStyle);
                dojo.addClass(gr_th, TOP_BORDER_CSS);
                headerSortTr.appendChild(gr_th);
            }
            this.createHeaderBorderLine(table, headerBotTr);			
        }
        
        for(var i = 0; i < nColumns; i++) {
            // skip displaying the group by column and non-visible columns.
            if(groupBy == i || !this.viewer.isColumnVisible(i)) {
                continue;
            }
            this.createHeaderBorderLine(table, headerTopTr, true);
            
            var title = provider.getColumnHeading(content, i);
            var th = doc.createElement("th");
             
            dojo.addClass(th, tableHeaderStyle);
            th.width = (this.viewer.getColumnWidth(i) + slackWidth - reduceWidth) + "%";				
	        th.vAlign = "center";
            var align = provider.getColumnHeadingAlignment(i);
           
            if(align == TableConstants.ALIGN_CENTER) {
               dojo.addClass(th, "align-center");
            } 
            else if(align == TableConstants.ALIGN_RIGHT) {
               dojo.addClass(th, "align-right");   
            } 
            else {
                dojo.addClass(th, "align-left")
            }
				            
            if (provider.getIsColumnSortable(content, i)) {
                var anchor = doc.createElement("a");
                var titleText = (title && title.length > 0) ? title : "\u00a0";
                var colIsSort = this.columnHasSort(i);
                
                anchor.appendChild(doc.createTextNode(titleText));
                anchor.colIndex = i;
                anchor.name = "anchor";
                anchor.className = "table-link";

                th.appendChild(anchor);
                this.headerConnect(anchor, "onclick", "_handleTableHeadSortBy");
                anchor.title = NoSort_Text;            

                // Create the sort indicator, but only when we're not showing
				// multi-column sort header
				if (!headerSortTr) {
					var img = this._locateHeaderSortDirImageNode(th, i, true);
					var isAscend = (this.sortDir == TableConstants.SORT_DIR_ASCEND);
					img.src = (isAscend) ? ImageCache.SORT_ASC : ImageCache.SORT_DSC;
					dojo.style(img, "display", ((colIsSort) ? "inline" : "none"));					
				}
                if (colIsSort) {
                    showMultiSortHeader = true;
                    this.setColumnTitleSortTooltip(i, anchor);
                }
            } 
            else if (title != null && dojo.trim(title).length > 0){
                th.appendChild(doc.createTextNode(title));
            }
            else {
                var spacer = doc.createElement('span');
                spacer.style.width = '8px';
                th.appendChild(spacer);
            }
                        
            // if the column header has a image, create it.
            var imgRef = provider.getColumnHeadingImage(content, i);
            if(imgRef) {
                var src = (dojo.isObject(imgRef) && imgRef.src) ? imgRef.src : imgRef;
                var isAction = (dojo.isObject(imgRef) && imgRef.actionId) ? true : false;
                
                // create a margin node if we already have
                // something created before the image.
                if(th.childNodes && th.childNodes.length > 0) {
                    var spacer = doc.createElement("span");
                    spacer.className = "table-head-spacer";
                    th.appendChild(spacer);                    
                }
                
                var img = doc.createElement("img");
                img.className = "table-head-image";
                img.src = src;
                th.appendChild(img);
                
                if (isAction) {
                    this.columnActions.push(img);

                    img.colIndex = i;
                    img.name = imgRef.actionId;
                    if(imgRef.tooltip) {
                        img.title = imgRef.tooltip;
                    }
                    dojo.addClass(img, "table-head-image-action");
                    this.headerConnect(img, "onclick", "_handleColumnActionClicked");
                }
            }

            headerContentTr.appendChild(th);
			
            // We're in multi-columns sort mode. Create the sort indicator
            // header for this sortable column.
            if(headerSortTr) {
                var sth = doc.createElement("th");
                dojo.addClass(sth, tableHeaderStyle);
                dojo.addClass(sth, TOP_BORDER_CSS);
                
                // create the sort control for this column in the
                // sort header.
				if (provider.getIsColumnSortable(content, i)) {
					this._createSortControls(sth, i);
				}
                headerSortTr.appendChild(sth);
            }   
            this.createHeaderBorderLine(table, headerBotTr);
            
            // store the column header in our associated array indexed
            // by the column position string key.
            this.tableHeaders[this._getTableHeadKey(i)] = {'thead': th};
        }
        

        // create a drop down column to open/collapse the sort header
		//
        if(headerSortTr) {
            this.createHeaderBorderLine(table, headerTopTr, true);
			this._createSortHeaderExpandControls(table, headerContentTr, headerSortTr);
        
            if(!showMultiSortHeader) {
                dojo.style(headerSortTr, "display", "none");            
            }
			else {
                dojo.style(this.openMultiSortColAction, "display", "none");				
			}
            this.createHeaderBorderLine(table, headerBotTr);
        }
		
        thead.appendChild(headerTopTr);
        thead.appendChild(headerContentTr);    
        if (headerSortTr) {
            thead.appendChild(headerSortTr);    
        }
        thead.appendChild(headerBotTr);    
        this.thead = thead;
          
        table.appendChild(thead);    
        this.viewer.contentRowStart = table.rows.length;
        
        // initialize the column's action states.
        this.setColumnActionEnablement();    
        
        if (showMultiSortHeader) {
            this._initSortIndicators(groupBy);
        }
    },

    _createSortHeaderExpandControls: function(table, headerContentTr, headerSortTr)
    {            
        var doc = document;
        var tableHeaderStyle = "table-header-content";
        var th = doc.createElement("th");
        var dropDown = doc.createElement("span");
        this.openMultiSortColAction = dropDown;
        
        dojo.addClass(dropDown, "dropdown-arrow-action");
        dojo.addClass(dropDown, "outline-down-icon-image");
        dropDown.title = OpenSortCtrl_Text;
        this.headerConnect(dropDown, "onclick", function(){
            this._toggleMultiSortHeader(true);
        });
        th.appendChild(dropDown);
        th.width = 16;
        th.align = "center";
        dojo.addClass(th, tableHeaderStyle);
        headerContentTr.appendChild(th);

        th = doc.createElement("th");
        dropDown = doc.createElement("span");
        dojo.addClass(dropDown, "dropdown-arrow-action");
        dojo.addClass(dropDown, "outline-up-icon-image");
        dropDown.title = CloseSortCtrl_Text;
        this.headerConnect(dropDown, "onclick", function(){
            this._toggleMultiSortHeader(false);
        });
        th.appendChild(dropDown);
        th.width = 16;
        th.align = "center";
        dojo.addClass(th, tableHeaderStyle);
        dojo.addClass(th, TOP_BORDER_CSS);
        
        headerSortTr.appendChild(th);
        this.multiSortHeader = headerSortTr;
    },

    _createSortControls: function(th, pos)
	{
        var sortCtrl = new GridTableHeaderRenderer.SortActionGroup({
            columnPos: pos,
            onSort: dojo.hitch(this, "_handleSortIndicatorClick"),
            onClearSort: dojo.hitch(this, "_handleClearSortCol")
        });
        this.sortColCtrls['col' + pos] = sortCtrl;
		th.appendChild(sortCtrl.domNode);
	},
	
    setColumnActionEnablement: function()
    {
        if (!this.columnActions) {
            return;
        }
        var disable = this.viewer.getContentItemCount() == 0 || this.viewer.getCheckedRows().length == 0;
        for(var i = 0; i < this.columnActions.length; i++) {
            var action = this.columnActions[i];
            action.disabled = disable;
            if(disable)
                dojo.addClass(action, "table-head-image-action-disable");
            else
                dojo.removeClass(action, "table-head-image-action-disable");
        }          
    },
    
    getColumnHeaderWidthInPixel: function(columnIndex)
    {
        var header = this.tableHeaders[this._getTableHeadKey(columnIndex)];
        if(!header.width) {
			var w = dojo.marginBox(header.thead).w;
			// only cache the width if we actually has a width.
            // reduce the column width to ensure that the column's 
            // desired width would not be excceeded
			if(w > 0)
                header['width'] = (w <= WidthSlack) ? w : (w - WidthSlack);
	        else 
			    return w;
        }
        return header.width;
    },
    
    getColumnHeaderAt: function(columnIndex)
    {
        return this.tableHeaders[this._getTableHeadKey(columnIndex)].thead;
    },
    
    _handleColumnActionClicked: function(event)
    {
        event = event || window.event;
        var column = event.currentTarget.colIndex;
        var actionId = event.currentTarget.name;
        
        if(event.currentTarget.disabled)
            return;
            
        var listener = this.viewer.getTableListener();
        if(listener) {
            listener.tableHeaderColumnActionInvoked(this.viewer, column, actionId);
        }
    },
    
    _getNextSortDirByColumn: function(colPos)
    {
        var curDir = null;

        if (this.viewer.allowMultiSort) {
            for(var i = 0; i < this.sortColumns.length; i++) {
                if(colPos == this.sortColumns[i].col) {
                    curDir = this.sortColumns[i].dir;
                }
            }
        }
        else if(colPos == this.sortByColumn) {
            curDir = this.sortDir;
        }
        return (curDir == TableConstants.SORT_DIR_ASCEND) ? TableConstants.SORT_DIR_DESCEND : TableConstants.SORT_DIR_ASCEND;
    },
    
    // Summary:
    //    Handles a column's sort control being cleared for sort. Update
    //    the sort indicator & the header title's sort tooltip and
    //    perform the sort.
    _handleClearSortCol: function(sortCtrl)
    {
        var colPos = sortCtrl.columnPos;
        
        // clear this column's sort indication and the main header's
        // title's sort tooltip.  
        sortCtrl.setSortAttributes(null, null);
        
        // remove this column from sort participations.
        for(var i = 0; i < this.sortColumns.length; i++) {
            if(colPos == this.sortColumns[i].col) {
                break;
            }
        }
        this.sortColumns.splice(i, 1);
        
        var th, anchor;
        
        // move column's sort position back up one.
        for(var j = i; j < this.sortColumns.length; j++) {
            this._updateSortIndicators(this.sortColumns[j].col, this.sortColumns[j].dir, j+1);

            // update the column title's sort tooltip
            th = this.getColumnHeaderAt(this.sortColumns[j].col);
            anchor = this._getHeaderTitleLink(th);
            if (anchor)
               this.setColumnTitleSortTooltip(this.sortColumns[j].col, anchor);
        }
        // clear the sort tooltip in the column being cleared.
        th = this.getColumnHeaderAt(colPos);
        anchor = this._getHeaderTitleLink(th);
        if (anchor)
            this.setColumnTitleSortTooltip(colPos, anchor);
        
        this.viewer.performMultiColumnSort(this.sortColumns);
    },
    
    // Summary:
    //    Handles a column's sort control being clicked (change sort dir). Update
    //    the sort indicator & the header title's sort tooltip and
    //    perform the sort.
    _handleSortIndicatorClick: function(sortCtrl, dirClicked)
    {
        this._updateSortColumn(sortCtrl.columnPos, dirClicked);
        
        // update the column title's sort message
        var th = this.getColumnHeaderAt(sortCtrl.columnPos);
        var anchor = this._getHeaderTitleLink(th);
        if (anchor)
           this.setColumnTitleSortTooltip(sortCtrl.columnPos, anchor);

        this.viewer.performMultiColumnSort(this.sortColumns);           
    },
    
    _handleTableHeadSortBy: function(event)
    {
        event = event || window.event;
        
        var lastSortCol = this.sortByColumn;
        var sortDir = this.sortDir = this._getNextSortDirByColumn(event.currentTarget.colIndex);
        var sortColIndex = this.sortByColumn = event.currentTarget.colIndex;
        var th = this.getColumnHeaderAt(this.sortByColumn);
        var isAscend = (sortDir == TableConstants.SORT_DIR_ASCEND);
		
        // don't show the simple sort icon if we're in multi-column sort mode.
        if (!this.multiSortHeader) {
            var img = this._locateHeaderSortDirImageNode(th, this.sortByColumn, true);
            
            img.src = isAscend ? ImageCache.SORT_ASC : ImageCache.SORT_DSC;
            dojo.style(img, "display", "inline");
        }
        
        // don't need to clear the last sort column image if we're in multi-sort
        if(!this.multiSortHeader && lastSortCol > -1 && lastSortCol != this.sortByColumn) {
            th = this.getColumnHeaderAt(lastSortCol);
            img = this._locateHeaderSortDirImageNode(th, lastSortCol);
            if(img) {
                img.src = ImageCache.SORT;
                dojo.style(img, "display", "none");
            }
            anchor = this._getHeaderTitleLink(th);
            if (anchor)
                anchor.title = NoSort_Text;            
        }
        
        if (this.multiSortHeader) {
            this._updateSortColumn(sortColIndex, sortDir);
            this.viewer.performMultiColumnSort(this.sortColumns);
        }
        else {
            this.viewer.performSortBy(this.sortByColumn, this.sortDir);
        }
        var anchor = this._getHeaderTitleLink(th);
        if (anchor) {
            this.setColumnTitleSortTooltip(sortColIndex, anchor);
        }
    },
    
    _getHeaderTitleLink: function(th)
    {
        var anchor = null;
        if(th.childNodes.length) {
            for(var i = 0; i < th.childNodes.length; i++) {
                if(th.childNodes[i].name == "anchor") {
                    anchor = th.childNodes[i];
                    break;
                }
            }
        }
        return anchor;        
    },
    
    _locateHeaderSortDirImageNode: function(th, col, create)
    {
        var img = null;
        var img_name = "head-sort-img" + col;
        var doc = document;

        if(th.childNodes.length) {
            for(var i = 0; i < th.childNodes.length; i++) {
                if(th.childNodes[i].name == img_name) {
                    img = th.childNodes[i];
                    break;
                }
            }
        }
        if(create && img == null) {
            // create a margin node if we already have
            // something created before the sort indicator.
            if(th.childNodes && th.childNodes.length > 0) {
                var spacer = doc.createElement("span");
                spacer.className = "table-head-spacer";
                th.appendChild(spacer);                    
            }
            img = doc.createElement("img");
            img.className = "table-head-image";
            img.name = img_name;
            img.src = ImageCache.SORT;
            th.appendChild(img);
        }
        return img;
    },
    
    columnHasSort: function(colPos)
    {
        if (this.viewer.allowMultiSort) {
            for(var i = 0; i < this.sortColumns.length; i++) {
                if(colPos == this.sortColumns[i].col) {
                    return true;
                }
            }
        }
        else {
            return colPos == this.sortByColumn;
        }
        return false;
    },
    
    setColumnTitleSortTooltip: function(colPos, anchor)
    {
        var sortDir = null;
        var sortPos = null;
        
        if (this.viewer.allowMultiSort) {
            for(var i = 0; i < this.sortColumns.length; i++) {
                if(colPos == this.sortColumns[i].col) {
                    sortDir = this.sortColumns[i].dir;
                    sortPos = i+1;
                    break;
                }
            }
        }
        else if(colPos == this.sortByColumn) {
            sortDir = this.sortDir;
        }
        if (sortDir) {
            var tooltip = (sortDir == TableConstants.SORT_DIR_ASCEND) ? AscendSort_Text : DescendSort_Text
            anchor.title = tooltip + ((sortPos != null) ? (' [' + sortPos + ']') : "");
        }
        else {
            anchor.title = NoSort_Text;             
        }
    },
    
    _initSortIndicators: function(groupBy)
    {
        for(var i = 0; i < this.sortColumns.length; i++) {
            var sortInfo = this.sortColumns[i];
            
            // skip the group by column.
            if(groupBy > -1 && groupBy == sortInfo.col)
                continue;
                
            this._updateSortIndicators(sortInfo.col, sortInfo.dir, i+1, /*openHeader*/false);           
        }
    },
    
    _updateSortColumn: function(colPos, sortDir) 
    {
        var sortCol = null;
        for(var i = 0; i < this.sortColumns.length; i++) {
            if(colPos == this.sortColumns[i].col) {
                sortCol = this.sortColumns[i];
                break;
            }
        }
        if(!sortCol) {
            sortCol = {
                col: colPos,
                dir: sortDir
            };
            this.sortColumns.push(sortCol);
        }
        sortCol.dir = sortDir;
        this._updateSortIndicators(colPos, sortDir, i+1);
    },
    
    _updateSortIndicators: function(pos, sortDir, nSort, openHeader)
    {
        if((typeof openHeader == "undefined" || openHeader) && this.multiSortHeader.style.display == "none")
            this._toggleMultiSortHeader(true);
        
        var ctrl = this.sortColCtrls['col' + pos];
        if(ctrl)
            ctrl.setSortAttributes(sortDir, nSort);
    },
    
	_toggleMultiSortHeader: function(open)
	{
        dojo.fx[open ? "wipeIn" : "wipeOut"]({
            node: this.multiSortHeader,
            duration: 400,
			onEnd: dojo.hitch(this, function(){
                // hide/show the dropdiown icon in the main header. The 'close' action
                // is in the sort header
                dojo.style(this.openMultiSortColAction, "display", open ? "none" : "inline-block");
			})
        }).play();
	},
    
    _getTableHeadKey: function(atCol)  
    {
        return "col-" + atCol;
    },

    // Summary:
    //    Internal method to connect header node's event to an internal function. 
    //
    headerConnect: function(node, event, func)
    {
        if(!this.headConnect) {
            this.headConnect = [];
        }
        var cnt = dojo.connect(node, event, this, func);  
        this.headConnect.push(cnt);  
    },
    
    headerDisconnectAll: function()
    {
        if(!this.headConnect) return;
        
        var len = this.headConnect.length;
        for(var i = 0; i < len; i++) {
            dojo.disconnect(this.headConnect[i]);
        }
        delete this.headConnect;    
    }
    
});


})();
