/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ImageCellRenderer");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");

(function() {

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ImageCellRenderer", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer,
{    
    // Summary:
    //    Renders the image cell content. The following cellData can be expected. 
    //    
    renderCell: function(viewer, cellData, extendedDisplayInfo, /*table.CellModifier*/cellModifier, rowObject, columnIndex)
    {
        var img = document.createElement("img");
        var tooltip = cellData;
        
        if(extendedDisplayInfo && dojo.isObject(extendedDisplayInfo)) {
            var imgSrc = (typeof extendedDisplayInfo['mouseOutImage'] != "undefined") ? extendedDisplayInfo['mouseOutImage'] : extendedDisplayInfo['image'];
            img.alt = cellData;
            img.src = imgSrc;
            img.extendData = extendedDisplayInfo;
            
            if(extendedDisplayInfo['tooltip'])
                tooltip = extendedDisplayInfo['tooltip'];
        }
        else {
            img.src = cellData;
        }
        this.connect(img, "onmouseover", "overImage");
        this.connect(img, "onmouseout", "outOfImage");
        
        img.tooltip = this._attachTooltipToCell(img, tooltip);        
        this.cellNodes.push(img);
        
        if (extendedDisplayInfo &&  typeof extendedDisplayInfo['href'] != "undefined") {
            var anchor = document.createElement("a");
            anchor.href = extendedDisplayInfo.href;
            anchor.appendChild(img);
            return anchor;
        }		
        return img;
    },
    
    overImage: function(e)
    {
        if(e.target.extendData && e.target.extendData['mouseOverImage']) {
            e.target.src = e.target.extendData['mouseOverImage'];
        }
    },
    
    outOfImage: function(e)
    {
        if(e.target.extendData && e.target.extendData['mouseOverImage']) {
            var data = e.target.extendData;
            e.target.src = (typeof data['mouseOutImage'] != "undefined") ? data['mouseOutImage'] : data['image'];
        }
    },
    
    disconnectAll: function()
    {
        this.inherited(arguments);
        //console.debug("================ImageCellRenderer disconnectAll");
        var cellNodes = this.cellNodes;
        var len = cellNodes.length;
        var d = dojo;
        for(var i = 0; i < len; i++) {
            var node = cellNodes[i];
            node.src = null;
            node.cellModifier = null;
            node.rowObject = null;
            node.columnIndex = null;
            node.extendData = null;
            var tooltip = node.tooltip;
            if(tooltip && tooltip.destroy) {
                tooltip.destroy();
            }
            node.tooltip = null;
            node.onclick = null;
            node.onmouseover = null;
            node.onmouseout = null;
            node.onhover = null;
            node.onunhover = null;
            d._destroyElement(node);
        }
        this.cellNodes = [];
    }
});

})();
