/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.LinkCellRenderer");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");

(function() {

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.LinkCellRenderer", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer,
{    
    // Summary:
    //    Renders the cell content. 
    //    
    renderCell: function(viewer, cellData, extendedDisplayInfo, cellModifier, rowObject, columnIndex, cellWidthHint)
    {
    	var doc = document;
		var linkExtHTML = null;
        
		//Fix wi 7584, add link type info
		if (extendedDisplayInfo) {			
            if (typeof extendedDisplayInfo['linkType'] != "undefined") {
				linkExtHTML = "<span style='left:-3000px; position:absolute; display:inline;'>" + extendedDisplayInfo['linkType'] + "</span>";
			} 
		}
		
        // empty link. Cannot be clicked. Just create a text node.
        if(cellData == null || dojo.isString(cellData) && cellData.length == 0) {
            return doc.createTextNode(" ");
        }
        
        var node = doc.createElement("a");
        var divNode = doc.createElement('div');
        var text = cellData;

        if (viewer.isCompactMode && typeof cellWidthHint != "undefined" && dojo.isString(cellData)) {
            text = this._handleCompactCellText(viewer, text, extendedDisplayInfo, divNode, cellWidthHint);
        }
        else if (extendedDisplayInfo && dojo.isObject(extendedDisplayInfo)) {
            node.tooltip = this._attachTooltipToCell(node, extendedDisplayInfo['tooltip']);
        }    
        if(linkExtHTML)
            node.innerHTML = linkExtHTML; 

        var textNode = doc.createTextNode(text);
        divNode.appendChild(textNode);
        node.appendChild(divNode);

        if (extendedDisplayInfo) {			
            if (typeof extendedDisplayInfo['href'] != "undefined") {
                node.href = extendedDisplayInfo['href'];
			} 
            else if (dojo.isFunction(extendedDisplayInfo['onclick'])) {
                node.setAttribute("href", "javascript:{}");
                
                // make sure the link onclick event isn't propagated
                // up. Jazz has a onclick handler that checks for link 
                // clicked to put up a leave page confirm. 
                node.extendedDisplayInfo = extendedDisplayInfo;
                this.connect(node, "onclick", function(e) {
                    // local event don't propagate the event up.
                    dojo.stopEvent(e);
        
                    var node = e.target;
                    if (node) {
                        var extendedInfo = node.extendedDisplayInfo;
                        if(!extendedInfo) {
                            node = node.parentNode;
                            extendedInfo = node.extendedDisplayInfo;
                        }
                        
                        var func = extendedInfo['onclick'];
                        func();
                    }
                });
			}
		}
        this.cellNodes.push(node);
        return node;
    },
    
    disconnectAll: function()
    {
        this.inherited(arguments);
        //console.debug("================LinkCellRenderer disconnectAll");
        var cellNodes = this.cellNodes;
        var len = cellNodes.length;
        var d = dojo;
        for(var i = 0; i < len; i++) {
            var node = this.cellNodes[i];
            node.cellModifier = null;
            node.rowObject = null;
            node.columnIndex = null;
            var tooltip = node.tooltip;
            if(tooltip && tooltip.destroy) {
                tooltip.destroy();
            }
            node.tooltip = null;
            node.onclick = null;
            node.onmouseover = null;
            node.onmouseout = null;
            node.onhover = null;
            node.onunhover = null;
            node.extendedDisplayInfo = null;
            d._destroyElement(node);
        }        
        this.cellNodes = [];
    }
});

})();
