/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableFilter");

dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility")
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.Util");

(function() {

var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableFilter", null,
{
    // Summary:
    //   Returns true to accept the row to be displayed in the table given
    //   the filter.
    accepts: function(viewer, rowObject, filter) 
    {
        var provider = viewer.getContentProvider();
        var content = viewer.getContent();
        var nColumns = provider.getNumColumns(content);
        var filterBy = filter.toLowerCase();
        
        for(var i = 0; i < nColumns; i++) {
            // the column is not visible. Skip it.
            if(!viewer.isColumnVisible(i))
                continue;
                
            var type = provider.getCellDisplayType(rowObject, i);
            var data = provider.getCellDisplayData(rowObject, i);
            
            if(data == null)
                continue;
            
            // normalize the column data to string, if possible    
            if(typeof data == "number")
                data = data + "";
                
            // this is a time stamp type. Convert the timestamp value to a Date
            // object first.
            //
            if(type == TableConstants.RENDER_TYPE_TIMESTAMP)
                data = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.Util.convertTimeStampToDate(data);

            if(data instanceof Date)
                data = com.ibm.asq.common.web.ui.internal.util.Utility.formatDate(data);

            // can't convert to string, skip it.
            if(!dojo.isString(data))
                continue;
                
            if(data.toLowerCase().indexOf(filterBy) > -1)
                return true;
        }
        return false;
    }

});

})();
