/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.wizard.WizardProjectArea");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.repository.web.ui.wizard.WizardStep");
dojo.require("com.ibm.team.repository.web.ui.wizard.WizardStepHeader");

dojo.require("com.ibm.team.repository.web.client.internal.licensing");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function() {

var WizardStep = com.ibm.team.repository.web.ui.wizard.WizardStep;
var WizardStepHeader = com.ibm.team.repository.web.ui.wizard.WizardStepHeader;

var LicenseAdminClient 		= com.ibm.team.repository.web.client.internal.licensing;
var ServiceRequest 			= com.ibm.team.repository.web.transport.ServiceRequest;
var TeamServerClient 		= com.ibm.team.repository.web.transport.TeamServerClient;
var ServiceResponseHandler  = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;

var Constants 				= com.ibm.rqm.planning.web.client.internal.Constants;

var REQUIRED_LICENSE = "com.ibm.rqm.tester";
var QM_REPO_INIT_SERVICE_URI = "com.ibm.rqm.process.deployment.rest.IQMRepoInitializerRestService";


dojo.declare("com.ibm.rqm.planning.web.ui.internal.wizard.WizardProjectArea",  [WizardStep, dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/wizard/templates/WizardProjectArea.html"),

	labelTitle: Constants.getMessage("labelSetupCreateProject"),
	labelCreateMessage: Constants.getMessage("msgSetupCreateProject"),
	
	constructor: function() {
		this._projectArea = null;
		this._hasRequiredLicense = false;
		this._currentUserId = dojo.getObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_USERID");
		this.stepTitle = this.labelTitle;
		this._description = Constants.getMessage("msgSetupProjectDescription");	
		this._descriptionNew = Constants.getMessage("msgSetupProjectProjectNew");
		this._descriptionExist = Constants.getMessage("msgSetupProjectProjectExist");
	},

	postCreate: function() {
		this.header =  new WizardStepHeader({stepTitle: this.stepTitle});
		this.stepHeaderArea.appendChild(this.header.domNode);
		this._setDescription();
		this._createCheckbox.checked = true;
	},
	
	_setDescription: function() {
		this._descriptionNode.innerHTML = this._description + " " + (this._projectArea == null ? this._descriptionNew : this._descriptionExist);
	},
	
	//@Override
	_refresh: function () {
		this._fetchDefaultProjectArea();
		this._fetchUserLicenseInfo();
	},
	
	_fetchDefaultProjectArea: function() {
		var responseHandler = new ServiceResponseHandler(this, "_fetchProjectSuccess", "_handleServiceFailure");
		var serviceRequest= new ServiceRequest(QM_REPO_INIT_SERVICE_URI, "getDefaultProjectArea", {});
		TeamServerClient.invokeService(serviceRequest, responseHandler);		
	},
	
	_fetchProjectSuccess: function(projectArea) {
		if (projectArea != null) {
			this._projectArea = projectArea;
			this._setDescription();
			this.triggerButtonStateUpdate();
		}
	},
	
	_fetchUserLicenseInfo: function() {
		var responseHandler = new ServiceResponseHandler(this, "_fetchUserLicenseInfoSuccess", "_handleServiceFailure");	
		LicenseAdminClient.getAssignedLicenses(responseHandler, {userId: this._currentUserId});
	},
	
	_fetchUserLicenseInfoSuccess: function(licenses) {
		this._licenses = licenses || [];
		for (var i = 0; i < this._licenses.length; i++) {
			if (this._licenses[i].id == REQUIRED_LICENSE) {
				this._hasRequiredLicense = true;
			}
		}
		if (!this._hasRequiredLicense && this._projectArea == null) {
			LicenseAdminClient.getLicenseInfo(new ServiceResponseHandler(this, "_fetchLicenseInfoSuccess", "_handleServiceFailure"));
		}
	},
	
	_fetchLicenseInfoSuccess: function(licenseInfo) {
		var types = licenseInfo && licenseInfo.types ? licenseInfo.types : []; 
		var reqLicenseName = REQUIRED_LICENSE;
		for (var i = 0; i < types.length; i++) {
			if (types[i].id == REQUIRED_LICENSE) {
				var calInfo = LicenseAdminClient.getClientAccessLicenseInfo(licenseInfo, types[i]);
				reqLicenseName = calInfo.name;
			}
		}			
		
		this.header.showStatus({
			type: WizardStepHeader.STATUS_TYPES.WARNING,
			summary: Constants.getMessage("msgSetupCreateProjectLicense", this._currentUserId, reqLicenseName)
		});		
	},
	
	_createProjectArea: function() {
		this.header.showStatus({
			type: "PROGRESS", //$NON-NLS$
			summary: Constants.getMessage("msgSetupCreatingProject")
		});
		var responseHandler = new ServiceResponseHandler(this, "_createSuccess", "_handleServiceFailure");
		var serviceRequest= new ServiceRequest(QM_REPO_INIT_SERVICE_URI, "postCreateDefaultProjectArea", {});
		TeamServerClient.invokeService(serviceRequest, responseHandler);
	},
	
	_createSuccess: function(projectArea) {
		this._projectArea = projectArea;
		this._stepCompleted();
	},

	_createFailure: function(e) {
		this.header.showStatus({
			type: WizardStepHeader.STATUS_TYPES.ERROR,
			summary: Constants.getMessage("msgSetupCreateProjectError", e.message || "Unknown Error.")
		});
		this._creatingProjectArea = false;
		this.triggerButtonStateUpdate();
	},
	
	_handleServiceFailure: function(error) {
		this.header.displayServiceCallbackErrorMessage(error);
	},	
	
	_stepCompleted: function() {
		//show completion message
		this.header.showStatus({
			type: WizardStepHeader.STATUS_TYPES.OK,
			summary: Constants.getMessage("msgSetupCreatedProject")
		});	
		this._creatingProjectArea = false;
		this.triggerButtonStateUpdate();		
		this.wizard.processNext();
	},
	
	_onCheckboxClick: function() {
		this.triggerButtonStateUpdate();
	},
		
	// SUPER WizardStep functions
	focusStep: function () {
		this._refresh();	
	}, 
	
	canFlipToNextPage: function() {
		return (this._projectArea != null || this._createCheckbox.checked) && !this._creatingProjectArea;
	},
	
	canFlipToPreviousPage: function() {
		return !this._creatingProjectArea;
	},
	
	onNext: function() {
		if (this._projectArea == null) {
			this._creatingProjectArea = true;
			this.triggerButtonStateUpdate();
			this._createProjectArea();
		} else {
			this._stepCompleted();
		}
	}
	
});

})();