/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rqm.defects.web.ui.workitems.WorkItemSelectionDialog");

dojo.require("dijit._Widget");
dojo.require("com.ibm.team.repository.web.foundation.NullProgressMonitor");

dojo.require("dijit._Templated");

dojo.require("jazz.ui.Dialog");

dojo.require("com.ibm.team.workitem.web.client.util");

dojo.require("com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil");

(function() {

var WorkItemUtil				= com.ibm.team.workitem.web.client.util;
var NullProgressMonitor			= com.ibm.team.repository.web.foundation.NullProgressMonitor;
var DefectsUIUtil 				= com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil;

var QUERY_DELAY					= 750;
var MAX_RESULTS					= 50;

dojo.declare("com.ibm.rqm.defects.web.ui.workitems.WorkItemSelectionDialog",
			 [dijit._Widget, dijit._Templated], {

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------	

	_dialogRoot: null,
	_msgArea: null,
	_queryInput: null,
	_resultTitle: null,
	_queryResults: null,
	_okCmd: null,
	_cancelCmd: null,

	_labelEnterSearchString: DefectsUIUtil.getMessage("_labelEnterSearchString"),
	_labelButtonOK: DefectsUIUtil.getMessage("_labelButtonOK"),
	_labelButtonCancel: DefectsUIUtil.getMessage("_labelButtonCancel"),
	
	_backingWorkItems: null, 
	_exclusionMap: null,
	_currentWorkItemId: null,
	_startSearchJobHandle: null,
	_currentOption: null,

	_workItemsProvider: null,
	_queryFunction: null,
	
	constructor: function() {
		this._exclusionMap= {};
        this._backingWorkItems = {};
	},

	// ---- api ------------------------------------------------------------------------------------------------------------	

	setFocus: function() {
		this._queryInput.focus();
	},
	
	setExclusionList: function(exclusionList) {
        this._exclusionMap = {};
        for (var i = 0; i < exclusionList.length; i++) {
            var id = exclusionList[i];
            this._exclusionMap[id] = true;
        }
	},

	// ---- events ---------------------------------------------------------------------------------------------------------	

	onOk: function(selection) {
	},

	onCancel: function() {
	},

	// ---- widget implementation ------------------------------------------------------------------------------------------

	templatePath: dojo.moduleUrl("com.ibm.rqm.defects.web", "ui/workitems/templates/WorkItemSelectionDialog.html"),

	postCreate: function() {
		this._dialog = new jazz.ui.Dialog({
			width: "475px",
			onClose: this._onClose,
			contentNode: this._dialogRoot,
			primaryTitle: "Select Work Item"
		});

		this.connect(this._okCmd, "onclick", "_handleOk");
		this.connect(this._queryResults, "ondblclick",  "_handleOk");
		this.connect(this._cancelCmd, "onclick",  "_handleCancel");
		this.connect(this._queryInput, "onkeyup", "_handleQueryInputKey");
		this.connect(this._queryResults, "onkeyup",  "_handleQueryResultsKey");
		this.connect(this._queryResults, "onchange", "_handleSelectionChange");
	},

	// ---- implementation -------------------------------------------------------------------------------------------------

	_handleOk: function() {
		var retArr= [];
		var options= this._queryResults.options;
		for (var i= 0; i < options.length; i++) {
			if (options[i].selected) {
				retArr.push(this._backingWorkItems[options[i].value]);
			}
		}
		this._dialog.close();
		this.onOk(retArr);
	},
	
	_handleCancel: function() {
		this._dialog.close();
		this.onCancel();
	},

	_handleSelectionChange: function() {
		// disable the 'OK' button if nothing is selected
		this._okCmd.disabled= this._queryResults.selectedIndex === -1;
		this._deselectPreviouslySelected();
	},

	_deselectPreviouslySelected: function() {
		var options= this._queryResults.options;
		var prevOption= this._currentOption ? this._currentOption : null;
		var newOptionSet= false;

        for (var i = 0; i < options.length; i++) {
            if (options[i].selected) {
                if (options[i] == prevOption || newOptionSet) {
                    options[i].selected = false;
                } else {
                    this._currentOption = options[i];
                    newOptionSet = true;
                }
			}
        }
	},

	_handleQueryResultsKey: function(e) {
        if (e.keyCode === dojo.keys.ENTER || e.keyCode === dojo.keys.SPACE) {
            this._handleOk();
        }
	},

	_handleQueryInputKey: function(e) {
        if (e.keyCode === dojo.keys.DOWN_ARROW) {
            this._queryResults.focus();
            return;
        }
        
        if (this._startSearchJobHandle) {
            clearTimeout(this._startSearchJobHandle);
        }
        
        this._startSearchJobHandle = setTimeout(dojo.hitch(this, this._doSearch), QUERY_DELAY);
	},
	
	_doSearch: function() {
		var queryText= this._queryInput.value;
		if (dojo.trim(queryText) === "") {
			// if there's no input, do nothing
			this._queryResults.length= 0;
			this._clearStatusMessage();
			return;
		}

		var query= { } 
		if (WorkItemUtil.isWorkItemId(queryText)) {
			query["id"]= WorkItemUtil.getWorkItemId(queryText);
		} else {
			query["fullText"]= queryText;
		}

		this._workItemsProvider[this._queryFunction](query, new NullProgressMonitor(), dojo.hitch(this, this._processQueryResults));		
		this._setStatusMessage(DefectsUIUtil.getMessage("labelRunningSearch"));
	},

	_processQueryResults: function(status, result) {
		if (status.isOK()) {
			dojox.data.dom.replaceChildren(this._resultTitle, document.createTextNode(DefectsUIUtil.getMessage("labelMatchingWorkItems")));
			
	        this._queryResults.length = 0;
	        this._backingWorkItems = {};
			
			dojo.forEach(result, dojo.hitch(this, function(workItem) {
				var option = new Option(this._restoreTags(workItem.label), workItem.id);
				this._queryResults.options[this._queryResults.options.length]= option;
	            this._backingWorkItems[workItem.id] = workItem;
	            if (this._exclusionMap[workItem.id]) {
	                this._disableOption(option, workItem.id);
	            }
			}))
			
        	this._setStatusMessage(DefectsUIUtil.getMessage("labelWorkitemResult", result.length));
		} else {
			this._setStatusMessage(DefectsUIUtil.getMessage("labelErrorRunningQuery"), true);
		}
	},

	_disableOption: function(option, id){
		option.style.color = "#CCCCCC";
		option.selected = false;
		option.title = DefectsUIUtil.getMessage("labelWorkitemLinked");
	},
	
	_queryError: function(errorObj) {
	},
	
	_clearStatusMessage: function() {
		this._msgArea.innerHTML= ""; /* innerHTML OK */
	},
	
	_setStatusMessage: function(msg, isError) {
		dojox.data.dom.replaceChildren(this._msgArea, document.createTextNode(msg));
		this._msgArea.className= isError ? "ErrorStatus" : "NormalStatus";
	},
	
	_restoreTags: function(value) {
		var rawText= value;
		rawText= rawText.replace(/\&gt;/gi,">");
		rawText= rawText.replace(/\&lt;/gi,"<");
		rawText= rawText.replace(/\&amp;/gi,"&");
		rawText= rawText.replace(/\&tilde;/gi,"~");
		rawText= rawText.replace(/\&quot;/gi,"\"");
		rawText= rawText.replace(/\&circ;/gi,"^");
		rawText= rawText.replace(/\&nbsp;/gi," ");
		rawText= rawText.replace(/<br\/>/gi,"\n");
		rawText= rawText.replace(/\&#09;/g, "\t");
		return rawText
	},

	__sentinel: null // terminates this class definition
});

com.ibm.rqm.defects.web.ui.workitems.WorkItemSelectionDialog.create= function(workItemsProvider, queryFunction) {
	return new com.ibm.rqm.defects.web.ui.workitems.WorkItemSelectionDialog({
		_workItemsProvider: workItemsProvider,
		_queryFunction: queryFunction
	}, null);
}

})();