/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2008. All
 * Rights Reserved.
 * 
 * U.S. Government Users Restricted Rights: Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
//
//
package com.ibm.rational.test.lm.spi.bf.internal;

import java.util.concurrent.LinkedBlockingQueue;

import com.buildforge.services.client.dbo.Server;
import com.ibm.rational.test.lm.spi.bf.common.BFAPIAuthenticationException;
import com.ibm.rational.test.lm.spi.bf.common.BFAPIClientConnectionException;
import com.ibm.rational.test.lm.spi.bf.common.BFAPIException;
import com.ibm.rational.test.lm.spi.bf.common.StatusPollUnit;
import com.ibm.rational.test.lm.spi.common.SPIException;

public class StatusPollThread extends Thread {

	private LinkedBlockingQueue<StatusPollUnit> statusPollUnits = new LinkedBlockingQueue<StatusPollUnit>();

	private static StatusPollThread instance = null;

	private volatile boolean addNew = false;

	private StatusPollUnit a;

	private StatusPollThread() {
		super();
	}

	public static synchronized StatusPollThread getInstance() {
		if (instance == null) {
			instance = new StatusPollThread();
		}
		return instance;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Thread#run()
	 */
	@Override
	public void run() {
		while (true) {
			if (!addNew) {
				try {
					Thread.sleep(1000);
				} catch (InterruptedException e) {
				}
			} else {
				if (!statusPollUnits.isEmpty()) {
					addNew = false;
					StatusPollUnit unit;
					synchronized (statusPollUnits) {
						unit = statusPollUnits.poll();
					}
					try {
						if (unit != null) {
							statusPool(unit.getHostname(), unit.getPort(), unit
									.getServerId());
						}
					} catch (SPIException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					}
					try {
						synchronized (statusPollUnits) {
							statusPollUnits.put(unit);
						}
					} catch (InterruptedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
				try {
					Thread.sleep(10000);// 10 second 
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	}

	public void addStatusPollUnit(StatusPollUnit unit) {
		synchronized (statusPollUnits) {
			statusPollUnits.add(unit);
		}
		if (!addNew) {
			addNew = true;
		}
	}

	public void cancelStatusPollUnit(StatusPollUnit unit) {
		synchronized (statusPollUnits) {
			statusPollUnits.remove(unit);
		}
	}

	/**
	 * Get the APIClientConnection as SystemConnection
	 * 
	 * @return APIClientConnection systemConnection
	 * @throws BFAPIClientConnectionException
	 * @throws BFAPIAuthenticationException
	 */
	private BFAPIClientConnection getBFAPIConnection(
			String provider_server_name, int provider_server_port)
			throws BFAPIException {
		BFAPIClientConnection BFAPIConn = ConnectionController
				.getSystemConnection(provider_server_name, provider_server_port);
		BFAPIConn.connect();
		return BFAPIConn;
	}

	public void statusPool(String hostname, int port, String serverId)
			throws SPIException {
		BFAPIClientConnection connection = null;
		try {
			connection = getBFAPIConnection(hostname, port);
			Server server = Server.findById(connection.getConnection(), Integer
					.valueOf(serverId));
			if (server != null) {
				server.test();
			} else {
				throw new SPIException(
						"Can't find out the Server: " + serverId + " in Build forge provider" + hostname + port); //$NON-NLS-1$
			}
		} catch (Exception e) {
			APIExceptionHandle.handleAPIExceptions(e);
		} finally {
			try {
				// Release the Build Forge connection resource to avoid the
				// connection leak
				if (connection != null) {
					connection.close();
				}
			} catch (BFAPIClientConnectionException e) {
				// Do not need to throw exception for it is not a big issue.
			}
		}
	}
}
